;=========================================================================
;
; Browser executable Application Programmer Interface (API)
; Copyright (c) 2010 Ray Palmer
;
;=========================================================================
;Name     API_GetWinInput
;Purpose  To get a key string using the Main Menu area.
;Input    A = Echo flag controls
;             Bit Description
;              7  0= single line input
;                 1= multi-line input (ESC terminates)
;             6-1 Unused/Reserved
;              0  1 = hidden text output control.
;                 0 = show entered text
;         B = Stream Number for OS clear stream
;         C = flag to get input (0 = new input, >0 use existing)
;         HL = Address for input
;         D = Start column for output
;         E = start row for output
;In Vars  KB_printlen = max number of characters which can be echoed
;                       back.
;         KB_MaxInpRC = Max row/column for echo back.
;         Window_ERow = bottom row of window
;         Window_ECol = right most column of window
;Output   A = ??
;         BC = ??
;         DE = Address of input (NULL terminated)
;         H = ??
;         L = ??
;Out Vars KB_BuffLoc
;         KB_StartCol
;         MsgStream
;         KB_Offset
;         KB_Buffer = input string.
;Uses     None.
;Call by  TBD
;Calls    KB_GetWinInput (via CPCB_CPCAccess)
;         API_AdjustRowCol
;         VIY_CalcAddr
;Notes    TBD
;=========================================================================
API_GetWinInput
   CALL  API_AdjustRowCol
   PUSH  HL
   PUSH  BC
   PUSH  DE
;
   LD    (KB_EchoFlag),A		;Setup echo back control
;
   LD    HL,KB_KeyTested
   LD    (HL),0			;Flag no key pressed.
;
   LD    HL,KB_printlen
   LD    DE,API_MAXECHOBKCHRS
   LD    (HL),E			;setup max echo back chars
   INC   HL
   LD    (HL),D
;
   LD    HL,Window_ECol
   LD    B,(HL)			;Get window end column
   POP   DE
   PUSH  DE
;
   LD    C,E			;Assume only 1 line of text
   BIT   7,A			;Single lie of text input?
   JR    Z,API_GetWinInp1	;Yes
   LD    HL,Window_ERow
   LD    C,(HL)			;Get window end row for multi-line
;
API_GetWinInp1
   LD    HL,KB_MaxInpRC
   LD    (HL),C			;setup end row/col of input window
   INC   HL
   LD    (HL),B
;
   POP   HL
   POP   BC
   POP   DE
   CALL  CPCB_CPCAccess
   DEFB  CPCB_C_GetWinInput
   RET
