;=====================================================================
;
; TCP/IP Application Program Interface (API) source
; Copyright (c) 2011 Ray Palmer
;
;=====================================================================
API_RSX_TCPIPIF
   DEFM  "TCPIP.I","F"+&80
;
API_ROMNumber
   DEFW  0
API_FarAddress
   DEFS  3
;
API_JumpTableLocation
   DEFW  0
;
API_Parameters1
  DEFW   3					;What to get (jump table location)
  DEFW   API_JumpTableLocation	;Where to save jump table location
  DEFW   API_ROMNumber			;Where to save rom number
;=====================================================================
;Name     API_Initialisation
;Purpose  To perform the API initailisation to prepare applications to
;         access the TCP/IP API.
;Input    None
;Output   API_FarAddress setup with far data.
;         Z flag set if system not configured else NZ.
;Uses     All registers may be corrupt
;Notes    1. Assume the TCP/IP ROM is installed and so the FIND RSX
;            kernel call is not expetced to fail.
;         2. Also ROM is not is position 0.
;=====================================================================
API_Initialisation
  LD    A,(TICK_CFGDONE)
  BIT   4,A				;TCP/IP configured?
  RET   Z				;No
;
;  Setup the far address for a call to TCP/IP get data elements of
;  TCP/IP
;
  LD    HL,API_RSX_TCPIPIF
  CALL  KL_FIND_COMMAND		;Find RSX command for TCP/IP API
  JR    C,API_Init2		;Command found
  XOR   A
  RET
API_Init2
  LD    A,C
  LD    (API_FarAddress+2),A
  LD    (API_FarAddress),HL
;
  LD    IX,API_Parameters1
  LD    A,3
  RST   3				;Call RSX to get API jump table location
  DEFW  API_FarAddress
;
  LD    A,(API_ROMNumber)
  LD    (API_FarAddress+2),A
  OR    A
  RET
;=====================================================================
;Name     API_CommonEntry
;Purpose  To provide a common entry point to access the TCP/IP API's.
;Input    As per API
;Output   As per API
;Uses     As per API
;Notes    1. Since no registers are preserved by this routine the
;            user must ensure that where they are required they are
;            saved before the call or this is routine modified to save
;            them.
;         2. The call format of this routine is -
;
;               <setup registers with parameters for API>
;               CALL  API_CommonEntry
;               DEFW  api_name
;         3. The routine is specifically form RAM usage as code has
;            self-modified parts.
;=====================================================================
API_CommonEntry
   LD    (API_REGHL),HL			;Save regs HL and DE
   LD    (API_REGDE),DE
   POP   HL
   LD    E,(HL)
   INC   HL
   LD    D,(HL)				;get API jump table offset
   INC   HL
   PUSH  HL				;restore return address
   LD    HL,(API_JumpTableLocation)
   ADD   HL,DE				;Calculate actual ROM location.
   LD    (API_FarAddress),HL		;setup FAR address
   DEFB  &11
API_REGHL
   DEFW  0
   DEFB  &21
API_REGDE
   DEFW  0
   RST   3				;perform FAR call to API
   DEFW  API_FarAddress
   RET
