;=====================================================================
;
; TCP/IP Variables
; Copyright (c) 2010 Ray Palmer
;
; Notes -
; 1.  The location of variables allocated is stored as little endian
;     format.
;=====================================================================
;  SYS_V_TCPIPON = Status of TCP/IP
;                  Bit   Description
;                   0    TCP/IP Status (0 = Off, 1 = enabled)
;                   1    keyboard char got (0=No, 1=Yes)
;                   2    Console Up/Down (0=Down , 1=Up)
;                   3    Console display prompt (0=No, 1=Yes)
;                   4    Application char control (0=No, 1=Yes)
;                   5    Configuration data loaded (0=No, 1=Yes)
;                   6    TCP/IP quit flag (0=No, 1=Yes)
;                   7    First start of main (0=running, 1=start)
;
;  Application char output is for telnet (or other) where returned
;  chars control the local display.
;
;=====================================================================
SYS_V_TCPIPON		EQU 6					;(1)
;=====================================================================
TCP_Kick		EQU SYS_V_TCPIPON+1			;(1)
;=====================================================================
;  datalink_flag = bit fields of data link setup for SLIP/PPP/RAW
;
;    Bit   Description
;     7    Datalink is up       (1=PPP/SLIP is Up, 0= Down)
;     6    IN ESC flag          (0=No, 1=Yes)
;     5    Datalink Mode        (0=PPP, 1=SLIP)
;     4    Comms device         (0=RS-232, 1=Ethernet)
;     3    Pkt complete         (0=No , 1=Yes)
;     2    iprxbuffer free      (0=Yes, 1=No)
;     1    External Acess       (0=No, 1=Yes)
;     0    Datalink is open     (0=Closed, 1=Open)
;=====================================================================
datalink_flags		EQU TCP_Kick+1				;(1)
;=====================================================================
;  process_flag = bit fields of data processing for SLIP/PPP
;  This variable is in the system area.
;    Bit   Description
;     7    ip handle status            (0=unhandled, 1=handled)
;     6    enable/disable RX/TX access (0=disable,   1=enable)
;     5    stdio print status          (0=enabled,   1=disabled)
;     4    Mode                        (0=Non-PPPoE, 1=PPPoE)
;     3    Data Encapsulation          (0=No,        1=Yes)
;     2    Unused
;     1    New Rx message              (0=No,        1=Yes)
;     0    TCP/UDP Validate Check      (0=No,        1=Yes)
;
;  Bit 4 is set if Datalink Mode is PPP and Device is Ethernet.
;=====================================================================
process_flags		EQU datalink_flags+1			;(1)
;=====================================================================
;  TstCtrl_flags = bit fields of data processing for testing
;  This variable is in the system area.
;    Bit   Description
;    7-2   Unused
;     2    Test mode                   (0=No,        1=Yes)
;     1    Test print mode             (0=No,        1=Yes)
;     0    Unused
;
;=====================================================================
TstCtrl_flags		EQU process_flags+1			;(1)
;=====================================================================
;
; iprxbufloc = Address of RX buffer
; iptxbufloc = Address of TX message buffer
; ipbuffloc  = Address of RX buffer for Packet handling.
;
;=====================================================================
iprxbufloc		EQU TstCtrl_flags+1			;(2)
iptxbufloc		EQU iprxbufloc+2			;(2)
ipbuffloc		EQU iptxbufloc+2			;(2)
ipbuffins		EQU ipbuffloc+2				;(2)
ipbuffwrk		EQU ipbuffins+2				;(2)
ipbuffsz		EQU ipbuffwrk+2				;(2)
;=====================================================================
;  tcp_bind_head = start location of TCP bind nodes
;  udp_bind_head = start location of UDP bind nodes
;  time_bind_head =  start lcoation of timeout nodes
;=====================================================================
tcp_bind_head		EQU ipbuffsz+2	     			;(2)
udp_bind_head    	EQU tcp_bind_head+2			;(2)
time_bind_head		EQU udp_bind_head+2			;(2)
;=====================================================================
main_parseC		EQU time_bind_head+2			;(1)
;=====================================================================
cons_incup		EQU main_parseC+1			;(1)
cons_incupX		EQU cons_incup+1			;(1)
cons_status		EQU cons_incupX+1			;(1)
cons_chrin		EQU cons_status+1			;(1)
;=====================================================================
keybbufferoffset        EQU cons_chrin+1			;(2)
keybbufferinsert	EQU keybbufferoffset+2			;(2)
keyb_echo		EQU keybbufferinsert+2			;(1)
keyb_client_keyhandler	EQU keyb_echo+1				;(2)
keyb_client_linehandler	EQU keyb_client_keyhandler+2		;(2)
;=====================================================================
ICMP_seqnum		EQU keyb_client_linehandler+2		;(2)
;=====================================================================
dnsd_requested_qclass	EQU ICMP_seqnum+2			;(2)
dnsd_requested_qtype	EQU dnsd_requested_qclass+2		;(2)
dnsd_res_recs		EQU dnsd_requested_qtype+2		;(2)
dnsd_bank		EQU dnsd_res_recs+2			;(1)
;=====================================================================
resolv_currentID	EQU dnsd_bank+1				;(2)
resolv_pquery		EQU resolv_currentID+2			;(2)
resolv_qtype		EQU resolv_pquery+2			;(2)
resolv_hasdot		EQU resolv_qtype+2			;(1)
resolv_client		EQU resolv_hasdot+1			;(2)
resolv_RRstart		EQU resolv_client+2			;(2)
resolv_retrycount	EQU resolv_RRstart+2			;(1)
;=====================================================================
;
; AMSDOS buffers are 2K in size and may be anywhere in main RAM
;
;=====================================================================
quietcas_txtoutputorig	EQU resolv_retrycount+1			;(1)
amsdos_message_previous	EQU quietcas_txtoutputorig+1		;(1)
amsdos_ftype		EQU amsdos_message_previous+1		;(1)
amsdos_quietcall_instr	EQU amsdos_ftype+1			;(1)
amsdosRDBufLoc		EQU amsdos_quietcall_instr+1		;(2)
amsdosWRBufLoc		EQU amsdosRDBufLoc+2			;(2)
;=====================================================================
tftpd_get_dr_status	EQU amsdosWRBufLoc+2			;(1)
tftpd_seqexpected	EQU tftpd_get_dr_status+1		;(2)
tftpd_portnbr		EQU tftpd_seqexpected+2			;(2)
tftpd_mode		EQU tftpd_portnbr+2			;(1)
tftpd_ctrl		EQU tftpd_mode+1			;(1)
;=====================================================================
slip_flags		EQU tftpd_ctrl+1			;(1)
slip_send_drop		EQU slip_flags+1			;(1)
ppp_send_drop		EQU slip_flags+1			;(1)
;=====================================================================
ip_ID            	EQU ppp_send_drop+1			;(2)
;=====================================================================
icmp_client_echoreply  	EQU ip_ID+2				;(2)
;=====================================================================
httpd_lastNL		EQU icmp_client_echoreply+2		;(1)
httpd_parseC		EQU httpd_lastNL+1			;(1)
httpd_state		EQU httpd_parseC+1			;(1)
httpd_fileopen		EQU httpd_state+1			;(1)
httpd_file_handle	EQU httpd_fileopen+1			;(2)
httpd_response_index	EQU httpd_file_handle+2			;(2)
httpd_posfilename	EQU httpd_response_index+2		;(2)
httpd_lenfilename	EQU httpd_posfilename+2			;(1)
httpd_request_index	EQU httpd_lenfilename+1			;(1)
httpd_bufloc		EQU httpd_request_index+2		;(2)
;
; httpd_request contains the text to serve HTTP file (128 bytes)
;
httpd_request		EQU httpd_bufloc+2			;(2)
httpd_ctrl		EQU httpd_request+2			;(1)
;=====================================================================
config_hosts_txt			EQU httpd_ctrl+1				;(2)
;=====================================================================
Cfg_DomainName			EQU config_hosts_txt+2			;(2)
;=====================================================================
;
;  TCP/IP IPv4 Addresses
;
;  resolv_addr           = IPv4 address to return from resolver
;  host_addr             = result from host lookup.
;  ifconfig_newip        = address returned from ipconfig
;  config_hostaddr       = Own internet Address (YIADDR if DHCP)
;  config_nameserveraddr = Local nameserver address
;  config_remoteaddr     = remote address of device to connect to.
;  gateway_addr          = gateway address from DHCP (GIADDR)
;  server_addr           = server address from DHCP (SIADDR)
;  DHCP_Server           = DHCP server address from DHCP options data
;  DHCP_SubNetMsk        = SubNet Mask from DHCP options data
;  DHCP_Router           = Router address from DHCP options data
;  DHCP_DNS              = First DNS server from DHCP options data
;                          (the remaining server entries ignored)
;
;=====================================================================
resolv_addr				EQU Cfg_DomainName+2			;(4)
host_addr				EQU resolv_addr
ifconfig_newip			EQU host_addr+4				;(4)
config_hostaddr			EQU ifconfig_newip+4			;(4)
config_nameserveraddr		EQU config_hostaddr+4			;(4)
config_remoteaddr			EQU config_nameserveraddr+4		;(4)
gateway_addr			EQU config_remoteaddr+4			;(4)
server_addr				EQU gateway_addr+4			;(4)
DHCP_Server				EQU server_addr+4				;(4)
DHCP_SubNetMsk			EQU DHCP_Server+4				;(4)
DHCP_Router				EQU DHCP_SubNetMsk+4			;(4)
DHCP_DNS				EQU DHCP_Router+4				;(4)
;=====================================================================
;
; The keyboard buffer has a maximum size of 256 bytes.
;
;=====================================================================
keybbuffer				EQU DHCP_DNS+4				;(2)
keybcmds				EQU keybbuffer+2				;(2)
;=====================================================================
;  Buffer Manager variables (See COMMS-Vars.INC)
;=====================================================================
BuffMgrVars				EQU keybcmds+2				;(S_BuffMgrVars)
;
;  TXMsgLen = length of TX message for external driver
;
TXMsgLen				EQU BuffMgrVars+S_BuffMgrVars
;=====================================================================
;
;  Address of HDOS data for Network communication via TCP/IP (IN)
;
;  HDOS_VARS_xx  = pointer to HDOS network data packet
;     Where xx = OS for HDOS network (server)
;                OC for HDOS network (client)
;  HDOS_Stat     = pointer to HDOS network status byte
;  HDOS_Hndlr    = Address of HDOS handler called by TCP/IP
;  HDOS_HndlrROM = ROM Number for HDOS Handler
;  HDOS_Flags    = TCP/IP internal status/control flags.
;                  Bit   Description
;                   7    New packet to send via TCP/IP (HDOS RD/WR)
;                   6    New packet to send via TCP/IP (HDOS NET)
;                  5-4   Reserved
;                   3    Client request active
;                   2    Server request active
;                  1-0   reserved.
;  HDOS_FARCALL  = FAR call routine into HDOS.
;=====================================================================
HDOS_VARS				EQU TXMsgLen+2
HDOS_VARS_OC			EQU HDOS_VARS				;(2)
HDOS_VARS_OS			EQU HDOS_VARS_OC+2			;(2)
HDOS_Stat				EQU HDOS_VARS_OS+2			;(2)
HDOS_Hndlr				EQU HDOS_Stat+1				;(2)
HDOS_HndlrROM			EQU HDOS_Hndlr+2				;(1)
HDOS_Flags				EQU HDOS_HndlrROM+2			;(1)
;=====================================================================
resolv_qtyp 			EQU HDOS_Flags+1				;(2)
resolv_status			EQU resolv_qtyp+2
resolv_error			EQU resolv_status+1
timer_tftpd_keepalive		EQU resolv_error+1			;(Sz_TIMEOUT)
;=====================================================================
;
; PPP Stuff
; This is the information to use in PPP authentication using PAP
; Remember to guard this file and the binary if you make use of PAP!
; PAP Authorisation Packet Structure is
;    Byte   Value             Details
;     0     PAP_AUTH_REQ
;     1     &42
;    2-3    Pkt Size          Size of PAP packet
;     4     Username length   Length of the user name text
;    4-N    Username text     User name text (length N)
;    N+1    Password Length   Length of the Password text
;    N+2    Password Text     Password text (Length M)
;
; Note Max PAP space allocated is 64 bytes.
;
;=====================================================================
cfg_pap_arps             	EQU timer_tftpd_keepalive+1		;(2)
;=====================================================================
;  PPP - See RFC1662 (B)
;
;  ppp_protocol = protocol of incomming ppp message.
;  ppp_wrkspace = location for ppp_header
;
;=====================================================================
ppp_protocol			EQU cfg_pap_arps+2			;(2)
ppp_wrkspace                  EQU ppp_protocol+2			;(2)
lcp_prot_rej_identifier		EQU ppp_wrkspace+2			;(2, S_lcp_prot_rej_id)
lcp_conf_req_identifier		EQU lcp_prot_rej_identifier+2		;(2, S_lcp_conf_req_id)
ipcp_conf_req_identifier	EQU lcp_conf_req_identifier+2		;(2, S_ipcp_conf_req_id)
lcp_configure_accm_in		EQU ipcp_conf_req_identifier+2	;(1)
lcp_configure_accm_out		EQU lcp_configure_accm_in+1		;(1)
lcp_configure_ap			EQU lcp_configure_accm_out+1		;(1)
;=====================================================================
;  bind structure allocations
;  Each defines the memory location of the bind structure.
;=====================================================================
bind_dnsd_server			EQU lcp_configure_ap+1
bind_resolv_active		EQU bind_dnsd_server+2
bind_tftpd_server			EQU bind_resolv_active+2
bind_tftpd_active			EQU bind_tftpd_server+2
bind_telnet_active		EQU bind_tftpd_active+2
bind_httpd_active			EQU bind_telnet_active+2
bind_httpd_sink			EQU bind_httpd_active+2
bind_hdos_input			EQU bind_httpd_sink+2
bind_dhcp_input			EQU bind_hdos_input+2
bind_end				EQU bind_dhcp_input+2
;=====================================================================
;  Hardware Data for Ethernet and PPPoE
;=====================================================================
ServiceName				EQU bind_end				;(2)
ACName				EQU ServiceName+2				;(2)
HostMACType				EQU ACName+2				;(1)
HostMACAddr				EQU HostMACType+1				;(6)
;=====================================================================
;  RX/TX Character/Message stuff
;=====================================================================
HandleRXBld				EQU HostMACAddr+6		;RX Pkt builder
HandleRXPkt				EQU HandleRXBld+3		;RX pkt handler
HandleTXPkt				EQU HandleRXPkt+3		;TX pkt handler
;=====================================================================
;  Device access location to setup the device service.
;=====================================================================
HandleDev				EQU HandleTXPkt+3		;Device Access
;=====================================================================
;  Ext ROM Handler stuff (15 bytes)
;=====================================================================
ExtROMCall				EQU HandleDev+3		;Ext ROM Handler
ExtROMCallSz			EQU 20
;=====================================================================
;  Ticker elements
;
;  Main_TckBlk = ticker block as per AMSTRAD OS specification
;
;=====================================================================
Main_TckBlk				EQU ExtROMCall+ExtROMCallSz
;=====================================================================
socklist				EQU Main_TckBlk+FastT_MinSz		;(2)
socket_IPv4				EQU socklist+2				;(4)
socket_ports			EQU socket_IPv4+4				;(4)
socket_port_src			EQU socket_ports
socket_port_dst			EQU socket_ports+2
socket_protcl			EQU socket_ports+4			;(1)
socket_TXBank			EQU socket_protcl+1			;(1)
socket_RXBank			EQU socket_TXbank+1			;(1)
socket_maxnum			EQU socket_RXbank+1			;(1)
socket_nbr				EQU socket_maxnum+1			;(1)
;=====================================================================
;  DHCP_State = state of DHCP processing.
;                State    Description.
;                  0      Discovery
;                  1      Offer
;                  2      Request
;                  3      Acknowledgement
;                  &10    Info (1st message) - LSB has secs count
;                  &20    Info (2nd message) - LSB has secs count
;                  &42    Request waiting for Ack
;                  &80    DHCP complete
;
;=====================================================================
DHCP_State				EQU socket_nbr+1				;(1)
;=====================================================================
;  DHCP_Expiry is time when the current Own IP addres will expire from
;  when the Ack has completed.  The expiry time is decremented to zero
;  at which point a new address is requested.
;
;  Note That if the DHCP_State is showing expired and not at Ack,
;       then the address is not requested.
;=====================================================================
DHCP_Expiry				EQU DHCP_State+1				;(4)
DHCP_XIdent				EQU DHCP_Expiry+4				;(4)
DHCP_SecCtr				EQU DHCP_XIdent+4				;(2)
;=====================================================================
;
;  Variables for configuration loading/setting.
;
;=====================================================================
cfg_loadaddr			EQU DHCP_SecCtr+2				;(2)
cfg_bankaddr			EQU cfg_loadaddr+2			;(2)
cfg_endscript			EQU cfg_bankaddr+2			;(2)
cfg_subloc				EQU cfg_endscript+2			;(2)
cfg_trace				EQU cfg_subloc+2				;(1)
cfg_scrpos				EQU cfg_trace+1				;(2)
;=====================================================================
pkt_counter				EQU cfg_scrpos+2				;(2)
scr_loadaddr			EQU pkt_counter+2				;(2)
tst_msgloc				EQU scr_loadaddr+2			;(2)
scr_wrk				EQU tst_msgloc+2				;(2)
appfilename				EQU scr_wrk+2				;(2)
;=====================================================================
;  appscrflg = bit fields of how a file is loaded.
;
;    Bit   Description
;     7    Load file as script    (extension is ".SCR")
;     6    Load file as command   (extension is ".EXE")
;     5    Load file as text      (extension is ".TXT")
;     4    Load file as Help text (extension is ".HLP")
;     3    Load file as binary    (extension is ".BIN")
;    2-1   Unused
;     0    Execute loaded file as script.
;
;=====================================================================
appscrflg				EQU appfilename+2				;(1)
AppLoadAddr				EQU appscrflg+1				;(2)
AppFileLen				EQU AppLoadAddr+2				;(2)
AppFileTyp				EQU AppFileLen+2				;(2)
AppFileLoad				EQU AppFileTyp+1				;(2)
;=====================================================================
App_ExecAddr			EQU AppFileLoad+2				;(2)
App_FileTyp				EQU App_ExecAddr+2			;(1)
;=====================================================================
SysName				EQU App_FileTyp+1				;(2)
;=====================================================================
TCP_cfg_timeout			EQU SysName+2				;(2)
TFTP_cfg_timeout			EQU TCP_cfg_timeout+2			;(2)
HTTP_cfg_timeout			EQU tftp_cfg_timeout+2			;(2)
;=====================================================================
;
;  Overlayed variables
;
;=====================================================================
keyb_loadaddr			EQU cfg_loadaddr
;=====================================================================
;  END OF TCP/IP ROM VARIABLES
;=====================================================================
VARS_End				EQU HTTP_cfg_timeout+2
