;=====================================================================
;
; Variables for communication buffer management
; Copyright (c) 2010 Ray Palmer
;
; Buffers are in different banked memory segments
; Each bank is 16K in size
;
; Created By     Ray Palmer
; Created on     01 Jul 2010
; Updated On     26 Dec 2010
;
;=====================================================================
SER_StartVars	EQU 0				;Correct as required
;=====================================================================
SER_SerBufSz	EQU SER_StartVars		;Size of buffers
;=====================================================================
SER_SERRXBUF	EQU SER_SerBufSz+2	;RX Buff addr
SER_RXBuffIns	EQU SER_SERRXBUF+2	;RX insert offset
SER_RXBuffRem	EQU SER_RXBuffIns+2	;RX remove offset
SER_RXBufSz		EQU SER_RXBuffRem+2	;RX Used Space
SER_RXBank		EQU SER_RXBufSz+2		;RX Bank
;=====================================================================
SER_SERTXBUF	EQU SER_RXBank+1		;TX Buff addr
SER_TXBuffIns	EQU SER_SERTXBUF+2	;TX insert offset
SER_TXBuffRem	EQU SER_TXBuffIns+2	;TX remove offset
SER_TXBufSz		EQU SER_TXBuffRem+2	;TX Used Space
SER_TXBank		EQU SER_TXBufSz+2		;TX Bank
;=====================================================================
;
;  SER_txflags = bit fields for device handling
;
;   BIT   Description
;    7    Device error/available (setup by H/W device driver)
;          0 = device not available
;          1 = device available
;
;    6    TX buffer control
;          0 = TX Buffer is free
;          1 = TX buffer is taken
;
;    5    RX buffer control
;          0 = RX buffer is free
;          1 = RX buffer is taken
;
;    4    Data access from RX/TX buffer flag
;          0 = no data got or put
;          1 = data got or put
;
;    3    TX buffer has Test Message
;          0 = TX Buffer unchanged.
;          1 = TX buffer message inserted.
;    2    Reserved
;
;    1    Banked Memory Flag
;         0 = Non-Banked buffers.
;         1 = Banked Buffers Used.
;
;    0    TCP/IP available (only applicable for drivers)
;         0 = No
;         1 = Yes
;
;=====================================================================
SER_txflags		EQU SER_TXBank+1		;
;=====================================================================
SER_RXSz		EQU SER_txflags+1		;For use by SLIP/PPP/RAW
;=====================================================================
SER_EndVars		EQU SER_RXSz+2
S_BuffMgrVars	equ SER_EndVars
;=====================================================================
;
;  Constants
;
;=====================================================================
SER_MAXBuff		EQU &4000			;16K buffer size
SER_BUFFLEN		EQU &200
