;=====================================================================
;
; TCP/IP Bind structure definitions and contants
; Copyright (c) 2010 Ray Palmer
;
;=====================================================================
;
; The linked-list of bind nodes start with a word which contains
; a pointer to the next node, or zero for the last one.
;
; The nodes used for the bind structure have the format -
;    Bytes  What         Details
;      2    next         pointer to next node
;      4    remoteaddr   IPv4 Address (as per internet standard)
;      2    remoteport   Remote Port (as per internet standard)
;      2    localport    Local Port (as per internet standard)
;      2    handler      Address of Handler
;      n    extrainfo    data for handler (if any)
;
;  Min Size of Bind Structure is 12 bytes.
;
;  The first octet of the remaddr and the two octets of the remport
;  may independently be set to all-zeroes to mean "any".  The octets
;  of the remaddr and the remport may all be set to all-ones to mean
;  "none" (effectively temporarily suspending the bind).
;
;  The same dispatcher can be used both for UDP and TCP because they
;  have the ports at the same offset.
;
;=====================================================================
BindStruct_NEXT		EQU 0  ;Pointer to Next node
BindStruct_REMADDR	equ 2  ;Remote IPv4 Address
BindStruct_REMPORT	equ 6  ;Remote Port
BindStruct_LCLPORT	equ 8  ;Local Port
BindStruct_HANDLER	equ 10 ;Address of RX Handler
BindStruct_EXTRA		equ 12 ;Extra data for handler
;=====================================================================
; TCP/IP Bind (TCB) Extra Data Structure
;
; Note 1. SNDNXT = SNDUNA + SNDNUN
;      2. TCB_OFFSETOF_RETXTO = Timeout control block
;
; Field                Description
; TCB_OFFSETOF_STATE   Current state of the bind
; TCB_OFFSETOF_CONNUP  Handler for connecting the bind
; TCB_OFFSETOF_RECVUP  Handler for Receiving
; TCB_OFFSETOF_SENTUP  Handler for Sending
; TCB_OFFSETOF_CLOSUP  Handler for closing
; TCB_OFFSETOF_SNDUNA  SND.UNA
; TCB_OFFSETOF_RCVNXT  RCV.NXT
; TCB_OFFSETOF_RCVWND  RCV.WND - Receive Window
; TCB_OFFSETOF_RETXTO  RETXTO - Re-Tx Timeout
; TCB_OFFSETOF_SNDNUN  Index into the bind data buffer
; TCB_OFFSETOF_SNDBUF  Location of the bind data buffer
;
;=====================================================================
TCB_OFFSETOF_STATE	equ  0				;(1) Must be 0
TCB_OFFSETOF_CONNUP	equ  TCB_OFFSETOF_STATE+1	;(2) Conn. Up
TCB_OFFSETOF_RECVUP	equ  TCB_OFFSETOF_CONNUP+2	;(2) Recv Pkt
TCB_OFFSETOF_SENTUP	equ  TCB_OFFSETOF_RECVUP+2	;(2) Send Pkt
TCB_OFFSETOF_CLOSUP	equ  TCB_OFFSETOF_SENTUP+2	;(2) Conn. Dwn
TCB_OFFSETOF_SNDUNA	equ  TCB_OFFSETOF_CLOSUP+2	;(4)
TCB_OFFSETOF_RCVNXT	equ  TCB_OFFSETOF_SNDUNA+4	;(4)
TCB_OFFSETOF_RCVWND	equ  TCB_OFFSETOF_RCVNXT+4	;(2)
TCB_OFFSETOF_RETXTO	equ  TCB_OFFSETOF_RCVWND+2	;(size of timeout)
TCB_OFFSETOF_SNDNUN	equ  TCB_OFFSETOF_RETXTO+BT_Sizeof	;(2)
TCB_OFFSETOF_SNDBUF	equ  TCB_OFFSETOF_SNDNUN+2	;(2)
;=====================================================================
;  TCB_Size = size of TCB structure except for buffer space.
;=====================================================================
TCB_Size			EQU  TCB_OFFSETOF_SNDBUF	;Size of TCB
TCB_End			EQU  BindStruct_EXTRA+TCB_Size
;=====================================================================
