;=====================================================================
;
; Bind-timeout structure definition
; Copyright (c) 2010 Ray Palmer
;
;=====================================================================
;  BT_Link   = link to next timeout object.
;  BT_Status = status of timeout for a bind.
;              Bit   Description
;               7    Timeout activity  (0=In-active, 1=Active)
;               6    Timeout detection (0=None, 1=Timed out)
;               5    Availability      (0=No, 1=Yes)
;               4    Handler assigned  (0=No, 1=Yes)
;              3-0   Unused
;
;  BT_CurrTime   = Number of timeout tick left.
;  BT_Timeout    = Time required to detect a timeout (fixed).
;  BT_Handler    = handler to execute when timeout detected.
;                  if zero then no handler assigned.
;  BT_MaxRepeats = Maximum number of time repeat timeout
;                  default is 1.
;  BT_RptCnt     = current repeat count
;=====================================================================
BT_Link			EQU 0				;Timeout chain link
BT_Status			EQU BT_Link+2		;Status byte
BT_CurrTime			EQU BT_Status+1		;When Time out occurs
BT_TimeOut			EQU BT_CurrTime+2		;Timeout length
BT_Handler			EQU BT_TimeOut+2		;Handler for timeout.
BT_MaxRepeats		EQU BT_Handler+1		;Max retries
BT_RptCnt			EQU BT_MaxRepeats		;Current repeat count
;=====================================================================
BT_Sizeof			EQU BT_RptCnt+1	      ;size of structure
