;=====================================================================
;
; TCP/IP Grpahics utility library
; Copyright (c) 2010 Ray Palmer
;
;=====================================================================
;Name     grph_prtstring
;Purpose  To print a string using graphic OS calls.
;Input    BC = pixel spaces ebwteen chars
;         HL = Y coordinate
;         DE = X coordinate
;         IX = address of string
;Output   string output
;Uses     All register corrupted
;Notes    None
;=====================================================================
grph_prtstring
   PUSH  BC
   PUSH  DE
   PUCH  HL
   CALL  SC_SET_CURSOR		;set cursor position
   POP   HL
   POP   DE
   POP   BC
   LD    A,(IX+0)
   OR    A				;End of string
   RET   Z				;Yes
   PUSH  BC
   PUSH  DE
   PUCH  HL
   CALL  SC_PRT_CHAR		;display char at position
   POP   HL
   POP   DE
   POP   BC
   EX    DE,HL
   ADD   HL,BC
   EX    DE,HL
   INC   IX
   JR    grph_prtstring
;=====================================================================
;Name     grph_txt_dblchar
;Purpose  To print a character using OS calls with double the height.
;Input    IX = address of char
;Output   string output
;Uses     All register corrupted
;Notes    None
;=====================================================================
grph_txt_dblchar
   CALL  KL_ROM_STATUS			;get current rom state
   PUSH  AF
   LD    A,(IX+0)
   CALL  TXT_GET_CHRPAT			;HL = address of pattern data
   LD    B,8
   LD    IX,grph_patternarray
txt_dblchar1
   LD    A,(HL)
   LD    (IX+0),A
   INC   IX
   LD    (IX+1),A
   INC   IX
   INC   HL
   DJNZ  txt_dblchar1
   POP   AF
   CALL  KL_ROM_RESTORE
   LD    A,254				;Setup where char 254 data is
   LD    HL,grph_patternarray
   CALL  TXT_SET_CHRPAT
   LD    A,255				;Setup where char 255 data is
   LD    HL,grph_patternarray+8
   CALL  TXT_SET_CHRPAT
   LD    A,254
   CALL  TXT_OUTPUT
   LD    A,LF
   CALL  TXT_OUTPUT
   LD    A,BS
   CALL  TXT_OUTPUT
   LD    A,255
   CALL  TXT_OUTPUT
   RET
;=====================================================================
;Name     grph_varchar
;Purpose  To print a character using graphic OS calls with a variable
;         the height (but same width)
;Input    IX = address of parameters
;         A = number of parameters
;Output   string output
;Uses     All register corrupted
;Notes    None
;=====================================================================
grph_varchar
   CP    2
   RET   NZ
   LD    A,(IX+0)
   LD    (grph_height),A
   CALL  KL_ROM_STATUS			;Get current rom state
   PUSH  AF
   LD    A,(IX+2)
   CALL  TXT_GET_CHRPAT			;Get location of char pattern
   LD    DE,grph_patternarray
   LD    BC,8
   LDIR
   POP   AF
   CALL  KL_ROM_RESTORE
;
   LD    IX,grph_patternarray
   LD    D,8
grph_vchr_loop1
   LD    A,(IX+0)
   LD    C,A
   LD    B,8
grph_vchr_loop2
   SLA   C
   CALL  grph_GETCURS
   JR    C,grph_ZERO
   CALL  grph_VERTS
   JR    grph_OUT
grph_ZERO
   CALL  grph_VERT
grph_OUT
   DJNZ  grph_vchr_loop2
   INC   IX
   CALL  grph_OUTH
   DEC   D
   JR    NZ,grph_vchr_loop1
   RET
;
grpg_printblock
   PUSH  HL
   PUSH  DE
   PUSH  BC
   LD    DE,6
   LD    HL,0
   CALL  SC_REL_DRWLNE
   POP   BC
   POP   DE
   POP   HL
   RET
;
grph_printspace
   PUSH  HL
   PUSH  DE
   PUSH  BC
   LD    DE,6
   LD    HL,0
   CALL  SC_REL_MOVE
   POP   BC
   POP   DE
   POP   HL
   RET
;
grph_OUTH
   PUSH  BC
   PUSH  HL
   PUSH  DE
   LD    HL,0
   LD    A,(grph_height)
   LD    B,A
grph_OUTH1
   DEC   HL
   DEC   HL
   DJNZ  grph_OUTH1
   LD    DE,-48			;Go back 8*6 pixels
   CALL  SC_REL_MOVE
   POP   DE
   POP   HL
   POP   BC
   RET
;
grph_getcurs
   PUSH  BC
   PUSH  HL
   PUSH  DE
   PUSH  AF
   CALL  SC_GET_CURSOR
   LD    B,6
grph_getcurs1
   INC   DE
   DJNZ  grph_getcurs1
   LD    (grph_tempx),DE
   LD    (grph_tempy),HL
   POP   AF
   POP   DE
   POP   HL
   POP   BC
   RET
;
grph_restore
   PUSH  BC
   PUSH  DE
   PUSH  HL
   LD    DE,-6
   LD    HL,-2
   CALL  SC_REL_MOVE
   POP   DE
   POP   HL
   POP   BC
   RET
;
grph_next
   PUSH  AF
   PUSH  BC
   PUSH  DE
   PUSH  HL
   LD    DE,(grph_tempx)
   LD    HL,(grph_tempy)
   CALL  SC_SET_CURSOR
   POP   HL
   POP   DE
   POP   BC
   POP   AF
   RET
;
grph_vert
   PUSH  BC
   CALL  grph_getcurs
   LD    A,(grph_height)
   LD    B,A
grph_vert_vloop
   CALL  grph_printblock
   CALL  grph_restore
   DJNZ  grph_vert_vloop
   CALL  grph_next
   POP   BC
   RET
;
grph_verts
   PUSH  BC
   CALL  grph_getcurs
   LD    A,(grph_height)
   LD    B,A
grph_verts_loop
   CALL  grph_printspace
   CALL  grph_restore
   DJNZ  grph_verts_vloop
   CALL  grph_next
   POP   BC
   RET
;=====================================================================
;Name     grph_gdraw
;Purpose  To draw an object using vectors.
;Input    IX = address of shape table
;Output   screen output
;Uses     All register corrupted
;Notes    1. Shape table entry format is as follows-
;             a. OpCode
;             b. X coordinate (word, LSB first)
;             c. Y coordinate (word, LSB first)
;            For colour entry
;             a. OpCode
;             b. Colour byte
;=====================================================================
grph_gdraw
   LD    A,(IX+0)
   CP    0				;End of table?
   RET   Z				;Yes
;
   CP    GDRAW_ABSMVE
   CALL  Z,grph_move
;
   CP    GDRAW_ABSDRW
   CALL  Z,grph_draw
;
   CP    GDRAW_COLOUR
   CALL  Z,grph_colour
;
   CP    GDRAW_ABSPLT
   CALL  Z,grph_plot
;
   INC   IX
   JR    grph_gdraw
;
grph_colour
   INC   IX
   LD    A,(IX+0)
   INC   IX
   CALL  SC_SET_PEN
   XOR   A
   RET
;
grph_move
   CALL  grph_coords
   CALL  SC_SET_CURSOR
   XOR   A
   RET
;
grph_draw
   CALL  grph_coords
   CALL  SC_ABS_DRWLNE
   XOR   A
   RET
;
grph_plot
   CALL  grph_coords
   CALL  SC_ABS_PLTLNE
   XOR   A
   RET
;
grph_coords
   LD    E,(IX+0)
   INC   IX
   LD    D,(IX+0)
   INC   IX
   LD    L,(IX+0)
   INC   IX
   LD    H,(IX+0)
   INC   IX
   RET
;=====================================================================
;Name     grph_bdraw
;Purpose  To draw a box using vectors.
;Input    IX = address of box table data
;          A = number of parameters
;Output   screen output
;Uses     All register corrupted
;Notes    1. For a unfilled box the parameters are -
;            a. X coordinate
;            b. Y coordinate
;            c. X length
;            d. Y height
;         2. For a filled box the parameters are -
;            a. X coordinate
;            b. Y coordinate
;            c. length
;            d. height
;            e. XI
;         3. X/Y coordinates are bottom left of drawn box
;=====================================================================
grph_bdraw
   CP    4
   JR    Z,grph_openb
   CP    5
   JR    Z,grph_closedb
   RET
grph_openb
   CALL  grph_movep
   LD    E,(IX+2)
   LD    D,(IX+3)
   LD    HL,0
   CALL  SC_REL_DRWLNE
   LD    DE,0
   LD    H,(IX+0)
   LD    L,(IX+1)
   CALL  SC_REL_DRWLNE
   CALL  grph_movep
   LD    DE,0
   LD    H,(IX+0)
   LD    L,(IX+1)
   CALL  SC_REL_DRWLNE
   LD    E,(IX+2)
   LD    D,(IX+3)
   LD    HL,0
   CALL  SC_REL_DRWLNE
   RET
grph_closedb
   CALL  SC_GET_ORIGIN
   PUSH  DE
   PUSH  HL
   LD    L,(IX+6)
   LD    H,(IX+7)
   LD    E,(IX+8)
   LD    D,(IX+9)
   CALL  SC_SET_ORIGIN
   LD    C,(IX+2)
   LD    B,(IX+3)
   LD    HL,0
grph_closedb_loop
   LD    E,(IX+4)
   LD    D,(IX+5)
   PUSH  BC
   PUSH  HL
   LD    HL,0
   CALL  SC_REL_DRWLNE
   POP   HL
   INC   HL
   PUSH  HL
   LD    DE,0
   CALL  SC_SET_CURSOR
   POP   HL
   POP   BC
   DEC   BC
   LD    A,B
   OR    C
   JR    NZ,grph_closedb_loop
   POP   HL
   POP   DE
   CALL  SC_SET_ORIGIN
   RET
;
grph_movep
   LD    L,(IX+4)
   LD    H,(IX+5)
   LD    E,(IX+6)
   LD    D,(IX+7)
   CALL  SC_REL_MOVE
   RET
;=====================================================================
;Name     grph_readchar
;Purpose  To read a character at an x/y text location.
;Input    L = Y coordinate
;         H = X coordinate
;Output   A = character number (0=unknown char)
;Uses     All register corrupted
;Notes    None
;=====================================================================
grph_readchar
   CALL  TXT_SET_CURSOR
   CALL  TXT_GET_CHAR
   RET   C				;character found
   XOR   A
   RET