;=========================================================================
;
; Application Routines (LIB_printdec8/Lib_printdec16)
; Copyright (c) 2010 Ray Palmer
;
;=========================================================================
printdec_pow10   defw  10000
                 DEFW  1000
printdec_pow10b  DEFW  100
                 DEFW  10
                 DEFW  1
;=========================================================================
;Name     LIB_printdigit
;Purpose  To display a number as a digit character.
;Input    A = number
;Output   TBD
;Uses     TBD
;Notes    printf ("%02x", HL)
;=========================================================================
LIB_printdigit
   add  a,'0'				;Convert value to digit character
   jp   TXT_OUTPUT
;=========================================================================
;Name     LIB_printdiv
;Purpose  To divide reg HL by the values indexed by IX.
;Input    HL = Number
;         C = output control
;             Bit 0 = Leading zero flag       (1=yes, 0=No)
;Output   A = result
;         HL = remainder
;         C = output control
;             Bit 0 = Leading zero flag       (1=yes, 0=No)
;Uses     TBD
;Notes    TBD
;=========================================================================
LIB_printdiv
   ld   d,(ix+1)
   ld   e,(ix+0)
   inc  ix
   inc  ix
   xor   a
LIB_prtdivloop
   inc   a
   sbc   hl,de
   jr    nc,LIB_prtdivloop
   add   hl,de
   dec   a
   ret   z
   set   0,c			;Flag non-zero value
   RET
;=========================================================================
;Name     LIB_printdec8
;Purpose  To display a number as a 3 digit decimal string
;Input    A = number
;         C = output control
;             Bit 0 = Leading zero flag       (1=yes, 0=No)
;                 1 = display leading zeros   (1=Yes, 0=No)
;                 2 = display leading values  (1=Yes, 0=No)
;Output   TBD
;Uses     TBD
;Notes    printf ("%u, A)
;=========================================================================
LIB_printdec8
   push hl
   PUSH BC
   push ix
   ld   h,0
   ld   l,a
   LD   B,2
   LD   IX,prtdec_pow10b
   jr   LIB_prtdec0
;=========================================================================
;Name     LIB_printdec16
;Purpose  To display a number as a 5 digit decimal string
;Input    HL = number
;         C = output control
;             Bit 0 = Leading zero flag       (1=yes, 0=No)
;                 1 = display leading zeros   (1=Yes, 0=No)
;                 2 = display leading values  (1=Yes, 0=No)
;Output   HL = corrupted value.
;Uses     TBD
;Notes    printf ("%u, HL)
;=========================================================================
LIB_printdec16
   push hl
   PUSH BC
   push ix
   ld   ix,LIB_prtdec_pow10
   ld   b,4			;Show first 4 digits
LIB_prtdec0
   push af
   push de
;
LIB_prtdecLoop
   CALL LIB_printdiv		;Calc Most Sig. Digit
   bit  0,c			;Non-zero value?
   jr   nz,LIB_prtdec1		;Yes (display it)
;
   bit  2,c			;Display leading values?
   JR   Z,LIB_prtdec2		;No
;
   bit  1,c			;display of leading zeros as spaces
   JR   Z,LIB_prtdec1		;No
   LD   A,&F0			;code for leading spaces
LIB_prtdec1
;
   call LIB_printdigit
;
LIB_prtdec2
   djnz LIB_prtdecLoop
;
   LD   A,L			;show last digit
   call LIB_printdigit
;
   pop  de
   pop  af
   pop  ix
   pop  bc
   pop  hl
   ret