;=========================================================================
;
; Standard print Routines (printstringsafe, printcharsafe)
; Copyright (c) 2010 Ray Palmer
;
; Requires -
; 1. kernel.h
; 2. keyboard.inc
;
;=========================================================================
;Name     printstringsafe
;Purpose  To display a string without obeying control chars
;Input    HL set to point after terminating NUL
;Output   TBD
;Uses     TBD
;Notes    printf ("%s, @HL)
;=========================================================================
printstringsafe
   push  af
   push  bc
   push  de
;
prtstrsafe_loop
   ld    a,(hl)
   or    a
   JR    Z,prtstrsafe_exit
   push  hl
   call  TXT_WR_CHAR
   pop   hl
   inc   hl
   jr    prtstrsafe_loop
;
prtstrsafe_exit
   pop   de
   pop   bc
   pop   af
   ret
;=========================================================================
;Name     printcharsafe
;Purpose  Like printcharverysafe except NUL, BEL, BS, TAB, LF, CR are
;         obeyed
;Input    A = char
;Output   TBD
;Uses     TBD
;Notes    1. For tab spacing the width is a power of 2 (e.g 8, 16, etc)
;=========================================================================
printcharsafe
   or    a			;NUL?
   ret   z
;
   cp    CR			;Carriage Return (CR, 13)?
   jp    z,TXT_OUTPUT		;yes
;
   cp    BEL			;Less than BEL (7)?
   jp    c,TXT_WR_CHAR		;yes (display on screen)
;
   cp    LF + 1			;More than LineFeed (LF, 10)?
   jp    nc,TXT_WR_CHAR		;Yes
;
   cp    TAB			;TAB?
   jp    nz,TXT_OUTPUT		;No
;
;  Display TAB characters as a number of spaces upto the TAB stop index
;
   call  TXT_GET_CURSOR		;Get current screen position.
;
   LD    A,H			;Get Screen Column
   and   TAB_SPACING - 1
   sub   TAB_SPACING
   neg
   ld    b,a
   ld    a,' '			;Moves one char right
prtcs_tab_loop
   call  TXT_OUTPUT
   djnz  prtcs_tab_loop
;
   ret