;=====================================================================
;Name     DTG_wrdiv
;Purpose  A division routine for conversion of numbers to text.
;Input    E = divisor
;         A = numerator
;Output   D = INT (A/E)
;         A = remainder
;Uses     Flags changed.
;Notes    None
;=====================================================================
DTG_wrdiv
   ld    d,0			;Start counter at 0
DTG_wrdiv1
   inc   d
   sub   e			;Is value > decimal base
   jr    nc,DTG_wrdiv1		;Yes
   add   a,e			;setup remainder
   DEC   D
   RET
;=====================================================================
;Name     DTG_insdigchr
;Purpose  To insert a digit char.
;Input    HL = address to insert char
;         A = value to add as char
;         RegC.Bit0 = show leading zeros flag
;Output   HL = HL + 1
;         RegC.Bit0 set upon nonzero value
;Uses     TBD
;Notes    None
;=====================================================================
DTG_insdigchr
   OR    A			;Is digit 0?
   JR    NZ,DTG_insdigchr1	;No
   BIT   0,C			;Display leading zeros?
   RET   Z			;No
DTG_insdigchr1
   add   a,'0'			;convert to number digit char
   ld    (hl),a			;Insert char
   inc   hl			;next location to save chars
   set   0,c			;flag now inserting zero's
   RET
;=====================================================================
;Name     UTL_wrdiv16
;Purpose  To insert a digit char.
;Input    BC = address to insert char
;         HL = value to add as char
;         DE = divisor
;Output   BC = BC + 1
;         HL = remainder
;Uses     TBD
;Notes    None
;=====================================================================
DTG_wrdiv16
   PUSH  BC
   CALL  Mth_Divide
   LD    E,C
   POP   BC
   LD    A,E
   ADD   A,'0'
   LD    (BC),A
   INC   BC
   RET
;=====================================================================
;Name     DTG_WriteDay
;Purpose  To convert an unsigned decimal number in the range 0 to 31
;         to a string with no leading zeros.
;Input    HL = address to save string
;         A = value to convert
;Output   HL += sprintf (HL, "%u", A) -- without trailing NUL
;Uses     AF corrupted.
;Notes    None
;=====================================================================
DTG_WriteDay
   push  bc
   push  de
   ld    c,1			;display leading zero
   LD    E,10
   CALL  DTG_wrdiv		;D = number of 10's
   ld    e,a			;Save current result
   ld    a,d
   CALL  DTG_insdigchr		;insert digit char
   ld    A,E
   SET   0,C			;force insert of digit char
   CALL  DTG_insdigchr
   pop   de
   pop   bc
   ret
;=====================================================================
;Name     DTG_WriteYear
;Purpose  To insert a year value with 4 digits.
;Input    BC = address to save string
;         HL = value to convert
;Output   ??
;Uses     AF corrupted.
;Notes    None
;=====================================================================
DTG_WriteYear
   push  bc
   push  de
   LD    DE,1000		;DE = number of 1000's
   CALL  DTG_wrdiv16
   LD    DE,100			;DE = number of 100's
   CALL  DTG_wrdiv16
   LD    DE,10			;DE = number of 10's
   CALL  DTG_wrdiv16
   LD    A,L
   ADD   A,'0'
   LD    (BC),A
   pop   de
   pop   bc
   ret
;=====================================================================
;Name     DTG_WriteMonth
;Purpose  To insert a month text (01-Jan, 02-Feb, etc)
;Input    A = month value (1-12)
;         HL = address to put month text
;Output   ??
;Uses     AF corrupted.
;Notes    1. all other values defeult to Jan.
;=====================================================================
DTG_WriteMonth
   LD   DE,MonthsText
   OR   A
   JR   Z,DTG_WriteMonth2
   CP   13
   JR   NC,DTG_WriteMonth2
   LD   DE,MonthsText-3
   LD   B,A
DTG_WriteMonth1
   INC  DE
   INC  DE
   INC  DE
   DJNZ DTG_WriteMonth1
;
DTG_WriteMonth2
   EX   DE,HL
   LD   BC,3
   LDIR
   RET
;
MonthsText
	DEFM	"Jan"
	DEFM	"Feb"
	DEFM	"Mar"
	DEFM	"Apr"
	DEFM	"May"
	DEFM	"Jun"
	DEFM	"Jul"
	DEFM	"Aug"
	DEFM	"Sep"
	DEFM	"Oct"
	DEFM	"Nov"
	DEFM	"Dec"