;=====================================================================
;
; Standard string conversion routines (writedec8)
; Copyright (c) 2010 Ray Palmer
;
;=====================================================================
;Name     UTL_insdigchr
;Purpose  To insert a digit char.
;Input    HL = address to insert char
;         A = value to add as char
;         RegC.Bit0 = show leading zeros flag
;Output   HL = HL + 1
;         RegC.Bit0 set upon nonzero value
;Uses     TBD
;Notes    None
;=====================================================================
UTL_insdigchr
   OR    A
   JR    NZ,insdigchr1
   BIT   0,C			;Leading zero?
   RET   Z			;Yes
;
insdigchr1
   add   a,'0'			;convert to number digit char
   ld    (hl),a			;Insert char
   inc   hl			;next location to save chars
   set   0,c			;flag now inserting zero's
   RET
;=====================================================================
;Name     UTL_wrdiv
;Purpose  A division routine for conversion of numbers to text.
;Input    E = divisor
;         A = numerator
;Output   D = INT (A/E)
;         A = remainder
;Uses     Flags changed.
;Notes    None
;=====================================================================
UTL_wrdiv
   ld    d,0			;Start counter at 0
wrdiv1
   inc   d
   sub   e				;Is value > decimal base
   jr    nc,wrdiv1		;Yes
   add   a,e			;setup remainder
   DEC   D
   RET
;=====================================================================
;Name     UTL_writedec
;Purpose  To convert an unsigned decimal number in the range 0 to 255
;         to a string with no leading zeros.
;Input    HL = address to save string
;         A = value to convert
;Output   HL += sprintf (HL, "%u", A) -- without trailing NUL
;Uses     AF corrupted.
;Notes    None
;=====================================================================
UTL_writedec
   push  bc
   push  de
   ld    c,0			;assume first char is 'zero'
   LD    E,100
   CALL  UTL_wrdiv		;D = number of 100's
   ld    e,a			;Save current result
   ld    a,d
   CALL  UTL_insdigchr		;insert digit char
   ld    a,e			;Get result back for next loop
   LD    E,10
   CALL  UTL_wrdiv		;D = number of 10's
   ld    e,a			;Save current result
   ld    a,d
   CALL  UTL_insdigchr		;insert digit char
   ld    A,E
   SET   0,C			;force insert of digit char
   CALL  UTL_insdigchr
   pop   de
   pop   bc
   ret