;=====================================================================
;
; OS kernel structure definition for event and ticker blocks
;
; Reference is
;  Soft 968, Appendix 10.
;
;=====================================================================
;
;  Event block defintion
;
;  Evnt_Chain       = Event chain link
;  Evnt_Count       = Number of ticks til event kicked off
;  Evnt_Class       = Class of event
;                     Bit  Details
;                      0   Address Type (0=Far, 1=Near)
;                     1-4  Synchronous event priority
;                      5   Always Zero
;                      6   Event type (0=Normal, 1=Express)
;                      7   Synch type (0=Synchronous, 1=Asynchronous)
;  Evnt_RoutineAddr = Location of event routine
;  Evnt_RoutineROM  = ROM location of routine
;  Evnt_UserData    = event routine data (n bytes)
;
;=====================================================================
Evnt_Chain			EQU	0				;(2)
Evnt_Count			EQU	Evnt_Chain+2		;(1)
Evnt_Class			EQU	Evnt_Count+1		;(1)
Evnt_RoutineAddr		EQU	Evnt_Class+1		;(2)
Evnt_RoutineROM		EQU	Evnt_RoutineAddr+2	;(1)
Evnt_UserData		EQU	Evnt_RoutineROM+1		;(n)
Evnt_MinSz			EQU	13
;=====================================================================
;
;  Ticker Block defintion
;
;  Tick_Chain    = Ticker Queue chain link
;  Tick_Count    = Number of ticks before event is kicked.
;  Tick_Recharge = Number of ticks to recharge after event kicked.
;  Tick_EventBlk = event block as described above.
;
;=====================================================================
Tick_Chain			EQU	0				;(2)
Tick_Count			EQU	Tick_Chain+2		;(2)
Tick_Recharge		EQU	Tick_Count+2		;(2)
;
Tick_EventBlk		EQU	Tick_Recharge+2		;(n)
Tick_MinSz			EQU   Tick_EventBlk+Evnt_MinSz
;=====================================================================
;
;  Fast Ticker defintion (need to check if correct)
;
;  FastT_Chain    = Address of next ticker block
;  FastT_EventBlk = event block as described above.
;
;=====================================================================
FastT_Chain			EQU	0				;(2)
FastT_EventBlk 		EQU	FastT_Chain+2		;(n)
FastT_MinSz			EQU	FastT_EventBlk+Evnt_MinSz
;=====================================================================
;
;  Frame Flyback defintion (need to check if correct)
;
;  Frame_Chain    = Address of next ticker block
;  Frame_EventBlk = event block as described above.
;
;=====================================================================
Frame_Chain			EQU	0				;(2)
Frame_EventBlk 		EQU	Frame_Chain+2		;(n)
Frame_MinSz			EQU   Frame_EventBlk+Evnt_MinSz
;