;=====================================================================
;
; TCP/IP socket structure definition Include file
; Copyright (c) 2011 Ray Palmer
;
;=====================================================================
;
;  This structure is attached after the bind structure to add more
;  to the bind structure functionalities in assisting the API's.
;
;  socket structure definition -
;
;  s_socket_Lnk      = linking for sockets (across both TCP/UDP)
;  s_socket_SktNum   = Socket number assigned at creation.
;  s_socket_Type     = socket type assigned at creation
;  s_socket_TXMemAd  = Application memory address of the TX data
;  s_socket_TXLen    = Length of TX data to send.
;  s_socket_RXMemAd  = Application memory address of the RX data
;  s_socket_RXLen    = Length of RX data got.
;  s_socket_errno    = Error number after call to service finishes.
;  s_socket_status   = Processing Status/Flags
;                      Bit   Description
;                       7    Initialised      (0=No, 1=Yes)
;                       6    Connect Status   (0=No, 1=Yes)
;                       5    Error Status     (0=No, 1=Yes)
;                       4    Socket Status    (0=Inactive,1=Active)
;                       3    Bind Status      (0=Unbound,1=Bound)
;                       2    Listen           (0=No, 1=Yes)
;                       1    Blocking flag    (0=No, 1=Yes)
;                       0    socket bind type (0=UDP, 1=TCP)
;  s_socket_flags    = Additional flags
;                      Bit   Description
;                       7    Flag to poll this socket (0=No, 1=Yes)
;                       6    Socket has read event
;                       5    Socket has write event
;                       4    Socket has error event
;                       3    Directions  (0=Bidirect, 1=Unidirect)
;                       2    Flag for select/poll read event
;                       1    Flag for select/poll write event
;                       0    Flag for select/poll error event
;
;=====================================================================
s_socket         EQU	0
s_socket_Lnk     EQU	s_socket		;(2) DB link
s_socket_SktNum  EQU	s_socket_Lnk+2		;(1) Socket Number
s_socket_Type    EQU	s_socket_SktNum+1	;(1) Socket Type
s_socket_TXMemAd EQU	s_socket_Type+1		;(2) Addr of TX data
s_socket_TXLen   EQU	s_socket_TxMemAd+2	;(2) TX pkt length
s_socket_RXMemAd EQU	s_socket_TXLen+2	;(2) Addr of RX Data
s_socket_RXLen   EQU	s_socket_RxMemAd+2	;(2) RX pkt length
s_socket_errno   EQU	s_socket_RXLen+2	;(1) Error number
s_socket_status  EQU	s_socket_errno+1	;(1) Proc. Status/Flags
s_socket_flags   EQU	s_socket_status+1	;(1) Additional flags
;
s_socket_sizeof  EQU	s_socket_flags+1	;(size of socket struct)
;