;=====================================================================
;
; TCP/IP ticker/event handling include file.
; Copyright (c) 2010 Ray Palmer
;
;=====================================================================
;
;  Locations &ABB0 to &AC00 are not used by AMSDOS, so will use it
;  For TCP/IP.
;
;  ABB0 - ABCF  used by HDOS (32 bytes)
;                 Contains the HDOS inter-rom access routine.
;  ABD0         Reserved/Unused
;  ABD2         TCP/IP location of work space
;  ABD4         Device driver location of work space (comms only)
;  ABD6 - ABDF  Common data stores for TCP/IP and Device
;               &ABD6   = Ticker control
;                         Bit  Description
;                          0   TCP/IP Event Handler Status
;                              (0=Not active, 1=Active)
;                          1   Device Event Handler Status
;                              (0=Not active, 1=Active)
;                          2   1 sec Timing Flag for DHCP
;                          3   ARP Packet Status
;                          4   TCP/IP Handler Switch (0=Off, 1=On)
;                          5   Device Handler Switch (0=Off, 1=On)
;                          6   DHCP Handler Switch   (0=Off, 1=On)
;                              =0 if DHCP is fixed
;                          7   ARP Handler Switch    (0=Off, 1=On)
;               &ABD7/8 = 1 Second Counter (in 1/300th ticks)
;               &ABD9   = TCP/IP Configuration/Flags
;                         Bit  Description (0=No, 1=Yes)
;                          0   DHCP being configured
;                          1   ARP being configured
;                          2   PPPoE being configured
;                          3   PPP (LCP, IPCP, etc) being configured
;                          4   Unused (Must Be zero)
;                          5   PPP Mode
;                          6   SLIP Mode
;                          7   Van Jacobson Encapsulation for PPP (RFC1144)
;               &ABDA   = TCP/IP Configuration completed phases
;                         Bits 0 to 3 match TICK_CFG
;                         Bit 4 - System configured.
;               &ABDB   = General Status/Flags.
;               &ABCC-F = Register HL/DE for Rom Manager.
;  ABE0 - ABEF  event data stores for TCP/IP
;  ABF0 - ABFF  event data stores for Comms Device Driver.
;=====================================================================
REGIYTCPIP		EQU &ABD2	   ;(2) TCP/IP Wrk Space
REGIYDEVICE		EQU &ABD4	   ;(2) TCP/IP Dev Wrk Space
;=====================================================================
TICK_CTRL		EQU &ABD6	   ;(1) Ticker control
;=====================================================================
TICK_SECCTR		EQU &ABD7	   ;(2) Ctr for 1 sec timing
;=====================================================================
TICK_CFG		EQU &ABD9	   ;(1) Config required
TICK_CFGDONE		EQU &ABDA	   ;(1) Config completed
;=====================================================================
;  TICK_CFG_WRK = work variable
;=====================================================================
TICK_CFG_WRK		EQU &ABDB	   ;(1) Work var for config
;=====================================================================
TempWrkLoc		EQU &ABDC	   ;(4) Temp HL/DE (rommgr)
;=====================================================================
TICK_REGS		EQU &ABE0	   ;(10) Regs BC,DE,HL,IX,IY
TICK_DEVHAND		EQU TICK_REGS+10   ;(2) Dev Handle for ticker
TICK_CNTR		EQU TICK_DEVHAND+2 ;(3) Ticker counter
TICK_CYCLE		EQU TICK_CNTR+3	   ;(1) Tick Cycle Counter
;=====================================================================
