;=========================================================================
;
; Browser application structures
; Copyright (c) 2010 Ray Palmer
;
;=========================================================================
;  Memory Address structure
;
;  S_MemAdr    = 16bit address of memory in Z80 address space
;  S_MemBank   = 8bit bank indexing byte
;                Bit  Description
;                7    Memory type (ROM/RAM)
;                     = 1 means ROM
;
;                     = 0 mean RAM
;
;                6-0  ROM/ROM Index or Bank Index
;
;                     For ROM indexing the limits to ROM access is based
;                     the type of machine used and hardware attached.
;
;                     For RAM bank indexing the maximum index is 32 for
;                     default 512k Dk'Tronics memory expansion.
;
;                     Where the memory expansion is 2 MegaBytes memory,
;                     the expansion makes use of bits 8/9 (normally used
;                     by the PPI chip) to extend the capacity.
;
;=========================================================================
S_Memory	EQU	0
S_MemAdr	EQU	S_Memory
S_MemBank	EQU	S_Memory+2
;
S_MemorySz	EQU	S_MemBank+1
;=========================================================================
;  Styles Database entry structure
;
;  S_StyleAdr  = HTML location of the style command
;  S_StyleBnk  = HTML Bank index
;  S_StyleEAd  = Address of loaded external style data.
;                0 = no data loaded.
;
;=========================================================================
S_Style		EQU	0
;
S_StyleAdr	EQU	S_Style
S_StyleBnk	EQU	S_Style+2
S_StyleEAd	EQU	S_Style+3
;
S_StyleSz	EQU	S_StyleEAd+3
;
MAX_STYLES	EQU	30			;max <STYLE> commands
StylesDB_Sz	EQU	MAX_STYLES*S_StyleSz
;=========================================================================
;  Styles Settings Database entry structure
;
;  S_StyleSFlags = bit flags of style handling
;                   BIT  Description
;                    7   Style change status (0=no change, 1=changed)
;                    6   inverse effect (0=No, 1=Yes)
;                    5   Font spacing (0=Fixed, 1=Proportional)
;                    4   Font type (0=Serif, 1=Sans-serif)
;                    3   Strike through (0=None, 1=strike through)
;                    2   Underline (0=None, 1=Underline)
;                    1   Font type (0=Normal, 1=Italic)
;                    0   Font has been changed (0=No, 1=new font selected)
;
;  S_StyleSStatus = bit flags of style handling
;                   BIT  Description
;                    7   Valid settings (0=Yes, 1=No)
;                    6   Object border start (0=No, 1=Yes)
;                    5   Object border end (0=No, 1=Yes)
;                    4   Unused/Reserved
;                    3   Border for <FORM> (0=No, 1=Yes)
;                    2   Border for <P> (0=No, 1=Yes)
;                    1   newline before flag (0=No, 1=Yes)
;                    0   newline after flag (0=No, 1=Yes)
;
;  S_StyleSBordT = Style border type (based on parameter type)
;  S_StyleSBordW = style border width
;                  0 = no border
;  S_StyleSFntNme = Index into DB of fonts loaded.
;
S_StyleSet	EQU	0
S_StylesFlags	EQU	S_StyleSet		;Flag
S_StylesStatus	EQU	S_StyleSet+1		;Status/Flags
S_StylesBordT	EQu	S_StyleSet+2		;Border type
S_StylesBordW	EQU	S_StyleSet+3		;Border width
S_StylesText	EQU	S_StyleSet+4		;Text control Flags
S_StylesMargL	EQU	S_StyleSet+5		;Left Margin offset
S_StylesMargR	EQU	S_StyleSet+6		;Right Margin offset
S_StylesFntNme	EQU	S_StyleSet+7		;DB index
;
S_StyleSetSz	EQU	S_StyleSFntNme+2
;
S_StyleSetMax	EQU	64			;Max entries
;
CFG_StylSetDBSz	EQU	S_StyleSetSz*S_StyleSetMax
;=========================================================================
;  Images Database entry structure
;
;  S_ImageFlag = File type of loaded object.
;  S_ImageAdr  = Address of the the HTML command loaded as 3 bytes
;  S_ImageDat  = location of the image data as 3 bytes
;  S_ImageMode = Screen mode image is formatted to.
;  S_ImageRows = number of characters rows.
;  S_ImageCols = number of characters columns.
;
;=========================================================================
S_Image		EQU	0
;
S_ImageFlag	EQU	S_Image
S_ImageAdr	EQU	S_Image+1	;3 byte address of HTML command
S_ImageDat	EQU	S_Image+4	;3 byte address of image data
S_ImageMode	EQU	S_ImageDat+3	;Screen mode of image data
S_ImageRows	EQU	S_ImageMode+1	;Number of character rows
S_IMageCols	EQU	S_ImageRows+2	;Number of characters columns
;
S_ImageSz	EQU	S_ImageCols+2	;size of db entry
;
Max_ImageDB	EQU	32
ImageDB_Sz	EQU	Max_ImageDB*S_ImageSz
;=========================================================================
;  CPC window structure
;
;  S_StartRow  = Absolute start row address on the web-page canvas
;  S_StartCol  = Absolute start column address on the web-page canvas
;  S_EndRow    = Absolute end row address on the web-page canvas
;  S_EndCol    = Absolute end column address on the web-page canvas
;
;=========================================================================
S_CPCWdw	EQU	0
;
S_StartRow	EQU	S_CPCWdw
S_StartCol	EQU	S_StartRow+2
S_EndRow	EQU	S_StartCol+2
S_EndCol	EQU	S_EndRow+2
;
S_CPCWdwSz	EQU	S_EndCol+2
;=========================================================================
;  Hyper-link structure for those links shown on the web-page during the
;  rendering process.
;
;  The data for all hyperlinks are kept in the main memory above &4000 in
;  the default memory segment.
;
;  Where the size of space for Hyperlinks is 4096 bytes there can be a
;  maximum of 370 links per screen shown of HTML.
;
;  S_HLSCol = Start column on the web-page (word)
;  S_HLSRow = Start row on the web-page (word)
;  S_HLECol = End column on the web-page (word)
;  S_HLERow = End row on the web-page (word)
;  S_HLLocn = 3 byte RAM bank location of the HTML command for hyperlink
;             information.
;             Byte    Description
;              0-1    Address in Bank
;               2     HTML Bank vector index
;
;  All column and row information is the absolute web-page location.
;
;=========================================================================
S_HLinkDb	EQU	0
;
S_HLSRow	EQU	S_HLinkDb+S_StartRow
S_HLSCol	EQU	S_HLinkDb+S_StartCol
S_HLERow	EQU	S_HLinkDb+S_EndRow
S_HLECol	EQU	S_HLinkDb+S_EndCol
S_HLLocn	EQU	S_HLinkDb+S_CPCWdwSz
;
S_HLSz		EQU	S_HLLocn+3
;=========================================================================
;  Links Table defintions (also for URL objects)
;
;   S_LnkTblState = Bit flags of URL access.
;                   Bit  Description
;                    7   URL Source (0=Other, 1=Link Mgr/Page Open/Hyperlink)
;                    6   URL type (0=HTTP, 1=File)
;                    5   Filename available (0=No, 1=Yes)
;                    4   Cache File (0=No, 1=Yes)
;                    3   Local File (0=No, 1=Yes)
;                    2   Reserved
;                    1   Reserved
;                    0   History Item to update (0=No, 1=Yes)
;
;   S_LnkTblName = Name of local file (max 15 chars)
;                  (the link DB has the name of the link)
;   S_LnkTblURL  = start of URL with 'http' or 'file'
;                  Max of 128 bytes
;
;=========================================================================
S_LnkTblNameSz	EQU	15
;
S_LinksTable	EQU	0
S_LnkTblState	EQU	S_LinksTable
S_LnkTblName	EQU	S_LnkTblState+1
S_LnkTblURL	EQU	S_LnkTblName+S_LnkTblNameSz
;
S_LinksTableSz	EQU	S_LnkTblURL+URL_Size
;=========================================================================
;  Applications Table defintions
;
;  S_AppState = State of Application for Manager
;                  Bit  Description
;                   7   App deletion status (0=Available, 1=Deleted)
;                   6   App state (0=Disabled, 1=Enabled)
;                   5   Entry Type (0=Application, 1=Handler)
;                          Applications are files with .exe
;                          Handler are files with .bin
;                   4   App image (0=File, 1=Rom)
;                   3   Error in Application occured during startup
;                   2   Execution type (0=Many times, 1=Once only)
;                   1   Entry taken (0=No, 1=Yes)
;                   0   Parameter Source (0=keyboard, 1=supplied)
;                  If all bits clear then entry is invalid
;  S_AppName       = General Name of Application (18 chars)
;  S_AppFileName   = Filename for Application (13 characters)
;
;                    If ROM based App then filename is xxxx-yy.ROM
;                    Where xxxx is name of rom and yy is ROM number.
;
;  Size of structure is 32 bytes
;
;=========================================================================
A_MaxAppName		EQU	18
A_MaxAppFileName	EQU	13		;HDOS/AMSDOS filename length
;
S_AppsTable		EQU	0
S_AppsState		EQU	S_AppsTable
S_AppsName		EQU	S_AppsState+1
S_AppsFileName		EQU	S_AppsName+A_MaxAppName
;
S_AppEntSize		EQU	S_AppsFileName+A_MaxAppFileName
;=========================================================================
;  Applications DB structure
;
;  S_AppFlag  = Flags/Status
;                Bit  Description
;                 7   Available Status (0=Yes, 1=No)
;                     CPCB can override bit to 1 if setting disables access
;                     on encountering <applet> commands.
;                6-0  Unused/Reserved
;               All bits set infers end of DB
;  S_AppIDNum = ID number of the application
;  S_AppsWdw  = Frame Number where Icon is shown
;  S_AppSRow  = web-page frame start row for icon
;  S_AppSCol  = web-page frame start column icon
;  S_AppsHTM  = HTML location of command for application data (if any)
;
;=========================================================================
S_AppsDB	EQU	0
;
S_AppFlag	EQU	S_AppsDB
S_AppIDNum	EQU	S_AppFlag+1
S_AppsWdw	EQU	S_AppIDNum+1
S_AppSRow	EQU	S_AppsWdw+1
S_AppSCol	EQU	S_AppSRow+2
S_AppsHTM	EQU	S_AppSCol+2
;
S_AppsDBSz	EQU	S_AppsHTM+3
;=========================================================================
;  Page Scrolling DB defintions
;
;  S_SP_WPRow      = Row number on web page (frames Independent)
;  S_SP_Addr       = Address in Bank for line number
;  S_SP_Flags      = Flags to handle the DB of Web Pages
;                    Bit  Description
;                     7   End of HTML data (0=No, 1=Yes)
;                         (Non Frames HTML only)
;                     6   DB entry setup
;                     5   Unused
;                     4   Unused/Reserved
;                    3-0  HTML Bank Number
;
;=========================================================================
S_SP_WPRow		EQU	0		;(2) Line number (0-65535)
S_SP_Addr		EQU	S_SP_WPRow+2	;(2) Address in HTML bank
S_SP_Flags		EQU	S_SP_Flags+1	;(1) Flags
;
S_SPSz			EQU	S_SP_Flags+1	;Size of structure
;=========================================================================
;  Z80 Register Stack structure
;
;=========================================================================
S_Stack			EQU	0
S_RegAF			EQU	S_Stack
S_RegBC			EQU	S_RegAF+2
S_RegDE			EQU	S_RegBC+2
S_RegHL			EQU	S_RegDE+2
S_RegIX			EQU	S_RegHL
S_RegIY			EQU	S_RegIX
;
S_StackSz		EQU	S_RegIY+2
;=========================================================================
;
;  Defintions of the configuration data bank containing the
;  CPC Web browser controls/settings.
;
;  The configuration settings are loaded from a file and some of these
;  are setup in the main application variables.
;
;=========================================================================
;
;   Define location of Home page URL
;
CFG_HomePage		EQU	SysBnkBuffer
;
;   Define location of the start up page URL.
;
CFG_StartUp		EQU	CFG_HomePage+URL_Size
;
;   Application location for external executables
;
CFG_AppLocn		EQU	CFG_StartUp+URL_Size
;
;   System location for internal components.
;
CFG_SysLocn		EQU	CFG_AppLocn+URL_Size
;
;   System Save location for downloads
;
CFG_SaveLocn		EQU	CFG_SysLocn+URL_Size
;
;   System cache location for the cache of HTTP requests.
;
CFG_CacheLocn		EQU	CFG_SaveLocn+URL_Size
;
;   Base URL to build URLs from
;   The variable is dynamic during any session of the browser
;
CFG_BaseURL		EQU	CFG_CacheLocn+URL_Size
;
CFG_CurKey_Up		EQU	CFG_baseURL+URL_Size
CFG_CurKey_Down		EQU	CFG_CurKey_Up+1
CFG_CurKey_Right	EQU	CFG_CurKey_Down+1
CFG_CurKey_Left		EQU	CFG_CurKey_Right+1
CFG_CurKey_Select	EQU	CFG_CurKey_Left+1
CFG_CurKey_TAB		EQU	CFG_CurKey_Select+1
CFG_CurKey_LSHFT	EQU	CFG_CurKey_TAB+1
CFG_CurKey_RSHFT	EQU	CFG_CurKey_LSHFT+1
CFG_Cursor_Icon		EQU	CFG_CurKey_RSHFT+1
CFG_Cursor_IcnSet	EQU	CFG_Cursor_Icon+1
;
;  Line Wrap Control Flag
;   = 0 means lines dont wrap until max line reached
;       (max line width is 65535 chars)
;   = 1 means lines wrap at screen width
;
CFG_WrapCtrl		EQU	CFG_Cursor_IcnSet+1
;
;  Page Scroll Lock flag
;   =0 means page scrolls 1 line at a time
;   =1 means page scrolls 20 lines at a time.
;
CFG_PageCtrl		EQU	CFG_WrapCtrl+1
;
;  CFG_HistURLs = History of URLs requested
;
CFG_HistURLsSz    	EQU	URL_MaxItems*URL_Size
CFG_HistURLs		EQU	CFG_PageCtrl+1
;
;  CFG_StylesDB = Database of location of the styles commands
;                 Max size is CFG_HistURLsSz (8 URLs)
;
CFG_StylesDB		EQU	CFG_HistURLs+CFG_HistURLsSz
;
;  CFG_StylesSetDB = database/stack of style settings for each command
;                 Max size is StylesDB_Sz (30 style settings)
;
CFG_StylesSetDB		EQU	CFG_StylesDB+StylesDB_Sz
;
;  CFG_ImagesDB = Database of image locations loaded
;                 Max size is ImageDB_Sz (32 images)
;
CFG_ImagesDB		EQU	CFG_StylesSetDB+CFG_StylSetDBSz
;
;  CFG_AppsDB = Database of image locations loaded
;               Max of 16 Apps per web-page shown.
;
CFG_MAXAPPSDB		EQU	16
CFG_AppsDB		EQU	CFG_ImagesDB+ImageDB_Sz
;
;  CFG_formsDB = Database of Forms input items like buttons, text input
;                and others.
;                As of 05-Jan-2013 address is &684A
;                The number of entries is expected to not go beyond &7000
;                meaning there can be 154 forms input items types.
;
;                The remaining space from &7000 to &7fff may be used for
;                text inputs. Each text input is limited to the following
;                a. for sinlge line inputs - 128 bytes (ie size of URL)
;                b. For multi-line text - 512 bytes.
;                The above makes it possible to have for example 2 multi
;                line inputs and 24 single inputs for a given form defn.
;
;  Databse entry size = &13 bytes (19 decimal) as of 05-Jan-2013
;
;  Max entries is approximately 200
;
CFG_FormsDB		EQU	CFG_AppsDB+S_AppsDBSz*CFG_MAXAPPSDB
;
;  CFG_FormsBuf = space for textural inputs for single line entries the
;                 allocations are 128 bytes, while for multi-line the
;                 allocations are 512 bytes.
;
CFG_FormsBuf		EQU	&7000
;