;===============================================================================
;
; Browser application Variables
; Copyright (c) 2010 Ray Palmer
;
;===============================================================================
;
;  Global variables
;
;   PageLock = Flags controlling page scrolling
;              Bit Description
;              7-4 Unused/Reserved
;               3  Horizontal Scroll lock (0 = No lock, 1 = Lock)
;               2  Vertical Scroll lock (0 = No lock, 1 = Lock)
;               1  Main Scroll lock (0 = No lock, 1 = Lock)
;                  if lock then bits 2/3 define which are locked
;                  If bits 2/3 both zero then no lock (Default)
;               0  Unused/Reserved
;              
;===============================================================================
TCPIP_State	EQU	CPCB_StartVars	;State of TCP/IP API.
MaxAPIs		EQU	TCPIP_State+1	;Max API entries
SysLogging	EQU	MaxAPIs+1	;Logging controls
Flag_Scroll	EQU	SysLogging+1	;Scroll Flag for cursor movement.
WrapLock	EQU	Flag_Scroll+1	;Assume no lock (text wraps)
PageLock	EQU	WrapLock+1	;Assume page will not scroll
LogURLs		EQU	PageLock+1	;Assume no logging of URLs
LogFiles	EQU	LogURLs+1	;Assume no file access logging.
SysExit		EQU	LogFiles+1	;Exit type (0=Normal,>0 Reboot)
FindResult	EQU	SysExit+1	;Assume not found
StreamNumbers	EQU	FindResult+1	;Current Streams for render
Stream_MainMenu	EQU	StreamNumbers+0	;Stream 0 (main menu)
Stream_WebPage	EQU	StreamNumbers+1	;Stream 1 (Web Page)
Stream_SubMenu	EQU	StreamNumbers+2	;Stream 2 (sub-menus in web-page)
Stream_Frame	EQU	StreamNumbers+3	;Stream 3 for frames.
;===============================================================================
;
;  Memory Banks for web browser
;
;  HTML Banks
;  ----------
;
;  There are 96 banks meaning the browser can handle at most 1536K bytes of
;  HTML source/objects.  The loading of the HTML page which was downloaded
;  from the internet may have any unhandled HTML command removed during the
;  loading process.
;
;  Configuration Bank
;  ------------------
;
;  The configuration bank contains the browser configuration settings.
;
;  The configuration data is loaded up via a parser which is also loaded
;  from the media as an independent app to setup the browser at startup.
;
;  Links Bank
;  ----------
;
;  The Links Bank contains the list of knowns URLs which the user has
;  saved for later access during current browser session. There can be a
;  maximum of 100 urls.
;
;  Code Banks
;  ----------
;
;  The 3 code banks are for internal applications required for the
;  browser to operate.
;  The first two banks are for internal applications/routines, while the third
;  bank is for use by the external applications (e.g hello.exe).
;
;  Bank 0       - Code for URL/HDOS
;  Bank 1       - Code for Menu and Font handling.
;  Bank 2 to 30 - User defined code (224K)
;
;  Application Bank
;  ----------------
;
;  The Application bank contains the applications and filenames for the
;  user defined applications for the browser.
;  Max Applications the browser can handle is 1536K.
;
;  Bank's Stack
;  ------------
;
;  This area of memory contains the stack of bank memory information as defined
;  by the S_Memory structure for the bank type. The stack allows for multiple
;  successive routine calls to know which bank to return to.
;
;===============================================================================
CurrMemBank	EQU	StreamNumbers+4	;Current swapped in bank
MaxCodeBanks	EQU	CurrMemBank+1	;Number of code banks (256K, 224K User)
MaxHTMLBanks	EQU 	MaxCodeBanks+1	;Number of HTML banks (1536K)
ConfigBank	EQU	MaxHTMLBanks+1	;RAM Bank of memory for config.
LinksBank	EQU	ConfigBank+1	;RAM Bank of loaded link URLs
AppsBank	EQU	LinksBank+1	;RAM Bank for Web Apps details
CodeBanks	EQU	AppsBank+1
HTMLBanks	EQU	CodeBanks+1
BankStack	EQU	HTMLBanks+1
BankIndx	EQU	BankStack+BankStackSz
;===============================================================================
;
;  General variables
;
;
;  SysHandlerData = general structure of how commands are to be handled.
;
;                 Byte  Description
;                  0    Command Table information type
;                       Bit Description
;                       7   Fixed at 1.
;                       6   Command has a matching end HTML command.
;                           e.g </html> ...... </html>
;                       5   Command can be handled by browser
;                           (0=No, 1=Yes)
;                       4   Command may have additional data following.
;                           (0=No, 1=Yes)
;                           e.g object data for jpgs
;                       3   Command restrictions (0=Head, 1=Body)
;                       2   Unused/reserved
;                       1   Command has style handler
;                       0   Command in action (0=off, 1=On)
;
;                 1-3   Handler Address and bank (see cpc-s.inc)
;
;  BrowserState = Bit flags of browser main loop actions
;                 Bit Description
;                  7  Open a New URL (1 = data in URL_Object, 0 = No new URL)
;                 6-1 Unused
;                  0  Fixed to 1.
;                 if all bits are zero then browser exits.
;
;===============================================================================
SysHandlerData	EQU	BankIndx+2
BrowserState	EQU	SysHandlerData+4	;State of app (0=exit else not)
URL_FnameLoc	EQU	BrowserState+1		;Location of filename
FileLoadAddr	EQU	URL_FnameLoc+2
CurrLocalDev	EQU	FileLoadAddr+2		;0=AMSDOS,1=HDOS
FileOpState	EQU	CurrLocalDev+1
CurrLocDevice	EQU	FileOpState+1
FileNumbr	EQU	CurrLocDevice+1		;File number for HTTP downloads
CPCB_Timer	EQU	FileNumbr+1		;Timer Counter for display
FIO_CodeBlk	EQU	CPCB_Timer+1		;FIO binary memory block to load
FIO_Ctrl	EQU	FIO_CodeBlk+1		;File load controller flag
;===============================================================================
;
;  Variables for the management of the Hyperlinks DB for for the Links Manager
;
;===============================================================================
LM_Index	EQU	FIO_Ctrl+1		;Index into list of DB entries
LM_LastDB	EQU	LM_Index+1		;Last DB index
LM_TopIndex	EQU	LM_LastDB+1		;Top DB entry in window
LM_LastIndex	EQU	LM_TopIndex+1
LM_NumWIdx	EQU	LM_LastIndex+1		;Bottom DB entry in window
LM_RowCol	EQU	LM_NumWIdx+1
LM_DBIndex	EQU	LM_RowCol+2		;Links Mgr DB index
;===============================================================================
;
;  Variables for the management of Fonts
;
;  FontType = Type of font
;             0 = Fixed space font
;             >0 = Proportional font
;
;===============================================================================
NumFonts	EQU	LM_DBIndex+1		;Number of fonts loaded
NumIcons	EQU	NumFonts+1
FontId		EQU	NumIcons+1		;Font ID number
DefFont		EQU	FontId+1
Font_Name	EQU	DefFont+1		;Location of fonts name
FontType	EQU	Font_Name+2		;Type of font
FontCC		EQU	FontType+1
FontNC		EQU	FontCC+1
FontOutput	EQU	FontNC+1		;Type of font output
;===============================================================================
;
;  Variables to handling screen colours and modes
;
;===============================================================================
Pen0		EQU	FontOutput+1		;ink 0
Pen1		EQU	Pen0+1			;ink 1
Pen2		EQU	Pen1+1			;ink 2
Pen3		EQU	Pen2+1			;ink 3
Border0		EQU	Pen3+1			;Border
;
Scrn_Mode	EQU	Border0+1		;Screen Mode
Scrn_MaxCol	EQU	Scrn_Mode+1		;Max coloumns on screen
;
Menu_ColIdx	EQU	Scrn_MaxCol+1		;Menu Text colours
Text_Colidx	EQU	Menu_ColIdx+1		;Mode 1 colours for text
Icon_ColIdx	EQU	Text_Colidx+1		;Mode 1 colours for icons
Curs_Colidx	EQU	Icon_ColIdx+1		;Mode 1 colours for cursor
Frme_ColIdx	EQU	Curs_Colidx+1		;Mode 1 Frame colours
Brdr_ColIdx	EQU	Frme_ColIdx+1		;Mode 1 Border colours
;===============================================================================
;
;  Variables to handle error messages/prompts
;
;  ErrMsg_Cntrl = Flags/Status Control or error handling
;                 Bit  Details
;                  7   0 = Display Error messages else 1 = no display.
;                 6-0  Unused/Reserved
;
;===============================================================================
ErrMsg_Loc	EQU	Brdr_ColIdx+1		;Location of message to show
ErrMsg_Cntrl	EQU	ErrMsg_Loc+2		;Error message control byte
MsgStream	EQU	ErrMsg_Cntrl+1
;===============================================================================
;
;  Variables for the management of URLs and URL history
;
;   URL_Object = Current URL object being processed
;                  (structure in cpcb-s.inc)
;
;  URL_ObjectNum = number of object downloaded and is used in part of the
;                  filename.
;                  e.g File-xxx.htm
;
;  URL_ObjectSize = Size of the object loaded.
;                   - For binary files the objects size is via header for AMSDOS
;                     only. For multi-segment binaries then the 3rd byte
;                     indicates the number of segments used.
;                   - For text the size is a 3 byte value where the 3rd byte is
;                     the number of 16k segments used.
;
;===============================================================================
URL_CurrURLSz	EQU	MsgStream+1		;current size of URL entered.
URL_OpenType	EQU	URL_CurrURLSz+1		;What sort of URL is being requested
URL_CurrItem	EQU	URL_OpenType+1		;Current URL index into history
URL_LastItem	EQU	URL_CurrItem+1		;Last available item in history
URL_AddrLoc	EQU	URL_LastItem+1		;Address of URL in table.
URL_HistFlag	EQU	URL_AddrLoc+2		;Flag status of history update
URL_NewLocn	EQU	URL_HistFlag+1
URL_LblAddr	EQU	URL_NewLocn+2		;Location of last # in URL (if any)
URL_LblType	EQU	URL_LblAddr+2		;Type of text to find
URL_EndAddr	EQU	URL_LblType+1		;Address of loaded URL object
URL_Position	EQU	URL_EndAddr+3
URL_Object	EQU	URL_Position+2
URL_ObjState	EQU	URL_Object+S_LnkTblState	;Object state
URL_ObjName	EQU	URL_Object+S_LnkTblName		;Filename
URL_ObjStrtTxt	EQU	URL_Object+S_LnkTblURL		;URL + object name
URL_ObjectNum	EQU	URL_Object+S_LinksTableSz	;Object Number
URL_ObjectSize	EQU	URL_ObjectNum+1			;Object size
;===============================================================================
;
;  Variables for the management of windows shown in the web page area.
;
;  WindowFrmeNum = Frame clicked on or the current frame for the applciation
;
;  WindowFrmeStat = status flags of frame display
;                   Bit  Description
;                    7   Frame update (1=display complete, 0=no)
;                   6-0  Unused/Reserved
;
;  Window_FrTmp = Contains the windows frame coordinates.
;                  the values all start from 0 rather than 1
;
;  Window_RowOff = Start offset for the frame to be drawn on the screen
;  Window_ColOff = Start offset for the frame to be drawn on the screen
;
;  Boths values are 0 for the Main menu frame and for sub-menus
;                   4 and 0 for the web-page frames in HTML
;
;  HTML_FrameFlg = Frames processing status
;                    Bit  Description
;                     7   Frames are to requested be loaded
;                         0 = no frames requested to be loaded else 1
;                     6   Frame load request in progress
;                         0 = no frame is being loaded else 1
;                     5   All frames loaded
;                         1 = frames are yet to be loaded else 0
;                     4   Frames not used
;                         1 = frames have been encountered.
;                         0 = no frameset/frame command encountered
;                    3-0  Frame Counter (0 = Reserved for internal use)
;                  Default value for bit 7 to 4 are 0
;
;  FrameItemSel = Which item was clicked on for a scrollable frame
;                  Bit  Description
;                   7   Frame type (0=Non scrollable)
;                   6   Frames Loaded
;                   5   Slider Icon (Horizontal)
;                   4   Slider Icon (Vertical)
;                   3   Scroll Up icon
;                   2   Scroll Down icon
;                   1   Scroll left icon
;                   0   Scroll right icon
;
;===============================================================================
Windw_state	EQU	URL_ObjectSize+3	;0=Open, 1=Closed
Windw_Row	EQU	Windw_state+1		;Base Col/Row for window items
Windw_Col	EQU	Windw_Row+1
Windw_NumCols	EQU	Windw_Col+1		;Num columns to window
Windw_NumRows	EQU	Windw_NumCols+1		;Num rows to window
Windw_ICol	EQU	Windw_NumRows+1		;Item Col/Row work variable
Windw_IRow	EQU	Windw_ICol+1
MenuWinDatLoc	EQU	Windw_IRow+1		;Location of menu window data
MenuWinKeyLoc	EQU	MenuWinDatLoc+2		;Location of window keys data
MenuKeysLoc	EQU	MenuWinKeyLoc+2		;Location of keys for menu
WindowFrameID	EQU	MenuKeysLoc+2		;Current Frames DB id
Window_FrameV	EQU	WindowFrameID+1		;Frames/Borders Object
Window_FrmeID	EQU	Window_FrameV+S_HTMLFramID
Window_FrmeSF	EQU	Window_FrameV+S_HTMLFramSF
Window_SRow	EQU	Window_FrameV+S_HTMLFramBr
Window_SCol	EQU	Window_FrameV+S_HTMLFramBc
Window_ERow	EQU	Window_FrameV+S_HTMLFramNr
Window_ECol	EQU	Window_FrameV+S_HTMLFramNc
Window_Roff	EQU	Window_FrameV+S_HTMLFramRo
Window_Coff	EQU	Window_FrameV+S_HTMLFramCo
Window_Addr	EQU	Window_FrameV+S_HTMLFramAd
WindowFrmeNum	EQU	Window_FrameV+S_HTMLFramSz	;Frame clicked on
WindowFrmeStat	EQU	WindowFrmeNum+1			;window frame status
Framechk	EQU	WindowFrmeStat+1
FrameCtr	EQU	Framechk+1
Window_FrTmp	EQU	FrameCtr+1		;Temp vars for frame borders
Window_FSRow	EQU	Window_FrTmp+S_StartRow
Window_FSCol	EQU	Window_FrTmp+S_StartCol
Window_FERow	EQU	Window_FrTmp+S_EndRow
Window_FECol	EQU	Window_FrTmp+S_EndCol
Window_FRowOff	EQU	Window_FrTmp+S_CPCWdwSz	;Relative offsets
Window_FColOff	EQU	Window_FRowOff+1
FramesDBLoc	EQU	Window_FColOff+1	;Location of Frames DB
FramesDBMax	EQU	FramesDBLoc+3		;max frames allowed
HTML_FrameFlg	EQU	FramesDBMax+1		;Frames handling flag
FrameItemSel	EQU	HTML_FrameFlg+1		;Which item on scroll frame
FrameWinType	EQU	FrameItemSel+1		;Frame type being displayed
BorderIconSet	EQU	FrameWinType+1
BorderIconStrt	EQU	BorderIconSet+1
BorderCorner	EQU	BorderIconStrt+1
SysBorderSet	EQU	BorderCorner+1		;Icon set for borders
BaseHTMLLoc	EQU	SysBorderSet+1		;Base HTML for new files on load
;===============================================================================
;
;  Variables to manage the input of user interaction (excluding cursor movement)
;  KB_Buffer may be used by the page rendering routines.
;
;  KB_EchoFlag = Flags controlling echo back of text on input.
;                BIT  Description
;                 7   Multi-line output (1 = Yes, 0 = No)
;                6-1  Unused
;                 0   Hidden output (1=Yes, 0 = No)
;
;  KB_KeyTested = 4 byte data structure indicating which key was pressed on the
;                 keyboard or joystick
;
;                 Byte  Description
;                  0    Key Number of the key pressed
;                 1-2   Address of handler for the key
;                  3    Memory type for handler (see cpcb-s.inc 'S_Memory')
;
;  Cursor_SelType = What type fo secltion has been engaged by the cursor select
;                   key.
;                   Values  Description
;                     0     Normal space key for entry of text
;                     1     Hyperlink action required.
;                     2     Forms item selection (including text input select)
;                     3     Frame select
;                     255   Nothing selected.
;
;  PageWdw_Var = CPC Screen offset into the web-page canvas.
;                The offsets are the actual web-page canvas locations.
;
;  WebPge_Row = clicked on row position on the web-page regardless of the frame.
;  WebPge_Col = clicked on col position on the web-page regardless of the frame.
;
;===============================================================================
KB_StartCol	EQU	BaseHTMLLoc+3
KB_StartRow	EQU	KB_StartCol+1
KB_BaseInpRC	EQU	KB_StartCol+2		;copy of KB start Row/Col
KB_MaxInpRC	EQU	KB_BaseInpRC+2
KB_MaxInpRow	EQU	KB_MaxInpRC
KB_MaxInpCol	EQU	KB_MaxInpRC+1
KB_EchoFlag	EQU	KB_MaxInpRC+2		;Echo text control Flag
KB_printlen	EQU	KB_EchoFlag+1		;Max length of echo text
KB_BuffLoc	EQU	KB_printlen+2		;Loc'n of KB buffer for KB input
KB_Buffer	EQU	KB_BuffLoc+2 		;default keyboard
KB_SrchTxt	EQU	KB_Buffer+KB_BuffSz	;Search Text
KB_Offset	EQU	KB_SrchTxt+KB_SrchSz	;Work var for KB input
KB_KeyListLoc	EQU	KB_Offset+2		;Address of keys to check.
KB_KeyTested	EQU	KB_KeyListLoc+2
KB_KeyDelayT	EQU	KB_KeyTested+4	;delay time for key wait (non-cursor)
Cursor_Icon	EQU	KB_KeyDelayT+2
Cursor_IcnSet	EQU	Cursor_Icon+1
Cursor_State	EQU	Cursor_IcnSet+1
Cursor_ICol	EQU	Cursor_State+1	;Col Position in CPC window
Cursor_IRow	EQU	Cursor_ICol+2	;Row Position in CPC window
Cursor_Speed	EQU	Cursor_IRow+2	;Cursor speed (configurable)
Cursor_Col	EQU	Cursor_Speed+2	;Col Position in CPC window
Cursor_Row	EQU	Cursor_Col+2	;Row Position in CPC window
Cursor_SelType	EQU	Cursor_Speed+2	;Type of selection from cursor select
PageWdw_Var	EQU	Cursor_Row+2	
PageWdw_SRow	EQU	PageWdw_Var+S_StartRow
PageWdw_SCol	EQU	PageWdw_Var+S_StartCol
PageWdw_ERow	EQU	PageWdw_Var+S_EndRow
PageWdw_ECol	EQU	PageWdw_Var+S_EndCol
WebPge_Row	EQU	PageWdw_Var+S_CPCWdwSz	;Web Page Row
WebPge_Col	EQU	WebPge_Row+2		;Web Page Column
Cursor_OffCol	EQU	WebPge_Col+2		;Col offset in CPC window
Cursor_OffRow	EQU	Cursor_OffCol+2		;Row offset in CPC window
;===============================================================================
;
;   HTML page rendering variable
;
;===============================================================================
ROM_ShowHTMLFile	EQU	Cursor_OffRow+2
;===============================================================================
;
;  Variables for the initial script parsing during the system startup and
;  for various menu function to load configuration changes.
;
;===============================================================================
PAR_CurrLocn	EQU	ROM_ShowHTMLFile+4	;Curr location in Parse Script
PAR_CurrBank	EQU	PAR_CurrLocn+2		;Current bank to work in
PAR_ScriptLoad	EQU	PAR_CurrBank+1		;Load address of script
PAR_ScriptEnd	EQU	PAR_ScriptLoad+2	;End loc of script.
PAR_RetLine	EQU	PAR_ScriptEnd+2		;line to return on sub-scripts
PAR_Temp	EQU	PAR_RetLine+2
;===============================================================================
;
;  Variables for the handling of Application objects
;
;  App_Object = Application DB entry object
;               App_State    = state of DB entry
;               App_Name     = name of the Application/Handler
;                              e.g javascript
;               App_FileName = filename to load
;                              e.g javas.exe
;
;  App_ObjName = Location of App handler Object name to load (if any) within
;                the HTML command.
;                e.g myjava.txt, newjava.obj
;                if zero then object already loaded.
;  App_ObjLocn = 3-Byte location of where object data is to be loaded or the
;                location of where to execute.
;
;  App_CtrlFlg = Apps control flags
;                 Bit  Description
;                  7   Apps insert from HTML (0=No, 1=Yes)
;                  6   App object required to load (0=No, 1=Yes)
;                  5   Unused/Reserved
;                  4   Unused/Reserved
;                  3   App load failed (0=No, 1=Yes)
;                  2   App object load failed (0=No, 1=Yes)
;                  1   Keyboard input (0=Yes, 1=None)
;                  0   Unused/Reserved
;
;===============================================================================
LastAppIndex	EQU	PAR_Temp+4		;Last App entry
CurrAppIndex	EQU	LastAppIndex+1		;Current App entry in Mgr
StartAppIndex	EQU	CurrAppIndex+1		;Start App entry for Mgr
App_Flag	EQU	StartAppIndex+1		;Flag for App Manager
App_ID		EQU	App_Flag+1
App_Object	EQU	App_ID+1
App_State	EQU	App_Object+S_AppsState
App_Name	EQU	App_Object+S_AppsName
App_FileName	EQU	App_Object+S_AppsFileName
App_ObjName	EQU	App_Object+S_AppEntSize	;Location of object name
App_ObjNameBnk	EQU	App_ObjName+2
App_ObjLocn	EQU	App_ObjNameBnk+1	;location of script.
App_DBIndx	EQU	App_ObjLocn+3
App_DBTmp	EQU	App_DBIndx+1
App_CtrlFlg	EQU	App_DBTmp+S_AppsDBSz
Vars_EndApps	EQU	App_CtrlFlg+1
;===============================================================================
;
;  Tables Specific variables
;
;===============================================================================
;
;  HTML_TblFmtDat = table header format data (each byte has format of column)
;                   Bit  Description
;                    7   column header cell
;                         0=No (default, column is for data)
;                         1=Yes
;                    6   Column data cell
;                         0=No (default, column is for header data)
;                         1=Yes
;                   5-4  column data alignment
;                         00 = left justify (default)
;                         01 = center justify
;                         10 = right justify
;                         11 = full justify (not implemented)
;                    3   column width 
;                         0=Auto (default)
;                         1=Specific width
;                         Auto width = based on cell content
;                         Specific width = as per width spec in table defs
;                                          (converted to chars width).
;                                          Max width of cell is 253, cell
;                                          borders take 2 chars (left/right).
;                    2   cell wrap control
;                         0=Wrap text (default)
;                         1=No wrap
;                    1   Data displayed flag
;                         0=No (default)
;                         1=Yes
;                         Once set, the flag remains unchanged until row span
;                         count decrements to 0.
;                    0   Column cells taken
;                         0=Free (default)
;                         1=Merged with previous cell
;                         The first cell cannot be merged.
;
;  Bits 6 and 7 are set mutually exclusively to define either header or data.
;  If bits 7 and 6 are set then no more data for cells format in the table.
;  if bits 7 and 6 are zero then array item marks end of row.
;  
;  Max table width in characters is 65535.
;
;  HTML_TblFmtDat = Array of bytes to define the column formats for all rows
;
;  HTML_TblColWid = array of column spans (max width = 64)
;                   Values for each cell
;                   =0 means cell spans remainder of row
;                   =1 display data in cell with no span count decrement
;                      (default)
;                   >1 decrement span count and dont show data.
;
;  HTML_TblRowWid = array of rows spanned by each column
;                   Values for each cell
;                   =0 means cell spans whole table (or next table break)
;                   =1 display data in cell with no span count decrement
;                      (default)
;                   >1 decrement span count and dont show data.
;
;  HTML_TblNRSWid = Array of rows used in each column where data is displayed.
;                   Each item contains the number of lines to display the data.
;                   The processing of <TR..> scans array to determine the next
;                   line of rendering on the web-page.
;
;===============================================================================
HTML_TblFmtDat	EQU	Vars_EndApps
HTML_TblColWid	EQU	HTML_TblFmtDat+2
HTML_TblRowWid	EQU	HTML_TblColWid+2
HTML_TblNRSWid	EQU	HTML_TblRowWid+2
HTML_TblCRows	EQU	HTML_TblNRSWid+2
;===============================================================================
PatchCopy	EQU	HTML_TblCRows+2
LogBank		EQU	PatchCopy+3
FirstFileIdx	EQU	LogBank+3		;First file in displayed list
EndFileIdx	EQU	FirstFileIdx+2		;Last file displayed.
MaxFilesIdx	EQU	EndFileIdx+2		;Max files displayable.
CurrFileIdx	EQU	MaxFilesIdx+2		;Current file in selection.
;===============================================================================
;
;  Image handling variables
;
;===============================================================================
IMG_Addr	EQU	CurrFileIdx+2
IMG_Var		EQU	IMG_Addr+3
IMG_DBFlag	EQU	IMG_Var+S_ImageFlag
IMG_DBAddr	EQU	IMG_Var+S_ImageAdr
IMG_DBData	EQU	IMG_Var+S_ImageDat
IMG_ScrnMde	EQU	IMG_Var+S_ImageMode
IMG_NRows	EQU	IMG_Var+S_ImageRows	;Num of rows in image
IMG_NCols	EQU	IMG_Var+S_ImageCols	;num of cols in image.
;
IMG_Locn	EQU	IMG_Var+S_ImageSz
IMG_Temp	EQU	PAR_Temp		;overlayed temp var
;===============================================================================
CPCB_EndVars	EQU	IMG_Locn+3
;===============================================================================
;
;  Overlayed variables
;
;===============================================================================
Font_Temp	EQU	KB_printlen		;overlayed temp var
PAR_echo	EQU	ErrMsg_Loc		;overlayed var
FIO_Temp	EQU	PAR_Temp		;overlayed var
FIO_FileNum	EQU	PAR_RetLine
FIO_FileIndex	EQU	PAR_ScriptLoad
Save_Temp	EQU	App_ObjLocn		;Temp for saving HTML
TMP_String	EQU	&8000			;temp string locn
;===============================================================================
;
