;=========================================================================
;
; Application Routines (Browser Applications handler)
; Copyright (c) 2010 Ray Palmer
;
;=========================================================================
;Name     App_CalcAppAddr
;Purpose  To calculate the table entry address
;In Regs  A = Application ID
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  N/A
;Out Regs AF = ??
;         BC = corrupted.
;         DE = ??
;         HL = address of table entry
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    Mem_SwapInApps
;Notes    TBD
;=========================================================================
App_CalcAppAddr
   PUSH  AF
   CALL  MEM_SwapInApps
   POP   AF
;=========================================================================
;Name     App_CalcAddr
;Purpose  To calculate the table entry address
;In Regs  A = Application Id
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  N/A
;Out Regs AF = ??
;         BC = corrupted.
;         DE = ??
;         HL = address of table entry
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    1. The Apps configuration memory bank has been swapped in.
;=========================================================================
App_CalcAddr
   LD    HL,SysBnkBuffer
   LD    BC,S_AppEntSize
App_CalcAppAddr1
   OR    A
   RET   Z
   ADD   HL,BC
   DEC   A
   JR    App_CalcAppAddr1
;=========================================================================
;Name     App_GetAppDB
;Purpose  To get the nth Db entry.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  App_ID = App ID
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars App_Object = updated with App DB entry data.
;Uses     TBD
;Call By  TBD
;Calls    App_CalcAppAddr
;Notes    None.
;=========================================================================
App_GetAppDB
   LD    A,(App_ID)
   CP    255
   RET   Z
   CALL  App_CalcAppAddr		;HL = address of App DB entry
   LD    DE,App_Object
   LD    BC,S_AppEntSize
   LDIR					;Get Application DB entry
   RET
;=========================================================================
;Name     App_DeleteApp
;Purpose  To delete an entry from the Applications DB.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  App_ID = App ID to delete
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    App_CalcAppAddr
;Notes    None.
;=========================================================================
App_DeleteApp
   LD    A,(App_ID)
   CALL  App_CalcAppAddr
   LD    (HL),&80			;clear flags
   RET
;=========================================================================
;Name     App_ToggleInput
;Purpose  To toggle the input type.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  App_ID = App ID to delete
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    App_CalcAppAddr
;Notes    None.
;=========================================================================
App_ToggleInput
   LD    A,(App_ID)
   CALL  App_CalcAppAddr
   LD    A,(HL)
   XOR   1				;Toggle input type.
   LD    (HL),A
   RET
;=========================================================================
;Name     App_StopAppAccess
;Purpose  To disable access to an entry from the Applications DB.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  App_ID = App ID to diable.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    App_CalcAppAddr
;Notes    None
;=========================================================================
App_StopAppAccess
   LD    A,(App_ID)
   CALL  App_CalcAppAddr
   RES   6,(HL)				;Flag app is unavailable
   RET
;=========================================================================
;Name     App_StartAppAccess
;Purpose  To enable access to an entry from the Applications DB.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  App_ID = App ID to diable.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    App_CalcAppAddr
;Notes    None
;=========================================================================
App_StartAppAccess
   LD    A,(App_ID)
   CALL  App_CalcAppAddr
   SET   6,(HL)				;Flag app is available
   RET
;=========================================================================
;Name     App_FindApp
;Purpose  To search for the named App.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = address of app name
;         IX = ??
;         IY = ??
;In Vars  App_Name = name of App to search for.
;Out Regs AF = 1 if found else 0
;         B = First entry into DB of deleted item
;         C = Index into the DB, first free entry if not found
;         DE = ??
;         HL = address at end of db if not found.
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    Mem_SwapInApps
;Notes    1. This routine can also be used to return the end address of
;            the database when an invalid name is searched for.
;         2. The routine will also return the index to entry which has the
;            delete flag marked in the entry.
;=========================================================================
App_FindApp
   CALL  Mem_SwapInApps
   LD    DE,App_Name
   LD    HL,SysBnkBuffer	;Start of Apps database
   LD    BC,255
App_FindApp0
   INC   C			;track app db entry index
   LD    A,(HL)			;end of DB?
   OR    A
   JR    Z,App_FindApp4		;Yes
;
   BIT   7,A			;App deleted?
   JR    Z,App_FindApp3		;No
   LD    A,B
   OR    A			;first deleted entry logged?
   JR    NZ,App_FindApp2	;Yes
   LD    B,C			;track where first deleted entry is
   JR    App_FindApp2		;Yes
App_FindApp3
   PUSH  BC
   PUSH  DE
   PUSH  HL
   LD    BC,S_AppsName
   ADD   HL,BC
   LD    B,A_MaxAppName
   CALL  UTL_strncmp		;Compare names
   POP   HL
   POP   DE
   JR    Z,App_FindApp2		;names match
   LD    BC,S_AppEntSize
   ADD   HL,BC			;next entry
   POP   BC
   JR    App_FindApp0
;
App_FindApp2
   LD    DE,App_Object
   LD    BC,S_AppEntSize
   LDIR
   POP   BC
   LD    A,1			;flag found the app.
   RET
;
App_FindApp4
   XOR   A			;flag not found
   RET
;=========================================================================
;Name     App_AddNewItem
;Purpose  To add the application details to the apps table
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  App_ID = ID number of application.
;         AppsBank = Bank number for Applications Database
;Out Regs AF = 1 if already in DB else 0 new entry added.
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars App_ID = update with App ID of new entry.
;         LastAppIndex = App_ID on exit
;Uses     TBD
;Call By  TBD
;Calls    App_FindApp
;Notes    TBD
;=========================================================================
App_AddNewItem
   CALL  App_FindApp
   OR    A				;App already exists?
   RET   NZ				;Yes
;
   LD    A,C
   LD    (App_ID),A			;Return App Index ID
   LD    (LastAppIndex),A		;Setup next last index
   EX    DE,HL
   LD    HL,App_State
   LD    BC,S_AppEntSize
   LDIR					;copy data into bank
   XOR   A
   LD    (DE),A				;flag next is unused.
   RET
;=========================================================================
;Name     App_Struct
;Purpose  To clear the variable of text
;In Regs  A = ??
;         BC = ??
;         DE = address of name to clear
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
App_Struct
   LD    B,S_AppEntSize
   XOR   A
   LD    DE,App_State
App_Struct1
   LD    (DE),A
   INC   DE
   DJNZ  App_Struct1
   RET
;=========================================================================
;Name     App_NewAppName
;Purpose  To add a application name to the apps table
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = address of app name
;         IX = ??
;         IY = ??
;In Vars  TBD
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars App_Name = name of application
;Uses     TBD
;Call By  TBD
;Calls    None.
;Notes    None.
;=========================================================================
App_NewAppName
   LD    DE,App_Name
   LD    B,MaxAppName
   LD    C,','
   JR    App_InsertNewName
;=========================================================================
;Name     App_NewAppFilename
;Purpose  To add a application filename to the apps table
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = address of app name
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars App_FileName = filename of application.
;Uses     TBD
;Call By  TBD
;Calls    None.
;Notes    None.
;=========================================================================
App_NewAppFilename
   LD    DE,App_FileName
   LD    B,MaxAppFileName
   LD    C,0
App_InsertNewName
   LD    A,(HL)
   CP    C				;At end of text?
   JR    Z,App_GetAppInfo2		;Yes (fill with spaces)
   LD    (DE),A
   INC   DE
   INC   HL
   DJNZ  App_InsertNewName		;continue to add name
   RET
;=========================================================================
;Name     App_GetAppInfo2
;Purpose  To fill in a variable string with spaces and a NULL
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TDB
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
App_GetAppInfo2
   LD    A,SPACE_CHAR
   LD    (DE),A
   INC   DE
   DJNZ  App_GetAppInfo2
   RET
;=========================================================================
;Name     App_GetAppInfo
;Purpose  To get the details of an application from the DB of applications
;In Regs  A = ID number of application.
;         BC = ??
;         DE = ??
;         HL = address of app name
;         IX = ??
;         IY = ??
;In Vars  AppsBank = Bank number for Applications Database
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars App_State = updated with App details
;         App_Name = name of application
;         KB_Buffer = URL of application
;Uses     TBD
;Call By  TBD
;Calls    Mem_SwapInMain
;         App_CalcAppAddr
;         UTL_strncpy
;Notes    TBD
;=========================================================================
App_GetAppInfo
   CALL  App_CalcAppAddr		;Calculate address of App entry
   LD    DE,App_State
   LD    A,(HL)				;Get App state
   LD    (DE),A				;Setup state
   OR    A				;last entry?
   JP    Z,Mem_SwapInMain		;Yes
   INC   HL				;skip status/flags
   INC   DE				;Point to start of App's Name
   LD    B,MaxAppName
   CALL  UTL_strncpy			;copy App name for display
   XOR   A
   LD    (DE),A				;insert EOL
;
   LD    DE,KB_Buffer
   LD    B,MaxAppFileName
   CALL  UTL_strncpy			;copy App filename
   XOR   A
   LD    (DE),A				;insert EOL
   RET
;=========================================================================
;Name     App_ShowAppDetails
;Purpose  To show the details of the applications.
;In Regs  A = ID number of application.
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TDB
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    App_GetAppInfo
;         MNU_ShowWindwData
;         MNU_AdjustLocn
;Notes    1. Stream is currently the App window
;=========================================================================
App_ShowAppDetails
   CALL  App_GetAppInfo		;get specific App information
;
   LD    D,7			;Column 7
   LD    E,3			;Row 2
   CALL  MNU_AdjustLocn
   PUSH  DE
   LD    HL,App_Name
   CALL  WIN_ShowWindwData	;Display App name
   POP   DE
   INC   E			;next row
   LD    HL,KB_Buffer
   CALL  WIN_ShowWindwData	;Display App filename
   RET
;=========================================================================
;Name     APP_GetStatus1
;Purpose  To get status of enabled/disabled Application.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  App_State = Status of application/handler
;Out Regs AF = ??
;         BC = ??
;         D = ??
;         E = Char to display
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    NOne
;=========================================================================
APP_GetStatus1
   LD    E,'-'
   LD    A,(App_State)
   BIT   1,A				;entry available
   RET   Z				;No
   LD    E,'D'
   BIT   6,A				;entry disabled?
   RET   Z				;Yes
   LD    E,'E'				;show entry enabled
   RET
;=========================================================================
;Name     APP_GetStatus2
;Purpose  To get status of type of Application entry
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  App_State = Status of application/handler
;Out Regs AF = ??
;         BC = ??
;         D = ??
;         E = Char to display
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    NOne
;=========================================================================
APP_GetStatus2
   LD    A,(App_State)
APP_GetStatus2a
   LD    E,'-'
   BIT   1,A				;entry available
   RET   Z				;No
   LD    E,'A'
   BIT   5,A				;Application?
   RET   Z				;No
   LD    E,'H'				;show entry handler
   RET
;=========================================================================
;Name     APP_GetStatus3
;Purpose  To get status of type of handler for the Application entry
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  App_State = Status of application/handler
;Out Regs AF = ??
;         BC = ??
;         D = ??
;         E = Char to display
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    NOne
;=========================================================================
APP_GetStatus3
   LD    E,'-'
   LD    A,(App_State)
   BIT   1,A				;entry available
   RET   Z				;No
   LD    E,'F'
   BIT   4,A				;File?
   RET   Z				;No
   LD    E,'R'				;show entry ROM
   RET
;=========================================================================
;Name     APP_GetStatus4
;Purpose  To get input type for the Application entry
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  App_State = Status of application/handler
;Out Regs AF = ??
;         BC = ??
;         D = ??
;         E = Char to display
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    NOne
;=========================================================================
APP_GetStatus4
   LD    E,'-'
   LD    A,(App_State)
   BIT   1,A				;entry available
   RET   Z				;No
   LD    E,'K'
   BIT   0,A				;Keyboard input?
   RET   Z				;No
   LD    E,'S'				;show entry Supplied by script.
   RET
;=========================================================================
;Name     App_ShowAppNames
;Purpose  To show the names of 4 applications.
;In Regs  A = start index for App names
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars Windw_Col = work var for column/row
;         App_State = corrupted.
;         App_Name = corrupted.
;         App_FileName = corrupted.
;Uses     TBD
;Call By  TBD
;Calls    App_GetAppInfo
;         TXT_SET_CURSOR (OS)
;         TXT_WR_CHAR (OS)
;         MNU_ShowWindwData (OS)
;Notes    TBD
;=========================================================================
App_ShowAppNames
   LD    C,A				;Start App Number
   LD    B,6				;Max Apps in Window
   LD    D,3				;Column 3
   LD    E,6				;Row 6
   CALL  MNU_AdjustLocn
App_ShowAppName1
   PUSH  DE
   PUSH  BC
   LD    A,C
   CALL  App_GetAppInfo			;Get App details
   POP   BC
   POP   HL
   LD    A,(App_State)
   OR    A				;Last app in DB?
   RET   Z				;yes
;
   PUSH  BC
   PUSH  HL
;
   LD    C,H
   INC   C
   CALL  APP_GetStatus1
   LD    A,E
   CALL  WIN_DisplayChr1		;Show Enabled/Disabled status
;
   CALL  APP_GetStatus2
   LD    A,E
   LD    C,H
   INC   C
   CALL  WIN_DisplayChr1		;Show App/Handler type
;
   CALL  APP_GetStatus3
   LD    A,E
   LD    C,H
   INC   C
   CALL  WIN_DisplayChr1		;Show File/ROM type
;
   CALL  APP_GetStatus4
   LD    A,E
   LD    C,H
   INC   C
   CALL  WIN_DisplayChr1		;Show input type
;
   EX    DE,HL
   DEC   E
   LD    HL,App_Name
   CALL  WIN_ShowWindwData		;Display App name
;
   POP   DE
   POP   BC
   INC   E				;next Row
   INC   C
   DJNZ  App_ShowAppName1
   RET
;=========================================================================
;Name     App_BuildAppCmd
;Purpose  To build the script command install apps entry.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_Buffer = script command.
;         SysGenBuffer = area corrupted for command text
;Uses     TBD
;Call By  TBD
;Calls    App_CalcAppAddr
;         UTL_strncpy
;         UTL_insertCRLF
;Notes    1. This routine is for the building of the script to install
;            and entry into the Apps database.
;=========================================================================
App_BuildAppCmd
   CALL  App_CalcAppAddr
   LD    DE,SysGenBuffer
   LD    BC,S_AppEntSize
   LDIR					;download App structure
   LD    A,(SysGenBuffer)
   BIT   1,A				;Entry available to access?
   RET   Z				;No
   LD    DE,KB_Buffer
   LD    HL,Script_App
   LD    BC,3
   LDIR					;Insert 'app' command
   LD    A,32
   LD    (DE),A
   INC   DE
   LD    A,(SysGenBuffer)
   CALL  APP_GetStatus2a		;app or handler char
   LD    A,E
   LD    (DE),A
   INC   DE
   LD    A,','
   LD    (DE),A
   INC   DE
   LD    HL,SysGenBuffer+S_AppsName
   LD    B,MaxAppName
   CALL  UTL_strncpy			;Copy app name
   LD    A,','
   LD    (DE),A
   INC   DE
   LD    HL,SysGenBuffer+S_AppsFileName
   LD    B,MaxAppFileName
   CALL  UTL_strncpy			;insert app filename
   JP    UTL_insertCRLF
;=========================================================================
;Name     App_LdObjScr
;Purpose  To load the script object for the application after the last
;         address of HTML.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = address of application object to load.
;         IX = ??
;         IY = ??
;In Vars  URL_EndAddr = current start address of loaded objects
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars App_ObjLocn = address of loaded object
;         URL_EndAddr = preserved on exit
;Uses     TBD
;Call By  html_cmd_script
;Calls    App_LoadNewObj
;Notes    1. The object must be text.
;         2. This routine can also be call by other functions so long as
;            the function does not expect complete scripts if tried.
;=========================================================================
App_LdObjScr
   LD    A,H
   OR    L			;Object to load?
   RET   Z			;No
   PUSH  HL
   LD    HL,URL_EndAddr
   LD    DE,App_ObjLocn
   LD    BC,3
   LDIR				;setup script address for app
   POP   HL
   CALL  URL_OpenNewObj		;Load script object after HTML.
   LD    DE,URL_EndAddr
   LD    HL,App_ObjLocn
   LD    BC,3
   LDIR				;reset URL_EndAddr for next app
   RET
;=========================================================================
;Name     App_LdObjExe
;Purpose  To load the script object handler/Application into memory.
;In Regs  A = Status of the App DB entry
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  App_Object = object with App name
;                      App_FileName = filename of app/handler
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_Buffer = URL of object to load
;Uses     TBD
;Call By  html_cmd_script
;Calls    MEM_SwapInCfg
;         UTL_GetCfgLocn
;         utl_strcpy
;         URL_OpenNewObj
;Notes    1. The object binary handler can only be a max of 16K bytes.
;=========================================================================
App_LdObjExe
   LD    A,(App_State)
   BIT   4,A				;Does handler need to be loaded?
   RET   NZ				;No
;
;  Load the Application object
;
   LD    HL,CFG_AppLocn			;Location of exectables
   BIT   5,A				;Application executable type?
   JR    Z,App_LdObjExe1		;Yes
   LD    HL,CFG_SysLocn			;location of handlers
App_LdObjExe1
   CALL  MEM_SwapInCfg
   CALL  UTL_GetCfgLocn			;KB_Buffer = URL location text
;
   LD    HL,App_FileName
   LD    B,MaxAppFileName
   CALL  utl_strcpy			;add filename after URL path
;
   LD    A,2
   LD    (FIO_CodeBlk),A
   LD    HL,0
   LD    (FileLoadAddr),HL
   LD    HL,KB_Buffer
   JP    URL_OpenNewObj			;Load object
;=========================================================================
;Name     App_InpObjNam
;Purpose  To enter the name of the Application object to access.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = Address of entered input.
;         IX = ??
;         IY = ??
;Out Vars App_ObjName = address of user input
;         App_ObjNameBnk - bank where user input is stored.
;Uses     TBD
;Call By  TBD
;Calls    MNU_ReqInputTxt
;Notes    TBD
;=========================================================================
App_InpObjNam
   LD    HL,GeneralRequest
   LD    BC,REQUEST_TYP3
   CALL  MNU_ReqInputTxt	;Request for input on URL line.
   CALL  Err_ClrErrTxt		;Clear message request 
   LD    HL,KB_Buffer		;No
   LD    A,(HL)
   OR    A			;Any input supplied?
   RET   Z			;No
   LD    (App_ObjName),HL	;address of app input
   LD    A,&C0
   LD    (App_ObjNameBnk),A	;bank where input is
   RET
;=========================================================================
;Name     App_GetAppObj
;Purpose  To get the application executable/handler and/or scripts.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  App_ID = Index to App DB entry
;         App_Object = Current object to handle the request from the HTML
;         App_ObjName = address to the name of object to process.
;         App_ObjNameBnk = bank where object exists.
;         App_ObjLocn = address to load object (if not already loaded)
;         App_CtrlFlg = status of what failed
;                       bit 7 = app load
;                       bit 6 = app object load
;Out Regs AF = ??
;         B = Error code
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FileOpState = corrupted
;         App_State = status of app handling (bit 3)
;         App_CtrlFlg = status of what failed
;                       bit 3 = app load
;                       bit 2 = app object load
;                       bit 1 = keyboard input
;Uses     TBD
;Call By  html_cmd_script
;Calls    App_LdObjScr
;         App_LdObjExe
;Notes    1. The object binary handler can only be a max of 16K bytes.
;=========================================================================
App_GetAppObj
   LD    A,(App_CtrlFlg)
   AND   &F0			;clear App status flags
   LD    (App_CtrlFlg),A
;
   LD    A,(App_State)
   BIT   1,A			;Entry enabled?
   JR    Z,App_GetAppObjErr	;No (access error)
;
   LD    A,(App_State)
   BIT   0,A			;Enter object name Supplied?
   JR    NZ,App_GetAppObj1	;Yes
;
   CALL  App_InpObjNam		;Yes (key keyboard input)
   LD    B,B_1
   JR    Z,App_GetAppObjErr	;No (no input-- error)
;
App_GetAppObj1
   XOR   A
   LD    (FileOpState),A	;clear file load status
;
   CALL  App_LdObjExe		;Load Application/handler
   LD    B,B_3			;flag app load failed
   LD    A,(FileOpState)
   OR    A			;handler/App accessed okay?
   JR    NZ,App_GetAppObjErr	;No
;
   LD    A,(App_CtrlFlg)
   BIT   6,A			;Load App object first?
   RET   Z			;No
;
   LD    HL,(App_ObjName)
   CALL  App_LdObjScr		;Load objects data (if any)
;
   LD    A,(FileOpState)
   OR    A			;App start up okay?
   RET   Z			;Yes
   LD    B,B_2			;object for app load failed
;
App_GetAppObjErr
   LD    A,(App_CtrlFlg)
   OR    B			;flag what failed
   LD    (App_CtrlFlg),A
   RET
;=========================================================================
;Name     App_StartCfgApp
;Purpose  To execute an internal configuration handler.
;In Regs  A = ??
;         BC = ??
;         DE = Address of App object.
;         HL = Address of the filename for executable to start
;         IX = ??
;         IY = ??
;In Vars  ??
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars App_Object = corrupted object
;Uses     TBD
;Call By  ??
;Calls    ??
;Notes    None
;=========================================================================
App_StartCfgApp
   LD    (App_ObjName),DE
   LD    A,(App_CtrlFlg)
   RES   6,A
   LD    (App_CtrlFlg),A
   LD    DE,App_FileName
   CALL  UTL_strcpy
   LD    A,&43
   LD    (App_State),A
   JR    App_StartApp2
;=========================================================================
;Name     App_StartApp
;Purpose  To execute the application which is not normally part of the
;         HTML browser. An example application might be a Web-chat
;         application started from the menu option.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  App_ID = Application/handler db index
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars App_Object = corrupted object
;Uses     TBD
;Call By  Sub-Menu selection.
;Calls    App_GetAppDB
;         App_DoAppReq
;         Err_ShowErrorMsg
;         Err_ClrErrTxt
;         WIN_ClearStream
;         URL_GetDbEntry
;         MNU_MainMenuFrame
;         MNU_PageRefresh
;Notes    None
;=========================================================================
App_StartApp
   CALL  App_GetAppDB			;Get App DB entry
;
   LD    HL,OpenAppError
   LD    A,(App_State)
   BIT   6,A				;App DB entry enabled?
   JR    Z,App_StartAppErr		;No
;
   LD    A,1
   LD    (WindowFrmeNum),A
;
   LD    A,(Stream_WebPage)
   CALL  WIN_ClearStream
;
App_StartApp2
;
   LD    HL,0
   LD    (App_ObjName),HL		;no additional script/data for object
;=========================================================================
;Name     App_StartApp2
;Purpose  To execute the application which is not normally part of the
;         HTML browser.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  App_Object = Object setup
;         App_ObjName = address of additional text
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars App_Object = corrupted object
;Uses     TBD
;Call By  Sub-Menu selection.
;Calls    App_GetAppDB
;         App_DoAppReq
;         Err_ShowErrorMsg
;         Err_ClrErrTxt
;         WIN_ClearStream
;         URL_GetDbEntry
;         MNU_MainMenuFrame
;         MNU_PageRefresh
;Notes    1. This routine is callable from the HTML as the HTML uses this
;            method to handle objects like pictures or specific apps to
;            provide more capabilities.
;=========================================================================
App_StartApp3
   CALL  App_DoAppReq			;Start Application
;
   LD    HL,GeneralAppError
   LD    A,(App_CtrlFlg)
   AND   &0F				;App startup failed?
   JR    NZ,App_StartAppErr		;Yes
;
   LD    HL,PressESC_Msg
   CALL  Err_ShowErrorMsg		;Display and wait to continue
   CALL  Err_ClrErrTxt
;
   CALL  URL_GetBaseURL
   CALL  MNU_MainMenuFrame
   CALL  MNU_ShowURL
   JP    MNU_PageRefresh		;re-display web-page
;
App_StartAppErr
   JP    Err_ShowErrorMsg
;=========================================================================
;Name     App_DoAppReq
;Purpose  To process a HTML request to handle an object such as javascript
;         from a web HTML page.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  App_Object = object to handle the request from the HTML or
;                      menu selection.
;                      App_State = State of the object
;                      App_ObjName = name of object to process.
;                      App_ObjLocn = address to load object
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  Jump Vector Entry
;Calls    MEM_SwapInCode
;         App_GetAppObj
;         UTL_readdec
;         KL_ROM_SELECT (OS)
;Notes    1. The application is expected to be loaded into the 3rd bank of
;            code blocks where it executes as a separate executeable.
;         2. There is currently no way to limit the executable so that it
;            does not interfere with the browser.
;=========================================================================
App_DoAppReq
   LD    A,(App_State)
   BIT   6,A				;App DB entry enabled?
   RET   Z				;No
;
   CALL  App_GetAppObj			;Load data + handler
;
   LD    A,(App_CtrlFlg)
   AND   &0F				;App startup failed?
   RET   NZ				;Yes
;
   LD    A,(WindowFrmeNum)
   CALL  WIN_GetFrame			;Get Frame data for web-page window
   CALL  HTML_CalcFrameCPC		;reset frame dimensions
;
   LD    A,(App_State)
   BIT   4,A				;ROM to handle application?
   JR    NZ,AppDoAppReq1		;Yes
;
;  Execute App via a loaded executable handler
;
   CALL  MEM_SwapInCode			;Swap in User Code bank
   LD    HL,SysBnkBuffer
   JR    AppDoAppReq2			;Start code at base address
;
;  Execute App via a ROM
;
;  The handler filename is defined to be xxxx-aa.ROM
;  where 'xxxx' is a hex address and
;        'aa' is the rom number.
;
AppDoAppReq1
   LD    DE,App_FileName
   CALL  UTL_readhex16			;Get address of ROM code to run
   PUSH  HL
   INC   DE				;skip dash
   EX    DE,HL
   CALL  UTL_readdec			;convert ROM to number
   LD    C,A
   CALL  KL_ROM_SELECT			;Select in ROM
   POP   HL
AppDoAppReq2
   PUSH  HL
;
;  Setup Parameters to call
;  HL = start address of web-browser variables.
;  BC = start address of HTML Rom variables.
;  DE = address of specific parameter to app
;
   LD    BC,HTML_StartVars
   LD    HL,CPCB_StartVars		;Start of CPCB variables
   LD    DE,App_ObjName
;
   RET					;Execute ROM handler