;=========================================================================
;
; Application Routines (File I/O handler)
; Copyright (c) 2010 Ray Palmer
;
;=========================================================================
;Name     FIO_IN_OPEN
;Purpose  A wrapper for CAS_IN_OPEN to disable print across I/O call.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  N/A
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    CAS_IN_OPEN (OS)
;Notes    TBD
;=========================================================================
FIO_IN_OPEN
   PUSH  AF
   LD    A,(TXT_OUTPUT)
   LD    (FIO_Temp),A
   LD    A,&C9
   LD    (TXT_OUTPUT),A
   POP   AF
   CALL  CAS_IN_OPEN
   JR    FIO_OpenRestore
;=========================================================================
;Name     FIO_OUT_OPEN
;Purpose  A wrapper for CAS_OUT_OPEN to disable print across I/O call.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  N/A
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    CAS_OUT_OPEN (OS)
;Notes    TBD
;=========================================================================
FIO_OUT_OPEN
   PUSH  AF
   LD    A,(TXT_OUTPUT)
   LD    (FIO_Temp),A
   LD    A,&C9
   LD    (TXT_OUTPUT),A
   POP   AF
   CALL  CAS_OUT_OPEN
FIO_OpenRestore
   PUSH  AF
   LD    A,(FIO_Temp)
   LD    (TXT_OUTPUT),A
   POP   AF
   RET
;=========================================================================
;Name     FIO_IN_CLOSE
;Purpose  A wrapper for CAS_IN_CLOSE to disable print across I/O call.
;In Regs  A = ??
;         B = Current state of File operations
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  N/A
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FileOpState = statue/error code of file operation.
;Uses     TBD
;Call By  TBD
;Calls    CAS_IN_CLOSE (OS)
;Notes    TBD
;=========================================================================
FIO_IN_CLOSE
   PUSH  HL
   LD    A,B
   LD    (FileOpState),A
   CALL  CAS_IN_CLOSE
   POP   HL
   RET
;=========================================================================
;Name     FIO_IN_ABANDON
;Purpose  A wrapper for CAS_IN_CLOSE to disable print across I/O call.
;In Regs  A = ??
;         B = Current state of File operations
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  N/A
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FileOpState = statue/error code of file operation.
;Uses     TBD
;Call By  TBD
;Calls    CAS_IO_RESET (OS)
;Notes    TBD
;=========================================================================
FIO_IN_ABANDON
   PUSH  HL
   LD    A,B
   LD    (FileOpState),A
   CALL  CAS_IO_RESET
   POP   HL
   RET
;=========================================================================
;Name     FIO_OUT_CLOSE
;Purpose  A wrapper for CAS_OUT_CLOSE to disable print across I/O call.
;In Regs  A = ??
;         B = Current state of File operations
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  N/A
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FileOpState = statue/error code of file operation.
;Uses     TBD
;Call By  TBD
;Calls    CAS_OUT_CLOSE (OS)
;Notes    TBD
;=========================================================================
FIO_OUT_CLOSE
   PUSH  HL
   LD    A,B
   LD    (FileOpState),A
   CALL  CAS_OUT_CLOSE
   POP   HL
   RET
;=========================================================================
;Name     FIO_WriteString
;Purpose  A routine to write a string to a file.
;In Regs  A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  KB_Buffer has string to write
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FileOpState = statue/error code of file operation.
;Uses     TBD
;Call By  TBD
;Calls    UTL_strlen
;         CAS_OUT_CHAR (OS)
;Notes    TBD
;=========================================================================
FIO_WriteString
   LD    HL,KB_Buffer
   CALL  UTL_strlen
   LD    B,A
;=========================================================================
;Name     FIO_WriteBytes
;Purpose  A routine to write n bytes to a file.
;In Regs  A = ??
;         B = Number of bytes to write
;         C = ??
;         DE = ??
;         HL = address of data
;         IX = ??
;         IY = ??
;In Vars  N/A
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FileOpState = statue/error code of file operation.
;Uses     TBD
;Call By  TBD
;Calls    CAS_OUT_CHAR (OS)
;Notes    1. If the byte to write is zero then the routine exits, that
;            means that the maximum string length is 255.
;=========================================================================
FIO_WriteBytes
   LD    A,(HL)
   OR    A
   RET   Z
   PUSH  HL
   PUSH  BC
   CALL  CAS_OUT_CHAR
   LD    A,FERR_WRITE
   POP   BC
   POP   HL
   JP    NC,FIO_OUT_CLOSE
   INC   HL
   DJNZ  FIO_WriteBytes
   RET
;=========================================================================
;Name     FIO_OpenDevice
;Purpose  To process the device for opening a file from the url.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = address of URL device specification text
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = device number
;         BC = ??
;         DE = ??
;         HL = address of chars after device specification
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    Ext_OpenMediaDevice (via SYS_RunCode1a)
;         SYS_SaveRegs
;         SYS_RunCode1a
;Notes    TBD
;=========================================================================
FIO_OpenDevice
   CALL  SYS_SaveRegs
   LD    HL,Ext_OpenMediaDevice
   JP    SYS_RunCode1		;execute Open HDOS Device (read stream)
;=========================================================================
;Name     FIO_OpenPath
;Purpose  To open the path to the file in the URL
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    Ext_OpenMediaPath (via SYS_RunCode1a)
;         SYS_RunCode1a
;         SYS_SaveRegs
;Notes    TBD
;=========================================================================
FIO_OpenPath
   CALL  SYS_SaveRegs		;Save registers (AF/BC/DE/HL)
   LD    HL,Ext_OpenMediaPath
   JP    SYS_RunCode1		;execute OpenPATH in file1.bin
;=========================================================================
;Name     FIO_OpenDevPath
;Purpose  To open the device path for a local device.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = address of URL element
;         IX = ??
;         IY = ??
;In Vars  URL_Object = URL object to be opened
;         URL_EndAddr = start address to load file data
;         CurrLocDevice = current local device to access (0=ASMDOS, 1=HDOS)
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = location of filename
;         IX = ??
;         IY = ??
;Out Vars FileOpState = current file operation state
;                       FERR_UNKNOWN = failure
;                       0 = okay
;         CurrLocDevice = current local device to access.
;Uses     TBD
;Call By  TBD
;Calls    FIO_OpenDevice (external routine in file1.bin)
;         FIO_OpenPath (external routine in file1.bin)
;Notes    None.
;=========================================================================
FIO_OpenDevPath
   LD    D,H			;DE = location of Path
   LD    E,L
   LD    A,(HL)
   CP    BSLASH_CHR		;device specification first?
   JR    NZ,FIO_OpnDevPath_0a	;No (check if path okay)
;
   CALL  FIO_OpenDevice 	;Open the device
   CP    255			;open device error?
   JR    Z,FIO_OpnDevPathErr	;Yes
   LD    (CurrLocDevice),A	;Save new device information
;
FIO_OpnDevPath_0a
   LD    A,(CurrLocDevice)
   CALL  FIO_OpenPath		;Open path (see file1.bin)
   CP    255			;path error code?
   JR    Z,FIO_OpnDevPathErr	;Yes
;
   EX    DE,HL			;HL = location of filename
   XOR   A
   JR    FIO_OpnDevPathErr2	;clear error status.
FIO_OpnDevPathErr
   LD    A,FERR_UNKNOWN
FIO_OpnDevPathErr2
   LD    (FileOpState),A	;return error
   RET
;=========================================================================
;Name     FIO_GetURLFileType
;Purpose  To determine what the URL file type is by the extension in the
;         file.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = address of path/filename
;         IX = ??
;         IY = ??
;In Vars  FExtTypes = list of filetypes
;Out Regs AF = File type
;              0-254 valid type as defined in cpc-do.inc
;              255 = unknown (problem in data - check if okay)
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    UTL_tolower
;Notes    TBD
;=========================================================================
FIO_GetURLFileType
   LD    DE,0
   CALL  URL_LastDot			;DE = address of char after last dot
   LD    A,D				;last dot found?
   OR    E
   JR    Z,FIO_GetURLFileTyp3		;No (no extension)
   EX    DE,HL				;HL = start of extension
   LD    DE,FExtTypes
FIO_GetURLFileTyp0
   PUSH  HL
   PUSH  DE
   LD    B,3
FIO_GetURLFileTyp1
   LD    A,(HL)
   CALL  UTL_tolower			;convert to URL char lower
   LD    C,A				;Save for compare below
   LD    A,(DE)				;get lookup type char
   INC   DE
   INC   HL
   CP    STAR_CHR			;Is it a wildcard?
   JR    Z,FIO_GetURLFileTyp1a		;Yes (char check not required)
   CP    C				;Matches URL char?
   JR    NZ,FIO_GetURLFileTyp2		;No
FIO_GetURLFileTyp1a
   DJNZ  FIO_GetURLFileTyp1		;Do next type char
   LD    A,(DE)				;Return file type
   POP   DE
   POP   HL
   RET
FIO_GetURLFileTyp2
   POP   HL
   LD    DE,4
   ADD   HL,DE				;next entry
   EX    DE,HL
   POP   HL
   LD    A,(DE)
   OR    A				;end of list?
   JR    NZ,FIO_GetURLFileTyp0		;No
FIO_GetURLFileTyp3
   DEC   A				;return 255 (unknown)
   RET
;=========================================================================
;Name     FIO_LoadObject
;Purpose  To load the URL Object.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  URL_Object = contains the information to access the URL item.
;         URL_EndAddr = current load address for HTML object
;         FIO_CodeBlk = Code bank index for binary images.
;                       Blocks 0/1 reserved
;                       Block 2 start of binary images.
;         FileLoadAddr = load address for binary files (0=use file header)
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars URL_OpenType = type of object being loaded
;         URL_EndAddr = End Current location of HTML
;                       if binary data then location is not set
;Uses     TBD
;Call By  TBD
;Calls    FIO_GetURLFileType
;         FIO_IN_OPEN
;         MEM_SwapInCode
;         Mem_SwapInHTML
;         Mem_SwapInMain
;         CAS_IN_DIRECT (OS)
;         CAS_IN_CHAR (OS)
;         FIO_IN_CLOSE (OS)
;         FIO_IN_ABANDON
;Notes    1. If the Objects file name ends in .exe/.bin then the binary
;            is loaded into the code bank. There is no checking of the
;            size of binary files to ensure that they dont corrupt existing
;            memory is not corrupted.
;         2. If the loading of a text file exceeds the maximum allowed the
;            remaining content is dropped.
;=========================================================================
FIO_LoadObject
   LD    HL,URL_ObjName
   CALL  FIO_GetURLFileType	;Determine the type of file to load
   LD    (URL_OpenType),A	;Save Object type
   LD    HL,URL_ObjName
   CALL  UTL_strlen
   LD    B,A			;B = length of filename spec
   LD    DE,AMSDOS_BUFFER	;AMSDOS 2k Buffer
   CALL  FIO_IN_OPEN		;open file for read
   LD    B,FERR_NOTFOUND
   JP    NC,FIO_IN_ABANDON	;error opening file (close stream)
   LD    B,FERR_BADTYPE
   CP    FILE_ASCII		;Is file a text file?
   JR    Z,FIO_LoadObjTxt	;Yes (load as text file)
;=========================================================================
;
; Load binary file
;
;=========================================================================
   LD    A,(FIO_CodeBlk)
   CALL  MEM_SwapBCBank		;Swap in code block
   LD    HL,(FileLoadAddr)
   LD    A,H
   OR    L			;Load address supplied
   JR    NZ,FIO_LdObjBin1	;Yes (use it)
   EX    DE,HL			;Setup load location
;
FIO_LdObjBin1
   CALL  CAS_IN_DIRECT		;Yes (load contents)
   LD    B,FERR_READ
   JR    NC,FIO_LoadObjTxt1b	;read file error
   LD    B,FERR_OKAY		;Flag okay
   LD    A,(URL_OpenType)
   BIT   7,A			;unknown file?
   JR    Z,FIO_LoadObjTxt1b	;No
   OR    5			;for it to be unknown binary
   LD    (URL_OpenType),A
   JR    FIO_LoadObjTxt1b
;=========================================================================
;
; Load text file
;
;=========================================================================
FIO_LoadObjTxt
   LD    A,(FIO_Ctrl)
   LD    L,A
   LD    A,(URL_OpenType)
   BIT   7,A			;unknown type of file?
   JR    Z,FIO_LoadObjTxta	;No
   OR    3			;force it to be unknown text
   LD    (URL_OpenType),A
FIO_LoadObjTxta
   CP    L			;Is file a script sequence?
   JR    Z,FIO_LoadObjScrpt	;Yes
;
   LD    A,(URL_EndAddr+2)	;Start at first HTML bank
   LD    HL,(URL_EndAddr)
FIO_LoadObjTxt0
   LD    (URL_EndAddr),HL	;save load address
   LD    (URL_EndAddr+2),A
;
   LD    B,A
   LD    A,(MaxHTMLBanks)
   CP    B			;At end of array of HTML blocks?
   JR    Z,FIO_LoadObjTxt1b	;Yes, At end of configured HTML banks
;
   LD    A,B
   CALL  Mem_SwapInHTML		;Swap in a HTML bank
   LD    HL,(URL_EndAddr)
FIO_LoadObjTxt1
   PUSH  HL
   CALL  CAS_IN_CHAR		;get file data byte char
   POP   HL
   LD    B,FERR_OKAY		;Read error/EOF
   JR    NC,FIO_LoadObjTxt1a	;Yes, close file and exit
   LD    (HL),A			;Save file data
   INC   HL
   BIT   7,H			;At limit of memory bank?
   JR    Z,FIO_LoadObjTxt1	;No, get next file data byte
   LD    HL,SysBnkBuffer
   LD    A,(URL_EndAddr+2)
   INC   A
   JR    FIO_LoadObjTxt0
;
FIO_LoadObjTxt1a
   LD    (HL),NULL_CHAR		;insert end of data
   INC   HL			;Assume within block still
   LD    (URL_EndAddr),HL	;save new start load address
FIO_LoadObjTxt1b
   CALL  FIO_IN_CLOSE
   JP    Mem_SwapInMain
;=========================================================================
;
;  Load script sequence file.
;
;=========================================================================
FIO_LoadObjScrpt
   LD    HL,(PAR_ScriptLoad)
   LD    C,0
   LD    B,FERR_READ			;assume read error
FIO_LoadObjScrptloop
   PUSH  BC
   PUSH  HL
   CALL  CAS_IN_CHAR			;Get char from file
   POP   HL
   POP   BC
   JP    NC,FIO_LoadObjScrptexit	;file read error
   LD    B,FERR_OKAY			;file has been read okay
   CP    ';'				;Start of comment?
   JR    NZ,FIO_LoadObjScrpt1		;No
   LD    C,&80				;Flag start of comment
   JR    FIO_LoadObjScrptloop		;Get next char from file.
FIO_LoadObjScrpt1
   BIT   7,C				;In comment?
   JR    Z,FIO_LoadObjScrpt2		;No
   CP    LINEFEED_CHAR			;LF char ?
   JR    NZ,FIO_LoadObjScrptloop	;No (get another char)
   LD    C,0				;flag no longer in comment
   JR    FIO_LoadObjScrptloop		;get another char
;
FIO_LoadObjScrpt2
   CP    SPACE_CHAR			;Is char a control code?
   JR    NC,FIO_LoadObjScrpt3		;No
   DEC   HL
   LD    A,(HL)				;Get previous save char
   INC   HL
   OR    A				;Previously inserted EOL char?
   JR    Z,FIO_LoadObjScrptloop		;Yes (dont insert another)
   XOR   A				;Flag EOL
FIO_LoadObjScrpt3
   LD    (HL),A
   INC   HL
   JR    FIO_LoadObjScrptloop		;No
;
FIO_LoadObjScrptexit
   LD    (HL),NULL_CHAR
   INC   HL
   LD    (HL),NULL_CHAR
   JP    FIO_IN_CLOSE			;Close input file
;=========================================================================
;Name     FIO_LoadBinFile
;Purpose  To load a binary file form the current device location. The
;         routine is specifically called for loading modules which after
;         they are loaded can be referenced for devices and paths.
;In Regs  AF = ??
;         B = ??
;         DE = ??
;         HL = address of filename text
;         IX = ??
;         IY = ??
;In Vars  FileLoadAddr = location to load file to.
;Out Regs AF = ??
;         B = read status
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FileOpState = statue of file operations
;Uses     TBD
;Call By  TBD
;Calls    FIO_IN_OPEN
;         CAS_IN_DIRECT (OS)
;         FIO_IN_CLOSE
;Notes    1. If fileloadaddr <> 0 then use it for loading else the address
;            in the file header is used.
;=========================================================================
FIO_LoadBinFile
   CALL  UTL_strlen
   LD    B,A
   LD    DE,AMSDOS_BUFFER	;AMSDOS 2k Buffer
   CALL  FIO_IN_OPEN		;open file for read
;
   LD    B,FERR_NOTFOUND
   JP    NC,FIO_IN_ABANDON	;error opening file
;
   LD    B,FERR_BADTYPE
   CP    FILE_BINARY		;Is file binary?
   JR    NZ,FIO_LdBinFileErr	;No (bad type of file)
;
   LD    HL,(FileLoadAddr)
   LD    A,H
   OR    L			;Load address supplied
   JR    NZ,FIO_LdBinFile1	;Yes (use it)
;
   EX    DE,HL			;No (use whats in header)
FIO_LdBinFile1
   CALL  CAS_IN_DIRECT		;Yes (load contents)
;
   LD    B,FERR_READ
   JR    NC,FIO_LdBinFileErr	;Read error
;
FIO_LdBinFileExit
   LD    B,FERR_OKAY		;Read okay
FIO_LdBinFileErr
   JP    FIO_IN_CLOSE		;Close handle
;=========================================================================
;Name     FIO_SaveHTML
;Purpose  To save the current HTML data to media.
;In Regs  AF = ??
;         B = length of filename
;         DE = ??
;         HL = address of filename
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    CAS_OUT_OPEN (OS)
;         CAS_OUT_CHAR (OS)
;         CAS_OUT_CLOSE (OS)
;         Mem_SwapInHTML
;         Mem_SwapInMain
;         UTL_Strlen
;Notes    1. This routine saves the HTML to a new file rather than rename
;            the loaded file.
;         2. This routine can be loaded from media to reduce the app
;            footprint . The routine would be built to operate at &8000.
;         3. Assume enough space for saved file exists.
;Fixes    1. change this routine to save all frames/objects.
;=========================================================================
FIO_SaveHTML
   LD    (Save_Temp),HL			;Save address of filename
   CALL  UTL_Strlen			;get length of filename
   LD    B,A
   LD    A,FILE_ASCII
   LD    DE,AMSDOS_BUFFER
   CALL  CAS_OUT_OPEN
   XOR   A
   LD    HL,(BaseHTMLLoc)
FIO_SveHTML_0
   LD    (HTML_CurrLoc+2),A		;Save RAM bank index.
   CALL  Mem_SwapInHTML
FIO_SveHTML_1
   LD    A,(HL)
   CP    NUL				;End of HTML data?
   JR    Z,FIO_SveHTML_2		;Yes, close file
   PUSH  HL
   CALL  CAS_OUT_CHAR			;write to file data byte char
   POP   HL
   INC   HL
   BIT   7,H				;At limit of memory bank?
   JR    Z,FIO_SveHTML_1		;No, get next data byte
   LD    A,(HTML_CurrLoc+2)
   INC   A
   LD    HL,SysBnkBuffer

   JR    FIO_SveHTML_0
FIO_SveHTML_2
   CALL  Mem_SwapInMain
   JP    CAS_OUT_CLOSE
;=========================================================================
;Name     FIO_CatDevice
;Purpose  To get a catalog of files from the current device.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TDB
;Uses     TBD
;Call By  TBD
;Calls    SC_SEL_STREAM (OS)
;         SYS_SaveRegs
;         Ext_CatalogDevice
;         SYS_RunCode1a
;         UTL_PatchOutBB5A
;         UTL_PatchInBB5A
;Notes    1. The current limit on the number of files that can be logged
;            is 896 (AMSDOS).
;=========================================================================
FIO_CatDevice
   LD    A,(Stream_WebPage)
   CALL  SC_SEL_STREAM		;ensure cataclog uses full width of screen
;
   LD    HL,LOGTEXT_LOCN
   LD    DE,AMSDOS_BUFFER
   CALL  SYS_SaveRegs
   CALL  UTL_PatchOutBB5A	;re-direct &BB5a to capture output.
   LD    HL,Ext_CatalogDevice
   CALL  SYS_RunCode1		;execute device catalog
   JP    UTL_PatchInBB5A	;restore &BB5a
;=========================================================================
;Name     FIO_GetCatFile
;Purpose  To get a filename from the catalog list.
;In Regs  AF = ??
;         BC = File number index into the catalog listing.
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TDB
;Uses     TBD
;Call By  TBD
;Calls    SYS_SaveRegs
;         Ext_GetCatalogItem
;         SYS_RunCode1a
;Notes    TBD
;=========================================================================
FIO_GetCatFile
   LD    HL,AMSDOS_BUFFER
   LD    DE,KB_SrchTXt
   CALL  SYS_SaveRegs
   LD    HL,Ext_GetCatalogItem
   JP    SYS_RunCode1			;execute external function.
;=========================================================================
;Name     MEM_FillMemory
;Purpose  To insert a blank filename
;In Regs  AF = ??
;         BC = Start file index into catalog of item to show.
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars EndFileIdx = index to last file displayed.
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    1. MOVE TO MEMORY HANDLER MODULE.
;=========================================================================
MEM_FillMemory
   LD    HL,AMSDOS_BUFFER
   LD    B,MAXFILENAMETXT
MEM_FillMemory1
   LD    (HL),SPACE_CHAR
   INC   HL
   DJNZ  MEM_FillMemory1
   LD    (HL),0
   RET
;=========================================================================
;Name     FIO_ShowCatItems
;Purpose  To display the files for the selection list.
;In Regs  AF = ??
;         BC = Start file index into catalog of item to show.
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars EndFileIdx = index to last file displayed.
;Uses     TBD
;Call By  TBD
;Calls    MNU_AdjustLocn
;         FIO_GetCatFile
;         MEM_FillMemory
;         WIN_ShowWindwData
;Notes    TBD
;=========================================================================
FIO_ShowCatItems
   LD    DE,CATFSC_POS			;Column 3, Row 4
   CALL  MNU_AdjustLocn
   LD    (FIO_Temp),DE
   LD    BC,(FirstFileIdx)
FIO_ShowCatItems1
   LD    (EndFileIdx),BC
   CALL  FIO_GetCatFile			;Get nth catalog item
   LD    A,(AMSDOS_BUFFER)
   OR    A				;Was item got?
   CALL  Z,MEM_FillMemory		;No, setup blank filename
;
   LD    HL,AMSDOS_BUFFER
   LD    DE,(FIO_Temp)			;Get window coordinates
   CALL  WIN_ShowWindwData		;display filename
;
   LD    DE,(FIO_Temp)			;Get window coordinates
   LD    A,(Window_FERow)
   DEC   A
   CP    E				;At bottom of window?
   RET   Z				;Yes
   INC   E				;Next row
   LD    (FIO_Temp),DE
   LD    BC,(EndFileIdx)
   INC   BC				;next file to display.
   JR    FIO_ShowCatItems1		;No (get another)
;=========================================================================
;Name     FIO_ShowCurrCatItem
;Purpose  To show current file from the selection list.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TDB
;Uses     TBD
;Call By  TBD
;Calls    MNU_AdjustLocn
;         FIO_SelectThisFile
;         WIN_ShowWindwData
;Notes    TBD
;=========================================================================
FIO_ShowCurrCatItem
   CALL  FIO_SelectThisFile   
   LD    DE,CATFCI_POS			;Column 17, Row 3
   CALL  MNU_AdjustLocn
   LD    HL,AMSDOS_BUFFER
   CALL  WIN_ShowWindwData
   LD    HL,AMSDOS_BUFFER
   LD    (HL),0				;kill selection
   RET
;=========================================================================
;Name     FIO_SelDevPath
;Purpose  To check if the device and path is valid
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         B = status of device path request
;             0 = ESC pressed
;             1 = no filename, get catalog and select
;             2 = input has filename was entered.
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_Buffer = current path to validate.
;         KB_KeyTested = corrupted
;Uses     TBD
;Call By  TBD
;Calls    MNU_ReqInputTxt
;         FIO_OpenDevPath
;         UTL_strlen
;         UTL_strEOL
;         UTL_strfind
;Notes    TBD
;=========================================================================
FIO_SelDevPath
   LD    HL,PageOpenReqTxt	;address of request text
   LD    BC,REQUEST_TYP2	;new input, leave it up.
   CALL  MNU_ReqInputTxt
;
   LD    B,0
   LD    A,(KB_KeyTested)
   CP    KEY_ESC		;Escaped pressed?
   JP    Z,Err_ClrErrTxt	;Yes
;
   LD    B,1
   LD    HL,KB_Buffer
   CALL  UTL_strlen
   OR    A			;Input entered?
   RET   Z			;No (Open catalog on curr dev)
;
   CALL  UTL_strEOL
   LD    B,2			;flag no filename entered
   DEC   HL			;Point to last char in input.
   LD    A,(HL)
   CP    SLASH_CHAR		;Was a file name entered?
   RET   NZ			;Yes
;
   LD    HL,KB_Buffer
   LD    C,BSLASH_CHR
   CALL  UTL_strfind		;goto start of device specification
   CALL  FIO_OpenDevPath	;select the device and path
   LD    B,1			;get catalog and select
   RET
;=========================================================================
;Name     FIO_SelectFile
;Purpose  To select a file from the list shown.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = address of buffer to contain the URL to open.
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TDB
;Uses     TBD
;Call By  TBD
;Calls    WIN_ClearWebPage
;         URL_GetBaseURL
;         UTL_strcpy
;         FIO_SelDevPath
;         FIO_CatDevice
;         MNU_ShowWindowItems
;         FIO_ShowCatFilter
;         FIO_ShowCurrCatItem
;         FIO_ShowCatItems
;         KB_Wait4akey
;         UTL_strEOL
;         MNU_PageRefresh
;Notes    1. The contents of the web-page window are removed.
;=========================================================================
FIO_SelectFile
   CALL  WIN_ClearWebPage
   LD    A,(CursorKey_Up)
   LD    (SelFile_CU),A
   LD    A,(CursorKey_Down)
   LD    (SelFile_CD),A
   LD    A,(CursorKey_Select)
   LD    (SelFile_Select),A
;
   CALL  URL_GetBaseURL
   LD    HL,URL_ObjStrtTxt
   LD    DE,KB_Buffer
   CALL  UTL_strcpy			;setup request string.
;
FIO_SelctFile0
   CALL  FIO_SelDevPath			;request device/path/filename
   LD    A,B
   OR    A				;Escape pressed?
   RET   Z				;Yes
;
   CP    2				;input has filename in it?
   RET   Z				;Yes
;
   CALL  FIO_CatDevice			;Get list of files
;
   LD    HL,0
   LD    (FirstFileIdx),HL		;Start index of files to show
   LD    (CurrFileIdx),HL		;current selected file
;
   LD    HL,SelectFileData
   CALL  MNU_ShowWindowItems		;Display window data with no files
;
FIO_SelctFile1
   CALL  FIO_ShowCatFilter
   CALL  FIO_ShowCurrCatItem		;display current file
   CALL  FIO_ShowCatItems		;Display the catalog items
;
   LD    HL,(MenuWinKeyLoc)
   CALL  KB_Wait4akey			;execute any key pressed
   LD    A,(KB_KeyTested)
   CP    KEY_ESC			;Abort selection?
   JR    Z,FIO_SelctFile3		;No
;
   CP    KEY_D				;New Device/path?
   JR    Z,FIO_SelctFile0		;Yes
;
   CP    KEY_F				;file Filter changed
   JR    Z,FIO_SelctFile1		;Yes
;
   LD    A,(AMSDOS_BUFFER)
   OR    A				;Was item got?
   JR    Z,FIO_SelctFile1		;No (re-display and select)
;
;  Goto to end of entry
;
   LD    HL,KB_Buffer
   CALL  UTL_strEOL			;HL = end of path string
;
;  Add in filename with no spaces after path
;
   LD    DE,AMSDOS_BUFFER
FIO_SelctFile4
   LD    A,(DE)
   LD    (HL),A
   OR    A				;end of filename?
   RET   Z				;yes
   INC   DE
   CP    SPACE_CHAR			;space in filename?
   JR    Z,FIO_SelctFile4		;yes (ignore it)
   INC   HL
   JR    FIO_SelctFile4
;
FIO_SelctFile3
   LD    HL,KB_Buffer
   LD    (HL),0				;kill KB buffer
   JP    MNU_PageRefresh		;initialise a refresh.
;=========================================================================
;Name     FIO_SelectPrevFile
;Purpose  To adjust the file selction display parameters.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  CurrFileIdx = current index into the catalog files list
;         FirstFileIdx = current file index of displayed items.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FirstFileIdx = current file index of displayed items.
;Uses     TBD
;Call By  TBD
;Calls    FIO_ShowCatItems
;         FIO_SelectThisFile
;Notes    TBD
;=========================================================================
FIO_SelectPrevFile
   LD    HL,AMSDOS_BUFFER
   LD    (HL),0				;kill selection
   LD    DE,(CurrFileIdx)
   LD    A,E				;At start of list?
   OR    D
   RET   Z				;Yes
   DEC   DE
   LD    (CurrFileIdx),DE
   LD    HL,(FirstFileIdx)
   OR    A
   SBC   HL,DE				;At top of displayed list?
   JR    NZ,FIO_SelPrevFile1		;No
   LD    HL,(FirstFileIdx)
   DEC   HL
   LD    (FirstFileIdx),HL
   CALL  FIO_ShowCatItems		;Display the catalog items
FIO_SelPrevFile1
   CALL  FIO_SelectThisFile
   LD    HL,AMSDOS_BUFFER
   LD    (HL),0				;kill selection
   RET
;=========================================================================
;Name     FIO_SelectNextFile
;Purpose  To adjust the file selction display parameters.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  CurrFileIdx
;         EndFileIdx
;         FirstFileIdx
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FirstFileIdx
;         CurrFileIdx
;         AMSDOS_BUFFER = no file selected
;Uses     TBD
;Call By  TBD
;Calls    FIO_ShowCatItems
;         FIO_SelectThisFile
;Notes    TBD
;=========================================================================
FIO_SelectNextFile
   LD    BC,(CurrFileIdx)
   LD    HL,(EndFileIdx)
   OR    A
   SBC   HL,BC				;At end of displayed list?
   JR    NZ,FIO_SelNextFile1		;No
;
   LD    HL,(FirstFileIdx)
   INC   HL
   LD    (FirstFileIdx),HL
   CALL  FIO_ShowCatItems		;Display the catalog items
FIO_SelNextFile1
   LD    BC,(CurrFileIdx)
   INC   BC
   LD    (CurrFileIdx),BC
   CALL  FIO_SelectThisFile
   LD    HL,AMSDOS_BUFFER
   LD    (HL),0				;kill selection
   RET
;=========================================================================
;Name     FIO_SelectThisFile
;Purpose  To add the selected file name to the keyboard buffer to complete
;         file to be opened.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  CurrFileIdx
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TDB
;Uses     TBD
;Call By  TBD
;Calls    FIO_GetCatFile
;Notes    TBD
;=========================================================================
FIO_SelectThisFile
   LD    BC,(CurrFileIdx)
   JP    FIO_GetCatFile			;Get nth catalog item
;=========================================================================
;Name     FIO_SelFilter
;Purpose  To enter the file filter reduce the file selections.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  KB_SrchTxt = current file filter setting.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_EchoFlag
;         AMSDOS_BUFFER
;         KB_printlen
;         KB_MaxInpRC
;Uses     TBD
;Call By  TBD
;Calls    MNU_AdjustLocn
;         UTL_strlen
;         KB_GetWinInput
;Notes    TBD
;=========================================================================
FIO_SelFilter
   XOR   A
   LD    (KB_EchoFlag),A		;normal input/echo
   LD    (AMSDOS_BUFFER),A		;kill selection
   LD    DE,CATFFF_POS			;Column 17, Row 2
   CALL  MNU_AdjustLocn
   LD    A,MAXFILENAME
   LD    (KB_printlen),A
   PUSH  DE
   ADD   A,D
   LD    D,A
   LD    (KB_MaxInpRC),DE
   POP   DE
   LD    C,0				;Get new input
   LD    HL,KB_SrchTxt
   EX    DE,HL
   CALL  KB_GetWinInput
;=========================================================================
;Name     FIO_ShowCatFilter
;Purpose  To display the current file filter parameter
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  KB_SrchTxt = current file filter setting.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars SysGenBuffer
;Uses     TBD
;Call By  TBD
;Calls    MNU_AdjustLocn
;         WIN_ShowWindwData
;         UTL_strncpy
;Notes    TBD
;=========================================================================
FIO_ShowCatFilter
   LD    HL,KB_SrchTxt
   LD    DE,SysGenBuffer
   LD    B,MAXFILENAME
   CALL  UTL_strncpy
   LD    A,B
   OR    A
   JR    Z,FIO_ShowCatFilter1
   LD    (HL),0
   LD    A,32
FIO_ShowCatFilter2
   LD    (DE),A
   INC   DE
   DJNZ  FIO_ShowCatFilter2
FIO_ShowCatFilter1
   XOR   A
   LD    (DE),A
   LD    DE,CATFCI_POS-1		;Column 17, Row 2
   CALL  MNU_AdjustLocn
   LD    HL,SysGenBuffer
   JP    WIN_ShowWindwData
;=========================================================================
;Name     FIO_RequestFile
;Purpose  To request for a filename.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_Buffer
;Uses     TBD
;Call By  TBD
;Calls    MNU_ReqInputTxt
;         FIO_SelectFile
;Notes    TBD
;=========================================================================
FIO_RequestFile
   LD    HL,FilenameReqTxt
   LD    BC,REQUEST_TYP3
   CALL  MNU_ReqInputTxt
   LD    HL,KB_Buffer
   LD    A,(HL)
   OR    A			;Any keyboard input?
   RET   NZ			;Yes
;
   CALL  FIO_SelectFile		;Display requestor for list of files.
   LD    HL,KB_Buffer
   LD    A,(HL)
   OR    A			;A file selected?
   RET
;=========================================================================
;Name     FIO_ReqFName
;Purpose  To request for a filename to save to or to use a default file
;         name.
;In Regs  A = ??
;         BC = ??
;         DE = Address of default filename
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    FIO_RequestFile
;Notes    TBD
;=========================================================================
FIO_ReqFName
   PUSH  DE
   CALL  FIO_RequestFile
   POP   DE
   RET   NZ
   EX    DE,HL
   RET