;=========================================================================
;
; Application Routines (HTML rendering commands)
; Copyright (c) 2010 Ray Palmer
;
; Notes
;
; 1. The rendering of the current command does not expect that the same
;    command to be recursed. This may lead to an infinite loop.
;
; 2. Where a command changes the font (e.g normal to italic) the original
;    command will have no way of reverting the font back to what it was.
;    This is due to manner in which fonts are handled.
;
;=========================================================================
;Name     HTML_ShowFile
;Purpose  To show the loaded file data from the banks of memory. If the
;         first entry into this routine determines that the file data is
;         not HTML then the rendering will be just ASCII text.
;In Regs  A = HTML_RenderState
;         BC = ??
;         DE = address of label text (if any)
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  FileOpState = state of laoding file
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_RenderState = updated rendering state
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTML_ShowFile
   RES   4,A				;page now displayed.
   LD    (HTML_RenderState),A
   XOR   A
   CALL  MEM_SwapInHTML
   LD    HL,(BaseHTMLLoc)
   LD    A,(HL)	 			;Get first char of loaded data
   OR    A				;Is there any data to display
   JP    Z,Mem_SwapInMain		;No
;
   LD    A,(URL_LblType)
   LD    C,A
   LD    A,(WrapLock)
   LD    B,A
   LD    DE,(URL_LblAddr)		;DE = location of text to find
   LD    HL,KB_Buffer			;buffer for HTML processing.
   CALL  SYS_SaveRegs			;Save AF/BC/DE/HL
;
   LD    A,(WindowFrmeStat)
   BIT   0,A				;Specific frame refresh ?
   JR    NZ,HTML_ShowFile1		;Yes
   LD    A,(Stream_WebPage)
   CALL  WIN_ClearStream
;
HTML_ShowFile1
   LD    BC,HTM_ROM_ShowFile
   CALL  HTML_ROMCall			;call rom routine to show file.
   LD    A,255
   LD    (URL_LblType),A		;kill text searching.
   RET
;=========================================================================
;Name     HTML_Rom_Init
;Purpose  To Initialise the HTML rom.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTML_Rom_Init
   LD    BC,HTM_ROM_Init
   JR    HTML_ROMCall
;=========================================================================
;Name     HTML_ViewSrce
;Purpose  To view the HTML source from the loaded file.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTML_ViewSrce
   LD    A,(Stream_WebPage)
   CALL  SC_SEL_STREAM			;Select stream for main menu
   CALL  TXT_CLR_WIN
   LD    A,(WindowFrmeNum)
   OR    A
   JR    NZ,HTML_VwSrce1
   INC   A
HTML_VwSrce1
   CALL  SYS_SaveRegs
   LD    BC,HTM_ROM_ViewHTML
   CALL  HTML_ROMCall
;
   CALL  HTML_Rom_Init			;Reset HTML rom for refresh of data
   JP    MNU_PageRefresh		;Flag main routine to refesh display
;=========================================================================
;
;  Web-Page/Menu handlers
;
;=========================================================================
;Name     HTML_FindHLink
;Purpose  To search the shown HTML for a clicked on hyperlink.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  KB_Buffer has null terminator at start.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. Since the hyperlinks database sits in banksed memory the use
;            routines which reside in banked memory can not yet be used.
;=========================================================================
HTML_FindHLink
   LD    BC,DB_HyperLink
   CALL  SYS_SaveRegs			;Save AF/BC/DE/HL
   LD    BC,HTM_FindHyperLink
   JR    HTML_ROMCall
;=========================================================================
;Name     HTML_FindText
;Purpose  To search the HTML for a text string.
;In Regs  A = First char in HTML data
;         B = seach type (1=Find first, 2=Find next)
;         C = ??
;         DE = address of search text
;         HL = where to save result byte
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTML_FindText
   CALL  SYS_SaveRegs	;Save AF/BC/DE/HL
   LD    BC,HTM_FindText
   JR    HTML_ROMCall
;=========================================================================
;Name     HTML_ActionFormsItem
;Purpose  To act on the clicked forms item.
;In Regs  A = Forms Db item number
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_Buffer maybe corrupted.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTML_ActionFormsItem
   CALL  SYS_SaveRegs	;Save AF/BC/DE/HL
   LD    BC,HTM_ActionFormItem
   JR    HTML_ROMCall
;=========================================================================
;Name     HTML_FindAppIcon
;Purpose  To find which App icon was selected
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTML_FindAppIcon
   LD    BC,HTM_FindAppIcon
   JR    HTML_ROMCall
;=========================================================================
;Name     HTML_FindFrameIcon
;Purpose  To find which frame icon was selected
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTML_FindFrameIcon
   LD    BC,HTM_FindFrameIcon
   JR    HTML_ROMCall
;=========================================================================
;Name     HTML_CalcFrameCPC
;Purpose  To caclulate the CPC screen dimensions for a frame.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTML_CalcFrameCPC
   LD    BC,HTM_CalcFrameCPC
   JR    HTML_ROMCall
;=========================================================================
;Name     HTML_FindFormsItem
;Purpose  To search the DB of Forms Items
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTML_FindFormsItem
   CALL  SYS_SaveRegs	;Save AF/BC/DE/HL
   LD    BC,HTM_FindFormItem
HTML_ROMCall
   LD    HL,ROM_BaseAddress
   ADD   HL,BC
   LD    A,(ROM_ShowHTMLFile+3)
   JP    SYS_RunCode4
