;=========================================================================
;
; Application Routines (Keyboard)
; Copyright (c) 2010 Ray Palmer
;
;=========================================================================
;Name     KB_WaitRelease
;Purpose  To wait for a specific key to be released.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  KB_KeyTested = structure updated
;Out Regs AF = Flags updated
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    KB_TstAKey
;Notes    TBD
;=========================================================================
KB_WaitRelease
   LD    BC,(Cursor_Speed)	;cursor control wait time
   LD    A,(KB_KeyTested)
   LD    E,A
   LD    A,(CursorKey_Up)
   CP    E
   JR    Z,KB_WaitRelease1
   LD    A,(CursorKey_Down)
   CP    E
   JR    Z,KB_WaitRelease1
   LD    A,(CursorKey_Left)
   CP    E
   JR    Z,KB_WaitRelease1
   LD    A,(CursorKey_Right)
   CP    E
   JR    Z,KB_WaitRelease1
;
   LD    BC,(KB_KeyDelayT)
KB_WaitRelease1
   LD    A,(KB_KeyTested)
   PUSH  BC
   CALL  KM_TESTKEY		;test if key is still pressed
   POP   BC
   RET   Z			;key is not pressed
   DEC   BC
   LD    A,B			;Wait time up?
   OR    C
   JR    NZ,KB_WaitRelease1	;No, check again.
   RET
;=========================================================================
;Name     KB_TstAKey
;Purpose  To test for a specific key
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = address of key test structure
;         IX = ??
;         IY = ??
;In Vars  N/A
;Out Regs AF = Key Id and Flags updated
;         BC = ??
;         DE = ??
;         HL = address of next key structure
;         IX = ??
;         IY = ??
;Out Vars KB_KeyTested = structure updated
;Uses     TBD
;Call By  TBD
;Calls    KM_TESTKEY (OS)
;Notes    TBD
;=========================================================================
KB_TstAKey
   LD    DE,KB_KeyTested
   LD    BC,4
   LDIR				;download key structure
   LD    A,(KB_KeyTested)
   PUSH  HL
   CALL  KM_TESTKEY		;check if key was pressed.
   POP   HL
   LD    A,(KB_KeyTested)	;return key being checked
   RET
;=========================================================================
;Name     KB_TstCursor
;Purpose  To test for a cursor control keys
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = NZ flag set if key pressed else Z flag
;         BC = ??
;         DE = corrupted.
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_KeyTested = structure updated with key pressed
;Uses     TBD
;Call By  TBD
;Calls    KB_TstAKey
;Notes    TBD
;=========================================================================
KB_TstCursor
   LD    HL,CursorKeys
   CALL  KB_TstAKey			;check cursor left shift
   RET   NZ
   CALL  KB_TstAKey			;check cursor right shift
   RET   NZ
   CALL  KB_TstAKey			;check cursor tab
   RET   NZ
;
   LD    HL,CursorKey_Up
   JR    KB_TstScroll1
;=========================================================================
;Name     KB_TestScroll
;Purpose  To test for a cursor control keys for frame scrolling.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = Flags updated
;         BC = ??
;         DE = corrupted.
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_KeyTested = structure updated
;Uses     TBD
;Call By  TBD
;Calls    KB_TstAKey
;Notes    TBD
;=========================================================================
KB_TestScroll
   LD    HL,ScrollCursorKeys
KB_TstScroll1
   CALL  KB_TstAKey			;check cursor up
   RET   NZ
   CALL  KB_TstAKey			;check cursor down
   RET   NZ
   CALL  KB_TstAKey			;check cursor left
   RET   NZ
   CALL  KB_TstAKey			;check cursor right
   RET   NZ
   CALL  KB_TstAKey			;check cursor select
   RET
;=========================================================================
;Name     KB_ResetCursor
;Purpose  To reset the cursor configuration and cursor position.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  N/A
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars Cursor_Row = cursor column position on the CPC screen.
;         Cursor_Col = cursor row position on the CPC screen
;Uses     TBD
;Call By  TBD
;Calls    Mem_SwapInMain
;         Mem_SwapInCfg
;Notes    1. Main memory configuration restored.
;=========================================================================
KB_ResetCursor
   LD    HL,0
   LD    (Cursor_IRow),HL
   LD    (Cursor_ICol),HL
;=========================================================================
;Name     KB_SetupCursor
;Purpose  To install the configured cursor keys into the browser vars
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  N/A
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars ??
;Uses     TBD
;Call By  TBD
;Calls    Mem_SwapInCfg
;Notes    1. Main memory configuration restored.
;=========================================================================
KB_SetupCursor
   CALL  Mem_SwapInCFG
   LD    HL,CFG_CurKey_Up
   LD    A,(HL)
   INC   HL
   LD    (CursorKey_Up),A
   LD    A,(HL)
   INC   HL
   LD    (CursorKey_Down),A
   LD    A,(HL)
   INC   HL
   LD    (CursorKey_Right),A
   LD    A,(HL)
   INC   HL
   LD    (CursorKey_Left),A
   LD    A,(HL)
   INC   HL
   LD    (CursorKey_Select),A
   LD    A,(CFG_Cursor_Icon)
;=========================================================================
;Name     KB_SetupCursorIcon
;Purpose  To install the configured cursor icon
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  N/A
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars ??
;Uses     TBD
;Call By  TBD
;Calls    SYS_SaveRegs
;         SYS_RunCode2
;Notes    1. Main memory configuration restored.
;=========================================================================
KB_SetupCursorIcon
   LD    (Cursor_Icon),A
   PUSH  AF
   LD    A,(Cursor_IcnSet)
   LD    B,A
   POP   AF
   CALL  SYS_SaveRegs	;Save all reg for external routine
   LD    HL,Ext_SetCursorIcon
   JR    KB_ExtCommon
;=========================================================================
;Name     KB_ResetCursorIcon
;Purpose  To reset cursor icon status.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  N/A
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars ??
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
KB_ResetCursorIcon
   LD    HL,Ext_ResetCursor
   JR    KB_ExtCommon
;=========================================================================
;Name     KB_RemoveCursor
;Purpose  To restore the cursor background
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  N/A
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars ??
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
KB_RemoveCursor
   LD    HL,Ext_RemoveCursor
   JR    KB_ExtCommon
;=========================================================================
;Name     KB_ShowCursor
;Purpose  To show the cursor on the web-page
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  Cursor_IRow = current CPC cursor row position
;         Cursor_ICol = current CPC cursor column position
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    Ext_ShowCursor - custom routine
;Notes    1. Cursor positions always start from 1
;=========================================================================
KB_ShowCursor
   LD    HL,(Cursor_IRow)	;L = Row
   LD    DE,(Cursor_ICol)
   LD    H,E			;H = Column
   LD    DE,0			;no offsets
   LD    C,&30
   CALL  SYS_SaveRegs		;Save all reg for external routine
   LD    HL,Ext_ShowCursor
KB_ExtCommon
   JP    SYS_RunCode2
;=========================================================================
;Name     KB_Test4akey
;Purpose  To test for a specific key in a list of keys to be pressed.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = address of keys list to check for.
;         IX = ???
;         IY = ??
;In Vars  N/A
;Out Regs AF = key id of key pressed.
;         BC = ??
;         DE = ??
;         HL = address of which key was pressed.
;         IX = ??
;         IY = ??
;Out Vars KB_KeyTested = structure updated with key pressed data
;Uses     TBD
;Call By  TBD
;Calls    KB_TstAKey
;         KB_TstCursor
;Notes    1. This routine also checks for cursor movement as well.
;=========================================================================
KB_Test4akey
   LD    A,(HL)
   CP    KEY_ENDLIST		;end of list of keys?
   RET   Z			;yes
   CP    KEY_CURSORS		;Testing for cursor controls
   JR    NZ,KB_Test4akey_2	;No
   PUSH  HL
   CALL  KB_TstCursor		;check cursor keys
   POP   HL
   RET   NZ			;One of the cursor keys pressed.
   INC   HL
   JR    KB_Test4akey
KB_Test4akey_2
   CALL  KB_TstAKey		;specific key pressed?
   JR    Z,KB_Test4akey		;No (check next in list)
   RET   			;Key is pressed
;=========================================================================
;Name     KB_Wait4akey
;Purpose  To wait for a specific key in a list of keys to be pressed.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = address of keys list to check for.
;         IX = ???
;         IY = ??
;In Vars  N/A
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = address of which key was pressed.
;         IX = ??
;         IY = ??
;Out Vars KB_KeyListLoc = location of keys to check for.
;         KB_KeyTested = structure updated
;Uses     TBD
;Call By  TBD
;Calls    KB_Test4akey
;         KB_WaitRelease
;         SYS_DoHandler
;Notes    1. The stack contains the menu order for exits.
;=========================================================================
KB_Wait4akey
   LD    A,(HL)
   CP    KEY_ENDLIST		;Any keys to check for?
   RET   Z			;No
KB_Wait4akey1
   PUSH  HL
   CALL  KB_Test4akey		;check which key was pressed
   POP   HL
   CP    KEY_ENDLIST		;No key pressed ?
   JR    Z,KB_Wait4akey1	;Yes (try again)
;
   CALL  KB_WaitRelease		;wait to release it
   LD    HL,(KB_KeyTested+1)	;get address of handler
   LD    A,H
   AND   L
   CP    &FF			;Genereal handler for ESC?
   JP    Z,Err_ClrErrTxt	;Yes, Clear text on ESC
;=========================================================================
;Name     KB_DoHandler
;Purpose  To handler the routine for the key pressed.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ???
;         IY = ??
;In Vars  KB_KeyTested = structure of key pressed.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    SYS_DoHandler
;Notes    TBD
;=========================================================================
KB_DoHandler
   LD    HL,KB_KeyTested	;Address of key pressed data
   JP    SYS_DoHandler		;execute key handler
;=====================================================================
;Name     KB_handlechar
;Purpose  To process a character entered by the keyboard and store it
;          into the keyboard buffer.
;Input    A = Character
;Output   C = EOL status
;Uses     None.
;Call By  TBD
;Calls    None
;Notes    1. Assume keyboard buffer is max 255 chars.
;=====================================================================
KB_handlechar
   LD    C,KB_EOL
   cp    32			;Any control char?
   RET   C			;Yes, treat as EOL
   BIT   7,A			;Any top-bit-set char is EOL
   RET   NZ			;Yes, treat as EOL
   LD    C,KB_NOTEOL
   LD    B,A
   ld    DE,(KB_Offset)		;DE = buffer offset
   ld    HL,(KB_BuffLoc)	;HL = buffer location
   CP    KB_DEL			;DEL Char ?
   JR    Z,KB_deletechar
   LD    A,E
   CP    255			;At max buffer?
   RET   Z			;yes
   ADD   HL,DE			;HL = address in key buffer
   LD    (HL),B			;Save char in keyboard buffer
   INC   HL
   LD    (HL),NUL
   INC   DE			;New offset
   LD    (KB_Offset),DE		;Save new offset
   RET
;
KB_deletechar
   DEC   E
   INC   E			;At start of buffer
   RET   Z			;Yes, del not possible
   DEC   E			;back 1 char
   ADD   HL,DE			;HL = address in key buffer
   LD    (HL),NUL
   LD    (KB_Offset),DE		;Save newoffset
   RET
;=========================================================================
;Name     KB_ReadChar
;Purpose  To read a charcter from the keyboard or test various keys.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = key from OS function KM_READ_CHAR if not special case
;              else contains the CPC keyboard key number
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_KeyTested = key that was pressed for special handling.
;Uses     TBD
;Call By  TBD
;Calls    KM_READ_CHAR (OS)
;         KM_TESTKEY (OS)
;         KB_WaitRelease
;Notes    TBD
;=========================================================================
KB_ReadChar
   LD    A,KEY_ENTER
   LD    (KB_KeyTested),A
   CALL  KM_TESTKEY		;Was ENTER key pressed
   LD    A,CR
   JR    NZ,KB_ReadChr1		;Yes
   LD    A,KEY_ESC
   LD    (KB_KeyTested),A
   CALL  KM_TESTKEY		;Was ESC key pressed
   LD    A,CTRL_C
   JR    NZ,KB_ReadChr1		;Yes
   LD    A,KEY_DEL
   LD    (KB_KeyTested),A
   CALL  KM_TESTKEY		;Was DEL key pressed
   LD    A,KB_DEL
   JR    NZ,KB_ReadChr1		;Yes
   CALL  KM_READ_CHAR		;get char from keyboard
   RET   C			;Character read in.
   XOR   A
   RET
KB_ReadChr1
   PUSH  AF
   CALL  KB_WaitRelease
KB_ReadChr2
   CALL  KM_READ_CHAR
   JR    C,KB_ReadChr2		;key in buffer, get another
   POP   AF
   RET
;=====================================================================
;Name     KB_GetWinInput
;Purpose  To get a key string for a Window menu.
;Input    AF = ??
;         B = ??
;         C = Input already filled in flag
;             0 means existing input is to be updated
;               else input cleared for new text
;         DE = location for keyboard input.
;         H = Start column for output
;         L = start row for output
;         IX = ??
;         IY = ??
;In Vars  KB_EchoFlag = input echo flags
;                       Bit Description
;                        7  0= single line input
;                           1= multi-line input (ESC terminates)
;                       6-1 Unused/Reserved
;                        0  1 = hidden text output control.
;                           0 = show entered text
;         KB_printlen = max number of characters which can be echoed
;                       back.
;         KB_MaxInpRC = Max row/column for echo back.
;         KB_Offset = offset to end of string (if any pre-exists)
;Output   The location in Reg DE will contain the keyboard input.
;Out Vars KB_BuffLoc
;         KB_StartCol
;         KB_Offset
;         (KB_BuffLoc) = input string.
;         KB_KeyTested
;Uses     all registers
;Call by  TBD
;Calls    WIN_PrintStr
;         KM_READ_CHAR (OS)
;         KB_ReadChar
;         KB_handlechar
;Notes    1. Upon entry the original data will have the NUL code
;            inserted at the start.
;=====================================================================
KB_GetWinInput
   LD    (KB_BuffLoc),DE		;Setup location for input
   LD    (KB_StartCol),HL		;Save X/Y print location
   LD    (KB_BaseInpRC),HL		;Save X/Y for multi-line input.
   LD    A,C
   OR    A				;Already has input?
   JR    NZ,KB_GetWInput0a		;Yes
   XOR   A
   LD    (DE),A				;clear current input.
   LD    HL,0
   LD    (KB_Offset),HL			;reset echo back offset
KB_GetWInput0a
   PUSH  BC
;
;  Clear current input buffer of any keys
;
KB_GetWInput0
   CALL  KM_READ_CHAR			;get char from keyboard
   JR    C,KB_GetWInput0		;yes char read, try again
;
   POP   BC
   LD    A,C				;text already to be displayed?
   OR    A
   JR    NZ,KB_GetWInput3		;Yes
;
KB_GetWInput1
   CALL  KB_ReadChar			;get char from keyboard
   OR    A				;any key pressed?
   JR    Z,KB_GetWInput1		;no, try again
;
   LD    B,A
   LD    A,(KB_EchoFlag)
   BIT   7,A				;Multi-line input?
   JR    Z,KB_GetWInput1a		;No
;
   LD    A,(KB_KeyTested)
   CP    KEY_ESC			;exiting multi-line input?
   JR    Z,KB_GetWInput1b		;Yes
   JR    KB_GetWInput2
;
KB_GetWInput1a
   LD    A,(KB_KeyTested)
   CP    CTRL_C				;Abort whole input?
   JR    NZ,KB_GetWInput2		;No
;
KB_GetWInput1b
   LD    HL,(KB_BuffLoc)
   LD    (HL),0				;Kill off any existing input
   RET
;
KB_GetWInput2
   LD    A,B
   CALL  KB_handlechar			;insert char into buffer
   LD    A,C
   CP    KB_EOL				;end of input?
   JR    Z,KB_GetWInput4		;Yes
;
KB_GetWInput3
   LD    HL,(KB_BaseInpRC)		;Get screen location.
   LD    (KB_StartCol),HL		;Save X/Y print location
   CALL  WIN_PrintStr			;echo back input string.
   JR    KB_GetWInput1			;yes (get more input chars)
;
KB_GetWInput4
   LD    A,(KB_EchoFlag)
   BIT   7,A				;Multi-line input?
   JR    NZ,KB_GetWInput1		;Yes
   RET
;=========================================================================
;Name     MoveCursorTAB
;Purpose  To handle the TAB key for cursor control.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
MoveCursorTAB
   RET
;=========================================================================
;Name     MoveCursorSHFT
;Purpose  To handle the SHIFT key for cursor control. This is for the
;         scrolling of the web-page frames.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
MoveCursorSHFT
   RET
;=========================================================================
;Name     MoveCursorUp
;Purpose  To move the cursor up. If the cursor is at the top of the window
;         the page is scrolled up one line if there is a line to goto.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  PageWdw_SRow = current CPC screen row offset on web-page
;         PageWdw_ERow = current CPC screen column offset on web-page
;         Cursor_IRow = current CPC screen cursor row
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars PageWdw_SRow = updated CPC screen row offset on web-page
;         PageWdw_ERow = updated CPC screen column offset on web-page
;         Cursor_IRow = updated CPC screen cursor row
;Uses     TBD
;Call By  TBD
;Calls    WIN_ChkFrmes
;Notes    TBD
;=========================================================================
MoveCursorUp
   LD    HL,(Cursor_IRow)	;current CPC window position
   LD    A,L
   OR    H
   OR    A			;at top of screen?
   JR    Z,MoveCursorUp1	;yes, check if at top of web-page
   DEC   HL
   LD    (Cursor_IRow),HL
   RET
MoveCursorUp1
   LD    A,(PageLock)
   BIT   1,A			;Screen scroll locked?
   RET   NZ			;yes
   LD    HL,(PageWdw_SRow)
   LD    A,H			;At minimum position?
   OR    L
   RET   Z
   DEC   HL
   LD    (PageWdw_SRow),HL
   LD    HL,(PageWdw_ERow)
   DEC   HL
   LD    (PageWdw_ERow),HL
   CALL  WIN_ChkFrmes		;Check all frames for visibility
   JP    MNU_PageRefresh	;Flag refresh required
;=========================================================================
;Name     MoveCursorLeft
;Purpose  To move the cursor Left. If the cursor is on the left edge of
;         the window the web page moves left one character if there is a
;         column to goto.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  PageWdw_SCol = current CPC screen column start
;         Cursor_ICol = current CPC screen cursor column
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars PageWdw_SCol = current CPC screen column start
;         PageWdw_ECol = current CPC screen column end
;         Cursor_ICol = current CPC screen cursor column
;Uses     TBD
;Call By  TBD
;Calls    WIN_ChkFrmes
;Notes    1. The minimum CPC screen cursor is at position 1.
;         2. The minimum screen offset is at position 0.
;         3. The left most column of the screen is not seen as usable.
;=========================================================================
MoveCursorLeft
   LD    HL,(Cursor_ICol)	;current CPC column position
   LD    A,L
   OR    H
   OR    A			;At left most of CPC screen.
   JR    Z,MoveCursorLeft1	;Yes (check if web page window can move)
   DEC   HL
   LD    (Cursor_ICol),HL
   RET
MoveCursorLeft1
   LD    A,(PageLock)
   BIT   1,A			;Screen scroll locked?
   RET   NZ			;yes
   LD    HL,(Cursor_IRow)	;current CPC row position
   LD    A,L
   CP    5			;Is row within main menau area?
   RET   C			;Yes (no left/right scroll allowed)
   LD    HL,(PageWdw_SCol)
   LD    A,H			;At minimum left position?
   OR    L
   RET   Z			;Yes
   DEC   HL
   LD    (PageWdw_SCol),HL	;Shift Start column left 1 position
   LD    HL,(PageWdw_ECol)
   DEC   HL
   LD    (PageWdw_ECol),HL	;adjust end column as well
   CALL  WIN_ChkFrmes		;Check all frames for visibility
   JP    MNU_PageRefresh	;Flag refresh required
;=========================================================================
;Name     MoveCursorDown
;Purpose  To move the cursor down. If the current row is at the bottom of
;         the window the screen then scrolls down one line.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  PageWdw_SRow = current start row onto web page
;         Cursor_IRow = current CPC screen cursor row
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars PageWdw_SRow = current start row onto web page
;         PageWdw_ERow = current end row onto web page
;         Cursor_IRow = current CPC screen cursor row
;Uses     TBD
;Call By  TBD
;Calls    WIN_ChkFrmes
;Notes    1. The maximum CPC screen cursor is at position 21
;         2. The minimum screen offset is at position 65535, however the
;            screen display may become corrupt since this is there is not
;            enough space to show screen.
;=========================================================================
MoveCursorDown
   LD    HL,(Cursor_IRow)	;current cursor position in stream window
   LD    A,L
   CP    ER_WebPage-1		;beyond bottom of HTML window?
   JR    Z,MoveCursorDwn1
   INC   HL
   LD    (Cursor_IRow),HL
   RET
MoveCursorDwn1
   LD    A,(PageLock)
   BIT   1,A			;Screen scroll locked?
   RET   NZ			;yes
   LD    HL,(PageWdw_ERow)
   INC   HL
   LD    (PageWdw_ERow),HL	;Shift end row down 1 line as well.
   LD    HL,(PageWdw_SRow)
   INC   HL
   LD    (PageWdw_SRow),HL	;Shift start row down 1 line
   CALL  WIN_ChkFrmes		;Check all frames for visibility
   JP    MNU_PageRefresh	;Flag refresh required
;=========================================================================
;Name     MoveCursorRight
;Purpose  To move the cursor right. If the cursor is on the right edge of
;         the web page scrolls one column to the left.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  PageWdw_SCol = current start column to the window on web page
;         PageWdw_ECol = current end column to the window on web page
;         Cursor_ICol = current CPC screen cursor column
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars PageWdw_SCol = updated start colmn to the window on web page
;         PageWdw_ECol = updated end column to the window on web page
;         Cursor_ICol = current CPC screen cursor column
;Uses     TBD
;Call By  TBD
;Calls    WIN_ChkFrmes
;Notes    1. The maximum CPC screen cursor is at position 80.
;         2. The minimum screen offset is at position 65535, however the
;            screen display may become corrupt since this is where is not
;            enough space to show screen.
;=========================================================================
MoveCursorRight
   LD    HL,(Cursor_ICol)	;current cursor position in stream window
   LD    A,(Scrn_MaxCol)
   DEC   A
   CP    L			;At edge of window?
   JR    Z,MoveCursorR2
   INC   HL
   LD    (Cursor_ICol),HL	;Save new cursor row
   RET
MoveCursorR2
   LD    A,(PageLock)
   BIT   1,A			;Screen scroll locked?
   RET   NZ			;yes
   LD    HL,(Cursor_IRow)	;current CPC row position
   LD    A,L
   CP    5			;Is row within main menau area?
   RET   C			;Yes (no left/right scroll allowed)
   LD    HL,(PageWdw_ECol)
   LD    DE,(PageWdw_SCol)
   INC   HL
   INC   DE
   LD    (PageWdw_ECol),HL	;Shift end column as well
   LD    (PageWdw_SCol),dE	;Shift start column by 1 right
   CALL  WIN_ChkFrmes		;Check all frames for visibility
   JP    MNU_PageRefresh	;Flag refresh required
;=========================================================================
;Name     CursorSelect
;Purpose  To process the cursor select function. The routine checks the
;         following -
;         1. Hyperlinks
;         2. Frame icons for scrolling.
;         3. Frame scoller icons
;         4. Application icons
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  Cursor_ICol = current column in CPC stream window
;         Cursor_IRow = current row in CPC stream window
;         PageWdw_SCol = current start colum on web page window
;         PageWdw_SRow = current start row on web page window
;         BrowserState = browser state
;         CursorKey_Up
;         CursorKey_Down
;         CursorKey_Left
;         CursorKey_Right
;         Window_Roff
;         Window_Coff
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_buffer = cleared on click (if any)
;         URL_NewLocn = location of URL (hyperlink click)
;         WebPge_Row = absolute row on web page canvas window (>=0)
;         WebPge_Col = absolute column on web page canvas window (>=0)
;         Cursor_Row = current row in CPC stream window
;         Cursor_Col = current column in CPC stream window
;         BrowserState = bit 7 set if hyperlink click.
;Uses     TBD
;Call By  TBD
;Calls    HTML_FindFormsItem (ROM call) -  Find Forms Input item.
;         HTML_FindHLink (ROM Call) - Find a hyperlink to act on.
;         HTML_ActionFormsItem - Execute the action for the forms item.
;         App_StartApp3
;         WIN_FndFrame
;         WIN_RefreshFrame
;         KB_TestScroll
;         KB_WaitRelease1
;         KB_DoHandler
;         MNU_ShowOffsetXY
;         WIN_CalcFrmeDB
;         MEM_SwapInMain
;Notes    1. The CPC cursor values start from 1.
;         2. The Web page offsets start from 0
;=========================================================================
CursorSelect
   LD    HL,(Cursor_IRow)
   LD    BC,5
   OR    A
   SBC   HL,BC				;in main menu area?
   RET   C				;Yes (no checks for icons yet)
;
;  Setup cursor location on the web-page area
;
   LD    (Cursor_Row),HL		;save cursor row for web-page
   LD    HL,(Cursor_ICol)
   LD    (Cursor_Col),HL		;save cursor column for web-page
;
   DEC   HL				;adjust column from base 1 to 0
   LD    DE,(PageWdw_SCol)		;Web-Page offset
   ADD   HL,DE
   LD    (WebPge_Col),HL		;Web-Page column (absolute)
   LD    HL,(Cursor_Row)
   DEC   HL				;adjust row from base 1 to 0
   LD    DE,(PageWdw_SRow)
   ADD   HL,DE
   LD    (WebPge_Row),HL		;Web-Page Row (absolute)
;
   CALL  WIN_FndFrame			;Search which frame cursor has selected
   LD    A,(FrameItemSel)
   LD    L,A
   AND   &0F				;scroll icon selected?
   JP    NZ,WIN_RefreshFrame		;Yes
   LD    A,L
   AND   &30				;Slider icons selected?
   JR    NZ,CursorSelect2		;Yes
;
;  Search for a web-page hyperlink (if any)
;
   LD    HL,KB_buffer			;location for URL to open
   XOR   A
   LD    (HL),A				;Assume no link found
   PUSH  HL
   CALL  HTML_FindHLink			;Check if a hyperlink clicked on
   POP   HL
   LD    A,(HL)
   OR    A				;Did we find a hyperlink to open?
   JR    Z,CursorSelect1		;No (check if forms command item)
   LD    (URL_NewLocn),HL		;setup location of URL to open.
   LD    HL,BrowserState
   SET   7,(HL)				;Flag new URL to load.
   RET
;
CursorSelect1
;
;  Search for a web-page Forms item (if any)
;
   CALL  HTML_FindFormsItem		;check if click forms input item
   CP    HTML_P_EODB			;Did we click on forms item?
   JP    NZ,HTML_ActionFormsItem	;Yes (handle the type)
;
   CALL  HTML_FindAppIcon		;Search DB of App icons
   CP    255				;Did we click on an App icon?
   JP    NZ,App_StartApp3		;Yes (execute the app)
   RET
;
CursorSelect2
;
;  slider icon has been selected
;
   LD    A,(CursorKey_Up)
   LD    (ScrollKey_Up),A
   LD    A,(CursorKey_Down)
   LD    (ScrollKey_Down),A
   LD    A,(CursorKey_Left)
   LD    (ScrollKey_Left),A
   LD    A,(CursorKey_Right)
   LD    (ScrollKey_Right),A
;
   LD    DE,(Window_Roff)
   LD    (Cursor_OffRow),DE
   LD    DE,(Window_Coff)
   LD    (Cursor_OffCol),DE		;Setup frame offsets to adjust
CursorSelect2a
   CALL  KB_TestScroll
   JR    Z,CursorSelect2b		;cursor select has been released
   LD    BC,KB_WAITTIME
   CALL  KB_WaitRelease1
   CALL  KB_DoHandler			;execute key handler (if any)
   CALL  MNU_ShowOffsetXY		;display new offsets
   JR    CursorSelect2a
;
CursorSelect2b
   CALL  WIN_CalcFrmeDB
   EX    DE,HL
   LD    HL,S_HTMLFramRo
   ADD   HL,DE
   LD    BC,(Cursor_OffRow)
   LD    (HL),C
   INC   HL
   LD    (HL),B
   INC   HL
   LD    BC,(Cursor_OffCol)
   LD    (HL),C
   INC   HL
   LD    (HL),B
   CALL  MEM_SwapInMain
   JP    WIN_RefreshFrame
;=========================================================================
;Name     Scroll_Left
;Purpose  To handle the key from cursor scroll movement.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  None.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars 
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
Scroll_Left
   LD    A,(FrameItemSel)
   BIT   5,A			;Scroll horizontal icon?
   RET   Z
   LD    BC,(Cursor_OffCol)
   LD    A,B
   OR    C
   RET   Z
   DEC   BC
   LD    (Cursor_OffCol),BC
   RET
;=========================================================================
;Name     Scroll_Right
;Purpose  To handle the key from cursor scroll movement.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  None.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars 
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. No check made if the scrolling is beyond the max width of
;            content displayed.
;=========================================================================
Scroll_Right
   LD    A,(FrameItemSel)
   BIT   5,A			;Scroll horizontal icon?
   RET   Z
   LD    BC,(Cursor_OffCol)
   INC   BC
   LD    (Cursor_OffCol),BC
   RET
;=========================================================================
;Name     Scroll_Up
;Purpose  To handle the key from cursor scroll movement.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  None.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars 
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
Scroll_Up
   LD    A,(FrameItemSel)
   BIT   4,A			;Scroll vertical icon?
   RET   Z
   LD    BC,(Cursor_OffRow)
   LD    A,B
   OR    C
   RET   Z
   DEC   BC
   LD    (Cursor_OffRow),BC
   RET
;=========================================================================
;Name     Scroll_Down
;Purpose  To handle the key from cursor scroll movement.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  None.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars 
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. No check made if the scrolling is beyond the max width of
;            content displayed.
;=========================================================================
Scroll_Down
   LD    A,(FrameItemSel)
   BIT   4,A			;Scroll vertical icon?
   RET   Z
   LD    BC,(Cursor_OffRow)
   INC   BC
   LD    (Cursor_OffRow),BC
   RET
;=========================================================================
;Name     CursorESC
;Purpose  To handle the escape key from cursor movement.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  None.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars 
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
CursorESC
   RET
