;=========================================================================
;
; Application Routines (Links Handler/Display routines)
; Copyright (c) 2010 Ray Palmer
;
;=========================================================================
;Name     LNK_CalcLMDBAddr
;Purpose  To calculate the address of a link DB entry
;In Regs  AF = DB index
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         BC = ??
;         DE = ??
;         HL = address of DB entry.
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. The Links DB memory banks is left swapped in.
;=========================================================================
LNK_CalcLMDBAddr
   LD    HL,LM_Database
   OR    A
   JR    Z,LNK_CalcLMDBAddr2
;
   LD    BC,S_LinksTableSz
LNK_CalcLMDBAddr1
   ADD   HL,BC
   DEC   A
   JR    NZ,LNK_CalcLMDBAddr1
;
LNK_CalcLMDBAddr2
   JP    MEM_SwapInLink
;=========================================================================
;Name     LNK_AddFiller
;Purpose  To add filler data with spaces and NULL
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
LNK_AddFiller
   LD    A,(SCrn_MaxCol)
   LD    B,A
;=========================================================================
;Name     LNK_AddFiller2
;Purpose  To add filler data with spaces and NULL
;In Regs  A = ??
;         B = number of spaces to insert
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
LNK_AddFiller2
   LD    A,SPACE_CHAR
LNK_AddFiller1
   LD    (DE),A
   INC   DE
   DJNZ  LNK_AddFiller1
   XOR   A
   LD    (DE),A			;Ensure Text is terminated
   RET
;=========================================================================
;Name     MNU_ShowDBLinks
;Purpose  To display the links of the user saved links
;In Regs  A = Start index into the DB
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  LM_NumWIdx =number of items displayable
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_SrchTxt = corrupted.
;         LM_RowCol = corrupted.
;         LM_NumWIdx = updated (if 0 then all displayed else parial)
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
MNU_ShowDBLinks
   LD    B,A
   LD    A,MAX_DBLINKS
   LD    (LM_NumWIdx),A			;setup num entries in window
   LD    A,B
   LD    DE,DBLINK_POS			;column 4, row 5
   CALL  MNU_AdjustLocn
MNU_ShowDBLinks0
   LD    (LM_RowCol),DE
   LD    (LM_DBIndex),A			;Save start of list to show
   CALL  LNK_CalcLMDBAddr
   LD    A,(HL)
   OR    A				;end of DB?
   RET   Z				;Yes
   LD    DE,KB_SrchTxt
   LD    B,S_LnkTblNameSz+1
   CALL  UTL_strncpy			;Copy State + Name
   CALL  LNK_AddFiller2
;
   LD    A,(KB_SrchTxt)			;Get State
   LD    (LM_LastIndex),A
   OR    A				;Is entry the last one?
   RET   Z				;Yes
;
   BIT   1,A				;Entry valid?
   JR    Z,MNU_ShowDBLinks1		;No (skip it)
;
   LD    B,SPACE_CHAR
   LD    A,(KB_SrchTxt)
   BIT   7,A
   JR    Z,MNU_ShowDBLinks2
   LD    B,'t'
MNU_ShowDBLinks2
;
   LD    HL,(LM_RowCol)			;E=Row,D=Column
   LD    A,B
   LD    BC,0
   LD    E,255
   LD    D,1
   CALL  FNT_WR_CHAR			;Display Tag status
   LD    DE,(LM_RowCol)			;E=Row,D=Column
   INC   D
   INC   D
   LD    HL,KB_SrchTxt+1
   CALL  WIN_ShowWindwData		;show link name
;
   LD    A,(LM_NumWIdx)
   DEC   A				;End bottom row of window?
   RET   Z				;Yes
   LD    (LM_NumWIdx),A
   LD    DE,(LM_RowCol)			;E=Row,D=Column
   INC   E
MNU_ShowDBLinks1
   LD    A,(LM_DBIndex)			;Next DB index
   INC   A
   JR    MNU_ShowDBLinks0
;=========================================================================
;Name     MNU_ShowDBLInfo
;Purpose  To display the information for the current DB link in the Links
;         manager window.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  LM_Index = current links DB index
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_Buffer = corrupted.
;Uses     TBD
;Call By  TBD
;Calls    LNK_CalcLMDBAddr
;Notes    TBD
;=========================================================================
MNU_ShowDBLInfo
   LD    A,(LM_Index)
   CALL  LNK_CalcLMDBAddr	;Calc link address and bank swapped in.
   INC   HL			;Skip status/flags byte
   PUSH  HL
;
   LD    DE,KB_Buffer
   LD    B,S_LnkTblNameSz
   CALL  UTL_strncpy		;copy name of URL link
   CALL  LNK_AddFiller2
;
   LD    DE,DBLINK2_POS		;column 6, row 3
   CALL  MNU_AdjustLocn
   PUSH  DE
   LD    HL,KB_Buffer
   CALL  WIN_ShowWindwData	;display URL name
;
   POP   DE
   POP   HL
   PUSH  DE
   LD    BC,S_LnkTblNameSz
   ADD   HL,BC			;Skip state/Name
   LD    DE,KB_Buffer
   LD    A,(Windw_NumCols)
   DEC   A
   LD    B,A
   CALL  UTL_strncpy		;Get name
   CALL  LNK_AddFiller2
;
   POP   DE
   INC   E
   LD    HL,KB_Buffer
   CALL  WIN_ShowWindwData   	;display URL
   RET
;=========================================================================
;Name     MNU_LinkOpen
;Purpose  To open and manage the user saved links. Upon opening the manager
;         the current URL is displayed with no name making it ready for
;         the user enter the name and then be able to add it to the list
;         of existing links for later reference.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
MNU_LinkOpen
   XOR   A			;Start at first DB entry
   LD    (LM_Index),A
   LD    (LM_TopIndex),A
;
   LD    HL,SubMenu_Link_Open
   CALL  MNU_ShowWindowItems	;display blank form
;
   LD    A,(Stream_Submenu)
   CALL  SC_SEL_STREAM		;Select Window stream
;
   LD    A,(Window_FSCol)
   LD    C,A			;end column
   LD    A,(Window_FSRow)
   LD    B,A			;end row
MNU_LinkOpen1
   CALL  MEM_SwapInLink		;Swap in Link Bank
;
   LD    A,(LM_Index)
   CALL  MNU_ShowDBLInfo	;display first link info
;
   LD    A,(LM_TopIndex)
   CALL  MNU_ShowDBLinks	;show remaining links
;
   CALL  Mem_SwapInMain
   LD    HL,(MenuWinKeyLoc)
   CALL  KB_Wait4akey		;execute any key pressed
   CALL  KB_WaitRelease
   LD    A,(KB_KeyTested)
   CP    KEY_ESC		;cancel menu options key?
   JP    Z,WIN_HandleESC	;Yes
   CP    KEY_O			;Open a link?
   RET   Z			;Yes
   JR    MNU_LinkOpen1
;=========================================================================
;Name     MNU_LinkLoad
;Purpose  To load the DB of links from other sessions.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_buffer = corrupted.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
MNU_LinkLoad
   LD    DE,Filename_Lnks
   CALL  FIO_ReqFName
   PUSH  HL
   LD    A,(LinksBank)
   CALL  Mem_ClearBank
   POP   HL
   XOR   A
   LD    (LM_LastDB),A
   JP    PAR_CfgSys			;parse loaded script
;=========================================================================
;Name     MNU_LnkOpnInsert
;Purpose  To insert a new URL into the DB of known links
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  URL_StartText = has latest link to save.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_buffer = corrupted.
;Uses     TBD
;Call By  TBD
;Calls    LNK_CalcLMDBAddr
;Notes    TBD
;=========================================================================
MNU_LnkOpnInsert
   LD    HL,URLNameReqTxt
   LD    BC,REQUEST_TYP3
   CALL  UTL_ReqInput			;Request user input
   RET   Z				;No (user input)
   CALL  MEM_SwapInLink
   LD    A,(LM_LastDB)
   CALL  LNK_CalcLMDBAddr
   LD    (HL),2				;flag entry taken
   EX    DE,HL				;DE = next free entry
   INC   DE				;Point to Link Name
   PUSH  DE
   LD    HL,KB_Buffer
   LD    B,S_LnkTblNameSz
   CALL  UTL_strncpy			;Insert Link's Name
   POP   HL
   LD    BC,S_LnkTblNameSz
   ADD   HL,DE				;Point to URL field
   EX    DE,HL
   LD    HL,URL_ObjStrtTxt
   LD    BC,URL_Size
   LDIR					;Insert URL details
   CALL  Mem_SwapInMain
   LD    A,(LM_LastDB)
   INC   A				;New last DB entry
   LD    (LM_LastDB),A
   RET
;=========================================================================
;Name     MNU_LnkOpnRemove
;Purpose  To remove a new URL from the DB of known links
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    LNK_CalcLMDBAddr
;Notes    1. There has to be at most one link tagged to remove any.
;         2. The routines starts from the last DB entry and works its
;            back to the first.
;=========================================================================
MNU_LnkOpnRemove
   CALL  MEM_SwapInLink
   LD    A,(LM_LastDB)
   DEC   A
MNU_LnkOpnRemove1
   PUSH  AF
   CALL  LNK_CalcLMDBAddr
   POP   AF
   DEC   A
   CP    255			;before start start of DB?
   JP    Z,Mem_SwapInMain	;Yes
   BIT   7,(HL)
   JR    Z,MNU_LnkOpnRemove1
   LD    (HL),1
   JR    MNU_LnkOpnRemove1
;=========================================================================
;Name     MNU_LnkOpnEditName
;Purpose  To edit the name of a URL from the DB of known links
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  LM_Index = current DB entry display with name and URL
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_Buffer = corrupted.
;Uses     TBD
;Call By  TBD
;Calls    LNK_CalcLMDBAddr
;Notes    TBD
;=========================================================================
MNU_LnkOpnEditName
   LD    HL,URLNameReqTxt
   LD    BC,REQUEST_TYP4
   CALL  UTL_ReqInput			;Request user input.
   RET   Z				;No (user input)
   CALL  Mem_SwapInLink
   LD    A,(LM_Index)
   CALL  LNK_CalcLMDBAddr
   LD    BC,S_LnkTblName
   ADD   HL,BC
   EX    DE,HL
   LD    HL,KB_Buffer
   LD    B,S_LnkTblNameSz
   CALL  UTL_Strncpy			;Update name of link
   JP    Mem_SwapInMain
;=========================================================================
;Name     MNU_LnkOpnEditURL
;Purpose  To edit the URL text from the DB of known links
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  LM_Index = current DB entry display with name and URL
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_Buffer = corrupted.
;Uses     TBD
;Call By  TBD
;Calls    LNK_CalcLMDBAddr
;Notes    TBD
;=========================================================================
MNU_LnkOpnEditURL
   LD    A,(LM_Index)
   CALL  LNK_CalcLMDBAddr
   LD    BC,S_LnkTblURL
   ADD   HL,BC
   LD    DE,KB_Buffer
   LD    B,URL_Size
   CALL  UTL_strncpy
   XOR   A
   LD    (DE),A				;Ensure null terminated
   LD    HL,PageOpenReqTxt
   LD    BC,REQUEST_TYP3
   CALL  UTL_ReqInput			;Request user input.
   JP    Z,Mem_SwapInMain		;No (user input)
   LD    A,(LM_Index)
   CALL  LNK_CalcLMDBAddr
   LD    BC,S_LnkTblURL
   ADD   HL,BC
   EX    DE,HL
   LD    HL,KB_Buffer
   LD    B,URL_Size
   CALL  UTL_Strncpy			;Update links with new URL
   JP    Mem_SwapInMain
;=========================================================================
;Name     MNU_LnkOpnEdit
;Purpose  To tag a URL in the DB of known links to enable multiple
;         deletes.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    LNK_CalcLMDBAddr
;Notes    TBD
;=========================================================================
MNU_LnkOpnTag
   LD    A,(LM_Index)
   CALL  LNK_CalcLMDBAddr
   SET   7,(HL)
   JP    Mem_SwapInMain
;=========================================================================
;Name     MNU_LnkOpnOpen
;Purpose  To open the currently highlighted link.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_Buffer = URL to open.
;Uses     TBD
;Call By  TBD
;Calls    LNK_CalcLMDBAddr
;Notes    TBD
;=========================================================================
MNU_LnkOpnOpen
   LD    A,(LM_Index)
   CALL  LNK_CalcLMDBAddr
   LD    BC,S_LnkTblURL
   ADD   HL,BC
   LD    DE,KB_Buffer
   LD    B,URL_Size
   CALL  UTL_Strncpy
   CALL  Mem_SwapInMain
   LD    HL,KB_Buffer
   JP    URL_OpenMgrLink
;=========================================================================
;Name     MNU_LnkOpnPrev
;Purpose  To cursor down to the previous link in the Links Manager window.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    LNK_CalcLMDBAddr
;Notes    TBD
;=========================================================================
MNU_LnkOpnPrev
   XOR   A
   CALL  LNK_CalcLMDBAddr
   LD    A,(HL)
   OR    A				;DB empty?
   JR    Z,MNU_LnkOpnPrev2		;yes
MNU_LnkOpnPrev0
   LD    A,(LM_Index)
   DEC   A				;Dec index
   CP    255				;Index below 0?
   JR    Z,MNU_LnkOpnPrev1		;Yes
   LD    (LM_Index),A
   CALL  LNK_CalcLMDBAddr
   BIT   1,(HL)				;Entry valid?
   JR    Z,MNU_LnkOpnPrev0		;No (try previous)
MNU_LnkOpnPrev1
   LD    A,(LM_TopIndex)		;Get current top view DB index
   OR    A				;Is Top at DB start?
   JR    Z,MNU_LnkOpnPrev2		;Yes
   DEC   A
   LD    (LM_TopIndex),A
MNU_LnkOpnPrev2
   JP    Mem_SwapInMain
;=========================================================================
;Name     MNU_LnkOpnNext
;Purpose  To cursor down to the next link in the Links Manager window.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  LM_LastDB = current end of DB
;         LM_TopIndex = start of displayed list.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars LM_TopIndex = dpdated start of displayed list.
;         LM_Index = current index into DB.
;Uses     TBD
;Call By  TBD
;Calls    LNK_CalcLMDBAddr
;Notes    TBD
;=========================================================================
MNU_LnkOpnNext
   XOR   A
   CALL  LNK_CalcLMDBAddr
   LD    A,(HL)
   OR    A				;DB empty?
   JR    Z,MNU_LnkOpnNext1		;Yes
MNU_LnkOpnNext0
   LD    A,(LM_LastDB)
   LD    B,A
   LD    A,(LM_Index)
   CP    B				;At end of DB?
   JR    Z,MNU_LnkOpnNext1		;Yes
   INC   A				;next DB entry
   LD    E,A
   CALL  LNK_CalcLMDBAddr
   LD    A,(HL)
   OR    A				;End of DB?
   JR    Z,MNU_LnkOpnNext1		;Yes
   LD    A,E
   LD    (LM_Index),A			;Save it
   BIT   1,(HL)				;Entry valid?
   JR    Z,MNU_LnkOpnNext0		;No (try next)
;
   LD    A,(LM_TopIndex)
   ADD   A,6				;calc end might be
   LD    B,A
   LD    A,(LM_LastDB)
   CP    B				;Last 6 items in DB?
   JR    Z,MNU_LnkOpnNext1		;Yes
;
   LD    HL,LM_TopIndex
   INC   (HL)
MNU_LnkOpnNext1
   JP    Mem_SwapInMain
;