;=========================================================================
;
; Application Routines (Main)
; Copyright (c) 2010 Ray Palmer
;
;=========================================================================
;Name     Main_Initialise
;Purpose  To initialise the various items of the browser application.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  keybbuffer = address of TCP/IP keyboard buffer
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars URL_Current = 0
;         URL_LastItem = 0
;         Main Menu window Setup
;         Web Page window setup
;Uses     TBD
;Call By  Start_App
;Calls    TBD
;Notes    1. The app also sets up the required system items via a parser
;            and script.
;=========================================================================
Main_Initialise
;
   CALL  SYS_calcMaxAPIs
;
   LD    A,1
   LD    (BrowserState),A		;Setup the current browser state
   LD    (WindowFrmeNum),A		;Assume 1st frame clicked on
   LD    (Stream_WebPage),A
   INC   A
   LD    (Stream_SubMenu),A
   LD    (Scrn_Mode),A			;Mode 2 screen
   INC   A
   LD    (Stream_Frame),A
   LD    A,&80
   LD    (CPCB_Timer),A
;
   XOR   A
   LD    (Stream_MainMenu),A
   LD    (Cursor_State),A
   LD    (HTML_RenderState),A		;Clear render flags
   LD    (WindowFrmeStat),A		;clear all flags
   LD    (CurrLocDevice),A		;Flag no device
   LD    (URL_ObjStrtTxt),A		;Clear URL
   LD    (URL_CurrItem),A		;Start at First URL Db item.
   DEC   A
   LD    (URL_LastItem),A
   LD    HL,SysBnkBuffer
   LD    (BaseHTMLLoc),HL
   LD    HL,KB_CURSWAIT
   LD    (KB_KeyDelayT),HL		;initial key delay time.
;
   CALL  PAR_ConfigSys			;Execute startup script
;
   LD    HL,DefFontName
   CALL  FNT_SelectFontByName
;
   CALL  WIN_SetupScreen
   CALL  KB_ResetCursor			;Setup cursor vars in browser vars
   JP    WIN_Initialise			;initialise the browser windows
;=========================================================================
;Name     Start_App
;Purpose  To handle the start of the CPC WEB Browser
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  keybbuffer = contains the command entered.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TCP/IP console command
;         TCPIP.APP RSX command
;Calls    API_Initialisation
;         Main_Initialise
;         KB_ShowCursor
;         VHS_BE_GetVarHL
;         UTL_skiptospace
;         UTL_skipspace
;         URL_GetStartUpURL
;         HTML_Rom_Init
;         URL_OpenNewURL
;         HTML_ShowFile
;         KB_FlashCursor
;         KB_Test4akey
;         KB_WaitRelease
;         KB_DoHandler
;         MNU_ShowPosXY
;         KM_INITIALISE (OS)
;         TXT_INITIALISE (OS)
;         SC_SET_MODE (OS)
;Notes    1. On exit the System is reset which may mean that the TCP/IP
;            stack may need to reset its settings for keyboard handling.
;=========================================================================
Start_App
   CALL  API_Initialisation		;Initialise the TCP/IP API
   LD    A,1				;Flag TCP/IP API okay
   JR    NZ,Start_App_0a
   XOR   A				;TCP/IP API failed
Start_App_0a
   LD    (TCPIP_State),A
;
   CALL  Main_Initialise		;Initialise the browser
;
   LD    IY,(REGIYTCPIP)
   ld    HL,keybbuffer
   CALL  VIY_BE_GetVarHL		;HL = address of keyboard input
   CALL  UTL_skiptospace		;Skip command to 1st space
   CALL  UTL_skipspace			;Skip space to 1st parameter
   LD    A,(HL)
   OR    A				;Any parameters?
   CALL  Z,URL_GetStartUpURL		;No (get start up URL)
   LD    (URL_NewLocn),HL		;save URL text address
;
   LD    HL,BrowserState
StartApp_0
   RES   7,(HL)				;clear new URL flag
   LD    HL,(URL_NewLocn)
   LD    A,(HL)				;Get first char in url
   OR    A				;URL defined?
   CALL  NZ,URL_OpenNewURL		;Yes, Load the HTML object
;
StartApp_1
   LD    HL,HTML_DispStatus
   BIT   0,(HL)				;complete screen refresh?
   JR    Z,StartApp_3			;No
;
   RES   0,(HL)
   LD    A,(URL_CurrItem)
   CALL  URL_GetDbEntry			;Get History Db entry
   LD    E,1
   CALL  SCR_fntInit			;Reset Screen handler routines
   CALL  WIN_SetupScreen		;setup screen mode/colours
   CALL  MNU_MainMenuFrame		;re-draw main menu/border
   CALL  WIN_MainPage
   LD    HL,HTML_RenderState
   SET   4,(HL)				;Flag re-display contents
;
StartApp_3
;
   LD    A,(HTML_RenderState)
   BIT   4,A				;re-display page requested?
   CALL  NZ,HTML_ShowFile		;Yes (Call rom routine)
;
   CALL  KB_ShowCursor			;show/update cursor
;
   LD    HL,MainMenuKeys
   CALL  KB_Test4akey			;check which key was pressed
   CP    KEY_ENDLIST			;Any key pressed ?
   JR    Z,StartApp_1			;No (try again)
;
   CALL  KB_WaitRelease			;Wait for a key to be released.
   CALL  KB_DoHandler			;execute key handler
   CALL  MNU_ShowPosXY			;Display Cursor Row/Column
;
   LD    HL,BrowserState
   BIT   7,(HL)				;New URL to access?
   JR    NZ,StartApp_0			;Yes
;
   LD    A,(HL)
   OR    A				;Exiting the Browser?
   JR    NZ,StartApp_1			;No
;
;  Wipe out start of code to allow basic to return (if called from there)
;
   LD    HL,BASIC_START
   LD    B,255
StartApp_2
   LD    (HL),0
   INC   HL
   DJNZ  StartApp_2
;
   CALL  KM_INITIALISE			;Reset key manager
   CALL  TXT_INITIALISE			;Reset Text screen manager
;
   LD    A,(SysExit)
   OR    A				;Normal Exit?
   JP    NZ,0				;No, Reboot machine
;
   LD    A,2
   JP    SC_SET_MODE			;Clear entire screen
