;=========================================================================
;
; Application Routines (Script commands)
; Copyright (c) 2010 Ray Palmer
;
;=========================================================================
;Name     SCR_Get1stParamAd
;Purpose  To address of first char parameter
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  KB_Buffer = parser string
;Out Regs A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = address of 1st parameter
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
SCR_Get1stParamAd
   LD    HL,KB_Buffer
   CALL  UTL_skiptospace		;skip command text
   JP    UTL_skipspace			;skip spaces to parameter
;=========================================================================
;Name     SCR_Get1stParam
;Purpose  To first char parameter and point to next parameter
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  KB_Buffer = parser string
;Out Regs A = first char parameter in lowercase
;         B = corrupted
;         C = ??
;         DE = ??
;         HL = address of 2nd parameter
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
SCR_Get1stParam
   CALL  SCR_Get1stParamAd
   LD    B,(HL)				;get parameter char
   INC   HL
   CALL  UTL_skipspace			;skip space to next parameter
   LD    A,B
   CALL  UTL_tolower
   RET
;=========================================================================
;Name     SCR_Objparm
;Purpose  To setup the 3 byte address
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars SysHandlerData = updated.
;Uses     TBD
;Call By  SCR_Object
;Calls    SCR_Ivar1
;Notes    TBD
;=========================================================================
SCR_Objparm
   LD    DE,SysHandlerData+1
   CALL  SCR_Ivar1		;get 3 byte address
   CALL  MEM_SwapInABank
   LD    DE,(SysHandlerData)
   RET
;=========================================================================
;Name     SCR_SkipParam
;Purpose  To go to the next parameter
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = address of parameters text
;         IX = ??
;         IY = ??
;In Vars  None
;Out Regs A = corrupted
;         BC = ??
;         DE = ??
;         HL = address of next parameter
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. Assumenext parameter exists.
;=========================================================================
SCR_SkipParam
   LD    A,(HL)
   INC   HL
   CP    ','
   JR    NZ,SCR_SkipParam
   RET
;=========================================================================
;Name     SCR_IVar
;Purpose  To initialise an internal variable.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  KB_Buffer = parser string
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. command parameters
;            a. 'f' - set frames base location
;            b. 'm' - Max frames
;            c. 'b' - Base HTML location
;            d. 't' - Table location (formats)
;=========================================================================
SCR_IVar
   CALL  SCR_Get1stParam	;A = 1st param, HL = addr of 2nd param
   LD    DE,FramesDBLoc
   CP    'f'			;Define frames configuration bank/address?
   JR    Z,SCR_IVar1		;Yes
   LD    DE,FramesDBMax
   CP    'm'			;max frames?
   JR    Z,SCR_IVar2		;Yes
   LD    DE,BaseHTMLLoc
   CP    'b'			;Base HTML Location?
   JR    Z,SCR_IVar1		;Yes
   LD    DE,HTML_TblFmtDat
   CP    't'			;Define Table formats location?
   JR    Z,SCR_IVar1		;Yes
   LD    DE,HTML_TblColWid
   CP    'c'			;Define Table col span location?
   JR    Z,SCR_IVar1		;Yes
   LD    DE,HTML_TblRowWid
   CP    'r'			;Define Table row span location?
   JR    Z,SCR_IVar1		;Yes
   LD    DE,HTML_TblCRows
   CP    'x'			;Define where table cell rows?
   JR    Z,SCR_IVar1		;Yes
   LD    DE,HTML_TblNRSWid
   CP    'n'			;Define Table num rows location?
   JR    Z,SCR_IVar1		;Yes
   LD    DE,LogBank
   CP    'l'			;Define log disk catalog bank
   JR    Z,SCR_IVar1		;Yes
   LD    DE,SysBorderSet
   CP    'i'			;border Icon set?
   JR    Z,SCR_IVar2		;Yes
   LD    DE,KB_KeyDelayT
   CP    'w'			;Key delay Timer set?
   JR    Z,SCR_IVar3		;Yes
   LD    DE,Scrn_Mode
   CP    's'			;Screen Mode?
   JR    Z,SCR_IVar2		;Yes
   LD    DE,pen0
   LD    B,5
   CP    'p'			;define pens 0,1,2,3 and border
   JR    Z,SCR_IVar4		;Yes
   LD    B,6
   LD    DE,Menu_ColIdx
   CP    'k'			;Colour settings?
   JR    Z,SCR_IVar5		;Yes
   RET
SCR_IVar1
;
;  Save the 3 byte address
;
   CALL  SCR_IVar3
   EX    DE,HL
SCR_IVar2
;
;  Save the byte value
;
   CALL  UTL_readhex8
   LD    (DE),A
   RET
SCR_IVar3
;
;  Save the 16-bit value
;
   PUSH  DE
   EX    DE,HL
   CALL  UTL_readhex16		;HL = value, DE = address of non-hex char
   INC   DE
   LD    C,L
   LD    B,H
   POP   HL
   LD    (HL),C
   INC   HL
   LD    (HL),B
   INC   HL
   RET
;
;  Setup pens and border (both mode 1 and 2)
;
SCR_IVar4
   CALL  UTL_readdec
   LD    (DE),A
   INC   DE
   DJNZ  SCR_IVar4
   RET
;
;  Setup colour indices
;
SCR_IVar5
   CALL  UTL_readhex8
   LD    (DE),A
   INC   DE
   DJNZ  SCR_IVar5
   RET
;=========================================================================
;Name     SCR_Bank
;Purpose  To define the memory configuration banks
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  KB_Buffer = parser string
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
SCR_Bank
   CALL  SCR_Get1stParam	;A = 1st param, HL = addr of 2nd param
   LD    DE,ROM_ShowHTMLFile+3
   CP    'r'			;Define HTML rom bank?
   JR    Z,SCR_Bank1		;Yes
   LD    DE,MaxCodeBanks
   CP    's'			;Define System banks?
   JR    Z,SCR_Bank1		;Yes
   CP    'i'
   RET   NZ
   CALL  UTL_readhex8
   JP    MEM_ClearBank		;wipeout previous data
;
SCR_Bank1
   CALL  UTL_readhex8
   LD    (DE),A
   INC   DE
   LD    A,(HL)			;Get char after hex digits
   OR    A			;Is it a EOL?
   JR    NZ,SCR_Bank1		;No, Get next bank number
;
   LD    HL,LM_LastDB
   LD    (HL),0			;Flag end of HTML banks
   RET
;=========================================================================
;Name     SCR_URL
;Purpose  To assign a URL for the home page and the start up page.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  KB_Buffer = parser string
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
SCR_URL
   CALL  SCR_Get1stParam	;A = 1st param, HL = addr of 2nd param
   LD    DE,CFG_HomePage
   CP    'h'			;Setup HOME page URL?
   JP    Z,SCR_URL1		;Yes
   LD    DE,CFG_StartUp
   CP    's'			;Setup Start Up page?
   JP    Z,SCR_URL1		;Yes
   LD    DE,CFG_BaseURL
   CP    'b'			;Setup Base URL?
   RET   NZ
SCR_URL1
   CALL  Mem_SwapInCfg
   LD    B,URL_Size
   CALL  UTL_strncpy
   JP    Mem_SwapInMain
;=========================================================================
;Name     SCR_link
;Purpose  To add a user defined link to the list of selectable links.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  KB_Buffer = parser string
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    LNK_CalcLMDBAddr
;Notes    TBD
;=========================================================================
SCR_link
   CALL  SCR_Get1stParamAd	;point to name of link
   PUSH  HL
   LD    A,(LM_LastDB)
   INC   A
   LD    (LM_LastDB),A
   DEC   A
   CALL  LNK_CalcLMDBAddr
   EX    DE,HL			;DE = next free entry
   LD    A,&02
   LD    (DE),A			;flag entry taken
   INC   DE
   POP   HL			;Get entry name address back
   PUSH  DE			;Save link name address
;
;  Save name of link
;
SCR_link2
   LD    A,(HL)
   INC   HL
   CP    ','			;end of link name?
   JR    Z,SCR_link2a		;Yes
   LD    (DE),A
   INC   DE
   JR    SCR_link2
SCR_link2a
;
   POP   DE
   EX    DE,HL
   LD    BC,15			;goto end of link name in the DB entry
   ADD   HL,BC
   EX    DE,HL
;
;  Insert URL text
;
   LD    B,URL_Size
   CALL  UTL_strncpy
   LD    A,(LM_LastDB)
   CALL  LNK_CalcLMDBAddr
   LD    (HL),0
   RET
;=========================================================================
;Name     SCR_font
;Purpose  To add a font to the system.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  KB_Buffer = parser string
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_printlen = corrupted via overlay of Font_Temp.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
SCR_font
   CALL  SCR_Get1stParamAd	;point to name of font
   LD    A,(HL)
   CP    '#'			;Initialise font handler?
   JR    NZ,SCR_fnt1		;No
;
   CALL  SCR_SkipParam
   CALL  UTL_readdec
   AND   &0F
   LD    C,A			;Setup border set
   CALL  UTL_readdec
   AND   &0F
   LD    B,A			;setup frame set
   LD    E,0			;normal initialisation
SCR_fntInit
   LD    A,(Frme_ColIdx)
   LD    H,A
   LD    A,(Brdr_ColIdx)
   LD    L,A
   LD    A,(Scrn_Mode)
   JP    FNT_InitHandler
SCR_fnt1
   LD    B,255			;data is icon set
   CP    '*'			;Is font HTML Icons
   JR    Z,SCR_fnt0		;Yes, insert icons data
   CALL  UTL_tolower
   LD    B,0			;data is font set
   CP    'f'			;Is font fixed space?
   JR    Z,SCR_fnt0		;Yes
   LD    B,1			;Flag proportional font
SCR_fnt0
   LD    A,B
   LD    (FontType),A
   CALL  SCR_SkipParam		;goto font name
   LD    (Font_Name),HL
   CALL  SCR_SkipParam		;goto font's filename
   LD    DE,URL_ObjStrtTxt
   CALL  UTL_strcpy		;setup object URL
   LD    A,2
   LD    (FIO_CodeBlk),A
   LD    DE,AMSDOS_BUFFER
   LD    (FileLoadAddr),DE
   CALL  URL_OpenObject		;load font data from the file
   LD    A,(FileOpState)
   CP    FERR_OKAY		;did we load font data ok?
   JP    NZ,SCR_FileError	;No
;
   LD    HL,AMSDOS_BUFFER
   LD    DE,(Font_Name)
   LD    A,(FontType)
   CP    255			;uploading a font data set
   JR    Z,SCR_SaveIcons	;Yes
;
   CALL  FNT_UploadFont		;Upload font data
   JP    Mem_SwapInMain
;
SCR_SaveIcons
   CALL  FNT_UploadIcon		;upload icon data
   JP    Mem_SwapInMain
;=========================================================================
;Name     SCR_fileloc
;Purpose  To define the location for files access by the application.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  KB_Buffer = parser string
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
SCR_fileloc
   CALL  SCR_Get1stParam	;A = 1st param, HL = addr of 2nd param
   LD    DE,CFG_AppLocn
   CP    'a'			;Applications location?
   JR    Z,SCR_fileloc1		;Yes
   LD    DE,CFG_SysLocn
   CP    's'			;System location?
   JR    Z,SCR_fileloc1		;Yes
   LD    DE,CFG_SaveLocn
   CP    'd'			;Setup download location?
   JR    Z,SCR_fileloc1		;Yes
   LD    DE,CFG_CacheLocn
   CP    'c'			;Setup cache location?
   RET   NZ
SCR_fileloc1
   CALL  MEM_SwapInCFG
   LD    B,URL_Size
   CALL  UTL_strncpy
   XOR   A
   LD    (DE),A
   JP    MEM_SwapInMain
;=========================================================================
;Name     SCR_App
;Purpose  To add the application details for the web browser to use.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  KB_Buffer = parser string
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars App_State = corrupted
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. Command format
;
;            app ab,App_Name,App_FileName
;
;            where a ='a' for application or
;                     'h' for handler
;                     'l' for loading app library file.
;                     'x' for executing an application directly.
;                  b = 's' passes script to app input or
;                      'k' passed keyboard input to app
;
;=========================================================================
SCR_App
   CALL  SCR_Get1stParamAd	;point to type of Application
   LD    A,(HL)			;setup default APP state
   CALL  UTL_tolower
   INC   HL			;skip type/function
;
   CP    'l'			;Load Application library file?
   JR    Z,SCR_App5		;Yes
;
   CP    'x'			;Execute an Application ?
   JR    Z,SCR_App4		;Yes
;
   LD    C,&42			;Bits 6 (enabled) and 1 set (taken)
   CP    'a'			;Application ?
   JR    Z,SCR_App0		;Yes
   SET   5,C			;Flag app entry is a handler
SCR_App0
   CP    'r'			;rom handler
   JR    NZ,SCR_App1		;No
   SET   4,C			;Flag ROM is the handler
SCR_App1
   LD    A,(HL)
   INC   HL
   CP    ','			;at delimiter?
   JR    Z,SCR_App3		;Yes get App Name
   INC   HL
   CP    's'			;Script passed to App?
   JR    NZ,SCR_App3		;No
   SET   0,C			;Flag app passes script text
SCR_App3
   CALL  App_Struct		;Clear structure.
   LD    A,C
   LD    (App_State),A		;Setup state of Application
   PUSH  HL
   CALL  App_NewAppName		;setup general name of application
   POP   HL
   CALL  SCR_SkipParam		;Skip name to filename of application
   CALL  App_NewAppFilename	;setup app filename
   JP    App_AddNewItem		;add app item to database
;
SCR_App4
   CALL  SCR_SkipParam
   EX    DE,HL
   JP    App_StartCfgApp	;start the application
;
SCR_App5
   CALL  UTL_readhex8
   LD    (FIO_CodeBlk),A	;setup which bank to load to
   EX    DE,HL
   INC   DE
   CALL  UTL_readhex16		;Get load location (HL)
   LD    (FileLoadAddr),HL
   EX    HL,DE
   INC   HL			;Point to filename
   LD    DE,URL_ObjStrtTxt
   CALL  UTL_strcpy
   CALL  URL_OpenObject		;load binary file
   JP    Mem_SwapInMain
;=========================================================================
;Name     SCR_LoadFile
;Purpose  To load a binary file.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  KB_Buffer = parser string
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
SCR_LoadFile
   CALL  SCR_Get1stParam	;A = 1st param, HL = addr of 2nd param
   sub   '0'			;Get code bank number
   AND   1			;ensure bank index is either 0 or 1
   CALL  MEM_SwapBCBank
   EX    DE,HL			;Save address of load loaction
   INC   DE
   CALL  UTL_readhex16		;Get load location (HL)
   LD    (FileLoadAddr),HL
   EX    HL,DE
   INC   HL			;Point to filename
   CALL  FIO_loadBinFile	;load binary file
   JP    Mem_SwapInMain
;=========================================================================
;Name     SCR_subscr
;Purpose  To load a script after the main script.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  KB_Buffer = parser string
;         PAR_CurrLocn = current next script line.
;         PAR_ScriptEnd = location for sub-scripts
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars PAR_RetLine = current script line to return to after subscript.
;         URL_ObjStrtTxt = URL
;         URL_ObjName = name of file loaded.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
SCR_subscr
   CALL  SCR_Get1stParamAd	;HL = address of filename of subscript
;
   LD    DE,(PAR_ScriptEnd)
   LD    (PAR_ScriptLoad),DE	;save load address
   LD    DE,(PAR_CurrLocn)
   LD    (PAR_RetLine),DE	;Setup return address
;
   LD    DE,URL_ObjStrtTxt
   CALL  UTL_strcpy
   CALL  URL_OpenObject
;
   LD    HL,(PAR_ScriptEnd)
   LD    (PAR_CurrLocn),HL	;Setup sub-script start
   RET
;=========================================================================
;Name     SCR_Cursor
;Purpose  To assign the keyboard keys to the cursor positioning keys.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  KB_Buffer = parser string
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
SCR_Cursor
   CALL  Mem_SwapInCFG
   CALL  SCR_Get1stParam	;A = 1st param, HL = addr of 2nd param
   LD    DE,CFG_CurKey_Up
   CP    'u'			;cursor Up key
   JR    Z,SCR_Cursor1
   LD    DE,CFG_CurKey_Down
   CP    'd'			;cursor down key
   JR    Z,SCR_Cursor1
   LD    DE,CFG_CurKey_Right
   CP    'r'			;cursor right key
   JR    Z,SCR_Cursor1
   LD    DE,CFG_CurKey_Left
   CP    'l'			;cursor left key
   JR    Z,SCR_Cursor1
   LD    DE,CFG_CurKey_Select
   CP    's'			;cursor select key
   JR    Z,SCR_Cursor1
   LD    DE,CFG_Cursor_Icon
   CP    'i'			;cursor icon
   JR    Z,SCR_Cursor2
   LD    DE,Cursor_Speed
   CP    't'			;cursor icon
   RET   NZ
   CALL  UTL_readdec		;convert key number
   CALL  UTL_MulABy10
   LD    (Cursor_Speed),HL	;setup cursor speed
   JP    Mem_SwapInMain  
SCR_Cursor1
   CALL  UTL_readdec		;convert key number
   LD    (DE),A			;setup cursor key
   JP    Mem_SwapInMain  
SCR_Cursor2
   CALL  UTL_readdec		;convert key number
   LD    (DE),A			;setup cursor key
   INC   HL
   LD    DE,Cursor_IcnSet
   JR    SCR_Cursor1
;=========================================================================
;Name     SCR_LogCtrl
;Purpose  To define the logging control flags.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  KB_Buffer = parser string
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars SysLogging = bit 7 = machine type (1=6128, 0=464)
;                      bit 0 = logging control (0=None, 1=yes)
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
SCR_LogCtrl
   CALL  SCR_Get1stParam	;A = 1st param, HL = addr of 2nd param
   CALL  UTL_readdec
   LD    (SysLogging),A
   RET
;=========================================================================
;Name     SCR_quit
;Purpose  To quit the parsing of a script.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  KB_Buffer = parser string
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars PAR_RetLine = 0
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
SCR_quit
   LD    HL,0
   LD    (PAR_RetLine),HL
   LD    HL,(PAR_CurrLocn)
   LD    (HL),0
   RET
;=========================================================================
;Name     SCR_msg
;Purpose  To display a message.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  KB_Buffer = parser string
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars PAR_RetLine = 0
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
SCR_msg
   CALL  SCR_Get1stParamAd		;Goto start of print text
   CALL  printstring
   JP    printcrlf
;=========================================================================
;Name     SCR_echo
;Purpose  To toggle the echo control
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  parse_echo = current echo control
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars parse_echo = current echo control toggled
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
SCR_echo
   LD    A,(PAR_echo)
   XOR   1			;Toggle echo output.
   LD    (PAR_echo),A
   RET
;=========================================================================
;=========================================================================
;Name     SCR_GetScriptLine
;Purpose  To get the current script line to process.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = Current script command address
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = Address of next command
;         IX = ??
;         IY = ??
;Out Vars KB_Buffer = script command to handle
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. Assume the KB_Buffer is in the lower RAM.
;=========================================================================
SCR_GetScriptLine
   LD    DE,KB_Buffer
   LD    B,KB_BuffSz-1
SCR_GetScriptLine0
   LD    A,(HL)
   INC   HL
   LD    (DE),A
   OR    A			;end of current line?
   JR    Z,SCR_GetScriptLine1	;Yes (fill rest of line with NUL)
   INC   DE
   DJNZ  SCR_GetScriptLine0
   LD    A,B
   INC   B
   DEC   DE
SCR_GetScriptLine1
   LD    (DE),A
   INC   DE
   DJNZ  SCR_GetScriptLine1
   RET
;=========================================================================
;Name     SCR_ChkCmd
;Purpose  To lookup the entered command for a valid one to execute.
;Input    DE = address of internal command to check.
;         HL = address of entered command.
;Output   DE = address of next command if not found
;         C = Status fo check
;             Bit 7 = 1 -> valid command
;                   = 0 -> invalid command
;         SysHandlerData = command handler data (if found)
;Uses     A corrupt
;Notes    None
;=========================================================================
SCR_ChkCmd
   LD    A,(DE)				;Get text of internal command
   LD    C,A
   LD    A,(HL)				;Get keyboard char
   BIT   7,C				;At command info byte?
   JR    NZ,SCR_endcmd			;Yes
   OR    A				;end of command?
   CALL  NZ,UTL_Tolower			;No (convert to lowercase)
   CP    C				;Matches requested char?
   JR    NZ,SCR_endcmd			;No
   INC   DE
   INC   HL
   JR    SCR_ChkCmd
SCR_endcmd
   OR    A				;end of command?
   JR    Z,SCR_ChkCmd_exit		;Yes (command match)
   CP    ' '				;Parameters after command?
   JR    Z,SCR_ChkCmd_exit		;Yes (command match)
SCR_badcmd
   LD    A,(DE)				;Goto end of command text
   INC   DE
   BIT   7,A				;end of internal command?
   JR    Z,SCR_badcmd			;no (keep going)
;
   INC   DE                         	;Skip handler address data
   INC   DE
   INC   DE				;skip handler memory type
   LD    C,0				;Flag command does not match
   RET
SCR_ChkCmd_exit
   LD    A,C
   EX    HL,DE
   LD    DE,SysHandlerData
   LD    BC,4
   LDIR					;upload handler data
   LD    C,A				;Flag command match
   RET
;=========================================================================
;Name     SCR_FindScriptCmd
;Purpose  To Find the script command in the list of known commands
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = Address of command text
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = result of find (0=not found, 1=Found)
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
SCR_FindScriptCmd
   LD    DE,ScriptCommands
SCR_FindHTMLCmd1
   PUSH  HL
   call  SCR_ChkCmd			;Check if command is valid
   POP   HL
   BIT   7,C				;Did we find a valid command?
   JR    NZ,SCR_FindHTMLCmd2		;Yes
   LD    A,(DE)
   OR    A				;End of list ?
   JR    NZ,SCR_FindHTMLCmd1		;No, check next comman
   RET
SCR_FindHTMLCmd2
   LD    A,Find_Okay
   RET
;=========================================================================
;Name     SCR_DoHandler
;Purpose  To execute the parser routine
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_Buffer = has command being executed.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
SCR_DoHandler
   PUSH  HL
   RET
;=========================================================================
;Name     SCR_FileError
;Purpose  To display a file access error.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = Address of error message
;         IX = ??
;         IY = ??
;In Vars  None.
;Out Regs A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
SCR_FileError
   LD    HL,GenErrMsgText
   JR    SCR_EchoText
;=========================================================================
;Name     SCR_EchoBack
;Purpose  To echo back to the screen using OS output routines.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  KB_Buffer = contains command to echo back.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars None.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
SCR_EchoBack
   LD    HL,KB_Buffer
SCR_EchoText
   CALL  printstring
   CALL  printcrlf
   RET
;=========================================================================
;Name     PAR_ParseScript
;Purpose  To parse the loaded script.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  PAR_CurrLocn = location of script to parse.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_Buffer = corrupted.
;         PAR_RetLine = corrupted if sub-scripts are executed.
;         PAR_CurrLocn = corrupted.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. Only 1 level routine calling via sub-script allowed.
;=========================================================================
PAR_ParseScript
   XOR   A
   LD    (PAR_echo),A			;allow echo of messages
   LD    (FindResult),A
   LD    HL,0
   LD    (PAR_RetLine),HL		;flag no sub-scripts called
PAR_ParseScript1
   CALL  MEM_SwapInMain			;only work in def mem cfg
   LD    HL,(PAR_CurrLocn)
   LD    A,(HL)
   OR    A				;End of script text?
   JR    Z,PAR_ParseScript2		;Yes (check if need to return)
;
   CALL  SCR_GetScriptLine		;KB_Buffer = next script command
   LD    (PAR_CurrLocn),HL		;Save addres of next command
;
   LD    A,(PAR_echo)
   BIT   0,A				;echo enabled (for debug)?
   CALL  NZ,SCR_EchoBack		;Yes
;
   LD    HL,KB_Buffer
   CALL  SCR_FindScriptCmd		;Look for command
   BIT   7,C				;Did we find it?
   JR    Z,PAR_ParseScriptErr		;No (abort processing)
;
   LD    HL,(SysHandlerData+1)
   CALL  SCR_DoHandler			;Execute script command
   JR    PAR_ParseScript1
;
PAR_ParseScript2
   LD    HL,(PAR_RetLine)		;Get return line
   LD    A,H
   OR    L				;Subscript executed?
   RET   Z				;No (exit parser completely)
   LD    (PAR_CurrLocn),HL		;setup where to continue from
   LD    HL,0
   LD    (PAR_RetLine),HL		;ensure no subscript is rerun
   JR    PAR_ParseScript1		;continue with main script
;
PAR_ParseScriptErr
   LD    A,1
   LD    (FindResult),A
   RET
;=========================================================================
;Name     PAR_ConfigSys
;Purpose  To configure the system via a script
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  PAR_CurrLocn = location of script to parse.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_Buffer = corrupted.
;         PAR_RetLine = corrupted if sub-scripts are executed.
;         PAR_CurrLocn = corrupted.
;         App_ID = reset to zero
;         LastAppIndex = reset to zero
;         NumFonts = reset to zero
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
PAR_ConfigSys
   LD    A,2
   CALL  SC_SET_MODE
;
   CALL  printstringimm
   DEFB  "Loading CPC browser startup script",13,10,0
;
   CALL  MEM_SwapInMain		;only work in def mem cfg
;
   LD    A,(CurrMemBank)
   PUSH  AF
   XOR   A
   LD    (App_ID),A
   LD    (LastAppIndex),A
   LD    (NumFonts),A
   LD    A,FILE_SCRPT
   LD    (FIO_Ctrl),A
;
   LD    HL,Sys_scriptname	;Address of startup script filename
   LD    DE,SysGenBuffer	;where to load it.
   LD    (PAR_CurrLocn),DE
   LD    (PAR_ScriptLoad),DE
   LD    DE,URL_ObjName
   CALL  UTL_strcpy
   CALL  FIO_LoadObject		;Load system initialise script
   JR    PAR_CfgSys1
;
PAR_CfgSys
   LD    A,(CurrMemBank)
   PUSH  AF
   CALL  MEM_SwapInMain		;only work in def mem cfg
   LD    (PAR_CurrLocn),DE
   LD    (PAR_ScriptLoad),DE
   LD    DE,URL_ObjStrtTxt
   CALL  UTL_strcpy
   CALL  URL_OpenObject		;load script (assume loads okay)
;
PAR_CfgSys1
   XOR   A
   LD    (PAR_Echo),A		;No echo of commands (default)
   LD    (PAR_ScriptEnd),HL	;Save start location for sub-scripts.
   CALL  PAR_ParseScript	;parse loaded script
   POP   AF
   JP    MEM_SwapInABank
