;=========================================================================
;
; Application Routines (UTL)
; Copyright (c) 2010 Ray Palmer
;
;=====================================================================
;Name     SYS_SaveRegs
;Purpose  To save the current registers values.
;Input    all registers
;Output   TBD
;Uses     TBD
;Notes    TBD
;=====================================================================
SYS_SaveRegs
   LD    (SYS_RegIY2),IY
   LD    (SYS_RegIX2),IX
   LD    (SYS_RegHL2),HL
   LD    (SYS_RegDE2),DE
   LD    (SYS_RegBC2),BC
   PUSH  AF
   POP   BC
   LD    (SYS_RegAF2),BC
   RET
;=====================================================================
;Name     SYS_calcMaxAPIs
;Purpose  To get the number of entries in the API jump vector
;Input    None
;Output   ??
;Uses     ??
;Notes    TBD
;=====================================================================
SYS_calcMaxAPIs
   LD    HL,SysAcc_VectorE-SysAcc_Vector
   SRA   L
   LD    A,L
   LD    (MaxAPIs),A
   RET
;=====================================================================
;Name     SYS_RunCode1
;Purpose  To execute the handler within Code bank 0
;Input    HL = address of banked routine to execute
;Output   ??
;Uses     ??
;Notes    1. Original memory bank configuration returned.
;=====================================================================
SYS_RunCode1
   XOR   A
   JR    SYS_RunCode3a
;=====================================================================
;Name     SYS_RunCode2
;Purpose  To execute the handler within Code bank 1
;Input    HL = address of banked routine to execute
;Output   ??
;Uses     ??
;Notes    1. Original memory bank configuration returned.
;=====================================================================
SYS_RunCode2
   LD    A,1
   JP    SYS_RunCode3a		;Call ext routine 1st/2nd code bank
;=====================================================================
;Name     SYS_RunCode3
;Purpose  To execute the handler within Code bank 2
;Input    HL = address of banked routine to execute
;Output   ??
;Uses     ??
;Notes    1. Original memory bank configuration returned.
;=====================================================================
SYS_RunCode3
   LD    A,2
;=====================================================================
;Name     SYS_RunCode3a
;Purpose  To execute the handler within Code bank 2
;Input    A = code bank
;         HL = address of banked routine to execute
;Output   ??
;Uses     ??
;Notes    1. Original memory bank configuration returned.
;=====================================================================
SYS_RunCode3a
   LD    B,A
   LD    A,(CodeBanks)
   LD    C,A
   CALL  MEM_CalcCfgData
   LD    A,C
;=====================================================================
;Name     SYS_RunCode4
;Purpose  To execute the handler within other banks of memory
;Input    A = bank id information.
;         HL = address with the banked memory.
;Output   ??
;Uses     ??
;Notes    1. Main memory may not be switch back in.
;=====================================================================
SYS_RunCode4
   LD    (SysHandlerData+1),HL	;Setup handler address
   LD    (SysHandlerData+3),A	;Setup RAM Bank number
   JR    Sys_DoHndler2
;=====================================================================
;Name     SYS_DoHandler
;Purpose  To execute the handler via a 3 byte address
;Input    HL = address of 4 byte handler data
;Output   SysHandlerData = has command handler data
;Uses     TBD
;Notes    1. Main memory may not be switch back in.
;=====================================================================
SYS_DoHandler
   LD    DE,SysHandlerData
   LD    BC,4
   LDIR				;Upload handler data
;
   LD    HL,(SysHandlerData+1)	;get handler address
;
Sys_DoHndler2
   LD    A,H			;Handler assigned?
   OR    L
   RET   Z			;No (do nothing)
;
   LD    A,H
   AND   &C0			;ROM address ?
   CP    &C0
   JR    Z,Sys_DoHndlerROM	;Yes
;
   LD    A,(SysHandlerData+3)	;get memory type
   OR    A			;default memory config?
   JR    NZ,Sys_DoHndler3	;No
   LD    A,&C0			;Setup default memory config
Sys_DoHndler3
;*********************************************************************
;*********************************************************************
;
; This part of the routine is for RAM memory access.
;
;*********************************************************************
;*********************************************************************
   CALL  MEM_PushBank		;Save current bank
   CALL  Sys_StartCode		;execute code
   JP    MEM_PopBank		;Restore bank
;*********************************************************************
;*********************************************************************
;
; This part of the routine is for ROM memory access.
;
;*********************************************************************
;*********************************************************************
Sys_DoHndlerROM
   CALL  Sys_SetupRegs		;Restore registers
   RST   3			;Call ROM routine
   DEFW  SysHandlerData+1	;address of RST 3 data
   RET
;=====================================================================
;Name     Sys_ExtCode
;Purpose  To execute a routine in another memory bank from the current
;         memory bank.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars BankStack = updated
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. The calling code sequence is -
;
;            CALL CPCB_ExtAccess	;Inter-Bank call
;            DEFW AddressToExecute
;            DEFB Index_to_CodeBanks
;
;=====================================================================
Sys_ExtCode
   CALL  SYS_SaveRegs
;
   POP   HL
   LD    E,(HL)			;Address of routine to call
   INC   HL
   LD    D,(HL)
   INC   HL
   LD    B,(HL)			;Bank offset
   INC   HL
   PUSH  HL			;Save return address
;
   LD    HL,MEM_PopBank
   PUSH  HL			;Setup auto bank swap on return
   LD    A,(CodeBanks)		;Location of code banks vector.
   LD    C,A
   CALL  MEM_CalcCfgData
   CALL  MEM_PushBank		;Save current memory bank on stack
   JR    SYS_SysFnctAcc0
;=========================================================================
;Name     SYS_SysFnctAccess	
;Purpose  To enable an external function to execute a routine within the
;         CPCB executble.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TOS = byte index of routine to execute.
;         SysAcc_Vector = vector of avilable routines.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  External routines to access local routines.
;Calls    As per index.
;Notes    1. Registers are saved external to call of routine
;         2. Call convention is
;
;            CALL CPCB_CPCAccess	;API call
;            DEFB API_Index
;
;=========================================================================
SYS_SysFnctAccess
   CALL  SYS_SaveRegs
   LD    A,(MaxAPIs)
   LD    E,A
   POP   HL
   LD    A,(HL)			;Get address index
   INC   HL
   PUSH  HL
   CP    E			;Is index greater then allowed?
   RET   NC			;Yes (do nothing)
;
   LD    DE,SysAcc_Vector	;Address of jump entries
   LD    L,A
   LD    H,0
   ADD   HL,HL			;double index to address
   ADD   HL,DE
   LD    E,(HL)			;get address of routine
   INC   HL
   LD    D,(HL)
   LD    A,D			;Available to run?
   OR    E
   RET   Z			;No
;
SYS_SysFnctAcc0
   EX    DE,HL
;=====================================================================
;Name     Sys_StartCode
;Purpose  To restore the registers and to execute the routine
;Input    HL = address of routine
;Output   As per routine.
;Uses     As per routine.
;Notes    1. As per routine exeuted.
;         2. Memory bank configured as required.
;=====================================================================
Sys_StartCode
   PUSH  HL
;=====================================================================
;Name     Sys_SetupRegs
;Purpose  To restore the registers for the routine
;Input    ??
;Output   As per routine.
;Uses     As per routine.
;Notes    1. Only handles standard RAM banks
;=====================================================================
Sys_SetupRegs
;
;  Setup Registers for routine being called.
;
   DEFB  &FD,&21		;LD IY,xxxx
SYS_RegIY2
   DEFW  0
;
   DEFB  &DD,&21		;LD IX,xxxx
SYS_RegIX2
   DEFW  0
;
   DEFB  &21			;LD HL,xxxx
SYS_RegHL2
   DEFW  0
;
   DEFB  &11			;LD DE,xxxx
SYS_RegDE2
   DEFW  0
;
   DEFB  &01			;LD BC,xxxx
SYS_RegAF2
   DEFW  0
   PUSH  BC
   POP   AF
;
   DEFB  &01			;LD BC,xxxx
SYS_RegBC2
   DEFW  0
;
   RET				;execute routine.
