;=========================================================================
;
; Application Routines (URL)
; Copyright (c) 2010 Ray Palmer
;
;=========================================================================
;Name     	
;Purpose  To handler the open url error.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
URL_ErrManager
   LD    A,(FileOpState)		;Get file status
   CP    FERR_OKAY			;Was access okay?
   RET   Z				;Yes
   LD    HL,OpenURLError
   CALL  Err_ShowErrorMsg		;display error
;=========================================================================
;Name     URL_ErrHandler
;Purpose  To handler the open url error.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
URL_ErrHandler
   LD    A,FERR_OKAY
   LD    (FileOpState),A		;clear error
   JP    Err_ClrErrTxt
;=========================================================================
;Name     URL_NxtFileName
;Purpose  To setup the next filename of a downloaded network item.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  FileNumbr = current file number for downloads.
;         KB_Buffer = current path ending with slash.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_Buffer = contains filename of cached item if download ok.
;                     filename is File-xxx.$$$ where xxx = file number
;         FileNumbr = updated.
;Uses     TBD
;Call By  TBD
;Calls    UTL_writedec
;         UTL_strncpy
;         UTL_strcpy
;Notes    1. Files from a network connection are fixed with the local name
;            'File-xxx.$$$', where xxx is a decimal number.
;=========================================================================
URL_NxtFileName
   LD    HL,KB_Buffer
   CALL  UTL_streol		;goto end of URL
   EX    DE,HL
;
   LD    HL,DefNetFilename
   CALL  UTL_strcpy		;insert 'File-' prefix
;
   LD    A,(FileNumbr)
   INC   A
   LD    (FileNumbr),A
   DEC   A
   EX    DE,HL
   CALL  UTL_writedec		;insert filenumber
   LD    (HL),'.'
   INC   HL
   EX    DE,HL
   LD    HL,FExtTypes
   LD    B,3
   CALL  UTL_strncpy		;insert '$$$' extension
   XOR   A
   LD    (DE),A
   RET
;=========================================================================
;Name     URL_CalcURLAddr
;Purpose  To calculate the address of the nth history item.
;In Regs  A = Index into the URL history table
;         BC = ??
;         DE = ??
;         HL = address of url text
;         IX = address of application variables
;         IY = ??
;In Vars  N/A
;Out Regs AF = corrupted
;         BC = corrupted
;         DE = ??
;         HL = address of nth history item
;         IX = ??
;         IY = ??
;Out Vars URL_History = first entry has URL
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
URL_CalcURLAddr
   AND   &7				;Ensure we dont excced table
   LD    BC,S_LinksTableSz
   LD    HL,CFG_HistURLs
   OR    A
   JP    Z,MEM_SwapInCfg
URL_CalcURLAddr1
   ADD   HL,BC
   DEC   A
   JR    NZ,URL_CalcURLAddr1
   JP    MEM_SwapInCfg
;=========================================================================
;Name     URL_GetDbEntry
;Purpose  To get an entry from the URL history database
;In Regs  A = Database index
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars URL_Object = updated with database entry
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
URL_GetDbEntry
   CALL  URL_CalcURLAddr		;HL = address in history table
   LD    DE,URL_Object
   LD    BC,S_LinksTableSz
   LDIR					;Download object
   RET
;=========================================================================
;Name     URL_SetDbEntry
;Purpose  To set an entry in the URL history database with values from the
;         local copy.
;In Regs  A = Database index
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
URL_SetDbEntry
   CALL  URL_CalcURLAddr		;HL = address in history table
   LD    DE,URL_Object
   LD    BC,S_LinksTableSz
   EX    DE,HL
   LDIR					;Upload object
   RET
;=========================================================================
;Name     URL_SaveObjFname
;Purpose  To save the name of the loaded object.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = address of filename to save into the URL object.
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = Address of object status/flags
;         IX = ??
;         IY = ??
;Out Vars URL_Object updated.
;Uses     TBD
;Call By  URL_OpenObject
;Calls    UTL_strncpy
;Notes    None
;=========================================================================
URL_SaveObjFname
   LD    DE,URL_ObjName
   LD    B,MAXFILENAME
   CALL  UTL_strncpy			;setup URL object filename to open
   LD    HL,URL_ObjState
   SET   5,(HL)				;Flag filename is now known
   RET
;=========================================================================
;Name     URL_AddNewItem
;Purpose  To add a new URL to the history table of URLs.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = address of URL text.
;         IX = ??
;         IY = ??
;In Vars  URL_LastItem = current last item in history table.
;         URL_AddrLoc = address of URL text
;Out Regs AF = ??
;         B = ??
;         C = state of URL
;             0 = URL add to table.
;            >0 = URL cannot be added (table is full)
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars URL_CurrItem = updated with new current History URL index
;         URL_LastItem = updated new last history URL item.
;         URL_HistFlag = status of History DB
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. Need to fix to move table down for newer URLs when history
;            is full.
;         2. The maximum length of an URL is 128 bytes.
;=========================================================================
URL_AddNewItem
   LD    A,(URL_CurrItem)		;Get current available table entry
   CP    URL_MaxItems			;At limit of table?
   JR    Z,URL_AddNewItemEx		;Yes
;
   LD    A,(URL_LastItem)
   CP    255				;First time entry?
   JR    Z,URL_AddNewItem1		;Yes
   LD    A,(URL_CurrItem)
URL_AddNewItem1
   INC   A
   LD    (URL_LastItem),A
   LD    (URL_CurrItem),A
   CALL  URL_SetDbEntry			;Save URL to database.
   XOR   A
   LD    (DE),A				;Flag end of database (bit0 = 0)
;
   CALL  Mem_SwapInMain
   XOR   A				;Flag URL added ok
URL_AddNewItemEx
   LD    (URL_HistFlag),A		;Save status of history update
   RET
;=========================================================================
;Name     URL_ResetHTMLLoc
;Purpose  To rest the HTML load location for a new HTML file.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    1. Assume the frames DB is < 16K.
;         2. The previous HTML data is removed.
;=========================================================================
URL_ResetHTMLLoc
   LD    A,1
   LD    (WindowFrameID),A
   CALL  WIN_CalcFrmeDB
   LD    (HL),&11		;Reset frames flags/settings.
   LD    DE,Window_FrameV
   LD    BC,S_HTMLFramSz
   LDIR				;Get default frame data
;
;  Clear follwing frames DB of data and first portion of HTML.
;
URL_RstHTMLLoc1
   LD    (HL),0
   INC   HL
   BIT   7,H 			;At end of 16K block
   JR    Z,URL_RstHTMLLoc1	;No
;
   LD    HL,BaseHTMLLoc
   LD    DE,URL_EndAddr
   LD    BC,3
   LDIR				;reset HTML load location
;
   CALL  MEM_SwapInCfg
   LD    HL,CFG_FormsDB
   LD    (HL),0			;reset forms DB
   RET
;=========================================================================
;Name     URL_ShowURL
;Purpose  To display the selected URL on the main menu window (line 2).
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = address of URL text
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    WIN_ShowText2
;Notes    1. The main menu window is stream#0, defined to be columns 1 to
;            80 lines 1 to 4.
;=========================================================================
URL_ShowURL
   EX    DE,HL
   LD    HL,URLT_POS
   LD    B,MaxURLText
   JP    WIN_ShowText2
;=========================================================================
;Name     URL_Lasthash
;Purpose  To goto the last "#" char in the URL text for a label.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = address within URL text.
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = 0 if no # in URL or has '#' if found.
;         BC = ??
;         DE = address of char after last # in URL or unchanged if
;              # not found
;         HL = address of null after URL text.
;         IX = ??
;         IY = ??
;Out Vars None
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    1. Where a URL has more than 1 # in the URL the others hashes
;            may result in unknown behaviour when label search is
;            performed following the object loading.
;=========================================================================
URL_Lasthash
   LD    B,121			;Max URL text allowed.
URL_Lsthash1
   LD    A,(HL)
   OR    A			;end of current URL?
   RET   Z			;Yes
   DEC   B
   INC   HL
   CP    HASH_CHAR		;Slash in URL?
   JR    NZ,URL_Lsthash1	;No, do next char in URL
   LD    D,H			;track where slash char is
   LD    E,L
   JR    URL_Lsthash1
;=========================================================================
;Name     URL_BuildURL
;Purpose  To Build a new URL.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = address of URL text from the keyboard or from click on the
;              html file or from the menu request to open a URL
;         IX = ??
;         IY = ??
;In Vars  URL_Object = contains current URL to build upon.
;                      URL_ObjStrtTxt = complete URL
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars URL_AddrLoc = location of added URL in history table.
;         URL_Object  = updated the URL
;                       URL_ObjStrtTxt = text of URL
;Uses     TBD
;Call By  TBD
;Calls    SYS_SaveRegs
;         URL_SetBaseURL
;         SYS_RunCode1
;         Mem_SwapInMain
;         Ext_BuildURL (via SYS_RunCode1 routine)
;Notes    1. URLs can be one of 3 forms -
;            a. relative reference from the base location.
;               e.g i.   "path1/file1.txt"
;                   ii.  "../file1.txt"
;                   iii. "../path2/file2.txt"
;                   iv.  "./path2/file2.txt"
;               The handling of the path information in the URL is
;               treated in the same manner as file manager handling of
;               directories.
;
;            b. absolute reference from base location.
;               e.g "/path1/index.htm"
;               if current path is http //www.text.com/path0/f.txt
;               then the new URL will be
;               http //www.text.com/path1/file.txt
;
;            c. A new base location to access.
;               e.g "http //www.fred.com/file1.txt"
;
;         2. Assume the complete abs reference is used to open a URL
;            that is not a reference to the current url.
;            i.e must have http //..... or file //......
;
;         3. Need to fix to handle references like 'www.fred.com/f.htm'
;            and '1.2.3.4/f.htm' to be internet rather than just files as
;            the first char is not a slash.
;
;         4. The URL cannot have a # in the path as it would be handled
;            as a label start (e.g www.myisp.com/#path/file.htm) and so
;            the URL is truncated at that point. Only the last # is of
;            concern rather than any others existing before it.
;
;         5. The use of the https prefix for secure communication has not
;            yet been implemented.
;
;=========================================================================
URL_BuildURL
   LD    DE,URL_ObjStrtTxt	;Where to save built URL
   CALL  SYS_SaveRegs		;Save all reg for external routine
   LD    HL,Ext_BuildURL
   JP    SYS_RunCode1		;call routine
;=========================================================================
;Name     URL_SetBaseURL
;Purpose  To Setup the Base URL
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  ConfigBank = configuration bank RAM number
;         URL_ObjStrtTxt = new base URL
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars CFG_BaseURL = updated with new base URL
;Uses     TBD
;Call By  TBD
;Calls    MEM_SwapInCfg
;         Mem_SwapInMain
;Notes    TBD
;=========================================================================
URL_SetBaseURL
   CALL  MEM_SwapInCfg
   LD    DE,CFG_BaseURL
   LD    HL,URL_ObjStrtTxt
   LD    BC,URL_Size
   LDIR
   JP    Mem_SwapInMain
;=========================================================================
;Name     URL_GetBaseURL
;Purpose  To get the current Base URL
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  ConfigBank = configuration bank RAM number
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars CFG_BaseURL = updated with new base URL
;         URL_ObjStrtTxt = new base URL
;Uses     TBD
;Call By  TBD
;Calls    MEM_SwapInCfg
;         Mem_SwapInMain
;Notes    TBD
;=========================================================================
URL_GetBaseURL
   CALL  MEM_SwapInCfg
   LD    HL,CFG_BaseURL
   LD    DE,URL_ObjStrtTxt
   LD    BC,URL_Size
   LDIR
   JP    Mem_SwapInMain
;=========================================================================
;Name     URL_CpyTheURL
;Purpose  To copy a configuration URL to a temp location.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = Address of URL
;         IX = ??
;         IY = ??
;In Vars  N/A
;Out Regs AF = ??
;         BC = Corrupted
;         DE = Corrupted
;         HL = address of downloaded URL
;         IX = ??
;         IY = ??
;Out Vars SysGenBuffer = contains the copied URL text.
;         URL_ObjState = state/flags cleared
;Uses     TBD
;Call By  TBD
;Calls    UTL_strncpy
;         Mem_SwapInMain
;Notes    1. The space for which the URL is copied cannot be in the normal
;            memory bank swap area (i.e &4000 to &7fff).
;=========================================================================
URL_CpyTheURL
   LD    DE,SysGenBuffer
   PUSH  DE
   LD    B,URL_Size
   CALL  UTL_strncpy		;Download URL Text to temp space
   LD    HL,URL_ObjState
   LD    (HL),0			;Clear URL object state flags.
   POP   HL
   JP    Mem_SwapInMain		;Restore main memory configuration.
;=========================================================================
;Name     URL_GetStartUpURL
;Purpose  To get the Start up Page URL.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  None.
;Out Regs AF = ??
;         BC = ??
;         DE = address of the startup page URL
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    MEM_SwapInCfg
;         URL_CpyTheURL
;Notes    TBD
;=========================================================================
URL_GetStartUpURL
   CALL  MEM_SwapInCfg
   LD    HL,CFG_StartUp
   JP    URL_CpyTheURL
;=========================================================================
;Name     URL_OpenHomePage
;Purpose  To open the Home Page URL.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars URL_Object = URL object to open
;Uses     TBD
;Call By  TBD
;Calls    MEM_SwapInCfg
;         URL_CpyTheURL
;         URL_AccessURL
;Notes    1. Assume the Home page URL is an absolute reference.
;=========================================================================
URL_OpenHomePage
   CALL  MEM_SwapInCfg
   LD    HL,CFG_HomePage
   CALL  URL_CpyTheURL
   LD    A,(URL_ObjStrtTxt)
   OR    A			;Is URL empty?
   RET   Z			;Yes (Do nothing)
   CALL  URL_ResetHTMLLoc
   JP    URL_AccessURL		;Open the URL
;=========================================================================
;Name     URL_OpenPrevURL
;Purpose  To select the previous history URL item.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  URL_CurrItem = current index into the history of URLs
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars URL_Object = contains link state/name
;         SysGenBuffer = contains link structure
;         URL_CurrItem = updated history index.
;Uses     TBD
;Call By  TBD
;Calls    URL_OpenHistURL
;Notes    TBD
;=========================================================================
URL_OpenPrevURL
   LD    A,(URL_CurrItem)
   OR    A			;At start of History table?
   RET   Z			;Yes
   DEC   A
   JR    URL_OpenHistURL
;=========================================================================
;Name     URL_OpenNextURL
;Purpose  To select the next history URL item.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  URL_LastItem = current end of the history
;         URL_CurrItem = current index into history
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars URL_Object = contains link state/name
;         SysGenBuffer = contains link structure
;         URL_CurrItem = updated history index.
;Uses     TBD
;Call By  TBD
;Calls    MEM_SwapInCfg
;         URL_GetDbEntry
;         WIN_ClearWebPage
;         URL_AccessURL
;Notes    1. The history URL object has the cache name and URL.
;=========================================================================
URL_OpenNextURL
   LD    A,(URL_LastItem)
   CP    255			;history table empty?
   RET   Z			;Yes
   LD    B,A
   LD    A,(URL_CurrItem)
   CP    B			;At last entry?
   RET   Z			;Yes
   INC   A
URL_OpenHistURL
   LD    (URL_CurrItem),A
   CALL  URL_GetDbEntry		;Get History Db entry
   CALL  WIN_ClearWebPage	;Clear web-page of previous data
   CALL  URL_ResetHTMLLoc
   CALL  HTM_ImgCalcDBRst
   JR    URL_AccessURL		;Open the URL
;=========================================================================
;Name     URL_OpenMgrLink
;Purpose  To open a saved link from a previous session via the links
;         manager.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  LM_Index = index of selected URL to open from Manager
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = address of URL
;         IX = ??
;         IY = ??
;Out Vars URL_Object = contains link state/name
;         SysGenBuffer = contains link structure
;Uses     TBD
;Call By  TBD
;Calls    UTL_GetSavedLink
;         URL_AccessURL
;Notes    TBD
;=========================================================================
URL_OpenMgrLink
   LD    A,(LM_Index)
   LD    DE,URL_Object
   CALL  UTL_GetSavedLink	;download link DB object
   LD    HL,URL_ObjState
   LD    (HL),&80		;Dont update history DB
   JR    URL_AccessURL
;=========================================================================
;Name     URL_OpenAppURL
;Purpose  To open a new URL via an external application
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = address of URL text from the keyboard or from click on the
;              html file or from the menu request to open a URL
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars URL_ObjState = updated URL object.
;         URL_HistFlag = status of history update.
;                        0 = okay, 1 = table full.
;Uses     TBD
;Call By  TBD
;Calls    URL_BuildURL
;         URL_AccessURL
;Notes    1. Assume the complete abs reference is used to open an URL
;            that is not a reference to the current url.
;            i.e must have http //..... or file //......
;                (with a colon after http or file)
;=========================================================================
URL_OpenAppURL
   CALL  URL_BuildURL		;Build URL to open
   LD    A,(URL_ObjState)
   AND   URL_C_NOFILENAME	;No filename or cached file.
   LD    (URL_ObjState),A
   JR    URL_AccessURL
;=========================================================================
;Name     URL_OpenNewURL
;Purpose  To open a new URL. The routine is functionally called from the
;         user clicking on a web page hyper-link or from the menu options
;         to open a new link via user entry of the url text.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = address of URL text from the keyboard or from click on the
;              html file or from the menu request to open a URL
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars URL_ObjState = updated URL object.
;         URL_HistFlag = status of history update.
;                        0 = okay, 1 = table full.
;Uses     TBD
;Call By  TBD
;Calls    URL_BuildURL
;         URL_AddNewItem
;         URL_Lasthash
;         MNU_ClearTitle
;         URL_OpenObject
;         HTML_Rom_Init
;         MNU_PageRefresh
;         URL_ErrManager
;Notes    1. Assume the complete abs reference is used to open an URL
;            that is not a reference to the current url.
;            i.e must have http //..... or file //......
;                (with a colon after http or file)
;=========================================================================
URL_OpenNewURL
   CALL  URL_BuildURL		;Build URL to open
   LD    A,(URL_ObjState)
   AND   URL_C_NOFILENAME	;No filename or cached file.
   OR    URL_C_HISTITM		;Flag history item
   LD    (URL_ObjState),A
   CALL  URL_AddNewItem		;Add URL to history DB
   CALL  URL_ResetHTMLLoc
   CALL  HTM_ImgCalcDBRst
   XOR   A
   LD    (HTML_FrameFlg),A	;clear frames loaded status/flags
   LD    (HTML_FCurrPosF),A	;No search has been done
;=========================================================================
;Name     URL_AccessURL
;Purpose  To access the URL object upon the user clicking on a link or
;         from a URL previously known to the browser (such as a history
;         URL or saved link).
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  URL_Object = URL object to access
;         URL_HistFlag = status of history update.
;                        0 = okay, 1 = table full.
;         URL_EndAddr = setup with load address
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars URL_EndAddr = end address of loaded data
;         HTML_CurrLoc = copy of end address of loaded data for frames
;Uses     TBD
;Call By  URL_OpenHomePage - Open the Home Page URL
;         URL_OpenNewURL   - Open a New URL from a HTML hyper link
;         URL_OpenAppURL   - Open an Application from a URL
;         URL_OpenMgrLink  - Open a selected manager URL link
;         URL_OpenNextURL  - Open a history URL link (next)
;         URL_OpenPrevURL  - Open a history URL link (previous)
;Calls    URL_Lasthash
;         MNU_ClearTitle
;         URL_OpenObject
;         HTML_Rom_Init
;         MNU_PageRefresh
;         URL_ErrManager
;         URL_SetBaseURL
;Notes    TBD
;=========================================================================
URL_AccessURL
   CALL  MNU_ShowURL		;Show URL being opened.
;
   CALL  URL_AccessObject
;
   LD    A,(FileOpState)
   CP    FERR_OKAY		;load okay?
   JP    NZ,URL_ErrManager	;No (display error)
;
   LD    A,(URL_OpenType)	;Get Object type
   AND   &FC			;Mask out object type
   OR    A			;Is file type HTML/Text/Script
   RET   NZ			;No
;
   LD    A,(URL_CurrItem)
   CALL  URL_SetDbEntry		;Update history db entry
;
   LD    HL,URL_EndAddr
   LD    DE,HTML_CurrLoc
   LD    BC,3
   LDIR				;Save end of HTML for new objects
;
   CALL  URL_SetBaseURL		;Setup Base URL
;
   CALL  HTML_Rom_Init		;Initialise HTML Rom for rendering
;
   CALL  WIN_ShowMainBorder	;Display main menu border
;
URL_AcessURL2
   LD    HL,BaseCPCOffset
   LD    (PageWdw_SCol),HL	;Reset CPC screen offset
   LD    (PageWdw_SRow),HL
   JP    MNU_PageRefresh	;Flag to render new view of html
;=========================================================================
;Name     URL_AccessObject
;Purpose  To access the URL object.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  URL_Object = URL object to access
;         URL_EndAddr = setup with load address
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars URL_EndAddr = end address of loaded data
;         HTML_CurrLoc = copy of end address of loaded data for frames
;Uses     TBD
;Call By  TBD
;Calls    URL_Lasthash
;Notes    TBD
;=========================================================================
URL_AccessObject
;
;  check if URL is just using #label
;
   LD    A,255
   LD    (URL_LblType),A	;no URL label search
   LD    HL,URL_ObjStrtTxt
   LD    A,(HL)
   CP    HASH_CHAR		;URL starts with # for label?
   JR    Z,URL_AcessURL2	;Yes (find label within current html)
;
;  Check if URL has #label at end
;
   LD    DE,0
   CALL  URL_Lasthash		;Find Label text (if any)
   LD    (URL_LblAddr),DE	;save addr of label text (if any)
   LD    A,D			;Did we find a #label ?
   OR    E
   JR    Z,URL_AcessURL1	;No (continue to acess the URL)
   XOR   A
   DEC   DE
   LD    (DE),A			;Kill off Label in URL for load
   LD    (URL_LblType),A	;Flag search within html commands
;
URL_AcessURL1
   LD    HL,ErrMsg_Cntrl
   RES   7,(HL)			;Enable Error messages
;=========================================================================
;Name     URL_OpenObject
;Purpose  To open an URL Object.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  URL_Object = URL object to be opened
;                      URL_ObjStrtTxt = URL
;         URL_EndAddr = start address to load file data if html/text
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_Buffer = has filename to load (if http request or filename
;                     to open from cache/local stores)
;         FileOpState = current file operation state
;         URL_EndAddr = end address of loaded file data
;         URL_Object = updated status, filename and URL
;                      Status = Bit 7 - set if local file
;                                   5 - Filename is known
;                                   4 - set if file is cached web object
;                      URL_ObjName = name of file loaded
;         URL_ObjectSize = size of binary object loaded
;Uses     TBD
;Call By  URL_OpenNewObj
;         URL_AccessURL
;Calls    UTL_strncmp
;         FIO_OpenDevPath
;         UTL_strncpy
;         MEM_SwapInHTML
;         UTL_GetCfgLocn
;         URL_NxtFileName
;         URL_OpenURLLink
;         OBJ_LoadObject
;Notes    1. Where the object to be loaded is a binary image, there is no
;            end address as these are in a fixed location in the memory
;            bank.
;         2. Where the item to be loaded is a binary file the swap bank
;            will be swapped in.
;         3. The specification for a URL to open is -
;
;            {URL_TYPE}{device}{URL_PATH_AND_FILENAME}
;
;            where {URL_TYPE} = 'file //' or 'http //' (space is colon)
;
;                  {device} = \device_spec
;                             e.g 1. \a0
;                                     - Floppy Drive A, User 0
;
;                                 2. \0$0$Partition$Password$0000
;                                    - Device 0, Unit 0
;                                      ParitionName = "Partition"
;                                      Password = "Password"
;                                      Key = 0000 (in hex)
;
;                  {URL_PATH_AND_FILENAME} = PathName and filename
;                             e.g 1. /thisfile.htm
;                                 2. /path1/path2/thisfile.htm
;                                 3. ThisFile.txt
;
;            if the device is AMSDOS then pathname is ignored.   
;=========================================================================
URL_OpenObject
   XOR   A
   LD    (FileOpState),A	;clear file access status
   LD    HL,URL_ObjStrtTxt
   LD    DE,URL_fileTxt
   LD    B,7
   CALL  UTL_strncmp		;Local file request?
   JR    NZ,URL_OpnHTTP		;No (open network object)
;=========================================================================
;
;  Open a Local file
;
;=========================================================================
   LD    HL,URL_ObjState
   SET   7,(HL)			;Flag local file
   LD    HL,URL_ObjStrtTxt+7	;Start of URL path after 'file //'
   CALL  FIO_OpenDevPath	;Attempt to open the device/path
   LD    A,(FileOpState)
   OR    A			;Error accessing device/path
   JR    NZ,URL_OpnObjErr	;Yes
   DEC   HL			;PATCH CODE (NEED TO FIX ABOVE
   LD    A,(HL)			;FIO_OpenDevPath why out by 1 chr)
   CP    SLASH_CHAR
   JR    NZ,URL_OpenObja
   INC   HL
URL_OpenObja
   CALL  URL_SaveObjFname	;Setup URL object with filename
   CALL  FIO_IN_CLOSE		;close previous opened file.
   JP    FIO_LoadObject		;Load the local file
;=========================================================================
;
;  Open a file from a network connection.
;
;=========================================================================
URL_OpnHTTP
   LD    A,(TICK_CTRL)
   BIT   4,A			;Is TCP/IP up and configured?
   JR    Z,URL_OpnObjErr	;No, flag access error
;
   LD    A,(URL_EndAddr+2)
   CALL  MEM_SwapInHTML		;Swap in bank of HTML data
   LD    HL,(URL_EndAddr)
   LD    (HL),0			;kill of existing loaded data
;
   LD    HL,CFG_CacheLocn
   CALL  UTL_GetCfgLocn		;KB_Buffer = cache location
   LD    HL,KB_Buffer
   CALL  FIO_OpenDevPath	;Open cache path
   LD    A,(FileOpState)
   OR    A			;Error accessing cache location?
   JR    NZ,URL_OpnObjErr	;Yes
;
   LD    HL,URL_ObjState
   BIT   5,(HL)			;Filename already known?
   JR    NZ,URL_OpnObject1	;Yes (load prior cached item)
;
   CALL  URL_NxtFileName	;Add cache filename with .$$$ extn
   LD    HL,URL_ObjState
   SET   6,(HL)			;Flag HTTP request
   CALL  URL_OpenURLLink	;Download the URL object from network
   LD    HL,KB_Buffer
   LD    A,(HL)
   OR    A			;Was download okay?
   JR    NZ,URL_OpnHTTP2	;Yes
;
URL_OpnObjErr
   LD    A,FERR_ACCESS
   LD    (FileOpState),A	;Flag file access error
   RET
;
URL_OpnHTTP2
   CALL  URL_SaveObjFname	;Setup URL object with filename
   SET   4,(HL)			;Flag cached file
;
URL_OpnObject1
   JP    FIO_LoadObject		;Load object data into browser
;=========================================================================
;Name     URL_OpenNewObj
;Purpose  To open a new object that is part of an existing HTML.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = address of URL text.
;         IX = ??
;         IY = ??
;In Vars  Window_FrmeID = Frame ID number
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars URL_ObjState = updated URL object.
;         Window_FrmeID = Updated Frame ID number
;Uses     TBD
;Call By  TBD
;Calls    URL_BuildURL
;         URL_OpenObject
;         WIN_CalcFrmeDB
;Notes    1. This routine is primarily for frames to be loaded after the
;            main frame html has been loaded.
;=========================================================================
URL_OpenNewObj
   CALL  URL_BuildURL		;Build URL to open
   LD    HL,ErrMsg_Cntrl
   SET   7,(HL)			;Disable error messages
   CALL  URL_OpenObject		;Load object into memory
   LD    HL,ErrMsg_Cntrl
   RES   7,(HL)			;Enable error messages
   LD    A,(FileOpState)
   LD    A,(URL_CurrItem)
   JP    URL_GetDbEntry			;Get History Db entry
;=========================================================================
;Name     URL_LoadNewObj
;Purpose  To load an object into a bank of memory and return the memory
;         configuration to previous setting to ensure proper returns.
;In Regs  AF = Bank index for code banks to load object
;         BC = ??
;         DE = Load Address (0=use file header)
;         HL = address of URL text.
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FIO_CodeBlk = updated.
;         FileLoadAddr = updated.
;Uses     TBD
;Call By  TBD
;Calls    URL_OpenNewObj
;         MEM_SaveBank
;         MEM_PopBank
;Notes    TBD
;=========================================================================
URL_LoadNewObj
   CALL  MEM_SaveBank
   LD    (FIO_CodeBlk),A	;Bank to load binary to
   LD    (FileLoadAddr),DE	;Address to load to.
   CALL  URL_OpenNewObj
   LD    A,(URL_CurrItem)
   CALL  URL_GetDbEntry		;Get History Db entry
   JP    MEM_PopBank
;=========================================================================
;Name     URL_SaveHyperLink
;Purpose  To save the item which the hyperlink points to. This routine
;         is essentially the "Save As" one normally expects when using
;         currently browsers like IE, Opera, Firefox, etc. The "Save as"
;         results in a the cache file being renamed.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_Buffer = filename of object to save.
;Uses     TBD
;Call By  TBD
;Calls    UTL_GetCfgLocn
;         MNU_ReqInputTxt
;         SYS_SaveRegs
;         SYS_RunCode1
;         Mem_SwapInMain
;         Ext_OpenHTTP (via external routine in file1.bin)
;Notes    1. The saving of the hyperlinked item is downloaded first and
;            then renamed to what the user has entered.
;=========================================================================
URL_SaveHyperLink
   LD    HL,CFG_SaveLocn
   CALL  UTL_GetCfgLocn		;KB_Buffer = location text
   LD    HL,FilenameReqTxt
   LD    BC,REQUEST_TYP2	;update input, refresh URL
   CALL  MNU_ReqInputTxt	;request filename as well.
;=========================================================================
;Name     URL_OpenURLLink
;Purpose  To open the item which the hyperlink points to.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  KB_Buffer = filename of object
;         URL_Object = URL object
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    SYS_SaveRegs
;         SYS_RunCode1
;         Mem_SwapInMain
;         Ext_OpenHTTP (via external routine in file1.bin)
;Notes    1. When the URL object to open is from a HTTP address the item
;            is downloaded to the cache area and is named accordingly for
;            loading.
;=========================================================================
URL_OpenURLLink
   LD    HL,URL_Object		;Full URL of hyperlink item
   LD    DE,KB_Buffer		;address of filename to save URL to
   CALL  SYS_SaveRegs		;Save registers (AF/BC/DE/HL)
   LD    HL,Ext_OpenHTTP
   JP    SYS_RunCode1		;Open URL via HTTP.
