;=========================================================================
;
; Application Routines (UTL)
; Copyright (c) 2010 Ray Palmer
;
;=========================================================================
;Name     UTL_Trap
;Purpose  A debug routine to enable WinAPE to trap HTML cmd processing.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
UTL_Trap
   RET
;=========================================================================
;Name     UTL_WithinBounds
;Purpose  To check if the point is with the bounds
;In Regs  A = ??
;         BC = Check point in question
;         DE = End point
;         HL = Start Point
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = result (1=outide, 0=inside)
;         BC = unchanged
;         DE = unchanged
;         HL = unchanged
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
UTL_WithinBounds
   LD    A,1		;Assume outside
   OR    A		;Clear Carry flag
   SBC   HL,BC		;Is check point >= start?
   JR    Z,UTL_WinBds1	;Check point at start (is ok)
   RET   NC		;point is before start (not inside at all)
UTL_WinBds1
   EX    DE,HL
   OR    A		;Clear Carry flag
   SBC   HL,BC		;Is check point <= End?
   RET   C		;No (point after end)
   XOR   A
   RET
;=========================================================================
;Name     UTL_BuildLinkCmd
;Purpose  To create the script command
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_Buffer has command
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
UTL_BuildLinkCmd
   LD    DE,KB_Buffer
   LD    HL,Script_Link
   LD    BC,4
   LDIR					;insert 'link' command
   LD    A,32
   LD    (DE),A
   INC   DE
   LD    HL,SysGenBuffer+S_LnkTblName
   LD    B,S_LnkTblNameSz
   CALL  UTL_strncpy			;insert link name
   LD    A,','
   LD    (DE),A
   INC   DE
   LD    HL,SysGenBuffer+S_LnkTblURL
   LD    B,URL_Size
   CALL  UTL_strncpy			;insert link URL
UTL_insertCRLF
   LD    A,13
   LD    (DE),A				;insert carriage return
   INC   DE
   LD    A,10
   LD    (DE),A				;insert linefeed
   INC   DE
   XOR   A
   LD    (DE),A				;insert terminator
   RET
;=========================================================================
;Name     UTL_BuildCursorCmd
;Purpose  To build the string of commands to define the cursor
;In Regs  AF = cursor value
;         B = cursor command char
;         DE = address to save command string.
;         HL = cursor speed
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_Buffer updated.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. Assume that the whole series of commands does not exceed the
;            the space allocated for the keyboard buffer.
;=========================================================================
UTL_BuildCursorCmd
   PUSH  HL
   LD    HL,Script_Cursor
   PUSH  BC
   LD    BC,6
   LDIR					;insert cursor script command
   LD    A,SPACE_CHAR
   LD    (DE),A				;insert a space
   INC   DE
   POP   BC
   LD    A,B
   LD    (DE),A				;insert cursor type
   INC   DE
   LD    A,SPACE_CHAR
   LD    (DE),A				;insert a space
   INC   DE
   LD    A,B
   CP    't'
   POP   HL
   JR    NZ,UTL_BuildCursorCmd1
   PUSH  DE
   LD    DE,10
   CALL  Mth_Divide			;divide by 10
   LD    A,C
   POP   DE
UTL_BuildCursorCmd1
   EX    DE,HL
   CALL  UTL_writedec			;insert decimal string
   EX    DE,HL
   JP    UTL_insertCRLF			;insert CR/LF and NULL
;=========================================================================
;Name     UTL_BuildURLCmd
;Purpose  To build the URL command.
;In Regs  AF = Which command to build
;              0 = HOME URL command
;              1 = Start Up command
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FileOpState = status of file handling.
;         KB_Buffer = script command
;Uses     TBD
;Call By  TBD
;Calls    Mem_SwapInCfg
;         Mem_SwapInMain
;Notes    TBD
;=========================================================================
UTL_BuildURLCmd
   LD    DE,CFG_HomePage
   LD    C,'h'
   CP    0
   JR    NZ,UTL_BldURLCmd
   LD    DE,CFG_StartUp
   LD    C,'s'
UTL_BldURLCmd
   PUSH  DE
   PUSH  BC
   CALL  Mem_SwapInCfg
   LD    DE,KB_Buffer
   LD    HL,Script_URL
   LD    BC,3
   LDIR					;insert script command
   POP   BC
   POP   HL
UTL_BldURLCmd2
   LD    A,32
   LD    (DE),A				;insert a space
   INC   DE
   LD    A,C
   LD    (DE),A				;insert URL type
   INC   DE
   LD    A,32
   LD    (DE),A				;insert a space
   INC   DE
   LD    B,URL_Size
   CALL  UTL_strncpy			;insert URL text
   CALL  UTL_insertCRLF
   JP    Mem_SwapInMain
;=========================================================================
;Name     UTL_BuildLocCmd
;Purpose  To build the fileloc script command
;In Regs  AF = ??
;         B = ??
;         C = fileloc type char
;         DE = address of location
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_Buffer has script command.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
UTL_BuildLocCmd
   PUSH  DE
   PUSH  BC
   CALL  Mem_SwapInCfg
   LD    DE,KB_Buffer
   LD    HL,Script_Fileloc
   LD    BC,7
   LDIR					;insert script command
   POP   BC
   POP   HL
   JP    UTL_BldURLCmd2
;=========================================================================
;Name     UTL_GetSavedLink
;Purpose  To get the nth DB entry from the saved links Database.
;In Regs  AF = DB entry number
;         BC = ??
;         DE = address to save URL object
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars SysGenBuffer = contains the DB entry data
;Uses     TBD
;Call By  TBD
;Calls    LNK_CalcLMDBAddr
;Notes    TBD
;=========================================================================
UTL_GetSavedLink
   CALL  LNK_CalcLMDBAddr
   LD    BC,S_LinksTableSz
   LDIR
   JP    Mem_SwapInMain
;=========================================================================
;Name     UTL_GetCfgLocn
;Purpose  To download the configuration setting URL.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = Address of URL to download.
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = address after URL/path created.
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_Buffer = URL downloaded.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. The configuration memory bank already swapped in.
;=========================================================================
UTL_GetCfgLocn
   LD    DE,KB_Buffer
   LD    B,URL_Size
   CALL  UTL_strncpy			;insert current download location
   DEC   DE
   LD    A,(DE)
   INC   DE
   CP    SLASH_CHAR			;Is last slash on path?
   JP    Z,Mem_SwapInMain		;Yes
   LD    A,SLASH_CHAR
   LD    (DE),A
   INC   DE
   JP    Mem_SwapInMain
;=========================================================================
;Name     UTL_LogText
;Purpose  To log text from a catalog device call.
;In Regs  A = character from the print call of &BB5A
;         BC = ??
;         DE = >>
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  LogBank = current address of data to log.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars LogBank = current address of data to log.
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    1. The maximum amount of text is limited to 16K.
;=========================================================================
UTL_LogText
   LD    HL,(LogBank)
   LD    (HL),A
   INC   HL
   BIT   7,H
   RET   NZ
   LD    (LogBank),HL
   RET
;=========================================================================
;Name     UTL_PatchOutBB5A
;Purpose  To patch the system print character routine for logging text.
;In Regs  A = ??
;         BC = ??
;         DE = >>
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars PatchCopy = copy of routine.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. Data is logged in the first of code banks.
;=========================================================================
UTL_PatchOutBB5A
   LD    HL,TXT_OUTPUT
   LD    DE,PatchCopy
   LD    BC,3
   LDIR				;Backup system command
;
   LD    A,&C3			;JP instruction
   LD    HL,UTL_LogText
   LD    (TXT_OUTPUT),A		;Setup new re-direction.
   LD    (TXT_OUTPUT+1),HL
;
   LD    HL,LOGTEXT_LOCN
   LD    (LogBank),HL		;Where in bank start logging text
;
   LD    A,0
   JP    MEM_SwapBCBank
;=========================================================================
;Name     UTL_PatchInBB5A
;Purpose  To restore the print character routine.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  PatchCopy = copy of routine.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars None.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
UTL_PatchInBB5A
   LD    DE,TXT_OUTPUT
   LD    HL,PatchCopy
   LD    BC,3
   LDIR				;Backup system command
   JP    Mem_SwapInMain
;=========================================================================
;Name     UTL_SetInk
;Purpose  To setup the program ink colour
;In Regs  A = ??
;         B = ink colour (0 to 27)
;         C = ink number (0 to 3)
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  None
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars None.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    None.
;=========================================================================
UTL_SetInk
   LD    A,C		;A = ink number
   LD    C,B		;no flashing colours (B = C)
   JP    SC_SET_INK
;=========================================================================
;Name     UTL_GetDateByte
;Purpose  To return address of the version/date data
;In Regs  A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  None
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = address of data
;         IX = ??
;         IY = ??
;Out Vars None.
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None.
;=========================================================================
UTL_GetDateByte
   LD    HL,CPCB_Vers
   LD    A,(HTML_FrmeSets)
   RET
;=========================================================================
;Name     UTL_ReqInput
;Purpose  To request user to enter input.
;In Regs  A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  None
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = address of data
;         IX = ??
;         IY = ??
;Out Vars None.
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None.
;=========================================================================
UTL_ReqInput
   CALL  MNU_ReqInputTxt		;edit the link
   LD    A,(KB_Buffer)
   OR    A				;Any input to process?
   RET
;=========================================================================
;Name     UTL_FlagRefresh
;Purpose  To request a screen refresh.
;In Regs  A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  None
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = address of data
;         IX = ??
;         IY = ??
;Out Vars None.
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None.
;=========================================================================
UTL_FlagRefresh
   PUSH  HL
   LD    HL,HTML_RenderState
   LD    (HL),16		;Flag browser to refresh to screen.
   POP   HL
   RET
