;=====================================================================
;
; Application Routines (Variables handler)
; Copyright (c) 2010 Ray Palmer
;
; This file is built from the base source for variables handling.
;
;=====================================================================
;Name     VIY_CalcAddr
;Purpose  To calc the address of a variable indexed by IY
;Input    IY = Address of System variables
;         HL = offset to variable
;Output   HL = address of variable 
;Notes    None.
;=====================================================================
VIY_CalcAddr
   PUSH  DE
   PUSH  IY
   JR    VIXIY_CAS1
;=====================================================================
;Name     VIX_CalcAddr
;Purpose  To calc the address of a variable indexed by IX
;Input    IX = Address of local data
;         HL = offset to variable
;Output   HL = address of variable 
;Notes    None.
;=====================================================================
VIX_CalcAddr
   PUSH  DE
   PUSH  IX
VIXIY_CAS1
   POP   DE		;DE = Base address of data/variables
   ADD   HL,DE		;HL = Address of variable
   POP   DE
   RET
;=====================================================================
;Name     VAR_BE_GetValDE
;Purpose  To get the value of the variable (16bit)
;Input    HL = address of 16bit variable.
;Output   DE = value of the variable.
;         HL = HL + 2
;Notes    1. Numer is in Big-Endian format.
;=====================================================================
VAR_BE_GetValDE
   LD    D,(HL)
   INC   HL
   LD    E,(HL)
   INC   HL
   RET
;=====================================================================
;Name     VIY_BE_GetVarHL
;Purpose  To get the value of a system variable
;Input    HL = offset to variable
;Output   HL = Value got
;Notes    None.
;=====================================================================
VIY_BE_GetVarHL
   CALL  VIY_CalcAddr
;=====================================================================
;Name     VAR_BE_GetValHL
;Purpose  To get the value of the variable (16bit)
;Input    HL = address of 16bit variable.
;Output   HL = value of the variable.
;Notes    1. Numer is in Big-Endian format.
;=====================================================================
VAR_BE_GetValHL
   PUSH  AF
   LD    A,(HL)
   INC   HL
   LD    L,(HL)
   LD    H,A
   POP   AF
   RET