;=========================================================================
;
; Application Routines (window handler)
; Copyright (c) 2010 Ray Palmer
;
;=========================================================================
;Name     MNU_AdjustLocn
;Purpose  To adjust the coordinates to fit within sub-menu window
;In Regs  A = ??
;         BC = ??
;         DE = current screen coordinate offsets (AMSDOS scale)
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
MNU_AdjustLocn
   PUSH  HL
   PUSH  AF
   LD    HL,(Window_FSRow)
   LD    A,(Window_FSCol)
   LD    H,A			;HL = Base Col/Base Row
   ADD   HL,DE
   EX    DE,HL
   DEC   D			;Adjust column back to base 0
   DEC   E			;same for Row
   POP   AF
   POP   HL
   RET
;=========================================================================
;Name     WIN_SetupScreen
;Purpose  To setup the screen mode and colours.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  Border0 = border colour
;         Scrn_Mode = screen mode (1 or 2)
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars Scrn_MaxCol = max columns for screen.
;Uses     TBD
;Call By  TBD
;Calls    UTL_SetInk
;         SC_SET_MODE (OS)
;         SC_GET_GRAHPIC (OS)
;         SC_VDU_ENABLE (OS)
;Notes    TBD
;=========================================================================
WIN_SetupScreen
;
   CALL  KB_ResetCursorIcon
   LD    A,(Scrn_Mode)
   CALL  SC_SET_MODE			;clear whole screen.
   XOR   A
   CALL  SC_GET_GRAHPIC			;enable graphics char option
   CALL  SC_VDU_ENABLE			;enable VDU
;
   LD    A,(Border0)
   LD    B,A
   LD    C,A
   CALL  SC_SET_BORDER			;set border colour
;
   LD    D,80
   LD    E,70
   LD    A,(Scrn_Mode)
   LD    B,A
   CP    2				;Mode 2 screen?
   JR    Z,Main_Init1			;Yes
   LD    D,40
   LD    E,35
Main_Init1
   LD    A,D
   LD    (Scrn_MaxCol),A		;setup max column on screen.
   LD    A,E
   LD    (SubMenu_OptManageApp1),A
   LD    (SubMenu_OptMgeGen1),A
   LD    (SubMenu_Link_Opn1),A
   LD    A,B
   CP    2
   JR    Z,Main_Init3
   LD    B,4				;Mode 1 (uses 4 colours)
   LD    C,0				;pen/ink index to set
   LD    HL,Pen0
Main_Init2
   PUSH  HL
   PUSH  BC
   LD    B,(HL)				;colour number (0-27)
   CALL  UTL_SetInk
   POP   BC
   POP   HL
   INC   HL				;next pen/ink data
   INC   C				;next ink for OS call
   DJNZ  Main_Init2
   RET
;
Main_Init3
   LD    C,0
   LD    A,(Pen0)
   LD    B,A
   CALL  UTL_SetInk
   LD    C,1
   LD    A,(Pen2)
   LD    B,A
   CALL  UTL_SetInk
   RET
;=========================================================================
;Name     WIN_SelMainWindw
;Purpose  To select the main window stream.
;In Regs  A = stream number
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  None
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars None.
;Uses     TBD
;Call By  TBD
;Calls    SC_SEL_STREAM (OS)
;Notes    TBD
;=========================================================================
WIN_SelMainWindw
   LD    A,(Stream_MainMenu)
   JP    SC_SEL_STREAM			;Select stream for main menu
;=========================================================================
;Name     WIN_ClearStream
;Purpose  To clear the window stream.
;In Regs  A = stream number
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  None
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars None.
;Uses     TBD
;Call By  TBD
;Calls    SC_SEL_STREAM (OS)
;         TXT_CLR_WIN (OS)
;Notes    TBD
;=========================================================================
WIN_ClearStream
   CALL  SC_SEL_STREAM			;Select stream for web page
   JP    TXT_CLR_WIN
;=========================================================================
;Name     WIN_ShowBorderIcon
;Purpose  To display the system icon symbol
;In Regs  A = Icon Number
;         B = 0 if relative else absolute
;         C = ??
;         DE = ??
;         HL = Row in L, Column in H
;         IX = ??
;         IY = ??
;In Vars  Window_FRowOff = frame start row/col offset
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    FNT_PrintIcon
;Notes    TBD
;=========================================================================
WIN_ShowBorderIcon
   PUSH  HL
   LD    DE,(Window_FRowOff)
   LD    C,SYSTEM_ICONSET
   CALL  FNT_PrintIcon
   POP   HL
   RET
;=========================================================================
;Name     WIN_GetFrameLoc
;Purpose  To get the web-page frame location details
;In Regs  A = ??
;         BC = Offset adjustment for the coordinates
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  Window_FSRow = CPC screen start row (base 0)
;         Window_FERow = CPC screen end row (base 0)
;         Window_FSCol = CPC screen start column (base 0)
;         Window_FECol = CPC screen end column (base 0)
;Out Regs A = ??
;         BC = ??
;         DE = end col/row
;         HL = start col/row
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
WIN_GetFrameLoc
   PUSH  AF
   LD    HL,(Window_FERow)
   ADD   HL,BC			;adjust start row for web-page
   LD    A,(Window_FECol)
   LD    H,A
   EX    DE,HL
   LD    HL,(Window_FSRow)
   ADD   HL,BC			;adjust end row for web-page
   LD    A,(Window_FSCol)
   LD    H,A
   POP   AF
   RET
;=========================================================================
;Name     WIN_NonScrollFrame
;Purpose  To display the non-scroll frame border
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = frame offset
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. put this routine into the loaded modules to reduce ROM code.
;=========================================================================
WIN_NonScrollFrame
   XOR   A
   JR    WIN_ShowScrollFrame		;show the frame
;=========================================================================
;Name     WIN_ScrollFrame
;Purpose  To display the scroll frame border
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = frame offset
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. put this routine into the loaded modules to reduce ROM code.
;=========================================================================
WIN_ScrollFrame
   LD    A,1
;
WIN_ShowScrollFrame
   LD    BC,&0005		;web-page row offset
   CALL  WIN_GetFrameLoc	;HL/DE has web-page frame border
   LD    C,FRAMES_ICONSET
   CALL  SYS_SaveRegs		;Save AF/BC/DE/HL
   LD    HL,Ext_DisplayFrame
   JP    FNT_DoExtRoutine
;=========================================================================
;Name     WIN_ShowBorder
;Purpose  To display the border of a window
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FrameWinType = corrupted
;         Window_FRowOff = corrupted
;Uses     TBD
;Call By  HTML routines for style handling.
;Calls    TBD
;Notes    TBD
;=========================================================================
WIN_ShowBorder
   LD    A,(BorderIconSet)
   BIT   7,A
   RET   NZ			;None/hidden border
;
   LD    C,A			;icon set
   LD    A,(BorderIconStrt)
   LD    B,A			;First icon in borders
   PUSH  BC
   LD    BC,&0005		;web-page row offset
   CALL  WIN_GetFrameLoc	;HL/DE has web-page frame border
   POP   BC
   LD    A,(BorderCorner)	;soft/hard corner setting.
   JR    WIN_ShwBorder
;=========================================================================
;Name     WIN_ShowAllBorders
;Purpose  To show all border on a sub-menu window on the CPC screen and
;         clear the sub-menu frame for display.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  Window_FSRow = CPC screen start row (base 0)
;         Window_FERow = CPC screen end row (base 0)
;         Window_FSCol = CPC screen start column (base 0)
;         Window_FECol = CPC screen end column (base 0)
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TXT_CLR_WIN (OS)
;         WIN_GetFrameLoc
;Notes    1. This routine for a window which fits entirely on the CPC
;            screen.
;         2. The selected CPC stream for the window has already been
;            selected.
;=========================================================================
WIN_ShowAllBorders
   CALL  TXT_CLR_WIN		;clear window of background
   LD    BC,&0001		;web-page row offset
   CALL  WIN_GetFrameLoc	;get frame coordinates
   JR    WIN_ShwBorder1
;=========================================================================
;Name     WIN_ShowMainBorder
;Purpose  To display the border of the main menu window
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FrameWinType = corrupted
;         Window_FRowOff = corrupted
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. CHANGE ME FOR HARD CODED BORDERS (20140215)
;=========================================================================
WIN_ShowMainBorder
   LD    A,(Scrn_MaxCol)
   DEC   A
   LD    D,A			;end column of screen
   LD    E,ER_MainMenu		;end row of menu
   LD    HL,0			;start column/row
WIN_ShwBorder1
   LD    A,(SysBorderSet)
   LD    C,A			;Icon set for CPCB borders
   LD    B,BORDERS_START	;First icon in borders
   LD    A,BORDERS_HARD		;Hard corners
WIN_ShwBorder
   CALL  SYS_SaveRegs		;Save AF/BC/DE/HL
   LD    HL,Ext_DisplayBorder
   JP    FNT_DoExtRoutine
;=========================================================================
;Name     WIN_SetWindow
;Purpose  To define the window onto the web-page canvas.
;In Regs  A = Frames ID
;         BC = ??
;         D = Start Column of CPC window
;         E = End Column of CPC window
;         H = Start Row of CPC window
;         L = End Row of CPC window
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars Window_SRow = start row
;         Window_ERow = end row
;         Window_SCol = start column
;         Window_ECol = end column
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. This routine is only used for defining the CPC screen for
;            the reserved frame (frame 0).
;=========================================================================
WIN_SetWindow
   LD    (Window_FrmeID),A
   LD    B,0
   LD    C,D
   LD    (Window_SRow),BC
   LD    C,E
   LD    (Window_ERow),BC
   LD    C,H
   LD    (Window_SCol),BC
   LD    C,L
   LD    (Window_ECol),BC
   RET
;=========================================================================
;Name     WIN_SetupLocSize
;Purpose  To setup the coordinates and size of the window.
;In Regs  A = ??
;         BC = ??
;         D = end column
;         E = end row
;         H = start column
;         L = start row
;         IX = ???
;         IY = ??
;In Vars  TBD
;Out Regs AF = Corrupted.
;         BC = Corrupted.
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars Windw_NumRows = number of rows (from 1)
;         Windw_NumCols = number of columns (from 1)
;         Window_FSRow = CPC screen start row (base 0)
;         Window_FERow = CPC screen end row (base 0)
;         Window_FSCol = CPC screen start column (base 0)
;         Window_FECol = CPC screen end column (base 0)
;Uses     A and BC corrupted.
;Call By  WIN_DefineWindow
;Calls    None
;Notes    None.
;=========================================================================
WIN_SetupLocSize
;
;  Setup frame dimensions
;
   LD    B,0
   LD    C,H
   DEC   C
   LD    (Window_FSCol),BC
   LD    C,D
   DEC   C
   LD    (Window_FECol),BC
   LD    C,L
   DEC   C
   LD    (Window_FSRow),BC
   LD    C,E
   DEC   C
   LD    (Window_FERow),BC
;
;  Setup sizes of window
;
   LD    A,E
   SUB   L
   INC   A
   LD    (Windw_NumRows),A
   LD    A,D
   SUB   B
   INC   A
   LD    (Windw_NumCols),A
   RET
;=========================================================================
;Name     WIN_DefineWindow
;Purpose  To define stream 3 with the window coordinates for the window
;         of menu option data.
;In Regs  A = CPC Window stream number
;         BC = ??
;         DE = ??
;         HL = address of window dimensions in AMSDOS specification.
;         IX = ???
;         IY = ??
;In Vars  None
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars Windw_NumRows = number of rows
;         Windw_NumCols = number of columns
;         Window_FSRow = CPC screen start row (base 0)
;         Window_FERow = CPC screen end row (base 0)
;         Window_FSCol = CPC screen start column (base 0)
;         Window_FECol = CPC screen end column (base 0)
;Uses     TBD
;Call By  TBD
;Calls    SC_SEL_STREAM (OS)
;         SC_SET_TXTWIN (OS)
;Notes    1. Menu data is in AMSTRAD CPC screen coordinates (i.e they
;            start from 1 and not 0).
;         2. The menu data is the absolute position on the screen. ie row
;            3 is row 3 from the top left of the screen.
;=========================================================================
WIN_DefineWindow
   PUSH  HL
   CALL  SC_SEL_STREAM		;Select stream for sub-menu window.
   POP   HL
;
;  Get CPC OS window coordinates (not existing window dependent)
;
   LD    B,(HL)			;B = Start column
   INC   HL
   LD    D,(HL)			;D = end column
   INC   HL
   LD    A,(HL)			;A = start row
   INC   HL
   LD    E,(HL)			;E = end row
   LD    H,B			;H = Start column
   LD    L,A			;L = Start Row
;
   CALL  WIN_SetupLocSize
;
;  Define stream window on CPC screen
;
   JP    SC_SET_TXTWIN		;define a AMSDOS window
;=========================================================================
;Name     WIN_PrintChar
;Purpose  To display a character on the screen.
;In Regs  A = Stream Column (for new line)
;         B = Max row
;         C = max column
;         D = Colour index (fore/back)
;         E = ASCII Character to show.
;         H = Stream Column
;         L = Stream Row
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         B = end row
;         C = end column
;         DE = ??
;         H = Column (updated)
;         L = Row (updated if new line)
;         IX = ??
;         IY = ??
;Out Vars Windw_NumRows = number of rows in the stream.
;                         Max value is 25.
;Uses     TBD
;Call By  WIN_ShowWindwData
;Calls    FNT_WR_CHAR
;Notes    1. If the cursor for the stream is outside then no output is
;            shown.
;=========================================================================
WIN_PrintChar
   LD    D,A			;save new column index
   LD    A,H			;Get current column index
   CP    C			;before right edge of window?
   RET   NC			;Yes
   LD    A,E
   LD    E,255			;Use normal font (by force)
   CP    SPACE_CHAR		;control char?
   JR    NC,WIN_DisplayChr1	;No (display using normal font)
   CP    LINEFEED_CHAR		;LF char?
   RET   NZ
   LD    H,D			;reset to left column 
   LD    A,B
   CP    L			;at bottom of window?
   RET   Z			;Yes
   INC   L			;next row
   RET
;=========================================================================
;Name     WIN_DisplayChar
;Purpose  To show text character in the window using a font.
;In Regs  A = Character to print (ASCII)
;         BC = ??
;         D = colour indices
;         E = ??
;         H = Column coordinate
;         L = Row coordinate
;         IX = ???
;         IY = ??
;In Vars  FontOutput = Type of font to use
;                      0 = current selected font
;                      1 = Proportional Font
;                      255 = fixed to default system font.
;Out Regs AF = ??
;         BC = ??
;         DE = corrupted
;         H = Updated Column coordinate
;         L = Updated Row coordinate (if new line printed)
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  HTML ROM (CPCB_DisplayChar)
;Calls    FNT_WR_CHAR (external routine)
;Notes    None.
;=========================================================================
WIN_DisplayChar
   LD    C,A
   LD    A,(FontOutput)
   LD    E,A			;Setup Font type
   LD    A,C
;=========================================================================
;Name     WIN_DisplayChar1
;Purpose  To show text character in the window.
;In Regs  A = Character to print (ASCII)
;         BC = ??
;         D = colour indices
;         E = Font Type (255 = system font)
;         H = Column coordinate
;         L = Row coordinate
;         IX = ???
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = corrupted
;         H = Updated Column coordinate
;         L = Updated Row coordinate (if new line printed)
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  WIN_ShowText2
;Calls    FNT_WR_CHAR (external routine)
;Notes    None.
;=========================================================================
WIN_DisplayChr1
   LD    BC,0			;no offset for stream/window
   LD    D,&10			;Setup colours
   JP    FNT_WR_CHAR		;Show it on screen in the stream
;=========================================================================
;Name     WIN_ShowText2
;Purpose  To show text in the window.
;In Regs  A = ??
;         B = ??
;         C = ??
;         DE = address of text to display
;         H = Column coordinate
;         L = Row coordinate
;         IX = ???
;         IY = ??
;In Vars  Scrn_MaxCol = max column for screen.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    WIN_PrintChar1
;Notes    1. Only 1 line of text allowed (not checked)
;=========================================================================
WIN_ShowText2
   LD    A,(DE)			;Get error message char
   OR    A			;End of message ?
   RET   Z			;Yes
;
   PUSH  DE
   PUSH  BC
   LD    D,A
   LD    A,(Scrn_MaxCol)
   DEC   A
   CP    H			;At right most column?
   LD    A,D
   LD    D,&01			;Setup colours
   LD    E,255			;Force normal font
   CALL  NZ,WIN_DisplayChr1
   POP   BC
   POP   DE
   INC   DE			;Next address of error text
   JR    WIN_ShowText2		;Do another if no exceeded max.
;=========================================================================
;Name     WIN_PrintStr
;Purpose  To print a line of text input into a window.
;In Regs  A = CPC OS window Stream ID
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  KB_StartCol = CPC Screen Row/Column (L=Row/H=Column)
;                       Row/Col must start from 1
;         KB_BuffLoc = location of text string (NULL termninated)
;         KB_Offset = offset into string
;         KB_printlen = max length of text printable (for multi-line
;                       output this is the total text that can be shown
;                       upto a max of 65535 characters).
;         KB_BaseInpRC = Base Row/Col printing dimensions.
;         KB_MaxInpRC = Max row/column for echo back.
;         KB_EchoFlag = echo back control flags
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    WIN_PrintStrChr
;Notes    1. The width of the CPC window guides how text is managed when
;            outputing multi-line data.
;         2. This routine is for the echo-back of keyboard input.
;=========================================================================
WIN_PrintStr
   OR    A
   LD    HL,(KB_Offset)			;HL = Offset into buffer
   LD    BC,(KB_printlen)		;BC = MAX print length
   SBC   HL,BC				;Is KB offset after max print length
   JR    NC,WIN_PrintStr2		;No
   LD    HL,0				;Start at begining of buffer
WIN_PrintStr2
   EX    DE,HL				;DE = start offset for output
   LD    HL,(KB_BuffLoc)
   ADD   HL,DE				;point to start of print text
WIN_PrintStr4
   LD    A,(HL)				;Get char for display
   CP    NUL				;end of string?
   JR    Z,WIN_PrintStr5		;Yes
;
   INC   HL				;next text location
   CP    SPACE_CHAR			;Is char > 31?
   JR    NC,WIN_PrintStr6		;Yes
;
   CP    LINEFEED_CHAR			;new line?
   JR    NZ,WIN_PrintStr4		;No (skip all control codes)
;
WIN_PrintStr4a
   LD    A,(KB_EchoFlag)
   BIT   7,A				;Multi-line input?
   RET   Z				;No (return stops print)
   LD    A,(KB_BaseInpRC)		;Get start column in multi-line input.
   LD    DE,(KB_StartCol)		;D=Col,E=Row
   INC   E				;Next Row for new line
   LD    D,A				;Reset start column
   LD    (KB_StartCol),DE
   LD    A,ER_WebPage
   CP    E				;At bottom of CPC screen?
   RET   Z				;Yes
   LD    A,(KB_MaxInpRow)
   CP    E				;At bottom of echo back area?
   JR    NZ,WIN_PrintStr4		;No
   RET
;
WIN_PrintStr6
;
   EX    DE,HL
   LD    HL,(KB_StartCol)		;H=Col/L=Row
   LD    L,A
   LD    A,(KB_MaxInpCol+1)
   CP    H				;At right edge of echo back area?
   JR    Z,WIN_PrintStr4a           	;Yes (check if wrap text)
   LD    A,L
   LD    HL,(KB_StartCol)		;H=Col/L=Row
   LD    H,A
   LD    A,(KB_MaxInpCol)
   CP    L				;bottom edge of window?
   RET   Z				;Yes
;
   PUSH  DE				;Save address of text
   LD    E,H				;save char to output
   LD    A,(KB_EchoFlag)
   BIT   0,A				;Hidden output?
   JR    Z,WIN_PrintStr7		;No
   LD    E,'*'				;Show input as hidden by '*' chars
WIN_PrintStr7
;
   PUSH  BC
   LD    D,&10				;Setup colours
   LD    A,E				;Get char to output back.
   LD    HL,(KB_StartCol)		;H=Col/L=Row
   CALL  WIN_DisplayChr1
   LD    (KB_StartCol),HL
   POP   BC
   POP   HL				;Get address of text back
   DEC   BC
   LD    A,B
   OR    C				;At end of max print?
   JR    NZ,WIN_PrintStr4		;No
   RET
WIN_PrintStr5
   LD    HL,(KB_StartCol)		;H=Col/L=Row
   LD    A,SPACE_CHAR
   JP    WIN_DisplayChr1
;=========================================================================
;Name     WIN_ShowWindwData
;Purpose  To show the a text string in the CPC stream window.
;In Regs  A = ??
;         BC = ??
;         DE = Column/Row to display text (E=Row/D=Column)
;         HL = location of string text
;         IX = ???
;         IY = ??
;In Vars  Window_FERow = end row 
;         Window_FSCol = start column
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = address of keys to check after data shown.
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     All registers
;Call By  TBD
;Calls    WIN_PrintChar
;Notes    1. Where the data attempts to overflow, the number of rows for
;            the routine exits.
;         2. if CR char is found then the LF char is expected to follow.
;         3. Cursor locations are offset from 1 in the stream for which
;            printing occurs.
;=========================================================================
WIN_ShowWindwData
   EX    DE,HL
   INC   L			;Account for frame
   INC   H			;Account for left column
   INC   H
   LD    A,(Window_FECol)
   LD    C,A			;end column
   LD    A,(Window_FERow)
   LD    B,A			;end row
WIN_ShowWinData1
   LD    A,(DE)			;Get char for display
   INC   DE
   OR    A			;End of text?
   JR    Z,WIN_ShowWinData2	;Yes
   PUSH  DE
   PUSH  BC
   LD    E,A			;E = Char to show
   LD    A,(Window_FSCol)	;start column for new line
   INC   A
   CALL  WIN_PrintChar
   POP   BC
   POP   DE
   JR    WIN_ShowWinData1
WIN_ShowWinData2
   EX    DE,HL
   RET
;=========================================================================
;Name     MNU_ShowWindowItems
;Purpose  To display a window on the screen with data in stream 3.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = address of window menu data.
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars MenuWinDatLoc = location of window data
;         MenuWinKeyLoc = location of keys list.
;Uses     TBD
;Call By  TBD
;Calls    WIN_DefineWindow
;         WIN_ShowAllBorders
;         WIN_ShowWindwData
;Notes    TBD
;=========================================================================
MNU_ShowWindowItems
   LD    (MenuWinDatLoc),HL		;Saved location of window data
   LD    A,(Stream_SubMenu)
   LD    (Windw_state),A
;
   CALL  WIN_DefineWindow		;Define window for submenu data
   CALL  WIN_ShowAllBorders		;Clear window, show borders
;
   LD    HL,(MenuWinDatLoc)
   LD    DE,4
   ADD   HL,DE				;Skip window dimensions data
   LD    DE,(Window_FSRow)		;Start at column 1, Row 1
   LD    A,(Window_FSCol)
   LD    D,A
   INC   D
   INC   E
   CALL  WIN_ShowWindwData		;Show data in stream window
   LD    (MenuWinKeyLoc),HL		;Save location of keys data
   RET
;=========================================================================
;Name     MNU_ShowWindowItem
;Purpose  To display a window on the screen with data in stream 3.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = address of window menu data.
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    MNU_ShowWindowItems
;         KB_Wait4akey
;         SYS_DoHandler
;Notes    TBD
;=========================================================================
MNU_ShowWindowItem
   CALL  MNU_ShowWindowItems
   LD    A,(HL)
   CP    KEY_ENDLIST		;Any keys to handle?
   RET   Z			;No
   JP    KB_Wait4akey		;wait for a key and handle it
;=========================================================================
;Name     WIN_CalcFrmeDB
;Purpose  To calculate the address of a frames DB item.
;In Regs  A = Frame ID number
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  WindowFrameID = Frame ID number
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = address within the frames DB.
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. The bank containing the frames DB is left swapped in.
;=========================================================================
WIN_CalcFrmeDB
   LD    HL,(FramesDBLoc)
   LD    A,(WindowFrameID)
   LD    B,A
   LD    DE,S_HTMLFramSz
WIN_CalcFrameDB1
   ADD   HL,DE
   DJNZ  WIN_CalcFrameDB1	;Try next frame DB entry
   JP    Mem_SwapInFramesDB
;=========================================================================
;Name     WIN_GetFrame
;Purpose  To get the nth frame DB item.
;In Regs  A = Frame ID number
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  None.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars WindowFrameID = Frame ID number
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
WIN_GetFrame
   LD    (WindowFrameID),A
   CALL  WIN_CalcFrmeDB
   LD    DE,Window_FrameV
   LD    BC,S_HTMLFramSz
   LDIR
   RET
;=========================================================================
;Name     WIN_NewFrame
;Purpose  To add a new frame to the frames DB
;In Regs  A = New Frame ID (0-15)
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  Window_FrameV
;           Window_SRow
;           Window_ERow
;           Window_SCol
;           Window_ECol
;           Window_Addr
;Out Regs A = statue (255 = added ok else zero)
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars Window_FrmeID = frame index into the frames DB
;Uses     TBD
;Call By  TBD
;Calls    MEM_SwapInCfg
;Notes    TBD
;=========================================================================
WIN_NewFrame
   CALL  Mem_SwapInFramesDB	;Swap in bank with frame DB
;
;  Search DB for a free entry within DB
;
   LD    HL,(FramesDBLoc)
   LD    A,(FramesDBMax)
   LD    B,A
   LD    DE,S_HTMLFramSz
WIN_NewFrme1
   BIT   0,(HL)			;end of DB?
   JR    Z,WIN_NewFrme2		;Yes (save new frame to DB)
   ADD   HL,DE
   DJNZ  WIN_NewFrme1		;Try next frame DB entry
   XOR   A			;flag frame counld not be saved.
   RET
WIN_NewFrme2
   EX    DE,HL
   LD    HL,Window_FrmeID
   LD    (HL),&11		;Flag db entry taken (clear flags)
   LD    HL,Window_FrameV
   LD    BC,S_HTMLFramSz
   LDIR				;Insert new frame into DB
   XOR   A
   LD    (DE),A			;Flag end of Database
   DEC   A			;Flag entry saved.
   RET
;=========================================================================
;Name     WIN_CheckInFrame
;Purpose  To check if the row/column location in within a frame.
;In Regs  A = ??
;         BC = Row location from zero.
;         DE = ??
;         HL = Column location from zero.
;         IX = ??
;         IY = ??
;In Vars  Window_SRow
;         Window_ERow
;         Window_SCol
;         Window_ECol
;Out Regs A = Z flag set if within frame else NZ
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
WIN_CheckInFrame
;
;  Check if cursor is within the frame dimensions
;
   PUSH  HL
   LD    HL,(Window_SRow)
   LD    DE,(Window_ERow)
   CALL  UTL_WithinBounds
   POP   BC
   OR    A					;within Row bounds?
   RET   NZ					;No (check next entry)
   LD    HL,(Window_SCol)
   LD    DE,(Window_ECol)
   CALL  UTL_WithinBounds
   OR    A					;within Row bounds?
   RET
;=========================================================================
;Name     WIN_CheckInCPC
;Purpose  To check if the row/column location in within CPC screen.
;In Regs  A = ??
;         BC = Row location from zero.
;         DE = ??
;         HL = Column location from zero.
;         IX = ??
;         IY = ??
;In Vars  PageWdw_SRow = screen row start on web-page
;         PageWdw_ERow = screen col start on web-page
;         PageWdw_SCol = screen row end of web-page
;         PageWdw_ECol = screen col end on web-page
;Out Regs A = Z flag set if within frame else NZ
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  WIN_ChkFrmeLim
;Calls    UTL_WithinBounds
;Notes    TBD
;=========================================================================
WIN_CheckInCPC
;
;  Check if location is within the frame dimensions
;
   PUSH  HL
   LD    HL,(PageWdw_SRow)
   LD    DE,(PageWdw_ERow)
   CALL  UTL_WithinBounds
   POP   BC
   OR    A					;within Row bounds?
   RET   NZ					;No (check next entry)
   LD    HL,(PageWdw_SCol)
   LD    DE,(PageWdw_ECol)
   CALL  UTL_WithinBounds
   OR    A					;within Row bounds?
   RET
;=========================================================================
;Name     WIN_FndFrame
;Purpose  To find a frame when the cursor select has been actioned.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  WebPge_Col = current web page column position.
;         WebPge_Row = current web page row position.
;         Window_SRow
;         Window_ERow
;         Window_SCol
;         Window_ECol
;Out Regs A = 255 (not found) else is frame number.
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars Window_FrameV = Frame which cursor is within.
;         WindowFrmeNum = frame number of clicked on frame.
;Uses     TBD
;Call By  CursorSelect (cpcb.exe)
;Calls    WIN_GetFrame
;         WIN_CheckInFrame
;         HTML_FindFrameIcon
;Notes    TBD
;=========================================================================
WIN_FndFrame
   XOR   A
   LD    (FrameItemSel),A	;Assume no frame selected.
   LD    (WindowFrmeNum),A
   LD    A,(HTML_FrameFlg)
   BIT   7,A
   JR    Z,WIN_FindFrmeDB1	;No frames (nothing to check)
   LD    A,2
   JR    WIN_FindFrmeDB0	;Start at 1st frame.
WIN_FindFrmeDB1
   LD    A,(WindowFrmeNum)
   INC   A
WIN_FindFrmeDB0
   LD    (WindowFrmeNum),A
   CALL  WIN_GetFrame
   LD    A,(Window_FrmeID)
   BIT   0,A			;end of DB?
   JR    Z,WIN_FindFrmeDB2	;Yes
;
   LD    BC,(WebPge_Row)	;Web-page row (starts from 0)
   LD    HL,(WebPge_Col)	;Web-page column (starts from 0)
   CALL  WIN_CheckInFrame
   JR    NZ,WIN_FindFrmeDB1	;No (check next entry)
;
   LD    HL,Window_FrmeID
   BIT   4,(HL)			;Border shown?
   RET   Z			;No
   BIT   6,(HL)			;scrollable window?
   CALL  NZ,HTML_FindFrameIcon	;Yes (check for scroll icon)
   LD    A,(FrameItemSel)
   BIT   7,A			;Icon clicked on?
   RET   Z			;No
   RES   7,A
   OR    A			;Any frame icon selected?
   RET   Z			;No
   LD    HL,HTML_RenderState
   SET   4,(HL)			;Flag a refresh is required.
   RET
WIN_FindFrmeDB2
   LD    A,1
   LD    (WindowFrmeNum),A	;flag no frame found
   CALL  WIN_GetFrame		;restore frame data.
   LD    A,255
   RET
;=========================================================================
;Name     WIN_ChkFrmeLim
;Purpose  To check if current frame is within the CPC screen
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  Window_SRow = Frame start row
;         Window_SCol = Frame start column
;         Window_ERow = Frame end row
;         Window_ECol = frame end column
;         Window_FrameV = window frame definition
;         WindowFrameID = Frames DB index
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars DB updated
;Uses     TBD
;Call By  TBD
;Calls    WIN_CheckInCPC
;         WIN_CalcFrmeDB
;Notes    Move this routine into the HTML Rom as part of HTML rendering.
;=========================================================================
WIN_ChkFrmeLim
;
   LD    BC,(Window_SRow)		;Start of frame (row)
   LD    HL,(Window_SCol)		;Start of frame (column)
   CALL  WIN_CheckInCPC			;Is location within frame?
   JR    Z,WIN_ChkFrmeLim0		;Yes (frame has potion on screen)
;
   LD    BC,(Window_ERow)		;End of frame (row)
   LD    HL,(Window_ECol)		;End of frame (column)
   CALL  WIN_CheckInCPC			;Is location within frame?
   JR    NZ,WIN_ChkFrmeLim0		;No (frame not in CPC window)
;
WIN_ChkFrmeLim0
;
;  Flag portion of frame is visible on CPC screen
;
   LD    A,(WindowFrameID)
   CALL  WIN_CalcFrmeDB
   RES   3,(HL)
   RET
WIN_ChkFrmeLim1
;
;  Flag no portion of frame is visible on CPC screen
;
   LD    A,(WindowFrameID)
   CALL  WIN_CalcFrmeDB
   SET   3,(HL)
   RET
;=========================================================================
;Name     WIN_ChkFrmes
;Purpose  To check if all/any frames are visible on CPC screen.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  PageWdw_SRow = current row offset to top left of CPC screen
;         PageWdw_SCol = current column offset to top left of CPC screen
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars Window_FrameV = corrupted.
;         WindowFrameID
;Uses     TBD
;Call By  TBD
;Calls    WIN_GetFrame
;         UTL_WithinBounds
;         WIN_CalcFrmeDB
;Notes    Move this routine into the HTML Rom as part of HTML rendering.
;=========================================================================
WIN_ChkFrmes
   LD    A,1
   LD    HL,HTML_FrameFlg
   BIT   7,(HL)				;Frames in use?
   JR    Z,WIN_ChkFrames0		;No
   INC   A				;Start at 1st loaded frame
WIN_ChkFrames0
   CALL  WIN_GetFrame			;Get nth frame DB entry
   LD    A,(Window_FrmeID)
   BIT   0,A				;End of DB?
   RET   Z				;Yes
   CALL  WIN_ChkFrmeLim			;update frame status bits
   LD    A,(WindowFrameID)
   INC   A
   JR    WIN_ChkFrames0			;check next frame db entry
;=========================================================================
;Name     Win_SetupMainWbPge
;Purpose  To setup the web-page default in the pages Db.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
Win_SetupMainWbPge
   XOR   A
   LD    HL,(BaseHTMLLoc)
   LD    (Window_Addr),HL		;Start address of base HTML
   LD    (Window_Addr+2),A		;Start HTML RAM bank index
   LD    BC,0
   LD    (Window_SCol),BC
   LD    (Window_SRow),BC
   DEC   BC				;Max web-page columns/rows
   LD    (Window_ECol),BC
   LD    (Window_ERow),BC
   LD    A,1
   JP    WIN_NewFrame			;Add window to Frames DB
;=========================================================================
;Name     WIN_Initialise
;Purpose  To initialise the windows and display the main menu.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  Scrn_Mode = Screen mode
;         Scrn_MaxCol = Max column for screen mode.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars URL_EndAddr = end of HTML data
;         Window_Addr = start of HTML data
;         Window_FSRow = CPC screen start row (base 0)
;         Window_FERow = CPC screen end row (base 0)
;         Window_FSCol = CPC screen start column (base 0)
;         Window_FECol = CPC screen end column (base 0)
;Uses     TBD
;Call By  TBD
;Calls    WIN_SetWindow
;         WIN_NewFrame
;         SC_SET_MODE (OS)
;         SC_GET_GRAHPIC (OS)
;         SC_VDU_ENABLE (OS)
;         WIN_MainMenu
;         MNU_ShowMainMenu
;         MNU_ShowURL
;         WIN_ShowMainBorder
;         WIN_MainPage
;         MNU_MainMenuFrame
;Notes    1. The first entry is for the browser main menu window.
;         2. The second entry is for the main browser HTML window.
;=========================================================================
WIN_Initialise
;
   LD    D,0				;SC_MainMenu
   LD    A,(Scrn_MaxCol)
   LD    H,A
   LD    L,0				;SR_MainMenu
   LD    E,ER_MainMenu
   XOR   A
   CALL  WIN_SetWindow			;Setup window dimensions
   CALL  WIN_NewFrame			;Add window to Frames DB
;
   CALL  Win_SetupMainWbPge
;
;  Setup frame for main menu
;
   LD    HL,4
   LD    (Window_FERow),HL
   LD    A,(Scrn_MaxCol)
   DEC   A
   LD    L,A
   LD    (Window_FECol),HL
   LD    HL,0
   LD    (Window_FSRow),HL
   LD    (Window_FSCol),HL
;
   CALL  WIN_MainMenu			;Define the main menu window
;
   CALL  MNU_MainMenuFrame
;=========================================================================
;Name     WIN_MainPage
;Purpose  To initialise the main HTML view space.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  Stream_WebPage = main page stream
;         Scrn_MaxCol = max column for screen mode.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    WIN_GetFrame
;         SC_SET_TXTWIN (OS)
;         SC_SEL_STREAM (OS)
;         TXT_CLR_WIN (OS)
;Notes    1. Stream window limits start from 0 and not 1 (see Doco for
;            CPC464 by Don Thomasson)
;=========================================================================
WIN_MainPage
   LD    A,(Stream_WebPage)
   CALL  SC_SEL_STREAM			;Select stream for web page
   LD    A,1
   CALL  WIN_GetFrame			;Get Frame data for web-page window
   LD    D,SC_WebPage-1			;start column
   LD    A,(Scrn_MaxCol)
   DEC   A
   LD    H,A				;end column
   LD    L,SR_WebPage			;start row
   LD    E,ER_WebPage-1			;End row
   CALL  SC_SET_TXTWIN			;define web-page window
   JP    TXT_CLR_WIN			;clear its contents
;=========================================================================
;Name     WIN_MainMenu
;Purpose  To initialise the main menu window and display the main menu.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    WIN_GetFrame
;         SC_SET_TXTWIN (OS)
;         SC_SEL_STREAM (OS)
;         TXT_CLR_WIN (OS)
;Notes    TBD
;=========================================================================
WIN_MainMenu
   LD    A,(Stream_MainMenu)
   CALL  SC_SEL_STREAM			;Select stream for main menu
   XOR   A
   CALL  WIN_GetFrame			;Get Frame data for main menu
   LD    D,SC_MainMenu-1
   LD    A,(Scrn_MaxCol)
   DEC   A
   LD    H,A
   LD    L,SR_MainMenu-1
   LD    E,ER_MainMenu-1
   CALL  SC_SET_TXTWIN			;define menu text window
   JP    TXT_CLR_WIN			;clear its contents
;=========================================================================
;Name     WIN_ClearWebPage
;Purpose  To clear the Web-Page window on the CPC screen.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ???
;         IY = ??
;In Vars  Stream_WebPage = stream ID for CPC to clear
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    SC_SEL_STREAM (OS)
;         TXT_CLR_WIN (OS)
;Notes    TBD
;=========================================================================
WIN_ClearWebPage
   LD    A,(Stream_WebPage)
   JR    WIN_ClearWindow1
;=========================================================================
;Name     WIN_ClearWindow
;Purpose  To clear the window.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ???
;         IY = ??
;In Vars  Stream_SubMenu
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    MNU_PageRefresh
;         SC_SEL_STREAM (OS)
;         TXT_CLR_WIN (OS)
;Notes    TBD
;=========================================================================
WIN_ClearWindow
   CALL  MNU_PageRefresh		;Flag web-page refresh.
   LD    A,(Stream_SubMenu)
WIN_ClearWindow1
   CALL  SC_SEL_STREAM			;Select stream for window.
   JP    TXT_CLR_WIN			;clear window if was up
;=========================================================================
;Name     WIN_HandleESC
;Purpose  To handle the escape key when pressed from the menu-window
;         displayed on the web-page canvas.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ???
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    WIN_ClearWindow
;         MNU_PageRefresh
;Notes    TBD
;=========================================================================
WIN_HandleESC
   CALL  WIN_ClearWindow	;clear sub-menu window
   JP    UTL_FlagRefresh	;flag page re-displayed
;=========================================================================
;Name     WIN_RefreshFrame
;Purpose  To clear the frame for a refresh of content.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    SC_SET_TXTWIN (OS)
;         SC_SEL_STREAM (OS)
;         SC_VDU_ENABLE (OS)
;         TXT_CLR_WIN (OS)
;Notes    TBD
;=========================================================================
WIN_RefreshFrame
   CALL  UTL_FlagRefresh
;=========================================================================
;Name     WIN_ClearArea
;Purpose  To clear the frame for a refresh of content.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  Window_FSRow = start row on CPC screen (0=top row of web-page)
;         Window_FERow = end row on CPC screen (20=bottom row of web-page)
;         Window_FSCol = start column on CPC screen (0=left column)
;         Window_FECol = end column on CPC screen (79=right column)
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    SC_SET_TXTWIN (OS)
;         SC_SEL_STREAM (OS)
;         SC_VDU_ENABLE (OS)
;Notes    1. The Frame stream of the CPC is left in action.
;=========================================================================
WIN_ClearArea
   LD    A,(Stream_Frame)
   CALL  SC_SEL_STREAM			;Select stream for frames
;
   LD    BC,(Window_FSRow)
   LD    A,5				;account for the menu area (5 lines)
   ADD   A,C
   LD    L,A
   LD    BC,(Window_FERow)
   LD    A,5				;account for the menu area (5 lines)
   ADD   A,C
   LD    E,A
;
   LD    BC,(Window_FSCol)
   LD    D,C
   LD    BC,(Window_FECol)
   LD    H,C
   CALL  SC_SET_TXTWIN			;define menu text window
   JP    TXT_CLR_WIN			;clear its contents