;=========================================================================
;
; External RAM Bank Application Routines (HDOS)
; Copyright (c) 2010 Ray Palmer
;
;Notes    1. Assume the HDOS ROM is installed.
;=========================================================================
HDOS_RSX_HDON		DEFM	"HD.O","N"+&80
HDOS_RSX_PSEL		DEFM	"HD.PSE","L"+&80
HDOS_RSX_ISTREAM	DEFM	"HD.ISTREA","M"+&80
HDOS_RSX_STRM		DEFM	"HD.STR","M"+&80
HDOS_RSX_RIN		DEFM	"HD.RI","N"+&80
HDOS_RSX_CD		DEFM	"HD.C","D"+&80
HDOS_RSX_CAT		DEFM	"HD.CA","T"+&80
;
HDOS_RSX_FAR		DEFS	3
;
HDOS_DeviceNum		DEFB	0		;Unit number of device (0-7)
HDOS_DeviceTyp		DEFB	0		;Device type (0-15)
;
HDON_PARAMS		DEFW	&8000		;Boot/block address
			DEFB 	1		;HDOS Device
;
HDISTREAM_PARAMS	DEFB	1		;Read Stream
HDPSEL_PARAMS     	DEFW	HDPSEL_PName
			DEFW	HDPSEL_PPwd
			DEFW	0		;Partition key
;
MAX_PName	EQU	24
MAX_PPass	EQU	14
;
HDPSEL_PName	DEFB	0		;Length
		DEFS	MAX_PName	;partition Name
;
HDPSEL_PPwd	DEFB	0		;Length
		DEFS	MAX_PPass	;Pasrtition Name
;
HDCAT_Parms	DEFW	HDCAT_txt	;Address file filter
		DEFB	0		;Show only filenames
;
HDCAT_txt	DEFB	1
		DEFM	"*"
;
AMSDOS_RSX_DISC	DEFM	"DIS","C"+&80	;ASMDOS RSX |DISC
AMSDOS_RSX_A	DEFM	"A"+&80		;ASMDOS RSX |A
AMSDOS_RSX_B	DEFM	"B"+&80		;ASMDOS RSX |B
;
OD_Temp		DEFW	0
OP_Temp		DEFW	0
OP_Path		DEFB	0
		DEFS	128		;Max path
HDOS_CD_PARAMS	DEFW	OP_Path		;RSX CD Params
;
HDOS_CurrURL	DEFW	0		;Current full URL address
HDOS_NewURL	DEFW	0		;Addr of new URL to adjust by
URL_Size	EQU	128
URL_fileTxt	DEFM	"file://"
;
MAXFILENAME	EQU	13
Cat_DefFilter	DEFB	"*.*"
		DEFB	0
Cat_filterTxt	DEFS	MAXFILENAME+1	;File Filter text
Cat_FileTemp	DEFS	MAXFILENAME+1	;Temp location for filename
;
CurrentDevice	DEFB	0
;
;   CurrentDeviceFS = File systems of logged catalog
;                     0 = AMSDOS
;                     1 = HDOS
;                     2 = MSDOS (FAT)
;
CurrentDeviceFS	DEFB	0		;File system
;
CatSaveLocn	DEFW	0		;Location of catalog items.
CatItemNum	DEFW	0		;Item number
CatLogLocn	DEFW	0
;=========================================================================
;Name     HDOS_SetupRSX
;Purpose  To setup the RSX command
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = address of RSX command
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HDOS_SetupRSX
   CALL  KL_FIND_COMMAND
   LD    A,C
   LD    (HDOS_RSX_FAR+2),A
   LD    (HDOS_RSX_FAR),HL
   RET
;=========================================================================
;Name     HDOS_OpenDevice
;Purpose  To open the AMSDOS/HDOS device to read a file.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = address of URL path/filename
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = Device type (0=AMSDOS, 1=HDOS, 255 = device error)
;         BC = ??
;         DE = Location after device specification.
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars AMSDOS_DR = hardwired address for AMSDOS floppy device
;         AMSDOS_USER = hardwired addresss for user number
;         CurrentDevice = device type selected
;                         0 = AMSDOS
;                         1 = HDOS
;         CurrentDeviceFS = File system
;                           0 = AMSDOS
;                           1 = HDOS
;                          >1 = OTHER (e.g MSDOS FAT32 - Long Filename)
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. To access a device attached to the CPC the parameter passed
;            is formatted after the '\' as follows
;
;            a. AMSDOS standard floppy device - 
;                DriverLetterUserNumber
;                e.g A0, A15, B23
;
;            b. HDOS device
;              DeviceNum$Unit,PartitionName$Password$HexKey
;
;            For format 'a' the use of PartitionName$Password$HexKey may
;            also be used for HDOS formatted floppy discs, but for now
;            only standard AMSDOS formats handled. However, a HDOS device
;            can be a floppy disc, so this should not be a problem.
;=========================================================================
HDOS_OpenDevice
   INC   HL			;Skip '\' after file //
   LD    A,(HL)			;Get device type
   CALL  UTL_tolower
   CP    "a"			;floppy drive A ?
   JR    Z,HDOS_OpenAMSDOS	;Yes
   CP    "b"			;floppy drive B ?
   JR    NZ,HDOS_OpenHDOS	;No (must be HDOS device)
;
HDOS_OpenAMSDOS
;
;  open local floppy drive
;
;  Fix this routine to use the RSX's |A, |B and |USER to avoid hard
;  wired location.
;
   SUB   A,"a"
   LD    (AMSDOS_DR),A		;Setup AMSDOS drive
   INC   HL			;Point to user number
   CALL  UTL_readdec		;Get user numnber
   LD    (AMSDOS_USER),A	;setup user number
   INC   HL			;Goto to path/filename
   LD    (OD_Temp),HL		;setup location of filename for return
   LD    HL,AMSDOS_RSX_DISC
   CALL  HDOS_SetupRSX
   LD    IX,0			;no parameters
   LD    A,0
   RST   3			;Call RSX for |DISC
   DEFW  HDOS_RSX_FAR
   XOR   A			;return AMSDOS device
   LD    (CurrentDevice),A
   LD    (CurrentDeviceFS),A
   LD    DE,(OD_Temp)		;return path/filename location.
   RET
;
HDOS_OpenHDOS
;
;  open local HDOS drive
;
;  Specification is DeviceNum$Unit,PartitionName$Password$HexKey
;
   CALL  UTL_readdec		;Get device Number
   LD    (HDOS_DeviceTyp),a
   INC   HL			;skip "$"
   CALL  UTL_readdec		;Get device Unit Number
   RL    A
   RL    A
   RL    A
   AND   &E0
   LD    (HDOS_DeviceNum),a
   INC   HL			;skip delimter
;
;  Get Partition Name
;
   LD    DE,HDPSEL_PName+1
   LD    B,MAX_PName
   LD    C,0
HDOS_OpenHDOS2a
   LD    A,(HL)
   INC   HL
   CP    "$"			;end of password text?
   JR    Z,HDOS_OpenHDOS2b	;Yes
   LD    (DE),A
   INC   DE
   INC   C
   DJNZ  HDOS_OpenHDOS2a
HDOS_OpenHDOS2b
   LD    A,C
   LD    (HDPSEL_PName),A
   CP    0
   LD    A,255			;bad partition name
   RET   Z
;
;  Get Partition Password
;
   LD    DE,HDPSEL_PPwd+1
   LD    B,MAX_PPass
   LD    C,0
HDOS_OpenHDOS2c
   LD    A,(HL)
   INC   HL
   CP    "$"			;end of password?
   JR    Z,HDOS_OpenHDOS2d	;yes
   LD    (DE),A
   INC   DE
   INC   C
   DJNZ  HDOS_OpenHDOS2c
HDOS_OpenHDOS2d
   LD    A,C
   LD    (HDPSEL_PPwd),A
;
;  Get Partition Key
;
   EX    DE,HL
   CALL  UTL_readhex16		;Get Partition key
   LD    (HDPSEL_PARAMS+4),HL
   LD    (OD_Temp),DE		;save address of path/filename info
;
   LD    A,(HDOS_DeviceTyp)
   LD    B,A
   LD    A,(HDOS_DeviceNum)
   OR    A
   LD    (HDON_PARAMS+2),A	;Setup device unit for HD.ON RSX
;
   LD    HL,HDOS_RSX_HDON
   CALL  KL_FIND_COMMAND
   LD    A,255
   RET   NC				;HDOS not found
   LD    A,C
   LD    (HDOS_RSX_FAR+2),A
   LD    (HDOS_RSX_FAR),HL
   LD    IX,HDON_PARAMS
   LD    A,2
   RST   3				;Call RSX for HD.ON (select device)
   DEFW  HDOS_RSX_FAR
;
   LD    HL,HDOS_RSX_ISTREAM
   CALL  HDOS_SetupRSX
   LD    IX,HDISTREAM_PARAMS
   LD    A,1
   RST   3				;Call RSX for HD.ISTREAM (init RD stream)
   DEFW  HDOS_RSX_FAR
;
   LD    HL,HDOS_RSX_STRM
   CALL  HDOS_SetupRSX
   LD    IX,HDISTREAM_PARAMS
   LD    A,1
   RST   3				;Call RSX for HD.STRM (Select RD stream)
   DEFW  HDOS_RSX_FAR
;
   LD    HL,HDOS_RSX_PSEL
   CALL  HDOS_SetupRSX
   LD    IX,HDPSEL_PARAMS
   LD    A,1
   RST   3				;Call RSX for HD.PSEL (Par. Sel for RD stream)
   DEFW  HDOS_RSX_FAR
;
   LD    HL,HDOS_RSX_RIN
   CALL  HDOS_SetupRSX
   LD    IX,0
   LD    A,0
   RST   3				;Call RSX for HD.RIN (re-direct CAS IN ops)
   DEFW  HDOS_RSX_FAR
;
   LD    A,1			;flag HDOS device.
   LD    (CurrentDevice),A
   LD    DE,(OD_Temp)
   RET
;=========================================================================
;Name     HDOS_LastSlash
;Purpose  To goto the last slash char in the URL text.
;In Regs  AF = ??
;         BC = ??
;         DE = address at start of URL where search begins.
;         HL = address within URL text.
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         B = flag of slash detection (1=yes else 0)
;         C = ??
;         DE = address of char after last slash in URL
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars None
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HDOS_LastSlash
   LD    H,D
   LD    L,E
   LD    B,0
HDOS_LastSlash1
   LD    A,(HL)
   INC   HL
   CP    0				;end of current URL?
   RET   Z				;Yes
   CP    "/"				;Slash in URL?
   JR    NZ,HDOS_LastSlash1		;No, do next char in URL
   LD    D,H				;DE = addr of char after slash
   LD    E,L
   LD    B,1				;flag found a slash
   JR    HDOS_LastSlash1
;=========================================================================
;Name     HDOS_OpenPath
;Purpose  To open the path on a HDOS device to read a file.
;In Regs  AF = Media type (0=AMSDOS, 1=HDOS)
;         BC = ??
;         DE = Address of the path and filename.
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  CurrentDevice = current device being accesssed.
;Out Regs AF = ??
;         BC = ??
;         DE = Location of last slash in path/filename
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. Assume the HDOS ROM is installed.
;=========================================================================
HDOS_OpenPath
   OR    A				;AMSDOS Device?
   JR    NZ,HDOSOpenPath0	;No
;
;  Open AMSDOS device and skip any path info to filename
;
;   CALL  HDOS_ResetDevice	;Re-log disk in drive
   JP    HDOS_LastSlash		;DE = char after last slash in URL
;
HDOSOpenPath0
   LD    (OD_Temp),DE		;Save current path/filename address
   CALL  HDOS_LastSlash		;DE = address of filename
   LD    A,B
   OR    A			;Any slashes detected?
   JR    Z,HDOS_OpenPath1	;No
;
   LD    (OP_Temp),DE		;save address of char after last slash
   LD    HL,(OD_Temp)		;get current start of path
   SBC   HL,DE			;calc path length
   LD    A,L
   LD    (OP_Path),A		;Setup path length for RSX
   LD    C,L
   LD    B,0
   LD    HL,(OD_Temp)
   LD    DE,OP_Path+1
   LDIR				;Insert path text for RSX
   XOR   A
   LD    (DE),A			;Ensure path is NUL terminated for RSX
;
   LD    HL,HDOS_RSX_STRM
   CALL  HDOS_SetupRSX		;Setup HD.STRM RSX Command
   LD    IX,HDISTREAM_PARAMS
   LD    A,1
   RST   3				;Call RSX for HD.STRM (Select RD stream)
   DEFW  HDOS_RSX_FAR
;
   LD    HL,HDOS_RSX_CD
   CALL  HDOS_SetupRSX		;Find HD.CD RSX command
   LD    IX,HDOS_CD_PARAMS
   LD    A,1
   RST   3				;Call RSX for HD.CD
   DEFW  HDOS_RSX_FAR
;
HDOS_OpenPath1
   LD    DE,(OP_Temp)
   RET
;=========================================================================
;Name     HDOS_SkipSlashes
;Purpose  To go to specified slash in the current URL
;In Regs  AF = ??
;         B = number of slashes to skip
;         DE = ??
;         HL = address of current URL text
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         B = ??
;         C = size of current URL path text
;         DE = location of char after the nth slash
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars 
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HDOS_SkipSlashes
   LD    C,120				;MAX URL text after http // or file //
   LD    DE,0
HDOS_SkpSlshes1
   LD    A,(HL)				;Get URL char
   CP    0					;end of current URL?
   RET   Z					;Yes
   INC   HL
   DEC   C
   CP    '/'				;Slash in URL?
   JR    NZ,HDOS_SkpSlshes1		;No, do next char in URL
   LD    D,H				;track char after slash
   LD    E,L
   DJNZ  HDOS_SkpSlshes1		;Do again until num slashes found
   RET
;=========================================================================
;Name     HDOS_CheckPath
;Purpose  To process the URL path for the reference type and to calculate
;         the number of slashes in the path.
;In Regs  AF = ??
;         BC = ??
;         DE = Address of current URL
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         B = Total number of slashes in URL
;         C = minimum slashes in a URL to get to path info.
;             3 = web reference, 2 = local file reference.
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars 
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. URL handling is as follows -
;            a. for a web reference the web address follows the web tag.
;               e.g http //www.test.com/path1/path2/object1.txt
;            b. for a local file reference there is no web reference, so
;               the URL would like file //path1/path2/object2.txt
;=========================================================================
HDOS_CheckPath
   PUSH  DE
   LD    HL,URL_fileTxt
   LD    B,4
   CALL  UTL_strncmp			;is current URL a file reference?
   POP   HL					;HL = address of current URL
   LD    B,0
   LD    C,2				;Number of slashes in file reference
   JR    Z,HDOS_ChkPath1		;Yes
   LD    C,3				;Number of slashes in web reference
;
HDOS_ChkPath1
   LD    A,(HL)
   INC   HL
   OR    A
   RET   Z
   CP    '/'				;Slash in current path?
   JR    NZ,HDOS_ChkPath1		;No
   INC   B					;Tally number of slashes in curr path
   JR    HDOS_ChkPath1
;=========================================================================
;Name     HDOS_BuildURLPath
;Purpose  To build the URL path.
;In Regs  AF = ??
;         BC = ??
;         DE = Address for new URL
;         HL = Address of current URL
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. URLs can be one of 3 forms -
;            a. relative reference from the base location.
;               e.g i.   "path1/file1.txt"
;                   ii.  "../file1.txt"
;                   iii. "../path2/file2.txt"
;                   iv.  "./path2/file2.txt"
;               The handling of the path information in the URL is
;               treated in the same manner as file manager handling of
;               directories.
;
;            b. absolute reference from base location.
;               e.g "/path1/index.htm"
;               if current path is http //www.text.com/path0/f.txt
;               then the new URL will be
;               http //www.text.com/path1/file.txt
;
;            c. A new base location to access.
;               e.g "http //www.fred.com/file1.txt"
;
;         2. Assume the complete abs reference is used to open a URL
;            that is not a reference to the current url.
;            i.e must have http //..... or file //......
;
;         3. Need to fix to handle references like 'www.fred.com/f.htm'
;            and '1.2.3.4/f.htm' to be internet rather than just files as
;            the first char is not a slash.
;
;         4. The URL cannot have a # in the path as it would be handled
;            as a label start (e.g www.myisp.com/#path/file.htm) and so
;            the URL is truncated at that point. Only the last # is of
;            concern rather than any others existing before it.
;
;         5. The use of the https prefix for secure communication has not
;            yet been implemented, however the path can be handled.
;
;         6. If the current URL is not defined then a relative path is
;            is assumed to be a file reference.
;
;=========================================================================
HDOS_BuildURLPath
   LD    (HDOS_CurrURL),DE		;Save address of current URL
   LD    (HDOS_NewURL),HL		;Save source location of URL text.
;
   LD    B,5
   LD    C,128
HDOS_BldURLPath0
   LD    A,(HL)
   CP    ':'				;Full URL path passed in?
   JR    Z,HDOS_BldURL2			;Yes (pass as is)
   INC   HL
   DJNZ  HDOS_BldURLPath0
;
   LD    A,(DE)				;get 1st char of current URL
   CP    0				;Is current URL defined?
   JR    Z,HDOS_BldURL5			;No (assume file is reference)
;
   CALL  HDOS_CheckPath			;B= number of slashs, C = min slashes to path
;
   LD    HL,(HDOS_NewURL)
   LD    A,(HL)				;get first char in new URL
   CP    '/'				;Is new URL an Absolute ref?
   JR    Z,HDOS_BldURL1a		;Yes
   CP    '.'				;Is new URL a relative ref?
   JR    NZ,HDOS_BldURL1b		;No, Append to last '/' in curr url
   LD    A,(HL)
   CP    '.'				;Is reference '..'?
   JR    NZ,HDOS_BldURL1b		;No, Append to last '/' in curr url
   INC   HL
   DEC   B
   JR    HDOS_BldURL1b			;Yes (skip 2 chars)
;
HDOS_BldURL5
   LD    HL,URL_fileTxt
   LD    BC,7
   LDIR					;Insert 'file //' start
   LD    C,121
   JR    HDOS_BldURL2			;Insert rest of URL
;
HDOS_BldURL1a
   INC   HL
   LD    B,C
HDOS_BldURL1b
   LD    (HDOS_NewURL),HL		;setup start from non-path char
   LD    HL,(HDOS_CurrURL)		;get path of current URL
   CALL  HDOS_SkipSlashes		;DE = char address after nth slash
;
HDOS_BldURL2
   LD    B,C				;B = max space for rest of the url
   LD    HL,(HDOS_NewURL)
   PUSH  DE
   CALL  utl_strncpy			;Insert new URL path text
   POP   DE
;
HDOS_BldURL3
   LD    A,(DE)
   CP    '"'
   JR    Z,HDOS_BldURL3b
   CP    0
   JR    Z,HDOS_BldURL3b
   INC   DE
   JR    HDOS_BldURL3
;
HDOS_BldURL3b
   XOR   A
HDOS_BldURL4
   LD    (DE),A				;Fill rest of URL object with nulls
   INC   DE
   DJNZ  HDOS_BldURL4
   RET
;=========================================================================
;Name     HDOS_ResetDevice
;Purpose  To reset the device so that the system knows what is being
;         accessed.
;In Regs  AF = Device Type (0=AMSDOS, 1=HDOS)
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  CurrentDevice = current device being accessed.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HDOS_RSX_FAR = updated with RSX command data.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HDOS_ResetDevice
   OR    A				;ASMDOS Device?
   RET   NZ				;No
;
   LD    HL,AMSDOS_RSX_A
   LD    A,(CurrentDevice)
   OR    A				;Floppy drive A?
   JR    Z,HDOS_ResetDev1	;Yes
   LD    HL,AMSDOS_RSX_B
HDOS_ResetDev1
   CALL  HDOS_SetupRSX
   LD    IX,0			;no parameters
   LD    A,0
   RST   3				;Call RSX |A or |B
   DEFW  HDOS_RSX_FAR
   RET
;=========================================================================
;Name     HDOS_SetupFilter
;Purpose  To setup the file catalog filkter
;In Regs  AF = ??
;         BC = ??
;         DE = Address of filter text
;         HL = TBD
;         IX = ??
;         IY = ??
;In Vars  Cat_DefFilter = initial filter specification
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars ??
;Uses     TBD
;Call By  CPCB.EXE
;Calls    TBD
;Notes    TBD
;=========================================================================
HDOS_SetupFilter
   EX    DE,HL
   LD    DE,CAT_FilterTxt
   LD    A,(HL)			;get first char of filter text
   OR    A			;is it empty string?
   JR    NZ,HDOS_CatDev1	;No
   LD    HL,Cat_DefFilter	;Use default for AMSDOS
   LD    A,(CurrentDeviceFS)
   OR    A			;AMSDOS Device?
   JR    Z,HDOS_CatDev1		;Yes
   INC   HL
   INC   HL
HDOS_CatDev1
   CALL  UTL_strcpy		;copy filter text for later use
;
;  Assume all wild character letters
;
   LD    B,MAXFILENAME
   LD    HL,Cat_FileTemp
HDOS_SetFilter1
   LD    (HL),'?'
   INC   HL
   DJNZ  HDOS_SetFilter1
;
   LD    HL,Cat_filterTxt
   LD    DE,Cat_FileTemp
   LD    B,MAXFILENAME
   LD    A,(CurrentDeviceFS)
   OR    A			;AMSDOS Device?
   JR    NZ,HDOS_SetFilter1a	;No
   INC   DE
;
HDOS_SetFilter1a
   LD    A,(HL)
   OR    A			;end of filter string?
   JR    Z,HDOS_SetFilter7	;Yes
;
   CP    '*'			;Is input a wildcard string char?
   JR    NZ,HDOS_SetFilter2b	;No
   LD    C,255
HDOS_SetFilter2
   INC   C			;count number of chars left
   INC   HL			;next char position
   LD    A,(HL)
   OR    A			;At end of input?
   JR    NZ,HDOS_SetFilter2	;No
;
   LD    A,C
   OR    A			;no further chars in filter?
   JR    Z,HDOS_SetFilter7	;Yes
;
;  Go back from end to new start position
;
   LD    B,C
   LD    DE,Cat_FileTemp+MAXFILENAME
HDOS_SetFilter2a
   DEC   DE
   DEC   HL
   DJNZ  HDOS_SetFilter2a
   LD    B,C
   JR    HDOS_SetFilter6
;
HDOS_SetFilter2b
   CALL  UTL_tolower
   LD    (DE),A
   INC   DE
   INC   HL
HDOS_SetFilter6
   DJNZ  HDOS_SetFilter1a
;
HDOS_SetFilter7
   LD    HL,Cat_FileTemp
   LD    DE,Cat_filterTxt
   JP    UTL_strcpy
;=========================================================================
;Name     HDOS_CatDevice
;Purpose  To get the catalog of files from a device.
;In Regs  AF = ??
;         BC = address of file filter text
;         DE = Address of AMSDOS 2k buffer.
;         HL = Address of saved logged data to clear.
;         IX = ??
;         IY = ??
;In Vars  Cat_DefFilter = initial filter specification
;Out Regs A = 255 catalog failed by HDOS as ROM not installed.
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars CatLogLocn = location of catalogged data
;Uses     TBD
;Call By  CPCB.EXE
;Calls    TBD
;Notes    1. Upon execution the code bank #0 is swapped in.
;=========================================================================
HDOS_CatDevice
   LD    (CatLogLocn),HL	;Location of data
HDOS_CatDevice0
   LD    (HL),0
   INC   HL
   BIT   7,H			;at end banked memory?
   JR    Z,HDOS_CatDevice0	;No
;
   LD    A,(CurrentDevice)
   OR    A
   JP    Z,CAS_CATALOG		;catalog AMSDOS device.
;
;  Catalog device using HDOS command |HD.CAT,"*",255
;  Assume the current HDOS device is already initialised.
;
   LD    HL,HDOS_RSX_CAT
   CALL  HDOS_SetupRSX
   LD    A,255			;catalog failed
   RET   NC			;HDOS command not found
   LD    IX,HDCAT_PARMS		;location of parameters
   LD    A,2			;Number of parameters to command
   RST   3			;Call RSX for HD.ON (select device)
   DEFW  HDOS_RSX_FAR
   RET
;=========================================================================
;Name     HDOS_MatchFile
;Purpose  To check if the extracted file matches the file filter text.
;In Regs  AF = ??
;         B = number of characters to check
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  Cat_DefFilter = filter specification.
;         Cat_FileTemp = catalog filename entry
;Out Regs A = NZ flag set if mismatch else Z flag
;         B = number of remaining characters mismatched (0=all matched)
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     All registers
;Call By  HDOS_GetCatItem
;Calls    TBD
;Notes    NEED TO FIX ME
;=========================================================================
HDOS_MatchFile
   LD    DE,Cat_filterTxt
   LD    HL,Cat_FileTemp
   LD    B,MAXFILENAME
   LD    A,(CurrentDeviceFS)
   OR    A			;AMSDOS catalog?
   JR    NZ,HDOS_MtchFile1
   DEC   B
   INC   DE
HDOS_MtchFile1
   LD    A,(DE)
   CP    '?'
   JR    Z,HDOS_MtchFile2
   CALL  UTL_tolower
   LD    C,A
   LD    A,(HL)
   CALL  UTL_tolower
   CP    C
   RET   NZ
HDOS_MtchFile2
   INC   HL
   INC   DE
   DJNZ  HDOS_MtchFile1
   RET
;=========================================================================
;Name     HDOS_GetCatItem
;Purpose  To get the nth catalog item.
;In Regs  AF = ??
;         BC = Nth item to get.
;         DE = Address of filter text
;         HL = Address to save filename of nth entry
;         IX = ??
;         IY = ??
;In Vars  Cat_DefFilter = filter specification.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. The filter specification guides which item's are counted.
;         2. The catalog of the AMSDOS device is in full width screen
;            mode so that the format is well structured for extraction.
;=========================================================================
HDOS_GetCatItem
   LD    (HL),0			;Assume no filename got
   LD    (CatSaveLocn),HL	;Where to save to
   INC   BC
   LD    (CatItemNum),BC	;Nth item to get
;
   CALL  HDOS_SetupFilter
;
   LD    A,(CurrentDeviceFS)
   OR    A			;AMSDOS catalog?
   JR    NZ,HDOS_GetCatItemHDOS
;
;  Get Nth item in a standard AMSDOS Catalog
;
   LD    HL,(CatLogLocn)
   LD    B,3			;skip first three CR/LF items
HDOS_GetCatItem0a
   LD    A,(HL)
   INC   HL
   CP    &0A			;line feed from CR/LF (&0D/&0A)
   JR    NZ,HDOS_GetCatItem0a	;No
   DJNZ  HDOS_GetCatItem0a
   JR    HDOS_GetCatItem0
;
HDOS_GetCatItem2a
   LD    DE,17			;Size of each cat entry
   ADD   HL,DE			;Skip cat entry
   LD    A,(HL)
   CP    &0D			;CR following information?
   JR    Z,HDOS_GetCatItem2b	;Yes
   INC   HL			;Skip first space
HDOS_GetCatItem2b
   INC   HL			;skip &0D/&0A or next 2 spaces
   INC   HL
HDOS_GetCatItem0
   PUSH  HL
   LD    DE,Cat_FileTemp
   LD    BC,MAXFILENAME
   LDIR				;copy entry to temp
;
;  Check if filter pattern matches
;
   LD    A,(Cat_FileTemp)
   OR    A			;was a non-empty entry got
   CALL  NZ,HDOS_MatchFile	;yes check if matches filter spec
   POP   HL
   JR    NZ,HDOS_GetCatItem2a	;No match, get next entry
;
   LD    A,(Cat_FileTemp)
   OR    A			;was entry invalid?
   RET   Z			;Yes, return blank entry
;
   LD    BC,(CatItemNum)
   DEC   BC
   LD    (CatItemNum),BC
   LD    A,B			;At required cata entry?
   OR    C
   JR    NZ,HDOS_GetCatItem2a	;No, get next
;
   LD    HL,Cat_FileTemp
   LD    DE,(CatSaveLocn)
   LD    BC,MAXFILENAME
   LD    A,(CurrentDeviceFS)
   OR    A			;AMSDOS catalog?
   JR    NZ,HDOS_GetCatItem3
   DEC   BC			;size of ASMDOS filename
HDOS_GetCatItem3
   LDIR				;get AMSDOS filename/extension
   XOR   A
   LD    (DE),A
   RET
HDOS_GetCatItemHDOS
;
;  Get HDOS entry item
;
   RET