;=========================================================================
;
; External RAM Bank Application Routines (URL)
; Copyright (c) 2010 Ray Palmer
;
;=========================================================================
HTML_I_FORM		EQU	15		;also is mask
HTML_I_BUTTONC		EQU	13
HTML_I_OPTION		EQU	12
HTML_I_OPTGRP		EQU	11
HTML_I_SELECT		EQU	10
HTML_I_TXTBOX		EQU	9
HTML_I_Chkbox		EQU	0
HTML_I_Radio		EQU	1
HTML_I_Text		EQU	2
HTML_I_Password		EQU	3
HTML_I_Submit		EQU	4
HTML_I_Reset		EQU	5
HTML_I_File		EQU	6
HTML_I_Image		EQU	7
HTML_I_Button		EQU	8
;
;=========================================================================
FormsAddType		DEFB	0	;Flag to mark Post (0) or Get (1)
FormsDBLoc		DEFW	0	;Location of Forms DB entry in main memory
FormsDBText		DEFW	0	;location of associated text for Forms DB
FormsDBHTML		DEFW	0	;location of associated HTML text for Forms DB
FormsURLType		DEFB	0	;Type of URL (0=post, 1=get)
;
FormsNameStr		DEFM	"name"
			DEFB	0
FormsLablStr		DEFM	"label"
			DEFB	0
FormsValuStr		DEFM	"value"
			DEFB	0
FormsIDStr		DEFM	"id"
			DEFB	0
FormsActnStr		DEFM	"action"
			DEFB	0
FormsMthdStr		DEFM	"method"
			DEFB	0
FormsNameVal		DEFS	65
;
CurrURLIx		DEFW	0	;Current Index into URL text
CurrURLIx1		DEFW	0	;Previous Index into URL text
CurrURLIx2		DEFW	0	;Current Index into URL text
;
Local_URL		EQU	&7000	;4K Space for the URL to be built
NetConnect_Status	DEFB	0
;
CONVCODES		DEFM	"<>{}|\^~[]`%#"
			DEFB	0
;=========================================================================
;Name     URL_NetAccess
;Purpose  To change the status of the connection to the communication
;         device
;In Regs  AF = status of TCP/IP interface
;              0 = Disconnected, 1 = Connected
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars NetConnect_Status = status byte
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
URL_NetAccess
   LD    (NetConnect_Status),A
   RET
;=========================================================================
;Name     URL_OpenHTTP
;Purpose  To open the HTTP item and download the item to a local file.
;In Regs  AF = status of TCP/IP interface
;              0 = failed, 1 = okay
;         BC = ??
;         DE = Address of location/name to save downloaded object.
;         HL = Address of URL object.
;               URL_ObjStrtTxt -> web URL of item to access.
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_Buffer = name of file assigned to the downloaded HTTP item.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. The API interface into the TCP/IP stack has already been
;            initialised and ready.
;         2. The use of the TCP/IP has no affect on the routines within
;            this module since they are executed from within the TCP/IP
;            rom. However they must restore the memory to its state
;            before the API call since if the swap space is not restored
;            the API call would return to the wrong place.
;=========================================================================
URL_OpenHTTP
;
;  Open a socket and get the HTML file from the URL
;  First query the DNS for the base URL.
;  (i.e DNS the web address from the url http //'web address'/....)
;  Now 'web address' is http //a.b.c.d/......
;  
   RET
;=========================================================================
;Name     URL_SendURL
;Purpose  To issue URLs for the forms SUBMIT action. The routine is also
;         capable of handling the opposite functions to SUBMIT which is
;         the 'get' function to get data for a form as preset data.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. The issuing of URLs to handle the 'get' and 'post' functions
;            are limited to the size of the ethernet packet size of 1500
;            bytes.
;=========================================================================
URL_SendURL
   LD    DE,(FormsDBLoc)
   LD    HL,Local_URL
   JP    UTL_strcpy
;=========================================================================
;Name     URL_Add2URL
;Purpose  To add elements of a form submission to the URL before being
;         sent.
;In Regs  AF = what to add to the URL.
;              0 = not used
;              1 = add URL web address/path
;              2 = add Final item to URL path for submission (e.g
;                  submit.cgi)
;              3 = add forms item to URL for submit.
;         BC = Address of HTML command text
;         DE = Address of text (if required)
;         HL = address of downloaded Forms database entry
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  HTML ROM - routine HTM_FormSubmitData (html_forms.asm)
;Calls    TBD
;Notes    1. This routine performs the required encoding to match what the
;            URL is required to be. For a secured submissions like HTTPS,
;            this routine cannot as yet be used. It is primarily for
;            submission's to be only via the URL as per the HTML spec. The
;            inclusion of object content like images is yet to be built,
;            which also means that only simple URLs can be built.
;         2. All data passed into this routine is within the AMSDOS 2K
;            buffer.
;=========================================================================
URL_Add2URL
   LD    (FormsDBLoc),HL
   LD    (FormsDBText),DE
   LD    (FormsDBHTML),BC
   CP    1			;Handle <FORM> and </FORM>
   JR    Z,URL_Add2URL1
   CP    2			;Add filename to URL
   JP    Z,URL_Add2URL2
   CP    3			;Add forms item to URL
   JP    Z,URL_Add2URL3
   CP    4			;Handle name from 'select'
   JP    Z,URL_Add2URL4
   RET
;
URL_Add2URL1
;
;  Download the web URL including path and filename for the action
;
   LD    A,(BC)
   CP    '/'			;end of form data?
   RET   Z			;Yes
   XOR   A
   LD    (FormsAddType),A
   LD    DE,FormsActnStr
   LD    B,6
   CALL  URL_GotoValue
   LD    DE,Local_URL
   LD    C,'"'
   CALL  UTL_strccpy		;insert Start of URL
   LD    (CurrURLIx),DE
;
;  Assess what sort of action is to take place 'GET' or 'POST'
;
   LD    DE,FormsMthdStr
   LD    B,6
   CALL  URL_GotoValue
   LD    A,(HL)
   CALL  UTL_tolower
   CP    'p'			;is method 'POST'?
   RET   Z			;Yes
   LD    HL,FormsAddType
   SET   7,(HL)			;Flag method is get
   RET
;
URL_Add2URL2
;
;  Insert filename to end of URL
;
   EX    DE,HL
   LD    DE,(CurrURLIx)		;DE = end address of URL text
URL_Add2URL2a
   LD    A,(DE)
   CP    '/'			;at last '/' in path?
   JR    Z,URL_Add2URL2b	;Yes
   DEC   HL			;go back one more char
   JR    URL_Add2URL2a		;check this char for last slash
;
URL_Add2URL2b
   INC   DE
   CALL  UTL_strcpy		;insert filename after last slash
   DEC   DE			;point to end char (non-zero)
   LD    (CurrURLIx),DE
   RET
;
URL_Add2URL4
;
;  Get name of value from 'SELECT'
;
   LD    HL,FormsAddType
   SET   5,(HL)
   LD    A,(BC)
   CP    '/'
   JR    Z,URL_Add2URL4a
   SET   6,(HL)
   RES   5,(HL)
   LD    DE,(CurrURLIx)
   LD    (CurrURLIx2),DE
   LD    DE,FormsNameStr
   LD    B,4
   CALL  URL_GotoValue		;goto value of the 'name' parameter
   LD    DE,FormsNameVal
   LD    C,'"'
   CALL  URL_CopyStr		;insert name text
   XOR   A
   LD    (DE),A
   RET
URL_Add2URL4a
   RES   6,(HL)
   LD    HL,(CurrURLIx)
   DEC   HL
   LD    A,(HL)
   CP    '='			;Was any option inserted?
   RET   NZ			;Yes (all okay)
   LD    HL,(CurrURLIx2)
   LD    (CurrURLIx),HL		;No (remove name)
   LD    (HL),0
   RET
;
URL_Add2URL3
;
;  Add a forms item to the URL
;  1. A forms item may -
;     a. insert the method URL
;     b. the parameters to the URL for successful controls.
;
   LD    A,(HL)			;Get forms type
   AND   HTML_I_FORM		;mask of forms type
   CP    HTML_I_OPTGRP		;options grouping?
   RET   Z			;Yes (ignore them)
   EX    DE,HL
   LD    HL,S_HTMLVarVal
   ADD   HL,DE
   BIT   6,(HL)			;Is item active?
   RET   Z			;No
;
;  look at HTML command for name and value if active.
;
   LD    E,'&'
   LD    A,(FormsAddType)
   BIT   0,A			;first parameter processed
   JR    NZ,URL_Add2URL5	;yes
;
   SET   0,A			;flag first parameter now passed
   LD    (FormsAddType),A
;
   BIT   7,A			;method = 'POST'
   JR    Z,URL_Add2URL5		;Yes
   LD    E,'?'
;
URL_Add2URL5
   LD    HL,(CurrURLIx)
   LD    (HL),E			;insert paremeter delimiter
   INC   HL
   LD    (CurrURLIx),HL
;
   LD    HL,FormsNameVal	;Address of 'SELECT' name
   BIT   6,A			;form item already have a name?
   JR    NZ,URL_Add2URL5a	;Yes
;
   LD    DE,FormsNameStr
   LD    B,4
   CALL  URL_GotoValue		;goto value of the 'name' parameter
   OR    A			;found value of 'name' parameter?
   JR    NZ,URL_Add2URL5a	;Yes
   LD    DE,FormsIDStr
   LD    B,2
   CALL  URL_GotoValue		;goto value of the 'id' parameter
;
URL_Add2URL5a
   LD    DE,(CurrURLIx)
   LD    (CurrURLIx1),DE
   LD    C,'"'
   CALL  URL_CopyStr		;insert name text
   LD    A,'='
   LD    (DE),A			;insert '='
   INC   DE
   LD    (CurrURLIx),DE
;
   LD    HL,(FormsDBLoc)
   LD    A,(HL)			;Get form item type?
   AND   HTML_I_FORM
   CP    HTML_I_TEXT		;text entry?
   JR    Z,URL_Add2URL6		;Yes
;
   CP    HTML_I_TXTBOX		;multi-line text ?
   JR    NZ,URL_Add2URL7	;no
;
URL_Add2URL6
   LD    HL,(FormsDBText)
   LD    A,(HL)
   OR    A			;Any text to insert?
   JR    NZ,URL_Add2URL8	;Yes
   LD    HL,(CurrURLIx1)	;no (cancel name)
   DEC   HL
   LD    (CurrURLIx),HL
   LD    (HL),0
   RET
;
URL_Add2URL7
   LD    DE,FormsValuStr
   LD    B,5
   CALL  URL_GotoValue		;goto value of the 'value' parameter
;
URL_Add2URL8
   LD    DE,(CurrURLIx)
   LD    C,'"'
   CALL  URL_CopyStr		;insert value for name
   XOR   A
   LD    (DE),A
   LD    (CurrURLIx),DE
   RET
;=========================================================================
;Name     URL_CopyStr
;Purpose  To copy the value of the parameter into the URL
;In Regs  AF = ??
;         B = ??
;         C = end of string character/byte
;         DE = address in URL
;         HL = address of the parameter name value.
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = address of the parameter name value.
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
URL_CopyStr
   LD    A,(HL)
   AND   &7F
   CP    C			;end of string?
   RET   Z			;yes
   OR    A			;EOT?
   RET   Z			;Yes
;
   CP    ' '			;space char
   JR    NZ,URL_CopyStr1	;no
   LD    A,'+'			;replace with '+'
URL_CopyStr1
   PUSH  HL
   CP    32			;Less then 32?
   JR    C,URL_CopyStr4		;Yes (convert to hex)
;
   LD    HL,CONVCODES
   LD    B,A
URL_CopyStr3
   LD    A,(HL)
   OR    A			;end of search?
   JR    Z,URL_CopyStr2		;Yes (accept char as is)
   CP    B			;matches char to convert?
   JR    Z,URL_CopyStr4		;yes
   INC   HL
   JR    URL_CopyStr3
URL_CopyStr4
   PUSH  AF
   SRL   A
   SRL   A
   SRL   A
   SRL   A
   CALL  UTL_Hex2Char
   LD    B,A
   LD    A,'%'
   LD    (DE),A
   INC   DE
   LD    A,B
   LD    (DE),A
   INC   DE
   POP   AF
   CALL  UTL_Hex2Char
   LD    B,A
URL_CopyStr2
   LD    A,B
   POP   HL
   INC   HL
   LD    (DE),A
   INC   DE
   JR    URL_CopyStr
;=========================================================================
;Name     URL_GotoValue
;Purpose  To goto the parameter value.
;In Regs  AF = ??
;         B = text length
;         C = ??
;         DE = address of parameter name to find.
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = Double quote if found else 0
;         BC = ??
;         DE = ??
;         HL = address of the parameter name value.
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. Assume that the text is not abnormally terminated.
;=========================================================================
URL_GotoValue
   LD    HL,(FormsDBHTML)	;start address of HTML command
URL_GotoValue1
   LD    A,(HL)
   OR    A			;end of command?
   RET   Z			;Yes
;
   CALL  UTL_skipspace		;skip spaces to parameter name
   PUSH  DE
   PUSH  BC
   CALL  UTL_strncmp		;matches parameter name?
   POP   BC
   POP   DE
   JP    Z,URL_FindDblQuote	;Yes (goto to value for it)
;
   CALL  UTL_skiptospace	;goto first space
   CALL  UTL_skipspace		;skip spaces
   LD    A,(HL)
   CP    '='			;has 'parameter = value' format
   JR    NZ,URL_GotoValue1	;No
;
   CALL  URL_FindDblQuote	;goto to value for parameter
   CALL  URL_FindDblQuote	;goto end of parameter
   JR    URL_GotoValue1
;=========================================================================
;Name     URL_FindDblQuote
;Purpose  To find a double quote
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = address of text
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = address of double quote
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
URL_FindDblQuote
   LD    A,(HL)
   INC   HL
   CP    '"'
   JR    NZ,URL_FindDblQuote
   RET
