;=========================================================================
;
; Application Routines (UTL)
; Copyright (c) 2010 Ray Palmer
;
;=====================================================================
;Name     UTL_isdigit
;Purpose  Carry = !isdigit (A)
;Input    A = character to check.
;Output   Carry flag set if A is not between 0 and 9 (inclusively)
;Uses     Flags corrupted.
;Notes    None.
;=====================================================================
UTL_isdigit
   cp    '0'
   ret   c			;A is < '0'
   cp    '9'+1
   ccf			;set C flag if A > '9'
   ret
;=====================================================================
;Name     UTL_Hex2Char
;Purpose  To convert a value 0-15 to ha hexadecimal character
;Input    A = value to insert
;Output   A = hexadecimal character
;Uses     AF corrupted.
;Notes    None
;=====================================================================
UTL_Hex2Char
   AND   &0f
   cp   10			;Is A > 10
   jr   c,Hex2Char1		;No
   add  a,'a' - '0' - 10	;Yes, adjust for Hex char
Hex2Char1
   add  a,'0'			;offset to numbers
   RET
;=====================================================================
;Name     UTL_hexdigit
;Purpose  A = !util_hexdigit (A)
;Input    A = character to check.
;Output   A = value (0-15 ok, 16=invalid)
;Uses     Flags corrupted.
;Notes    None.
;=====================================================================
UTL_hexdigit
   CALL  UTL_isdigit			;Is valid numerical digit?
   JR    C,UTL_hexdigit_1		;No (check if hex letter)
   sub   '0'
   RET
;
UTL_hexdigit_1
   CALL  UTL_tolower
   CP    'a'				;is char < 'a'?
   JR    C,UTL_hexdigit_2		;Yes
   sub   'a'+10				;A = 10 to 15 for a to F
   LD    B,A
   AND   &f0				;Is A > 15?
   JR    NZ,UTL_hexdigit_2		;Yes (error)
   LD    A,B
   RET
UTL_hexdigit_2
   LD    A,16
   ret
;=====================================================================
;Name     UTL_readdec_add
;Purpose  To update the 8 bit conversion result.
;Input    A = digit character.
;         B = current converted result.
;Output   B = updated converted result
;Uses     F flag corrupted.
;Notes    None
;=====================================================================
UTL_readdec_add
   sub    '0'			;Convert to digit char to number
   ld     c,a			;D = converted number
;
;  Multuply previous result by 10
;
   ld     a,b
   add    a,a			;A = E * 2
   add    a,a			;A = E * 4
   add    a,b			;A = E * 5
   add    a,a			;A = E * 10
;
;  Add in next converted number
;
   add    a,c			;A = (E * 10) + D
   LD     B,A			;Save current result
   RET
;=====================================================================
;Name     UTL_readdec
;Purpose  Read a decimal string in the range 0-255
;Input    HL = source string of decimal number characters
;Output   A = value read
;         HL = next charcter after string cnverted
;Uses     F flag corrupted.
;Notes    1. Only a maximum of 3 digit characters converted.
;         2. If string is > 255 then result is uncertain.
;=====================================================================
UTL_readdec
   push   bc
   LD     B,0			;B = current result.
UTL_rddec_loop
   ld     a,(HL)
   INC    HL			;Next digit string character
   call   UTL_isdigit		;Valid digit character?
   jr     c,UTL_rddec_done	;No (end of string)
   CALL   UTL_readdec_add	;Update result
   JR     UTL_rddec_loop
UTL_rddec_done
   LD     A,B
   pop    bc
   ret
;=====================================================================
;Name     UTL_readhex8
;Purpose  Read a hexidecimal string in the range 0-FF
;Input    HL = source string of decimal number characters
;Output   A = result
;         HL = address of first non-digit character.
;Uses     AF corrupted.
;Notes    1. If string is > 255 then result is uncertain.
;=====================================================================
UTL_readhex8
   push  bc
   LD    B,0
UTL_rdhex8_loop
   ld    a,(HL)
   inc   HL
   PUSH  BC
   call  UTL_hexdigit
   POP   BC
   cp    16				;Is character a hex digit?
   jr    Z,UTL_rdhex8_done	;No (end of numerical string)
   ld    c,a
   ld    a,b
   add   a,a
   add   a,a
   add   a,a
   add   a,a
   add   a,c
   ld    b,a
   jr    UTL_rdhex8_loop
UTL_rdhex8_done
   ld    a,b
   pop   bc
   ret
;=====================================================================
;Name     UTL_readdec2_add
;Purpose  To update the 16 bit conversion result.
;Input    A = digit character.
;         HL = current converted result.
;Output   HL = updated converted result
;Uses     F flag corrupted.
;Notes    None
;=====================================================================
UTL_readdec2_add
   sub   '0'			; A contains next digit to shift in
;
;  Multuply previous result by 10
;
   ld    b,h			; BC = Current result
   ld    c,l
   add   hl,hl			; HL = 2 * BC
   add   hl,hl			; HL = 4 * BC
   add   hl,bc			; HL = 5 * BC
   add   hl,hl			; HL = 10 * BC
;
;  Add in next converted number
;
   ld    b,0
   ld    c,a
   add   hl,bc			; HL = (10 * BC) + A
   RET
;=====================================================================
;Name     UTL_readdec2
;Purpose  Read a decimal in the range 0-65535
;Input    DE = source string of decimal number characters
;Output   HL = result
;         DE = address of first non-digit character.
;Uses     AF corrupted (has invalid digit)
;Notes    1. If string is > 65535 then result is uncertain.
;=====================================================================
UTL_readdec2
   push  bc
   ld    hl,0
UTL_rddec2_loop
   ld    a,(de)
   call  UTL_isdigit		;Is character a digit?
   jr    c,UTL_rddec2_done	;No (end of numerical string)
   inc   de
   CALL  UTL_readdec2_add	;Update result in HL
   JR    UTL_rddec2_loop
UTL_rddec2_done
   pop   bc
   ret
;=====================================================================
;Name     UTL_readhex16
;Purpose  Read a hexidecimal string in the range 0-FFFF
;Input    DE = source string of decimal number characters
;Output   HL = result
;         DE = address of first non-digit character.
;Uses     AF corrupted.
;Notes    1. If string is > 65535 then result is uncertain.
;=====================================================================
UTL_readhex16
   push  bc
   ld    hl,0
UTL_rdhex16_loop
   ld    a,(de)
   call  UTL_hexdigit		;Is character a hex digit?
   cp    16
   jr    Z,UTL_rdhex16_done	;No (end of numerical string)
   inc   de
   add   hl,hl			;multiply by 2
   add   hl,hl			;multiply by 2
   add   hl,hl			;multiply by 2
   add   hl,hl			;multiply by 2
   ld    b,0
   ld    c,a
   add   hl,bc
   JR    UTL_rdhex16_loop
UTL_rdhex16_done
   pop   bc
   ret
;=====================================================================
;Name     UTL_insdigchr
;Purpose  To insert a digit char.
;Input    HL = address to insert char
;         A = value to add as char
;         RegC.Bit0 = show zeros flag
;Output   HL = HL + 1
;         RegC.Bit0 set
;Uses     TBD
;Notes    None
;=====================================================================
UTL_insdigchr
   CP    0
   JR    NZ,UTL_insdigchr1
   BIT   0,C
   RET   Z
UTL_insdigchr1
   add   a,'0'			;convert to number digit char
   ld    (hl),a			;Insert char
   inc   hl				;next location to save chars
   set   0,c			;flag now inserting zero's
   RET
;=====================================================================
;Name     UTL_wrdiv
;Purpose  A division routine for conversion of numbers to text.
;Input    E = divisor
;         A = numerator
;Output   D = INT (A/E)
;         A = remainder
;Uses     Flags changed.
;Notes    None
;=====================================================================
UTL_wrdiv
   ld    d,0			;Start counter at 0
UTL_wrdiv1
   inc   d
   sub   e				;Is value > decimal base
   jr    nc,UTL_wrdiv1		;Yes
   add   a,e			;setup remainder
   DEC   D
   RET
;=====================================================================
;Name     util_writedec
;Purpose  To convert an unsigned decimal number in the range 0 to 255
;         to a string with no leading zeros.
;Input    HL = address to save string
;         A = value to convert
;Output   HL += sprintf (HL, "%u", A) -- without trailing NUL
;Uses     AF corrupted.
;Notes    None
;=====================================================================
UTL_writedec
   push  bc
   push  de
   ld    c,0			;assume first char is 'zero'
   LD    E,100
   CALL  UTL_wrdiv		;D = number of 100's
   ld    e,a			;Save current result
   ld    a,d
   CALL  UTL_insdigchr		;insert digit char
   ld    a,e			;Get result back for next loop
   LD    E,10
   CALL  UTL_wrdiv		;D = number of 10's
   ld    e,a			;Save current result
   ld    a,d
   CALL  UTL_insdigchr		;insert digit char
   ld    A,E
   SET   0,C			;force insert of digit char
   CALL  UTL_insdigchr
   pop   de
   pop   bc
   ret
;=====================================================================
;
; Increment HL until it doesn't point to ' '
; All other regs preserved
;
UTL_skipspace
   push  af
UTL_skipspace_loop
   XOR   A
   CP    (HL)				;NULL ?
   JR    Z,UTL_skipspace_exit		;Yes
   ld    a,' '
   cp   (hl)				;Space?
   jr   nz,UTL_skipspace_exit		;No
   inc  hl
   jr   UTL_skipspace_loop
UTL_skipspace_exit
   pop  af
   ret
;=====================================================================
;
; Increment HL until it points to ' ' or NUL
; All other regs preserved
;
UTL_skiptospace
   push  af
UTL_skiptospace_loop
   XOR   A
   CP    (HL)				;NULL ?
   JR    Z,UTL_skiptospace_exit	;Yes
   ld    a,' '
   cp   (hl)				;Space?
   jr   z,UTL_skiptospace_exit	;Yes
   inc  hl
   jr   UTL_skiptospace_loop
UTL_skiptospace_exit
   pop  af
   ret
;=====================================================================
;
; A = tolower (A)
; F corrupted
;
; Note only lower-cases ASCII letters.
; COULD try to extend for ISO 8859
;
UTL_tolower
   cp    'A'		;Is char < 'A' ?
   ret   c           	;Yes
   cp    'Z' + 1		;Is char > 'Z' ?
   ret   nc			;Yes
   or    &20		;Force to letter to lower
   ret
;-----------------------------------------------------------------
;Name     Multiply
;PURPOSE  Routine to multiply numbers
;Input    Reg HL = Number
;         Reg DE = Multiplier
;Output   Reg HL = HL * DE
;Changes  None
;-----------------------------------------------------------------
Mth_Multiply
   LD    A,H
   OR    L
   JR    Z,Mth_ZERO_Res  ;HL=0, Zero Answer
   LD    A,D
   OR    E
   JR    Z,Mth_ZERO_Res  ;DE=0, Zero Answer
   PUSH  BC
   LD    B,D             ;BC = Number of time to add HL to itself
   LD    C,E
   LD    D,H
   LD    E,L
   LD    HL,0            ;Init HL
Mth_Mul_L1
   ADD   HL,DE
   DEC   BC
   LD    A,B
   OR    C
   JR    NZ,Mth_Mul_L1
   POP   BC
   RET
Mth_ZERO_res
   LD    HL,0
   RET
;-----------------------------------------------------------------
;Name     Mth_Divide
;PURPOSE  Divide HL by DE, result in Reg BC
;Input    Reg HL = Number
;         Reg DE = Divisor
;Output   Reg BC = Number of DE in HL
;         Reg HL = Remainder
;         A = 0 if not divide by zero
;             255 if divide by zero (BC=0)
;Changes  None
;-----------------------------------------------------------------
Mth_Divide
   LD    BC,0
   LD    A,D
   OR    E
   JR    Z,Mth_DIV_ERR  ;Can not divide by Zero
Mth_DIV_LP
   SBC   HL,DE          ;Subtract DE from HL
   JR    C,Mth_DIV_EX   ;Underflow, too much taken
   INC   BC             ;Still more can be taken
   JR    Mth_DIV_LP     ;Do again
Mth_DIV_EX
   ADD   HL,DE          ;Restore remainder
   XOR   A              ;Flag ok
   RET
Mth_DIV_ERR
   LD    A,&FF          ;Flag error
   RET
;