;=========================================================================
;
; Browser Application Routines (Font)
; Copyright (c) 2010 Ray Palmer
;
; The First 32 user def characters are used for icons
; The next 96 user def characters are specific to fonts
;
; Notes -
;
; 1. Proportional font is created on the foly.
;
; 2. The mode 1 character display is based on the mode 2 character
;    definition, which is expanded based on the foreground and background
;    colours. These colours start at bit 7 & 3 for the left most pixel
;    colour index. See below for picture of how this looks. For the whole
;    8 pixels of a single mode 2 character the mode 1 equivalent needs 2
;    consective bytes.
;
;    +--------------+---+---+---+---+---+---+---+---+
;    |scanline bit  | 7 | 6 | 5 | 4 | 3 | 2 | 1 | 0 |
;    +--------------+---+---+---+---+---+---+---+---+
;    |colour Index  | 0 | 1 | 2 | 3 | 0 | 1 | 2 | 3 |
;    +--------------+---+---+---+---+---+---+---+---+
;
;     Letter 'A'
;
;      Mode 2 scanline data   Mode 1 scanline data
;
;      Scanline  7654 3210    Scanline   7654 3210  7654 3210
;      1         ---1 1---    1          0001 0001  1000 1000
;      2         --11 11--    2          0011 0011  1100 1100
;      3         -11- -11-    3          0100 0100  0110 0110
;      4         -11- -11-    4          0100 0100  0110 0110
;      5         -111 111-    5          0111 0111  1110 1110
;      6         -11- -11-    6          0100 0100  0110 0110
;      7         -11- -11-    7          0100 0100  0110 0110
;      8         ---- ----    8          0000 0000  0000 0000
;
;      Where '-' means bit is clear and '1' means bit is set.
;
;      For the mode 1 scanline data above, '0' means the background colour
;      is used for the respective bits as shown in the above mapping of
;      bit to colour index per byte. The value '1' means the colour is the
;      foreground index.
;
;    Mode 1 has 4 colours, hence two bits per scanline pixel is needed.
;    For font character display the 2 remaining colours will be lost as
;    the character only uses 2 of the available 4 colours.
;
;    Mode 2 has 2 colours, so scanlines and definition data are the same.
;
;=========================================================================
FNT_BaseScreen	EQU	&C000		;start of screen
FNT_ChrsPerLine	EQU	80		;number of bytes per scan line.
SCR_MAXROW	EQU	25		;max rows on screen.
;=========================================================================
SCR_Mode	DEFB	0		;Screen mode (1 or 2)
SCR_EndCol	DEFB	0		;End column for the mode.
;=========================================================================
;  Font Table Structre (max 16 * 8 = 128 bytes)
;
;  S_FontName  = Name of font filename
;  S_FontIndex = RAM address of font data in this external module.
;  S_FontType  = Type of font
;                0 = fixed space font (Default)
;                1 = proportional font.
;
;=========================================================================
;
;  S_FontName = Name of font
;  S_FontIndex = location of font pattern data
;  S_FontType = type of font (255=fixed space, 0=proportional spaced)
;
S_FontNameSz	EQU	13
S_FontName	EQU	0
S_FontIndex	EQU	S_FontName+S_FontNameSz
S_FontType	EQU	S_FontIndex+2
;
S_FontEntrySz	EQU	S_FontType+1	;Size of structure
;=========================================================================
FontChrDefSz	EQU	8		;Char Data size in OS
FontChrDefSz1	EQU	FontChrDefSz+1	;chr num + 8 byte matrix data
;
MaxFonts	EQU	8
MaxFontChars	EQU	96			;chars 32 to 127 only
FontSize	EQU	MaxFonts*MaxFontChars
FontDataSz	EQU	FontSize*FontChrDefSz
;
MaxIcons	EQU	16				;must be power of 2
MaxIconChars	EQU	32
IconSetSz	EQU	MaxIconChars*FontChrDefSz1	;size of icon set
IconDataSz	EQU	MaxIcons*IconSetSz		;size of alloc
;=========================================================================
FNT_temp	DEFW	0
;
FNT_IconLocn	DEFW	0
FNT_LastFont	DEFW	0
FNT_LastData	DEFW	0
ICN_LastData	DEFW	0
ICN_NumItems	DEFB	0
FNT_NumItems	DEFB	0
;
;	FNT_FontObject = object of font data to process
;
FNT_FontObject	DEFS	S_FontEntrySz
FNT_FontName	EQU	FNT_FontObject+S_FontName
FNT_FontAddr	EQU	FNT_FontObject+S_FontIndex
FNT_FontTyp	EQU	FNT_FontObject+S_FontType
;
FNT_CharPosn	DEFW	0		;screen position to display font/icon
FNT_CharPosOff	DEFW	0		;screen position offsets
FNT_CharData	DEFW	0		;Location of char data for display
;
FNT_CurrCursAdr		DEFW	0	;screen location address on screen
;
;  Cursor variables
;
;  Cursor position for the screen are based on the standard CPC screen
;  coordinates which start from 1 upwards.
;
;  FNT_CurrData = contain the 8 bytes of data for the character the cursor
;                 is currently on.
;                 For mode 2 the routines use 8 bytes.
;                 For mode 1 the routines use 16 bytes.
;
;  FNT_CurrScrAdr = calculated position for the cursor character.
;
FNT_CursorPosn		DEFW	0	;System Screen position for cursor
FNT_CharNum		DEFB	0
FNT_CurrData		DEFS	FontChrDefSz*2	;Bkgrnd where cursor is on.
FNT_PrevData		DEFS	FontChrDefSz*2
FNT_CursorPos		DEFW	0
FNT_CursorIconDat	DEFW	0
FNT_CursorState		DEFB	0
;;
FNT_CurrScrAdr		DEFW	0	;screen start location for char/cursor
;
FNT_IconSet		DEFB	0
FNT_IconSetAddr		DEFW	0	;Location of Icon Set
FNT_LastIconSet		DEFW	&ffff
FNT_LastIconAdr		DEFW	0
;
FNT_FontsTable		DEFS	S_FontEntrySz*MaxFonts
;
;  8 bytes for each character to display on screen for proportional fonts
;
FNT_CurrChPat		DEFS	FontChrDefSz
FNT_CurrChPatShift	DEFB	0
;
FNT_NextChPat		DEFS	FontChrDefSz
FNT_NextChPatShift	DEFB	0
;
FNT_ChrPatTemp		DEFS	8	;char pattern to expand/use
FNT_ChrPattMode1	DEFS	16	;char pattern for mode 1 screen
;
FNT_NumShifts		DEFB	0
FNT_FontMode		DEFB	0
FNT_EffectsMask		DEFB	0
;
;  FNT_BCKGND = colour bit pattern for mode 1 starting from left pixel
;  FNT_FORGND = colour bit pattern for mode 1 starting from left pixel
;
;  The colour settings start at bit 7/3 for pixel 0 and then shifting 1
;  bit down for each mode 1 pixel shown in the font character.
;
FNT_BCKGND		DEFB	0	;Ink for background of font/icon
FNT_FORGND		DEFB	0	;Ink for foreground of font/icon
;
;  Variables for frame displaying
;
FNT_StartRow		DEFB	0
FNT_StartCol		DEFB	0
FNT_EndRow		DEFB	0
FNT_EndCol		DEFB	0
FNT_FrameIconSt		DEFB	0
FNT_FrameType		DEFB	0	;0=non-scrollable else scrollable
;
FNT_BordrSetNum		DEFB	0	;Main menu border set number
FNT_FrameSetNum		DEFB	0	;Frame border set number
;
FNT_PatScrMode		DEFB	0
;
FNT_ImgScrMode		DEFB	0
FNT_ImgPatDat		DEFW	0
;=========================================================================
;Name     FNT_DISABLED
;Purpose  To dummy routine for jump vector
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  None
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars None
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
FNT_DISABLED
    RET
;=========================================================================
;Name     FNT_SetFrameIcnSet
;Purpose  To setup which frame set icons to use.
;In Regs  AF = frames icons set number
;         B = colours for frames
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FNT_FrameSetNum = frames icons set number
;         FRME_FramColrs = colours for frame.
;Uses     TBD
;Call By  CPCB_Main (cpcb.exe)
;Calls    None
;Notes    TBD
;=========================================================================
FNT_SetFrameIcnSet
   LD    (FNT_FrameSetNum),A
   LD    A,B
   LD    (FRME_FramColrs),A
   RET
;=========================================================================
;Name     FNT_SetBrdrIcnSet
;Purpose  To setup which borders set icons to use.
;In Regs  AF = Border icon set number.
;         B = colours for borders
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  None
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FNT_BordrSetNum = borders icons set number
;         FRME_BrdrColrs = colours for border
;Uses     TBD
;Call By  CPCB_Main (cpcb.exe)
;Calls    None
;Notes    TBD
;=========================================================================
FNT_SetBrdrIcnSet
   LD    (FNT_BordrSetNum),A
   LD    A,B
   LD    (FRME_BrdrColrs),A
   RET
;=========================================================================
;Name     FNT_Inithandler
;Purpose  To initialise the Fonts handler variables.
;In Regs  AF = Screen Mode (1 or 2)
;         B  = Web page frame Set icon number
;         C  = Frames Set icon number
;         D = ??
;         E = Initialisation mode
;             0=Normal, >0 = screen mode/colours/columns
;         H = Frame Icons Colours
;         L = Bordr Icons Colours
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars SCR_Mode = screen mode
;         SCR_EndCol = end column on screen
;         FNT_PatternLoc = address of char pattern data
;         ICN_LastData = start address of icons data
;         ICN_NumItems = max icons set available to load
;         FNT_LastData = start address of font pattern data
;         FNT_LastFont = start address of fonts table
;         FNT_NumItems = max fonts available to load
;         FNT_BordrSetNum - iconset number and offset
;         FNT_CurrChPatShift = proportional font shift cleared
;         FNT_CharPosn = current row/col position not on screen.
;         FNT_EffectsMask = cleared
;         FRME_FramColrs = frame colours
;         FRME_BrdrColrs = border colours
;Uses     TBD
;Call By  CPCB_Main (cpcb.exe)
;Calls    None
;Notes    TBD
;=========================================================================
FNT_Inithandler
   LD    (SCR_Mode),A
   LD    D,40
   CP    1
   JR    Z,FNT_Inithandler1
   LD    D,80
FNT_Inithandler1
   LD    A,D
   LD    (SCR_EndCol),A
   LD    A,H
   LD    (FRME_FramColrs),A
   LD    A,L
   LD    (FRME_BrdrColrs),A
   LD    A,E			;get initialisation type
   OR    A			;Only mode/colours?
   RET   NZ			;Yes
;
   LD    (FNT_BordrSetNum),BC
;
   LD    HL,FNT_IconsStore
   LD    (ICN_LastData),HL	;Setup the location of Icons data
;
   LD    A,MaxIcons
   LD    (ICN_NumItems),A	;Setup max icon sets
;
   LD    A,MaxFonts
   LD    (FNT_NumItems),A	;Setup max font sets
;
   LD    HL,FNT_FontsStore
   LD    (FNT_LastData),HL	;Setup the location of Fonts data
;
   LD    HL,FNT_FontsTable
   LD    (FNT_LastFont),HL	;Setup location of fonts table.
;
   LD    A,255
   LD    (FNT_CurrChPatShift),A	;Clear shift flag for prop fonts.
;
   LD    HL,&FFFF
   LD    (FNT_CharPosn),HL
   XOR   A			;No font effects
;=========================================================================
;Name     FNT_NewEffect
;Purpose  To apply and effect to the current font.
;In Regs  A = font effect (each bit in the value indicates which scan
;                          line is xor'ed with 255 to invert).
;         B = mask to get scan-line bit pattern(s).
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  N/A
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FNT_EffectsMask = setup for which scan line is to be set.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. Repeated call with same value will turn off the effect.
;=========================================================================
FNT_NewEffect
   AND   B			;Setup scan line(s)
   OR    A			;Any scan lines to invert?
   JR    Z,FNT_NewEffect1	;No
   LD    B,A
   LD    A,(FNT_EffectsMask)
   XOR   B			;Toggle scan-line(s)
FNT_NewEffect1
   LD    (FNT_EffectsMask),A	;Save new scan-line(s).
   RET
;=========================================================================
;Name     FNT_CalcLclIcnAddr
;Purpose  To calculate the address of the Icon pattern set
;In Regs  AF = Icon set number
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = corrupted.
;         DE = ??
;         HL = address of pattern data
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. A max of 16 icons sets are allowed
;=========================================================================
FNT_CalcLclIcnAddr
   AND   MaxIcons-1
   LD    HL,FNT_IconsStore		;Location of Icon data.
   OR    A
   RET   Z
   LD    BC,IconSetSz			;BC = size of each Icon set chars
   JR	 FNT_CalcAddr0
;=========================================================================
;Name     FNT_CalcLclFntAddr
;Purpose  To calculate the start address of the Font pattern data location
;         for the local font data stores.
;In Regs  AF = Font character number (chars 0-96 means 32-127)
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = corrupted.
;         DE = ??
;         HL = address of pattern data in font set.
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
FNT_CalcLclFntAddr
   LD    HL,(FNT_LastData)	;Location of Font Pattern data.
   AND   &7F			;ensure character is < 128
   LD    BC,FontChrDefSz	;BC = size of font pattern set chars
   OR    A			;At required font entry?
   RET   Z			;Yes
FNT_CalcAddr0
   ADD   HL,BC			;Next entry address
   DEC   A
   JR    NZ,FNT_CalcAddr0
   RET
;=========================================================================
;Name     FNT_SetupFntEntry
;Purpose  To setup the Font entry structure.
;In Regs  AF = ??
;         B = ??
;         C = Font type
;         DE = Address to save font table entry.
;         HL = address of font name
;         IX = ??
;         IY = ??
;In Vars  N/A
;Out Regs AF = ??
;         BC = ??
;         DE = address of where fonts data is to be stored.
;         HL = corrupted.
;         IX = ??
;         IY = ??
;Out Vars FontId = current font identifier
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
FNT_SetupFntEntry
   PUSH  DE
;
;  Insert fonts name (all lowercase)
;
   LD    B,S_FontNameSz
FNT_SetupFntEnt2
   LD    A,(HL)
   CP    ','				;At end of name in script?
   JR    Z,FNT_SetupFntEnt3		;Yes (fill rest with nul)
   CP    '"'				;At end of name in HTML?
   JR    Z,FNT_SetupFntEnt3		;Yes (fill rest with nul)
   OR    A				;Name ends in nul?
   JR    Z,FNT_SetupFntEnt3		;Yes
   CALL  UTL_tolower
   INC   HL
   LD    (DE),A
   INC   DE
   DJNZ  FNT_SetupFntEnt2		;continue to add
   JR    FNT_SetupFntEnt4
;
FNT_SetupFntEnt3
   XOR   A
   LD    (DE),A
   INC   DE
   DJNZ  FNT_SetupFntEnt3
;
FNT_SetupFntEnt4
   POP   HL
   LD    DE,S_FontIndex
   ADD   HL,DE
   LD    DE,(FNT_LastData)
   LD    (HL),E				;setup location of pattern data
   INC   HL
   LD    (HL),D
   INC   HL
   LD    (HL),C				;setup font type
   INC   HL
   RET
;=========================================================================
;Name     FNT_FindFontByName
;Purpose  To search the font database table for a font character set by
;         its name.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  FNT_FontName = name of font to find
;         FNT_FontsTable = table of known fonts.
;Out Regs AF = 0 if not found else 1
;         B = ??
;         C = font type
;         DE = Address of DB entry (if found)
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
FNT_FindFontByName
   LD    DE,FNT_FontsTable
FNT_FFBN0
   LD    HL,(FNT_LastFont)
   XOR   A			;Assume font's name does not exist
   SBC   HL,DE			;At end of table ?
   RET   Z			;Yes (font not found)
;
   PUSH  DE
   EX    DE,HL
   LD    DE,FNT_FontName
   LD    B,S_FontNameSz
   CALL  UTL_strncmp		;Names match?
   POP   DE
   JR    NZ,FNT_FFBN2		;No (goto next db entry)
   LD    A,1			;Flag font entry found
   RET
FNT_FFBN2
   LD    HL,S_FontEntrySz
   ADD   HL,DE			;point to next Db entry
   EX    DE,HL
   JR    FNT_FFBN0		;continue checking
;=========================================================================
;Name     FNT_UploadIcon
;Purpose  To upload the icon data to local data stores.
;In Regs  AF = Icon Number to upload
;         BC = ??
;         DE = ??
;         HL = Address of icon data
;         IX = ??
;         IY = ??
;In Vars  ICN_LastData = current end of icons data
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars ICN_LastData = updated end of icons data
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. The icons are not saved by name.
;=========================================================================
FNT_UploadIcon
   LD    A,(ICN_NumItems)
   OR    A			;max icon sets loaded up?
   RET   Z			;No
   DEC   A			;dec remaining sets available
   LD    (ICN_NumItems),A	;save count
   LD    DE,(ICN_LastData)
   LD    BC,IconSetSz
   LDIR				;copy to local data stores.
   LD    (ICN_LastData),DE
   XOR   A
   LD    (DE),A			;Flag end of icons data
   RET
;=========================================================================
;Name     FNT_UploadFont
;Purpose  To upload the font data from main memory to local data stores.
;In Regs  AF = Font type
;              0 = fixed spacing font
;              1 = proportional font
;         BC = ??
;         DE = address of font name
;         HL = address of font data
;         IX = ??
;         IY = ??
;In Vars  FNT_LastFont = Current end of Table of font names
;         FNT_LastData = Current end addres of local font data.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FNT_LastFont = Updated end of Table of font names
;         FNT_LastData = Updated end addres of font data.
;Uses     TBD
;Call By  TBD
;Calls    FNT_SetupFntEntry
;         FNT_CalcLclFntAddr
;Notes    1. Assume the definitions for all characters start from 128+32
;=========================================================================
FNT_UploadFont
   LD    C,A			;Save font type
   LD    A,(FNT_NumItems)
   OR    A			;Font table full?
   RET   Z			;Yes
   DEC   A
   LD    (FNT_NumItems),A	;Decrement available table entries
   PUSH  HL
   EX    DE,HL			;HL = address of fonts name
   LD    DE,(FNT_LastFont)	;Last table entry address
   CALL  FNT_SetupFntEntry	;Setup db entry in the font table.
   LD    (FNT_LastFont),HL	;Setup for next table entry
   POP   DE
;
;  Upload Font data by moving it to correct local addresses
;
FNT_UploadFnt1
   LD    A,(DE)			;Get char# from main memory
   OR    A			;end of data?
   JR    Z,FNT_UploadFnt2	;Yes
   SUB   32			;reset chars 32-127 to start from 0
   INC   DE			;skip char number byte
   CALL  FNT_CalcLclFntAddr	;HL = address in local data
   EX    DE,HL
   LDIR				;copy char pattern to local
   EX    DE,HL
   JR    FNT_UploadFnt1		;next char pattern to save
FNT_UploadFnt2
   LD    HL,(FNT_LastData)
   LD    DE,FontSize
   ADD   HL,DE
   LD    (FNT_LastData),HL
   RET
;=========================================================================
;Name     FNT_SelectFontByName
;Purpose  To select a loaded font by a reference to its name.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = Address of name
;         IX = ??
;         IY = ??
;In Vars  FNT_PatLocnChars = maim memory location of system chars
;         FNT_FontObject = name of font to search for.
;Out Regs AF = Font type.
;         BC = Address of named font
;         DE = DB Address of font info
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FNT_FontAddr
;         FNT_FontTyp
;         FNT_CurrChPatShift
;Uses     TBD
;Call By  TBD
;Calls    FNT_SetupFntEntry
;         FNT_FindFontByName
;Notes    1. The first font loaded must be 'Normal'.
;=========================================================================
FNT_SelectFontByName
   LD    DE,FNT_FontObject
   CALL  FNT_SetupFntEntry		;get font's name.
   CALL  FNT_FindFontByName		;Search for the Font by Name
   OR    A				;Does the font exist?
   JR    NZ,FNT_SelectFontByIndex	;Yes
   LD    DE,FNT_FontsTable		;Use default font ('Normal')
;=========================================================================
;Name     FNT_SelectFontByIndex
;Purpose  To select a loaded font by a reference to its name.
;In Regs  AF = ??
;         BC = ??
;         DE = DB Address of font info
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = font type
;         BC = font pattern data address
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FNT_FontAddr
;         FNT_FontTyp
;         FNT_CurrChPatShift
;Uses     TBD
;Call By  TBD
;Calls    FNT_SetupFntEntry
;         FNT_FindFontByName
;Notes    1. The first font loaded must be 'Normal'.
;=========================================================================
FNT_SelectFontByIndex
   LD    HL,S_FontIndex
   ADD   HL,DE				;Calc address of font loc'n
   LD    C,(HL)				;Get address of font Patt. data
   INC   HL
   LD    B,(HL)
   INC   HL
   LD    A,(HL)				;Get font type
   LD    (FNT_FontAddr),BC		;Address of pattern data
   LD    (FNT_FontTyp),A		;Get font type being uploaded.
   OR    A				;Fixed space font?
   RET   Z				;Yes
   LD    A,255
   LD    (FNT_CurrChPatShift),A		;Setup first entry conditions
   LD    A,(FNT_FontTyp)
   RET
;=========================================================================
;Name     FNT_OnScreen
;Purpose  To check if the cursor position is within the screen based on
;         the screen mode.
;In Regs  A = ??
;         B = ??
;         C = ??
;         DE = ??
;         H = screen column
;         L = screen row
;         IX = ??
;         IY = ??
;In Vars  SCR_EndCol = end column.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars None.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. For all screen mode there are a maximum of 25 lines, but for
;            mode 2 there are at most 80 columns and for mode 1 there are
;            40 columns.
;=========================================================================
FNT_OnScreen
   LD    A,L
   CP    SCR_MAXROW	;At or above max screen row?
   RET   NC
   PUSH  HL
   LD    A,(SCR_EndCol)
   LD    L,A
   LD    A,H
   CP    L		;At or above max screen column?
   POP   HL
   RET
;=========================================================================
;Name     FNT_SetIconData
;Purpose  To setup the scan lines of icon data to display.
;In Regs  AF = ??
;         B = Icon number
;         C = Icon set number
;         DE = ??
;         HL = address of icons scan lines data for 1 character.
;         IX = ??
;         IY = ??
;In Vars  N/A
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FontId = current font identifier
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
FNT_SetIconData
   PUSH  HL
   LD    DE,(FNT_LastIconAdr)
   LD    HL,(FNT_LastIconSet)	;latest icon set indices
   SBC   HL,BC
   LD    A,H			;same icom as previous?
   OR    L
   JR    Z,FNT_SetIconData1	;Yes
   PUSH  BC
   LD    A,C
   CALL  FNT_CalcLclIcnAddr	;HL = address of icon set
   POP   BC
   LD    A,B
   CALL  FNT_FindIconChar	;HL = address of icon pattern
   INC   HL
   LD    (FNT_LastIconAdr),HL
   EX    DE,HL
FNT_SetIconData1
   POP   HL
   LD    BC,FontChrDefSz
   LDIR				;upload icon data
   RET
;=========================================================================
;Name     FNT_SetCursorIcon
;Purpose  To select the cursor icon.
;In Regs  A = Character for cursor icon (0-31)
;         B = icon set number
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FNT_CursorIconDat = address of pattern data for cursor icon.
;         FNT_IconLocn = location of selected Icon set.
;Uses     TBD
;Call By  CPCB.EXE (KB_ShowCursor)
;Calls    FNT_CalcLclIcnAddr
;         FNT_FindIconChar
;Notes    TBD
;=========================================================================
FNT_SetCursorIcon
   PUSH  AF
   LD    A,B
   CALL  FNT_CalcLclIcnAddr	;calc which set of icons to use
   POP   AF
   LD    (FNT_IconLocn),HL	;setup address of icon data
   CALL  FNT_FindIconChar	;Calc address of font data.
   INC   HL			;skip icon number
   LD    DE,FNT_CurrData	;Location of cursor icon data
   LD    BC,FontChrDefSz
   LDIR				;download Cursor icon data.
   RET
;=========================================================================
;Name     FNT_ResetCursor
;Purpose  To reset the cursor icon status
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  None
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = corrupted.
;         IX = ??
;         IY = ??
;Out Vars FNT_CurrCursAdr = 0 (no cursor yet on screen)
;Uses     TBD
;Call By  TBD
;Calls    None.
;Notes    None.
;=========================================================================
FNT_ResetCursor
   LD    HL,0
   LD    (FNT_CurrCursAdr),HL	;flag no icon data address
   RET
;=========================================================================
;Name     FNT_ShowCursor
;Purpose  To display the cursor icon on the screen.
;In Regs  A = ??
;         B = External Cursor Status (0=1st entry, >0 repeat entry)
;         C = cursor colours (foreground L/Background U)
;         DE = Current CPC screen stream start position
;               E = Row, D = Column
;         HL = Cursor Position
;               L = Row, H = Column
;         IX = ??
;         IY = ??
;In Vars  N/A
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FNT_CurrScrAdr = screen memory address of cursor icon
;         FNT_CharData = location of cursor icon pattern data
;Uses     TBD
;Call By  CPCB.EXE (KB_ShowCursor)
;Calls    FNT_OnScreen
;         FNT_SetColour
;         FNT_RemoveCursor
;         FNT_CalcScrnAddr
;         FNT_SaveIcnPattrn
;Notes    1. The CPC screen coordinates are based from 1 and not zero.
;=========================================================================
FNT_ShowCursor
   ADD   HL,DE			;setup absolute screen position
;
   LD    DE,(FNT_CursorPosn)	;Get previous position
   PUSH  HL
   OR    A
   SBC   HL,DE			;Any change in position?
   POP   HL
   RET   Z			;No
;
   CALL  FNT_OnScreen
   RET   NC			;not within the screen
;
   LD    A,C
   CALL  FNT_SetColour		;setup cursor colours
;
   LD    (FNT_CursorPosn),HL	;save new position for re-entry
   LD    (FNT_CharPosn),HL	;setup new position for calcs
;
   CALL  FNT_RemoveCursor	;Restore current character data
;
   CALL  FNT_CalcScrnAddr	;Calculate new position
   LD    (FNT_CurrScrAdr),HL	;Save screen address of byte data
   LD    (FNT_CurrCursAdr),HL	;save for FNT_RemoveCursor
;
   LD    DE,FNT_PrevData
   CALL  FNT_SaveIcnPattrn	;Save background
;
   LD    DE,FNT_CurrData
   CALL  FNT_CnvChrPatrn	;convert pattern for screen mode
;
   JP    FNT_DispCharPat	;Show cursor icon pattern
;=========================================================================
;Name     FNT_RemoveCursor
;Purpose  To remove the cursor icon off the screen.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  FNT_CurrCursAdr = contains last position of cursor.
;         FNT_PrevData = contains previous pattern data
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    FNT_ShwChrPattrn0
;Notes    1. The CPC screen columns are limited to between 1 and 80.
;=========================================================================
FNT_RemoveCursor
   LD    HL,(FNT_CurrCursAdr)
   LD    A,H
   OR    L			;Has cursor been first shown?
   RET   Z			;No (dont restore yet)
   LD    DE,FNT_PrevData
   JP    FNT_DispCharPat0	;restore previous character pattern
;=========================================================================
;Name     FNT_ShowImgChar
;Purpose  To display the image character data.
;In Regs  A = Image Screen mode (1 or 2)
;         BC = Address of image char data.
;         DE = Current CPC screen stream start position
;               E = Row, D = Column
;         HL = Cursor Position
;               L = Row, H = Column
;         IX = ??
;         IY = ??
;In Vars  N/A
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  ??
;Calls    FNT_DispCharPat
;Notes    1. The CPC screen coordinates are based from 1 and not zero.
;=========================================================================
FNT_ShowImgChar
   LD    (FNT_ImgScrMode),A
   LD    (FNT_ImgPatDat),BC
   ADD   HL,DE			;Convert relative to absolute position.
   CALL  FNT_OnScreen
   RET   NC			;not within screen
   LD    (FNT_CharPosn),HL	;Row/Column of character to display
   CALL  FNT_CalcScrnAddr	;calc address of memory for position.
   LD    (FNT_CurrScrAdr),HL	;Save screen address of byte data for chars
;
   CALL  FNT_CnvImagePatrn
;
   JP    FNT_DispCharPat	;show converted data on screen
;=========================================================================
;Name     FNT_ShowIconChar
;Purpose  To display the icon char on the screen.
;In Regs  A = character number (0-31)
;         B = Colour map for mode 1
;         C = icon set number
;         DE = Current CPC screen stream start position (base 0)
;              E = Row, D = Column
;         HL = Current CPC screen position (base 0)
;              L = Row, H = Column
;         IX = ??
;         IY = ??
;In Vars  FNT_IconLocn = location of selected Icon set.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FNT_CharNum = icon character number
;         FNT_IconSetAddr = address of icon set.
;         FNT_CurrScrAdr = memory address of character to show
;         FNT_CharPosn = Character position coordinates (base 0)
;                        Low byte = Row, Upper = Column
;Uses     TBD
;Call By  TBD
;Calls    FNT_CalcLclIcnAddr
;         FNT_CalcScrnAddr
;         FNT_FindIconChar
;         FNT_OnScreen
;         FNT_SetColour
;         FNT_CnvChrPatrn
;         FNT_DispCharPat
;Notes    1. The CPC screen columns are limited to between 1 and 80 for
;            mode 2 while for mode 1 they are limited to 1 upto 40.
;         2. This routine does not auto scroll the screen if the position
;            invalid. Where the position becomes invalid the routine
;            simply exits with error.
;=========================================================================
FNT_ShowIconChar
   AND   MaxIconChars-1		;ensure in range 0-31
   LD    (FNT_CharNum),A	;save icon number
;
   ADD   HL,DE			;Convert relative to absolute position.
   CALL  FNT_OnScreen
   RET   NC			;not within screen
   LD    (FNT_CharPosn),HL	;Row/Column of character to display
;
   LD    A,B
   CALL  FNT_SetColour
;
   LD    A,C
   CALL  FNT_CalcLclIcnAddr	;Calc address of icon set.
   LD    (FNT_IconSetAddr),HL
;
   CALL  FNT_CalcScrnAddr	;calc address of memory for position.
   LD    (FNT_CurrScrAdr),HL	;Save screen address of byte data for chars
;
   LD    A,(FNT_CharNum)	;Icon Number
   LD    HL,(FNT_IconSetAddr)	;address of icon data
   CALL  FNT_FindIconChar	;Calc address of pattern data for the icon.
   INC   HL			;Point to pattern data
   EX    DE,HL
   CALL  FNT_CnvChrPatrn	;Convert the pattern data for screen mode
;
   JR    FNT_DispCharPat	;show data on screen
;=========================================================================
;Name     FNT_ShowChrPattn
;Purpose  To display the external character pattern on the screen.
;In Regs  A = Screen mode for Pattern data.
;         B = Address of pattern data to display.
;         DE = Current CPC screen stream start position (base 0)
;              E = Row, D = Column
;         HL = Current CPC screen position (base 0)
;              L = Row, H = Column
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FNT_PatScrMode
;         FNT_IconSetAddr = address of icon set.
;         FNT_CurrScrAdr = memory address of character to show
;         FNT_CharPosn = Character position coordinates (base 0)
;                        Low byte = Row, Upper = Column
;Uses     TBD
;Call By  CPC.EXE (API access)
;Calls    FNT_CalcLclIcnAddr
;         FNT_CalcScrnAddr
;         FNT_FindIconChar
;         FNT_OnScreen
;         FNT_SetColour
;         FNT_DispCharPat
;Notes    1. The CPC screen columns are limited to between 1 and 80 for
;            mode 2 while for mode 1 they are limited to 1 upto 40.
;=========================================================================
FNT_ShowChrPattn
   LD    (FNT_PatScrMode),A
   LD    (FNT_IconSetAddr),BC
   ADD   HL,DE			;Convert relative to absolute position.
   CALL  FNT_OnScreen
   RET   NC			;not within screen
;
   LD    (FNT_CharPosn),HL	;Row/Column of character to display
   CALL  FNT_CalcScrnAddr	;calc address of memory for position.
   LD    (FNT_CurrScrAdr),HL	;Save screen address of byte data for chars
;
   LD    DE,(FNT_IconSetAddr)
   CALL  FNT_CnvChrPatrn	;Convert the pattern data
;
   JR    FNT_DispCharPat	;show data on screen
;=========================================================================
;Name     FNT_ShowChrPattrn
;Purpose  To display the font char on the screen by copying local data to
;         the memory region of the CPC screen.
;In Regs  A = ??
;         B = ??
;         C = Character scan line effects
;         DE = Address of pattern data to display.
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  FNT_CurrScrAdr = physical memory location within CPC screen.
;         SCR_Mode = screen mode (1 or 2)
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    FNT_BuildChrPatrn
;Notes    1. This routine can also be use to display Icon characters.
;=========================================================================
FNT_ShowChrPattrn
   CALL  FNT_BuildChrPatrn
;=========================================================================
;Name     FNT_DispCharPat
;Purpose  To display the character scan line data.
;In Regs  A = ??
;         BC = ??
;         DE = Address of pattern data to display.
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  FNT_CurrScrAdr = physical memory location within CPC screen.
;         SCR_Mode = screen mode (1 or 2)
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    TBD
;=========================================================================
FNT_DispCharPat
   LD    HL,(FNT_CurrScrAdr)	;Screen memory location
FNT_DispCharPat0
   DI
   EXX
   LD   E,C
   SET  2,E
   SET  3,E
   OUT  (C),E			;Swap out ROMs
   EXX
;
   LD    B,FontChrDefSz
FNT_DispCharPat1
   LD    A,(DE)			;get pattern data for the nth scanline
   LD    (HL),A			;insert to screen memory
   INC   DE
   LD    A,(SCR_Mode)
   CP    2			;screen mode 2?
   JR    Z,FNT_DispCharPat2	;Yes
   INC   HL
   LD    A,(DE)			;get pattern data for the nth scanline
   LD    (HL),A			;insert to screen memory
   DEC   HL
   INC   DE
FNT_DispCharPat2
   LD    A,8
   ADD   A,H			;add 2K to for next scan line
   LD    H,A
   DJNZ  FNT_DispCharPat1
;
   EXX
   OUT  (C),C			;Restore ROMs
   EXX
   EI
;
   RET
;=========================================================================
;Name     FNT_SaveIcnPattrn
;Purpose  To save the current icon background.
;In Regs  A = character number
;         BC = ??
;         DE = Address to save pattern data.
;         HL = Address in screen ram to save
;         IX = ??
;         IY = ??
;In Vars  SCR_Mode = screen mode (1 or 2)
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  FNT_ShowCursor
;Calls    None.
;Notes    TBD
;=========================================================================
FNT_SaveIcnPattrn
   DI
   EXX
   LD   E,C
   SET  2,E
   SET  3,E
   OUT  (C),E			;Swap out ROMs
   EXX
;
   LD    B,FontChrDefSz		;number of scan lines for character
FNT_SaveIcnPatt1
   LD    A,(HL)			;Get 1st screen byte mode 1 and 2
   LD    (DE),A			;save previous for restore
   INC   DE
   LD    A,(SCR_Mode)
   CP    2			;mode 2 screen
   JR    Z,FNT_SaveIcnPatt2	;Yes
;
   INC   HL
   LD    A,(HL)			;Get 2nd screen byte (mode 1)
   DEC   HL
   LD    (DE),A			;save previous for restore
   INC   DE
FNT_SaveIcnPatt2
   LD    A,8
   ADD   A,H			;Add 2k for next scan line
   LD    H,A
   DJNZ  FNT_SaveIcnPatt1
;
   EXX
   OUT  (C),C			;Restore ROMs
   EXX
   EI
;
   RET
;=========================================================================
;Name     FNT_NumBits2Shift
;Purpose  To determine the number of bits to shift from left to right for
;         a proportional font.
;In Regs  A = shift pattern control
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  N/A
;Out Regs AF = corrupted
;         B  = Number of bit shifts for next font char
;         C  = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars N/A
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
FNT_NumBits2Shift
   LD    B,8
FNT_NumB2Sh1
   BIT   7,A		;at start free bits for next char?
   RET   NZ		;Yes
   SLA   A		;shift used bits up
   DJNZ  FNT_NumB2Sh1
   RET
;=========================================================================
;Name     FNT_ShiftFont
;Purpose  To shift the bit patterns for chars to get a proportional font.
;In Regs  A = ??
;         B = current scan line bit pattern to merge with next font char
;         C = next scan line of font char to merge.
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  FNT_NumShifts
;Out Regs AF = 0 and flags changed.
;         B  = updated
;         C  = updated
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
FNT_ShiftFont
   LD    A,(FNT_NumShifts)
   CP    8			;shift in whole pattern?
   JR    Z,FNT_ShiftFnt3	;Yes
FNT_ShiftFont1
   SRL   B			;shift down for free bits to move in
   DEC   A
   JR    NZ,FNT_ShiftFont1
   LD    A,(FNT_NumShifts)
FNT_ShiftFnt2
   SLA   C			;shift bit 7 to carry flag
   RL    B			;shift carry into
   DEC   A
   JR    NZ,FNT_ShiftFnt2
   RET
FNT_ShiftFnt3
   LD    B,C			;no merge just shift 8 bits
   LD    C,0
   RET
;=========================================================================
;Name     FNT_MergeChars
;Purpose  To merge the two proportional fonts for display.
;In Regs  A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  FNT_CurrChPat = Current displayed proportional font character(s)
;         FNT_NextChPat = next proportionla font to show.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FNT_CurrChPat = updated displayed proportional font character(s)
;         FNT_NextChPat = next proportionla font to show.
;Uses     TBD
;Call By  TBD
;Calls    FNT_ShiftFont
;Notes    1. If the current displayed pattern is free for all of the
;            proportional font the next pattern is shift entirely and then
;            displayed.
;=========================================================================
FNT_MergeChars
;
;  shift next into current
;
   LD    HL,FNT_CurrChPat
   LD    DE,FNT_NextChPat
   LD    B,FontChrDefSz-1
FNT_ShowPropFnt0
   PUSH  BC
   LD    B,(HL)
   LD    A,(DE)
   LD    C,A
   CALL  FNT_ShiftFont		;merge bit patterns of this scan line
   LD    (HL),B
   LD    A,C
   LD    (DE),A
   INC   HL
   INC   DE
   POP   BC
   DJNZ  FNT_ShowPropFnt0
   RET
;=========================================================================
;Name     FNT_ShowPropFont
;Purpose  To display the characters for a proportional font character set.
;In Regs  A = ??
;         B = current character to display.
;         C = ??
;         DE = Previous cpc screen position.
;         HL = Current cpc screen position.
;         IX = ??
;         IY = ??
;In Vars  N/A
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    FNT_NumBits2Shift
;         FNT_MergeChars
;         FNT_ShiftFont
;         FNT_OnScreen
;         FNT_CalcScrnAddr
;         FNT_ShowChrPattrn
;Notes    1. Highest compression of chars is the letter i where there can
;            be two i's per normal fixed spaced font char.
;=========================================================================
FNT_ShowPropFont
   LD    A,(FNT_CurrChPatShift)
   CALL  FNT_NumBits2Shift	;B = Number of bits for next char
   LD    A,B
   LD    (FNT_NumShifts),A	;save number of shifts
;
   CALL  FNT_MergeChars		;merge the font for display.
;
   LD    A,(FNT_CurrChPatShift)
   LD    B,A
   LD    A,(FNT_NextChPatShift)
   LD    C,A
   CALL  FNT_ShiftFont		;adjust current shift pattern
   LD    A,B
   OR    A			;Is current char full?
   JR    Z,FNT_ShwPropFont1	;Yes
;
;  Current pattern has more room for next proportional font char
;
   LD    (FNT_CurrChPatShift),A
   LD    HL,FNT_CurrChPat
   JP    FNT_ShowChrPattrn	;Display updated pattern data
;
FNT_ShwPropFont1
   LD    HL,FNT_CurrChPat
   CALL  FNT_ShowChrPattrn	;Display updated current pattern
;
   LD    A,(FNT_NextChPatShift)
   LD    B,A
   LD    A,(FNT_NumShifts)
FNT_ShwPropFont2
   SCF
   RL    B			;move free bits at end of pattern
   DEC   A
   JR    NZ,FNT_ShwPropFont2
   LD    (FNT_CurrChPatShift),A	;save new fee bits for next char
;
   LD    HL,(FNT_CharPosn)	;Row/Column of character to display
   INC   H			;next column
   CALL  FNT_OnScreen		;within screen?
   JR    Z,FNT_ShwPropFont3	;No (do not show character)
;
   LD    (FNT_CharPosn),HL	;save new position for return to caller
   CALL  FNT_CalcScrnAddr
   LD    (FNT_CurrScrAdr),HL	;Save screen address of byte data for chars
;
   LD    HL,FNT_NextChPat
   CALL  FNT_ShowChrPattrn	;Display next pattern data
;   
FNT_ShwPropFont3
   LD    DE,FNT_CurrChPat
   LD    HL,FNT_NextChPat
   LD    BC,FontChrDefSz-1
   LDIR				;copy next to current for next char
   RET
;=========================================================================
;Name     FNT_SetChrCol_F
;Purpose  To insert the foreground colour for fonts.
;In Regs  AF = ??
;         BC = ??
;         DE = Address of character pattern data
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = no change
;         BC = ??
;         DE = Address of character pattern data
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    TBD
;=========================================================================
FNT_SetChrCol_F
   PUSH  AF
   LD    A,(DE)
   OR    C
   LD    (DE),A
   POP   AF
   RET
;=========================================================================
;Name     FNT_SetChrCol_B
;Purpose  To insert the background colour for fonts.
;In Regs  AF = ??
;         BC = ??
;         DE = Address of character pattern data
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = No change
;         BC = ??
;         DE = Address of character pattern data
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    TBD
;=========================================================================
FNT_SetChrCol_B
   PUSH  AF
   LD    A,(DE)
   OR    B
   LD    (DE),A
   POP   AF
   RET
;=========================================================================
;Name     FNT_Color2Indx
;Purpose  To return the colour index in a bit map form for mode 1.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  None
;Out Regs AF = ??
;         B = colour index for mode 1 bit pattern.
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars None
;Uses     TBD
;Call By  TBD
;Calls    None.
;Notes    None
;=========================================================================
FNT_Color2Indx
   LD    B,&88		;colour index 3 for ink 3
   CP    3
   RET   Z
   LD    B,&80		;colour index 2 for ink 2
   CP    2
   RET   Z
   LD    B,&08		;colour index 1 for ink 1
   CP    1
   RET   Z
   LD    B,0		;colour index 0 for ink 0
   RET
;=========================================================================
;Name     FNT_Indx2COlor
;Purpose  To convert the mode 1 colour index into mode 2 colours
;In Regs  AF = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = Address of Pattern data
;         IX = ??
;         IY = ??
;In Vars  None
;Out Regs AF = Z flag means clear mode 2 bit else set mode 2 bit
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars None
;Uses     TBD
;Call By  TBD
;Calls    None.
;Notes    1. Maps mode 1 inks 1 to 3 to mode 2 ink 1.
;=========================================================================
FNT_Indx2COlor
   LD    A,(HL)
   AND   C		;Mask out mode 1 colour bits
   OR    A		;Is mode 1 ink > 0 ?
   RET
;=========================================================================
;Name     FNT_BuildChrPatrn
;Purpose  To build up the character bit pattern of the font characters
;         which are defined only as mode 2 for display on the screen
;         in mode 1 or mode 2.
;In Regs  AF = ??
;         B = ??
;         C = Font effects
;         DE = Address of character pattern for mode 2
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  FNT_ChrPatTemp = current char bits to display.
;         FNT_BCKGND = foreground/background colours.
;Out Regs AF = ??
;         BC = ??
;         DE = address of FNT_ChrPattMode1
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FNT_ChrPattMode1 = pattern to display for Mode 1
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. The initial character pattern is for the default mode 2
;            screen.
;         2. The Font effects parameter define which scan line data is
;            inverted. If all bits in parameter are set the font character
;            is inverted.
;=========================================================================
FNT_BuildChrPatrn
   LD    A,C
   OR    A			;Any effect to apply?
   JR    Z,FNT_CnvChrPatrn	;No
;
;  Setup mode 2 pattern with any effects
;
   EX    DE,HL
   LD    DE,FNT_ChrPatTemp
   LD    B,FontChrDefSz
FNT_BuildChrPatrn1
   LD    A,(HL)			;get pattern data for the nth scanline
   BIT   0,C			;Is scan line to be inverted
   JR    Z,FNT_BuildChrPatrn2	;No
   XOR   255			;invert mode 2 scanline data
FNT_BuildChrPatrn2
   LD    (DE),A			;insert to screen memory
   OR    A
   SRL   C			;shift next scan line effect down
   INC   DE
   INC   HL
   DJNZ  FNT_BuildChrPatrn1
   LD    DE,FNT_ChrPatTemp
;=========================================================================
;Name     FNT_CnvChrPatrn
;Purpose  To convert the font/icon pattern (in mode 2 only) for display on
;         the screen.
;In Regs  AF = ??
;         B = ??
;         C = ??
;         DE = Address of mode 2 scanline data.
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  FNT_BCKGND = foreground/background colours.
;Out Regs AF = ??
;         BC = ??
;         DE = address of pattern data to display.
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FNT_ChrPattMode1 = pattern to display for Mode 1
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. The initial character pattern is for the default mode 2
;            screen.
;=========================================================================
FNT_CnvChrPatrn
   LD    C,2			;Source pattern is mode 2
   JR    FNT_CnvPatrn
;=========================================================================
;Name     FNT_CnvImagePatrn
;Purpose  To convert the image pattern for display on the screen.
;In Regs  AF = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  FNT_PatScrMode = initial mode the char patter is in.
;         FNT_ImgPatDat = address of image pattern data
;         FNT_BCKGND = foreground/background colours.
;Out Regs AF = ??
;         BC = ??
;         DE = address of FNT_ChrPattMode1
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FNT_ChrPattMode1 = pattern to display for Mode 1
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. The initial character pattern is for the default mode 2
;            screen.
;=========================================================================
FNT_CnvImagePatrn
   LD    DE,&30
   LD    (FNT_BCKGND),DE	;BG=ink 3, FG=Ink 0
   LD    DE,(FNT_ImgPatDat)
   LD    A,(FNT_ImgScrMode)
   LD    C,A
;=========================================================================
;Name     FNT_CnvPatrn
;Purpose  To convert the image pattern data for the current screen mode.
;In Regs  AF = ??
;         B = Current screen mode
;         C = Screen mode of pattern data
;         DE = Addres of scan line data
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  SCR_Mode = screen mode (1 or 2)
;Out Regs AF = ??
;         BC = ??
;         DE = address of FNT_ChrPattMode1 or original address
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FNT_ChrPattMode1 = pattern data to display on screen
;Uses     TBD
;Call By  TBD
;Calls    FNT_SetChrCol_F
;         FNT_SetChrCol_B
;         FNT_Indx2COlor
;         FNT_SetChrCol_B
;Notes    1. If the current screen mode is 2 and the pattern data is in
;            mode 1 then inks 1 to 3 are reduced to ink 1 while ink 0 is
;            left as is.
;         2. If the current screen is mode 1 and the data is in mode 2 the
;            conversion is to merely re-adjust the pattern for mode 1
;            display, but inks used are inks 0 and 1.
;         3. If the current mode and the pattern mode are the same then
;            no changes are made.
;=========================================================================
FNT_CnvPatrn
   LD    A,(SCR_Mode)
   LD    B,A
   LD    A,C
   CP    B			;Is pattern and current modes same?
   RET   Z			;Yes (do nothing)
   CP    2			;Is source pattern in mode 2?
   EX    DE,HL			;HL = address of pattern to convert
   LD    DE,FNT_ChrPattMode1	;DE = address of converted pattern
   LD    B,FontChrDefSz
   JP    NZ,FNT_CnvM1toM2	;No (convert mode 1 to mode 2)
;
;  Convert mode 2 pattern to mode 1 pattern
;
FNT_CnvM2toM1
   PUSH  BC
;
;  Setup first 4 pixels of display character scan line (left to right)
;
   XOR   A
   LD    (DE),A
   LD    BC,(FNT_BCKGND)	;B=Background, C=Foreground
   BIT   7,(HL)			;Bit 7 set?
   CALL  NZ,FNT_SetChrCol_F	;Yes
   CALL  Z,FNT_SetChrCol_B	;No
;
   SRL   B
   SRL   C
   BIT   6,(HL)			;Bit 6 set?
   CALL  NZ,FNT_SetChrCol_F	;Yes
   CALL  Z,FNT_SetChrCol_B	;No
;
   SRL   B
   SRL   C
   BIT   5,(HL)			;Bit 5 set?
   CALL  NZ,FNT_SetChrCol_F	;Yes
   CALL  Z,FNT_SetChrCol_B	;No
;
   SRL   B
   SRL   C
   BIT   4,(HL)			;Bit 4 set?
   CALL  NZ,FNT_SetChrCol_F	;Yes
   CALL  Z,FNT_SetChrCol_B	;No
;
   INC   DE			;next pattern display byte
   XOR   A
   LD    (DE),A
;
;  Setup second 4 pixels of display character scan line (left to right)
;
   LD    BC,(FNT_BCKGND)	;B=Background, C=Foreground
   BIT   3,(HL)			;Bit 3 set?
   CALL  NZ,FNT_SetChrCol_F	;Yes
   CALL  Z,FNT_SetChrCol_B	;No
;
   SRL   B
   SRL   C
   BIT   2,(HL)			;Bit 2 set?
   CALL  NZ,FNT_SetChrCol_F	;Yes
   CALL  Z,FNT_SetChrCol_B	;No
;
   SRL   B
   SRL   C
   BIT   1,(HL)			;Bit 1 set?
   CALL  NZ,FNT_SetChrCol_F	;Yes
   CALL  Z,FNT_SetChrCol_B	;No
;
   SRL   B
   SRL   C
   BIT   0,(HL)			;Bit 0 set?
   CALL  NZ,FNT_SetChrCol_F	;Yes
   CALL  Z,FNT_SetChrCol_B	;No
;
   INC   DE			;next pattern display byte
   INC   HL			;next font pattern byte
   POP   BC
   DJNZ  FNT_CnvM2toM1
;
   LD    DE,FNT_ChrPattMode1
   RET
;
;  Convert mode 1 scanline pattern to mode 2 scanline pattern
;
FNT_CnvM1toM2
   PUSH  BC
   XOR   A
   LD    (DE),A
;
   LD    B,&80			;Bit mask for mode 2 ink 1
   LD    C,&88			;bit mask left-most pixel pattern
   CALL  FNT_Indx2COlor		;Is mode 1 ink 2 or 3?
   CALL  NZ,FNT_SetChrCol_B	;Yes (set for mode 2)
   SRL   B
   SRL   C
   CALL  FNT_Indx2COlor		;Is mode 1 ink > 0?
   CALL  NZ,FNT_SetChrCol_B	;Yes (set for mode 2)
   SRL   B
   SRL   C
   CALL  FNT_Indx2COlor		;Is mode 1 ink > 0?
   CALL  NZ,FNT_SetChrCol_B	;Yes (set for mode 2)
   SRL   B
   SRL   C
   CALL  FNT_Indx2COlor		;Is mode 1 ink > 0?
   CALL  NZ,FNT_SetChrCol_B	;Yes (set for mode 2)
;
   INC   HL			;next mode 1 byte for char pattern
;
   SRL   B
   LD    C,&88			;bit mask left-most pixel pattern
   CALL  FNT_Indx2COlor		;Is mode 1 ink > 0?
   CALL  NZ,FNT_SetChrCol_B	;Yes (set for mode 2)
   SRL   B
   SRL   C
   CALL  FNT_Indx2COlor		;Is mode 1 ink > 0?
   CALL  NZ,FNT_SetChrCol_B	;Yes (set for mode 2)
   SRL   B
   SRL   C
   CALL  FNT_Indx2COlor		;Is mode 1 ink > 0?
   CALL  NZ,FNT_SetChrCol_B	;Yes (set for mode 2)
   SRL   B
   SRL   C
   CALL  FNT_Indx2COlor		;Is mode 1 ink > 0?
   CALL  NZ,FNT_SetChrCol_B	;Yes (set for mode 2)
;
   INC   DE
   INC   HL
   POP   BC
   DJNZ  FNT_CnvM1toM2
;
   LD    DE,FNT_ChrPattMode1
   RET
;=========================================================================
;Name     FNT_SetColour
;Purpose  To setup the foreground/background colours for mode 1.
;In Regs  AF = Colours (L=Foreground, U=Background)
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  None
;Out Regs AF = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FNT_FORGND = forgorund colour mode 1
;         FNT_BCKGND = background colour mode 1
;Uses     TBD
;Call By  TBD
;Calls    FNT_Color2Indx
;Notes    None
;=========================================================================
FNT_SetColour
   PUSH  BC
   LD    C,A
   AND   &0F
   CALL  FNT_Color2Indx
   LD    A,B
   LD    (FNT_FORGND),A
   LD    A,C
   SRL   A
   SRL   A
   SRL   A
   SRL   A
   AND   &0F
   CALL  FNT_Color2Indx
   LD    A,B
   LD    (FNT_BCKGND),A
   POP   BC
   RET
;=========================================================================
;Name     FNT_ShowFontChar
;Purpose  To display the font char on the screen.
;In Regs  A = 7-bit ASCII character number
;         BC = Current CPC screen stream start position
;              C = Row, B = Column
;         D = Colour index for foreground (L)/background (U)
;         E = font type (0=current font, 255=force normal)
;         HL = Current CPC screen position
;              L = Row, H = Column
;         IX = ??
;         IY = ??
;In Vars  FNT_FontAddr = address of selected pattern data for chars
;         FNT_FontTyp = type of font being displayed
;                       0 = Fixed space font
;                       >1 = font by selection.
;Out Regs AF = ??
;         BC = original CPC screen start position.
;         D = Error Status (0=Cursor position invalid else 0)
;         HL = original row/column
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    FNT_CalcScrnAddr
;         FNT_FindChar
;         FNT_OnScreen
;         FNT_ShowChrPattrn
;         FNT_ShowPropFont
;Notes    1. The CPC screen columns are limited to between 1 and 80.
;         2. This routine does not auto scroll the screen if the position
;            invalid. Where the position becomes invalid the routine
;            simply exits with error D=0).
;=========================================================================
FNT_ShowFontChar
   LD    (FNT_CharNum),A
;
   LD    A,D
   CALL  FNT_SetColour
;
   LD    D,0			;Assume function not-successful
   ADD   HL,BC			;calc absolute screen positions from 0
;
FNT_ShwFontChar0b
   CALL  FNT_OnScreen
   JR    NC,FNT_ShwFontChar1	;Not on screen
;
   LD    (FNT_CharPosOff),BC	;Row/Column Offset
   LD    (FNT_CharPosn),HL	;Row/Column of character to display
   PUSH  DE
   CALL  FNT_CalcScrnAddr
   LD    (FNT_CurrScrAdr),HL	;Save screen address of byte data for chars
   POP   DE
   LD    HL,(FNT_FontAddr)	;address of selected font set
   LD    A,(FNT_EffectsMask)
   LD    C,A
   LD    A,(FNT_FontTyp)	;Get selected font type
   LD    B,A
   LD    A,E
   OR    A			;Force use of current font?
   JR    Z,FNT_ShwFontChar0	;No
   LD    HL,FNT_FontsStore	;default 'normal' font character set
   LD    B,0			;fixed space font
   LD    C,0
FNT_ShwFontChar0
   LD    A,B
   LD    (FNT_FontMode),A	;save type of font
   LD    A,(FNT_CharNum)
   AND   127			;ensure char between 0-127
   SUB   A,32			;reset to base 0 for FNT_FindChar
   PUSH  BC
   CALL  FNT_FindChar		;HL = address of font data.
   POP   BC
;
   LD    A,(FNT_FontMode)
   CP    1			;Proportional font?
   JR    Z,FNT_ShwFontChar3	;Yes
;
;  Display new font char
;
   LD    A,(FNT_CurrChPatShift)
   CP    255			;Previous char was proportional font?
   JR    Z,FNT_ShwFontChar0a	;No (show current character)
;
   LD    A,255
   LD    (FNT_CurrChPatShift),A ;set next proportional shift factor
   LD    HL,(FNT_CharPosn)
   INC   H			;next position for new font
   LD    (FNT_CharPosn),HL	;setup Row/Column
   JR    FNT_ShwFontChar0b	;check if beyond screen
;
FNT_ShwFontChar0a
   EX    DE,HL			;DE now has address of pattern data
   CALL  FNT_ShowChrPattrn	;Display Pattern data
   LD    A,255
   LD    (FNT_CurrChPatShift),A	;Clear shift flag for prop fonts.
   LD    HL,(FNT_CharPosn)
   INC   H			;next position for new font
   LD    (FNT_CharPosn),HL	;setup Row/Column
FNT_ShwFontChar2
   LD    D,1			;flag displayed char
   LD    HL,(FNT_CharPosn)
   LD    BC,(FNT_CharPosOff)
FNT_ShwFontChar1
   OR    A
   SBC   HL,BC			;restore screen position on return
   LD    A,(FNT_FontMode)
   RET
;
FNT_ShwFontChar3
;
;  Proportional font handling starts here
;
   LD    DE,FNT_NextChPat
   LD    BC,FontChrDefSz-1
   LDIR				;Copy pattern to temp area
   LD    A,(HL)
   LD    DE,FNT_NextChPatShift
   LD    (DE),A
;
   CALL  FNT_ShowPropFont	;show proportional font data
   JR    FNT_ShwFontChar2
;=========================================================================
;Name     FNT_CalcScrnAddr
;Purpose  To calculate the physical memory address in the CPC screen to
;         be where the cursor position is to be.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  FNT_CharPosn = location of cursor
;         SCR_Mode = screen mode
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     All registers
;Call By  TBD
;Calls    TBD
;Notes    1. The FNT_CharPosn has the cursor position which starts from 0.
;         2. Algorithm to calculate the start memory position is -
;
;            MemoryStart = BaseAddress + Column + (Row x BytesPerScanLine)
;                        = &C000 + Column + (Row x 80)
;                          For mode 1, Column = 2 * mode2Column
;
;            The reason for this algorithm is due to the fact the screen
;            in mode 2 has 80 column by 25 rows which matches the screen
;            configuration in hardware. Each 2K block of screen memory is
;            used to contain the nth scan line of each row of mode text,
;            which explains why the screen memory broken down into the 8
;            areas.
;         3. Each scan line is then accessed by adding 2048.
;         4. The routine has been designed for mode 2 operation, however
;            for modes 1 and 0 the byte at the address defines the number
;            of pixels available. For mode 1 this is 4 pixels per byte and
;            for mode 0 it is 2 pixels. This means it takes double the
;            number of bytes to show a single character scan line for each
;            reduced mode.
;=========================================================================
FNT_CalcScrnAddr
   LD    HL,FNT_BaseScreen
   LD    DE,(FNT_CharPosn)	;D = Column, E = Row
   LD    B,E			;Save row
   LD    E,D
   LD    D,0			;DE = column as byte offset
   ADD   HL,DE			;convert column to scanline byte offset
   LD    A,(SCR_Mode)
   CP    2
   JR    Z,FNT_CalcScrnAddr0
   ADD   HL,DE			;double column as mode 1 uses 2 bytes
FNT_CalcScrnAddr0
   LD    A,B
   OR    A			;Row 0?
   RET   Z			;Yes
;
   LD    DE,FNT_ChrsPerLine	;number of bytes per scan line
FNT_CalcScrnAddr1
   ADD   HL,DE			;calc screen row for start of byte data.
   DEC   A
   JR    NZ,FNT_CalcScrnAddr1
   RET
;=========================================================================
;Name     FNT_FindChar
;Purpose  To find the address of a specific character pattern to display.
;In Regs  A = Character number from 0
;         BC = ??
;         DE = ??
;         HL = Address of Font/icon pattern data.
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = Address of specific character.
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     All registers
;Call By  TBD
;Calls    None
;Notes    1. The routine merely calculates the address of the character.
;         2. To calculate the character index based on the actual ASCII
;            value, the value has 32 subtracted first.
;=========================================================================
FNT_FindChar
   OR    A
   RET   Z
   LD    DE,FontChrDefSz
FNT_FindChar1
   ADD   HL,DE
   DEC   A
   JR    NZ,FNT_FindChar1
   RET
;=========================================================================
;Name     FNT_FindIconChar
;Purpose  To find the specific icon character pattern to display.
;In Regs  A = Icon number (range 0 to 31 inclusive)
;         BC = ??
;         DE = ??
;         HL = Address of icon patterns data.
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = Address of specific icon character.
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     All registers
;Call By  TBD
;Calls    None.
;Notes    None.
;=========================================================================
FNT_FindIconChar
   LD    DE,FontChrDefSz1	;Size of Icon pattern data
   SET   7,A			;ensure matching with bit 7=1
FNT_FindIChar1
   CP    (HL)
   RET   Z
   ADD   HL,DE
   JR    FNT_FindIChar1
FNT_EndCode	DEFB	0
;==========================================================================
;
;   FNT_IconsStore = Data store for all icon files
;   FNT_FontsStore = Data store for all Font Files
;
;==========================================================================
FNT_IconsStore	EQU	FNT_EndCode
FNT_FontsStore	EQU	FNT_IconsStore+IconDataSz