;=========================================================================
;
; Application Routines (HTML rendering commands - Forms)
; Copyright (c) 2010 Ray Palmer
;
; Notes
;
; 1. The rendering of the current command does not expect that the same
;    command to be recursed. This may lead to an infinte loop if attempted.
;
; 2. Where a command changes the font (e.g normal to italic) the original
;    command will have no way of reverting the font back to what it was.
;
; 3. Forms information is rendered as the forms database is created.
;
;=========================================================================
;
;  HTML command handlers
;
;=========================================================================
;Name     html_cmd_form
;Purpose  To handle the html command <form....> to handle a form of
;         input.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of web-page rendering
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_CmdEnd = char after </form...> command
;Uses     TBD
;Call By  TBD
;Calls    HTM_Get1stCmdChr
;         HTM_FormFindHTML
;         HTM_FormParms
;         HTM_ObjectBrdr
;         HTM_FormItem2DB
;Notes    1. The use of forms is not expected to be nested, for example
;            <form ....>
;              <form ....>
;              </form>
;            </form>
;
;            The specification makes no mention of recursion, but for
;            consistency with normal usage, the occurence may cause the
;            browser to fail.
;=========================================================================
html_cmd_form
   CALL  HTM_Get1stCmdChr		;</...> command ?
   JR    Z,html_cmd_form2		;Yes
;
   CALL  HTM_FormFindHTML		;Find <Form> in Database
   CP    HTML_P_EODB			;Did we find it?
   RET   NZ				;Yes
;
   CALL  HTM_FormParms			;process parameters
;
   LD    HL,CFG_FormsBuf		;Start memory allocs for text
   LD    (HTML_FrmsMalloc),HL
;
   LD    HL,HTML_FormsFlag
   SET   7,(HL)				;Flag collecting forms info
   LD    HL,HTML_FormsPF
   SET   7,(HL)				;flag now in forms processing
   RES   0,(HL)				;flag started new form
;
   LD    HL,HTML_FrmsIDNum
   INC   (HL)				;Next form Id for Db entires
   XOR   A
   LD    (HTML_FrmsOptG),A		;reset grouping for this form
   LD    (HTML_FrmsTab),A		;reset tab index
   LD    BC,(HTML_Row)
   LD    (HTML_FrmsBRow),BC
   LD    BC,(HTML_Col)
   LD    (HTML_FrmsBCol),BC
;
   LD    HL,HTML_BordrSt
   BIT   0,(HL)				;Form object has border ?
   JR    Z,html_cmd_form3		;No
   SET   6,(HL)
   RES   5,(HL)
   CALL  HTM_ObjectBrdr
;
html_cmd_form3
   LD    C,HTML_I_FORM
   LD    B,0
   JP    HTM_FormItem2DB		;add item to flag form start
;
;  Found end of forms data
;
html_cmd_form2
   CALL  html_cmd_form3			;add end of form marker
   LD    HL,HTML_FormsPF
   RES   7,(HL)				;flag processing a form
   LD    HL,HTML_FormsFlag
   RES   7,(HL)				;Flag not collecting forms info
   LD    HL,HTML_BordrSt
   BIT   0,(HL)				;Form object has border ?
   RET   Z				;No
   JP    HTM_ObjectBrdr
;=========================================================================
;Name     html_cmd_input
;Purpose  To handle the html command <input....> to handle a form of
;         input.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_FrmsTab = tabbing index
;         HTML_RenderState = state of web-page rendering
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_ParamLoc = corrupted.
;Uses     TBD
;Call By  TBD
;Calls    HTM_Get1stCmdChr
;         HTM_FormFindHTML
;         HTM_GetCmdParam
;         HTM_FindParmLbl
;         HTM_GetCmdParmVal
;         UTL_readdec2
;         HTM_SetActive
;         HTM_ClrActive
;         HTM_FrmsGetInpSz
;         HTM_FormItem2DB
;         HTM_FormDisplayItem
;Notes    1. The tab index can only handle upto value 255.
;=========================================================================
html_cmd_input
   CALL  HTM_Get1stCmdChr		;</...> command ?
   RET   Z				;Yes (no action)
;
   CALL  HTM_FormFindHTML		;Find Form item in Database
   CP    HTML_P_EODB			;Did we find it?
   JP    NZ,HTM_FormDisplayItem		;Yes
;
   LD    BC,0
   LD    (HTML_FrmsNRows),BC		;Assume one row/one column
   LD    (HTML_FrmsTxtLoc),BC
;
   LD    A,(HTML_FrmsTab)
   INC   A
   LD    (HTML_FrmsTab),A		;setup next tab index
;
;  Handle the <INPUT> parameters
;
   LD    HL,(HTML_CmdTemp)
   LD    (HTML_ParamLoc),HL
html_cmd_inp0
   CALL  HTM_GetCmdParam		;Locate next parameter
   LD    A,(HL)
   CP    EHTML_CHR			;no more parameters?
   JR    Z,html_cmd_inp4		;Yes (save item into DB)
   PUSH  HL
   CALL  HTM_FindParmLbl		;Determine the parameter nametype
   POP   HL
   LD    A,HTM_P_tabindex
   CP    B				;Is parameter 'tabindex' ?
   JR    Z,html_cmd_inp2		;Yes
   LD    A,HTM_P_checked
   CP    B				;Is parameter 'checked' ?
   JR    Z,html_cmd_inp3		;Yes
   LD    A,HTM_P_value
   CP    B				;Is parameter 'value' ?
   JR    Z,html_cmd_inp6		;Yes
   LD    A,HTM_P_Type
   CP    B				;Is parameter 'type' ?
   JR    NZ,html_cmd_inp0		;No
;
;  Handle the different input types
;
   CALL  HTM_GetCmdParmVal		;HL = address of parameter value
   CALL  HTM_FindParmLbl		;determine type of input.
   LD    A,B
   SUB   HTM_P_Chkbox			;adjust to be index from 0
   LD    (HTML_FrmsTyp),A		;Save input type to add 2 db
   JR    html_cmd_inp0			;next parameter
;
html_cmd_inp2
;
;  Handle the 'tabindex' parameter
;
   CALL  HTM_GetCmdParmVal		;HL = address of parameter value
   EX    DE,HL
   CALL  UTL_readdec2			;Get tabindex in HL
   LD    A,L
   LD    (HTML_FrmsTab),A
   JR    html_cmd_inp0
;
html_cmd_inp3
;
;  Handle the 'checked' parameter
;
   CALL  HTM_SetActive
   JR    html_cmd_inp0
;
html_cmd_inp6
   CALL  HTM_GetCmdParmVal		;HL = address of parameter value
   LD    (HTML_LblTemp),HL		;save address of value for button
   JR    html_cmd_inp0
;
html_cmd_inp4
;
;  Process the <INPUT> command parameters
;
   LD    A,(HTML_FrmsTyp)
   AND   HTML_I_FORM
   CALL  HTM_FrmsGetInpSz		;get size of input type
   LD    A,C
   LD    (HTML_FrmsNCols),A		;setup size of input cols
   LD    A,B
   LD    (HTML_FrmsNRows),A		;setup size of input rows
;
   LD    A,(HTML_FrmsTyp)
   AND   HTML_I_FORM
   CP    HTML_I_Password		;Password input
   JR    NZ,html_cmd_inp5		;No
   CALL  HTM_ClrActive			;no preset for password
   LD    A,(HTML_FrmsTyp)
   SET   6,A				;Flag masked input/output.
html_cmd_inp5
   LD    C,A
   SET   4,C				;flag item is to be shown
   SET   7,C
   LD    B,0
   CALL  HTM_FormItem2DB		;add input item to DB
;
   JP    HTM_FormDisplayItem		;Display the initial item
;=========================================================================
;Name     html_cmd_textarea
;Purpose  To handle the html command <textarea....> to handle a form of
;         input.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of web-page rendering
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    HTM_Get1stCmdChr
;         HTM_FormFindHTML
;         HTM_textareaSz
;         HTM_FormItem2DB
;         HTM_FormDisplayItem
;Notes    TBD
;=========================================================================
html_cmd_textarea
   CALL  HTM_Get1stCmdChr		;</...> command ?
   RET   Z					;Yes (no action)
;
   CALL  HTM_FormFindHTML		;Find Form item in Database
   CP    HTML_P_EODB			;Did we find it?
   JR    NZ,html_cmd_txtarea2		;Yes
;
   LD    A,(HTML_FrmsTab)
   INC   A
   LD    (HTML_FrmsTab),A		;setup next tab index
;
   CALL  HTM_textareaSz			;setup fixed text area size
;
;  Add item to forms DB
;
   LD    B,B_5				;Multi-line/unhidden
   LD    C,HTML_I_TXTBOX
   SET   4,C				;Flag item is to be shown
   SET   7,C				;Flag size known
   CALL  HTM_FormItem2DB		;add input item to DB
;
html_cmd_txtarea2
   JP    HTM_FormDisplayItem
;=========================================================================
;Name     html_cmd_button
;Purpose  To handle the html command <button....> to handle a form of
;         input. This button is a general PUSH button, which executes a
;         script (like java) to perform a function (if the button is not
;         reset or submit).
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of web-page rendering
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    HTM_Get1stCmdChr
;         HTM_FrmUpdSz0
;         HTM_FormFindHTML
;         HTM_GetCmdParam
;         HTM_FindParmLbl
;         HTM_GetCmdParmVal
;         HTM_FormItem2DB
;         HTM_FormDisplayItem
;Notes    1. The position of PUSH button will not assume they occur on
;            new lines, since it's best to leave the rendering format for
;            the user to arrange.
;         2. The size of a button is based on the content between the
;            <button>.....</button> commands.
;=========================================================================
html_cmd_button
   CALL  HTM_Get1stCmdChr		;</...> command ?
   JR    NZ,html_cmd_but0		;No
;
   CALL  HTM_FrmUpdSz0			;Update size of button
   JP    HTM_FormsUpdDD
;
html_cmd_but0
;
;  Setup base location of button render (if any)
;
   LD    DE,(HTML_Row)
   LD    (HTML_FrmsBRow),DE
   LD    DE,(HTML_Col)
   LD    (HTML_FrmsBCol),DE
   LD    DE,0
   LD    (HTML_ImgRow),DE
;
   CALL  HTM_FormFindHTML		;Find Form item in Database
   CP    HTML_P_EODB			;Did we find it?
   JR    NZ,html_cmd_but5		;Yes
;
   LD    HL,HTML_FrmsTab
   INC   (HL)				;setup next tab index
;
;  Process the parameter to the command
;
   LD    HL,(HTML_CmdTemp)
   LD    (HTML_ParamLoc),HL
html_cmd_but2
   CALL  HTM_GetCmdParam		;Locate next parameter
   LD    A,(HL)
   CP    EHTML_CHR			;no more parameters?
   JR    Z,html_cmd_but3		;Yes (save item into DB)
   PUSH  HL
   CALL  HTM_FindParmLbl		;Determine the parameter nametype
   POP   HL
   LD    A,HTM_P_Type
   CP    B				;Is parameter 'type' ?
   JR    NZ,html_cmd_but2		;No
;
;  Handle the different input types
;
   CALL  HTM_GetCmdParmVal		;HL = address of parameter value
   CALL  HTM_FindParmLbl		;determine type of input.
   LD    A,B
   SUB   HTM_P_Chkbox			;adjust to be index from 0
   LD    C,HTML_I_BUTTON
   CP    HTML_I_SUBMIT			;type = "submit" ?
   JR    Z,html_cmd_but4		;Yes
   CP    HTML_I_RESET			;type = "reset" ?
   JR    Z,html_cmd_but4		;Yes
   LD    C,HTML_I_BUTTONC		;reset to custom button
html_cmd_but4
   LD    A,(HTML_FrmsTyp)
   AND   &F0
   OR    C
   SET   4,A				;Flag item is to be shown
   LD    (HTML_FrmsTyp),A
   JR    html_cmd_but2
;
html_cmd_but3
;
;  Add item to forms DB
;
   LD    A,(HTML_FrmsTyp)
   LD    C,A
   LD    B,0				;no init value
   CALL  HTM_FormItem2DB		;add input item to DB
;
html_cmd_but5
   LD    HL,HTML_FormsFlag
   BIT   7,(HL)				;collecting forms info?
   RET   NZ				;Yes
   JP    HTM_FormDisplayItem
;=========================================================================
;Name     html_cmd_option
;Purpose  To handle the html command <option....> to handle a form of
;         input for a grouped set of options.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = State of rendering
;         HTML_FormsFlag = State of handling forms
;         HTML_CmdEnd = HTML address for end of HTML command
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    HTM_Get1stCmdChr
;         HTM_FormFindHTML
;         HTM_ClrShown
;         HTM_SetShown
;         HTM_ClrLabel
;         HTM_GetCmdParam
;         HTM_FindParmLbl
;         HTM_SetIniActive
;         HTM_SkipLblTxt
;         HTM_SetLabel
;         HTM_FormItem2DB
;         HTM_NewLine
;         HTM_FormDisplayItem
;Notes    1. No check is made to see if multi-select is enabled or not,
;            so it is possible to have many options enabled when only one
;            is allowed.
;=========================================================================
html_cmd_option
   CALL  HTM_Get1stCmdChr		;</...> command ?
   RET   Z				;Yes
;
   CALL  HTM_FormFindHTML		;Find Form item in Database
   CP    HTML_P_EODB			;Did we find it?
   JR    NZ,html_cmd_opt3		;Yes
;
   LD    HL,HTML_FrmsTab
   INC   (HL)				;setup next tab index
;
   CALL  HTM_ClrShown			;Assume hidden by group label.
   CALL  HTM_ClrLabel			;Assume no label text
   RES   6,(HL)				;Assume not selected
   BIT   0,(HL)				;In option group?
   CALL  Z,HTM_SetShown			;No (show option text)
;
   LD    HL,(HTML_CmdTemp)
   LD    (HTML_ParamLoc),HL
html_cmd_opt0
   CALL  HTM_GetCmdParam		;Locate next parameter
   LD    A,(HL)
   CP    EHTML_CHR			;no more parameters?
   JR    Z,html_cmd_opt1		;Yes (no more info)
   PUSH  HL
   CALL  HTM_FindParmLbl		;Determine the parameter nametype
   POP   HL
   LD    A,HTM_P_selected
   CP    B				;Is parameter 'selected' ?
   CALL  Z,HTM_SetIniActive		;Yes, flag option is active.
   LD    A,HTM_P_Label2
   CP    B				;Is parameter 'Label' ?
   JR    Z,html_cmd_opt4		;Yes
   JR    html_cmd_opt0
;
html_cmd_opt4
   CALL  HTM_SkipLblTxt			;HL = address of label text
   LD    (HTML_FrmsTxtLoc),HL
   CALL  HTM_SetLabel
   JR    html_cmd_opt0
;
html_cmd_opt1
;
;  Add item to forms DB
;
   LD    BC,0
   LD    (HTML_FrmsNRows),BC		;Assume one row/one column
   LD    A,(HTML_FrmsTyp)
   AND   &F0
   OR    HTML_I_OPTION
   LD    C,A
   LD    A,(HTML_FrmsIniVal)
   LD    B,A
   CALL  HTM_FormItem2DB		;add input item to DB
;
html_cmd_opt3
;
   LD    A,(HTML_FrmsTyp)
   BIT   4,A				;Option is to be displayed?
   JP    Z,HTM_SkipCmdObj		;No (skip any data)
   LD    HL,HTML_FormsPF
   BIT   1,(HL)				;Form processing underway
   CALL  NZ,HTM_NewLine			;Yes (show a new line)
   LD    HL,HTML_FrmsIniVal
   BIT   0,(HL)				;option in a group?
   JR    Z,html_cmd_opt2		;No
   LD    BC,(HTML_Col)
   INC   BC				;indent for grouped items
   LD    (HTML_Col),BC
html_cmd_opt2
   JP    HTM_FormDisplayItem		;display option Icon
;=========================================================================
;Name     html_cmd_optgroup
;Purpose  To handle the html command <optgroup....> to handle a form of
;         input.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_FormsFlag = bit 7 (forms info collection flag)
;         HTML_RenderState = state of web-page rendering
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_FormsFlag = bit 1 set/cleared
;Uses     TBD
;Call By  TBD
;Calls    HTM_Get1stCmdChr
;         HTM_FormFindHTML
;         HTM_ClrLabel
;         HTM_GetCmdParam
;         HTM_FindParmLbl
;         HTM_SkipLblTxt
;         HTM_SetLabel
;         HTM_FormItem2DB
;         HTM_NewLine
;         HTM_FormDisplayItem
;Notes    1. Currently can not handle nested groupings.
;         2. All option groups are displayed by default.
;=========================================================================
html_cmd_optgroup
   CALL  HTM_Get1stCmdChr		;</...> command ?
   JR    NZ,html_cmd_optgrp0		;No
;
   LD    HL,HTML_FrmsIniVal
   RES   0,(HL)				;flag no longer in a group
   RET
;
html_cmd_optgrp0
   CALL  HTM_FormFindHTML		;Find Form item in Database
   CP    HTML_P_EODB			;Did we find it?
   JR    NZ,html_cmd_optgrp3		;Yes
;
   LD    HL,HTML_FrmsTab
   INC   (HL)				;setup next tab index
;
   LD    HL,HTML_FrmsOptG
   INC   (HL)				;Next option group number
   CALL  HTM_ClrLabel			;Assume no label text
;
   LD    HL,(HTML_CmdTemp)
   LD    (HTML_ParamLoc),HL
html_cmd_optgrp1
   CALL  HTM_GetCmdParam		;Locate next parameter
   LD    A,(HL)
   CP    EHTML_CHR			;no more parameters?
   JR    Z,html_cmd_optgrp2		;Yes (no more info)
   PUSH  HL
   CALL  HTM_FindParmLbl		;Determine the parameter nametype
   POP   HL
   LD    A,HTM_P_Label2
   CP    B				;Is parameter 'Label' ?
   JR    NZ,html_cmd_optgrp1
   CALL  HTM_SkipLblTxt			;HL = address of label text
   LD    (HTML_FrmsTxtLoc),HL
   CALL  HTM_SetLabel
   JR    html_cmd_optgrp1
;
html_cmd_optgrp2
;
;  Add item to forms DB
;
   LD    A,(HTML_FrmsIniVal)
   AND   B_4+B_3			;mask out Select/label flags
   OR    B_0				;Now in a group of options
   LD    B,A
   LD    C,HTML_I_OPTGRP+B_4
   CALL  HTM_FormItem2DB		;add input item to DB
;
html_cmd_optgrp3
   LD    HL,HTML_FormsPF
   BIT   1,(HL)				;Form processing underway
   CALL  NZ,HTM_NewLine			;Yes (show a new line)
   JP    HTM_FormDisplayItem
;=========================================================================
;Name     html_cmd_select
;Purpose  To handle the html command <select....> to handle a form of
;         input.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of web-page rendering
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    HTM_Get1stCmdChr
;         HTM_GetCmdParam
;         HTM_FindParmLbl
;         HTM_FormFindHTML
;         HTM_FormItem2DB
;         HTM_NewLine
;Notes    1. This HTML command does not produce any output, as the result
;            of the command is to group selections together such that on
;            clicking, the options are then selected/deselected.
;=========================================================================
html_cmd_select
   CALL  HTM_Get1stCmdChr		;</...> command ?
   JR    Z,html_cmd_sel1		;Yes (add end of selection)
;
   LD    HL,HTML_FrmsTab
   INC   (HL)				;setup next tab index
   LD    HL,HTML_FrmsIniVal
   LD    (HL),0				;clear flags
   LD    HL,HTML_FrmsSID
   INC   (HL)				;Next select ID
;
;  Process parameters to command
;
   LD    HL,(HTML_CmdTemp)
   LD    (HTML_ParamLoc),HL
html_cmd_sel0
   CALL  HTM_GetCmdParam		;Locate next parameter
   LD    A,(HL)
   CP    EHTML_CHR			;no more parameters?
   JR    Z,html_cmd_sel1		;Yes (add to database)
   PUSH  HL
   CALL  HTM_FindParmLbl		;Determine the parameter nametype
   POP   HL
   LD    A,HTM_P_Multiple
   CP    B				;Is parameter 'multiple' ?
   JR    NZ,html_cmd_sel0		;no
   LD    A,(HTML_FrmsIniVal)
   SET   3,A				;Flag many selectable.
   LD    (HTML_FrmsIniVal),A
   JR    html_cmd_sel0
;
html_cmd_sel1
   CALL  HTM_FormFindHTML		;Find Form item in Database
   CP    HTML_P_EODB			;Did we find it?
   JP    NZ,html_cmd_sel2		;Yes
;
;  Add item to forms DB
;
   LD    A,(HTML_FrmsIniVal)
   LD    B,A
   LD    C,HTML_I_SELECT
   CALL  HTM_FormItem2DB		;add input item to DB
;
html_cmd_sel2
   CALL  HTM_Get1stCmdChr		;</...> command ?
   RET   NZ
   LD    HL,HTML_FormsPF
   SET   1,(HL)				;Form processing underway
   JP    HTM_NewLine			;Yes
