;=========================================================================
;
; Application Routines (HTML rendering commands)
; Copyright (c) 2010 Ray Palmer
;
; Notes
;
; 1. The rendering of the current command does not expect that the same
;    command to be recursed. This may lead to an infinte loop.
;
; 2. Where a command changes the font (e.g normal to italic) the original
;    command will have no way of reverting the font back to what it was.
;
; 3. The frame definitions are defined to start from (1,1) while the
;    rendering row and columns start from 0. This was chosen so that the
;    screen calculations match the CPC screen location references which
;    start from (1,1).
;=========================================================================
;
;  HTML command handlers
;
;=========================================================================
;Name     html_cmd_Iframe
;Purpose  To handle the html command <iframe...>.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
html_cmd_Iframe
   RET
;=========================================================================
;Name     html_cmd_frame
;Purpose  To handle the html command <frame> to assign the HTML to a
;         frame defined by the frameset command(s).
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_FrmeSets = current state of frameset handling
;         HTML_RCurrPos = HTML location of frame command
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_FrameFlg = bit 8 set
;Uses     TBD
;Call By  TBD
;Calls    HTM_DefFrame
;Notes    1. This routine can be executed outside of the normal FRAME
;            definition and as such may result in unknown behaviour.
;=========================================================================
html_cmd_frame
   LD    A,(HTML_FrmeSets)
   AND   &22
   CP    &22				;Are Frames enabled?
   RET   Z					;No
;
   LD    HL,(HTML_CmdTemp)
   LD    (HTML_ParamLoc),HL
html_cmd_frme1
   CALL  HTM_GetCmdParam		;Locate next parameter
   LD    B,0				;Assume borderless frame
   LD    A,(HL)
   CP    '>'				;End of command parameters?
   JR    Z,html_cmd_frme2		;Yes
;
   CALL  HTM_FindParmLbl		;Determine the parameter nametype
   LD    A,HTM_P_frameborder
   CP    B					;Is parameter 'frameborder' ?
   JR    NZ,html_cmd_frme1		;No
   LD    B,1				;frame has border
;
html_cmd_frme2
;
   LD    A,B
   LD    (HTML_FrameBS),A		;Save Border Status
   CALL  HTM_DefFrame			;Define a new frame in frames DB
   LD    HL,HTML_FrameFlg
   SET   7,(HL)
   LD    A,(FrameCtr)
   INC   A					;Tally number of frames to load
   LD    (FrameCtr),A
   RET
;=========================================================================
;Name     html_cmd_frameset
;Purpose  To handle the html command <frameset>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_CmdTemp = start address of HTML command
;         HTML_FrmeSets = State of frameset definition/flags
;         HTML_FrmeSetN = Current nested count of command
;                         0 = first occurrence of command
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_ParamLoc = corrupted
;         HTML_FrmeSets = updated state of frameset definition/flags
;         HTML_FrmeSetN = Current nested count of command
;Uses     TBD
;Call By  TBD
;Calls    HTM_Get1stCmdChr
;         HTM_GetCmdParam
;         HTM_FindParmLbl
;         HTM_GetFrmesCols
;         HTM_GetFrmesRows
;         HTM_DefFrames
;Notes    1. The HTML processing of nested frameset is limited to the
;            definition of rows (or columns) followed by the opposite
;            type. Therefore defining more rows with in a frame cannot be
;            handled.
;            So to define a frame and then define further internal frames
;            within it are not possible as the code has not been designed
;            to handle such an occurrence.
;            Where this is tried the results of the frames will be ignored.
;         2. In future version there may be a fix to allow further uses of
;            the frameset command to allow for more available web-page
;            formatting.
;         3. Frame 0 in the database is reserved by the system, while frame
;            1 is the first HTML which defines the subsequent frames.
;=========================================================================
html_cmd_frameset
   LD    A,(HTML_FrmeSets)
   AND   &22
   CP    &22				;Are Frames enabled?
   RET   Z				;No
;
   LD    HL,PageLock
   RES   1,(HL)				;page scroll unlocked
;
   CALL  HTM_Get1stCmdChr		;end of command ?
   JR    NZ,html_cmd_frmeset3		;No
;
html_cmd_frmesetEx
   LD    HL,HTML_FrmeSetN
   DEC   (HL)				;dec nesting count
   RET   NZ					;No at first command
   LD    HL,HTML_RenderState
   RES   3,(HL)				;release renderer control
   RET
;
html_cmd_frmeset3
   LD    HL,HTML_FrmeSetS
   BIT   7,(HL)				;frames now being used?
   JR    NZ,html_cmd_frmeset0		;Yes
   LD    (HL),0				;Flag now using frames
;
   LD    HL,0
   LD    (HTML_FrmeSetI),HL		;clear row/col array indice
   INC   H
   INC   L
   LD    (HTML_FrmeCCtr),HL		;clear row/col array counts
   LD    HL,HTML_RenderState
   SET   3,(HL)				;contol renderer (deny output)
;
html_cmd_frmeset0
   LD    HL,HTML_FrmeSetN
   INC   (HL)				;inc nesting count
   LD    HL,(HTML_CmdTemp)
   LD    (HTML_ParamLoc),HL
;
;  Process the parameters to setup the array of rows and/or columns
;  definitions.
;
html_cmd_frmeset1
   CALL  HTM_GetCmdParam		;Locate next parameter
   LD    A,(HL)
   CP    '>'				;End of command parameters?
   JR    Z,html_cmd_frmesetEx		;Yes
;
   CALL  HTM_FindParmLbl		;Determine the parameter nametype
   LD    A,HTM_P_rows
   CP    B					;Is parameter 'rows' ?
   JR    Z,html_cmd_frmeset2		;Yes
   LD    A,HTM_P_cols
   CP    B					;Is parameter 'cols' ?
   JR    NZ,html_cmd_frmeset1		;No
;
;  Setup the columns values into the columns array.
;
   LD    A,(HTML_FrmeSets)
   BIT   2,A				;Cols already defined?
   CALL  Z,HTM_GetFrmesCols		;No, Get Col defs for frames
   JR    html_cmd_frmeset1
;
html_cmd_frmeset2
;
;  Setup the row values into the rows array.
;
   LD    A,(HTML_FrmeSets)
   BIT   6,A				;Rows already defined?
   CALL  Z,HTM_GetFrmesRows		;No, Get Row defs for frames
   JR    html_cmd_frmeset1
;=========================================================================
;Name     html_cmd_noframes
;Purpose  To handle the html command <noframes>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_CmdEnd = end Location of HTML command/HTML text which is
;                       the start of displayable HTML.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_CmdEnd =  address of <noframe> content.
;Uses     TBD
;Call By  TBD
;Calls    HTM_FrmeCalcAddr
;Notes    TBD
;=========================================================================
html_cmd_noframes
   LD    A,(HTML_FrmeSets)
   AND   &22
   CP    &22				;Is Frames enabled?
   JR    NZ,html_cmd_noframes1	;Yes (skip content)
;
   CALL  HTM_Get1stCmdChr		;end of command ?
   RET   Z					;Yes
;
   LD    A,1
   CALL  HTM_FrmeCalcAddr		;Get default frame DB entry address
   LD    BC,S_HTMLFramAd
   ADD   HL,BC
   EX    DE,HL
   LD    HL,HTML_CmdEnd
   LD    BC,3
   LDIR					;Insert HTML start Location
   JP    MEM_SwapInMain
;
html_cmd_noframes1
;
;  Skip NO FRAMES object content
;
   LD    HL,(HTML_RCurrPos)
   LD    A,(HTML_RCurrPos+2)
   LD    B,A
html_cmd_noframes2
   LD    A,B
html_cmd_noframes2a
   INC   HL
   CALL  HTM_NextHTMBank		;Swap In HTML Bank/adjust location
   LD    B,A
   LD    A,(HL)
   CP    0					;end of HTML?
   JR    Z,html_cmd_noframes3
   CP    '<'				;Start of HTML command?
   JR    NZ,html_cmd_noframes2	;No
;
   LD    A,B
   CALL  HTM_GetStartEnd		;Get HTML cmd start/end, copy cmd
   LD    HL,(HTML_CmdTemp)
   INC   HL					;Skip / if present
   LD    DE,HTML_C_NoFrames
   LD    B,8
   CALL  UTL_strncmp			;</Noframes> command?
   RET   Z					;Yes
   LD    HL,(HTML_CmdEnd)
   LD    A,(HTML_CmdEnd+2)
   JR    html_cmd_noframes2a
html_cmd_noframes3
   LD    (HTML_CmdEnd),HL
   LD    A,B
   LD    (HTML_CmdEnd+2),A
   RET
