;=========================================================================
;
; Application Routines (HTML rendering commands)
; Copyright (c) 2010 Ray Palmer
;
; Notes
;
; 1. The rendering of the current command does not expect that the same
;    command to be recursed. This may lead to an infinte loop.
;
; 2. Where a command changes the font (e.g normal to italic) the original
;    command will have no way of reverting the font back to what it was.
;
;=========================================================================
;
;  HTML command handlers
;
;=========================================================================
;Name     HTML_cmd_a
;Purpose  To handle the HTML command <a .....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;         HTML_CmdTemp = current command extracted from HTML data
;         HTML_CmdEnd = end of current command from the HTML data
;         HTML_Col = current absolute column on the web-page
;         HTML_Row = current absolute row on the web-page.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_RCurrPos = address of next command for rendering
;         HTML_ObjLocn = current object being rendered on the web-page
;         HTML_Col = updated absolute col on the web-page
;         HTML_Row = updated absolute row on the web-page.
;         HTML_CmdTemp = corrupted.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. If underline font effect is in effect then the hyperlink
;            view will be shown as other normal text setting.
;=========================================================================
HTML_cmd_a
   CALL  UTL_ChkRendState		;Collecting web-page info?
   RET   NZ				;Yes
   CALL  HTM_Get1stCmdChr		;End command Tag?
   JR    NZ,HTML_cmd_a1			;No
;
   LD    A,(HTML_href)			;get current state of <A...> command
   BIT   0,A				;Is command still active?
   RET   Z				;No
;
   CALL  HTM_FinalNewHLink		;Setup hyperlink db entry
   JP    HTM_AddDBHyperLink		;Save DB entry
;
HTML_cmd_a1
   LD    HL,(HTML_CmdTemp)
   LD    (HTML_ParamLoc),HL
HTML_cmd_a_chk1
   CALL  HTM_GetCmdParam		;Locate next parameter
   LD    A,(HL)
   CP    ">"				;more parameters?
   RET   Z				;No
   CALL  HTM_FindParmLbl		;Determine the parameter nametype
   LD    A,HTM_P_href
   CP    B				;Is parameter 'href' ?
   JR    NZ,HTML_cmd_a_chk1		;No
;
   CALL  HTM_StrtNewHLink
;
HTML_cmd_a4
   LD    HL,(HTML_CmdEnd)
   LD    A,(HTML_CmdEnd+2)
   CALL  UTL_ShowCmdText		;Display text until HTML cmd found
   CALL  HTM_GetCmd			;Get HTML command
   LD    (HTML_CmdEnd),HL		;Save HTML end of command address
   LD    A,C
   LD    (HTML_CmdEnd+2),A
;
   LD    HL,(HTML_CmdTemp)
   CALL  HTM_FindHTMLCmd		;Attempt to find HTML command
   CP    Find_Okay			;Did we find it?
   CALL  Z,HTM_DoCmdHandler		;Yes, Execute Handler for command
;
   LD    A,(HTML_href)			;get current state of <A...> command
   BIT   0,A				;Is href command still active?
   JR    NZ,HTML_cmd_a4			;Yes
   RET
;=========================================================================
;Name     html_cmd_link
;Purpose  To handle the html command <link....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
html_cmd_link
   CALL  UTL_ChkRendState	;Collecting web-page info?
   RET   NZ			;Yes
   LD    A,(HTML_headbody)
   AND   FLAGS_HEAD		;mask out start/end flags
   CP    FLAGS_HEADS		;Start of head region of HTML data ?
   RET   NZ			;No
;
;  process the link command
;
   LD    HL,HTML_href
   RES   7,(HL)			;flag dont load linked object
   LD    HL,(HTML_CmdTemp)
   LD    (HTML_ParamLoc),HL
html_cmd_lnk2
   CALL  HTM_GetCmdParam	;Locate next parameter
   LD    A,(HL)
   CP    ">"			;end of parameters?
   JR    Z,html_cmd_lnkEx	;Yes
;
   PUSH  HL
   CALL  HTM_FindParmLbl	;Determine the parameter nametype
   POP   HL
;
   LD    A,HTM_P_type
   CP    B
   JR    Z,html_cmd_lnk5
;
   LD    A,HTM_P_href
   CP    B			;Is parameter 'href' ?
   JR    NZ,html_cmd_Lnk2	;No (skip the parameter)
;
   CALL  HTM_GetCmdParmVal	;Get address of parameter value
   LD    (HTML_TmpAddr),HL	;save address of the URL
   JR    html_cmd_lnk2
;
html_cmd_lnk5
   CALL  HTM_GetCmdParmVal	;Get address of parameter value
   LD    DE,STYLE_CssTyp
   LD    B,8
   CALL  UTL_strncmpCI		;is type = "text/css" ?
   JR    NZ,html_cmd_lnk2	;No
   LD    HL,HTML_href
   SET   7,(HL)			;flag load this object now
   JR    html_cmd_lnk2
;
html_cmd_lnkEx
   LD    HL,HTML_href
   BIT   7,(HL)			;load link object now?
   RET   Z			;No
   RES   7,(HL)
;
   LD    HL,(HTML_TmpAddr)
   LD    DE,0
html_cmd_lnk3
   LD    A,(HL)
   INC   HL
   CP    DOT_CHAR		;found a '.' in the value string?
   JR    NZ,html_cmd_lnk4	;No
   LD    E,L			;Note where it is
   LD    D,H
html_cmd_lnk4
   CP    DBLEQ_CHR		;end of value string?
   JR    NZ,html_cmd_lnk3	;No
;
   LD    A,D
   OR    E			;Any dots in string?
   RET   Z			;No (load nothing)
   LD    HL,STYLE_CSS
   LD    B,3
   CALL  UTL_strncmpCI		;is extension CSS?
   RET   NZ			;No
;
   LD    HL,(HTML_TmpAddr)
   CALL  CPCB_OpenNewObj	;Load new object
   RET   NZ			;object load failed.
;
   LD    HL,HTML_CmdEnd
   LD    DE,(URL_EndAddr)
   JP    HTM_AddStyles2DB
;=========================================================================
;Name     html_cmd_base
;Purpose  To handle the html command <base....> to setup the base URL.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars URL_BaseAddr = new base URL
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
html_cmd_base
   LD    A,(HTML_headbody)
   OR    FLAGS_HEAD
   CP    FLAGS_HEADS		;Start of head region of HTML data ?
   RET   NZ				;No
;
;  setup new base URL for relative URL's
;
   LD    HL,(HTML_CmdTemp)
   LD    (HTML_ParamLoc),HL
html_cmd_base0
   CALL  HTM_GetCmdParam	;Locate next parameter
   LD    A,(HL)
   CP    '>'			;no more parameters?
   RET   Z				;Yes (get table info)
   CALL  HTM_FindParmLbl	;Determine the parameter nametype
   LD    A,HTM_P_href
   CP    B
   JR    NZ,html_cmd_base0	;No (skip the parameter)
   CALL  HTM_GetCmdParmVal	;HL = address of parameter value
   LD    DE,URL_ObjStrtTxt
   LD    B,URL_Size-1
html_cmd_base1
   LD    A,(HL)
   CP    '"'
   JR    Z,html_cmd_base2
   LD    (DE),A
   INC   HL
   INC   DE
   DJNZ  html_cmd_base1
html_cmd_base2
   XOR   A
   LD    (DE),A
   RET
;=========================================================================
;Name     html_cmd_blockquote
;Purpose  To handle the html command <blockquote....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. NEED TO FIX TO CHECK FOR URL AND UNDERLINE OF TEXT
;=========================================================================
html_cmd_blockquote
   CALL  UTL_ChkRendState		;Collecting web-page info?
   RET   NZ					;Yes
   LD    B,DOUBLE_QUOTE
   LD    C,0
   JP    HTM_ShowChar
;=========================================================================
;Name     html_cmd_cite
;Purpose  To handle the html command <cite> which allows for URLs for a
;         cite reference.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
html_cmd_Cite
   RET
;=========================================================================
;Name     html_cmd_Label
;Purpose  To handle the html command <label... >
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
html_cmd_Label
   RET
