;=========================================================================
;
; Application Routines (HTML rendering commands)
; Copyright (c) 2010 Ray Palmer
;
; Notes
;
; 1. The rendering of the current command does not expect that the same
;    command to be recursed. This may lead to an infinte loop.
;
; 2. Where a command changes the font (e.g normal to italic) the original
;    command will have no way of reverting the font back to what it was.
;
;=========================================================================
;
;  HTML command handlers
;
;=========================================================================
;Name     html_cmd_li
;Purpose  To handle the html command <li....>. To display the list item
;         number or the bullet character for an unordered list.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_CmdEnd = start of next command.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. List item numbers for ordered list can be up to 255.
;=========================================================================
html_cmd_li
   CALL  UTL_ChkRendState		;Collecting web-page info?
   RET   NZ				;Yes
;
   CALL  HTM_Get1stCmdChr		;end of command ?
   RET   Z				;Yes
;
;  Scan the parameters to <...> command
;
   LD    HL,(HTML_CmdTemp)
   LD    (HTML_ParamLoc),HL
html_cmd_li2
   CALL  HTM_GetCmdParam		;Locate next parameter
   LD    A,(HL)
   CP    EHTML_CHR				;no more parameters?
   JR    Z,html_cmd_li1			;Yes (get table info)
   CALL  HTM_FindParmLbl		;Determine the parameter nametype
   LD    A,HTM_P_Value
   CP    B				;Is parameter 'value' ?
   JR    NZ,html_cmd_li2		;No
;
   CALL  HTM_GetCmdParmVal		;Get address of parameter value
   CALL  UTL_readdec			;convert to value
   DEC   A
   LD    (HTML_ListNum),A
;
html_cmd_li1
   CALL  HTM_NewLine			;New line
   LD    A,(HTML_Indent)
   LD    C,A
   LD    B,0
   ADD   HL,BC
   LD    (HTML_Col),HL			;setup column for item
   LD    A,(HTML_ListIdnt)
   BIT   7,A				;Indent back?
   JR    NZ,html_cmd_li3		;No
;
   LD    HL,(HTML_Row)
   LD    C,A
   ADD   HL,BC
   LD    (HTML_Row),HL			;Setup row for next level
;
html_cmd_li3
   CALL  HTM_ChkListTyp			;Display list item type/number
   LD    B,32
   LD    C,0
   JP    HTM_ShowChar			;Display space char
;=========================================================================
;Name     html_cmd_ul
;Purpose  To handle the html command <ul....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;         HTML_Row = current web-page row
;         HTML_CmdEnd = current command end
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_Row = updated web-page row.
;         HTML_CmdEnd = updated position to point to next HTML command.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
html_cmd_ul
   CALL  UTL_ChkRendState		;Collecting web-page info?
   RET   NZ					;Yes
   CALL  HTM_Get1stCmdChr		;end of command ?
   LD    C,HTML_List_UnOrd
   JR    html_cmd_ol1
;=========================================================================
;Name     html_cmd_ol
;Purpose  To handle the html command <ol....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_ListTyp = type of list set to ordered list.
;         HTML_ListNum = 0 for new ordered list.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
html_cmd_ol
   CALL  UTL_ChkRendState		;Collecting web-page info?
   RET   NZ				;Yes
   XOR   A
   LD    (HTML_ListNum),A
   CALL  HTM_Get1stCmdChr		;end of command ?
   LD    C,HTML_List_Ord
html_cmd_ol1
   JR    Z,html_cmd_ol2			;Yes
   LD    A,(HTML_ListTyp)
   SLA   A
   SLA   A
   AND   &3C
   OR    C				;Enable list type mode
   LD    (HTML_ListTyp),A
   AND   &3c				;already in list data
   RET   Z				;No
   LD    A,(HTML_Indent)
   ADD   A,5
   LD    (HTML_Indent),A
   RET
html_cmd_ol2
   LD    A,(HTML_ListTyp)
   AND   &3f
   SRA   A
   SRA   A
   LD    (HTML_ListTyp),A
   LD    A,(HTML_Indent)
   CP    0
   RET   Z
   SUB   A,5
   LD    (HTML_Indent),A
   RET
;=========================================================================
;Name     html_cmd_dl
;Purpose  To handle the html command <dl....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
html_cmd_dl
   LD    HL,0
   LD    (HTML_BR_DL),HL
   CALL  HTM_Get1stCmdChr	;end of command ?
   CALL  Z,HTM_NewLine		;Yes
   RET
;=========================================================================
;Name     html_cmd_dt
;Purpose  To handle the html command <dt....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
html_cmd_dt
   LD    HL,0
   LD    (HTML_BR_DL),HL
   CALL  HTM_NewLine
   LD    HL,(HTML_BR_DL)
   LD    BC,HTML_DD_Ident
   ADD   HL,BC
   LD    (HTML_BR_DL),HL
   RET
;=========================================================================
;Name     html_cmd_dd
;Purpose  To handle the html command <dd....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
html_cmd_dd
   CALL  HTM_NewLine
   RET
