;=========================================================================
;
; Application Routines (HTML rendering commands)
; Copyright (c) 2010 Ray Palmer
;
; Notes
;
; 1. The rendering of the current command does not expect that the same
;    command to be recursed. This may lead to an infinte loop.
;
; 2. Where a command changes the font (e.g normal to italic) the original
;    command will have no way of reverting the font back to what it was.
;
;=========================================================================
;
;  HTML command handlers
;
;=========================================================================
;Name     html_cmd_doctype
;Purpose  To handle the html command <!doctype....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_dtype = doc type (0=strict, 1=loose, 2=frameset)
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. The doc type, if not encountered, defaults to be strict.
;=========================================================================
html_cmd_doctype
   RET
;=========================================================================
;Name     HTML_cmd_html
;Purpose  To handle the html command <html....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_html = updated
;         HTML_RenderState = Bit 7 set if </HTML> found
;         WindowFrmeStat = reset to 0
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTML_cmd_html
   XOR   A
   LD    (HTML_headbody),A
   LD    DE,HTML_html
   LD    BC,FLAGS_HEADB
   LD    L,FLAGS_HEADR
   CALL  html_cmdproc			;setup state of <HTML> proc.
   AND   FLAGS_HEAD			;Mask off status flags
   CP    C				;</HTML> flag set?
   JR    NZ,HTML_cmd_html1		;no
   LD    A,(HTML_RenderState)
   SET   7,A				;Force end of rendering to web-page
   LD    (HTML_RenderState),A
   RET
HTML_cmd_html1
   XOR   A
   LD    (WindowFrmeStat),A
   RET
;=========================================================================
;Name     html_cmd_head
;Purpose  To handle the html command <head....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_headbody = updated
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
html_cmd_head
   LD    DE,HTML_headbody
   LD    BC,FLAGS_HEADB
   LD    L,FLAGS_HEADR
   JR    html_cmdproc
;=========================================================================
;Name     html_cmd_body
;Purpose  To handle the html command <body....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_body = updated
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
html_cmd_body
   LD    DE,HTML_headbody
   LD    BC,FLAGS_BODYB
   LD    L,FLAGS_BODYR
html_cmdproc
   PUSH  HL
   CALL  HTM_Get1stCmdChr	;Is first '/'?
   POP   HL
   JR    NZ,html_cmdproc1	;No
   LD    A,(DE)
   OR    C			;mask in end command found
   LD    (DE),A			;flag end command found
   RET
html_cmdproc1
   LD    A,(DE)
   AND   L			;clear flags
   OR    B
   LD    (DE),A			;Flag start command found
   RET
;=========================================================================
;Name     html_cmd_title
;Purpose  To handle the html command <title....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_CmdEnd = updated to account for text following.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
html_cmd_title
   CALL  UTL_ChkRendState		;Collecting web-page info?
   RET   NZ				;Yes
   CALL  HTM_Get1stCmdChr		;</...> command ?
   JP    Z,MNU_RestoreCfg		;Yes
   LD    HL,(HTML_CmdEnd)
   INC   HL
   LD    A,(Scrn_MaxCol)
   DEC   A
   LD    C,A
   LD    DE,(HTML_CmdTemp)
;
;  Copy title to kb buffer for output.
;
html_cmd_title1
   LD    A,(HTML_CmdEnd+2)
   CALL  HTM_NextHTMBank
   LD    (HTML_CmdEnd+2),A
;
   LD    A,(HL)
   CP    SHTML_CHR
   JR    Z,html_cmd_title2
   LD    (DE),A
   INC   HL
   LD    A,C
   CP    255
   JR    Z,html_cmd_title1		;Yes (drop rest)
   DEC   C
   INC   DE
   JR    html_cmd_title1
html_cmd_title2
   DEC   HL
   LD    (HTML_CmdEnd),HL
   LD    A,C
   OR    A				;Title is empty?
   RET   Z				;Yes
   XOR   A
   LD    (DE),A
   JP    CPCB_ShowPageTitle
;=========================================================================
;Name     html_cmd_object
;Purpose  To handle the html command <object....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
html_cmd_object
   CALL  UTL_ChkRendState		;Collecting web-page info?
   RET   NZ				;Yes
   LD    B,ICON_OBJECT
   LD    C,PRINT_ICONCHR
   JP    HTM_ShowChar			;Show image icon character
;=========================================================================
;Name     html_cmd_applet
;Purpose  To handle the html command <applet....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. This command deviates from the specification where if the
;            command is encountered before the <BODY> command there is the
;            availability to add new apps to the CPCB apps DB and to
;            download the latest version. For this to be, the command is
;            defined as follows
;
;            <APPLET id="myapp" name="mapp.exe" href="url">
;            optional data for the applet as per specification (ignored)
;            </APPLET>
;
;            The use of the applet command above must supply all three
;            parameters to be a valid app to add to the known applications
;            to be available from the menu or HTML.
;
;            To access the application after the <BODY> the command can
;            take one of the following formats
;
;            a. To access the application using data after the command and
;               only show an icon.
;
;            <APPLET id="myapp">
;            parameter data for myapp
;            </APPLET>
;
;            This will display the App Icon with no labe following.
;
;            b. To access the application with data within the command and
;               to display a label with the icon.
;
;            <APPLET id="myapp" object="parameter data for myapp">
;            label for applet
;            </APPLET>
;
;            This will display the App Icon and a label following.
;
;         2. If the applet is a handler then call is immediately performed
;            to process the request. For handlers there is no data from
;            the HTML command shown and no icon shown.
;
;=========================================================================
html_cmd_applet
   CALL  UTL_ChkRendState		;Collecting web-page info?
   RET   NZ				;Yes
;
   CALL  HTM_Get1stCmdChr		;</...> command ?
   RET   Z				;Yes
;=========================================================================
;  Scan the parameters to <...> command
;=========================================================================
   CALL  HTM_ProcAppCmd
;=========================================================================
;  Process the parameters to the <APPLET> command
;=========================================================================
   LD    A,(HTML_headbody)
   CP    FLAGS_HEAD			;No <Body> command yet?
   JR    Z,html_cmd_applt1		;Yes, insert new app to database
;
   CALL  CPCB_FindApp
   OR    A				;App exists in DB?
   JR    NZ,html_cmd_Applt5c		;Yes
;
   CALL  HTM_SkipCmdObj			;No (skip applet content)
   LD    B,ICON_APPLET+1
   JR    html_cmd_Applt5d		;Display icon
;
html_cmd_Applt5c
   LD    A,(App_State)
   BIT   5,A				;Handler?
   JR    Z,html_cmd_Applt5b		;No
;=========================================================================
;  Start App as handler
;=========================================================================
   LD    C,DBLEQ_CHR
   LD    DE,MEMORY_BANK2
   LD    HL,(App_ObjName)
   LD    A,H				;Was href passed in command?
   OR    L
   JR    Z,html_cmd_Applt5a		;No
   LD    A,(HTML_CmdEnd+2)
   LD    B,A
   LD    HL,(HTML_CmdEnd)
   LD    C,SHTML_CHR
html_cmd_Applt5a
   CALL  HTM_strccpy			;download to common memory
   LD    A,(App_State)
   SET   0,A
   LD    (App_State),A
   LD    DE,MEMORY_BANK2		;Address of app parameter
   CALL  CPCB_DoAppReq
   JP    HTM_SkipCmdObj			;skip any additional data
;
html_cmd_Applt5b
;=========================================================================
;  May need to execute the application icon,
;  but for now the user needs to click on the shown icon to start
;  the application.
;=========================================================================
   LD    A,C
   LD    (App_DBTmp+S_AppIDNum),A
   CALL  HTM_CreateAppDb
   CALL  HTM_AddDBAppLocn
;
   LD    HL,(App_ObjName)
   LD    A,H				;Was URL passed in command?
   OR    L
   CALL  Z,HTM_SkipCmdObj		;No, assume data following is URL
;
   LD    B,ICON_APPLET
html_cmd_Applt5d
   SET   7,B
   LD    C,PRINT_ICONCHR
   JP    HTM_ShowChar			;Show App image icon character
;
html_cmd_applt1
;=========================================================================
;  Insert new application into database.
;=========================================================================
   CALL  HTM_SkipCmdObj			;goto </APPLET> command
;
   LD    A,(App_CtrlFlg)
   BIT   7,A				;App insertion allowed?
   RET   Z				;No
;
   LD    A,(App_Name)
   CP    SPACE_CHAR			;Was App name supplied?
   RET   Z				;No
;
   LD    A,(App_FileName)
   CP    SPACE_CHAR			;Was Filename passed ?
   RET   Z				;No
;
   LD    A,&42
   LD    (App_State),A
   CALL  CPCB_AddNewApp			;Add new app to local DB.
   OR    A				;App is already in DB?
   RET   NZ				;Yes
;
   LD    HL,(App_ObjName)
   LD    A,H				;Was URL passed ?
   OR    L
   RET   Z				;No (use existing App exectuable)
;
;  Download the latest Application executable.
;
   CALL  MEM_SwapInCfg
   LD    HL,CFG_AppLocn
   LD    DE,(HTML_CmdTemp)
   CALL  UTL_strcpy			;setup save location location
   LD    HL,(App_FileName)
   CALL  UTL_strcpy			;insert filename to save to
;
   LD    DE,URL_ObjStrtTxt		;location for URL and filename.
   LD    C,DBLEQ_CHR
   CALL  UTL_strccpy			;get URL
   LD    HL,(App_FileName)
   LD    B,A_MaxAppFileName
   CALL  UTL_strncpy
   LD    A,SPACE_CHAR
   LD    (DE),A
   LD    HL,MEMORY_BANK2
   JP    CPCB_OpenURLLink		;download application to cache
;=========================================================================
;Name     html_cmd_div
;Purpose  To handle the html command <div....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    UTL_ChkRendState
;         HTM_Get1stCmdChr
;         HTM_NewLine
;Notes    1. The implementation of the command is to first display a new
;            line followed by the setting up of the formats for following
;            content.
;=========================================================================
html_cmd_div
   CALL  UTL_ChkRendState	;Collecting web-page info?
   RET   NZ			;Yes
;
   CALL  HTM_Get1stCmdChr	;</...> command ?
   JP    Z,HTM_NewLine		;Yes (insert new line)
;
   CALL  HTM_NewLine		;insert a new line
   JP    HTM_DivSpan		;setup format
;=========================================================================
;Name     html_cmd_span
;Purpose  To handle the html command <span....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
html_cmd_span
   CALL  UTL_ChkRendState	;Collecting web-page info?
   RET   NZ			;Yes
   JP    HTM_DivSpan		;setup format
;=========================================================================
;Name     html_cmd_img
;Purpose  To handle the html command <img....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;         HTML_CmdTemp = address of HTML command
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars URL_OpenType = type of URl accessed
;         HTML_ParamLoc = address within HTML command
;         IMG_DBAddr = image database entry structure
;         FileOpState = status of file access
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. This routine displays the CPC structured image. Refer to the
;            associated documentation for the CPC structured image.
;=========================================================================
html_cmd_img
   CALL  UTL_ChkRendState		;Collecting web-page info?
   RET   NZ				;Yes
;
   CALL  HTM_ImageFindHTML		;Check if Image already loaded.
   OR    A				;Did we find DB entry?
   JR    NZ,html_cmd_img1		;Yes, Display Image
;
;  NEED TO CHECK IF DB IS FULL AS WELL (20140321)
;
;  Process the HTML command parameters
;
   LD    HL,(HTML_CmdTemp)
   LD    (HTML_ParamLoc),HL
html_cmd_img0
   CALL  HTM_GetCmdParam		;Locate next parameter
   LD    A,(HL)
   CP    SHTML_CHR			;no more parameters?
   RET   Z				;Yes (no image shown)
;
   PUSH  HL
   CALL  HTM_FindParmLbl		;Determine the parameter nametype
   POP   HL
   LD    A,HTM_P_src
   CP    B				;Is parameter 'src' ?
   JR    NZ,html_cmd_img0		;No
;
;  Setup location of html image file source.
;
   PUSH  HL
   LD    DE,IMG_DBAddr
   LD    HL,HTML_CmdStart
   LD    BC,3
   LDIR					;IMG_DBAddr = HTML cmd loc'n
   LD    HL,URL_EndAddr
   LD    BC,3
   LDIR					;IMG_DBData = data loc'n
   POP   HL
;
;  Load the image object
;  if text the it is loaded after last HTMl character
;  if binary then object is in the code banks and is limited to 16K
;
   LD    A,FILE_UNK
   LD    (IMG_DBFlag),A			;Assume unknown file type
;
   CALL  HTM_LoadSrcObj			;load object
   LD    A,(FileOpState)		;Get file status
   OR    A				;Was object accessed okay?
   JR    NZ,html_cmd_img2		;No (display pict. icon only)
;
   LD    A,(URL_OpenType)
   LD    (IMG_DBFlag),A			;setup file type in DB
   AND   &FC				;text file?
   CALL  NZ,HTM_LoadImageObj		;No (put binary data after HTML)
   CALL  HTM_GetImageSize		;setup Db with image size
   CALL  HTM_ImageItm2DB		;add new item to images db
;
html_cmd_img1
;
;  Image loaded okay, so display contents on screen.
;
   CALL  HTM_ShowImage			;display picture's data
   JP    Mem_SwapInMain
;
html_cmd_img2
;
;  Image failed to be loaded, so display an icon to show image not found
;
   LD    B,ICON_PICTURE + 128
   LD    C,HTML_F_FntIcns		;Display icon char type
   JP    HTM_ShowChar
;=========================================================================
;Name     html_cmd_code
;Purpose  To handle the html command <code>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
html_cmd_Code
   RET
;=========================================================================
;Name     html_cmd_samp
;Purpose  To handle the html command <samp....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
html_cmd_Samp
   RET
;=========================================================================
;Name     html_cmd_kbd
;Purpose  To handle the html command <kbd....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
html_cmd_Kbd
   RET
;=========================================================================
;Name     html_cmd_abbr
;Purpose  To handle the html command <abbr...>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
html_cmd_Abbr
   RET
;=========================================================================
;Name     html_cmd_acronym
;Purpose  To handle the html command <acronym...>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
html_cmd_Acronym
   RET
;=========================================================================
;Name     html_cmd_var
;Purpose  To handle the html command <var>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. This command will define a variable for use in the HTML
;            command.
;            To define a variable the format is -
;
;               <var label="aaaaa" type="bbbbb">
;            where
;                 aaaaa = name with no spaces
;                 bbbbb = type
;                         integer, string, float
;
;         2. References to variables is via the use of '!' preceeding the
;            name.
;
;=========================================================================
html_cmd_Var
   RET
;=========================================================================
;Name     html_cmd_map
;Purpose  To handle the html command <map> to display an image for
;         cursor clicking for URL parameters.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
html_cmd_Map
   RET
;=========================================================================
;Name     html_cmd_area
;Purpose  To handle the html command <area>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
html_cmd_Area
   RET
;=========================================================================
;Name     html_cmd_param
;Purpose  To handle the html command <param>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
html_cmd_Param
   RET
;=========================================================================
;Name     html_cmd_menu
;Purpose  To handle the html command <menu>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
html_cmd_Menu
   RET
;=========================================================================
;Name     html_cmd_meta
;Purpose  To handle the html command <meta.....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
html_cmd_Meta
   RET
;=========================================================================
;Name     html_cmd_address
;Purpose  To handle the html command <address....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
html_cmd_Address
   JP    HTM_SkipCmdObj
;=========================================================================
;Name     html_cmd_script
;Purpose  To handle the html command <script....>.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  HTM_DoHTMLCmd (via HTM_ShowFile)
;Calls    HTM_GetCmdParam
;         HTM_FindParmLbl
;         HTM_GetCmdParmVal
;         HTM_LdObjScr
;         HTM_DoExecScr
;         HTM_FindSrceType
;Notes    1. This HTML command may be used to execute a JavaScript or any
;            other type of script. Normally the script handler would be in
;            a ROM or in a loaded executable.
;            Example uses
;            a.
;             <SCRIPT type="text/javascript">
;             -- Java script commands
;             </SCRIPT>
;
;            b.
;             <SCRIPT type="text/javascript" src="javascript.obj">
;             </SCRIPT>
;
;         2. The script command content cannot have the < and > characters
;            used as these are used to determine start/end of HTML
;            commands (all other valid characters are allowed).
;
;=========================================================================
html_cmd_script
   CALL  HTM_Get1stCmdChr		;</...> command ?
   RET   Z				;Yes (no action)
;
   LD    A,255
   LD    (App_ID),A			;Assume unknown App
   LD    HL,0
   LD    (App_ObjName),HL		;Assume no object to load
;
   LD    HL,HTML_CmdEnd
   LD    DE,App_ObjLocn
   LD    BC,3
   LDIR					;Assume object follows script cmd
;
   LD    HL,(HTML_CmdTemp)
   LD    (HTML_ParamLoc),HL
html_cmd_scr2
   CALL  HTM_GetCmdParam		;Locate next parameter
   LD    A,(HL)
   CP    EHTML_CHR			;end of parameters?
   JR    Z,html_cmd_scr0		;Yes
   CALL  HTM_FindParmLbl		;Determine the parameter nametype
   LD    A,HTM_P_type			;Is parameter 'type' ?
   CP    B
   JR    Z,html_cmd_scr3		;Yes
   LD    A,HTM_P_src
   CP    B				;Is parameter 'src' ?
   JR    NZ,html_cmd_scr2		;No (skip the parameter)
;
   CALL  HTM_GetCmdParmVal		;HL = address of parameter value
   LD    (App_ObjName),HL		;Save address of object name
   JR    html_cmd_scr2			;continue with cmd parameters
;
html_cmd_scr3
   CALL  HTM_GetCmdParmVal		;HL = address of parameter value
   CALL  HTM_FindSrceType		;Search App DB for handler
   JR    html_cmd_scr2
;
html_cmd_scr0
   CALL  HTM_SkipCmdObj
   LD    A,(App_ID)
   CP    255				;valid App to execute?
   JP    NZ,CPCB_DoAppReq		;Yes start app/handler
   RET
   
