;=========================================================================
;
; Application Routines (Style handling routines - Main)
; Copyright (c) 2010 Ray Palmer
;
; Notes
;
;=========================================================================
;Name     html_cmd_style
;Purpose  To handle the html command <style....> for style sheets.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_CmdEnd = address of command content.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_CmdEnd = address after end of command </style>.
;Uses     TBD
;Call By  TBD
;Calls    HTM_Get1stCmdChr
;         HTM_AddStyles2DB
;         HTM_SkipCmdObj
;Notes    1. This command simply
;=========================================================================
html_cmd_Style
   LD    A,(HTML_headbody)
   AND   FLAGS_HEAD
   CP    FLAGS_HEADS			;Started body of <HEAD>?
   JR    NZ,html_cmd_Style1		;Yes
;
   CALL  HTM_Get1stCmdChr		;</...> command ?
   RET   Z				;Yes (Do nothing)
;
;  Add address of style command as new entry into the DB
;
   LD    HL,HTML_CmdEnd
   LD    DE,HTML_CmdEnd			;location of style commands
   CALL  HTM_AddStyles2DB
;
html_cmd_Style1
   JP    HTM_SkipCmdObj			;ignore style object data
;=========================================================================
;Name     STY_FontEffect
;Purpose  To select a font effect
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_StyleDbIdx = address of style's command
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_StyleFlgs = updated on effect flags and bit 0
;         SysGenBuffer = address of style command and parameters.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. Looks at variant to determine what effect is used
;            a. underline
;            b. strike-through
;            c. inverse
;            d. italic
;            e. normal
;            The normal effect removes current effects
;=========================================================================
STY_FontEffect
   CALL  HTM_StyleParam		;HL = address of font effect name
   EX    DE,HL
   LD    A,(DE)
   CALL  UTL_tolower
;
   LD    B,B_2+B_7
   CP    'u'			;Underline?
   JR    Z,STY_FontEffect1	;yes
;
   LD    B,B_3+B_7
   CP    's'			;strike through?
   JR    Z,STY_FontEffect1	;Yes
;
   CP    'i'			;inverse/italic effect?
   RET   NZ			;No
;
   INC   DE
   LD    A,(DE)			;get next character
   CALL  UTL_tolower
;
   LD    B,B_6+B_7
   CP    'n'			;inverse effect
   JR    Z,STY_FontEffect1	;yes
;
   CP    't'			;italic effect
   RET   NZ			;No (so no changes)
   LD    HL,HTML_StyleFlgs
   RES   0,(HL)
   LD    B,B_1+B_7		;Flag Italic effect via new font
;
STY_FontEffect1
   LD    A,(HTML_StyleFlgs)
   OR    B			;Add new effects and flag change
   LD    (HTML_StyleFlgs),A
   RET
;=========================================================================
;Name     STY_FontSelect
;Purpose  To select a font
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_StyleDbIdx = address of the style's command
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars ??
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
STY_FontSelect
   CALL  HTM_StyleParam		;HL = address of font effect name
;=========================================================================
;Name     STY_FontSelect2
;Purpose  To select a font
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = Address of fonts name
;         IX = ??
;         IY = ??
;In Vars  HTML_StyleStatus = styles flags
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_StyleStatus = styles flags (bit 7 cleared)
;         HTML_StyleFlgs = change n format flag bit 7 set.
;Uses     TBD
;Call By  TBD
;Calls    ERoutine_SelectFontByName
;Notes    TBD
;=========================================================================
STY_FontSelect2
   CALL  ERoutine_SelectFontByName
   LD    (HTML_StyleFntNme),DE
;
STY_FlagChnge
   LD    HL,HTML_StyleFlgs
   SET   7,(HL)			;flag change in format settings.
   LD    HL,HTML_StyleStatus
   RES   7,(HL)			;Flag HTML font name
   RET
;=========================================================================
;Name     STY_BorderWidth
;Purpose  To select a border width thickness using the style command
;         "border-width"
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_StyleDbIdx = address of style's command
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars SysGenBuffer = address of style command and parameters.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. Only the types Thin, Medium and Thick are handled.
;=========================================================================
STY_BorderWidth
   CALL  STY_PCmdParams
   JR    STY_SelBorder1
;=========================================================================
;Name     STY_BorderType
;Purpose  To select a border thickness using the style command "border"
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_StyleDbIdx = address of style's command
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars SysGenBuffer = address of style command and parameters.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. Only the types Thin, Medium and Thick are handled.
;=========================================================================
STY_BorderType
   CALL  STY_PCmdParams
   LD    A,B
   LD    B,1			;1 pixel border (thin)
   LD    C,HTM_P_thin
   CP    C
   JR    Z,STY_SelBorder1
   INC   B			;2 pixel border (medium)
   INC   C
   CP    C
   JR    Z,STY_SelBorder1
   INC   B			;3 pixel border (thick)
   INC   C
   CP    C
   JR    Z,STY_SelBorder1
   LD    B,1			;default 1 pixel line (thin)
STY_SelBorder1
   LD    A,B
   LD    (HTML_StyleBordw),A	;setup border thickness by type
   JP    STY_FlagChnge
;=========================================================================
;Name     STY_BorderStyle
;Purpose  To select a border style using the style command "border-style"
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_StyleDbIdx = address of style's command
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars SysGenBuffer = address of style command and parameters.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. The border styles available are -
;            a. None/hidden
;            b. Line
;            c. Double-line
;            d. Dotted
;            e. groove
;            All others in CSS 2/2.1 are unavaliable.
;         2. The treatment of hidden to be same as none is so that
;            screen real-estate is maximized.
;=========================================================================
STY_BorderStyle
   CALL  STY_PCmdParams
   LD    A,B
   LD    (HTML_StyleBord),A	;setup border type
   LD    A,(HTML_StyleBordw)
   OR    A
   JP    NZ,STY_FlagChnge
   INC   A
   LD    (HTML_StyleBordw),A
   JP    STY_FlagChnge
;=========================================================================
;Name     STY_newlineb
;Purpose  To setup the new line affect (before)
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars ??
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
STY_newlineb
   CALL  HTM_StyleParam
   LD    B,B_1
   JR    STY_newline0
;=========================================================================
;Name     STY_newlinea
;Purpose  To setup the new line affect (after)
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars ??
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
STY_newlinea
   CALL  HTM_StyleParam
   LD    B,B_0
STY_newline0
   LD    C,B
   LD    A,(HL)
   CALL  UTL_tolower
   CP    "Y"
   JR    Z,STY_newlinea1
   LD    C,0
STY_newlinea1
   LD    A,(HTML_StyleStatus)
   OR    B
   XOR   B
   OR    C
   LD    (HTML_StyleStatus),A
   JP    STY_FlagChnge
;