;=========================================================================
;
; Application Routines (HTML rendering commands - Text)
; Copyright (c) 2010 Ray Palmer
;
; Notes
;
; 1. The rendering of the current command does not expect that the same
;    command to be recursed. This may lead to an infinte loop.
;
; 2. Where a command changes the font (e.g normal to italic) the original
;    command will have no way of reverting the font back to what it was.
;
;=========================================================================
;
;  HTML command handlers
;
;=========================================================================
;Name     html_cmd_pre
;Purpose  To handle the html command <pre....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_RCurrPos = address of first byte after </pre> command
;         HTML_ObjLocn = corrupted
;Uses     TBD
;Call By  TBD
;Calls    UTL_ChkRendState
;         HTM_SkipCmd
;         MEM_SwapInHTML
;         HTM_ShowChar
;         HTM_GetCmd
;         HTM_strncmp
;Notes    1. This command is the only one that outputs all of the text
;            upto the end command at which point the routine returns to
;            the normal web-page processing with the adjusted HTML
;            location.
;=========================================================================
html_cmd_pre
   CALL  UTL_ChkRendState		;Collecting web-page info?
   RET   NZ				;Yes
;
   LD    HL,HTML_PreserveFlag
   SET   7,(HL)
;
   LD    DE,HTML_ObjLocn
   LD    HL,HTML_CmdEnd
   LD    BC,3
   LDIR					;Setup start block index
html_cmd_pre0
   LD    HL,(HTML_ObjLocn)
   LD    A,(HTML_ObjLocn+2)
   INC   HL
   CALL  HTM_NextHTMBank		;swap in html bank
   LD    (HTML_ObjLocn+2),A		;setup start block index
   LD    (HTML_ObjLocn),HL		;setup start address in block
   LD    A,(HL)
   OR    A				;End of HTML ?
   JR    Z,html_cmd_pre4		;Yes
   LD    B,A
   CP    SHTML_CHR			;Start of HTML command?
   JR    Z,html_cmd_pre1		;Yes
html_cmd_pre3
   LD    C,HTML_F_SysChrs
   CALL  HTM_ShowChar			;Show char
   JR    html_cmd_pre0
;
html_cmd_pre1
   PUSH  BC				;Save char
   CALL  HTM_GetCmd			;get HTML command
   LD    HL,KB_Buffer+1
   LD    DE,HTML_C_pre
   LD    C,0
   LD    B,3
   CALL  HTM_strncmp			;Is command '</pre>' ?
   POP   BC				;restore char
   JR    NZ,html_cmd_pre3		;No (print char out)
;
   LD    HL,(HTML_ObjLocn)
   LD    A,(HTML_ObjLocn+2)
   CALL  HTM_SkipCmd			;Skip HTML command
   LD    (HTML_RCurrPos),HL
   LD    A,C
   LD    (HTML_RCurrPos+2),A		;setup new start HTML location.
html_cmd_pre4
   LD    HL,HTML_PreserveFlag
   RES   7,(HL)
   RET
;=========================================================================
;Name     html_cmd_quote
;Purpose  To handle the html command <quote....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;         HTML_quoteLevel = current quote level
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_quoteLevel = current quote level
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
html_cmd_quote
   CALL  UTL_ChkRendState		;Collecting web-page info?
   RET   NZ				;Yes
   CALL  HTM_Get1stCmdChr		;First char '/'?
   JR    Z,HTML_quoteLevel1		;Yes </quote> found
;
   LD    A,(HTML_quoteLevel)
   INC   A				;next level of quotes
   LD    (HTML_quoteLevel),A
   BIT   1,A				;single quote?
   JR    NZ,HTML_quoteLevel2		;Yes
;
HTML_quoteLevel0
   LD    B,DOUBLE_QUOTE
   JR    HTML_quoteLevel3
;
HTML_quoteLevel2
   LD    A,2
   LD    (HTML_quoteLevel),A
HTML_quoteLevel2a
   LD    B,SINGLE_QUOTE
;
HTML_quoteLevel3
   LD    C,0
   JP    HTM_ShowChar
;
HTML_quoteLevel1
   LD    A,(HTML_quoteLevel)
   LD    B,A
   DEC   A
   LD    (HTML_quoteLevel),A
   BIT   1,B
   JR    NZ,HTML_quoteLevel2a		;show end single quote
   JR    HTML_quoteLevel0		;show end double quote
;=========================================================================
;Name     html_cmd_q
;Purpose  To handle the html command <q> to display a quote symbol
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    NEED TO CHECK IF REQUIRED
;=========================================================================
html_cmd_q
   RET
;=========================================================================
;Name     html_cmd_font
;Purpose  To handle the html command <font....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
html_cmd_font
   RET
;=========================================================================
;Name     html_cmd_hr
;Purpose  To handle the html command <hr....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;         Windw_NumCols = number of window columns
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    UTL_ChkRendState
;         HTM_ShowChar
;         HTM_NewLine
;Notes    1. This command simply displays a single line of dashes in the
;            current window frame.
;=========================================================================
html_cmd_hr
   CALL  UTL_ChkRendState		;Collecting web-page info?
   RET   NZ				;Yes
   LD    A,(Windw_NumCols)
   LD    B,A
   LD    C,HTML_F_SysChrs
html_cmd_hr1
   PUSH  BC
   LD    B,'-'
   CALL  HTM_ShowChar
   POP   BC
   DJNZ  html_cmd_hr1
   JP    HTM_NewLine
;=========================================================================
;Name     html_cmd_br
;Purpose  To handle the html command <br....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_BR_Flag = margin control flag
;                        Bit  Description
;                         7   clear control for the left margin
;                         6   clear control for the right margin
;                        5-0  Unused/Reserved
;Uses     TBD
;Call By  TBD
;Calls    HTM_ChkMaxCol
;         HTM_GetCmdParam
;         HTM_FindParmLbl
;         HTM_GetCmdParmVal
;         HTM_SearchParmLbl
;         HTM_NewLine
;Notes    NEED TO CHECK IF THE FORMS CODE REQUIRES SPECIAL TREATMENT.
;=========================================================================
html_cmd_br
   CALL  HTM_ChkMaxCol
;
   LD    HL,(HTML_CmdTemp)
   LD    (HTML_ParamLoc),HL
html_cmd_br0
   CALL  HTM_GetCmdParam		;Locate next parameter
   LD    A,(HL)
   CP    EHTML_CHR			;no more parameters?
   JR    Z,html_cmd_br2			;Yes (save item into DB)
   PUSH  HL
   CALL  HTM_FindParmLbl		;Determine the parameter nametype
   POP   HL
   LD    A,HTM_P_clear
   CP    B				;Is parameter 'clear' ?
   JR    NZ,html_cmd_br0		;No
;
   CALL  HTM_GetCmdParmVal		;HL = address of parameter value
   LD    DE,HTM_ClearParams
   CALL  HTM_SearchParmLbl		;B = clear type
;
   LD    C,&80
   LD    A,HTM_ClrPLeft
   CP    B				;Clear parameter 'Left'
   JR    Z,html_cmd_br1
   LD    C,&40
   LD    A,HTM_ClrPRight
   CP    B				;Clear parameter 'Right'
   JR    Z,html_cmd_br1
   LD    C,&C0
   LD    A,HTM_ClrPAll
   CP    B				;Clear parameter 'All'
   JR    Z,html_cmd_br1
   LD    C,0				;Default to none.
html_cmd_br1
   LD    A,(HTML_BR_Flag)
   AND   &C0				;remove existing handle
   OR    C				;insert new handle
   LD    (HTML_BR_Flag),A		;Flag how to handle BR.
;
html_cmd_br2
   JP    HTM_NewLine
;=========================================================================
;Name     html_cmd_p
;Purpose  To handle the html command <p>. The command define text until
;         end of the <P> to </P> block as encapulsated text with a border.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;         HTML_Col = updated
;         HTML_Row = updated
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_Col = updated
;         HTML_Row = updated
;         HTML_BR_FormsT = updated
;         HTML_MaxCol = setup/cleared
;         HTML_P_SCol = start column of <P> border
;         HTML_P_SRow = start row of <P> border
;         HTML_P_ECol = end column of <P> border
;         HTML_P_ERow = end row of <P> border
;         Window_FSRow = start row (from zero)
;         Window_FSCol = Start column (from zero)
;         Window_FERow = end row (from zero)
;         Window_FECol = end column (from zero)
;         Windw_NumRows = number of rows (from zero)
;         Windw_NumCols = number of columns (from zero)
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. The HTML specification says </P> is optional but the code
;            requires it so it can properly know where to define the end.
;=========================================================================
html_cmd_p
   CALL  UTL_ChkRendState	;Collecting web-page info?
   RET   NZ			;Yes
;
   CALL  HTM_Get1stCmdChr	;</P> command?
   LD    HL,HTML_BordrSt
   JR    Z,html_cmd_p1		;Yes
;
   LD    DE,1
   LD    (HTML_Col),DE
   LD    DE,(HTML_Row)
   INC   DE
   LD    (HTML_Row),DE
;
   BIT   2,(HL)			;border for <P> command?
   RET   Z			;No
;
   DEC   DE
   LD    (HTML_Row),DE
   SET   6,(HL)
   RES   5,(HL)
   JP    HTM_ObjectBrdr
;
html_cmd_p1
   BIT   2,(HL)			;border for <P> command?
   JP    NZ,HTM_ObjectBrdr	;Yes
   RET
;=========================================================================
;Name     html_cmd_i
;Purpose  To handle the html command <i> to setup the italic font.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    UTL_ChkRendState
;         HTM_Get1stCmdChr
;         STY_UndoFontByIndx
;         STY_ChngeFont2
;         HTM_PushStyle
;Notes    TBD
;=========================================================================
html_cmd_i
   CALL  UTL_ChkRendState		;Collecting web-page info?
   RET   NZ				;Yes
   CALL  HTM_Get1stCmdChr		;end of italics?
   JP    Z,STY_UndoFontByIndx
;
   CALL  STY_ChngeFont2			;Select Italic font
   JP    HTM_PushStyle
;=========================================================================
;Name     html_cmd_tt
;Purpose  To handle the html command <tt> to setup the true-type font.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    UTL_ChkRendState
;         HTM_Get1stCmdChr
;         STY_ChngeFont1
;         STY_UndoFontByIndx
;Notes    1. The true type font is defaulted to italics.
;=========================================================================
html_cmd_tt
   CALL  UTL_ChkRendState		;Collecting web-page info?
   RET   NZ				;Yes
   CALL  HTM_Get1stCmdChr		;end of italics?
   JP    Z,STY_UndoFontByIndx		;Yes
;
   LD    HL,FONT_teletype
   JP    STY_ChngeFont1
;=========================================================================
;Name     html_cmd_small
;Purpose  To handle the html command <small> to setup the italic font.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    UTL_ChkRendState
;         HTM_Get1stCmdChr
;         STY_ChngeFont1
;         STY_UndoFontByIndx
;Notes    TBD
;=========================================================================
html_cmd_small
   CALL  UTL_ChkRendState		;Collecting web-page info?
   RET   NZ				;Yes
   CALL  HTM_Get1stCmdChr		;end of italics?
   JP    Z,STY_UndoFontByIndx		;Yes
;
   LD    HL,FONT_small
   JP    STY_ChngeFont1
;=========================================================================
;Name     html_cmd_u
;Purpose  To handle the html command <u> to setup the underline font
;         effect.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    CPCB_FontEffect
;Notes    TBD
;=========================================================================
html_cmd_u
   LD    B,255
   LD    A,FONT_E_UNDERLINE
   JP    CPCB_FontEffect		;toggle effect in char data
;=========================================================================
;Name     html_cmd_strike
;Purpose  To handle the html command <strike>.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    CPCB_FontEffect
;Notes    TBD
;=========================================================================
html_cmd_strike
   LD    B,255
   LD    A,FONT_E_STRIKE
   JP    CPCB_FontEffect		;toggle effect in char data
;=========================================================================
;Name     html_cmd_big
;Purpose  To handle the html command <big> for big fonts.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    UTL_ChkRendState
;         HTM_Get1stCmdChr
;         STY_ChngeFont1
;         STY_UndoFontByIndx
;Notes    TBD
;=========================================================================
html_cmd_big
   CALL  UTL_ChkRendState		;Collecting web-page info?
   RET   NZ				;Yes
   CALL  HTM_Get1stCmdChr		;end of command?
   JP    Z,STY_UndoFontByIndx		;Yes
;
   LD    HL,FONT_teletype
   JP    STY_ChngeFont1
;=========================================================================
;Name     html_cmd_b
;Purpose  To handle the html command <b> for bold fonts.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    UTL_ChkRendState
;         HTM_Get1stCmdChr
;         STY_ChngeFont1
;         STY_UndoFontByIndx
;Notes    TBD
;=========================================================================
html_cmd_b
   CALL  UTL_ChkRendState		;Collecting web-page info?
   RET   NZ				;Yes
   CALL  HTM_Get1stCmdChr		;end of command?
   JP    Z,STY_UndoFontByIndx		;Yes
;
   LD    HL,FONT_teletype
   JP    STY_ChngeFont1
;=========================================================================
;Name     html_cmd_em
;Purpose  To handle the html command <EM> to perform emphasis font/style.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    CPCB_FontEffect
;Notes    1. The <EM> command will produce an inverted text.
;=========================================================================
html_cmd_Em
   LD    B,255
   LD    A,FONT_E_INVERSE
   JP    CPCB_FontEffect		;toggle effect in char data
;=========================================================================
;Name     html_cmd_strong
;Purpose  To handle the html command <strong> to handle the strong font
;         setting.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    UTL_ChkRendState
;         HTM_Get1stCmdChr
;         STY_ChngeFont1
;         STY_UndoFontByIndx
;Notes    TBD
;=========================================================================
html_cmd_Strong
   CALL  UTL_ChkRendState		;Collecting web-page info?
   RET   NZ				;Yes
   CALL  HTM_Get1stCmdChr		;end of command?
   JP    Z,STY_UndoFontByIndx		;Yes
;
   LD    HL,FONT_teletype
   JP    STY_ChngeFont1
;=========================================================================
;Name     html_cmd_sub
;Purpose  To handle the html command <sub> for a sub-script font.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    UTL_ChkRendState
;         HTM_Get1stCmdChr
;         STY_ChngeFont1
;         STY_UndoFontByIndx
;Notes    TBD
;=========================================================================
html_cmd_Sub
   CALL  UTL_ChkRendState		;Collecting web-page info?
   RET   NZ				;Yes
   CALL  HTM_Get1stCmdChr		;end of command?
   JP    Z,STY_UndoFontByIndx		;Yes
;
   LD    HL,FONT_Sub
   JP    STY_ChngeFont1
;=========================================================================
;Name     html_cmd_sup
;Purpose  To handle the html command <sup> for super script font.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    UTL_ChkRendState
;         HTM_Get1stCmdChr
;         STY_ChngeFont1
;         STY_UndoFontByIndx
;Notes    TBD
;=========================================================================
html_cmd_Sup
   CALL  UTL_ChkRendState		;Collecting web-page info?
   RET   NZ				;Yes
   CALL  HTM_Get1stCmdChr		;end of command?
   JP    Z,STY_UndoFontByIndx		;Yes
;
   LD    HL,FONT_Sup
   JP    STY_ChngeFont1
;=========================================================================
;Name     html_cmd_ins
;Purpose  To handle the html command <ins> to mark text as inserted.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. FIX ME to control how this is seen.
;            a. Setting to default hide INVERSION, when cleared the
;               text is seen as INVERSE.
;=========================================================================
html_cmd_Ins
   RET
;=========================================================================
;Name     html_cmd_del
;Purpose  To handle the html command <del> to mark text as deleted.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    CPCB_FontEffect
;Notes    1. FIX ME to control how this is seen.
;            a. if cleared the text is seen else text is hidden.
;=========================================================================
html_cmd_Del
   LD    B,255
   LD    A,FONT_E_DELETE
   JP    CPCB_FontEffect		;toggle effect in char data
;=========================================================================
;Name     html_cmd_dir
;Purpose  To handle the html command <dir> to define which direction text
;         is displayed.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
html_cmd_Dir
   RET
;=========================================================================
;Name     html_cmd_h1
;Purpose  To handle the html command <h1....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_H1Addr = address of the H1 object which contains the
;                       styling information.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    html_cmd_Hx
;Notes    1. The styling information is limited to -
;            a. Font selection.
;            b. Border selection.
;            c. Scripts (for auto-numbering)
;         2. The use of other aspects of styling are ignored for now.
;=========================================================================
html_cmd_H1
   LD    A,1
   JR    html_cmd_Hx
;=========================================================================
;Name     html_cmd_h2
;Purpose  To handle the html command <h2...>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    html_cmd_Hx
;Notes    TBD
;=========================================================================
html_cmd_H2
   LD    A,2
   JR    html_cmd_Hx
;=========================================================================
;Name     html_cmd_h3
;Purpose  To handle the html command <h3...>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    html_cmd_Hx
;Notes    TBD
;=========================================================================
html_cmd_H3
   LD    A,3
   JR    html_cmd_Hx
;=========================================================================
;Name     html_cmd_h4
;Purpose  To handle the html command <h4....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    html_cmd_Hx
;Notes    TBD
;=========================================================================
html_cmd_H4
   LD    A,4
   JR    html_cmd_Hx
;=========================================================================
;Name     html_cmd_h5
;Purpose  To handle the html command <h5...>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    html_cmd_Hx
;Notes    TBD
;=========================================================================
html_cmd_H5
   LD    A,5
   JR    html_cmd_Hx
;=========================================================================
;Name     html_cmd_h6
;Purpose  To handle the html command <h6....>
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
html_cmd_H6
   LD    A,6
html_cmd_Hx
;
;  Setup the format according to hierarchy of h1 to h6
;  H1 highest, H6 lowest.
;
   RET
