;=========================================================================
;
; Application Routines (HTML rendering commands)
; Copyright (c) 2010 Ray Palmer
;
; Notes
;
; 1. The rendering of the current command does not expect that the same
;    command to be recursed. This may lead to an infinte loop if it does.
;
; 2. Where a command changes the font (e.g normal to italic) the original
;    command will have no way of reverting the font back to what it was.
;
;=========================================================================
;Name     HTM_Get1stCmdChr
;Purpose  To get the first char from the HTML command after the '<'.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_CmdTemp = location of command
;Out Regs AF = character, flag updated
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars None
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
HTM_Get1stCmdChr
   LD    HL,(HTML_CmdTemp)
   LD    A,(HL)
   CP    SLASH_CHAR
   RET
;=========================================================================
;Name     HTM_FindCmdEnd
;Purpose  To locate the end of the HTML command delimited by the '>'
;         character.
;In Regs  A = index into the HTML banks vector.
;         BC = ??
;         DE = ??
;         HL = start address of current HTML command
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = corrupted.
;         BC = ??
;         D = ??
;         E = index into the HTML banks vector.
;         HL = address of end of command or end of HTML data
;         IX = ??
;         IY = ??
;Out Vars None
;Uses     TBD
;Call By  TBD
;Calls    HTM_NextHTMBank
;Notes    1. HTML bank left swapped in.
;=========================================================================
HTM_FindCmdEnd
   LD    C,A
   LD    B,EHTML_CHR
   JR    HTM_SkipToChar
;=========================================================================
;Name     HTM_FindCmdStrt
;Purpose  To locate the start of a HTML command delimited by the '<' 
;         character.
;In Regs  A = start index into the HTML banks vector.
;         BC = ??
;         DE = ??
;         HL = start address into the HTML bank
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = corrupted.
;         B = '<' character
;         C = HTML bank
;         D = corrupted.
;         E = index into the HTML banks vector.
;         HL = address of end of command or end of HTML data
;         IX = ??
;         IY = ??
;Out Vars None
;Uses     TBD
;Call By  TBD
;Calls    HTM_NextHTMBank
;Notes    1. HTML bank left swapped in.
;=========================================================================
HTM_FindCmdStrt
   LD    C,A
;=========================================================================
;Name     HTM_SkipCmd
;Purpose  To find the start of the next HTML command.
;In Regs  A = ??
;         B = ??
;         C = HTML bank
;         DE = ??
;         HL = address of HTML data
;         IX = ??
;         IY = ??
;In Vars  None
;Out Regs AF = ??
;         B = '<' character
;         C = HTML bank
;         D = corrupted.
;         E = index into the HTML banks vector.
;         HL = address of end of command or end of HTML data
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    HTM_NextHTMBank
;Notes    1. HTML bank left swapped in.
;=========================================================================
HTM_SkipCmd
   LD    B,SHTML_CHR
   JR    HTM_SkipToChar
;=========================================================================
;Name     HTM_SkipToLBrace
;Purpose  To find the '{' character.
;In Regs  AF = ??
;         B = ??
;         C = HTML bank
;         DE = ??
;         HL = address of HTML data
;         IX = ??
;         IY = ??
;In Vars  None
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars None
;Uses     TBD
;Call By  TBD
;Calls    HTM_NextHTMBank
;Notes    1. HTML bank left swapped in.
;=========================================================================
HTM_SkipToLBrace
   LD    B,SSTYL_CHR
   JR    HTM_SkipToChar
;=========================================================================
;Name     HTM_SkipToRBrace
;Purpose  To find the '}' character.
;In Regs  AF = ??
;         B = ??
;         C = HTML bank
;         DE = ??
;         HL = address of HTML data
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars None
;Uses     TBD
;Call By  TBD
;Calls    HTM_NextHTMBank
;Notes    1. HTML bank left swapped in.
;=========================================================================
HTM_SkipToRBrace
   LD    B,ESTYL_CHR
;=========================================================================
;Name     HTM_SkipToChar
;Purpose  To find start of specific character within the HTML data.
;In Regs  AF = ??
;         B = char to stop at
;         C = HTML bank
;         DE = ??
;         HL = address of HTML data
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         B = ??
;         C = HTML bank index
;         DE = ??
;         HL = address after matching character at address of NULL
;         IX = ??
;         IY = ??
;Out Vars None
;Uses     TBD
;Call By  TBD
;Calls    HTM_NextHTMBank
;Notes    1. HTML bank left swapped in.
;=========================================================================
HTM_SkipToChar
   LD    A,C
   CALL  HTM_NextHTMBank
;
HTM_SkipToChr
   LD    A,C
   BIT   7,H
   CALL  NZ,HTM_NextHTMBank
   LD    C,A
   LD    A,(HL)
   OR    A			;At end of all data?
   RET   Z			;Yes
   INC   HL
   CP    B			;At required char?
   JR    NZ,HTM_SkipToChr	;Yes
   RET
;=========================================================================
;Name     HTM_SkipCmdObj
;Purpose  To skip the HTML command data following the HTML command.
;In Regs  AF = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_CmdEnd = Current end of the HTML command
;Out Regs AF = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_CmdEnd = start address of the next HTML command.
;Call By  TBD
;Calls    HTM_SkipCmdData
;Notes    None
;=========================================================================
HTM_SkipCmdObj
   LD    HL,(HTML_CmdEnd)
   LD    A,(HTML_CmdEnd+2)
   CALL  HTM_SkipCmdData	;Skip object data to command
   LD    (HTML_CmdEnd),HL
   LD    (HTML_CmdEnd+2),A
   RET
;=========================================================================
;Name     HTM_SkipLblTxt
;Purpose  To skip the label text between two double quotes.
;In Regs  AF = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = address of HTML parameter
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = start address of parameter text
;         IX = ??
;         IY = ??
;Out Vars HTML_ParamLoc = address after second double quote
;Call By  TBD
;Calls    HTM_GetCmdParmVal
;Notes    None
;=========================================================================
HTM_SkipLblTxt
   CALL  HTM_GetCmdParmVal		;HL = address of label text
   PUSH  HL
   CALL  HTM_GetCmdParmVal		;Skip label text
   LD    (HTML_ParamLoc),HL
   POP   HL
   RET
;=========================================================================
;Name     HTM_SkipCmdData
;Purpose  To skip the HTML command data which follows the HTML.
;Input    A = RAM bank index
;         HL = start address of HTML command after the '<'
;Output   HL = address of HTML text
;         A = Bank index
;Uses     TBD
;Notes    1. This routine is for HTML commands which are bound by the end
;            command (e.g <table>.....</table>).
;=========================================================================
HTM_SkipCmdData
   LD    (HTML_ObjLocn),HL
   LD    (HTML_ObjLocn+2),A
;
;  Copy out command to HTML_LblTemp for the find
;
   LD    HL,(HTML_CmdTemp)
   LD    DE,HTML_LblTemp
   LD    B,0				;B = length of text
HTM_SkipCmdDat1
   LD    A,(HL)
   CP    SPACE_CHAR			;below space or a space
   JR    Z,HTM_SkipCmdDat2		;Yes
   CP    EHTML_CHR			;end of HTML command?
   JR    Z,HTM_SkipCmdDat2		;Yes
   CALL  UTL_tolower
   LD    (DE),A
   INC   HL
   INC   DE
   INC   B
   JR    HTM_SkipCmdDat1
;
;  Search for matching </....> command
;
HTM_SkipCmdDat2
   LD    (HL),0
   LD    A,B
   LD    (HTML_LblTempL),A
;
   LD    HL,(HTML_ObjLocn)
HTM_SkipCmdDat3
   CALL  CPCB_ShowTimer			;Display timer icon
;
   LD    A,(HTML_ObjLocn+2)
   CALL  HTM_FindCmdStrt		;Find start of HTML command
   LD    A,C
;
   CALL  HTM_GetCmdObj			;Get HTML Command Object
;
   CALL  HTM_Get1stCmdChr		;command starts with '/'
   JR    NZ,HTM_SkipCmdDat3a		;No
   INC   HL				;Skip slash to command name
HTM_SkipCmdDat3a
;
   LD    DE,HTML_LblTemp
   LD    A,(HTML_LblTempL)
   LD    B,A
   CALL  UTL_strncmp			;End of command found?
   JR    Z,HTM_SkipCmdDat4		;Yes
;
   LD    HL,(HTML_ObjLocn)
   INC   HL
   JR    HTM_SkipCmdDat3
;
HTM_SkipCmdDat4
   LD    A,(HTML_ObjLocn+2)
   LD    HL,(HTML_ObjLocn)
   INC   HL
   JP    HTM_NextHTMaddr		;setup next HTML address
;=========================================================================
;Name     HTM_ChkCmd
;Purpose  To lookup the entered command for a valid one to execute.
;Input    DE = address of internal commands
;         HL = address of command to find.
;Output   DE = address of next command if not found
;         C = Status of check
;             Bit 7 = 1 -> valid command
;                   = 0 -> invalid command
;         HTML_CmdData = command handler data (if found)
;Uses     A corrupt
;Notes    None
;=========================================================================
HTM_ChkCmd
   LD    A,(DE)			;Get text of internal command
   BIT   7,A			;At command info byte?
   JR    NZ,HTM_endcmd		;Yes
;
   LD    C,A
   LD    A,(HL)
   CALL  UTL_tolower		;Make L.C to check with internal data
   CP    C			;Matches requested char?
   JR    NZ,HTM_badcmd		;No
   INC   DE
   CP    SPACE_CHAR		;Is char a space?
   JR    Z,HTM_ChkCmd		;Yes (dont go farther on command)
   INC   HL
   JR    HTM_ChkCmd
;
HTM_endcmd
   LD    C,A			;Save command info byte
   LD    A,(HL)
   CP    SPACE_CHAR		;Parameters after HTML command?
   JR    Z,HTM_ChkCmd_exit	;Yes (command match)
;
   CP    EHTML_CHR		;end of HTML command format?
   JR    Z,HTM_ChkCmd_exit	;Yes (command match)
;
   CP    ESTYL_CHR		;end of style data?
   JR    Z,HTM_ChkCmd_exit	;Yes (command match)
;
   CP    DSTYL_CHR		;end of style command ?
   JR    Z,HTM_ChkCmd_exit	;Yes (command match)
;
   CP    COLON_CHR		;style command parameter delimiter?
   JR    Z,HTM_ChkCmd_exit	;Yes (command match)
;
HTM_badcmd
   LD    A,(DE)			;Goto end of command text
   INC   DE
   BIT   7,A			;end of internal command?
   JR    Z,HTM_badcmd		;no (keep going)
;
   INC   DE			;Skip handler address data
   INC   DE
   INC   DE			;skip handler memory type
   LD    C,0			;Flag command does not match
   RET
;
HTM_ChkCmd_exit
   LD    A,C
   EX    HL,DE
   LD    DE,HTML_CmdData
   LD    BC,4
   LDIR				;upload handler data for found cmd
   LD    C,A			;Flag command match
   RET
;=========================================================================
;Name     HTM_FindCmdStyle
;Purpose  To Find the style command in the list of known commands
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = Address of style command text
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = result of find (0=not found, 1=Found)
;         BC = ??
;         DE = ??
;         HL = corrupted
;         IX = ??
;         IY = ??
;Out Vars HTML_CmdAddr = updated address of style command DB entry
;         HTML_CmdID = command index (if found a match)
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    TBD
;=========================================================================
HTM_FindCmdStyle
   LD    DE,HTM_StyleCmds
   JR    HTM_FindHTMLCmd0
;=========================================================================
;Name     HTM_FindHTMLCmd
;Purpose  To Find the HTML command in the list of known HTML commands
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = Address of HTML command text
;         IX = ??
;         IY = ??
;In Vars  HTML_Commands = list of HTML command
;Out Regs A = result of find (0=not found, 1=Found)
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_CmdData = command DB entry if found.
;         HTML_CmdAddr = updated address of HTML command if </....>
;         HTML_CmdID = command index (if found a match)
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_FindHTMLCmd
   LD    DE,HTML_Commands
HTM_FindHTMLCmd0
   XOR   A
   LD    (HTML_CmdID),A			;assume matches first valid command
   LD    A,(HL)
   CP    SLASH_CHAR			;is first char of command '/' ?
   JR    NZ,HTM_FindHTMLCmd1		;No
   INC   HL				;Skip it to command text
HTM_FindHTMLCmd1
   PUSH  HL
   call  HTM_ChkCmd			;Check if command is valid
   POP   HL
   BIT   7,C				;Did we find a valid command?
   JR    NZ,HTM_FindHTMLCmd2		;Yes
   LD    A,(HTML_CmdID)
   INC   A
   LD    (HTML_CmdID),A
   LD    A,(DE)
   OR    A				;End of command lookup list ?
   JR    NZ,HTM_FindHTMLCmd1		;No, check next command
   RET
HTM_FindHTMLCmd2
   LD    A,Find_Okay
   RET
;=========================================================================
;Name     HTM_GetCmd
;Purpose  To get the HTML command delimited by the '<' and '>' characters.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_ObjLocn = 3 byte address of HTML command start
;         HTML_CmdTemp = location of KB_Buffer
;Out Regs AF = ??
;         B = corrupted
;         C = html bank index.
;         DE = ??
;         HL = address of ">" which ends the command
;         IX = ??
;         IY = ??
;Out Vars KB_Buffer = HTML command for checking (Max 255 chars)
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. The HTML banked of memory already switched in.
;         2. Assume the HTML command is complete and not incomplete.
;=========================================================================
HTM_GetCmd
   LD    DE,(HTML_CmdTemp)
   LD    HL,(HTML_ObjLocn)
   LD    A,(HTML_ObjLocn+2)
;=========================================================================
;Name     HTM_CpyCmd
;Purpose  To copy the HTML command delimited by the '<' and '>' characters.
;In Regs  A = HTML Bank index
;         BC = ??
;         DE = Location to copy to.
;         HL = HTML address of command at '<'
;         IX = ??
;         IY = ??
;In Vars  HTML_html = lower
;Out Regs AF = ??
;         B = corrupted
;         C = html bank index.
;         DE = ??
;         HL = address of ">" which ends the command
;         IX = ??
;         IY = ??
;Out Vars HTML_html = updated.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. The HTML banked of memory already switched in.
;         2. Does not currently handle parameters contained by double
;            quotes to protect value.
;=========================================================================
HTM_CpyCmd
   LD    C,A
   LD    B,KB_BuffSz-1
   LD    A,(HL)
   CP    SHTML_CHR
   JR    Z,HTM_GetCmd3
   DEC   HL
   JR    HTM_GetCmd3
;
HTM_GetCmd1
   LD    A,(HL)
   CP    SPACE_CHAR		;control code?
   JR    C,HTM_GetCmd3		;Yes (skip it)
;
   OR    A			;end of HTML ?
   JR    Z,HTM_GetCmd4		;Yes
;
   LD    (DE),A			;Save HTML command text
   INC   DE
;
   CP    EHTML_CHR		;end of command?
   JR    Z,HTM_GetCmd4		;Yes
;
   CP    DASH_CHR		;Comment char
   JR    NZ,HTM_GetCmd3		;No
;
   DEC   DE
   DEC   DE
   LD    A,(DE)			;Get previous char
   INC   DE
   INC   DE
   CP    '-'			;Are last 2 chars '--'?
   JR    NZ,HTM_GetCmd3		;No (continue on as normal)
;
;  find next "--" or '>'
;
   DEC   DE			;back to start of comment
   DEC   DE
HTM_GetCmd5
   LD    A,(HTML_html)
   AND   &F0			;clear '-' counter
   LD    (HTML_html),A
HTM_GetCmd6
   INC   HL			;next char after '--'
   BIT   7,H			;crossed bank memory region?
   LD    A,C
   CALL  NZ,HTM_NextHTMBank	;Yes (next HTML bank)
   LD    C,A
;
   LD    A,(HL)
   CP    EHTML_CHR		;end of command?
   JR    Z,HTM_GetCmd4		;Yes
;
   CP    DASH_CHR		;found a '-'?
   JR    NZ,HTM_GetCmd5		;no (reset counter and try again)
;
   LD    A,(HTML_html)
   INC   A			;tally number of '-'
   LD    (HTML_html),A
   CP    2			;found "--"?
   JR    NZ,HTM_GetCmd6		;No
;
HTM_GetCmd3
   INC   HL			;next HTML byte
   BIT   7,H			;crossed bank memory region?
   LD    A,C
   CALL  NZ,HTM_NextHTMBank	;Yes (next HTML bank)
   LD    C,A
   DJNZ  HTM_GetCmd1
;
;  HTML command exceeds buffer, so skip rest
;  May cause issue with handling as last parameter is likely to be
;  incomplete.
;
HTM_GetCmd7
   INC   HL			;next char after '--'
   BIT   7,H			;crossed bank memory?
   LD    A,C
   CALL  NZ,HTM_NextHTMBank	;Yes (next HTML bank)
   LD    C,A
   LD    A,(HL)
   OR    A			;end of All HTML ?
   JR    NZ,HTM_GetCmd4		;Yes (unexpected)
;
   CP    EHTML_CHR		;end of command?
   JR    NZ,HTM_GetCmd7		;Yes
;
HTM_GetCmd4
   XOR   A
   LD    (DE),A			;terminate command for find.
   RET
;=========================================================================
;Name     HTM_GetNxtCmd
;Purpose  To get the next HTML command delimited by the '<' and '>'
;         characters.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_ObjLocn = 3 byte address of HTML command start
;         HTML_CmdTemp = location of KB_Buffer
;Out Regs AF = ??
;         B = corrupted
;         C = html bank index.
;         DE = ??
;         HL = address of ">" which ends the command
;         IX = ??
;         IY = ??
;Out Vars KB_Buffer = HTML command for checking (Max 255 chars)
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. The HTML banked of memory already switched in.
;         2. Assume the HTML command is complete.
;=========================================================================
HTM_GetNxtCmd
   CALL  HTM_FindCmdStrt		;Search for start of HTML command
   LD    A,C
;=========================================================================
;Name     HTM_GetCmdObj
;Purpose  To get the HTML command delimited by the '<' and '>' characters.
;In Regs  A = bank index of html data
;         BC = ??
;         DE = ??
;         HL = address of HTML data
;         IX = ??
;         IY = ??
;In Vars  HTML_CmdTemp = location of KB_Buffer
;         HTML_ObjLocn = start address of HTML command
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_Buffer = HTML command for checking (Max 127 chars)
;         HTML_ObjLocn = end address of HTML command
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. The HTML banked of memory already switched in.
;         2. Assume the HTML command is complete and not incomplete.
;=========================================================================
HTM_GetCmdObj
   LD    (HTML_ObjLocn),HL		;setup var for HTM_GetCmd
   LD    (HTML_ObjLocn+2),A
   CALL  HTM_GetCmd			;Get HTML command into KB_Buffer
   LD    A,C
   LD    (HTML_ObjLocn),HL		;Save end address of command
   LD    (HTML_ObjLocn+2),A
   RET
;=========================================================================
;Name     HTM_GetCmdParam
;Purpose  To get address of the the next parameter delimited by a space.
;In Regs  A = ??
;         B = Parameter Number.
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_ParamLoc = start address for parameter location
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = address of parameter.
;         IX = ??
;         IY = ??
;Out Vars HTML_ParamLoc = location of next parameter in command
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. assume no spaces in parameter values and only one space
;            between parameters.
;         2. Assume command is not prematurely terminated with NUL.
;=========================================================================
HTM_GetCmdParam
   LD    HL,(HTML_ParamLoc)
HTM_GetCmdParm1
   LD    A,(HL)
   CP    EHTML_CHR			;end of HTML command
   JR    Z,HTM_GetCmdParm2		;Yes
   INC   HL
   CP    SPACE_CHAR			;space delimiter between parameters
   JR    NZ,HTM_GetCmdParm1		;No (skip current text)
;
HTM_GetCmdParm2
   LD    (HTML_ParamLoc),HL		;Next parameter found
   RET
;=========================================================================
;Name     HTM_GetCmdParmVal
;Purpose  To get the address of the parameter value
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = Address of parameter
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = address of parameter value text
;         IX = ??
;         IY = ??
;Out Vars HTML_ParamVLoc = location of parameter value
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_GetCmdParmVal
   LD    A,(HL)
   INC   HL
   CP    DBLEQ_CHR
   JR    NZ,HTM_GetCmdParmVal
   LD    (HTML_ParamVLoc),HL		;Parameter location found
   RET
;=========================================================================
;Name     HTM_GetStartEnd
;Purpose  To locate the start and end of the HTML command delimited by
;         the '<' and '>' characters.
;In Regs  A = index into the HTML banks vector.
;         BC = ??
;         DE = ??
;         HL = address of start of the HTML command (i.e at "<")
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_CmdStart = 3 byte address of HTML command start
;         HTML_CmdEnd = 3 byte address of HTML command end.
;         HTML_CmdTemp = first 16 bytes of HTML command for checking.
;         HTML_ObjStart = start location of HTML object
;         HTML_ObjLocn = end location of command.
;         HTML_CmdTemp = address of buffer containing the HTML command.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. Upon entry the current position points to the start of HTML
;            command type.
;         2. The HTML memory bank is swapped in.
;=========================================================================
HTM_GetStartEnd
   LD    (HTML_CmdStart+2),A	;setup start block index
   LD    (HTML_CmdStart),HL	;setup start address in block
;
   LD    (HTML_ObjLocn+2),A	;setup start block index
   LD    (HTML_ObjLocn),HL	;setup start address in block
   LD    DE,(HTML_CmdTemp)
   CALL  HTM_CpyCmd		;Copy out command for search
   LD    A,C
   LD    (HTML_CmdEnd),HL	;save end address of command
   LD    (HTML_CmdEnd+2),A
;
   JP    MEM_SwapInMain		;restore memory config.
;=========================================================================
;Name     HTM_CmpHTMLCmd
;Purpose  To compare the current got command with the original.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_ObjTemp = command to compare against
;         HTML_CmdTemp = current got command
;Out Regs AF = corrrupted
;         B = corrupted.
;         C = status of check (0=no match, 1=matches)
;         DE = corrupted
;         HL = corrupted
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_CmpHTMLCmd
   CALL  HTM_Get1stCmdChr
   JR    NZ,HTM_CmpHTMLCmd1
   INC   HL			;Skip slash in cmd name
HTM_CmpHTMLCmd1
   LD    DE,HTML_ObjTemp
   LD    B,HTML_ObjTempSz
   LD    C,0			;Assume original match latest
HTM_CmpHTMLCmd2
   LD    A,(DE)
   CP    (HL)			;commands matchs?
   RET   NZ			;No
   CP    SPACE_CHAR		;at start of parameters in cmd?
   RET   Z			;Yes (must bae valid cmd)
   CP    EHTML_CHR		;at end of command?
   RET   Z			;Yes (must be valid cmd)
   INC   HL
   INC   DE
   DJNZ  HTM_CmpHTMLCmd2
   INC   C
   RET
;=========================================================================
;Name     HTM_chkHTMLCmd
;Purpose  To check if there is a HTML command is to be processed at the
;         current loaded file position.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_CurrLoc = current block address of loaded file.
;Out Regs AF = ??
;         B = 0 means not HTML command
;             1 mean found HTML command is known
;             2 means found HTML command not known.
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_CmdStart = 3 byte address of HTML command start
;         HTML_CmdEnd = 3 byte address of HTML command end.
;         HTML_CmdTemp = first 32 bytes of HTML command for checking.
;         HTML_CmdData = 4 bytes of handler data for found HTML command.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_chkHTMLCmd
   LD    HL,(HTML_CurrLoc)
   LD    A,(HTML_CurrLoc+2)
   CALL  Mem_SwapInHTML
   LD    A,(HL)
   LD    B,0
   CP    SHTML_CHR		;Start of HTML command
   RET   NZ			;No
;
   LD    A,(HTML_CurrLoc+2)
   CALL  HTM_GetStartEnd	;Get start/end of cmd and copy start
;
   LD    HL,(HTML_CmdTemp)
   CALL  HTM_FindHTMLCmd	;lookup command
   LD    B,1			;HTML command is known
   CP    Find_Okay		;Find Okay?
   RET   Z			;Yes
   LD    B,2			;HTML command is unknown
   RET
;=========================================================================
;Name     HTM_FindParmLbl
;Purpose  To locate which parameter label was found in the HTML command.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = Address of text to check against
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = corrupted
;         B = parameter from list (255=not found)
;         DE = ??
;         HL = corrupted
;         IX = ??
;         IY = ??
;Out Vars HTML_FrmsTmp = Location of parameter text.
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    TBD
;=========================================================================
HTM_FindParmLbl
   LD    DE,HTM_ParamTypes
;=========================================================================
;Name     HTM_SearchParmLbl
;Purpose  To search a list of parameter texts,
;In Regs  A = ??
;         BC = ??
;         DE = address of text to match against.
;         HL = Address of text to check against
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = corrupted
;         B = parameter from list (255=not found)
;         DE = ??
;         HL = corrupted
;         IX = ??
;         IY = ??
;Out Vars HTML_FrmsTmp = Location of parameter text.
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    1. If string is longer then internal then issue is there.
;            e.g checkbox = checkbox, checkboxes, checkboxing, etc
;=========================================================================
HTM_SearchParmLbl
   LD    (HTML_FrmsTmp),HL	;Save address of parameter label
   EX    DE,HL
   LD    B,0
;
HTM_SrchPLbl0
   LD    DE,(HTML_FrmsTmp)
;
HTM_SrchPLbl1
   LD    A,(DE)			;Get html param label char
   CALL  UTL_tolower
   LD    C,(HL)
   XOR   C			;If same then bits 6-0 = 0
   AND   &7F			;remove bit 7 (end of string flag)
   OR    A			;both chars the same?
   JR    NZ,HTM_SrchPLbl3	;No
   INC   HL
   INC   DE
   BIT   7,C			;end of lookup text?
   JR    Z,HTM_SrchPLbl1	;No (continue on)
   DEC   HL
;
   LD    A,(DE)			;Get char in parameter text
   OR    A
   RET   Z			;Yes end of text
   CP    EQUAL_CHR		;Is next char '='?
   RET   Z			;Yes match found
   CP    SPACE_CHAR		;Is char space?
   RET   Z			;Yes match found
   CP    DBLEQ_CHR		;Is char end of string?
   RET   Z			;Yes match found
   CP    ESTYL_CHR		;Is char end of style commands?
   RET   Z			;Yes match found
   CP    DSTYL_CHR		;Is char end of style command name?
   RET   Z			;Yes match found
;
HTM_SrchPLbl2
   INC   HL
HTM_SrchPLbl3
   BIT   7,(HL)			;At end of lookup string?
   JR    Z,HTM_SrchPLbl2	;Yes (check next lookup string)
;
   INC   HL			;Point start of next string
   INC   B			;next parameter label type
   LD    A,(HL)
   CP    128			;End of list
   JR    NZ,HTM_SrchPLbl0	;No
   LD    B,HTM_P_BadParam
   RET
;=========================================================================
;Name     HTM_DoCmdHandler
;Purpose  To execute the HTML command handler
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = address of HTML_CmdStates byte
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    HTM_Get1stCmdChr
;         HTM_GetStyleInfo
;         HTM_DoStyleCmds
;         HTM_UndoStyleCmds
;         Mem_SwapInCFG
;         MEM_SwapInMain
;         TOS = Routine to execute
;Notes    TBD
;=========================================================================
HTM_DoCmdHandler
   LD    HL,HTML_CmdData
   BIT   5,(HL)			;Can command be handled?
   RET   Z			;No
   LD    HL,(HTML_CmdData+1)
   LD    A,H
   OR    L
   RET   Z
   PUSH  HL			;setup address to execute via TOS
;
   LD    A,(HTML_CmdData)
   BIT   1,A			;Style's allowed in command?
   JR    Z,HTM_DoCmdHandler1	;No
;
   CALL  HTM_Get1stCmdChr	;First char '/'?
   JR    NZ,HTM_DoCmdHandler2	;No
;
   CALL  HTM_UndoStyleCmds	;undo the style settings.
   JR    HTM_DoCmdHandler1	;continue with command
;
HTM_DoCmdHandler2
;
;  Setup style format
;
   LD    A,(HTML_CmdID)
   LD    (HTML_StyleCmd),A
   PUSH  AF
   CALL  HTM_ChangeStyles	;change style format.
   POP   AF
   LD    (HTML_CmdID),A
;
HTM_DoCmdHandler1
   LD    A,(HTML_CmdID)
   LD    E,A
   LD    D,0
   LD    HL,HTML_CmdStates
   ADD   HL,DE
   LD    (HTML_CmdLoc),HL	;Setup location of command's state
   RET				;execute handler
;=========================================================================
;Name     HTM_DoHTMLCmd
;Purpose  To process the HTML command.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RCurrPos = start location of HTML command.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_Buffer has HTML command
;         HTML_ObjStart = start of object
;         HTML_RCurrPos = end of HTML command/Object
;         HTML_CmdEnd = end address of the HTML command
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_DoHTMLCmd
   LD    A,(HTML_RCurrPos+2)
   LD    HL,(HTML_RCurrPos)
;
   LD    (HTML_ObjStart),HL		;Save object start
   LD    (HTML_ObjStart+2),A
;
   CALL  HTM_GetStartEnd		;Get HTML cmd start/end, copy cmd
;
   LD    HL,(HTML_CmdTemp)
   CALL  HTM_FindHTMLCmd		;Attempt to find HTML command
;
   CP    Find_Okay			;Did we find it?
   CALL  Z,HTM_DoCmdHandler		;Yes, Execute Handler
;
   LD    HL,HTML_CmdEnd
   LD    DE,HTML_RCurrPos
   LD    BC,3
   LDIR					;RCurrPos = end of HTML command
   RET
