;=========================================================================
;
; Application Routines (accessing CPCB routines)
; Copyright (c) 2010 Ray Palmer
;
; Notes
;
; 1. This module allows external routines in the banked memory to be
;    directly accessed with out the need for routines in cpcb.exe. This
;    is for performance reasons, however where cpcb.exe variables need to
;    be set then the routines in cpcb.exe need to be called.
;
;=========================================================================
;Name     ERoutine_OpnCodeBank
;Purpose  Select the first code bank
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ???
;         IY = ??
;In Vars  N/A
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    MEM_SwapBCBank
;Notes    TBD
;=========================================================================
ERoutine_OpnCodeBank
   PUSH  AF
   PUSH  BC
   XOR   A
   JR    ERoutine_OpnCdeBnk
;=========================================================================
;Name     ERoutine_OpnCodeBank2
;Purpose  Select the second code bank
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ???
;         IY = ??
;In Vars  N/A
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    MEM_SwapBCBank
;Notes    TBD
;=========================================================================
ERoutine_OpnCodeBank2
   PUSH  AF
   PUSH  BC
   LD    A,1
ERoutine_OpnCdeBnk
   CALL  MEM_SwapBCBank
   POP   BC
   POP   AF
   RET
;=========================================================================
;Name     ERoutine_Add2URL
;Purpose  build the URL for the Forms submit process.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ???
;         IY = ??
;In Vars  N/A
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    ERoutine_OpnCodeBank
;         Ext_Add2URL
;Notes    TBD
;=========================================================================
ERoutine_Add2URL
   CALL  ERoutine_OpnCodeBank
   JP    Ext_Add2URL
;=========================================================================
;Name     ERoutine_OpenHTTP
;Purpose  Issue the build URL via the network
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ???
;         IY = ??
;In Vars  N/A
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    ERoutine_OpnCodeBank
;         Ext_OpenHTTP
;Notes    TBD
;=========================================================================
ERoutine_OpenHTTP
   CALL  ERoutine_OpnCodeBank
   JP    Ext_OpenHTTP
;=========================================================================
;Name     ERoutine_SendURL
;Purpose  Issue the built URL via the network
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TCPIP_state
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    ERoutine_OpnCodeBank
;         Ext_SendURL
;Notes    TBD
;=========================================================================
ERoutine_SendURL
   LD    A,(TCPIP_state)
   OR    A
   JP    Z,HTM_NetErrMsg
   CALL  ERoutine_OpnCodeBank
   LD    A,(TCPIP_state)
   JP    Ext_SendURL
;=========================================================================
;Name     ERoutine_SelectFontByName
;Purpose  To select a font by a name
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = Address of font name text
;         IX = ??
;         IY = ??
;In Vars  N/A
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    ERoutine_OpnCodeBank2
;         Ext_SelectFontByName
;Notes    TBD
;=========================================================================
ERoutine_SelectFontByName
   CALL  ERoutine_OpnCodeBank2
   JP    Ext_SelectFontByName
;=========================================================================
;Name     ERoutine_SelectFontByIndx
;Purpose  To select a font by it index into the fonts table dd.
;In Regs  A = ??
;         BC = ??
;         DE = Address in font table DB.
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  N/A
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    ERoutine_OpnCodeBank2
;         Ext_SelectFontByIndex
;Notes    TBD
;=========================================================================
ERoutine_SelectFontByIndx
   CALL  ERoutine_OpnCodeBank2
   JP    Ext_SelectFontByIndex
;=========================================================================
;Name     ERoutine_FontEffect
;Purpose  To change a fonts effect.
;In Regs  A = ??
;         B = Bit mask to show current setting
;         C = Bit mask to toggle current setting
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  N/A
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    ERoutine_OpnCodeBank2
;         Ext_NewEffect
;Notes    TBD
;=========================================================================
ERoutine_FontEffect
   CALL  ERoutine_OpnCodeBank2
   JP    Ext_NewEffect
;=========================================================================
;Name     ERoutine_ShowImgChr
;Purpose  To display the image bytes for a character
;In Regs  A = Screen mode for data to be displayed.
;         BC = Address of scan line data
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_LblTemp = bytes of image data for character on screen
;         IMG_ScrnMde = image screen mode
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    CPCB_CPCAccess
;         ERoutine_OpnCodeBank
;         Ext_Add2URL
;Notes    TBD
;=========================================================================
ERoutine_ShowImgChr
   CALL  CPCB_AdjustRow
   CALL  ERoutine_OpnCodeBank2
   LD    BC,HTML_LblTemp
   LD    A,(IMG_ScrnMde)
   LD    DE,0
   JP    EXT_ShowImgPattn
;
