;=========================================================================
;
; Application Routines (Forms rendering commands)
; Copyright (c) 2010 Ray Palmer
;
;=========================================================================
;
;  Routines specific to the forms commands and related commands.
;
;=========================================================================
;Name     HTM_ChkFrmsState
;Purpose  To determine the state of processing forms data.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState =  status flags
;Out Regs AF = NZ flag set if acquiring table info else Z
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars None.
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
HTM_ChkFrmsState
   LD    HL,HTML_FormsFlag
   BIT   7,(HL)
   RET
;=========================================================================
;Name     HTM_SetActive
;Purpose  To flag the state of the input item as being active/selected.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = corrupted
;         IX = ??
;         IY = ??
;Out Vars HTML_FrmsValue = updated on bit 6
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
HTM_SetActive
   LD    HL,HTML_FrmsValue
   SET   6,(HL)				;Flag item is active.
   RET
;=========================================================================
;Name     HTM_ClrActive
;Purpose  To flag the state of the input item as not being active/selected.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = corrupted
;         IX = ??
;         IY = ??
;Out Vars HTML_FrmsValue = updated on bit 6
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
HTM_ClrActive
   LD    HL,HTML_FrmsValue
   RES   6,(HL)				;Flag item is not active.
   RET
;=========================================================================
;Name     HTM_SetIniActive
;Purpose  To flag the state of the input item as being active/selected.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = corrupted
;         IX = ??
;         IY = ??
;Out Vars HTM_SetIniActive = updated on bit 6
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
HTM_SetIniActive
   LD    HL,HTML_FrmsIniVal
   SET   6,(HL)				;Flag item is active.
   RET
;=========================================================================
;Name     HTM_ResIniActive
;Purpose  To flag the state of the input item as being not active/selected.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = corrupted
;         IX = ??
;         IY = ??
;Out Vars HTM_SetIniActive = updated on bit 6
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
HTM_ResIniActive
   LD    HL,HTML_FrmsIniVal
   RES   6,(HL)				;Flag item is not active.
   RET
;=========================================================================
;Name     HTM_SetLabel
;Purpose  To flag the state of the input item has a label for display.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = corrupted
;         IX = ??
;         IY = ??
;Out Vars HTML_FrmsValue = updated on bit 4
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
HTM_SetLabel
   LD    HL,HTML_FrmsIniVal
   SET   4,(HL)				;Flag item has label
   RET
;=========================================================================
;Name     HTM_ClrLabel
;Purpose  To flag the state of the input item has no label for display.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = corrupted
;         IX = ??
;         IY = ??
;Out Vars HTML_FrmsValue = updated on bit 4
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
HTM_ClrLabel
   LD    HL,HTML_FrmsIniVal
   RES   4,(HL)				;Flag item has no label
   RET
;=========================================================================
;Name     HTM_SetShown
;Purpose  To enable the option to be displayed.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = corrupted
;         IX = ??
;         IY = ??
;Out Vars HTML_FrmsIniVal = updated on bit 4
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
HTM_SetShown
   LD    HL,HTML_FrmsTyp
   SET   4,(HL)				;Flag item is displayable
   RET
;=========================================================================
;Name     HTM_ClrShown
;Purpose  To disable the option to be displayed.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = corrupted
;         IX = ??
;         IY = ??
;Out Vars HTML_FrmsIniVal = updated on bit 4
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
HTM_ClrShown
   LD    HL,HTML_FrmsTyp
   RES   4,(HL)				;Flag item is not displayable
   RET
;=========================================================================
;Name     HTM_FormCalcDB
;Purpose  To calculate the DB address for a Db item.
;In Regs  A = DB index.
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = address of DB entry
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. The Forms DB memory bank is not swapped in.
;=========================================================================
HTM_FormCalcDB
   LD    HL,CFG_FormsDB
   LD    BC,S_HTMLVarSz
HTM_FormCalcDB1
   OR    A
   RET   Z
   ADD   HL,BC
   JR    HTM_FormCalcDB1
;=========================================================================
;Name     HTM_FormCalcDBEnd
;Purpose  To calculate the DB address for end Db item.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  ConfigBank = configuration bank for Forms DB.
;Out Regs A = corrupted
;         BC = corrupted
;         DE = ??
;         HL = address of DB entry
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. configuration bank left swapped in.
;=========================================================================
HTM_FormCalcDBEnd
   CALL  MEM_SwapInCFG			;Swap in Config bank
   LD    HL,CFG_FormsDB
   LD    BC,S_HTMLVarSz
HTM_FormCalcDBEnd0
   LD    A,(HL)
   CP    HTML_P_EODB			;entry free?
   RET   Z				;Yes
   ADD   HL,BC
   JR    HTM_FormCalcDBEnd0		;try next entry
;=========================================================================
;Name     HTM_FrmsNewLine
;Purpose  To setup a new line for forms handling.
;In Regs  A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_Row = current forms web-page row
;         HTML_Col = base forms web-page column
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_FrmsNewLine
   LD    HL,(HTML_Row)
   INC   HL			;next row in forms window
   LD    (HTML_Row),HL
   LD    HL,HTML_FormsFlag
   BIT   2,(HL)			;Group active?
   RET   NZ			;Yes
   LD    HL,(HTML_FrmsECol)
   LD    (HTML_Col),HL		;reset start forms window column
   RET
;=========================================================================
;Name     HTM_FormParms
;Purpose  To process the parameters to the <FORM..> command
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  KB_Buffer = HTML command.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_FormParms
   LD    HL,(HTML_CmdTemp)
   LD    (HTML_ParamLoc),HL
HTM_FormParms1
   CALL  HTM_GetCmdParam		;Locate start of next parameter
   LD    A,(HL)
   CP    EHTML_CHR			;no more parameters?
   RET   Z				;Yes (end of info)
   PUSH  HL
   CALL  HTM_FindParmLbl		;Determine the parameter nametype
   POP   HL
   LD    A,HTM_P_method
   CP    B				;Parameter 'method' ?
   JR    Z,FormParms2			;Yes
   LD    A,HTM_P_action
   CP    B				;Parameter 'action' ?
   JR    Z,FormParms3			;Yes
   LD    A,HTM_P_enctype
   CP    B				;Parameter 'enctype' ?
   JR    Z,FormParms4			;Yes
   JR    HTM_FormParms1
FormParms2
   CALL  HTM_GetCmdParmVal		;HL = address of parameter value
   LD    A,(HL)
   CALL  UTL_tolower
   LD    HL,HTML_FrmsIniVal
   CP    "g"				;Get method ?
   JR    Z,FormParms2a			;Yes
   RES   7,(HL)
   JR    HTM_FormParms1
FormParms2a
   SET   7,(HL)				;Flag method is get
   JR    HTM_FormParms1
FormParms3
   CALL  HTM_GetCmdParmVal		;HL = address of parameter value
   LD    (HTML_FrmsTxtLoc),HL		;save location of URL for action.
   JR    HTM_FormParms1
FormParms4
   CALL  HTM_GetCmdParmVal		;HL = address of parameter value
   LD    A,(HL)
   CALL  UTL_tolower
   LD    HL,HTML_FrmsIniVal
   CP    "a"				;url encoding?
   JR    NZ,FormParms4a
   RES   6,(HL)				;application/x-www-form-urlencoded
   JR    HTM_FormParms1
FormParms4a
   SET   6,(HL)				;multipart/form-data
   JR    HTM_FormParms1
;=========================================================================
;Name     HTM_textareaSz
;Purpose  To get the number of rows/columns for the text box from the
;         parameters to the html <textarea> command parameters.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  KB_Buffer = HTML command.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_FrmsNCols = Number of columns (1-255)
;         HTML_FrmsNRows = number of rows (1-255)
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_textareaSz
   LD    A,10
   LD    (HTML_FrmsNRows),A	;Default box to size to 10,10
   LD    (HTML_FrmsNCols),A
;
   LD    HL,(HTML_CmdTemp)
   LD    (HTML_ParamLoc),HL
HTM_textareaSz1
   CALL  HTM_GetCmdParam	;Locate start of next parameter
   LD    A,(HL)
   CP    EHTML_CHR		;no more parameters?
   RET   Z			;Yes (end of info)
   PUSH  HL
   CALL  HTM_FindParmLbl	;Determine the parameter nametype
   POP   HL
   LD    A,HTM_P_Rows
   CP    B			;Parameter 'Rows' ?
   JR    Z,HTM_textareaSz2	;Yes
   LD    A,HTM_P_Cols
   CP    B			;Parameter 'Cols' ?
   JR    Z,HTM_textareaSz3	;Yes
   JR    HTM_textareaSz1
;
HTM_textareaSz2
   CALL  HTM_GetWidthValue	;BC = number of rows
   LD    A,C
   LD    (HTML_FrmsNRows),A	;setup number of rows
   JR    HTM_textareaSz1	;check next parameter
;
HTM_textareaSz3
   CALL  HTM_GetWidthValue	;BC = number of columns
   LD    A,C
   LD    (HTML_FrmsNCols),A	;setup number of columns
   JR    HTM_textareaSz		;check next parameter
;=========================================================================
;Name     HTM_FrmsGetInpSz
;Purpose  To get the size of text for forms Input item.
;In Regs  A = input type from forms Db entry structure.
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  None.
;Out Regs A = ??
;         B = number of rows to item type.
;         C = number of columns to item type
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars None
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_FrmsGetInpSz
   LD    BC,8
   CP    HTML_I_SUBMIT
   RET   Z
   LD    BC,7
   CP    HTML_I_RESET
   RET   Z
   LD    BC,20				;Space for password/text
   CP    HTML_I_Text
   RET   Z
   CP    HTML_I_Password
   RET   Z
   CP    HTML_I_Button
   JP    Z,HTM_ButtonSz
   LD    BC,0				;size of icons
   RET
;=========================================================================
;Name     HTM_ButtonSz
;Purpose  To get the size of a button from the <INPUT...> command.
;In Regs  A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = Address of text ending with "
;         IX = ??
;         IY = ??
;In Vars  None
;Out Regs A = ??
;         BC = ??
;         DE = Size of text max size is 255
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars None.
;Uses     TBD
;Call By  html_cmd_option
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_ButtonSz
   LD    HL,(HTML_LblTemp)
   LD    BC,0
HTM_ButtonSz1
   LD    A,(HL)
   CP    DBLEQ_CHR			;no more parameters?
   RET   Z				;Yes (end of info)
   INC   HL
   INC   C
   JR    HTM_ButtonSz1
;=========================================================================
;Name     HTM_FormUpdSz
;Purpose  To update the dimensions of the forms item for buttons.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_Row = current rendering row
;         HTML_Col = current rendering column
;         HTML_FrmsBRow = base row of form item
;         HTML_FrmsBCol = base column of form item
;         HTML_FrmsTyp = Forms type
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_FormUpdSz
   LD    HL,HTML_FrmsTyp
   SET   7,(HL)			;Flag rendered item.
   LD    A,(HL)
   AND   HTML_I_FORM
   CP    HTML_I_BUTTON
   JR    Z,HTM_FrmUpdSz0
   CP    HTML_I_BUTTONC
   RET   NZ
HTM_FrmUpdSz0
   LD    HL,(HTML_ImgRow)
   LD    A,H			;was image displayed for button?
   OR    L
   JR    NZ,HTM_FrmUpdSz1	;Yes
   LD    HL,(HTML_Row)
HTM_FrmUpdSz1
   LD    DE,(HTML_FrmsBRow)
   OR    A
   SBC   HL,DE
   LD    A,L
   LD    (HTML_FrmsNRows),A
   LD    HL,(HTML_Col)
   LD    DE,(HTML_FrmsBCol)
   OR    A
   SBC   HL,DE
   LD    A,L
   DEC   A
   LD    (HTML_FrmsNCols),A
   RET
;=========================================================================
;Name     HTM_FormFindHTML
;Purpose  To check if the Form item is already in DB by its HTML address
;         location.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_CmdStart = start address of HTML command to find.
;Out Regs A = Find result (HTML_P_EODB = 'Not found' else 0 found)
;         B = ??
;         C = Db entry number
;         DE = ??
;         HL = Address of forms database entry.
;         IX = ??
;         IY = ??
;Out Vars HTML_FrmsDBLocn = Address of forms database entry.
;         HTML_FormsObj = downloaded object if found
;Uses     TBD
;Call By  TBD
;Calls    MEM_SwapInCfg
;Notes    1. The Config bank is left swapped in.
;=========================================================================
HTM_FormFindHTML
   CALL  MEM_SwapInCfg			;Swap in Forsm DB bank.
   LD    HL,CFG_FormsDB
   LD    C,0
HTM_FormFndHTML1
   LD    A,(HL)				;Get DB item type
   CP    HTML_P_EODB			;End of DB
   RET   Z				;Yes
   PUSH  HL
   LD    DE,S_HTMLVarAdr
   ADD   HL,DE
   LD    DE,HTML_CmdStart
   LD    B,3
   CALL  UTL_strncmp			;Compare 3 bytes
   POP   HL
   JR    Z,HTM_FormFndHTML3		;Yes (down load main memory)
   INC   C
   LD    DE,S_HTMLVarSz
   ADD   HL,DE				;next DB entry
   JR    HTM_FormFndHTML1
HTM_FormFndHTML3
   LD    (HTML_FrmsDBLocn),HL
   LD    DE,HTML_FormsObj
   LD    BC,S_HTMLVarSz
   LDIR					;download copy of db to work on
   XOR   A
   RET
;=========================================================================
;Name     HTM_FormsUpdDD
;Purpose  To update the database entry.
;In Regs  A = ??
;         BC = ??
;         DE = Address of database entry to update.
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    MEM_SwapInCfg
;         Mem_SwapInMain
;Notes    TBD
;=========================================================================
HTM_FormsUpdDD
   LD    A,(HTML_FrmsTyp)
   BIT   4,A				;Item to be displayed?
   RET   Z				;No
HTM_FormsUpdDD2
   CALL  MEM_SwapInCfg			;Swap in Forms DB memory bank
   LD    DE,(HTML_FrmsDBLocn)		;Address in DB to update
   LD    HL,HTML_FormsObj		;Local DB entry to update with
   LD    BC,S_HTMLVarSz
   LDIR					;update new data
   JP    Mem_SwapInMain			;Restore memory config
;=========================================================================
;Name     HTM_FormsGetDB
;Purpose  To copy the current Db entry.
;In Regs  A = ??
;         BC = ??
;         DE = Address of database entry to update.
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    MEM_SwapInCfg
;         Mem_SwapInMain
;Notes    TBD
;=========================================================================
HTM_FormsGetDB
   CALL  MEM_SwapInCfg
   CALL  HTM_FormsCpyDB
   JP    Mem_SwapInMain
;=========================================================================
;Name     HTM_FormsCpyDB
;Purpose  To get the first DB entry.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_FormsCpyDB
   LD    HL,CFG_FormsDB
HTM_FormsCpyDBEnt
   LD    DE,HTML_FormsObj
   LD    BC,S_HTMLVarSz
   LDIR
   RET
;=========================================================================
;Name     HTM_FormGetHTML
;Purpose  To get the address of the HTML command from the Db entry
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = HTML bank index
;         BC = ??
;         DE = ??
;         HL = address of HTML command
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_FormGetHTML
   LD    HL,(HTML_FormsAddr)
   LD    A,(HTML_FormsAddr+2)
   RET
;=========================================================================
;Name     HTM_FormsShowTextItem
;Purpose  To display the text for a single line input.
;In Regs  A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_FormsObj = setup with object to display.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. Dont expect text to over flow from the forms text buffers.
;=========================================================================
HTM_FormShowTextItem
   LD    A,(HTML_FrmsNCols)		;size of text displayable
   LD    (HTML_TextStrMax),A		;Setup max width output of text
   LD    (HTML_TextStrlen),A
   LD    A,(HTML_FrmsNRows)
   LD    (HTML_TextLinNum),A		;Setup num lines of output
;
   CALL  Mem_SwapinCFG
   LD    HL,(HTML_FrmsTxtLoc)
   LD    A,(ConfigBank)
   LD    B,0				;end of string char
   LD    C,HTML_F_FntChrs
   CALL  HTM_ShowString			;Display the text
;=========================================================================
;Name     HTM_FormFillBlanks
;Purpose  To display the text input with balnaks where no input has been
;         shown.
;In Regs  A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    HTM_ShowChar
;Notes    TBD
;=========================================================================
HTM_FormFillBlanks
   LD    A,(HTML_TextLinNum)
   LD    C,A
   LD    A,(HTML_TextStrlen)		;Get remaining chars
   OR    A
   JR    Z,HTM_FormFillBlank1
   LD    B,A
HTM_FormFillBlank0
   PUSH  BC
   LD    B,HTML_P_BLOCK			;solid blank input line.
   LD    C,PRINT_ICONCHR
   CALL  HTM_ShowChar			;display block character
   POP   BC
   DJNZ  HTM_FormFillBlank0
HTM_FormFillBlank1
   LD    A,C
   OR    A
   RET   Z
   DEC   C
   LD    A,(HTML_TextStrMax)
   LD    B,A
   JR    HTM_FormFillBlank0
;=========================================================================
;Name     HTM_FormShowSUBMIT
;Purpose  To show SUBMIT button text
;In Regs  A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_FormShowSUBMIT
   LD    HL,HTM_C_SUBMIT
   LD    B,6
   JR    HTM_FormsSUBRES
;=========================================================================
;Name     HTM_FormShowRESET
;Purpose  To show RESET button text
;In Regs  A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_FormShowRESET
   LD    HL,HTM_C_RESET
   LD    B,5
HTM_FormsSUBRES
   LD    A,B
   ADD   A,2			;account for left/right characters
   LD    (HTML_TextStrlen),A	;max chars to output.
   XOR   A
   LD    (HTML_TextLinNum),A	;only one line
   LD    DE,App_Name		;overuse this var
   LD    A,"["
   LD    (DE),A
   INC   DE
   CALL  UTL_strncpy
   LD    A,"]"
   LD    (DE),A
   INC   DE
   XOR   A
   LD    (DE),A
;
   LD    A,&C0
   LD    HL,App_Name
   LD    B,0
   LD    C,HTML_F_FntChrs
   JP    HTM_ShowString		;Display the text
;=========================================================================
;Name     HTM_FormShowButton
;Purpose  To show button text.
;In Regs  A = button type
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_CmdTemp = address of keyboard buffer with command
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_ParamLoc = location parameters
;Uses     TBD
;Call By  TBD
;Calls    HTM_GetCmdParam
;         HTM_FindParmLbl
;         HTM_GetCmdParmVal
;         HTM_ShowChar
;Notes    1. The text for the button is from the <INPUT..> command.
;=========================================================================
HTM_FormShowButton
   LD    HL,(HTML_CmdTemp)
   LD    (HTML_ParamLoc),HL
HTM_FormsShwButn0
   CALL  HTM_GetCmdParam		;Locate next parameter
   LD    A,(HL)
   CP    EHTML_CHR			;no more parameters?
   RET   Z
   PUSH  HL
   CALL  HTM_FindParmLbl		;Determine the parameter nametype
   POP   HL
   LD    A,HTM_P_Value
   CP    B					;Is parameter 'value' ?
   JR    NZ,HTM_FormsShwButn0		;No
;
   CALL  HTM_GetCmdParmVal		;HL = address of parameter value
   LD    B,'['
   LD    C,HTML_F_FntChrs		;use System font
   PUSH  HL
   CALL  HTM_ShowChar			;Show char on web-page
   POP   HL
HTM_FormsShwButn1
   LD    A,(HL)
   CP    DBLEQ_CHR
   JR    Z,HTM_FormsShwButn2
   LD    B,A
   LD    C,HTML_F_SysChrs		;use System font
   PUSH  HL
   CALL  HTM_ShowChar			;Show char on web-page
   POP   HL
   INC   HL
   JR    HTM_FormsShwButn1
HTM_FormsShwButn2
   LD    B,']'
   LD    C,HTML_F_FntChrs		;use System font
   JP    HTM_ShowChar			;Show char on web-page
;=========================================================================
;Name     HTM_SetIconType
;Purpose  To setup the icon type.
;In Regs  A = Item type's value
;         B = icon type for active/Up
;         C = icon type for inactive/Down
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_FormsObj = Forms Object to display
;                         HTML_FrmsValue = current value/flags
;Out Regs A = ??
;         B = Icon to display
;         C = Flag to use icon/font characters
;         DE = ??
;         HL = corrupted.
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_SetIconType
   LD    A,(HTML_FrmsValue)
   BIT   6,A				;Is item active?
   JR    NZ,HTM_SetIconType1		;Yes
   LD    B,C				;Show option group icon (Down)
HTM_SetIconType1
   LD    C,HTML_F_FntChrs		;Flag use Font chars for text
   SET   7,B
   RET
;=========================================================================
;Name     HTM_FormShowIcon
;Purpose  To display the applicable icon for the forms input item.
;In Regs  A = ??
;         B = Icon number for inactive state.
;         C = Icon number for active state.
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_FormShowIcon
   LD    HL,HTML_FrmsTyp
   SET   7,(HL)				;Flag Item rendered
   CALL  HTM_SetIconType		;check if icon is 'Up' or 'Down'
   LD    C,PRINT_ICONCHR
   JP    HTM_ShowChar			;Show Item icon
;=========================================================================
;Name     HTM_FormShowOption
;Purpose  To display the Forms Option Item within a group if the options
;         are the active for the group.
;In Regs  A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_FormsObj = setup with object to display.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_FormShowOption
   CALL  HTM_FormShowIcon
   LD    A,(HTML_FrmsIniVal)
   BIT   4,A				;Label in option?
   RET   Z				;No
   LD    A,255
   LD    (HTML_TextStrlen),A		;Max length
   INC   A
   LD    B,DBLEQ_CHR
   LD    C,HTML_F_FntChrs
   LD    HL,(HTML_FrmsTxtLoc)
   CALL  HTM_ShowString			;display label as text for option
   LD    HL,(HTML_CmdEnd)
   LD    A,(HTML_CmdEnd+2)
   CALL  HTM_FindCmdStrt
   DEC   HL
   DEC   HL
   LD    (HTML_CmdEnd),HL
   LD    A,C
   LD    (HTML_CmdEnd+2),A
   RET
;=========================================================================
;Name     HTM_FormShowItem
;Purpose  To render the forms input item.
;In Regs  A = Item type to display (0-14, 15 not allowed)
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_Col = form item column in stream window
;         HTML_Row = form item row in stream window
;         HTML_FormsObj = setup with object to display.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_FormsObj = updated fields.
;         HTML_FormsPF = updated on bit 1 = 1
;Uses     TBD
;Call By  TBD
;Calls    HTM_FrmsShwTxt - shows button with custom text
;         HTM_FrmIcnTxtShw - shows a character icon
;         HTM_FrmOptShw - shows a grouped/option item
;         HTM_FrmTxtInLShw - shows a box for line input
;         HTM_FrmTxtBxShw = shows a text box
;Notes    1. All form input types use the system font.
;=========================================================================
HTM_FormShowItem
   LD    DE,(HTML_Row)
   LD    (HTML_FrmsBRow),DE		;setup base row/column of item
   LD    DE,(HTML_Col)
   LD    (HTML_FrmsBCol),DE
   LD    HL,HTML_FormsPF
   SET   1,(HL)				;flag now underway
;
   CP    HTML_I_Button
   JP    Z,HTM_FormShowButton		;show inverted text as button
;
   CP    HTML_I_Submit
   JP    Z,HTM_FormShowSubmit		;show inverted text as button
;
   CP    HTML_I_Reset
   JP    Z,HTM_FormShowReset		;show inverted text as button
;
   LD    B,HTML_P_RADIO_D
   LD    C,HTML_P_RADIO_U
   CP    HTML_I_Radio
   JP    Z,HTM_FormShowIcon		;show icon
;
   LD    B,HTML_P_CHKBOX_U
   LD    C,HTML_P_CHKBOX_D
   CP    HTML_I_Chkbox
   JP    Z,HTM_FormShowIcon		;show icon
;
   LD    B,HTML_P_IMAGE
   LD    C,HTML_P_IMAGE
   CP    HTML_I_Image
   JP    Z,HTM_FormShowIcon		;show icon
;
   LD    B,HTML_P_FILE
   LD    C,HTML_P_FILE
   CP    HTML_I_File
   JP    Z,HTM_FormShowIcon		;show icon
;
   LD    B,HTML_P_GROUP_U
   LD    C,HTML_P_GROUP_D
   CP    HTML_I_OPTGRP
   JP    Z,HTM_FormShowOption		;show group icon
;
   LD    B,HTML_P_OPTION_U
   LD    C,HTML_P_OPTION_D
   CP    HTML_I_OPTION
   JP    Z,HTM_FormShowOption		;show option icon
;
   CP    HTML_I_Text
   JP    Z,HTM_FormShowTextItem		;show text + spaces for input
;
   CP    HTML_I_Password
   JP    Z,HTM_FormShowTextItem		;show text + spaces for input
;
   CP    HTML_I_TXTBOX
   JP    Z,HTM_FormShowTextItem		;Show text box/text
   RET
;=========================================================================
;Name     HTM_FormDisplayItem
;Purpose  To render the forms input item.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_Col = form item column in stream window
;         HTML_Row = form item row in stream window
;         HTML_FormsObj = setup with object to display.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_FormsObj = updated fields.
;Uses     TBD
;Call By  TBD
;Calls    HTM_FrmsShwTxt - shows button with custom text
;         HTM_FrmIcnTxtShw - shows a character icon
;         HTM_FrmOptShw - shows a grouped/option item
;         HTM_FrmTxtInLShw - shows a box for line input
;         HTM_FrmTxtBxShw = shows a text box
;Notes    1. All form input types use the system font.
;=========================================================================
HTM_FormDisplayItem
   LD    A,(HTML_FrmsTyp)
   BIT   4,A				;Item displayed?
   RET   Z				;No
   AND   HTML_I_FORM			;remove flags/status info
   CP    HTML_I_FORM			;Forms marker item?
   RET   Z				;Yes (do nothing)
;
   CALL  HTM_FormShowItem		;Display item on web-page
   CALL  HTM_FormUpdSz
   CALL  HTM_FormsUpdDD			;Update the DB with its position
   JP    HTM_ChkMaxCol			;Check the Max displayed column.
;=========================================================================
;Name     HTML_FrmsCalcBox
;Purpose  To calculate the form's item box for text inputs.
;In Regs  A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_FormsObj = setup with object to check.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_FrmsTmp = updated object
;                         HTML_FrmsSRow = start row
;                         HTML_FrmsSCol = start column
;                         HTML_FrmsERow = end row
;                         HTML_FrmsECol = end column
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTML_FrmsCalcBox
   LD    HL,(HTML_FrmsBRow)		;get item base row
   LD    (HTML_FrmsSRow),HL		;Setup start row
   LD    A,(HTML_FrmsNRows)
   LD    E,A
   LD    D,0
   ADD   HL,DE				;calc end row
   LD    (HTML_FrmsERow),HL		;setup end row
;
   LD    HL,(HTML_FrmsBCol)		;Get item base column
   LD    (HTML_FrmsSCol),HL		;setup start column
   LD    A,(HTML_FrmsNCols)
   LD    E,A
   LD    D,0
   ADD   HL,DE				;calc end column
   LD    (HTML_FrmsECol),HL		;setup end column
   RET
;=========================================================================
;Name     HTM_FrmTxtBxChk
;Purpose  To check if the cursor is on the forms input item.
;In Regs  A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  Cursor_Row = current cursor position (row)
;         Cursor_Col = current cursor position (column)
;         HTML_FormsObj = setup with object to display.
;Out Regs A = 0 inside box, else 1
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_FormsObj = updated.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_FrmChkInBox
   CALL  HTML_FrmsCalcBox	;calc input item area
   JP    UTL_CheckInBox		;check cursor in item area
;=========================================================================
;Name     HTM_FormsChkIcon
;Purpose  To check if the cursor is on the forms input item icon.
;In Regs  A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  WebPge_Row = cursor row position on web-page.
;         WebPge_Col = cursor column position on web-page.
;         HTML_FormsObj = current object to check.
;Out Regs A = 0 means on icon else 1
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_FormsChkIcon
   LD    A,1			;assume not on icon
   LD    HL,(HTML_FrmsBRow)
   LD    BC,(WebPge_Row)
   OR    A
   SBC   HL,BC			;At same row?
   RET   NZ			;No
   LD    HL,(HTML_FrmsBCol)
   LD    BC,(WebPge_Col)
   OR    A
   SBC   HL,BC			;At same column?
   RET   NZ			;No
   DEC   A			;Flag on icon
   RET
;=========================================================================
;Name     HTM_ChkItemOn
;Purpose  To check what sort of item cursor is on.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = 0 means on item else non-zero
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_FormsObj = forms object with item clicked on.
;Uses     TBD
;Call By  ??
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_ChkItemOn
   LD    A,(HTML_FrmsTyp)
   AND   HTML_I_FORM
   CP    HTML_I_BUTTONC		;Custom Button ?
   JP    Z,HTM_FrmChkInBox	;Yes (check if within dimensions of it)
;
   CP    HTML_I_Button		;Button with text ?
   JP    Z,HTM_FrmChkInBox	;Yes (check if within dimensions of it)
;
   CP    HTML_I_Submit		;Button with text ?
   JP    Z,HTM_FrmChkInBox	;Yes (check if within dimensions of it)
;
   CP    HTML_I_Reset		;Button with text ?
   JP    Z,HTM_FrmChkInBox	;Yes (check if within dimensions of it)
;
   CP    HTML_I_Text		;text followed input window?
   JP    Z,HTM_FrmChkInBox	;Yes (check if within dimensions of it)
;
   CP    HTML_I_Password	;text followed input window?
   JP    Z,HTM_FrmChkInBox	;Yes (check if within dimensions of it)
;
   CP    HTML_I_TXTBOX		;text box area
   JP    Z,HTM_FrmChkInBox	;Yes (check if within dimensions of it)
;
;  Check if on an icon
;
   JP    HTM_FormsChkIcon
;=========================================================================
;Name     HTM_FindFormItem
;Purpose  To search for a Forms command clicked on (if any)
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_Col = current column in stream window
;         HTML_Row = current row in stream window
;Out Regs A = Item number (HTML_P_EODB=Not found, 0=found item)
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_FrmsDBIx = address of the DB entry found. if not found then
;                         has address of last entry.
;         HTML_FormsObj = forms object with item clicked on.
;Uses     TBD
;Call By  Cursor_select (on cursor select)
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_FindFormItem
   XOR   A
   LD    (HTML_FrmsDBIx2A),A		;Start at 1st DB entry
   CALL  MEM_SwapInCFG			;Swap in Config bank
   LD    HL,CFG_FormsDB
HTM_FindFormItem0
   LD    A,(HL)				;Get db input item type
   CP    HTML_P_EODB			;End of DB?
   RET   Z
   CP    HTML_I_FORM			;Start of form?
   JR    NZ,HTM_FindFormItem0a		;No
HTM_FindFormItem2
   LD    BC,S_HTMLVarSz
   ADD   HL,BC				;Next DB entry address
   JR    HTM_FindFormItem0
;
HTM_FindFormItem0a
   BIT   4,A				;Is item displayed?
   JR    Z,HTM_FindFormItem2		;No, goto next item
;
   LD    (HTML_FrmsDBIx),HL
   CALL  HTM_FormsCpyDBEnt		;copy DB entry to local
   CALL  HTM_ChkItemOn			;check if cursor on item
   LD    HL,(HTML_FrmsDBIx)
   OR    A				;Is cursor on item?
   JR    NZ,HTM_FindFormItem2		;No, Cursor is not on item
   XOR   A				;Flag Found
   RET
;=========================================================================
;Name     HTM_FrmsTogGrpDisp
;Purpose  To toggle the items under a group for display.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_FormsObj = forms object with item clicked on.
;         HTML_FrmsDBLocn = address of clicked item.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_FrmsTogGrpDisp
;
;  Toggle group select for other items in group
;
   CALL  Mem_SwapinCFG
   LD    A,(HTML_FrmsID)	;Get Form ID from click item
   LD    C,A
   LD    A,(HTML_FrmsOptG)
   LD    B,A
   LD    DE,CFG_FormsDB
HTM_FormGrpDisp1
   LD    A,(DE)			;Get entry type
   CP    HTML_P_EODB		;End of DB?
   RET   Z
   LD    HL,S_HTMLVarID
   ADD   HL,DE
   LD    A,(HL)			;Get forms ID Number
   CP    C			;Item for this form
   JR    NZ,HTM_FormGrpDisp2	;No
   LD    HL,S_HTMLVarOpt
   ADD   HL,DE
   LD    A,(HL)			;Get Grouping ID
   CP    B			;Part of same group
   JR    NZ,HTM_FormGrpDisp2	;No
   LD    HL,S_HTMLVarTyp
   ADD   HL,DE
   LD    A,(HL)
   AND   &0F
   CP    HTML_I_OPTION
   JR    NZ,HTM_FormGrpDisp2
   LD    A,(HL)
   XOR   B_4
   LD    (HL),A			;Toggle display status
HTM_FormGrpDisp2
   LD    HL,S_HTMLVarSz
   ADD   HL,DE			;next entry
   EX    DE,HL
   JR    HTM_FormGrpDisp1
;=========================================================================
;Name     HTM_FrmsClrSel
;Purpose  To clear the items active under a select
;In Regs  A = ??
;         B = Select ID
;         C = Forms ID
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_FrmsClrSel
   LD    DE,CFG_FormsDB
HTM_FrmsClrSel1
   LD    A,(DE)			;Get entry type
   CP    HTML_P_EODB		;End of DB?
   RET   Z			;Yes (Refresh form display)
   LD    HL,S_HTMLVarID
   ADD   HL,DE
   LD    A,(HL)			;Get forms ID Number
   CP    C			;ID matches current form?
   JR    NZ,HTM_FrmsClrSel2	;No
   LD    HL,S_HTMLVarSID
   ADD   HL,DE
   LD    A,(HL)			;Get Select ID
   CP    B			;Matching select ID
   JR    NZ,HTM_FrmsClrSel2	;No
   LD    HL,S_HTMLVarTyp
   ADD   HL,DE
   LD    A,(HL)
   AND   HTML_I_FORM
   CP    HTML_I_OPTION		;option forms item?
   JR    NZ,HTM_FrmsClrSel2	;Yes
   LD    HL,S_HTMLVarVal
   ADD   HL,DE
   RES   6,(HL)			;deselect option in select
HTM_FrmsClrSel2
   LD    HL,S_HTMLVarSz
   ADD   HL,DE			;next entry
   EX    HL,DE
   JR    HTM_FrmsClrSel1
;=========================================================================
;Name     HTM_FrmsReset
;Purpose  To reset the forms items to their initial state.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_FormsObj = forms object with item clicked on.
;         HTML_FrmsDBLocn = address of clicked item.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     All registers
;Call By  TBD
;Calls    TBD
;Notes    1. Assume the config bank of memory is in.
;=========================================================================
HTM_FrmsReset
   LD    HL,HTML_FormsFlag
   SET   0,(HL)				;Flag form is to be refreshed
   LD    A,(HTML_FrmsID)		;Get Form ID from click search
   LD    C,A
   LD    DE,CFG_FormsDB
HTM_FrmReset1
   LD    A,(DE)				;Get entry type
   CP    HTML_P_EODB			;End of DB?
   RET   Z				;Yes (Refresh form display)
   LD    HL,S_HTMLVarID
   ADD   HL,DE
   LD    A,(HL)				;Get forms ID Number
   CP    C				;ID matches current form?
   JR    NZ,HTM_FrmReset2		;No
   LD    HL,S_HTMLVARIni
   ADD   HL,DE
   LD    A,(HL)				;Get initial value
   LD    HL,S_HTMLVarVal
   ADD   HL,DE
   LD    (HL),A				;reset Db item state.
   LD    HL,S_HTMLVarLoc
   ADD   HL,DE
   LD    A,(HL)
   LD    B,A
   INC   HL
   OR    (HL)				;has text address?
   JR    Z,HTM_FrmReset2		;No
   LD    H,(HL)
   LD    L,B
   LD    (HL),0				;Clear text with EOL
HTM_FrmReset2
   LD    HL,S_HTMLVarSz
   ADD   HL,DE				;next entry
   EX    HL,DE
   JR    HTM_FrmReset1
;=========================================================================
;Name     HTM_FormSubmitData
;Purpose  To submit the active selected fields to the build URL process
;         and then issue a send URL request.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_FrmsDBIx = address of db entry for SUBMIT option.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars AMSDOS_BUFFER = corrupted
;Uses     TBD
;Call By  TBD
;Calls    UTL_strccpy
;         Mem_SwapInHTML
;         Mem_SwapInCFG
;         ERoutine_Add2URL
;         ERoutine_SendURL
;Notes    1. The Submit button may also have a script to execute before
;            the URL is built and/or sent
;=========================================================================
HTM_FormSubmitData
;
;  Go back to start of form's fields.
;
   LD    HL,(HTML_FrmsDBIx)
   LD    BC,S_HTMLVarSz
HTM_FrmSubmitDat1
   LD    A,(HL)
   AND   HTML_I_FORM
   CP    HTML_I_FORM
   JR    Z,HTM_FrmSubmitDat2
   OR    A
   SBC   HL,BC
   JR    HTM_FrmSubmitDat1
;
HTM_FrmSubmitDat2
   LD    BC,S_HTMLVarSz
   LD    DE,AMSDOS_BUFFER
   LDIR					;Copy down DB entry
;
   LD    B,1				;create initial URL
   LD    C,DBLEQ_CHR			;Text EOL for parameter
   LD    A,(AMSDOS_BUFFER+S_HTMLVarTyp)
   AND   HTML_I_FORM
   CP    HTML_I_FORM			;Form marker?
   JR    Z,HTM_FrmSubmitDat2a		;Yes
   CP    HTML_I_SELECT
   JR    NZ,HTM_FrmSubmitDat2b
   LD    B,4				;Add parameter to URL
   LD    C,0				;Text EOL
   JR    HTM_FrmSubmitDat2a
;
HTM_FrmSubmitDat2b
   LD    A,(AMSDOS_BUFFER+S_HTMLVarVal)
   BIT   6,A				;Active item in form?
   JR    Z,HTM_FrmSubmitDat3		;No
   LD    B,3				;Add parameter to URL
   LD    C,0				;Text EOL
;
HTM_FrmSubmitDat2a
   LD    (HTML_FrmsDBIx2A),HL		;save next DB address
   PUSH  BC
   LD    A,(AMSDOS_BUFFER+S_HTMLVarAdr+2)
   CALL  Mem_SwapInHTML
   LD    HL,(AMSDOS_BUFFER+S_HTMLVarAdr)
   LD    DE,(HTML_CmdTemp)
   CALL  HTM_CpyCmd			;Download HTML command
   POP   BC
;
   LD    HL,AMSDOS_BUFFER+S_HTMLVarLoc
   LD    E,(HL)				;Get text address
   INC   HL
   LD    A,(HL)
   OR    E				;No text address?
   JR    Z,HTM_FrmSubmitDat2c		;Yes
   LD    H,(HL)
   LD    L,E
   CALL  Mem_SwapInCFG
   LD    DE,AMSDOS_BUFFER+S_HTMLVarSz
   CALL  UTL_strccpy			;copy text (if any)
   XOR   A
   LD    (DE),A				;Insert EOT
;
HTM_FrmSubmitDat2c
   LD    A,B
   LD    HL,AMSDOS_BUFFER		;Address of forms DB entry
   LD    DE,AMSDOS_BUFFER+S_HTMLVarSz	;Address of text
   LD    BC,(HTML_CmdTemp)		;Address of HTML cmd
   CALL  ERoutine_Add2URL		;Call external Add 2 URL
;
   CALL  Mem_SwapInCFG
   LD    HL,(HTML_FrmsDBIx2A)
;
HTM_FrmSubmitDat3
   LD    A,(HL)
   AND   HTML_I_FORM
   CP    HTML_I_FORM
   JR    NZ,HTM_FrmSubmitDat2
;
   JP    ERoutine_SendURL		;Call external issue HTTP URL
;=========================================================================
;Name     HTM_ActionFormItem
;Purpose  To action the Forms input command just clicked on.
;In Regs  A = Item number (0=Do nothing, 1-255 = forms DB item number)
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_FormsObj = forms object with item clicked on.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_ActionFormItem
   CALL  MEM_SwapInCFG			;Swap in Config bank
;
   LD    A,(HTML_FrmsTyp)
   AND   HTML_I_FORM
;
   CP    HTML_I_Button			;<Button> type
   JR    Z,HTM_FrmDoAction		;Execute action for the button.
;
   CP    HTML_I_Submit			;SUBMIT botton from <INPUT>
   JP    Z,HTM_FormSubmitData		;Execute action for the button.
;
   CP    HTML_I_Reset			;RESET botton from <INPUT>
   JP    Z,HTM_FrmReset			;Execute action for the button.
;
   CP    HTML_I_Radio
   JP    Z,HTM_FrmToggleState		;Toggle state Up/Down (only one)
;
   CP    HTML_I_OPTION
   JP    Z,HTM_FrmToggleState		;Toggle option state
;
   CP    HTML_I_Chkbox
   JP    Z,HTM_FrmToggleState		;Toggle state Up/Down (many)
;
   CP    HTML_I_OPTGRP
   JP    Z,HTM_FormGrpSel		;Display/Hide grouped options.
;
   LD    HL,KB_EchoFlag
   CP    HTML_I_Text
   JP    Z,HTM_FormTextReq		;Request text (unhidden, 1-line)
;
   CP    HTML_I_Password
   JP    Z,HTM_FormTextReq		;Request text (hidden by '*')
;
   CP    HTML_I_TXTBOX
   JP    Z,HTM_FormTextReqBx		;Request text (unhidden, many lines)
   RET
;
HTM_FrmDoAction
;
; Perform action when custom button clicked on, normally this would be a script
; like Java.
;
   LD    DE,(HTML_CmdTemp)
   LD    HL,(HTML_FormsAddr)
   LD    A,(HTML_FormsAddr+2)
   CALL  Mem_SwapInHTML
   CALL  HTM_CpyCmd			;KB_Buffer has HTML command
   CALL  MEM_SwapInCFG			;Swap in Config bank
   LD    HL,(HTML_CmdTemp)
   LD    (HTML_ParamLoc),HL
HTM_FrmDoAct0
   CALL  HTM_GetCmdParam		;Locate next parameter
   LD    A,(HL)
   CP    EHTML_CHR			;no more parameters?
   RET   Z				;No
   PUSH  HL
   CALL  HTM_FindParmLbl		;Determine the parameter nametype
   POP   HL
   LD    A,HTM_P_type
   CP    B				;Is parameter 'type' ?
   JR    NZ,HTM_FrmDoAct0		;no
;
   CALL  HTM_GetCmdParmVal		;HL = address of parameter value
   CALL  HTM_FindParmLbl		;determine type of input.
   LD    A,B
   SUB   HTM_P_Chkbox			;adjust to be index from 0
;
   CP    HTML_I_Reset			;"RESET"
   JR    Z,HTM_FrmReset
;
   CP    HTML_I_Submit			;"SUBMIT"
   JP    Z,HTM_FormSubmitData		;Execute action for the button.
;
;  Type is neither reset or submit, so must be custom which means a
;  script is expected to be run.
;
   RET
;
HTM_FrmReset
;
;  A form RESET will reset all toggles items to the default setting and
;  any previous text input is cleared.
;
   CALL  HTM_FrmsReset			;reset all fields.
   JR    HTM_ActnFrmEx
;
HTM_FormTextReq
;
;  Request a single line of text
;
   RES   7,(HL)				;Flag single line input
   CP    HTML_I_Password
   JR    NZ,HTM_FormTextReq1
   SET   0,(HL)				;Flag hidden text output
   JR    HTM_FormTextReq1
;
HTM_FormTextReqBx
;
;  Request a mutliple number of lines of text
;
   SET   7,(HL)				;Flag mult-line input
HTM_FormTextReq1
   CALL  CPCB_RemoveCursor
   LD    HL,(HTML_FrmsERow)		;L = CPC screen row
   LD    DE,(HTML_FrmsECol)
   LD    H,E				;H = CPC screen column
   INC   H				;adjust H/L from base 0 to base 1
   INC   L
   CALL  CPCB_AdjustRow
   LD    (KB_MaxInpRC),HL		;setup max screen row/column
   CALL  Mem_SwapInCFG
   LD    A,(Stream_WebPage)
   LD    B,A
   LD    C,0				;assume empty line--- NEED TO FIX
   LD    HL,(HTML_FrmsSRow)		;L = CPC screen row
   LD    DE,(HTML_FrmsSCol)
   LD    H,E				;H = CPC screen column
   INC   H				;adjust H/L from base 0 to base 1
   INC   L
   CALL  CPCB_AdjustRow
   LD    DE,(HTML_FrmsTxtLoc)		;Location of text buffer
   CALL  CPCB_GetWinInput		;Use URL line for input
   JR    HTM_ActnFrmEx			;force cursor to refresh
;
HTM_FrmToggleState
;
;  The toggle state option for grouped items is to turn the display bit
;  off for all items under that group. Where the item is not grouped the
;  active status is toggled.
;
   CALL  MEM_SwapInCFG			;Swap in Config bank
   LD    DE,(HTML_FrmsDBIx)
   LD    HL,S_HTMLVarSID
   ADD   HL,DE
   LD    B,(HL)				;Get select ID
   LD    HL,HTML_FrmsID
   ADD   HL,DE
   LD    C,(HL)				;Get forms ID
   LD    HL,S_HTMLVARIni
   ADD   HL,DE
   BIT   3,(HL)				;Multi-selectable item?
   CALL  Z,HTM_FrmsClrSel		;No (reset all to inactive)
   LD    DE,(HTML_FrmsDBIx)
   LD    HL,S_HTMLVarVal
   ADD   HL,DE
   LD    A,(HL)
   XOR   B_6				;Toggle Active state (bit 6)
   LD    (HL),A
   JR    HTM_ActnFrmEx			;No
;
HTM_FormGrpSel
   LD    DE,(HTML_FrmsDBIx)
   LD    HL,S_HTMLVarVal
   ADD   HL,DE
   LD    A,(HL)
   XOR   B_6+B_0			;Toggle (bit 0) group active state
   LD    (HL),A
   CALL  HTM_FrmsTogGrpDisp		;Toggle display bit in group items.
;
HTM_ActnFrmEx
   CALL  Mem_SwapInMain
   JP    UTL_ScreenRefresh
;=========================================================================
;Name     HTM_FormGetTxtAddr
;Purpose  To calculate the address for textural input.
;In Regs  A = Forms item type
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_FrmsMalloc = current address of free memory for text.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = address of text to be allocated.
;         IX = ??
;         IY = ??
;Out Vars HTML_FrmsMalloc = updated to next free memory space.
;         HTML_FrmsTxtLoc = address of text allocation.
;         HTML_FrmsIniVal = bit 6 set
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_FormGetTxtAddr
   LD    BC,HTML_I_LineSz		;sizeof 1 line text
   CP    HTML_I_TXTBOX
   JR    NZ,HTM_FormGetTxtAdr1
   LD    BC,HTML_I_TxtBoxSz		;sizeof multi-line text
HTM_FormGetTxtAdr1
   LD    HL,(HTML_FrmsMalloc)
   LD    (HTML_FrmsTxtLoc),HL
   LD    E,L
   LD    D,H
   ADD   HL,BC
   LD    (HTML_FrmsMalloc),HL
   EX    DE,HL
HTM_FormGetTxtAdr2
   LD    (HL),0
   INC   HL
   DEC   BC
   LD    A,B
   OR    C
   DJNZ  HTM_FormGetTxtAdr2
   LD    HL,HTML_FrmsIniVal
   SET   6,(HL)				;Flag always active.
   LD    HL,HTML_FrmsValue
   SET   6,(HL)				;Flag always active.
   RET
;=========================================================================
;Name     HTM_FormsInitDB
;Purpose  To initialise the local copy of the DB entry.
;In Regs  A = ??
;         B = Initial value for form DB item.
;         C = Forms Input type (bit 0 to 3)
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_CmdStart = address of HTML command
;         HTML_Row = current web-page render row
;         HTML_Col = current web-page render column
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars Forms DB in HTML vars updated
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_FormsInitDB
   LD    HL,HTML_FrmsIniVal
   LD    (HL),B				;initial value (non-input)
   LD    HL,HTML_FrmsValue
   LD    (HL),B				;Setup init value for reset
;
   LD    A,(HTML_FrmsIDNum)
   LD    (HTML_FrmsID),A
;
   LD    A,(HTML_FrmsTyp)
   AND   &F0
   OR    C				;mask in forms type (0-15)
   LD    (HTML_FrmsTyp),A
;
   LD    A,C
   AND   HTML_I_FORM
   CP    HTML_I_TXTBOX
   CALL  Z,HTM_FormGetTxtAddr		;Allocate memory for text box
;
   CP    HTML_I_Text
   CALL  Z,HTM_FormGetTxtAddr		;Allocate memory for text line
;
   CP    HTML_I_Password
   CALL  Z,HTM_FormGetTxtAddr		;Allocate memory for password
;
   LD    HL,HTML_CmdStart
   LD    DE,HTML_FormsAddr
   LD    BC,3
   LDIR					;Setup where HTML command is.
;
   RET
;=========================================================================
;Name     HTM_FormItem2DB
;Purpose  To add a forms input item to the DB of forms inputs.
;In Regs  A = ??
;         B = ??
;         C = Db item type
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_CmdStart = address of HTML command
;         HTML_FrmsNRows = size of object from 0
;         HTML_FrmsCols = size of object from 0
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars Forms DB in the config bank contains the new item.
;         HTML_FrmsDBLocn = address of saved forms db entry
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. There is no check for the number of items to be in the DB, so
;            it is possible to exceed the allocated memory.
;=========================================================================
HTM_FormItem2DB
   CALL  HTM_FormsInitDB
;
   CALL  HTM_FormCalcDBEnd		;Goto end of DB (bank swaped in)
   LD    (HTML_FrmsDBLocn),HL		;Save Db entry address
   EX    DE,HL
   LD    HL,HTML_FormsObj
   LD    BC,S_HTMLVarSz
   LDIR					;add forms item to DB
;
   LD    A,HTML_P_EODB
   LD    (DE),A				;flag end of DB
   JP    MEM_SwapInMain
