;=========================================================================
;
; Application Routines (routines to support HTML frames)
; Copyright (c) 2010 Ray Palmer
;
; Notes
; 1.	A web-page has a maximum of 65535 characters for columns and
;     65535 rows.
;
; 2.  The display of a frame with columns seems to cause major issue,
;     so for now that option is left as is til more fresh eyes look back
;     at the problem to see if a solution exists (16Dec2012).
;
;=========================================================================
;Name     HTM_FrmeIStrt
;Purpose  To setup the start position for frame page render
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_BR_xxx = start column for <BR> command
;         HTML_Col = current rendering column on web-page canvas
;         HTML_Row = current rendering row on web-page canvas
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    1. See commentry at strat of the frames command source to
;            understand why rendering start at (0,0)
;=========================================================================
HTM_FrmeIStrt
   LD    A,(HTML_RenderState)
   RES   7,A			;Flag start of rendering.
   OR    1			;Flag started of rendering.
   LD    (HTML_RenderState),A
;
;  Reset text new line control variables
;
   XOR   A
   LD    HL,HTML_BR
   LD    BC,HTML_E_VrsCurP-HTML_BR
   CALL  HTM_ResetMem
;
;  Reset start row/column
;
   LD    HL,0
   LD    (HTML_MaxCol),HL
   LD    (HTML_Row),HL		;setup start row/column for render
   LD    (HTML_Col),HL
;
   LD    (HTML_quoteLevel),A	;Start of with double quotes
   LD    (HTML_FontEffects),A	;No font effects
   LD    (HTML_FormsFlag),A	;Clear forms control for <BR>

   LD    A,(HTML_DispStatus)
   AND   &0F
   LD    (HTML_DispStatus),A	;assume content is displayable
;
   LD    A,-5
   LD    (HTML_ListIdnt),A	;Setup start of ident for lists
;
;  Reset Style handling variables (re-write to just download setting)
;
   XOR   A
   LD    HL,HTML_StyleIdx
   LD    BC,HTML_StyleVrEnd-HTML_StyleIdx
   CALL  HTM_ResetMem
;
   LD    A,&80
   LD    (HTML_StyleFlgs),A	;Invalid style setting
   LD    A,THINBRDR_ICNST
   LD    (HTML_StyleBordw),A
   LD    A,HTM_P_btsolid
   LD    (HTML_StyleBord),A
   LD    HL,FONT_System
   CALL  STY_ChngeFont1
;
   JP    HTM_SetupStyleFormat
;=========================================================================
;Name     HTM_FrmeCalcAddr
;Purpose  To calculate the address of a frames DB entry
;In Regs  A = Frame ID (starts from 0)
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  FramesDBLoc = location of frames Db in main memory
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = address of Db entry
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    Mem_SwapInFramesDB
;Notes    1. The first frame in the DB is reserved for the menu line.
;         2. The second entry of the frames Database would contain the
;            base HTML file where frames are defined. Upon loading of this
;            file the frames db is loaded with entires and then the first
;            entry is flagged as not available since it's reference would
;            be to re-load the frames HTML every time.
;         3. This routines does not check if the index is valid.
;=========================================================================
HTM_FrmeCalcAddr
   LD    B,A
   LD    HL,(FramesDBLoc)
   LD    DE,S_HTMLFramSz
HTM_FrmeCalcAdr1
   ADD   HL,DE
   DJNZ  HTM_FrmeCalcAdr1	;Try next frame DB entry
   JP    Mem_SwapInFramesDB	;swap in frames DB memory.
;=========================================================================
;Name     HTM_FrameTaken
;Purpose  To flag a frame as no longer available (i.e not visible)
;In Regs  A = Frame ID number
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    HTM_FrmeCalcAddr
;Notes    None.
;=========================================================================
HTM_FrameTaken
   CALL  HTM_FrmeCalcAddr
   SET   3,(HL)
   RET
;=========================================================================
;Name     HTM_GetFrame
;Purpose  To get a frame from the frames DB based on a frame ID
;In Regs  A = Frame ID Number
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  ConfigBank
;Out Regs A = Frame ID/Status
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars Window_FrameV = frame object
;Uses     TBD
;Call By  TBD
;Calls    HTM_FrmeCalcAddr
;Notes    TBD
;=========================================================================
HTM_GetFrame
   CALL  HTM_FrmeCalcAddr
   LD    DE,Window_FrameV
   LD    BC,S_HTMLFramSz
   LDIR
   LD    A,(Window_FrmeID)		;return status
   RET
;=========================================================================
;Name     HTM_GetFrameH
;Purpose  To get location of the HTML for the frame.
;In Regs  A = Frame ID
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  None.
;Out Regs A = HTML Bank index number
;         BC = ??
;         DE = ??
;         HL = Address of HTML
;         IX = ??
;         IY = ??
;Out Vars None
;Uses     TBD
;Call By  TBD
;Calls    HTM_FrmeCalcAddr
;Notes    TBD
;=========================================================================
HTM_GetFrameH
   CALL  HTM_FrmeCalcAddr
   LD    DE,S_HTMLFramAd
   ADD   HL,DE
   CALL  UTL_GetVarWord
   LD    A,(HL)
   EX    DE,HL
   RET
;=========================================================================
;Name     HTM_GetArrayVal
;Purpose  To get the nth array item and return flag if end of array
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = Address of array of values to access
;         IX = ??
;         IY = ??
;In Vars  None
;Out Regs A = Z flag if value is end of array value else NZ
;         BC = ??
;         DE = array value
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars None
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
HTM_GetArrayVal
   CALL  UTL_GetVarWord
   LD    A,E
   OR    D					;End of Array?
   RET
;=========================================================================
;Name     HTM_SetArrayVal
;Purpose  To set the nth array item.
;In Regs  A = ??
;         BC = array value to save
;         DE = ??
;         HL = Address of array of values to access
;         IX = ??
;         IY = ??
;In Vars  None
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = incremented to next array item
;         IX = ??
;         IY = ??
;Out Vars None
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
HTM_SetArrayVal
   RES   7,B				;Flag item is chars value
   RES   6,B
HTM_SetArrVal2
   LD    (HL),C
   INC   HL
   LD    (HL),B
   INC   HL
   RET
;=========================================================================
;Name     HTM_ResetRC
;Purpose  To initialize the rows/cols array for cases where there is no
;         frames have been designed.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = Address of array of values.
;         IX = ??
;         IY = ??
;In Vars  None
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars None
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    1. The number of frames is limited 64
;=========================================================================
HTM_ResetRC
   LD    BC,HTML_MaxWebPgeRC
   INC   HL			;skip first entry (Frame 0)
   INC   HL
   JP    HTM_SetArrVal2		;setup frame 1 with max value.
;=========================================================================
;Name     HTM_GetFrmesRows
;Purpose  To get the frameset row values
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_FrmeSets = bit 8 define if 1st frameset
;         HTML_ParamLoc = address of HTML parameters list
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_FrmeSets = state of values got
;                         Bit 6 = values got for rows from 1st frameset
;                         Bit 4 = value got for row defs
;         HTML_FrameTmp = location of data value saved to
;         HTML_FrmeSetR = array of row values
;         HTML_FrmeSetI = 0 (start index to array)
;         HTML_FrmeRCtr = number of row entries
;Uses     TBD
;Call By  TBD
;Calls    HTM_GetFrmesVals
;         HTM_FrameConv
;Notes    TBD
;=========================================================================
HTM_GetFrmesRows
;
;  Initiallise array
;
   LD    HL,(HTML_FrmeSetR)
   CALL  HTM_ResetRC
;
   XOR   A
   LD    (HTML_FrmeSetI),A
   INC   A
   LD    (HTML_FrmeRCtr),A
   LD    HL,HTML_FrmeSets
   BIT   7,(HL)				;Frames already in use?
   JR    NZ,HTM_GetFrmesRows1		;Yes
   SET   6,(HL)				;Flag def rows on cols or rows only
HTM_GetFrmesRows1
   SET   4,(HL)				;flag rows parameter passed
   SET   7,(HL)				;Flag now using frames
;
   LD    HL,(HTML_FrmeSetR)		;Where to save values
   CALL  HTM_GetFrmesVals		;Get list of values
;
   LD    A,(HTML_FrmeICtr)
   OR    A					;empty list?
   RET   Z					;Yes
   LD    (HTML_FrmeRCtr),A
;
   LD    HL,MAXWEBPAGECOLS
   LD    (HTML_FrameTmp2),HL		;Setup max dimensions
   LD    DE,(HTML_FrmeSetR)
   CALL  HTM_FrameConv			;Convert values to #characters
;
   LD    HL,(HTML_FrmeSetR)
   JR    HTM_FrameLocs			;Convert array to locations
;=========================================================================
;Name     HTM_GetFrmesCols
;Purpose  To get the frameset column values
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_FrmeSets = bit 8 define if 1st frameset
;         HTML_ParamLoc = address of HTML parameters list
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_FrmeSets = state of values got
;                         Bit 2 = values got for cols from 1st frameset
;                         Bit 0 = value got for column defs
;         HTML_FrameTmp = location of data value saved to
;         HTML_FrmeSetC = array of column values
;         HTML_FrmeSetIC = 0 (start index to array)
;         HTML_FrmeCCtr = number of Column entries
;Uses     TBD
;Call By  TBD
;Calls    HTM_GetFrmesVals
;         HTM_FrameConv
;Notes    TBD
;=========================================================================
HTM_GetFrmesCols
;
;  Initiallise array
;
   LD    HL,(HTML_FrmeSetC)
   CALL  HTM_ResetRC
;
   XOR   A
   LD    (HTML_FrmeSetIC),A
   INC   A
   LD    (HTML_FrmeCCtr),A
   LD    HL,HTML_FrmeSets
   BIT   7,(HL)				;Frames already in use?
   JR    NZ,HTM_GetFrmesCols1		;Yes
   SET   2,(HL)				;Flag def cols on rows or cols only
HTM_GetFrmesCols1
   SET   0,(HL)				;flag cols parameter found
   SET   7,(HL)				;Flag now using frames
;
   LD    HL,(HTML_FrmeSetC)		;Where to save values
   CALL  HTM_GetFrmesVals		;Get list of values
;
   LD    A,(HTML_FrmeICtr)
   OR    A				;empty list?
   RET   Z				;Yes
   LD    (HTML_FrmeCCtr),A
;
   LD    HL,MAXWEBPAGECOLS
   LD    (HTML_FrameTmp2),HL		;Setup max dimensions
   LD    DE,(HTML_FrmeSetC)
   CALL  HTM_FrameConv			;Convert values to #characters
;
   LD    HL,(HTML_FrmeSetC)
;=========================================================================
;Name     HTM_FrameLocs
;Purpose  To convert the array of frame sizes to locations on the webpage.
;         This amount to the following -
;          Array of sizes     = 200,100,300,500,...
;          is converted to
;          Array of locations = 0,200,300,600,1100,...
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = Array of items
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars Array of value updated.
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    1. All location on the web page start from position 1 and not
;            zero.
;=========================================================================
HTM_FrameLocs
   LD     DE,WEBPAGE_BASELOC
HTM_FrameLocs1
   LD     C,(HL)			;Get Current size
   LD     (HL),E			;Setup start location
   INC    HL
   LD     B,(HL)
   LD     (HL),D
   INC    HL
   EX     HL,DE			;Save array address
   ADD    HL,BC			;Calc new start location for next entry
   EX     DE,HL			;get array address back
   LD     A,B			;end of array items?
   OR     C
   JR     NZ,HTM_FrameLocs1	;No
   LD     DE,HTML_MaxWebPgeRC
   JP     UTL_SetVarWord
;=========================================================================
;Name     HTM_FrameConv
;Purpose  To get the convert the values to character values.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_FrameTmp = location of data value saved to
;         HTML_FrameTmp2 = max dimension
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    HTM_FrmeCnvPix = convert pixel values to chars
;         HTM_FrmeCnvNw = calc remains chars in dimension
;         HTM_FrmeCnvPer = convert percentages to chars
;         HTM_FrmeCnvPor = convert portions to chars
;Notes    1. This routine converts the array of values to characters
;            by multiple loops to arrive at the require conversion.
;            a. 1st convert pixel values to characters
;            b. 2nd convert % values of remaining width to characters
;            c. 3rd convert * values of remaining width to characters
;=========================================================================
HTM_FrameConv
   LD    HL,(HTML_FrameTmp)		;Get location of value to convert
   CALL  HTM_FrmeCnvPix			;Convert pixels to chars
   LD    HL,(HTML_FrameTmp)
;
   LD    DE,(HTML_FrameTmp2)
   CALL  HTM_FrmeCnvNw			;calc remaining space (chars)
   LD    (HTML_FrameTmp2),DE
;
   LD    HL,(HTML_FrameTmp)		;Get location of value to convert
   CALL  HTM_FrmeCnvPer			;Convert percentages to chars
   LD    HL,(HTML_FrameTmp)
;
   LD    DE,(HTML_FrameTmp2)
   CALL  HTM_FrmeCnvNw			;calc remaining space (Chars)
   LD    (HTML_FrameTmp2),DE
;
   LD    HL,(HTML_FrameTmp)		;Get location of value to convert
   JP    HTM_FrmeCnvPor			;convert portions to chars
;=========================================================================
;Name     HTM_FrmeCnvNw
;Purpose  To calculate the number of positions left on the web-page.
;In Regs  A = ??
;         BC = ??
;         DE = Current remaining chars
;         HL = Address of array of values to access
;         IX = ??
;         IY = ??
;In Vars  None directly
;Out Regs A = ??
;         BC = ??
;         DE = remaining chars left
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars None directly
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    TBD
;=========================================================================
HTM_FrmeCnvNw
   LD    C,(HL)
   INC   HL
   LD    B,(HL)		;BC = number chars allocated
   OR    B			;space allocation zero?
   RET   Z			;Yes (end of list)
   INC   HL
   RES   7,B
   RES   6,B
   EX    DE,HL
   SBC   HL,BC		;Subtract from current remaining
   EX    DE,HL
   JR    HTM_FrmeCnvNw
;=========================================================================
;Name     HTM_FrmeCnvPix
;Purpose  To convert the pixel values to the characters they fill.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = Address of HTML_FrameTmp
;         IX = ??
;         IY = ??
;In Vars  HTML_FrameTmp = location of data value saved to
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = corrupted.
;         IX = ??
;         IY = ??
;Out Vars HTML_FrameTmp = location of data value saved to
;Uses     TBD
;Call By  TBD
;Calls    Mth_Divide
;Notes    1. The array of values are widths and not actual web-page
;            locations.
;=========================================================================
HTM_FrmeCnvPix
   CALL  HTM_GetArrayVal
   RET   Z					;end of array of values
   LD    A,D
   AND   &C0
   CP    &C0				;Pixel value to convert?
   JR    NZ,HTM_FrmeCnvPix		;No
   DEC   HL
   DEC   HL
   PUSH  HL
   RES   7,D
   RES   6,D
   EX    DE,HL
   LD    DE,8				;Number of pixesl to a character
   CALL  Mth_Divide			;BC = number of chars
   POP   HL
   CALL  HTM_SetArrayVal
   JR    HTM_FrmeCnvPix			;Check next entry
;=========================================================================
;Name     HTM_FrmeCnvPer
;Purpose  To convert the percentage values to the remaining chars left.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = Address of array of items
;         IX = ??
;         IY = ??
;In Vars  HTML_FrameTmp = location of data value saved to
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = corrupted.
;         IX = ??
;         IY = ??
;Out Vars HTML_FrameTmp = location of data value saved to
;Uses     TBD
;Call By  TBD
;Calls    Mth_Divide
;Notes    1. The array of values are widths and not actual web-page
;            locations.
;=========================================================================
HTM_FrmeCnvPer
   CALL  HTM_GetArrayVal
   RET   Z					;end of array of values
   LD    A,D
   CP    &C0
   CP    &80				;Is entry percentage value?
   JR    NZ,HTM_FrmeCnvPer		;No
   DEC   HL
   DEC   HL
   PUSH  HL
   LD    HL,100
   CALL  Mth_Divide			;Calc 100/value
   LD    E,C
   LD    D,B
   LD    HL,(HTML_FrameTmp2)		;Get current num chars left
   CALL  Mth_Divide			;calculate number of chars
   POP   HL
   CALL  HTM_SetArrayVal
   JR    HTM_FrmeCnvPer			;process next entry
;=========================================================================
;Name     HTM_FrmeCnvPor
;Purpose  To convert the poriton values to the remaining chars left.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = Address of HTML_FrameTmp
;         IX = ??
;         IY = ??
;In Vars  HTML_FrameTmp = location of data value saved to
;         HTML_FrameTmp2 = current remaining chars left
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = corrupted.
;         IX = ??
;         IY = ??
;Out Vars HTML_FrameTmp = location of data value saved to
;Uses     TBD
;Call By  TBD
;Calls    Mth_Divide
;         Mth_Multiply
;Notes    1. The array of values are widths and not actual web-page
;            locations.
;=========================================================================
HTM_FrmeCnvPor
   LD    DE,0
;
;  Tally all the portion values
;
HTM_FrmeCnvPor0b
   LD    C,(HL)
   INC   HL
   LD    B,(HL)				;DE = percentage value
   INC   HL
   LD    A,B
   OR    C					;End of array?
   JR    Z,HTM_FrmeCnvPor0		;Yes
   LD    A,B
   CP    &C0
   CP    &40				;Portions value?
   JR    NZ,HTM_FrmeCnvPor0b		;No
   RES   7,B
   RES   6,B
   EX    DE,HL
   ADD   HL,BC				;Tally portions
   EX    DE,HL
   JR    HTM_FrmeCnvPor0b
;
HTM_FrmeCnvPor0
;
;  Calculate the number of characters per unit portion
;  Dont expect the result to be zero.
;
   LD    HL,(HTML_FrameTmp2)		;Get current num chars left
   CALL  Mth_Divide			;Calc chars per unit porition
   LD    (HTML_FrameTmp3),BC		;Save chars per unit portion
;
HTM_FrmeCnvPor1
   CALL  HTM_GetArrayVal
   RET   Z					;end of array of values
   LD    A,D
   CP    &C0
   CP    &40				;portions value?
   JR    NZ,HTM_FrmeCnvPor1		;No
   DEC   HL
   DEC   HL
   PUSH  HL
   LD    HL,(HTML_FrameTmp3)		;Get unit portions
   CALL  Mth_Multiply			;HL = num of unit portions
   LD    C,L
   LD    B,H
   POP   HL
   JR    HTM_FrmeCnvPor1
;=========================================================================
;Name     HTM_GetFrmesVals
;Purpose  To get the frameset values into the array of values for
;         frameset rows or columns.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = address of array to save the frameset values.
;         IX = ??
;         IY = ??
;In Vars  HTML_ParamLoc = location of values list.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_FrameTmp = location of data value saved to
;         HTML_ParamLoc = 1st char after list.
;Uses     TBD
;Call By  TBD
;Calls    HTM_GetCmdParmVal
;         UTL_readdec2
;Notes    1. The maximum values for all array items is 16383.
;         2. The upper 2 bits of each array item define the type as
;            follows -
;            a. 01 - '*' Portion values
;            b. 10 - '%' Percentage values
;            c. 11 - ',' Pixels
;            d. 00 -     Characters (specific to CPCB only,default)
;            With characters the maximum theoretical pixels would be
;            16383 * 8 which is 128K.
;            For pixels the conversion to characters will always be to
;            round up to the next character.
;            For percentage/portion values the maximum is 255, there is
;            no checks for percentages above 99.
;=========================================================================
HTM_GetFrmesVals
   LD    (HTML_FrameTmp),HL
   XOR   A
   LD    (HTML_FrmeICtr),A
;
   LD    HL,(HTML_ParamLoc)
   CALL  HTM_GetCmdParmVal		;Get address of parameter value
   EX    DE,HL				;DE = location of 1st value
   LD    A,(DE)
   CP    '"'				;empty list?
   RET   Z					;yes
;
   LD    A,(FramesDBMax)		;Max array elements
   LD    B,A
   LD    C,0				;Index into array
HTM_GetFrmesVls1
   CALL  UTL_readdec2			;HL = converted to value, A = char
;
   LD    A,H
   AND   &3f
   OR    L					;Is value > 0 ?
   JR    NZ,HTM_GetFrmesVls2		;Yes (accept it)
   LD    HL,1				;No (setup Min value)
;
HTM_GetFrmesVls2
   LD    A,(DE)
   RES   7,H
   SET   6,H				;Flag portions value
   CP    '*'				;fraction of remaining width?
   JR    Z,HTM_GetFrmesVls4		;Yes
;
   SET   7,H
   RES   6,H				;Flag percentage value
   CP    '%'				;Percentage of width?
   JR    Z,HTM_GetFrmesVls4		;Yes
;
   SET   7,H				;Assume pixels (Default)
   SET   6,H
   CP    ','				;Pixels width?
   JR    Z,HTM_GetFrmesVls4		;Yes
;
   RES   7,H				;Flag Characters
   RES   6,H
;
HTM_GetFrmesVls4
;
;  Save value in array
;
   PUSH  DE
   EX    DE,HL				;Put value into DE
   LD    HL,(HTML_FrameTmp)
   CALL  UTL_SetVarWord			;Save value
   LD    (HTML_FrameTmp),HL
   LD    HL,HTML_FrmeICtr
   INC   (HL)				;tally number of values
   POP   DE
;
HTM_GetFrmesVls5
   LD    A,(DE)
   CP    '"'				;End of parameter values?
   JR    Z,HTM_GetFrmesVls7		;Yes
   CP    ','				;parameter values delimiter?
   JR    Z,HTM_GetFrmesVls6		;Yes (skip any leading spaces)
   INC   DE
   JR    HTM_GetFrmesVls5
;
HTM_GetFrmesVls6
   INC   DE
   LD    A,(DE)
   CP    '"'				;End of parameter values?
   JR    Z,HTM_GetFrmesVls7		;Yes
   CP    ' '				;leading space?
   JR    Z,HTM_GetFrmesVls6		;No (check next for space)
   DJNZ  HTM_GetFrmesVls1		;Next array item
;
;  Too many values, goto to end of values list
;
HTM_GetFrmesVls7
   LD    A,(DE)
   INC   DE
   CP    '"'
   JR    NZ,HTM_GetFrmesVls7
   LD    (HTML_ParamLoc),DE		;Setup end address of parameters
   LD    DE,0
   LD    HL,(HTML_FrameTmp)
   CALL  UTL_SetVarWord			;insert end of array
   RET
;=========================================================================
;Name     HTM_GetRowItm
;Purpose  To get the nth array item from the row array.
;In Regs  A = ??
;         BC = ??
;         E = index into array
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = array item value
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars None
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
HTM_GetRowItm
   LD    A,D
   LD    HL,(HTML_FrmeSetR)
   JR    HTM_GetRCItm
;=========================================================================
;Name     HTM_GetColItm
;Purpose  To get the nth array item from the column array.
;In Regs  A = ??
;         BC = ??
;         D = index into array
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = array item value
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars None
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
HTM_GetColItm
   LD    A,E
   LD    HL,(HTML_FrmeSetC)
;
HTM_GetRCItm
   ADD   A,A			;convert to word index
   LD    C,A
   LD    B,0
   ADD   HL,BC			;Calc array address of nth item
   LD    C,(HL)
   INC   HL
   LD    B,(HL)
   RET
;=========================================================================
;Name     HTM_DefRowCol
;Purpose  To setup the frame's object windows dimensions and save the
;         object into the frames database.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_FrmeSetIC = row and column indices
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars None
;Uses     TBD
;Call By  TBD
;Calls    HTM_NewFrame
;         HTM_GetColItm
;Notes    1. Since the calculations for the frames dimensions are based
;            from (0,0) while the size of frames are based from 1 the end
;            coordinates for the bottom right of each is reduced by 1 to
;            account for the difference in bases. So for a frame size of
;            (10,10) would equate to a db entry having (0,0) to (9,9) to
;            represent the frame. The base of zero is also required for
;            content display as well.
;=========================================================================
HTM_DefRowCol
   LD    DE,(HTML_FrmeSetIC)	;E=Col/D=Row indices
   CALL  HTM_GetRowItm
   LD    (Window_SRow),BC	;setup top row
   CALL  HTM_GetColItm
   LD    (Window_SCol),BC	;setup left column
   INC   E
   INC   D
   CALL  HTM_GetRowItm
   DEC   BC			;adjust end for base of 0 (not 1)
   LD    (Window_ERow),BC	;setup bottom row
   CALL  HTM_GetColItm
   DEC   BC			;adjust end for base of 0 (not 1)
   LD    (Window_ECol),BC	;setup right column
;
   LD    HL,Window_FrmeID
   LD    (HL),1			;Flag new frame (no border)
   LD    A,(HTML_FrameBS)
   OR    A				;Dos frame have a scrollable frame?
   JR    Z,HTM_DefRC1		;No
   SET   6,(HL)			;Flag frame has border (ie scrollable)
   SET   4,(HL)			;Flag show border
HTM_DefRC1
   LD    DE,Window_Addr
   LD    HL,HTML_RCurrPos
   LD    BC,3
   LDIR				;Setup location of HTML for loading
;=========================================================================
;Name     HTM_NewFrame
;Purpose  To add a new frame to the frames DB
;In Regs  A = Frame ID (0-15)
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  Window_FrameV = frame object
;Out Regs A = status (255 = added ok else zero)
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars Window_FrameV = update
;Uses     TBD
;Call By  TBD
;Calls    MEM_SwapInCfg
;Notes    TBD
;=========================================================================
HTM_NewFrame
   CALL  Mem_SwapInFramesDB
   LD    HL,(FramesDBLoc)
   LD    A,(FramesDBMax)
   LD    B,A
   XOR   A					;flag entry not added
   LD    DE,S_HTMLFramSz
HTM_NewFrme1
   BIT   0,(HL)				;end of DB?
   JR    Z,HTM_NewFrme2			;Yes
   ADD   HL,DE
   DJNZ  HTM_NewFrme1			;Try next frame DB entry
   RET
HTM_NewFrme2
   EX    DE,HL
   LD    HL,Window_FrameV
   SET   0,(HL)				;Ensure DB entry is taken.
   LD    BC,S_HTMLFramSz
   LDIR					;Insert new frame
   LD    (DE),A				;Add end of DB marker
   DEC   A					;flag entry added
   LD    HL,HTML_FrameFlg
   SET   5,(HL)				;Flag new frame created.
   RET
;=========================================================================
;Name     HTM_DefFrame
;Purpose  To define the frames after all frameset commands has been
;         encountered before a <frame> command.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_FrmeSetS = state of values got
;         HTML_FrmeSetC = array of column values
;         HTML_FrmeSetR = array of row values
;         HTML_FrmeSetI = index into array of row values
;         HTML_FrmeSetIC = index into array of column values
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  html_cmd_frame
;Calls    HTM_RowAndCols via JP
;         HTM_RowsOnCols via JP
;         HTM_ColsOnRows via JP
;         HTM_RowsOnly via JP
;         HTM_ColsOnly via JP
;Notes    TBD
;=========================================================================
HTM_DefFrame
   LD    A,(HTML_FrmeSetS)
   AND   &55				;Mask of frameset controls
;
   CP    &55				;define both at once (i.e matrix)
   JP    Z,HTM_RowAndCols		;yes
;
   CP    &15				;define rows on nth col def
   JP    Z,HTM_RowsOnCols		;Yes
;
   CP    &51				;define cols on the row def
   JP    Z,HTM_ColsOnRows		;Yes
;
   CP    &50				;Only Rows def
   JP    Z,HTM_RowsOnly			;Yes
;
   CP    &05				;Only Columns def
   JP    Z,HTM_ColsOnly			;Yes
   RET
;=========================================================================
;Name     HTM_RowAndCols
;Purpose  To define the frames with both rows and cols defined by the one
;         frameset command.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_FrmeSetC = array of column values
;         HTML_FrmeSetR = array of row values
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars WindowFrame = Object in CPCB.EXE corrupted structure.
;Uses     TBD
;Call By  HTM_DefFrame
;Calls    HTM_DefRowCol
;Notes    1. This routine handles the occurrence of the frameset like
;             <framset cols="100,100,100", rows="200,200,200">
;             </frameset>
;            This will result in a matrix of 3 x 3 areas on the web page
;            each 100 x 200 in size.
;=========================================================================
HTM_RowAndCols
   LD    DE,(HTML_FrmeRCtr)
   LD    A,E
   OR    D				;all frames defined?
   RET   Z				;Yes
   CALL  HTM_DefRowCol
   LD    HL,HTML_FrmeSetIC
   INC   (HL)
   LD    B,(HL)
   LD    A,(HTML_FrmeCCtr)
   CP    B				;At end of columns for this row?
   RET   NZ				;No
   LD    (HL),0			;Yes reset columns index
   LD    HL,HTML_FrmeSetI
   INC   (HL)			;Next row
   LD    B,(HL)
   LD    A,(HTML_FrmeRCtr)
   CP    B				;at end of rows for this column?
   RET   NZ				;No
   LD    DE,0
   LD    (HTML_FrmeRCtr),DE	;Clear counters
   RET
;=========================================================================
;Name     HTM_RowsOnCols
;Purpose  To define the frames with both rows and cols defined by the two
;         frameset commands.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_FrmeSetC = array of column values
;         HTML_FrmeSetR = array of row values
;         HTML_FrmeSetI = Frames DB index
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_FrmeSets = state of values got (bit 7 updated)
;         HTML_FrmeSetI = incremented
;         HTML_FrmeSetIC = incremented
;         WindowFrame = Object in CPCB.EXE corrupted structure.
;Uses     TBD
;Call By  HTM_DefFrame
;Calls    HTM_DefRowCol
;Notes    1. This routine handles the occurrence of the frameset like
;            <framset cols="100,100,100">
;               <frameset rows="200,200,200">
;               </frameset>
;            </frameset>
;            This results in the first column having 3 rows while the next
;            2 have only 1 row.
;=========================================================================
HTM_RowsOnCols
   LD    DE,(HTML_FrmeRCtr)
   LD    A,E
   OR    D				;all frames defined?
   RET   Z				;Yes
   CALL  HTM_DefRowCol		;setup frame
   LD    HL,HTML_FrmeSetI
   INC   (HL)			;next row array item.
   LD    D,(HL)
   LD    A,(HTML_FrmeRCtr)
   CP    D				;at end of rows for this column?
   RET   NZ				;No
   LD    (HL),0			;Yes reset rows index
   LD    HL,(HTML_FrmeSetR)
   CALL  HTM_ResetRC		;reset rows array
   LD    HL,HTML_FrmeCCtr
   INC   (HL)			;next Column array item.
   LD    B,(HL)
   LD    A,(HTML_FrmeCCtr)
   CP    B				;At max columns?
   RET   NZ				;No
   LD    DE,0
   LD    (HTML_FrmeRCtr),DE	;Clear counters
   RET
;=========================================================================
;Name     HTM_ColsOnRows
;Purpose  To define the frames with both rows and cols defined by the two
;         frameset commands.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_FrmeSetC = array of column values
;         HTML_FrmeSetR = array of row values
;         HTML_FrmeSetI = current index into the Rows definition
;         WindowFrame = has current rows defintion where column are to be
;                       defined against
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_FrmeSetI = incremented
;         WindowFrame = corrupted structure.
;Uses     TBD
;Call By  HTM_DefFrame
;Calls    HTM_DefRowCol
;Notes    1. This routine handles the occurrence of the frameset like
;            <framset rows="100,100,100">
;               <frameset cols="200,200,200">
;               </frameset>
;            </frameset>
;            This results in the first row having 3 columns while the next
;            2 have only 1 column.
;=========================================================================
HTM_ColsOnRows
   LD    DE,(HTML_FrmeRCtr)
   LD    A,E
   OR    D				;all frames defined?
   RET   Z				;Yes
   CALL  HTM_DefRowCol		;setup frame
   LD    HL,HTML_FrmeSetIC
   INC   (HL)			;next column array item
   LD    E,(HL)
   LD    A,(HTML_FrmeCCtr)
   CP    E				;at end of columns for this row?
   RET   NZ				;No
   LD    (HL),0			;Yes reset column index
   LD    HL,(HTML_FrmeSetC)
   CALL  HTM_ResetRC		;reset columns array
   LD    HL,HTML_FrmeSetI
   INC   (HL)			;next row array item.
   LD    B,(HL)
   LD    A,(HTML_FrmeRCtr)
   CP    B				;At max rows?
   RET   NZ				;No
   LD    DE,0
   LD    (HTML_FrmeRCtr),DE	;Clear counters
   RET
;=========================================================================
;Name     HTM_RowsOnly
;Purpose  To define the frames with only rows defined by the frameset
;         command.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_FrmeSetC = array of column values
;         HTML_FrmeSetR = array of row values
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars WindowFrame = Object in CPCB.EXE corrupted structure.
;Uses     TBD
;Call By  HTM_DefFrame
;Calls    HTM_DefRowCol
;Notes    1. This routine handles the occurrence of the frameset like
;             <frameset rows="200,200,200">
;             </frameset>
;=========================================================================
HTM_RowsOnly
   LD    DE,(HTML_FrmeRCtr)
   LD    A,E
   OR    D				;all frames defined?
   RET   Z				;Yes
   CALL  HTM_DefRowCol		;setup frame
   LD    HL,HTML_FrmeSetI
   INC   (HL)			;nexr row
   LD    B,(HL)
   LD    A,(HTML_FrmeRCtr)
   CP    B				;At max rows?
   RET   NZ				;No
   LD    DE,0
   LD    (HTML_FrmeRCtr),DE	;Clear counters
   RET
;=========================================================================
;Name     HTM_ColsOnly
;Purpose  To define the frames with only columns defined by the frameset
;         command.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_FrmeSets = state of values got
;         HTML_FrmeSetC = array of column values
;         HTML_FrmeSetR = array of row values
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_FrmeSets = state of values got
;                         bits 7/3 updated both set to 1
;         WindowFrame = Object in CPCB.EXE corrupted structure.
;                       window_SRow = 0
;                       window_ERow = 65535
;Uses     TBD
;Call By  HTM_DefFrame
;Calls    HTM_DefRowCol
;         HTM_RowAndCols3
;Notes    1. This routine handles the occurrence of the frameset like
;             <frameset cols="200,200,200">
;             </frameset>
;=========================================================================
HTM_ColsOnly
   LD    DE,(HTML_FrmeRCtr)
   LD    A,E
   OR    D				;all frames defined?
   RET   Z				;Yes
   CALL  HTM_DefRowCol		;setup frame
   LD    HL,HTML_FrmeSetIC
   INC   (HL)			;next column
   LD    B,(HL)
   LD    A,(HTML_FrmeCCtr)
   CP    B				;At max columns?
   RET   NZ				;No
   LD    DE,0
   LD    (HTML_FrmeRCtr),DE	;Clear counters
   RET
;=========================================================================
;Name     HTM_GetFrameAdr
;Purpose  To setup the web rendering start RAM location.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  Window_FrameV = Current WIndow frame to handle
;           Window_Addr = Start RAM address of frame HTML to render
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_RCurrPos = RAM address for rendering function
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
HTM_GetFrameAdr
   LD    HL,Window_Addr
   LD    DE,HTML_RCurrPos
   LD    BC,3
   LDIR					;Setup new frame to display
   RET
;=========================================================================
;Name     HTM_LoadFrameObj
;Purpose  To load up the Frames Object.
;In Regs  A = Frame DB entry number
;         BC = ??
;         DE = ??
;         HL = address of frames DB entry
;         IX = ??
;         IY = ??
;In Vars  HTML_CmdTemp = address of buffer
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_ParamLoc = corrupted.
;         Frame DB entry updated with objects loaded address.
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    1. The frames DB config ram bank is currently swapped in
;=========================================================================
HTM_LoadFrameObject
   LD    (HTML_FrameID),A
   LD    BC,S_HTMLFramAd
   ADD   HL,BC
   PUSH  HL
   LD    DE,HTML_ObjLocn
   LD    BC,3
   LDIR				;HTML_ObjLocn = location of HTML cmd
   POP   DE
   LD    HL,URL_EndAddr
   LD    BC,3
   LDIR				;Setup address of frames loaded object
;
   CALL  HTM_GetCmd		;Get HTML command
;
   LD    HL,(HTML_CmdTemp)
   LD    (HTML_ParamLoc),HL
HTM_LoadFrameObjt1
   CALL  HTM_GetCmdParam	;Locate next parameter
   LD    A,(HL)
   CP    EHTML_CHR		;no more parameters?
   RET   Z			;Yes
   PUSH  HL
   CALL  HTM_FindParmLbl	;Determine the parameter nametype
   POP   HL
   LD    A,HTM_P_src		;Is parameter 'src' ?
   CP    B
   JR    NZ,HTM_LoadFrameObjt1	;No (check next parameter)
;
   CALL  HTM_GetCmdParmVal	;HL = address of parameter value
   CALL  CPCB_OpenNewObj	;Load frame object
   RET   Z			;object loaded okay
   LD    A,(HTML_FrameID)
   CALL  HTM_FrmeCalcAddr
   SET   4,(HL)			;Flag Frame unavailable due to error
   SET   1,(HL)			;Flag load failed
   RET
;=========================================================================
;Name     HTM_LoadFrameObjs
;Purpose  To load up the Frames Objects.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_FrameFlg = current state of frames usage.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_FrameFlg = updated on frame loaded counter (lower)
;Uses     TBD
;Call By  TBD
;Calls    HTM_FrmeCalcAddr
;         HTM_LoadFrameObj
;Notes    None
;=========================================================================
HTM_LoadFrameObjs
   LD    A,2				;First user defined frame.
   CALL  HTM_FrmeCalcAddr		;Calc frames Db addr for object
   LD    C,2				;First frame to load
HTM_LoadFrmeObjs1
   BIT   0,(HL)				;end of user frames?
   JR    Z,HTM_LoadFrmeObjs2		;Yes
   BIT   2,(HL)				;Frame contents request already done?
   JR    NZ,HTM_LoadFrmeObjs0		;Yes
   SET   2,(HL)				;prevent re-request of object
   PUSH  BC
   PUSH  HL
   LD    A,C
   CALL  HTM_LoadFrameObject		;Loads frames object
   CALL  Mem_SwapInFramesDB
   POP   HL
   POP   BC
HTM_LoadFrmeObjs0
   LD    DE,S_HTMLFramSz
   ADD   HL,DE
   INC   C
   JR    HTM_LoadFrmeObjs1		;load next frames object
HTM_LoadFrmeObjs2
   LD    HL,HTML_FrameFlg
   RES   2,(HL)				;Flag all frame objects loaded
   JP    CPCB_ChkFrmes			;check frames visibility
;=========================================================================
;Name     HTML_FrmeChkS
;Purpose  To check if the start location is valid for the CPC screen.
;In Regs  A = ??
;         BC = PageWdw_SRow/PageWdw_SCol
;         DE = ??
;         HL = Window_FSRow/Window_FSCol
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. The values are all based from 0.
;=========================================================================
HTML_FrmeChkS
   OR    A
   SBC   HL,BC			;Before Start location?
   RET   NC				;No (accept calculated start location)
   LD    HL,0			;Yes (only start at zero)
   RET
;=========================================================================
;Name     HTML_FrmeChkE
;Purpose  To calculate the number of row/cols in the CPC frame view.
;In Regs  A = ??
;         BC = max position
;         DE = end item position
;         HL = end item position
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. The values are all based from 0.
;=========================================================================
HTML_FrmeChkE
   PUSH  HL
   OR    A
   SBC   HL,DE			;before end?
   POP   HL
   RET   C				;Yes (restore HL)
   EX    DE,HL			;HL = limit
   RET
;=========================================================================
;Name     HTM_CalcFrameCPC
;Purpose  To calculate the CPC screen coordinates for a web-page frame.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  Window_FrameV = current frame object
;         PageWdw_SRow = CPC screen row offset on web page
;         PageWdw_SCol = CPC screen column offset on web page
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars Window_FSRow = start row (from zero)
;         Window_FSCol = Start column (from zero)
;         Window_FERow = end row (from zero)
;         Window_FECol = end column (from zero)
;         Windw_NumRows = number of rows (from zero)
;         Windw_NumCols = number of columns (from zero)
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. Assume the frame (or a portion) is shown on CPC screen
;            (somewhere).
;=========================================================================
HTM_CalcFrameCPC
;
;  Calc start and end row on CPC screen.
;
   LD    HL,(Window_SRow)		;Frame start row
   LD    BC,(PageWdw_SRow)		;CPC screen offset onto web-page
   CALL  HTML_FrmeChkS			;Check before start row offset
   LD    (Window_FSRow),HL		;setup start row on CPC screen
;
   LD    HL,(Window_ERow)
   SBC   HL,BC				;Calculate CPC end row
   LD    DE,MaxWebPgRow-1		;adjust for base 0
   CALL  HTML_FrmeChkE			;calculate number of rows
   LD    (Window_FERow),HL		;setup end row on CPC screen
   LD    DE,(Window_FSRow)
   SBC   HL,DE
   LD    A,L
   LD    (Windw_NumRows),A
;
;  Calc start and end column on CPC screen.
;
   LD    HL,(Window_SCol)		;Frame start row
   LD    BC,(PageWdw_SCol)		;CPC screen offset onto web-page
   CALL  HTML_FrmeChkS			;Check if before start col offset
   LD    (Window_FSCol),HL		;setup column on CPC screen
;
   LD    HL,(Window_ECol)
   SBC   HL,BC				;Calculate CPC end column
   LD    A,(Scrn_MaxCol)
   DEC   A				;adjust for base 0
   LD    D,0
   LD    E,A
   CALL  HTML_FrmeChkE			;calculate number of rows
   LD    (Window_FECol),HL		;setup end column on CPC screen
   LD    DE,(Window_FSCol)
   SBC   HL,DE
   LD    A,L
   LD    (Windw_NumCols),A
   RET
;=========================================================================
;Name     HTM_CheckFrameSz
;Purpose  To check if the frame size is valid
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  Window_FrameV = frame to display on screen.
;Out Regs A = Z flag set if frame is too small else NZ
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_CheckFrameSz
   LD    A,(Windw_NumRows)
   AND   &FC				;frame > 3 rows?
   RET   Z				;No, cant show frame at all
   LD    A,(Windw_NumCols)
   AND   &FC				;Frame > 3 columns?
   RET
;=========================================================================
;Name     HTM_ShwFrameBorder
;Purpose  To display the border of a frame.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  Window_FrameV = frame info to display on screen.
;         Stream_WebPage = AMSTRAD CPC screen window for web-page.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    SC_SEL_STREAM
;         HTM_FrmeCalcAddr
;         HTM_CheckFrameSz
;         CPCB_ScrollFrame
;         CPCB_NonScrollFrame
;Notes    1. Assume the frame (or a portion) is shown on CPC screen
;           (somewhere).
;=========================================================================
HTM_ShwFrameBorder
   LD    A,(Stream_WebPage)
   CALL  SC_SEL_STREAM			;select web-page window.
;
;  If the frame definition has no frameborder set then no border is shown.
;
   LD    A,(Window_FrmeID)
   BIT   4,A				;Show border?
   RET   Z				;No
;
   LD    A,(HTML_FrameCurrID)
   CALL  HTM_FrmeCalcAddr		;HL = address of frame DB data
   LD    A,(Window_FrmeID)
   LD    (HL),A				;Update scroll flag state in DB
   BIT   6,A				;Scrollable frame?
   JP    NZ,CPCB_ScrollFrame		;Yes
;
   CALL  HTM_CheckFrameSz		;Frame to small?
   JP    NZ,CPCB_NonScrollFrame		;No
   RET
;=========================================================================
;Name     HTM_Get1stFrame
;Purpose  To get the first visible frame.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_FrmeSetR = address of rows array
;         HTML_FrmeSetC = address of colunms array
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_FrameCurrID = updated
;         Window_FrameV = frame to display on screen.
;Uses     TBD
;Call By  TBD
;Calls    HTM_GetFrame
;         HTM_ResetRC
;Notes    None
;=========================================================================
HTM_Get1stFrame
   LD    A,1
   LD    (HTML_FrameCurrID),A
   CALL  HTM_GetFrame			;Window_FrameV = first frame
   BIT   3,A				;Frame 1 available to be seen?
   JR    Z,HTM_Get1stFrme1		;Yes
;
;  The first frame is not visible as other frames have been defined
;  so the first has become hidden for processing purposes.
;
   LD    A,2
   LD    (HTML_FrameCurrID),A
   JP    HTM_GetFrame			;Window_FrameV = second frame
;
HTM_Get1stFrme1
   LD    HL,(HTML_FrmeSetR)
   CALL  HTM_ResetRC
   LD    HL,(HTML_FrmeSetC)
   JP    HTM_ResetRC
;=========================================================================
;Name     HTM_FindFrameIcon
;Purpose  To find which icon of the frame was clicked on.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  Cursor_Row = current web-page row (starts from 0)
;         Cursor_Col = current web-page column (starts from 0)
;         Window_FrameV = Window frame object with frame dimension in CPC
;                         context.
;         WindowFrmeNum = current frame number.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FrameItemSel = updated.
;         Window_FSRow = start row (corrupted)
;         Window_FSCol = Start column (corrupted)
;         Window_FERow = end row (corrupted)
;         Window_FECol = end column (corrupted)
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. IMPROVE ROUTINE USING REG A FOR STATUS FLAGS.
;=========================================================================
HTM_FindFrameIcon
   CALL  HTM_CalcFrameCPC		;Calculate the CPC frame location
   LD    A,(FrameItemSel)
   AND   &40				;Clear all bits (except bit 6)
;
   LD    DE,5
   LD    HL,(Window_FECol)
   LD    BC,(Cursor_Col)		;Get CPC screen location
   SBC   HL,BC				;On right-hand vertical frame?
   JR    Z,WIN_FrmeDBCk1		;Yes (check which row on)
;
   LD    HL,(Window_FERow)
   LD    BC,(Cursor_Row)		;Get CPC screen location
   OR    A
   SBC   HL,BC				;On bottom row of frame?
   JR    NZ,WIN_FrmeDBCk3		;No
;=========================================================================
;  Check which column on the bottom of the frame
;=========================================================================
   LD    BC,(Cursor_Col)		;Get CPC screen location
   LD    HL,(Window_FSCol)
   INC   HL				;location of scroll left icon
   OR    A
   SBC   HL,BC				;At scroll left icon?
   JR    Z,WIN_FrmeDBCk1SL		;Yes
;
   LD    HL,(Window_FSCol)
   INC   HL				;location of scroll left icon
   INC   HL
   OR    A
   SBC   HL,BC				;At slider left/rigth icon?
   JR    Z,WIN_FrmeDBCk1SLR		;Yes
;
   LD    HL,(Window_FECol)
   DEC   HL
   OR    A
   SBC   HL,BC				;At scroll right icon?
   JR    NZ,WIN_FrmeDBCk3		;No
;
   SET   0,A				;flag bottom right icon selected
   JR    WIN_FrmeDBCk2
;
WIN_FrmeDBCk1SL
   SET   1,A
   JR    WIN_FrmeDBCk2
;
WIN_FrmeDBCk1SLR
   SET   5,A				;flag slider left/right icon
   JR    WIN_FrmeDBCk2
;=========================================================================
;  Check which row on the far right vertical frame
;=========================================================================
WIN_FrmeDBCk1
   LD    DE,(Cursor_Row)		;Get CPC screen location
;
   LD    HL,(Window_FSRow)
   INC   HL
   OR    A
   SBC   HL,DE				;At scroll up icon?
   JR    Z,WIN_FrmeDBCk1SU		;Yes
;
   LD    HL,(Window_FSRow)
   INC   HL
   INC   HL
   OR    A
   SBC   HL,DE				;At scroller up/down icon?
   JR    Z,WIN_FrmeDBCk1SUD		;Yes
;
   LD    HL,(Window_FERow)
   DEC   HL
   OR    A
   SBC   HL,DE				;At scroll down icon?
   JR    NZ,WIN_FrmeDBCk3		;No
;
   SET   2,A				;flag scroll down request
   JR    WIN_FrmeDBCk2
;
WIN_FrmeDBCk1SU
   SET   3,A				;flag scroll up request
   JR    WIN_FrmeDBCk2
;
WIN_FrmeDBCk1SUD
   SET   4,A				;flag slider up/down icon
WIN_FrmeDBCk2
   SET   7,A				;flag an icon was clicked on
;=========================================================================
WIN_FrmeDBCk3
   LD    HL,WindowFrmeStat
   RES   0,(HL)				;Assume no frame refresh
   LD    (FrameItemSel),A
   AND   &0F				;Any scroll icon?
   RET   Z				;No
   SET   0,(HL)				;Flag frame refresh is required
;
   LD    A,(WindowFrmeNum)
   CALL  HTM_FrmeCalcAddr		;HL = address of frame structure
   INC   HL				;Point to S_HTMLFramSF
   SET   0,(HL)				;Flag frame is updated.
   DEC   HL
   JP    HTML_NewFrameOff1		;adjust frame scoll offsets
;=========================================================================
;Name     HTML_NewFrameOffset
;Purpose  To update the frame offsets upon frame icon clicking.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  WindowFrmeNum = frame number.
;         FrameItemSel = frame item selection flags
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars FrameItemSel = frame item selection flags updated.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    None
;=========================================================================
HTML_NewFrameOffset
   LD    DE,0
   LD    A,(FrameItemSel)
   BIT   7,A			;Frame icons selected?
   RET   Z			;No
;
   RES   7,A
   LD    (FrameItemSel),A	;Clear Frame select.
   AND   &0F			;frame scroll icon?
   JR    Z,HTM_OffFrameP3	;no (check for scoller icon)
;
;  One of the scroll icons selected
;
   LD    A,(WindowFrmeNum)
   CALL  HTM_FrmeCalcAddr	;HL = address of frame structure
;
HTML_NewFrameOff1
   EX    DE,HL
   LD    HL,S_HTMLFramRo	;offset to frame row scroll offsets.
   LD    BC,&FFFF		;-1 for decrement
   LD    A,(FrameItemSel)
   BIT   3,A			;scroll up
   JR    NZ,HTM_OffFrameUpd	;Yes
;
   LD    HL,S_HTMLFramCo	;offset to frame column scroll offsets.
   BIT   1,A			;scroll Left
   JR    NZ,HTM_OffFrameUpd	;Yes
;
   LD    HL,S_HTMLFramRo	;offset to frame row scroll offsets.
   LD    BC,1
   BIT   2,A			;scroll down
   JR    NZ,HTM_OffFrameUpd	;Yes
;
   LD    HL,S_HTMLFramCo	;offset to frame column scroll offsets.
   JR    HTM_OffFrameUpd	;scroll right
;
HTM_OffFrameP3
   LD    A,(FrameItemSel)
   AND   &30			;slider icon ?
   RET   Z
;
;  One of the slider icons selected
;
   LD    HL,S_HTMLFramRo
   LD    A,(FrameItemSel)
   BIT   4,A
   JR    NZ,HTM_OffFrameUpd
   LD    BC,(Cursor_OffCol)
   LD    HL,S_HTMLFramCo
HTM_OffFrameUpd
   ADD   HL,DE
   CALL  UTL_GetVarWord		;DE = offset value to adjust
   EX    DE,HL
   ADD   HL,BC			;adjust value
   BIT   7,H			;less than zero?
   JR    Z,HTM_OffFrameP3a	;No
   LD    HL,0			;ensure never less than 0
HTM_OffFrameP3a
   EX    DE,HL
   DEC   HL			;back to start of variable
   DEC   HL
   JP    UTL_SetVarWord
