;=========================================================================
;
; Application Routines (image handler)
; Copyright (c) 2010 Ray Palmer
;
;=========================================================================
;Name     HTM_LoadSrcObj
;Purpose  To load the source object referenced by the HTML command.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = address of HTML 'src' command parameter
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    HTM_GetCmdParmVal
;         URL_GetBaseURL
;         CPCB_OpenNewObj
;Notes    1. Assume that the source is just a filename and not include a
;            path.
;         2. The object loaded will either be after the HTML for text or
;            loaded into the code bank if binary image.
;=========================================================================
HTM_LoadSrcObj
   CALL  HTM_GetCmdParmVal		;HL = address of parameter value
   PUSH  HL
   LD    DE,SysGenBuffer
   CALL  URL_GetBaseURL			;setup base URL
;
   LD    DE,SysGenBuffer
   LD    C,SLASH_CHAR
   CALL  UTL_LastChar			;go to address for filename
;
   POP   HL
   LD    C,DBLEQ_CHR
   CALL  UTL_strccpy			;insert path/filename
   XOR   A
   LD    (DE),A
;
   LD    A,2
   LD    (FIO_CodeBlk),A
   LD    HL,&4000
   LD    (FileLoadAddr),HL
   LD    HL,SysGenBuffer
   JP    CPCB_OpenNewObj		;load object
;=========================================================================
;Name     HTM_ImgCalcDBEnd
;Purpose  To calculate the DB address for end of image Db items
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  ConfigBank = configuration bank for Forms DB.
;Out Regs A = corrupted
;         BC = corrupted
;         DE = ??
;         HL = address of DB entry
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    MEM_SwapInCFG
;Notes    1. configuration bank left swapped in.
;=========================================================================
HTM_ImgCalcDBEnd
   CALL  MEM_SwapInCFG			;Swap in Config bank
   LD    HL,CFG_ImagesDB
   LD    BC,S_ImageSz
HTM_ImgCalcDBEnd0
   LD    A,(HL)
   OR    A				;end of db?
   RET   Z				;Yes
   ADD   HL,BC
   JR    HTM_ImgCalcDBEnd0		;try next entry
;=========================================================================
;Name     HTM_ImageItm2DB
;Purpose  To add an image item to the DB of images.
;In Regs  A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_CmdStart = address of HTML command
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars Forms DB in the config bank contains the new item.
;         IMG_Addr = address of saved image db entry
;Uses     TBD
;Call By  TBD
;Calls    HTM_ImgCalcDBEnd
;         MEM_SwapInMain
;Notes    1. There is no check for the number of items to be in the DB, so
;            it is possible to exceed the allocated memory.
;=========================================================================
HTM_ImageItm2DB
   CALL  HTM_ImgCalcDBEnd		;Goto end of DB (bank swaped in)
   LD    (IMG_Addr),HL
   EX    DE,HL
   LD    HL,IMG_Var
   LD    BC,S_ImageSz
   LDIR					;Save where image data is loaded.
   EX    DE,HL
   LD    (HL),0
   JP    MEM_SwapInMain
;=========================================================================
;Name     HTM_ImageFindHTML
;Purpose  To check if the image item is already in DB by its HTML address
;         location.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_CmdStart = start address of HTML command to find.
;Out Regs A = Find result (0 = 'Not found' else 1 found)
;         B = ??
;         C = Db entry number
;         DE = ??
;         HL = Address of forms database entry.
;         IX = ??
;         IY = ??
;Out Vars IMG_Var = Updated with Image DB item if found.
;Uses     TBD
;Call By  TBD
;Calls    MEM_SwapInCfg
;Notes    1. The Config bank is left swapped in.
;=========================================================================
HTM_ImageFindHTML
   CALL  MEM_SwapInCfg			;Swap in Forsm DB bank.
;
   LD    HL,CFG_ImagesDB
HTM_ImageFndHTML1
   LD    A,(HL)
   OR    A				;End of DB
   RET   Z				;Yes
   PUSH  HL
   LD    DE,S_ImageAdr
   ADD   HL,DE				;point to address of html locn
   LD    DE,HTML_CmdStart
   LD    B,3
   CALL  UTL_strncmp			;Compare 3 bytes
   POP   HL
   JR    Z,HTM_ImageFndHTML3		;Yes (download to main memory)
   LD    DE,S_ImageSz
   ADD   HL,DE				;next DB entry
   JR    HTM_ImageFndHTML1
;
HTM_ImageFndHTML3
   LD    DE,IMG_Var
   LD    BC,S_ImageSz
   LDIR					;copy Image DB to local
   LD    A,1
   RET
;=========================================================================
;Name     HTM_LoadImageObj
;Purpose  To load the binary image object after the HTML content from the
;         code bank where binary image data is normally loaded.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars IMG_ScrnMde = image screen mode
;         URL_EndAddr = updated with new address for next object.
;Uses     TBD
;Call By  TBD
;Calls    Mem_SwapInCode
;         UTL_GetVarWord
;         Mth_Multiply
;         HTM_NextHTMBank
;Notes    FIXME TO USE VARIABLES FOR ROWS/COLS TO GET ALL PATTERN DATA.
;         (2014-02-28)
;=========================================================================
HTM_LoadImageObj
   CALL  Mem_SwapInCode
   LD    HL,SysBnkBuffer
   LD    A,(HL)				;Get Image screen mode
   INC   HL
   LD    (IMG_ScrnMde),A
   CALL  UTL_GetVarWord			;DE = number of rows
   PUSH  DE
   CALL  UTL_GetVarWord			;DE = number of columns
   POP   HL
   CALL  Mth_Multiply			;HL = Calc rows x columns
   CALL  HTM_GetBytesPerChar
   LD    E,B
   LD    D,0
   CALL  Mth_Multiply			;HL = total bytes of data
   LD    DE,5
   ADD   HL,DE				;adjust for image header data
;
;  Copy data after HTML
;
   LD    C,L				;BC = length of binary data
   LD    B,H
   LD    HL,(URL_EndAddr)
   LD    DE,SysBnkBuffer
HTM_LoadImageObj3
   PUSH  BC
   PUSH  HL
   CALL  Mem_SwapInCode			;Swap in bank with binary data
   LD    A,(DE)
   PUSH  AF
   LD    A,(URL_EndAddr+2)
   CALL  HTM_NextHTMBank		;swap in html bank to save to.
   LD    (URL_EndAddr+2),A
   POP   AF
   POP   HL
   LD    (HL),A				;save byte of image data
   INC   DE
   INC   HL
   POP   BC
   DEC   BC
   LD    A,B				;All data copied?
   OR    C
   JR    NZ,HTM_LoadImageObj3		;No
   LD    (URL_EndAddr),HL
   RET
;=========================================================================
;Name     HTM_FindDigitChr
;Purpose  To find the next digit character
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = address of text
;         IX = ??
;         IY = ??
;In Vars  IMG_Locn = 3 byte location of text data
;Out Regs A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars IMG_Locn = updated 3 byte location of text data
;Uses     TBD
;Call By  TBD
;Calls    Mem_SwapInHTML
;         UTL_isdigit
;         HTM_NextHTMBank
;Notes    TBD
;=========================================================================
HTM_FindDigitChr
   LD    A,(IMG_Locn+2)
   CALL  Mem_SwapInHTML
   LD    HL,(IMG_Locn)
HTM_FindDigitChr0
   LD    A,(HL)			;get character
   CALL  UTL_isdigit		;Is it a digit?
   RET   Z			;Yes
   LD    A,(IMG_Locn+2)
   INC   HL
   CALL  HTM_NextHTMBank	;swap in html bank to save to.
   LD    (IMG_Locn+2),A
   JR    HTM_FindDigitChr0
;=========================================================================
;Name     HTM_GetDigit
;Purpose  To get the string of digits terminated by a non-digit char
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  IMG_Locn = 3 byte location of data
;Out Regs A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars IMG_Locn = updated 3 byte location of data
;         HTML_ObjTemp = corrupted.
;Uses     TBD
;Call By  TBD
;Calls    HTM_FindDigitChr
;         HTM_NextHTMBank
;         UTL_isdigit
;         UTL_readdec
;Notes    1. Assume the HTML bank of memory is swapped in.
;         2. Assume the character to be processed is a digit.
;=========================================================================
HTM_GetDigit
   CALL  HTM_FindDigitChr	;find start of digit string
   LD    HL,(IMG_Locn)
   LD    DE,HTML_ObjTemp
;
HTM_GetDigit0
   LD    A,(HL)
   LD    (DE),A
   INC   DE
   INC   HL
   LD    A,(IMG_Locn+2)
   CALL  HTM_NextHTMBank	;swap in html bank to save to.
   LD    (IMG_Locn+2),A
   LD    A,(HL)			;get byte of string data
   CALL  UTL_isdigit		;Is it a digit character
   JR    Z,HTM_GetDigit0	;Yes (get another)
   XOR   A
   LD    (DE),A			;add NUL to end of string.
   LD    (IMG_Locn),HL
   RET
;=========================================================================
;Name     HTM_GetBytesPerChar
;Purpose  To determine if the image character has 8 or 16 bytes.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  Scrn_Mode = screen mode.
;Out Regs A = ??
;         B = Number of bytes 
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars NOne.
;Uses     TBD
;Call By  TBD
;Calls    NOne
;Notes    TBD
;=========================================================================
HTM_GetBytesPerChar
   LD    B,IMG_BYTEPERCHR	;Bytes for mode 2 character
   LD    A,(IMG_ScrnMde)
   CP    2
   RET   Z
   LD    B,IMG_BYTEPERCHR*2
   RET
;=========================================================================
;Name     HTM_GetImageData
;Purpose  To get the 8 (or 16) bytes of image data for a character to be
;         displayed on the screen and put this data after the loaded HTML.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = address of image data in the HTML banks
;         IX = ??
;         IY = ??
;In Vars  IMG_Locn = location of data
;         HTML_LblTemp = current image data to load from (binary)
;         IMG_DBFlag = bit 7-2 has image type
;Out Regs A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = Address of next image character byte data
;         IX = ??
;         IY = ??
;Out Vars IMG_Locn = updated location of data
;Uses     TBD
;Call By  TBD
;Calls    MEM_SwapInHTML
;         HTM_FindDigitChr
;         HTM_GetDigit
;         HTM_NextHTMBank
;Notes    TBD
;=========================================================================
HTM_GetImageData
   LD    A,(IMG_Locn+2)
   CALL  MEM_SwapInHTML		;Swap in HTML bank for image data
;
   LD    HL,(IMG_Locn)
   LD    DE,HTML_LblTemp	;Address of image display temp space.
   CALL  HTM_GetBytesPerChar	;B = number of bytes per image character.
   LD    A,(IMG_DBFlag)
   AND   &FC			;text file?
   JP    NZ,UTL_CopyMem		;No, copy binary image data down.
;
;  Convert textural image to binary after HTML
;
HTM_GetImageData1
   PUSH  DE
   CALL  HTM_GetDigit		;Get digit value
   LD    HL,HTML_ObjTemp
   CALL  UTL_readdec		;convert to byte
   POP   DE
   LD    (DE),A
   INC   DE
   LD    A,(IMG_Locn+2)
   CALL  HTM_NextHTMBank	;swap in html bank to save to.
   LD    (IMG_Locn+2),A
   DJNZ  HTM_GetImageData1
;
   LD    (IMG_Locn),HL
   RET
;=========================================================================
;Name     HTM_GetImageSize
;Purpose  To setup the DB structure with the image configuration (screen
;         mode, number of rows and number of columns).
;In Regs  A = Memory bank where textural image is kept.
;         BC = ??
;         DE = ??
;         HL = address of textural image data in the HTML banks
;         IX = ??
;         IY = ??
;In Vars  IMG_DBData = 3 byte address of data within HTML banks.
;Out Regs A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = Address of first byte of image data
;         IX = ??
;         IY = ??
;Out Vars IMG_NRows = number of rows
;         IMG_NCols = number of columns
;         IMG_ScrnMde = image screen mode.
;         IMG_Locn = updated location of data
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_GetImageSize
   LD    A,(IMG_DBData+2)
   CALL  MEM_SwapInHTML
;
   LD    HL,(IMG_DBData)
   LD    DE,IMG_ScrnMde
   LD    B,5			;size of image def size hdr
   LD    A,(IMG_DBFlag)
   AND   &FC			;text file?
   JP    NZ,UTL_CopyMem		;No, copy data from binary
;
;  Setup DB structure with  screen mode, rows and columns
;
   CALL  HTM_GetDigit		;get digit string
   LD    HL,HTML_ObjTemp
   CALL  UTL_readdec		;convert to byte
   LD    (IMG_ScrnMde),A
;
   CALL  HTM_GetDigit		;get digit string
   LD    DE,HTML_ObjTemp
   CALL  UTL_readdec2		;convert to integer (16-bit)
   LD    (IMG_NRows),HL
;
   CALL  HTM_GetDigit		;get digit string
   LD    DE,HTML_ObjTemp
   CALL  UTL_readdec2		;convert to integer (16-bit)
   LD    (IMG_NCols),HL
   RET
;=========================================================================
;Name     HTM_ShowImage
;Purpose  To display the image loaded into the HTML banks of memory.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  IMG_Var = Updated with found Image DB item.
;         Window_SRow = start row of HTML frame on CPC screen
;         Window_SCol = start columun of HTML frame on CPC screen
;         HTML_Row = row offset into the HTML window frame
;         HTML_Col = col offset into the HTML window frame
;Out Regs A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_BR_Row = top row of image
;         HTML_Row = updated row offset into the HTML window frame
;         HTML_Col = updated col offset into the HTML window frame
;Uses     TBD
;Call By  TBD
;Calls    HTM_GetImageSize
;         HTM_GetImageData
;         HTM_ShowChar
;Notes    1. The image displayed on the screen is rendered using one of
;            user defined characters. That means all pictures are defined
;            in terms of character widths. So for example if an image is
;            normally 200 x 40 (in pixels) then the processing is
;            calculated to be 25 x 5 in characters. 
;            Where the image is not a whole number of pixels as
;            characters then the calculations should be rounded up.
;         2. Any existing mode 2 image can be re-packaged in this format
;            to be made available to the browser.
;
;         NEED TO FIX TO CHECK IF WITIHIN FRAME FOR PICTURE CHARACTERS.
;=========================================================================
HTM_ShowImage
   LD    A,(IMG_DBData+2)
   LD    (IMG_Locn+2),A
   LD    HL,(IMG_DBData)
   LD    DE,5
   ADD   HL,DE			;Skip image header info
   LD    (IMG_Locn),HL
   LD    HL,HTML_BR_Flag
   SET   1,(HL)			;flag <BR> object in place.
;
   LD    HL,(HTML_Col)
   LD    (HTML_ImgCol),HL	;record left of image
   LD    HL,(HTML_Row)
   PUSH  HL
;
   LD    DE,(IMG_NRows)
;
HTM_ShowImage0
   LD    BC,(IMG_NCols)
;
HTM_ShowImage1
   PUSH  BC			;save # columns
   PUSH  DE			;Save # rows
   CALL  HTM_GetImageData	;get icon char data
   LD    C,B_6
   CALL  HTM_ShowChar		;Show image character data
   LD    HL,(HTML_Col)
   INC   HL			;next web-page column
   LD    (HTML_Col),HL
   POP   DE
   POP   BC
   DEC   BC
   LD    A,B			;end of image columns?
   OR    C
   JR    NZ,HTM_ShowImage1	;no (do next column)
;
   LD    HL,(HTML_ImgCol)
   LD    (HTML_Col),HL		;reset left image col
   LD    HL,(HTML_Row)
   INC   HL			;next row
   LD    (HTML_Row),HL
   DEC   DE
   LD    A,E			;end of image row data
   OR    D
   JR    NZ,HTM_ShowImage0	;yes
;
   INC   HL
   LD    (HTML_ImgRow),HL	;save bottom row of image for <BR>
   LD    HL,(HTML_Col)
   LD    BC,(IMG_NCols)
   ADD   HL,BC
   INC   HL
   LD    (HTML_ImgCol),HL	;save right column of image
   POP   HL
   LD    (HTML_Row),HL
   RET
