;=========================================================================
;
; Application Routines (HTML rendering commands)
; Copyright (c) 2010 Ray Palmer
;
; Notes
;
; 1. The rendering of the current command does not expect that the same
;    command to be recursed. This may lead to an infinte loop if it does.
;
; 2. Where a command changes the font (e.g normal to italic) the original
;    command will have no way of reverting the font back to what it was.
;
;=========================================================================
;
;  Links handlers
;
;=========================================================================
;Name     HTM_StrtNewHLink
;Purpose  To startup a new hyperlink DB entry
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_Col = Current display column
;         HTML_Row = Current display row
;         Window_SCol = Base column of frame
;         Window_SRow = Base row of frame.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_HLDbItem = Initialised with the start location of the
;                         hyperlink including the start HTML address.
;         HTML_href = flagged to show now in hyperlink item
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
HTM_StrtNewHLink
   LD    HL,HTML_href
   SET   0,(HL)				;flag in hyperlinked command
   CALL  HTM_GetWebPgeCR		;get current Row/Column
   LD    (HTML_HLDbItem+S_HLSCol),DE
   LD    (HTML_HLDbItem+S_HLSRow),HL
   LD    HL,HTML_CmdStart
   LD    DE,HTML_HLDbItem+S_HLLocn
   LD    BC,3
   LDIR					;Save HTML command location
   RET
;=========================================================================
;Name     HTM_FinalNewHLink
;Purpose  To Finalise a new hyperlink DB entry and save it to the DB. The
;         local hyperlink structure is upon finalisation is added to the
;         hyperlinks database.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_Col = Current display column
;         HTML_Row = Current display row
;         Window_SCol = Base column of frame
;         Window_SRow = Base row of frame.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_HLDbItem = updated and saved to the DB
;         HTML_href = flagged to show not in hyperlink item
;Uses     TBD
;Call By  TBD
;Calls    HTM_AddDBHyperLink
;Notes    None
;=========================================================================
HTM_FinalNewHLink
   LD    HL,HTML_href
   RES   0,(HL)				;flag not in hyperlinked command
   CALL  HTM_GetWebPgeCR
   LD    (HTML_HLDbItem+S_HLECol),DE
   LD    (HTML_HLDbItem+S_HLERow),HL
   RET
;=========================================================================
;Name     HTM_CalcHREFDBAddr
;Purpose  To calculate the address of a Hyper-link DB entry.
;In Regs  AF = Index number.
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  None.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
HTM_CalcHREFDBAddr
   LD    HL,DB_HyperLink
   OR    A
   RET   Z
   LD    DE,S_HLSz
HTM_CalcHREFDBAddr1
   ADD   HL,DE
   DEC   A
   JR    NZ,HTM_CalcHREFDBAddr1
   RET
;=========================================================================
;Name     URL_AddDBHyperLink
;Purpose  To add a new hyperlink to the DB.
;In Regs  AF = Bank to add hyperlink into.
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  WPHLIndex = last index into the DB of hyperlinks.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars WPHLIndex = updated last index into the DB of hyperlinks
;Uses     TBD
;Call By  TBD
;Calls    HTM_CalcHREFDBAddr
;         Mem_SwapInMain
;Notes    1. Hyperlink Database updated with new entry information.
;=========================================================================
HTM_AddDBHyperLink
   CALL  Mem_SwapInMain
   LD    A,(HTML_WPHLIndex)
   CP    DB_MaxLinks			;At max DB entries
   RET   Z				;Yes
   INC   A
   LD    (HTML_WPHLIndex),A
   DEC   A
   CALL  HTM_CalcHREFDBAddr
   EX    DE,HL
   LD    HL,HTML_HLDbItem
   LD    BC,S_HLSz
   LDIR					;save new link to DB
   RET
;=========================================================================
;Name     HTM_CheckHLink
;Purpose  To check the hyperlink DB entry for a clicked on hyperlink.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  WebPge_Row = web page row
;         WebPge_Col = web page column
;         HTML_HLDbItem = DB entry from the hyperlink's DB
;Out Regs A = 0 if not within hyper-link content else 1
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars None.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_CheckHLink
   XOR   A
   LD    HL,(WebPge_Row)
   LD    DE,(HTML_HLDbItem+S_HLSRow)	;DE = start row of hyperlink
   SBC   HL,DE
   JR    Z,HTM_CheckHLnk0		;at top row (Check columns)
   RET   C				;cursor above start row
   ADD   HL,DE
   LD    BC,(HTML_HLDbItem+S_HLERow)	;BC = end row of hyperlink
   OR    A
   SBC   HL,BC
   JR    Z,HTM_CheckHLnk0		;cursor on end row (check column)
   RET   NC				;after end row
;
HTM_CheckHLnk0
   LD    HL,(WebPge_Col)
   LD    DE,(HTML_HLDbItem+S_HLSCol)	;DE = start column of hyperlink
   OR    A
   SBC   HL,DE
   JR    Z,HTM_CheckHLnk1
   RET   C				;cursor before start column
   ADD   HL,DE
   LD    BC,(HTML_HLDbItem+S_HLECol)	;BC = end column of hyperlink
   OR    A
   SBC   HL,BC
   JR    Z,HTM_CheckHLnk1
   RET   NC				;cursor after end col
HTM_CheckHLnk1
   INC   A
   RET
;=========================================================================
;Name     HTM_FindHLink
;Purpose  To search the shown HTML for a possible clicked on hyperlink.
;In Regs  A = ??
;         BC = ??
;         DE = Current cursor Column on CPC screen
;         HL = Current cursor Row on CPC screen
;         IX = ??
;         IY = ??
;In Vars  WebPge_Row = web page row
;         WebPge_Col = web page column
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars KB_Buffer has URL for new item to open.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. Search the database of links.;
;=========================================================================
HTM_FindHLink
   CALL  Mem_SwapInMain			;ensure hyperlinks DB is available
;
   XOR   A
HTM_FindHLnk1
   LD    (HTML_href),A			;Start at first entry to DB
   LD    HL,HTML_WPHLIndex
   CP    (HL)				;At end of db ?
   RET   Z				;Yes
   CALL  HTM_CalcHREFDBAddr		;HL = address of link DB data entry.
   LD    DE,HTML_HLDbItem
   LD    BC,S_HLSz
   LDIR					;get DB item into temp work space.
   CALL  HTM_CheckHLink			;check if hyperlink DB matches
   OR    A
   JR    NZ,HTM_FindHLnk2		;Yes (get command and check for HREF)
   LD    A,(HTML_href)
   INC   A
   JR    HTM_FindHLnk1
;
HTM_FindHLnk2
   LD    HL,HTML_HLDbItem+S_HLLocn
   LD    DE,HTML_ObjLocn
   LD    BC,3
   LDIR					;Setup location of HTML command
;
   LD    A,(HTML_ObjLocn+2)
   CALL  Mem_SwapInHTML
   CALL  HTM_GetCmd			;KB_Buffer has complete command
;
;  Open the href path (if any)
;
   LD    HL,(HTML_CmdTemp)
   LD    (HTML_ParamLoc),HL
HTM_FindHLnk3
   CALL  HTM_GetCmdParam		;Locate next parameter
   LD    A,(HL)
   CP    EHTML_CHR			;more parameters?
   RET   Z				;No
   CALL  HTM_FindParmLbl		;Determine the parameter nametype
   LD    A,HTM_P_href
   CP    B				;Is parameter 'href' ?
   JR    NZ,HTM_FindHLnk3		;No
;
;  download hyperlink data to KB_buffer by shifting down the URL to start of
;  KB_Buffer.
;
   LD    HL,(HTML_ParamLoc)
   CALL  HTM_GetCmdParmVal		;Goto start of parameter value
   LD    DE,(HTML_CmdTemp)
   LD    B,URL_Size
HTM_FindHLnk4
   LD    A,(HL)
   CP    DBLEQ_CHR			;End of URL text?
   JR    Z,HTM_FindHLnk5		;Yes
   LD    (DE),A
   INC   HL
   INC   DE
   DJNZ  HTM_FindHLnk4
;
HTM_FindHLnk5
   EX    DE,HL
   LD    (HL),0				;add nul to end URL text
   RET
;=========================================================================
;Name     HTM_SaveLinkLbl
;Purpose  To save the label from a HTML URL.
;In Regs  A = ??
;         BC = ??
;         DE = Address of label text to find.
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_LblTemp = label to find.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. If the label occurs after a change in the font the page
;            rendering may not be quite be as expected since page rendering
;            forces the start font to be normal.
;=========================================================================
HTM_SaveLinkLbl
   LD    HL,(URL_LblAddr)
   LD    DE,HTML_LblTemp
   LD    B,HTML_ObjTempSz
   CALL  UTL_strncpy			;Copy out label text
   LD    A,DBLEQ_CHR
   LD    (DE),A				;Add end quote for exact checks
   LD    A,HTML_ObjTempSz+1
   SUB   B				;Calc size of URL text
   LD    (HTML_LblTempL),A		;Save length of text copied
   RET
;=========================================================================
;Name     HTM_ChkHRefLabel
;Purpose  To check if the HTML command has a label for a clicked on
;         hyperlink.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  KB_Buffer has complete HTML Command
;         HTML_CmdTemp = address of KB_Buffer
;         HTML_LblTemp = label text to find.
;Out Regs A = 0 if found else non-zero
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_ChkHRefLabel
;
;  Search <A ...> command for # label reference in parameters
;
   LD    HL,(HTML_CmdTemp)
   LD    (HTML_ParamLoc),HL
HTM_FindURLLabel2a
   CALL  HTM_GetCmdParam		;Locate next parameter
   LD    A,(HL)
   CP    EHTML_CHR			;more parameters?
   RET   Z				;No (check rest of HTML)
   PUSH  HL
   CALL  HTM_FindParmLbl		;Determine the parameter nametype
   POP   HL
   LD    A,HTM_P_id
   CP    B				;Is parameter 'name' ?
   JR    Z,HTM_FindURLLabel2b		;Yes (check if text matches #label)
;
   LD    A,HTM_P_name
   CP    B				;Is parameter 'name' ?
   JR    NZ,HTM_FindURLLabel2a		;No
;
;  Check if text to parameter matches the #label
;
HTM_FindURLLabel2b
   CALL  HTM_GetCmdParmVal		;HL = address of parameter value
   LD    DE,HTML_LblTemp		;Address of label to check
   LD    A,(HTML_LblTempL)
   LD    B,A
   CALL  UTL_strncmp			;matches label?
   RET   NZ				;No match in label
   XOR   A				;Found name for #label
   RET
;=========================================================================
;Name     HTM_FindURLLabel
;Purpose  To search the HTML for a label in a command which matches the
;         label text from a href # tag after the URL that was clicked on.
;         e.g http //www.website.com/index.htm#mylabel
;In Regs  A = ??
;         BC = ??
;         DE = Address of label text to find.
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_LblTemp = label to find.
;         HTML_LblTempL = length of label text
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_FCurrPos = address of label from hyperlink #tag
;         HTML_ObjLocn = corrupted.
;         HTML_FCurrPosT = Bit 7 updated if found
;         HTML_FCurrPosF = frame where label is found (=0 if not found)
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. If the label occurs after a change in the font the page
;            rendering may not be quite be as expected since page rendering
;            forces the start font to be normal.
;         2. The minimum command forms are -
;
;             <Hx id="hyperlinklabel">     where x is between 1 and 6.
;
;             <A name="hyperlinklabel">
;
;=========================================================================
HTM_FindURLLabel
   LD    HL,HTML_FCurrPosT
   RES   7,(HL)
   LD    HL,HTML_FCurrPos+2
   SET   7,(HL)				;Assume not found
   XOR   A
   LD    (HTML_FCurrPosF),A		;Clear Frame Find ID (not found)
   INC   A
HTM_FindURLLabel5
   LD    (HTML_FrmeSetI),A		;Save Frame Number
   CALL  HTM_FrmeCalcAddr		;Calc Frame DB address and swap in cfg
   LD    A,(HL)				;Get Frame ID/Status
   CP    HTML_C_EODB			;End of frames DB?
   RET   Z				;Yes
   BIT   3,A				;Frame can be seen?
   JR    Z,HTM_FindURLLabel4		;Yes
HTM_FindURLLabel5a
   LD    A,(HTML_FrmeSetI)
   INC   A				;setup next Frame
   JR    HTM_FindURLLabel5		;Get frame location and search it
;
HTM_FindURLLabel4
   LD    DE,S_HTMLFramAd
   ADD   HL,DE
   LD    E,(HL)				;Get Frame HTML start address
   INC   HL
   LD    D,(HL)
   INC   HL
   LD    A,(HL)				;Get Frame RAM Bank number
   EX    DE,HL
;
;  Found a HTML command check if it is valid for #label processing
;
HTM_FindURLLabel0
   CALL  HTM_GetNxtCmd			;Get next HTML command
   CALL  HTM_Get1stCmdChr		;Is command </....> ?
   JR    Z,HTM_FindURLLabel1a		;Yes
   OR    A				;end of html data?
   JR    Z,HTM_FindURLLabel5a		;yes
   JR    HTM_FindURLLabel2		;No (check for label)
;
HTM_FindURLLabel1a
   LD    HL,(HTML_ObjLocn)
   JR    HTM_FindURLLabel0		;continue with HTML search
;
;  Check if commands is has id/name for hyperlink
;
HTM_FindURLLabel2
   CALL  HTM_ChkHRefLabel		;Search command for label
   OR    A				;Did we find matching label?
   JR    NZ,HTM_FindURLLabel1a		;No, search rest of HTML
;
   XOR   A
   LD    (HTML_LblTempL),A		;Prevent another search
   LD    HL,(HTML_ObjLocn)
   INC   HL
   LD    A,(HTML_ObjLocn+2)
   LD    (HTML_FCurrPos),HL		;Setup RAM address of find
   LD    (HTML_FCurrPos+2),A
   LD    A,(HTML_FrmeSetI)
   LD    (HTML_FCurrPosF),A		;save found frame number
   LD    HL,HTML_FCurrPosT
   SET   7,(HL)
   RET
;=========================================================================
;Name     HTM_FindURLHlink
;Purpose  To search the loaded HTML for a label in a command which matches
;         the label text from a href # tag after the URL.
;         e.g http //www.website.com/index.htm#mylabel
;         The searched URL label will make the display of text be adjusted
;         within the frame for which the label exists in. Where no frames
;         are used the shift in text will be to where the label occurs.
;In Regs  A = ??
;         BC = ??
;         DE = Address of label text to find.
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_FCurrPos = address of label from href #tag
;         HTML_ObjLocn = corrupted.
;         HTML_LblTemp = label to find.
;         HTML_FCurrPosT = bit 7 set if label found
;         HTML_FCurrPosF = frame of found label.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. If the label occurs after a change in the font the page
;            rendering may not be quite be as expected since page rendering
;            forces the start font to be normal.
;=========================================================================
HTM_FindURLHlink
   CALL  HTM_SaveLinkLbl		;Save Search Label
   CALL  HTM_FindURLLabel		;Find URL #label in All HTML data
   LD    A,(HTML_FCurrPosF)
   CP    0
   JP    NZ,HTM_AdjustFindAddr		;Goto end of command to start render
   RET
