;=========================================================================
;
; Application Routines (HTML memory handler)
; Copyright (c) 2010 Ray Palmer
;
; Notes
; 1.  MEM_SwapInHTML is contained in cpc-mem.asm
;
;=========================================================================
;Name     HTM_NextHTMaddr
;Purpose  To calculate the HTM address if address is > &7fff
;In Regs  A = index into HTML banks
;         BC = ??
;         DE = ??
;         HL = address of HTML data
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = updated index into HTML banks
;         BC = ??
;         DE = ??
;         HL = updated address of HTML data
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    TBD
;=========================================================================
HTM_NextHTMaddr
   BIT   7,H
   RET   Z
   LD    HL,MEMORY_SWAP_ST
   INC   A
   RET
;=========================================================================
;Name     HTM_NextHTMBank
;Purpose  To calculate the HTM address if address is > &7fff and if
;         required swap in the new bank.
;In Regs  A = index into HTML banks
;         BC = ??
;         DE = ??
;         HL = address of HTML data
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = updated index into HTML banks
;         BC = ??
;         DE = ??
;         HL = updated address of HTML data
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    MEM_SwapInHTML
;Notes    1. Assume no previous bank is already swapped in when no new
;            bank is required.
;=========================================================================
HTM_NextHTMBank
   CALL  HTM_NextHTMaddr
   PUSH  AF
   CALL  MEM_SwapInHTML
   POP   AF
   RET
;=========================================================================
;Name     HTM_AdjustIndex
;Purpose  To adjust the find indices.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = current RAM bank address
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = updated RAM bank address
;         IX = ??
;         IY = ??
;Out Vars HTML_ObjLocn = current search location.
;Uses     TBD
;Call By  TBD
;Calls    HTM_NextHTMBank
;Notes    1. No check is made of the number of commands.
;=========================================================================
HTM_AdjustIndex
   INC   HL			;Next address in RAM Bank
   BIT   7,H			;Exceeded Page Size?
   RET   Z			;No
   LD    A,(HTML_ObjLocn+2)
   CALL  HTM_NextHTMBank	;Swap In HTML Bank/adjust location
   LD    (HTML_ObjLocn),HL
   LD    (HTML_ObjLocn+2),A
   RET
;=========================================================================
;Name     HTM_ChkRAMLoc
;Purpose  To check if the current RAM location allows the main routine to
;         display HTML rendering data.
;In Regs  A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_FCurrPos = start position of rendering after 'find'
;                         Address has location of found text
;                         Block has block of foud text (bit 7 = 1, the
;                         find is not requested)
;Out Regs A = ??
;         B = rendering state
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_FCurrPos+2 = updated to prevent changes to offset once the
;                           address is reached.
;         HTML_RCurrPos = New start Rendering position.
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    1. The resultant start location for rendering may not produce
;            the correct output, since the start may be within a hyperlink
;            and not be handled correctly (the same applies font text
;            effects).
;=========================================================================
HTM_ChkRAMLoc
   LD    A,(HTML_FCurrPos+2)	;Get Search RAM bank number
   BIT   7,A			;Find text performed?
   RET   NZ				;No (no check is required)
   LD    (HTML_RCurrPos+2),A	;Save new render start bank
   LD    HL,(HTML_FCurrPos)		
   LD    (HTML_RCurrPos),HL	;Reset new start RAM address
   LD    HL,HTML_FCurrPos+2
   SET   7,(HL)			;Stop checks on next entry
   LD    A,(HTML_FCurrPosT)
   BIT   6,A			;Adjusting for text search?
   RET   NZ				;Yes
;=========================================================================
;Name     HTM_AdjustFindAddr
;Purpose  To adjust the find location for rendering after a #label has
;         been found or from a text search.
;In Regs  AF = HTML bank index
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_FCurrPos = start location of HTML command with #label
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_FCurrPos = start location to show data.
;         HTML_RCurrPos = start location to show data.
;Uses     TBD
;Call By  TBD
;Calls    MEM_SwapInHTML (cpcb-mem.asm)
;Notes    1. Determine if the start text of the RAM bank data is from a
;            HTML command which may have been in the previous RAM bank
;            overlapping into this RAM bank.
;=========================================================================
HTM_AdjustFindAddr
   LD    A,(HTML_FCurrPos+2)
   CALL  MEM_SwapInHTML			;Swap in HTML bank
   LD    HL,(HTML_FCurrPos)		;RAM start location of HTML with label
   LD    A,(HTML_FCurrPos+2)
   LD    B,A
HTM_MemGetStLoc0
   BIT   7,H				;Above Top of banked memory?
   CALL  NZ,HTM_NextHTMBank		;Yes, Swap in next HTML bank
   LD    B,A				;Save new bank number
   LD    A,(HL)
   OR    A				;End of HTML data found?
   RET   Z				;Yes
   CP    SHTML_CHR				;Start of new HTML command?
   JR    Z,HTM_MemGetStLoc3		;Yes (save new start RAM address)
   INC   HL
   CP    EHTML_CHR				;End of HTML before start?
   JR    Z,HTM_MemGetStLoc3		;Yes (save new start RAM address)
   LD    A,B
   JR    HTM_MemGetStLoc0
HTM_MemGetStLoc3
   LD    A,B
   LD    (HTML_RCurrPos+2),A		;Save new render start bank		
   LD    (HTML_RCurrPos),HL		;Reset new start RAM address
   RET
;=========================================================================
;Name     HTM_ResetMem
;Purpose  To clear/Reset a segment of memory.
;In Regs  AF = data to memory to.
;         BC = Size of memory to set
;         DE = ??
;         HL = Address of memory
;         IX = ??
;         IY = ??
;In Vars  None.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars Memory chunk set ot specified by
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
HTM_ResetMem
   PUSH  DE
   LD    D,A
HTM_ResetMem1
   LD    (HL),D
   INC   HL
   DEC   BC
   LD    A,B
   OR    C
   JR    NZ,HTM_ResetMem1
   POP   DE
   RET