;=========================================================================
;
; Application Routines (Miscelleanous routines)
; Copyright (c) 2010 Ray Palmer
;
; Notes
;
;=========================================================================
;Name     HTM_FindSrceType
;Purpose  To determine the type of script source
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = Address of parameter text
;         IX = ??
;         IY = ??
;In Vars  AppsBank = bank id for App Db
;Out Regs AF = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars App_ID = Db index of application/handler
;                  0-254 valid ID, 255 invalid
;Uses     TBD
;Call By  html_cmd_script
;Calls    Mem_SwapInApps
;         utl_tolower
;Notes    TBD
;=========================================================================
HTM_FindSrceType
;
;  Skip 'text/' from 'text/handler_name'
;
   LD    A,(HL)
   INC   HL
   CP    "/"
   JR    NZ,HTM_FindSrceType
   LD    (HTML_FrmsTmp),HL
   CALL  MEM_SwapInApps		;Swap in the Apps bank.
   XOR   A
   LD    (App_ID),A
   LD    (App_State),A		;Flag no app found
   LD    HL,SysBnkBuffer	;Start of Apps DB
HTM_FindSrc1
   LD    A,(HL)			;Get DB entry state
   OR    A			;End of DB?
   JR    Z,HTM_FindSrc3		;Yes (no handler can be found)
   BIT   6,A			;Handler enabled?
   JR    Z,HTM_FindSrc2a	;No
   BIT   5,A			;Handler entry?
   JR    Z,HTM_FindSrc2a	;No
;
   PUSH  HL
   LD    DE,S_AppsName
   ADD   HL,DE			;Point to App/handler name
   LD    DE,(HTML_FrmsTmp)
   LD    B,MaxAppName
HTM_FindSrc1a
   LD    C,(HL)			;Get db name char
   LD    A,(DE)			;Get HTML object char
   CALL  utl_tolower		;ensure is lowercase
   CP    C			;both chars the same?
   JR    NZ,HTM_FindSrc2	;No (check if end of db name)
   INC   HL
   INC   DE
   DJNZ  HTM_FindSrc1a
   JR    HTM_FindSrc2b
;
HTM_FindSrc2
   LD    A,C
   OR    A			;At end of App Db name?
   JR    NZ,HTM_FindSrc2a	;No (next App Db)
HTM_FindSrc2b
   LD    A,(DE)
   POP   HL
   CP    DBLEQ_CHR		;End of source type text
   JR    Z,HTM_FindSrc4		;Yes (found a match)
HTM_FindSrc2a
   LD    A,(App_ID)
   INC   A
   LD    (App_ID),A
   LD    DE,S_AppEntSize
   ADD   HL,DE			;Point ot next App db entry
   JR    HTM_FindSrc1
;
HTM_FindSrc3
   LD    A,255
   LD    (App_ID),A
   RET
HTM_FindSrc4
   LD    DE,App_Object
   LD    BC,S_AppEntSize
   LDIR
   RET
;=========================================================================
;Name     HTM_FindText
;Purpose  To search the loaded file data for a text string. Where the file
;         is a HTML file the commands are skipped.
;In Regs  A = ??
;         B = search type (1=New, 2=Next)
;         DE = address of search text
;         HL = address to save result flag
;         IX = ??
;         IY = ??
;In Vars  HTML_PreserveFlag = data type to search (bit 7)
;         HTML_FCurrPos = current RAM addr start for find text
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_ObjLocn = updated
;         HTML_FCurrPos = address of found text or unchanged if not
;         HTML_ParamVLoc = address of status flag to update
;                          at address = 0 mean not found else 255
;         HTML_LblTemp = address of text to find.
;Uses     TBD
;Call By  TBD
;Calls    UTL_strlen
;         HTM_SkipCmd
;         UTL_strncmp
;         HTM_NextHTMBank
;Notes    1. The routine does not search for text within HTML commands.
;         2. The find always begins from the currently selected frame.
;Fixes    1. Fix code to handles frames!!!!
;=========================================================================
HTM_FindText
   LD    (HTML_LblTemp),DE		;Save save search text location.
   LD    (HTML_ParamVLoc),HL		;Save result flag location
   LD    (HL),0				;Flag no result
   EX    DE,HL
   CALL  UTL_strlen
   LD    (HTML_LblTempL),A		;Setup string length
;
   LD    HL,HTML_FCurrPosT
   RES   6,(HL)
   LD    HL,HTML_FCurrPos		;Assume find request for next
   LD    A,B
   CP    HTML_C_FindTxtNxt		;Find next request?
   JR    Z,HTM_FindTxt1			;Yes
   LD    A,1
HTM_FindTxt0a
   LD    (HTML_FCurrPosF),A
   PUSH  BC
   CALL  HTM_FrmeCalcAddr
   POP   BC
   LD    A,(HL)
   OR    A				;End of frames DB?
   RET   Z				;Yes
   BIT   4,A				;Frame available?
   JR    NZ,HTM_FindTxt0b		;No (check next frame)
;
   LD    A,B
   LD    DE,S_HTMLFramAd
   ADD   HL,DE				;Point to start of frames HTML
HTM_FindTxt1
   LD    DE,HTML_ObjLocn
   LD    BC,3
   LDIR					;Setup start search location
;
   CP    HTML_C_FindTxtNxt		;Find next request?
   JR    Z,HTM_FindTxt5			;Yes
   JR    HTM_FindTxt5a
;
HTM_FindTxt0
   LD    A,(HL)
   OR    A				;End of HTML data?
   JR    Z,HTM_FindTxt0b		;Yes
   LD    B,A
;
   LD    A,(HTML_PreserveFlag)
   BIT   7,A				;Plain text file search?
   JR    NZ,HTM_FindTxt4		;Yes
;
   LD    A,B
   CP    '<'				;Start of HTML command?
   JR    NZ,HTM_FindTxt4		;No
   CALL  HTM_SkipCmd			;Skip HTML command
   LD    A,C
   INC   HL	
   JR    HTM_FindTxt5a			;Continue to search
;
HTM_FindTxt0b
   LD    A,(HTML_FCurrPosF)
   INC   A				;Next frame to check
   JR    HTM_FindTxt0a
;
;  Check if string is here
;
HTM_FindTxt4
   LD    A,B
   CP    32				;control code?
   JR    C,HTM_FindTxt5			;Yes (ignore them)
;
   LD    DE,(HTML_LblTemp)		
   EX    DE,HL
   LD    A,(HTML_LblTempL)
   LD    B,A
   CALL  UTL_strncmp			;does search string occur here?
   JR    Z,HTM_FindTxt6			;Yes
;
HTM_FindTxt5
   LD    HL,(HTML_ObjLocn)
   LD    A,(HTML_ObjLocn+2)
   INC   HL
HTM_FindTxt5a
   CALL  HTM_NextHTMBank		;Swap In HTML Bank/adjust location
   LD    (HTML_ObjLocn+2),A
   LD    (HTML_ObjLocn),HL
   JR    HTM_FindTxt0
;
;  Return found status and setup refresh parameters
;
HTM_FindTxt6
   LD    HL,HTML_ObjLocn
   LD    DE,HTML_FCurrPos
   LD    BC,3
   LDIR					;Setup Location of Found text
   LD    HL,(HTML_ParamVLoc)
   LD    (HL),255			;Flag search succeeded.
   LD    HL,HTML_FCurrPosT
   SET   6,(HL)				;Flag adjust for text
   RET
;=========================================================================
;Name     HTM_ShowFileObjectData
;Purpose  To render the frames object.
;In Regs  A = ??
;         B = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_PreserveFlag = flags/settings
;         DefFont= default font to start rendering.
;         HTML_DispStatus = current display status.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_RCurrPos = Last rendering position
;         HTML_RenderState = updated.
;Uses     TBD
;Call By  TBD
;Calls    HTM_NextHTMBank - Swap in HTML memory bank
;         HTM_DoHTMLCmd   - Execute a HTML command
;         HTM_ShowChar    - Show a rendering character
;Notes    1. Where there are no frames this routine display the current
;            HTML.
;=========================================================================
HTM_ShowFileObjectData
;
   CALL  HTM_CalcFrameCPC		;Calculate CPC loc'n for frame.
;
   LD    HL,Window_FrmeID
   RES   6,(HL)				;Assume non-scrolling frame.
;
   CALL  HTM_CheckFrameSz		;Frame to small?
   RET   Z				;Yes
;
   XOR   A
   LD    (CPCB_Timer),A
;
   LD    A,(Window_Addr+2)
   CALL  MEM_SwapInHTML
   LD    HL,(Window_Addr)
   LD    A,(HL)
   LD    L,HTML_C_UseFonts		;Flag data is HTML/Use loaded fonts
   CP    SHTML_CHR			;Is first char start of HTML command?
   JR    Z,HTM_ShowFileDa		;Yes
   LD    L,HTML_C_PlainText		;Assume data is shown as text
HTM_ShowFileDa
   LD    A,L
   LD    (HTML_PreserveFlag),A		;Setup what sort of render to do
;
   BIT   7,A				;Non-HTML file?
   JR    NZ,HTM_ShowFileDe		;Yes (#label search not allowed)
;
   LD    A,(URL_LblType)
   CP    HTML_C_FindURL			;Need to find #label?
   CALL  Z,HTM_FindURLHlink		;Yes, Search for Label from hyperlink
;
HTM_ShowFileDe
;
   CALL  HTM_GetFrameAdr		;HTML_RCurrPos = start of frame data
   LD    A,(HTML_FCurrPosF)		;Get find frame ID
   LD    HL,HTML_FrameCurrID
   CP    (HL)				;Need to adjust start position?
   CALL  Z,HTM_ChkRAMLoc		;Yes (search for new start)
;
;  Display the contents of this frame
;
   CALL  HTM_FrmeIStrt			;Initialise vars for frame render.
;
   LD    HL,(HTML_RCurrPos)
HTM_ShowFileObjD0
   LD    A,(HTML_RCurrPos+2)
   CALL  HTM_NextHTMBank		;Swap In HTML Bank/adjust location
   LD    (HTML_RCurrPos+2),A		;Save updated bank index
   LD    (HTML_RCurrPos),HL		;Save new HTML bank location
;
   LD    A,(HTML_PreserveFlag)
   LD    C,A
   LD    A,(HL)				;Get current HTML char
   OR    A				;End of HTML data or frame?
   RET   Z				;Yes
;
   BIT   7,C				;Non-HTML file encountered?
   JR    NZ,HTM_ShowFileObjD1		;Yes (dont obey HTML cmds)
   CP    SHTML_CHR			;Start of HTML command ?
   JR    NZ,HTM_ShowFileObjD1		;No (display char)
;
   CALL  CPCB_ShowTimer			;Display timer icon
;
   CALL  HTM_DoHTMLCmd			;execute handler for HTML cmd
   JR    HTM_ShowFileObjD2		;continue with page render
;
HTM_ShowFileObjD1
   LD    B,A
   RES   7,C
   CALL  HTM_ShowChar			;show char in the web-page area
;
HTM_ShowFileObjD2
   LD    HL,(HTML_RCurrPos)
   INC   HL				;Next byte of RAM bank data
   LD    A,(HTML_DispStatus)
   AND   B_6				;past bottom of CPC screen?
   JR    Z,HTM_ShowFileObjD0		;No (continue to render)
   RET
;=========================================================================
;Name     HTM_ShowFileObject
;Purpose  To render the frames object.
;In Regs  A = ??
;         B = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RCurrPos = Rendering position
;         HTML_PreserveFlag = flags/settings
;         DefFont= default font to start rendering.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_RCurrPos = Last rendering position
;         HTML_RenderState = updated.
;Uses     TBD
;Call By  TBD
;Calls    HTM_NextHTMBank - Swap in HTML memory bank
;         HTM_DoHTMLCmd   - Execute a HTML command
;         HTM_ShowChar    - Show a rendering character
;Notes    1. Where there are no frames this routine display the current
;            HTML.
;=========================================================================
HTM_ShowFileObject
   LD    HL,WindowFrmeStat
   BIT   0,(HL)				;Specific frame refresh?
   JR    Z,HTM_ShowFileObject1		;No (all frames refresh)
   LD    A,(Window_FrmeSF)
   BIT   0,A				;This frame is to be refreshed?
   RET   Z				;No
   RES   0,(HL)				;Clear refresh request
   SET   7,(HL)				;Flag refresh has been done.
HTM_ShowFileObject1
   LD    A,(HTML_FrameCurrID)
   CALL  HTM_FrmeCalcAddr
   INC   HL				;Point to HTML_FramSF
   RES   0,(HL)				;clear refresh flag for this frame
   CALL  HTM_ShowFileObjectData
   LD    A,(HTML_FrameFlg)
   BIT   5,A				;Are frames to be loaded?
   CALL  Z,HTM_ShwFrameBorder		;No, show frame's border
   RET
;=========================================================================
;Name     htm_rom_init
;Purpose  To initialise the HTML rom.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_CmdStates = contains the command states for all HTML
;                          commands.
;Uses     TBD
;Call By  TBD
;Calls    HTM_GetFrame
;         HTM_GetFrameAdr
;         HTM_FrmeIStrt
;Notes    1. No check if made of the number of commands will exceed the
;            allowed number built into the program.
;=========================================================================
htm_rom_init
   LD    A,255
   LD    (HTML_NumStyles),A		;no styles info.
   INC   A
;
;  Clear the ROM variables
;
   LD    HL,HTML_StartVars
   LD    BC,HTML_EndVars-HTML_StartVars-1
   CALL  HTM_ResetMem
;
;  Reset the indices to frame rows/columns
;
   LD    HL,HTML_C_FrameRows
   LD    (HTML_FrmeSetR),HL		;Put into script for user define
   CALL  HTM_ResetRC
;
   LD    HL,HTML_C_FrameCols
   LD    (HTML_FrmeSetC),HL		;Put into script for user define
   CALL  HTM_ResetRC
;
   CALL  Mem_SwapInCFG
   LD    A,HTML_P_EODB
   LD    (CFG_FormsDB),A		;flag forms gone.
;
   LD    HL,0
   LD    (CFG_StylesDB),HL		;clear DB
;
   LD    HL,PageLock
   SET   1,(HL)				;asuume page scroll lock
;
;  Download Command Status/Flag bytes
;  Note ensure there is enough space for all commands.
;
   LD    HL,HTML_Commands
   LD    DE,HTML_CmdStates
htm_rom_init1
   LD    A,(HL)				;Get command byte
   OR    A				;At end of commands?
   RET   Z				;Yes
   BIT   7,A				;At command state byte?
   JR    Z,htm_rom_init2		;No
   LD    (DE),A				;copy out command state
   INC   DE				;next state byte
   INC   HL				;point to start of next command
   INC   HL
   INC   HL
htm_rom_init2
   INC   HL
   JR    htm_rom_init1			;check next command byte
;=========================================================================
;Name     HTM_ViewSrce
;Purpose  To view the source of the loaded HTML file.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  WindowFrmeNum = frame number of clicked item.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    HTM_GetFrameH
;         HTM_NextHTMBank
;         KM_TESTKEY
;         TXT_SET_CURSOR (OS)
;         TXT_WR_CHAR (OS)
;         TXT_CLR_WIN (OS)
;Notes    TBD
;=========================================================================
HTM_ViewSrce
   LD    A,(WindowFrmeNum)
   CALL  HTM_GetFrameH
   LD    B,A			;Start RAM bank
   LD    A,(Scrn_MaxCol)
   DEC   A
   LD    C,A
HTM_VwSrce3
   LD    DE,&0101		;Start column/row on screen
HTM_VwSrce0
   LD    A,B
   CALL  HTM_NextHTMBank	;Swap In HTML Bank/adjust location
   LD    B,A
   LD    A,(HL)
   AND   &7f			;Ensure below 128
   OR    A			;End of HTML data?
   JR    Z,HTM_VwSrce2
   INC   HL			;Next byte of RAM bank data
   CP    10			;CR/LF?
   JR    Z,HTM_VwSrce0a		;Yes
   CP    32			;Is char viewable?
   JR    C,HTM_VwSrce0		;No
   PUSH  HL
   PUSH  DE
   PUSH  BC
   PUSH  AF
   EX    DE,HL			;H = Column, L = Row
   CALL  TXT_SET_CURSOR		;Locate cursor on stream
   POP   AF
   CALL  TXT_WR_CHAR		;Display char on screen
   POP   BC
   POP   DE
   POP   HL
   INC   D			;Next column
   CP    C			;at limit of screen width?
   JR    NZ,HTM_VwSrce0		;No
HTM_VwSrce0a
   LD    D,1			;Reset column
   INC   E			;Next Row
   LD    A,E
   CP    MaxWebPgRow 		;At max row of screen?
   JR    NZ,HTM_VwSrce0		;No (show more source)
;
HTM_VwSrce2
   PUSH  HL
   PUSH  DE
   PUSH  BC
HTM_VwSrce2a
   LD    A,KEY_ESC
   CALL  KM_TESTKEY		;Escape key pressed?
   JR    NZ,HTM_VwSrce4		;Yes
   LD    A,KEY_SPACE
   CALL  KM_TESTKEY		;space pressed?
   JR    Z,HTM_VwSrce2a		;No
   CALL  TXT_CLR_WIN		;clear window for new data
   POP   BC
   POP   DE
   POP   HL
   LD    A,(HL)			;get HTML character
   OR    A			;End of HTML
   JR    NZ,HTM_VwSrce3		;No, display more text
   RET
HTM_VwSrce4
   POP   AF
   POP   AF
   POP   AF
   RET
;=========================================================================
;Name     HTM_DivSpan
;Purpose  To process the parameters to the DIV and SPAN commands.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_CmdTemp = location of keyboard buffer
;Out Regs A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_ParamLoc = corrupted
;Uses     TBD
;Call By  TBD
;Calls    HTM_GetCmdParam
;         HTM_FindParmLbl
;Notes    TBD
;=========================================================================
HTM_DivSpan
   LD    HL,(HTML_CmdTemp)
   LD    (HTML_ParamLoc),HL
HTM_DivSpan1
   CALL  HTM_GetCmdParam	;Locate next parameter
   LD    A,(HL)
   CP    '>'			;end of parameters?
   RET   Z			;Yes
;
   PUSH  HL
   CALL  HTM_FindParmLbl	;Determine the parameter nametype
   POP   HL
   LD    A,HTM_P_class
   CP    B			;Is parameter 'class' ?
   JR    Z,HTM_DivSpan2		;Yes
;
   LD    A,HTM_P_style
   CP    B			;Is parameter 'style' ?
   JR    NZ,HTM_DivSpan1	;No (skip the parameter)
;
;  setup format according to the style parameters
;
   RET
HTM_DivSpan2
;
;  Lookup up class in style db for format adjustments
;
   RET
;=========================================================================
;Name     HTM_CalcAppDBaddr
;Purpose  To calculate the address of an App db.
;In Regs  AF = db index
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  App_DBIndx
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = Address in the DB.
;         IX = ??
;         IY = ??
;Out Vars NOne
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_CalcAppDBaddr
   LD    BC,S_AppsDBSz
   LD    HL,CFG_AppsDB
HTM_CalcAppDBaddr1
   OR    A
   RET   Z
   ADD   HL,BC
   DEC   A
   JR    HTM_CalcAppDBaddr1
;=========================================================================
;Name     HTM_GetAppDBObj
;Purpose  To get the Applications DB entry for app startup.
;In Regs  AF = App Db index
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars App_Object = updated with DB entry
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_GetAppDBObj
   LD    BC,S_AppEntSize
   LD    HL,SysBnkBuffer	;Start of Apps database
HTM_GetAppDBOb1
   OR    A
   JR    Z,HTM_GetAppDBOb2
   ADD   HL,BC
   DEC   A
   JR    HTM_GetAppDBOb1
HTM_GetAppDBOb2
   LD    DE,App_Object
   LDIR
   RET
;=========================================================================
;Name     HTM_AddDBAppLocn
;Purpose  To add a new App HTML location to the DB of App Icons.
;In Regs  AF = Bank to add hyperlink into.
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  App_DBIndx = last index into the DB of Apps
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars App_DBIndx = updated last index into the DB
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_AddDBAppLocn
   LD    A,(App_DBIndx)
   CP    CFG_MAXAPPSDB
   RET   Z
   CALL  HTM_CalcAppDBaddr
   CALL  MEM_SwapInCFG
   EX    DE,HL
   LD    HL,App_DBTmp
   LDIR					;save new link to DB
   LD    HL,App_DBIndx
   INC   (HL)
   RET
;=========================================================================
;Name     HTM_CreateAppDb
;Purpose  To create the application DB entry.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  WebPge_Row = web page row
;         WebPge_Col = web page column
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars App_DBTmp = App DB entry setup.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_CreateAppDb
   XOR   A
   LD    (App_DBTmp),A			;Clear flags
   CALL  HTM_GetWebPgeCR
   LD    (App_DBTmp+S_AppSRow),HL	;setup row
   LD    (App_DBTmp+S_AppSCol),DE	;setup column
   LD    HL,HTML_CmdStart
   LD    DE,App_DBTmp+S_AppsHTM
   LD    BC,3
   LDIR					;setup HTML location.
   RET
;=========================================================================
;Name     HTM_ProcAppCmd
;Purpose  To process the parameters to the Applet command
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  KB_Buffer has command
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_ProcAppCmd
   LD    HL,0
   LD    (App_ObjName),HL		;no referenced object for app
;
   LD    HL,App_Object
   LD    B,32
HTM_ProcAppCmd0
   LD    (HL),SPACE_CHAR
   INC   HL
   DJNZ  HTM_ProcAppCmd0
;
   LD    HL,(HTML_CmdTemp)
   LD    (HTML_ParamLoc),HL
HTM_ProcAppCmd3
   CALL  HTM_GetCmdParam		;Locate next parameter
   LD    A,(HL)
   CP    EHTML_CHR			;no more parameters?
   RET   Z				;Yes (get table info)
   PUSH  HL
   CALL  HTM_FindParmLbl		;Determine the parameter nametype
   POP   HL
   LD    A,HTM_P_href
   CP    B				;Is parameter 'href' ?
   JR    Z,HTM_ProcAppCmd6
   LD    A,HTM_P_id
   CP    B				;Is parameter 'id' ?
   JR    Z,HTM_ProcAppCmd4
   LD    A,HTM_P_name
   CP    B				;Is parameter 'name' ?
   JR    NZ,HTM_ProcAppCmd3
;
;  Setup application filename
;
   CALL  HTM_GetCmdParmVal		;Get address of parameter value
   LD    DE,App_FileName
   LD    B,A_MaxAppFileName
   JR    HTM_ProcAppCmd4a
;
;  Setup application name
;
HTM_ProcAppCmd4
   CALL  HTM_GetCmdParmVal		;Get address of parameter value
   LD    DE,App_Name
   LD    B,A_MaxAppName
HTM_ProcAppCmd4a
   LD    C,DBLEQ_CHR
   CALL  UTL_strnccpy
   LD    A,SPACE_CHAR
   LD    (DE),A
   JR    HTM_ProcAppCmd3
;
;  Setup the location of the executable to download from
;  For applet usage within <BODY> of html the location is of the
;  applciation object to handle.
;
HTM_ProcAppCmd6
   CALL  HTM_GetCmdParmVal
   LD    (App_ObjName),HL		;save location of URL to load object
   JR    HTM_ProcAppCmd3
;=========================================================================
;Name     HTM_FindAppIcon
;Purpose  To search the DB of Apps for icon clicked on.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  App_DBIndx = last index into the DB of Apps
;         WebPge_Row = web page row where clicked on
;         WebPge_Col = web page column where clicked on
;Out Regs AF = Index into the DB or 255 (means not found)
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_FindAppIcon
   CALL  MEM_SwapInCFG
   LD    A,(App_DBIndx)
;
HTM_FindAppIcn0
   PUSH  AF
   CALL  HTM_CalcAppDBaddr
   LD    DE,App_DBTmp
   LDIR
   POP   AF
;
   LD    HL,(App_DBTmp+S_AppSRow)
   LD    DE,(WebPge_Row)
   OR    A
   SBC   HL,DE				;At same row?
   JR    NZ,HTM_FindAppIcn1		;No
   LD    HL,(App_DBTmp+S_AppSCol)
   LD    DE,(WebPge_Col)
   OR    A
   SBC   HL,DE				;At same column?
   JR    Z,HTM_FindAppIcn4		;Yes (found matching location)
HTM_FindAppIcn1
   DEC   A
   CP    255
   JR    NZ,HTM_FindAppIcn0
   RET
;
HTM_FindAppIcn4
;
;  Found matching location
;
   LD    DE,(HTML_CmdTemp)
   LD    HL,(App_DBTmp+S_AppsHTM)
   LD    A,(App_DBTmp+S_AppsHTM+2)
   CALL  Mem_SwapInHTML
   CALL  HTM_CpyCmd			;get HTML command
   CALL  HTM_ProcAppCmd			;process parameters
   CALL  CPCB_FindApp
   OR    A				;App exists in DB?
   JR    NZ,HTM_FindAppIcn3		;Yes
   DEC   A
   RET
;
HTM_FindAppIcn3
   LD    C,DBLEQ_CHR
   LD    HL,(App_ObjName)
   LD    A,H
   OR    L
   JR    NZ,HTM_FindAppIcn5
;
   CALL  Mem_SwapInHTML
   LD    A,(App_DBTmp+S_AppsHTM+2)
   LD    B,A
   LD    HL,(App_DBTmp+S_AppsHTM)
   CALL  HTM_FindCmdEnd
   INC   HL
   LD    C,SHTML_CHR
HTM_FindAppIcn5
   LD    DE,MEMORY_BANK2
   CALL  HTM_strccpy			;setup copy out parameter
   XOR   A
   LD    (DE),A
;
   LD    A,(App_State)
   LD    B,A
   SET   0,B				;Assume parameter supplied
   LD    HL,MEMORY_BANK2
   LD    A,(HL)				;Get first char copied
   OR    A				;start of HTML command?
   JR    NZ,HTM_FindAppIcn2		;Yes (no following data)
   LD    HL,0
   RES   0,B				;no parameter supplied
HTM_FindAppIcn2
   LD    A,B
   LD    (App_State),A
   LD    (App_ObjName),HL
   XOR   A
   RET
;=========================================================================
;Name     HTM_NetErrMsg
;Purpose  To display the general network error message.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_NetErrMsg
   LD    HL,Netwrk_Msg
   JP    CPCB_ShwErrorMsg
;