;=========================================================================
;
; Application Routines (Other)
; Copyright (c) 2010 Ray Palmer
;
;=========================================================================
;Name     HTM_GetAlignType
;Purpose  To determine the alignment type
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_ParamVLoc = location of parameter value text
;Out Regs AF = corrupted
;          B = ??
;          C = alignment type
;              1=left, 2=right, 3=center, 4=justify, 5=top, 6=bottom
;              0=invalid
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    REWRITE FOR FASTER SMALLER ROUTINE
;=========================================================================
HTM_GetAlignType
   LD    HL,(HTML_ParamVLoc)
   CALL  HTM_FindParmLbl
   LD    A,HTM_P_AlignL
   LD    C,1				;align left (c=1)
   CP    B
   RET   Z
   LD    A,HTM_P_AlignR
   INC   C				;align right (c=2)
   CP    B
   RET   Z
   LD    A,HTM_P_AlignC
   INC   C				;align center (c=3)
   CP    B
   RET   Z
   LD    A,HTM_P_AlignJ
   INC   C				;align justify (c=4)
   CP    B
   RET   Z
   LD    A,HTM_P_AlignT
   INC   C				;align top (c=5)
   CP    B
   RET   Z
   LD    A,HTM_P_AlignB
   INC   C				;align bottom (c=6)
   CP    B
   RET   Z
   LD    C,HTM_ALIGNUNK
   RET
;=========================================================================
;Name     HTM_ChkListTyp
;Purpose  To check the list type for what type of order is to be show
;         for various lists.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_ListTyp = type of list to handle
;         HTML_ListNum = list item number for ordered lists
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_ListNum = updated list item number for ordered lists
;Uses     TBD
;Call By  TBD
;Calls    UTL_writedec
;         HTM_ShowChar
;Notes    TBD
;=========================================================================
HTM_ChkListTyp
   LD    B,ICON_BIGDOT+128
   SET   1,C				;Flag char is above 127
   LD    A,(HTML_ListTyp)
   BIT   0,A				;Is list type unordered?
   JP    NZ,HTM_ShowChar		;Yes, Display Big Dot char
;
   LD    A,(HTML_ListNum)
   INC   A
   LD    (HTML_ListNum),A
   LD    HL,HTML_TmpStr
   CALL  UTL_writedec			;convert number to text
;
   LD    (HL),0				;end converted string
   LD    A,(HTML_TmpStr)
   LD    B,A
   LD    C,0
   OR    A
   CALL  NZ,HTM_ShowChar		;display MSB if non-zero
;
   LD    A,(HTML_TmpStr+1)
   LD    B,A
   LD    C,0
   LD    A,(HTML_TmpStr)
   OR    B				;Is digit non-zero?
   CALL  NZ,HTM_ShowChar		;Display next digit
;
   LD    C,0
   LD    A,(HTML_TmpStr+2)
   LD    B,A
   JP    HTM_ShowChar			;Display LSB
;=========================================================================
;Name     HTM_SetNextRow
;Purpose  To setup the web-page render with the next row.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_BR = start column for next row.
;         HTML_Row = current web-page row.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = Corrupted
;         IX = ??
;         IY = ??
;Out Vars HTML_Row = next row
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
HTM_SetNextRow
   LD    HL,(HTML_Row)
   INC   HL
   LD    (HTML_Row),HL
   RET
;=========================================================================
;Name     HTM_NewLine
;Purpose  To setup the web-page render with a new line.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_BR = start column for next row.
;         HTML_Row = current web-page row.
;         HTML_BR_Table = offset within table.
;         HTML_BR_FormsT = offset within forms input item.
;         HTML_BR_List = offset for list items.
;Out Regs A = ??
;         BC = ??
;         DE = HTML_BR_List indent
;         HL = new column start
;         IX = ??
;         IY = ??
;Out Vars HTML_Row = next row
;         HTML_Col = start column
;Uses     A, DE,HL registers
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
HTM_NewLine
   LD    DE,(HTML_Row)
   INC   DE
   LD    (HTML_Row),DE
;
   LD    HL,HTML_BR_Flag
   BIT   1,(HL)			;Is new line to wrap on object?
   JR    Z,HTM_NewLine1		;No
;
   LD    HL,(HTML_ImgRow)
   OR    A
   SBC   HL,DE			;Is HTML_Row before HTML_ImgRow?
   LD    HL,(HTML_ImgCol)	;Start at right margin of obect.
   JR    NC,HTM_NewLine2	;Yes (cannot start within object)
;
   LD    HL,HTML_BR_Flag
   RES   1,(HL)			;flag not along object
;
HTM_NewLine1
   LD    HL,(HTML_BR)		;Get base start column
;
   LD    DE,(HTML_BR_Table)
   ADD   HL,DE			;add in table cell offset
;
   LD    DE,(HTML_BR_FormsT)
   ADD   HL,DE			;add in <P> item offset
;
   LD    DE,(HTML_BR_List)
   ADD   HL,DE			;add in list items offset
;
   LD    DE,(HTML_BR_DL)
   ADD   HL,DE			;add in def list items offset
;
HTM_NewLine2
   LD    (HTML_Col),HL		;save new start column
   CALL  HTM_CheckOnScrn	;CHECK WHY DO THIS ????
   LD    A,(HTML_DispStatus)
   AND   &F0			;Is position on screen?
   RET   Z			;Yes
   LD    HL,Window_FrmeID
   SET   6,(HL)			;flag need scrolling frame
   RET
;=========================================================================
;Name     HTM_ObjectBrdr
;Purpose  To display the border around an object.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_BordrSt = border processing status
;         HTML_Row = current web-page row
;         HTML_Col = current web-page column
;         HTML_MaxCol = base column for border calcs
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_P_SCol = start column for border
;         HTML_P_SRow = start row for border
;         HTML_P_ECol = end column for border
;         HTML_P_ERow = end row for border
;         HTML_Col = column for bordered content
;         HTML_Row = row for border content.
;         HTML_MaxCol = base column for border calcs
;         HTML_BR_FormsT = <BR> offset for new line
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    1. The var HTML_BordrSt is configured to control how border
;            processing is handled.
;            a. where Bit 6 = 1 then border processing is enabled
;            b. Where Bit 5 = 0 then the start of processing requested.
;            c. Where Bit 5 = 1 then the end processing is required.
;=========================================================================
HTM_ObjectBrdr
   LD    HL,HTML_BordrSt
   BIT   6,(HL)			;start of border processing?
   RET   Z			;No
;
   BIT   5,(HL)			;processing end of border?
   JR    NZ,HTM_ObjectBrdr1	;Yes
;
   SET   7,(HL)			;Flag border underway
   SET   5,(HL)
   LD    HL,(HTML_Col)
   LD    (HTML_P_SCol),HL
   INC   HL			;indent 1 column for content
   LD    (HTML_Col),HL
;
   LD    (HTML_MaxCol),HL
   LD    (HTML_BR_FormsT),HL	;setup <BR> handling.
;
   LD    HL,(HTML_Row)
   LD    (HTML_P_SRow),HL
   INC   HL			;indent 1 row for content
   LD    (HTML_Row),HL
   RET
;
HTM_ObjectBrdr1
   RES   5,(HL)
   RES   6,(HL)
   RES   7,(HL)			;Flag border complete
;
;  Display borders for the object
;
   CALL  HTM_ChkMaxCol
   LD    HL,(HTML_MaxCol)
   INC   HL			;account for border
   LD    (HTML_P_ECol),HL
;
   LD    HL,(HTML_Row)
   INC   HL
   INC   HL			;account for row 0 being row 1
   LD    (HTML_P_ERow),HL
;
   LD    HL,0
   LD    (HTML_BR_FormsT),HL	;reset <BR> handling.
   LD    (HTML_MaxCol),HL
;
   CALL  HTM_Calc_P_CPC		;Calc boundary for <P> content.
   CALL  CPCB_ShowBorder	;display the border
   JP    HTM_CalcFrameCPC	;restore window frame values.