;=========================================================================
;
; Application Routines (STR)
; Copyright (c) 2010 Ray Palmer
;
;=========================================================================
;Name     HTM_strccpy
;Purpose  To copy characters until a specific character is encountered
;In Regs  A = ??
;         B = HTML Bank index
;         C = character to stop at
;         DE = destination location for data
;         HL = HTML address of source text
;         IX = ???
;         IY = ??
;In Vars  None.
;Out Regs AF = corrupted.
;         B = ??
;         C = Unchanged.
;         DE = corrupted.
;         HL = corrupted.
;         IX = ??
;         IY = ??
;Out Vars None.
;Uses     TBD
;Call By  TBD
;Calls    None.
;Notes    1 Assume the banked memory is already swapped in.
;=========================================================================
HTM_strccpy1
   LD    A,(HL)
   LD    (DE),A
   CP    C
   RET   Z
   INC   DE
   INC   HL
HTM_strccpy
   BIT   7,H
   JR    Z,HTM_strccpy1
;
   LD    HL,MEMORY_SWAP_ST
   INC   B
   LD    A,B
   PUSH  BC
   CALL  MEM_SwapInHTML
   POP   BC
   JR    HTM_strccpy
;=========================================================================
;Name     HTM_strncmp
;Purpose  To perform a case insensitive compare of a n byte string with
;         another.
;In Regs  A = ??
;         B = length of string to compare
;         C = current value
;         DE = address of string to compare.
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_ParamLoc = location of text to compare with.
;Out Regs AF = Flags updated (Z if match else NZ)
;         B = 0 if all chars matched
;         C = C + 1
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    UTL_tolower
;Notes    TBD
;=========================================================================
HTM_strncmp
   INC   C
   LD    HL,(HTML_ParamLoc)
;=========================================================================
;Name     UTL_strncmp2
;Purpose  To compare two strings of n characters (case insensitive)
;In Regs  A = ??
;         B = number of characters
;         C = ??
;         DE = address of string 1 (must be lower case)
;         HL = address of string 2 (can be mixed case)
;         IX = ???
;         IY = ??
;In Vars  HTML_ObjLocn = location of object to compare
;Out Regs A = corrupted
;         B = 0 if both n characters the same.
;         C = preserved
;         DE = corrupted
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_ObjLocn = updated location of object to compare
;Uses     TBD
;Call By  TBD
;Calls    UTL_tolower
;Notes    None.
;=========================================================================
UTL_strncmp2
   PUSH  HL
   PUSH  BC
UTL_strncmp2a
   LD    A,(DE)			;Get internal char to compare against
   CALL  UTL_tolower		;force to lower for comparison
   LD    C,A
   LD    A,(HL)			;Get external char
   CALL  UTL_tolower		;force to lower for comparison
   CP    C			;Is it same?
   JR    NZ,UTL_strncmp2b	;No
   INC   HL
   CALL  HTM_AdjustIndex
   INC   DE
   DJNZ  UTL_strncmp2a
UTL_strncmp2b
   LD    A,B
   POP   BC
   LD    B,A			;return result
   POP   HL
   RET
;=========================================================================
;Name     UTL_strlen2
;Purpose  To get the length of a HTML button text string.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = address of string
;         IX = ???
;         IY = ??
;In Vars  N/A
;Out Regs A = string length
;         BC = ??
;         DE = ??
;         HL = address of string
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    None.
;Notes    1. This function returns the length of text following a HTML
;            command so that the render can determine widths of text.
;         2. No bank switching employed.
;=========================================================================
UTL_strlen2
   push  bc
   push  hl
   LD    B,0
UTL_strlen2a
   LD    A,(HL)
   CP    SHTML_CHR
   JR    Z,UTL_strlen2b
   INC   B
   INC   HL
   JR    UTL_strlen2a
UTL_strlen2b
   LD    A,B
   pop   hl
   pop   bc
   RET
;=====================================================================
;Name     HTM_strlen3
;Purpose  To calculate the length of the text following a HTML
;         command
;Input    HL = HTML address
;         A = HTML bank index
;Output   B = length
;Uses     Flags corrupted.
;Call By  TBD
;Calls    HTM_NextHTMBank
;Notes    1. This function returns the length of text following a HTML
;            command so that the render can determine widths of text.
;         2. Bank switching employed.
;         3. If the text length exceed 255 then the length is trimmed
;            by 255.
;=====================================================================
HTM_strlen3
   LD    C,A
   LD    B,0
HTM_strlen3a
   LD    A,(HL)
   OR    A
   RET   Z
   CP    SHTML_CHR
   RET   Z
   INC   HL
   INC   B
   LD    A,C
   CALL  HTM_NextHTMBank
   JR    HTM_strlen3a
