;=========================================================================
;
; Application Routines (Style handling routines)
; Copyright (c) 2010 Ray Palmer
;
; Notes
;
;=========================================================================
;Name     STY_PCmdParams
;Purpose  To process the HTML command setting when styles are actioned for
;         borders.
;In Regs  A = database index
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_StyleStatus = styles status flags.
;Out Regs A = ??
;         B = Style Parameter number by type
;         C = ??
;         DE = ??
;         HL = address of parameter text (if any)
;         IX = ??
;         IY = ??
;Out Vars HTML_BordrSt = Border status
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
STY_PCmdParams
   LD    A,(HTML_StyleCmd)
   LD    HL,HTML_BordrSt
   RES   2,(HL)
   CP    82			;<P> command?
   JR    NZ,STY_PCmdParams1	;No
   SET   2,(HL)			;Flag border on the <P> command
;
STY_PCmdParams1
   RES   0,(HL)
   CP    69			;<FORM> command?
   JR    NZ,STY_PCmdParams2	;No
   SET   0,(HL)			;Flag border on the <FORM> command
STY_PCmdParams2
;
   CALL  HTM_StyleParam
   PUSH  HL
   CALL  HTM_FindParmLbl	;Determine the parameter nametype
   POP   HL
   RET
;=========================================================================
;Name     HTM_StyleParam
;Purpose  To point to the parameter of the style command
;In Regs  A = database index
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  SysGenBuffer = address of style command and parameters.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = address of style command parameter value
;         IX = ??
;         IY = ??
;Out Vars ??
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_StyleParam
   LD    HL,SysGenBuffer
HTM_StyleParm0
   LD    A,(HL)
   INC   HL
   CP    COLON_CHR		;start of value?
   JR    NZ,HTM_StyleParm0	;No
   JP    UTL_skipspace		;skip any spaces to the value
;=========================================================================
;Name     HTM_CalcStAddr
;Purpose  To calc styles db address of HTML <STYLE> commands.
;In Regs  A = database index
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_CmdStart = address of style command
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = address in styles DB.
;         IX = ??
;         IY = ??
;Out Vars ??
;Uses     TBD
;Call By  TBD
;Calls    MEM_SwapInCFG
;Notes    1. The configuration bank is left swapped in.
;         2. No check is made to ensure DB not exceeded.
;=========================================================================
HTM_CalcStAddr
   LD    HL,CFG_StylesDB
   LD    BC,S_StyleSz
HTM_CalcStAddr1
   OR    A
   JP    Z,MEM_SwapInCFG	;swap in cfg bank
   ADD   HL,BC
   DEC   A
   JR    HTM_CalcStAddr1
;=========================================================================
;Name     HTM_AddStyles2DB
;Purpose  To add a new entry to the styles database.
;In Regs  A = ??
;         BC = ??
;         DE = address of style command data
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_NumStyles = number of styles loaded
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_NumStyles = updated number of styles loaded
;Uses     TBD
;Call By  TBD
;Calls    MEM_SwapInMain
;Notes    None
;=========================================================================
HTM_AddStyles2DB
   LD    A,(HTML_NumStyles)
   CP    MAX_STYLES		;At max DB capacity?
   RET   Z			;Yes
;
   CALL  MEM_SwapInCFG
;
   EX    DE,HL
   LD    E,(HL)
   INC   HL
   LD    D,(HL)
   INC   HL
   LD    B,(HL)
;
   LD    HL,CFG_StylesDB
HTM_AddStyl2DB0
   PUSH  HL
   LD    C,(HL)
   INC   HL
   LD    A,(HL)
   OR    C			;at end of Db entries
   JR    Z,HTM_AddStyl2DB2	;Yes
;
   LD    A,E			;compare LSB of address
   CP    C
   JR    NZ,HTM_AddStyl2DB1
   LD    A,D			;compare MSB of address
   CP    (HL)
   JR    NZ,HTM_AddStyl2DB1
   INC   HL
   LD    A,B
   CP    (HL)			;compare bank index
   JR    NZ,HTM_AddStyl2DB1
   POP   HL
   RET
;
HTM_AddStyl2DB1
   POP   HL
   INC   HL
   INC   HL
   INC   HL
   JR    HTM_AddStyl2DB0
;
HTM_AddStyl2DB2
   POP   HL
   LD    (HL),E
   INC   HL
   LD    (HL),D
   INC   HL
   LD    (HL),B
   INC   HL
   LD    (HL),0			;add end of DB marker
   INC   HL
   LD    (HL),0
;
   LD    HL,HTML_NumStyles
   INC   (HL)			;increment number of <style> commands
   JP    MEM_SwapInMain		;restore memory configuration
;=========================================================================
;Name     HTM_PopStyle
;Purpose  To get the entry from the Top of the styles stack.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_stylSetDBIndx = current TOS index
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_stylSetDBIndx = updated TOS index
;Uses     TBD
;Call By  TBD
;Calls    Mem_SwapInCFG
;         CPCB_GetPrevFont
;Notes    TBD
;=========================================================================
HTM_PopStyle
   LD    HL,(HTML_stylSetDBIndx)	;Current top of stack index
   LD    BC,S_StyleSetSz
   SBC   HL,BC				;Point to start of db entry
   BIT   7,H				;before start of DB?
   RET   NZ				;Yes
   LD    (HTML_stylSetDBIndx),HL
;
   CALL  Mem_SwapInCFG
   LD    DE,CFG_StylesSetDB
   ADD   HL,DE				;calc address of Db entry
   LD    DE,HTML_StyleSetting
   LDIR					;download settings
   JP    Mem_SwapInMain
;=========================================================================
;Name     HTM_PushStyle
;Purpose  To save the style settings object onto the stack.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_stylSetDBIndx = index offset onto stack
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_stylSetDBIndx = updated index offset onto stack
;Uses     TBD
;Call By  TBD
;Calls    Mem_SwapInCFG
;         MEM_SwapInMain
;Notes    TBD
;=========================================================================
HTM_PushStyle
   CALL  Mem_SwapInCFG
   LD    DE,(HTML_stylSetDBIndx)	;Stack index
   LD    HL,CFG_StylesSetDB
   ADD   HL,DE				;calc address in stack
   EX    DE,HL
   LD    HL,HTML_StyleSetting
   LD    BC,S_StyleSetSz
   LDIR					;insert new TOS entry
;
   LD    HL,(HTML_stylSetDBIndx)
   LD    BC,S_StyleSetSz
   ADD   HL,BC
   LD    (HTML_stylSetDBIndx),HL	;setup new stack index
   JP    MEM_SwapInMain
;=========================================================================
;Name     HTM_GetStyleName
;Purpose  To get the style command id/class name
;In Regs  A = Parameter type
;         BC = ??
;         DE = ??
;         HL = address of style name in the KB_Buffer
;         IX = ??
;         IY = ??
;In Vars  KB_Buffer has HTML command to get data from.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_ObjTemp = style name from HTML command
;Uses     TBD
;Call By  TBD
;Calls    UTL_strccpy
;Notes    1. The data to get the name is from the KB_Buffer which
;            contains the current HTML command.
;         2. The format of the text to match is as follows -
;            xxxxxxdyyyyyyyy
;            where xxxxxx = HTML command
;                  d = style type (# or .)
;                  yyyyyyyy = parameter text from HTML command.
;
;            e.g a. H1dmyid (d = style type value)
;
;                b. DIVdH1.myclass (d = paremeter type value)
;
;            Note - example b is a badly formed style reference, but is
;                   handled.
;=========================================================================
HTM_GetStyleName
   LD    A,(HTML_ClassIDType)
   LD    DE,HTML_LblTemp	;where to save name
   LD    B,HASH_CHAR		;Assume name is for id type
   CP    HTM_P_id		;parameter is 'id=...'?
   JR    Z,HTM_GetStyleNme0
   LD    B,DOT_CHAR		;name is for class type
HTM_GetStyleNme0
   LD    A,B
   PUSH  AF
   PUSH  HL
   LD    HL,(HTML_CmdTemp)
HTM_GetStyleNme1
   LD    A,(HL)
   CP    SPACE_CHAR		;end of command name?
   JR    Z,HTM_GetStyleNme2	;yes now add type and name
   LD    (DE),A
   INC   HL
   INC   DE
   JR    HTM_GetStyleNme1
HTM_GetStyleNme2
   POP   HL
   POP   AF
   LD    (DE),A			;Insert style type (# or .)
   INC   DE
   LD    C,DBLEQ_CHR
   CALL  UTL_strccpy		;copy down style name from parameter
   XOR   A
   LD    (DE),A			;Add NUL to end string.
   RET
;=========================================================================
;Name     HTM_NextStyleID
;Purpose  Go to start of next def name/id in style data.
;Input    A = HTML bank index
;         B = ??
;         C = ??
;         DE = ??
;         HL = Address in HTML memory
;         IX = ??
;         IY = ??
;Output   A = 0 if found else not zero
;         B = ??
;         C = HTML bank index
;         DE = ??
;         HL = Address in HTML memory
;         IX = ??
;         IY = ??
;Uses     TBD
;Call By  HTM_FndHxStyleName
;Calls    Mem_SwapInHTML
;         HTM_NextHTMBank
;         HTM_SkipToRBrace
;Notes    1. The HTML bank is remains swapped in
;=========================================================================
HTM_NextStyleID
   CALL  HTM_NextHTMBank
   LD    C,A
   LD    A,(HL)
   CP    SHTML_CHR		;Start of next HTML command ?
   RET   Z			;yes
;
   OR    &80
   CP    &80			;End of all data?
   RET   Z			;yes
   AND   &7F
;
   CP    COMMA_CHR		;start of another name?
   JR    Z,HTM_NextStyleID1	;Yes
;
   CP    SSTYL_CHR		;Start of style command data
   CALL  Z,HTM_SkipToRBrace	;Yes, goto end of data
HTM_NextStyleID1
   INC   HL			;point to next character
;=========================================================================
;Name     HTM_SetHTMLBank
;Purpose  Go to next HTML data byte in the current swapped in HTML bank.
;Input    A = HTML bank index
;         B = ??
;         C = ??
;         DE = ??
;         HL = Address in HTML memory
;         IX = ??
;         IY = ??
;Output   A = 0
;         B = ??
;         C = HTML bank index
;         DE = ??
;         HL = Address in HTML memory
;         IX = ??
;         IY = ??
;Uses     TBD
;Call By  TBD
;Calls    HTM_NextHTMBank
;Notes    1. The HTML bank is remains swapped in
;=========================================================================
HTM_SetHTMLBank
   LD    A,C
   BIT   7,H
   CALL  NZ,HTM_NextHTMBank	;point to definition text (if any)
   LD    C,A
   XOR   A
   RET
;=====================================================================
;Name     HTM_GetStyleID
;Purpose  Go to start of def name/id and copy the text
;Input    AF = HTML bank index
;         BC = ??
;         DE = ??
;         HL = Address in HTML
;         IX = ??
;         IY = ??
;In Vars  HTML_StyleDbIdx = address of style Tag in HTML data
;         HTML_CmdTemp = address of text buffer
;Output   AF = 0 if name of style found else not zero
;         B = ??
;         C = HTML bank index
;         DE = ??
;         HL = Address in HTML
;         IX = ??
;         IY = ??
;Out Vars HTML_StyleDbIdx = address of style data to interpret.
;         HTML_ObjTemp = text buffer with style name.
;Uses     AF corrupted.
;Call By  HTM_FndHxStyleName
;Calls    HTM_NextHTMBank
;         UTL_tolower
;Notes    1. The HTML bank is already swapped in.
;         2. Assume that the style command is complete.
;         3. If more than 1 name is found then each is merged to form
;            the name to match against.
;=====================================================================
HTM_GetStyleID
   LD    C,A
   LD    B,HTML_ObjTempSz
   LD    DE,HTML_ObjTemp
HTM_GetStylID1
   XOR   A
   LD    (DE),A
   LD    A,(HL)
;
   OR    &80
   CP    &80			;End of all data?
   RET   Z			;yes
   AND   &7F
;
   CP    SHTML_CHR		;start of next HTML command
   RET   Z
;
   CP    SPACE_CHAR+1		;invalid char for name?
   JR    C,HTM_GetStylID3	;Yes (ignore it)
;
   CP    COMMA_CHR		;start of another name?
   JR    Z,HTM_GetStylID2	;Yes
;
   CP    SSTYL_CHR		;start of style definition?
   JR    Z,HTM_GetStylID2	;Yes
;
   CALL  UTL_tolower
   LD    (DE),A
   LD    A,B
   OR    A			;at max of text byffer for name?
   JR    Z,HTM_GetStylID3	;Yes
   DEC   B
   INC   DE
HTM_GetStylID3
   INC   HL
   LD    A,C
   CALL  HTM_SetHTMLBank
   JR    HTM_GetStylID1
;
HTM_GetStylID2
   XOR   A
   RET
;=========================================================================
;Name     HTM_GetStyleCmd
;Purpose  To get a style command and value
;In Regs  AF = HTML bank
;         BC = ??
;         DE = ??
;         HL = address of HTML data
;         IX = ??
;         IY = ??
;In Vars  None.
;Out Regs AF = ??
;         B = ??
;         C = last bank of HTMl with style command data
;         DE = ??
;         HL = address of byte after style command/value
;         IX = ??
;         IY = ??
;Out Vars SysGenBuffer = style command and value
;Uses     TBD
;Call By  HTM_DoStyleCmds
;Calls    HTM_NextHTMBank
;Notes    1. The HTML bank is already swapped in.
;=========================================================================
HTM_GetStyleCmd
   LD    C,A
   LD    DE,SysGenBuffer
   LD    B,SPACE_CHAR
   LD    A,HTML_MAXSTR
   LD    (HTML_ObjTempL),A
HTM_GetStyleCmd1
   LD    A,(HL)
   CP    ESTYL_CHR		;end of style command data?
   JR    Z,HTM_GetStyleCmd3	;Yes
;
   CP    DBLEQ_CHR		;end of style parameter data?
   JR    Z,HTM_GetStyleCmd3	;Yes
;
   INC   HL
;
   CP    B			;leading space?
   JR    Z,HTM_GetStyleCmd2	;Yes
   LD    B,0			;No
;
   CP    SPACE_CHAR		;below SPACE char ?
   JR    C,HTM_GetStyleCmd2	;Yes (ignore it)
;
   CP    DSTYL_CHR		;style command delimiter?
   JR    Z,HTM_GetStyleCmd3	;Yes
;
   CALL  UTL_tolower
   LD    (DE),A
   LD    A,(HTML_ObjTempL)
   OR    A
   JR    Z,HTM_GetStyleCmd2
   DEC   A
   LD    (HTML_ObjTempL),A
   INC   DE
;
HTM_GetStyleCmd2
   CALL  HTM_SetHTMLBank
   JR    HTM_GetStyleCmd1
;
;  Remove any trailing spaces.
;
HTM_GetStyleCmd3
   DEC   DE
   LD    A,(DE)
   CP    SPACE_CHAR		;Is trailing space in name?
   JR    Z,HTM_GetStyleCmd3	;Yes
   INC   DE
   XOR   A
   LD    (DE),A			;add end of text char
   RET
;=========================================================================
;Name     HTM_GetStyleInfo
;Purpose  To get information to determine what style, class or id is.
;In Regs  A = style type character
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  None
;Out Regs A = No change (F updated)
;             Z flag set if valid stype ID type else NZ
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars None
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_CheckStyleID
   CP    HASH_CHAR		;'#' (class type) ?
   RET   Z
   CP    DOT_CHAR		;'.' (id type) ?
   RET
;=========================================================================
;Name     HTM_Skip2StyleID
;Purpose  To skip the HTML command name to the style information.
;In Regs  A = style type character
;         BC = ??
;         DE = ??
;         HL = address of formatted style info from HTML command.
;         IX = ??
;         IY = ??
;In Vars  None
;Out Regs A = Corrupted
;             Z flag set if matching same type ID type.
;         BC = ??
;         DE = ??
;         HL = address of style name after the style ID type character.
;         IX = ??
;         IY = ??
;Out Vars None
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_Skip2StyleID
   LD    A,(HL)
   CALL  HTM_CheckStyleID	;'#' or '.' ?
   RET   Z			;Yes
   OR    A			;end of text?
   RET   Z			;Yes
   INC   HL
   JR    HTM_Skip2StyleID
;=========================================================================
;Name     HTM_GetStyleInfo
;Purpose  To get information to determine what style, class or id is.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_CmdStart = address of style command
;         HTML_NumStyles = number of styles Db entries
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_ParamLoc = corrupted.
;         HTML_StyleAddr = address of style commands to execute.
;                          or address of style name to lookup.
;         HTML_ClassIDType = flag indicating style or class or id parameter
;         HTML_LblTemp = text of class name.
;         HTML_LblTempL = length of class text
;Uses     TBD
;Call By  HTM_DoCmdHandler
;Calls    HTM_GetCmdParam
;         HTM_FindParmLbl
;         HTM_GetCmdParmVal
;         HTM_GetStyleName
;         Mem_SwapInCFG
;         UTL_GetVarWord
;         HTM_NextHTMBank
;         HTM_GetStyleID
;         HTM_NextStyleID
;Notes    1. The data to get the information is from the KB_Buffer which
;            contains the current HTML command.
;=========================================================================
HTM_GetStyleInfo
   XOR   A
   LD    (HTML_ClassIDType),A
   LD    HL,HTML_StyleFlgs
   RES   7,(HL)
;
   LD    HL,(HTML_CmdTemp)
   LD    (HTML_ParamLoc),HL
HTM_GetStyleInf0
   CALL  HTM_GetCmdParam	;Locate next parameter
   LD    A,(HL)
   CP    EHTML_CHR		;no more parameters?
   RET   Z			;Yes
   PUSH  HL
   CALL  HTM_FindParmLbl	;Determine the parameter nametype
   POP   HL
;
   LD    A,HTM_P_style
   CP    B			;Is parameter 'style' ?
   JR    Z,HTM_GetStyleInf1	;Yes
;
   LD    A,HTM_P_id
   CP    B			;Is parameter 'id' ?
   JR    Z,HTM_GetStyleInf1	;Yes (get name)
;
   LD    A,HTM_P_class
   CP    B			;Is parameter 'class' ?
   JR    NZ,HTM_GetStyleInf0	;No (continue to next parameter)
;
HTM_GetStyleInf1
   LD    A,B
   LD    (HTML_ClassIDType),A
;
   CALL  HTM_GetCmdParmVal	;HL = address of style name
   LD    (HTML_StyleAddr),HL	;save address of parameter data
   LD    A,(HTML_ClassIDType)
   CP    HTM_P_style		;"style" parameter?
   RET   Z			;Yes
;
   LD    A,(HTML_NumStyles)
   INC   A			;Any entries in the DB?
   JR    NZ,HTM_GSI5		;No
;
HTM_GSI_Ex
   XOR   A
   LD    (HTML_ClassIDType),A	;name not found
   RET
;
HTM_GSI5
   CALL  HTM_GetStyleName	;HTML_LblTemp = HTML style name
;
;  Search style HTML command for name
;
   LD    HL,CFG_StylesDB
HTM_GSILoop0
   CALL  Mem_SwapInCFG		;swap in configbank
   CALL  UTL_GetVarWord		;get Db address of HTML command
   LD    A,D			;end of DB?
   OR    E
   JR    Z,HTM_GSI_Ex		;Yes
   LD    C,(HL)			;Get HTML bank index
   INC   HL
   LD    (HTML_StyleDb),HL	;Save DB index.
   LD    A,C
   EX    DE,HL
   INC   HL			;Point to command content
   CALL  HTM_NextHTMBank	;Now swap in HTML bank
;
HTM_GSILoop1
   CALL  HTM_GetStyleID		;HTML_ObjTemp = style name from DB
   EX    DE,HL
   LD    HL,(HTML_StyleDb)
   OR    A			;DB style name got?
   JR    NZ,HTM_GSILoop0	;No, Goto next <style> cmd
;
   LD    A,C
   LD    (HTML_StyleDbIdx+2),A
   LD    (HTML_StyleDbIdx),DE	;Save addr after DB style name
;
;  compare style name with DB style name
;
   LD    HL,HTML_LblTemp	;HL = address of style name from HTML
   LD    DE,HTML_ObjTemp	;DE = address from styles DB
;
;  Get first character of the styles DB name
;
   LD    A,(DE)
   CALL  HTM_CheckStyleID	;db style name starts with # or .?
   JR    NZ,HTM_GSI1		;No (compare all text)
;
   LD    B,A
   CALL  HTM_Skip2StyleID	;Skip to HTML style name & chk id type
   CP    B			;same name type?
   JR    NZ,HTM_GSI2		;not same type id (i.e # <> .)
   LD    A,(HL)
   OR    A			;Any text to compare?
   JR    NZ,HTM_GSI1		;Yes
;
   LD    HL,(HTML_CmdTemp)	;HL = address of name from HTML
;
HTM_GSI1
;
;  Check if command name in Db matches name from HTML command parameter
;
   CALL  UTL_strlen
   LD    B,A
   CALL  UTL_strncmpCI		;strcmp (case in-sensitative)
   RET   Z			;Yes (both same)
;=========================================================================
;Name     HTM_GSI2
;Purpose  Re-entry point to handle cascading style sheets.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_StyleDbIdx = address of format definition.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_CmdTemp = Buffer of current HTML Command
;         HTML_StyleFlgs = bit 7 set to indicate style commands
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_GSI2
   LD    HL,(HTML_StyleDbIdx)	;Get address of HTML command
   LD    A,(HTML_StyleDbIdx+2)
   CALL  HTM_NextStyleID	;goto end of style data
   CP    SHTML_CHR		;Start of HTML command
   JR    NZ,HTM_GSILoop1	;No, do next DB style name
   JR    HTM_GSILoop0		;Continue with other <STYLE> commands
;=========================================================================
;Name     HTM_DoStyleCmds
;Purpose  To exceute the style available commands.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_StyleDbIdx = HTML address of format definition.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_CmdTemp = KB_Buffer data
;         HTML_StyleFlgs = bit 7 set to indicate style commands
;         SysGenBuffer = corrupted.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. Where the commands are in the style parameter the address is
;            in the KB_Buffer in the lower memory.
;=========================================================================
HTM_DoStyleCmds
   LD    HL,(HTML_StyleAddr)
   LD    C,0
   LD    A,(HTML_ClassIDType)
   CP    HTM_P_style			;Style parameter?
   JR    Z,HTM_DoStyleCmds0		;Yes
;
;  Point to start of the commands belonging to the matched style name
;
   LD    A,(HTML_StyleDbIdx+2)
   LD    HL,(HTML_StyleDbIdx)
   CALL  HTM_SkipToLBrace		;goto start of style commands
HTM_DoStyleCmds0
   LD    A,C
   LD    (HTML_StyleDbIdx+2),A
   LD    (HTML_StyleDbIdx),HL
;
HTM_DoStyleCmds1
   LD    A,(HTML_StyleDbIdx+2)
   LD    HL,(HTML_StyleDbIdx)
   LD    (HTML_StyleDbIdxB+2),A		;save a copy of the cmd address
   LD    (HTML_StyleDbIdxB),HL
   CALL  HTM_GetStyleCmd		;get style command and parameter
   LD    DE,SysGenBuffer
   LD    A,(DE)
   OR    A				;command got?
   RET   Z				;No
   LD    A,C
   LD    (HTML_StyleDbIdx+2),A
   LD    (HTML_StyleDbIdx),HL
;
;  execute the style command
;
   EX    DE,HL
   CALL  HTM_FindCmdStyle		;search known style commands
   CP    Find_Okay			;Did we find it?
   CALL  Z,HTM_DoCmdHandler		;Yes, Execute style cmd routine
;
   JR    HTM_DoStyleCmds1
;=========================================================================
;Name     HTM_ChangeStyles
;Purpose  To change the current style format settings.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_StyleSetting = Current styles settings object
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_StyleSetting = updated styles settings object
;         HTML_CurrHxDbIdx = bits 0 and 7
;Uses     TBD
;Call By  HTM_DoCmdHandler
;Calls    HTM_GetStyleInfo
;         HTM_DoStyleCmds
;         HTM_GSI2
;         HTM_SetupStyleFormat
;Notes    1. This routine first resets the style settings before it
;            applies the new settings to the main variables in the CPCB
;            executables.
;         2. The current display settings are simply altered to suit the
;            requested changes (i.e the current settings are not reset to
;            defaults as a new request is applied).
;=========================================================================
HTM_ChangeStyles
;
   CALL  HTM_GetStyleInfo	;Get class/style/id parameter from HTML cmd
;
HTM_ChngeStyles1
   LD    A,(HTML_ClassIDType)
   OR    A			;any style type parameters got?
   JR    Z,HTM_ChngeStyles2	;No
;
   CALL  HTM_DoStyleCmds	;change the current style format settings
;
   LD    A,(HTML_ClassIDType)
   CP    HTM_P_style		;'style' parameter in HTML command?
   JR    Z,HTM_ChngeStyles2	;Yes
;
   CALL  HTM_GSI2		;re-enter (HTM_GetStyleInfo) for CSS
;
   JR    HTM_ChngeStyles1	;process the style format
;
HTM_ChngeStyles2
   LD    HL,HTML_StyleFlgs
   BIT   7,(HL)			;any changes?
   RET   Z			;No
;
   RES   7,(HL)			;flag now setting up changes
   JR    HTM_SetupStyleFormat
;=========================================================================
;Name     HTM_UndoStyleCmds
;Purpose  To undo the style command setting to a previous setting.
;In Regs  AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_StyleFlgs = status of style handling.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_StyleFlgs = bit 0 cleared.
;         BorderIconSet = icon set number for border
;                         if bit 7 set then no border.
;Uses     TBD
;Call By  HTM_DoCmdHandler
;Calls    HTM_PopStyle
;         HTM_PushStyle
;         STY_ChngeFontByIndx
;         STY_ChngeFontEffect
;         STY_ChngeBorder
;Notes    TBD
;=========================================================================
HTM_UndoStyleCmds
   CALL  STY_ChngeFontEffect	;remove effects
   CALL  HTM_PopStyle		;remove current setting's from stack
   CALL  HTM_PopStyle		;Get previous style settings
;=========================================================================
;Name     HTM_SetupStyleFormat
;Purpose  Common entry point for changing the style settings.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = TBD
;         IX = ??
;         IY = ??
;Out Vars ??
;Uses     TBD
;Call By  TBD
;Calls    HTM_PushStyle
;         STY_ChngeFontByIndx
;         STY_ChngeFontEffect
;         STY_ChngeBorder
;Notes    TBD
;=========================================================================
HTM_SetupStyleFormat
   CALL  HTM_PushStyle		;push onto stack new style settings.
   CALL  STY_ChngeFontByIndx	;setup font by DB index
   CALL  STY_ChngeFontEffect	;setup font effect
   JP    STY_ChngeBorder	;setup border type/width
;=========================================================================
;Name     STY_ChngeFont
;Purpose  To change to a new font
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_StyleFntNme = addr in HTML of style command in KB_Buffer
;                          = HTML address of style command
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars ??
;Uses     TBD
;Call By  TBD
;Calls    UTL_strcpy
;         HTM_GetStyleCmd
;         HTM_StyleParam
;         ERoutine_SelectFontByName
;Notes    TBD
;=========================================================================
STY_ChngeFont
   LD    HL,(HTML_StyleFntNme)
   LD    A,(HTML_StyleFntNme+2)
   CALL  HTM_GetStyleCmd		;get style command and parameter
   CALL  HTM_StyleParam			;HL = address of font name
   JR    STY_ChngeFont1
;
STY_ChngeFont2
   LD    HL,FONT_Italic
STY_ChngeFont1
   CALL  ERoutine_SelectFontByName	;select font via a name
   LD    (HTML_StyleFntNme),DE		;Save DB index of font name
   RET
;=========================================================================
;Name     STY_UndoFontByIndx
;Purpose  To undo a font to a new font by it db index
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars ??
;Uses     TBD
;Call By  TBD
;Calls    ERoutine_SelectFontByName
;         HTM_PopStyle
;Notes    TBD
;=========================================================================
STY_UndoFontByIndx
   CALL  HTM_PopStyle			;Remove current style from stack
   CALL  HTM_PopStyle			;get previous style from stack
;=========================================================================
;Name     STY_ChngeFontByIndx
;Purpose  To change to a new font
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_StyleFntNme = Index to fonts DB.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars ??
;Uses     TBD
;Call By  TBD
;Calls    ERoutine_SelectFontByName
;Notes    TBD
;=========================================================================
STY_ChngeFontByIndx
   LD    DE,(HTML_StyleFntNme)
   JP    ERoutine_SelectFontByIndx	;change font by the index.
;=========================================================================
;Name     STY_ChngeFontEffect
;Purpose  To change to a new font effect
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  SysGenBuffer = address of style command and parameters.
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars ??
;Uses     TBD
;Call By  TBD
;Calls    ERoutine_FontEffect
;Notes    1. Looks at variant to determine what effect is used
;            a. underline
;            b. strike-through
;            c. inverse
;            d. italic
;            e. normal
;            The normal effect removes current effects
;=========================================================================
STY_ChngeFontEffect
   LD    A,(HTML_StyleFlgs)
   LD    B,A
;
   BIT   1,B			;Setup italic effect via font?
   JR    NZ,STY_ChngeFont2	;Yes
;
   LD    C,128			;Underline bit mask
   BIT   2,B			;Underline effect?
   JR    NZ,STY_FontEffect4	;Yes
   LD    C,&40			;strike-throught bit mask
   BIT   3,B			;strike-through effect?
   JR    NZ,STY_FontEffect4	;Yes
   LD    C,0
   BIT   6,B			;inverse effect?
   JR    Z,STY_FontEffect4	;No (no effects)
   LD    C,255
STY_FontEffect4
   LD    B,0
   LD    A,C
   OR    A			;No effects?
   JP    Z,ERoutine_FontEffect	;Yes, remove any current effects
   LD    B,255
   JP    ERoutine_FontEffect	;setup font effects
;=========================================================================
;Name     STY_ChngeBorder
;Purpose  To update a border style
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_StyleBord = border type
;         HTML_StyleBordw = Border width/thickness
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars BorderIconSet = updated with icon set to use for border/frame
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    1. The border styles available are -
;            a. None/hidden
;            b. Line
;            c. Double-line
;            d. Dotted
;            e. groove
;            All others in CSS 2/2.1 are unavaliable.
;         2. The treatment of hidden to be same as none is so that
;            screen real-estate is maximized.
;=========================================================================
STY_ChngeBorder
;
;  Setup border style type
;
   LD    A,(HTML_StyleBord)
   LD    B,DBLEBRDR_ICNST
   CP    HTM_P_btdble
   JR    Z,STY_SelBrderS2
;
   LD    B,DOTDBRDR_ICNST
   CP    HTM_P_btdottd
   JR    Z,STY_SelBrderS2
;
   LD    B,TRPLBRDR_ICNST
   CP    HTM_P_btridg
   JR    Z,STY_SelBrderS2
;
   LD    B,GROVBRDR_ICNST
   CP    HTM_P_btgroov
   JR    Z,STY_SelBrderS2
;
   LD    B,DASHBRDR_ICNST
   CP    HTM_P_btdashd
   JR    Z,STY_SelBrderS2
;
   LD    B,128			;None/Hidden
   CP    HTM_P_btsolid
   JR    NZ,STY_SelBrderS2
;
   LD    A,(HTML_StyleBordw)	;thin/medium/thick
   LD    B,A
;
STY_SelBrderS2
   LD    A,B
   LD    (BorderIconSet),A
   RET
