;=========================================================================
;
; Application Routines (Table rendering commands)
; Copyright (c) 2010 Ray Palmer
;
;=========================================================================
;
;  Routines specific to the table command and related commands.
;
;=========================================================================
;Name     HTM_ChkTblState
;Purpose  To determine the state of processing table data.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState =  status flags
;Out Regs AF = NZ flag set if acquiring table info else Z
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars None.
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
HTM_ChkTblState
   LD    A,(HTML_RenderState)
   BIT   2,A
   RET
;=========================================================================
;Name     HTM_ChkTblBorder
;Purpose  To determine the state of processing table data.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_TblState =  Table border status flags
;Out Regs AF = NZ flag set if table has borders else Z
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars None.
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
HTM_ChkTblBorder
   LD    A,(HTML_TblState)
   BIT   2,A				;Table has borders?
   RET
;=========================================================================
;Name     HTM_ChkTblEnd
;Purpose  To determine if the table arraindex point to end of columns.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  ??
;Out Regs AF = NZ flag set if table has borders else Z
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars None.
;Uses     TBD
;Call By  TBD
;Calls    HTM_CalcTblFrmtCF
;Notes    None
;=========================================================================
HTM_ChkTblEnd
   CALL  HTM_CalcTblFrmtCF
   LD    A,(HL)
   CP    &C0				;Any column defs stored?
   RET
;=========================================================================
;Name     HTM_CalcTblFrmtCF
;Purpose  To determine the address of the current cell in HTML_TblFmtDat.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_TblIndex =  index into the table.
;         HTML_TblFmtDat = address of table array.
;Out Regs AF = ??
;         BC = ??
;         DE = corrupted.
;         HL = address in the table.
;         IX = ??
;         IY = ??
;Out Vars None.
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
HTM_CalcTblFrmtCF
   LD    HL,(HTML_TblFmtDat)		;Location of table column formats.
   JR    HTM_CalcArrayLocn
;=========================================================================
;Name     HTM_CalcTblFrmtRS
;Purpose  To calculate the address in the row spans array during info
;         gathering.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_TblIndex =  index into the table.
;         HTML_TblRowWid = address of table array for row spans
;Out Regs AF = ??
;         BC = ??
;         DE = corrupted.
;         HL = address in the table.
;         IX = ??
;         IY = ??
;Out Vars None.
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
HTM_CalcTblFrmtRS
   LD    HL,(HTML_TblRowWid)		;Location of table row spans.
   JR    HTM_CalcArrayLocn
;=========================================================================
;Name     HTM_CalcCellAddrCS
;Purpose  To determine the address of the current cell in HTML_TblColWid.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_TblIndex = index ino the table array
;         HTML_TblColWid = location of the table array.
;Out Regs AF = ??
;         BC = ??
;         DE = corrupted.
;         HL = address in the table.
;         IX = ??
;         IY = ??
;Out Vars None.
;Uses     TBD
;Call By  TBD
;Calls    None.
;Notes    None.
;=========================================================================
HTM_CalcTblFrmtCS
   LD    HL,(HTML_TblColWid)		;Location of table col spans
   JR    HTM_CalcArrayLocn
;=========================================================================
;Name     HTM_CalcCellAddrCW
;Purpose  To determine the address of the current cell in HTML_TblNRSWid.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_TblIndex = index ino the table array
;         HTML_TblNRSWid = location of the table array.
;Out Regs AF = ??
;         BC = ??
;         DE = HTML_TblIndex
;         HL = address in the table.
;         IX = ??
;         IY = ??
;Out Vars None.
;Uses     TBD
;Call By  TBD
;Calls    None.
;Notes    None.
;=========================================================================
HTM_CalcTblFrmtCW
   LD    HL,(HTML_TblNRSWid)		;Location of table col spans
HTM_CalcArrayLocn
   LD    DE,(HTML_TblIndex)		;index into the table array
   ADD   HL,DE
   RET
;=========================================================================
;Name     HTM_NextFrmtCol
;Purpose  To point to the next formats COlumn during info gathering.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_TblRowWid = location of Table row spans array.
;         HTML_TblIndex = index into the array.
;Out Regs AF = ??
;         BC = ??
;         DE = Number of columns
;         HL = HTML_TblIndex
;         IX = ??
;         IY = ??
;Out Vars None.
;Uses     TBD
;Call By  TBD
;Calls    HTM_CalcTblFrmtRS
;Notes    1. Dont expect the next column to also be a multi row column.
;=========================================================================
HTM_NextFrmtCol
   CALL  HTM_CalcTblFrmtCS
   LD    E,(HL)				;Get number of columns spanned
   LD    D,0
   LD    HL,(HTML_TblIndex)
   ADD   HL,DE				;next table column
   LD    (HTML_TblIndex),HL
   RET
;=========================================================================
;Name     HTM_SetColGrpSpan
;Purpose  To setup the columns span for a group.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_ParamVLoc = address of parameter value
;Out Regs AF = corrupted
;         BC = ??
;         DE = corrupted
;         HL = corrupted
;         IX = ??
;         IY = ??
;Out Vars HTML_colgrpspan = number of columns the group spans.
;Uses     TBD
;Call By  TBD
;Calls    UTL_readdec
;Notes    1. Assume a max of 255 columns can be spanned.
;=========================================================================
HTM_SetColGrpSpan
   LD    DE,(HTML_ParamVLoc)		;address of param val after " char
   CALL  UTL_readdec			;A = span value
   LD    (HTML_colgrpspan),A
   RET
;=========================================================================
;Name     HTM_DecRowSpans
;Purpose  To decrement the row spans.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_TblRowWid = Table row spans.
;         HTML_TblFmtDat = array of table column formats.
;Out Regs AF = ??
;         B = Number of rows which span > 1 row
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_TblRowWid = updated rows spans
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_DecRowSpans
   LD    DE,(HTML_TblFmtDat)
   LD    HL,(HTML_TblRowWid)
   LD    B,0
HTM_DecRowSpns1
   LD    A,(DE)
   CP    &C0
   RET   Z
   LD    A,(HL)
   OR    A				;multiple rows used up?
   JR    Z,HTM_DecRowSpns2	;Yes (dont decrement)
   DEC   (HL)
   JR    Z,HTM_DecRowSpns2	;span now zero
   INC   B
HTM_DecRowSpns2
   INC   HL				;next array entry
   INC   DE
   JR    HTM_DecRowSpns1
;=========================================================================
;Name     HTM_ReInitRowSpans
;Purpose  To reset the column spans for those columns which span only 1
;         row.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_TblRowWid = Table row spans.
;Out Regs AF = ??
;         B = Number of spanned rows left
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_TblRowWid = updated rows spans
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_ReInitColSpans
   LD    DE,0
HTM_ReInitColSpns1
   LD    HL,(HTML_TblFmtDat)
   ADD   HL,DE
   LD    A,(HL)
   CP    &C0
   RET   Z
;
   LD    HL,(HTML_TblColWid)
   ADD   HL,DE
   LD    A,(HL)				;get current number of columns
   LD    C,A
   CP    1					;spans only 1 column?
   JR    Z,HTM_ReInitColSpns2		;Yes
;
   LD    HL,(HTML_TblRowWid)
   ADD   HL,DE
   LD    A,(HL)
   OR    A					;spans no row ?
   JR    NZ,HTM_ReInitColSpns2	;No (dont adjust yet)
;
   LD    HL,(HTML_TblColWid)
   ADD   HL,DE
   LD    B,C
HTM_ReInitColSpns3
   LD    (HL),1
   INC   HL
   DJNZ  HTM_ReInitColSpns3
;
HTM_ReInitColSpns2
   EX    DE,HL
   LD    B,0
   ADD   HL,BC
   EX    DE,HL
   JR    HTM_ReInitColSpns1
;=========================================================================
;Name     HTM_CaptionShow
;Purpose  To display the table caption text.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = state of rendering HTML data
;         HTML_CapLocn = location of caption text
;Out Regs AF = ??
;         BC = ??
;         D = length of caption text
;         E = corrupted.
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    HTM_NextHTMaddr
;         NextHTMBank
;         HTM_CpyCmd
;         UTL_strncmpCI
;Notes    1. The caption text is centered by the table width.
;         2. Caption is just purely text and has no HTML formatting.
;         3. Expect caption to be a max of 255 characters.
;=========================================================================
HTM_CaptionShow
   CALL  HTM_CaptionLen
   LD    A,D
   LD    (HTML_CapSize),A
   SRL   A
   LD    E,A				;DE = caption lenght / 2
   LD    D,0
   LD    HL,(HTML_Tblwidth)
   SRL   H					;HL = table width / 2
   RR    L
   OR    A					;clear crary flag
   SBC   HL,DE				;expected start col offset (caption)
   EX    DE,HL
   LD    HL,(HTML_TblSRX)
   JR    C,HTM_CaptionShow1
   ADD   HL,DE
HTM_CaptionShow1
   LD    (HTML_Col),HL
   LD    HL,(HTML_CapLocn)
   LD    A,(HTML_CapLocn+2)
   LD    E,A
HTM_CaptionShow3
   INC   HL
   LD    A,E				;Point to start of caption text
   CALL  HTM_NextHTMBank
   LD    E,A
   LD    A,(HL)
   CP    '<'				;Start of next HTML command?
   JR    Z,HTM_CaptionShow4		;Yes
   PUSH  DE
   PUSH  HL
   LD    B,A
   CALL  HTM_ShowChar
   POP   HL
   POP   DE
   JR    HTM_CaptionShow3
HTM_CaptionShow4
   PUSH  DE
   LD    A,E
   LD    B,KB_BuffSz-1
   LD    DE,HTML_LblTemp
   CALL  HTM_CpyCmd			;Copy Command text to variable
   PUSH  HL
   LD    HL,HTML_LblTemp+1
   LD    DE,HTML_C_caption
   LD    B,7
   CALL  UTL_strncmpCI			;command </caption> ?
   POP   HL
   POP   DE
   JR    NZ,HTM_CaptionShow3		;No
   RET
;=========================================================================
;Name     HTM_CaptionLen
;Purpose  To get the size of the cpation text.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_CapLocn = location of caption text
;Out Regs AF = ??
;         BC = ??
;         D = length of caption text
;         E = corrupted.
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    HTM_NextHTMaddr
;         NextHTMBank
;         HTM_CpyCmd
;         UTL_strncmpCI
;Notes    TBD
;=========================================================================
HTM_CaptionLen
   LD    HL,(HTML_CapLocn)
   LD    A,(HTML_CapLocn+2)
   LD    E,A
   LD    D,0
HTM_CapLen3
   INC   HL
   LD    A,E				;Point to start of caption text
   CALL  HTM_NextHTMBank
   LD    E,A
   LD    A,(HL)
   CP    '<'				;Start of next HTML command?
   JR    Z,HTM_CapLen4			;Yes
   INC   D
   JR    HTM_CapLen3
HTM_CapLen4
   PUSH  DE
   LD    A,E
   LD    B,KB_BuffSz-1
   LD    DE,HTML_LblTemp
   CALL  HTM_CpyCmd			;Copy Command text to variable
   PUSH  HL
   LD    HL,HTML_LblTemp+1
   LD    DE,HTML_C_caption
   LD    B,7
   CALL  UTL_strncmpCI			;command </caption> ?
   POP   HL
   POP   DE
   JR    NZ,HTM_CapLen3			;No
   RET
;=========================================================================
;Name     HTM_GetWidthValue
;Purpose  To value of the width parameter.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = char which stopped conversion.
;         BC = Width value in chars.
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    UTL_readdec2
;         Mth_Divide
;Notes    TBD
;=========================================================================
HTM_GetWidthValue
   LD    DE,(HTML_ParamVLoc)
   CALL  UTL_readdec2		;HL = width value
   CP    '$'			;width spec in chars?
   JR    Z,HTM_GetWidthVal2	;Yes
   CP    "%"			;was last digit %
   JR    Z,HTM_GetWidthVal1	;convert percent to chars
HTM_GetWidthVal0
   LD    DE,8			;number of pixels/character
HTM_GetWidthVal0a
   PUSH  AF
   CALL  Mth_Divide		;BC = number of characters for table width
   POP   AF
   RET
HTM_GetWidthVal2
   LD    C,L
   LD    B,H
   RET
HTM_GetWidthVal1
   EX    DE,HL
   LD    HL,65528			;Max of 65528 chars width on web page
   JR    HTM_GetWidthVal0a	;calc % max chars
;=========================================================================
;Name     HTM_SetTableAlign
;Purpose  To setup the alignment of the whole table on the web page.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_TblAlign
;Uses     TBD
;Call By  TBD
;Calls    HTM_GetAlignType
;Notes    1. no check is made about other alignments (i.e top,justify,etc)
;=========================================================================
HTM_SetTableAlign
   CALL  HTM_GetAlignType
   LD    A,C
   LD    (HTML_TblAlign),A
   RET
;=========================================================================
;Name     HTM_SetTableWidth
;Purpose  To setup the width of the whole table on the web page.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_Tblwidth
;Uses     TBD
;Call By  TBD
;Calls    HTM_GetWidthValue
;Notes    1. Need to check the width against cell contents to adjust the
;            width down if we can.
;=========================================================================
HTM_SetTableWidth
   CALL  HTM_GetWidthValue
   LD    (HTML_Tblwidth),BC
   RET
;=========================================================================
;Name     HTM_FlagTableBorder
;Purpose  To flag the table cells have borders
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = address of border value to get.
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_TblState = updated on bit 2
;Uses     TBD
;Call By  TBD
;Calls    None.
;Notes    TBD
;=========================================================================
HTM_FlagTableBorder
   CALL  HTM_GetWidthValue		;get border size (is ignored)
   LD    HL,HTML_TblState
   SET   2,(HL)				;Flag table has borders
   RET
;=========================================================================
;Name     HTML_ResetTblData
;Purpose  To reset the tbale data array.
;In Regs  A = Initialisation value for each array element.
;         BC = ??
;         DE = ??
;         HL = Address of array
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    None.
;Notes    TBD
;=========================================================================
HTML_ResetTblData
   LD    B,HTML_TblMaxCells
HTML_RsetTblData
   LD    (HL),A
   INC   HL
   DJNZ   HTML_RsetTblData
   RET
;=========================================================================
;Name     HTML_GetTblInfo
;Purpose  To obtain the table information for rendering.
;         The information gained in the align of the columns, the number
;         of columns and the number of rows (if alignment is auto).
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_ObjStart = start of table object data.
;         HTML_RenderState = current state of table gathering info
;         HTML_Row = current web-page canvas row.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_TblMaxRows = max number of rows in the table
;         HTML_TblMaxCols = max number of column in the table
;         HTML_TblHdrLoc = location of THEAD data
;         HTML_TblFtrLoc = location of TFOOT data
;         HTML_TblCapLoc = Location of table body.
;         HTML_TblBdyCnt = number of <TBODY..> commands
;         HTML_TblIndex = corrupted
;         HTML_RenderState = updated for table info gathering.
;         HTML_TblSRY = start of table render canvas row
;         HTML_TblSRX = start of table render canvas column
;Uses     TBD
;Call By  TBD
;Calls    HTM_GetCmd
;         HTM_FindHTMLCmd
;         HTM_DoHTMLCmd
;         HTM_FindCmdStrt
;Notes    TBD
;=========================================================================
HTM_GetTblInfo
;
;  Calculate the web-page start column to render the table.
;
   LD    A,(Scrn_MaxCol)
   LD    C,A
   LD    B,0
;
   LD    HL,0				;left margin of canvas
   LD    A,(HTML_TblAlign)
   CP    HTM_ALIGNLEFT			;Alignment on Left of web-page?
   JR    Z,HTM_GetTblInfo_5		;Yes
   CP    HTM_ALIGNCENTER		;Table centered on web-page?
   JR    Z,HTM_GetTblInfo_2		;Yes
   CP    HTM_ALIGNRIGHT			;Table on right of web-page?
   JR    NZ,HTM_GetTblInfo_5		;No (default to left)
;
   LD    E,A
   LD    D,0
   LD    HL,(HTML_Tblwidth)
   OR    A
   SBC   HL,BC	   			;Does whole table fit in CPC window?
   JR    C,HTM_GetTblInfo_4		;Yes
   LD    DE,MAXWEBPAGECOLS-1		;Start from right most edge
   JR    HTM_GetTblInfo_3
HTM_GetTblInfo_2
   SRL   A
   LD    D,0
   LD    E,A				;center is mid point of CPC window
   LD    HL,(HTML_Tblwidth)
   OR    A
   SBC   HL,BC				;Does centered table fit in window?
   JR    C,HTM_GetTblInfo_3		;Yes
   LD    DE,MAXWEBPAGECOLS/2		;center is mid point of canvas
HTM_GetTblInfo_3
   LD    HL,(HTML_Tblwidth)
   SRL   H
   RR    L				;HL = half table width
HTM_GetTblInfo_4
   OR    A
   SBC   HL,DE				;HL = web-page canvas start column
HTM_GetTblInfo_5
   LD    (HTML_TblSRX),HL
   RET
;=========================================================================
;Name     HTM_SetCGWTyp
;Purpose  To setup the columns width for the cells in the table cell
;         widths data.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_colgrpspan = number of columns to set the alignment to.
;         HTML_colgrpcc = current offset to the start column of the group.
;         HTML_colgrpwid = width setting.
;Out Regs AF = corrupted
;         BC = corrupted
;         DE = corrupted
;         HL = corrupted
;         IX = ??
;         IY = ??
;Out Vars HTML_TblFmtWid = updated with the width value
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    1. Only the first n cells making up the first row are updated.
;=========================================================================
HTM_SetCGWidth
   LD    A,(HTML_colgrpspan)		;Get number of columns
   LD    B,A
   LD    D,0
   LD    A,(HTML_colgrpcc)
   LD    E,A
   LD    HL,(HTML_TblColWid)
   ADD   HL,DE				;setup start column for fixed widths
   LD    A,(HTML_colgrpwid)
html_cmd_colgrp3d
   LD    (HL),A				;Setup fixed column width
   INC   HL
   DJNZ  html_cmd_colgrp3d
   RET
;=========================================================================
;Name     HTM_SetCGWTyp
;Purpose  To setup the columns width for the cells in the table format
;         data.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_colgrpspan = number of columns to set the alignment to.
;         HTML_colgrpcc = current offset to the start column of the group.
;         HTML_colgrpwid = width setting.
;Out Regs AF = corrupted
;         BC = corrupted
;         DE = corrupted
;         HL = corrupted
;         IX = ??
;         IY = ??
;Out Vars HTML_TblFmtDat = updated with the widt type (auto/fixed)
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    TBD
;=========================================================================
HTM_SetCGWTyp
   LD    A,(HTML_colgrpwid)		;column wifth value
   LD    E,&F7				;Mask for bit 3
   LD    D,0				;Auto width
   CP    255				;At max column width?
   JR    Z,HTM_SetCGAlign3b		;Yes (flag auto is to be used)
   LD    D,&08				;Flag fixed width
   JR    HTM_SetCGAlign3b
;=========================================================================
;Name     HTM_SetCGAlign
;Purpose  To setup the columns alignment for the cells in the table format
;         data.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_colgrpspan = number of columns to set the alignment to.
;         HTML_colgrpcc = current offset to the start column of the group.
;         HTML_colgrpalign = alignment setting.
;Out Regs AF = corrupted
;         BC = corrupted
;         DE = corrupted
;         HL = corrupted
;         IX = ??
;         IY = ??
;Out Vars HTML_TblFmtDat = updated
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    1. Only the first n cells making up the first row are updated.
;=========================================================================
HTM_SetCGAlign
   LD    E,&CF				;Mask for bits 4/5
   LD    A,(HTML_colgrpalign)
   LD    D,&10				;center align
   CP    HTM_ALIGNCENTER
   JR    Z,HTM_SetCGAlign3b
   LD    D,&20				;right align
   CP    HTM_ALIGNRIGHT
   JR    Z,HTM_SetCGAlign3b
   LD    D,0				;align left (default)
HTM_SetCGAlign3b
   LD    B,0
   LD    A,(HTML_colgrpcc)
   LD    C,A
   LD    HL,(HTML_TblFmtDat)
   ADD   HL,BC
   LD    A,(HTML_colgrpspan)
   LD    B,A				;B = number of columns affected.
HTM_SetCGAlign3c
   LD    A,(HL)
   AND   E					;mask out required bits
   OR    D					;insert the settings.
   LD    (HL),A
   INC   HL
   DJNZ  HTM_SetCGAlign3c
   RET
;=========================================================================
;Name     HTM_SetColGrpAlign
;Purpose  To setup the columns alignment for a group.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_ParamVLoc = address of parameter value
;Out Regs AF = corrupted
;         BC = ??
;         DE = corrupted
;         HL = corrupted
;         IX = ??
;         IY = ??
;Out Vars HTML_colgrpalign = alignment for the column in the group.
;Uses     TBD
;Call By  TBD
;Calls    HTM_GetAlignType
;Notes    None
;=========================================================================
HTM_SetColGrpAlign
   CALL  HTM_GetAlignType
   LD    (HTML_colgrpalign),A
   RET
;=========================================================================
;Name     HTM_SetColGrpWidth
;Purpose  To setup the columns width for a group.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_ParamVLoc = address of parameter value
;Out Regs AF = corrupted
;         BC = ??
;         DE = corrupted
;         HL = corrupted
;         IX = ??
;         IY = ??
;Out Vars HTML_colgrpwid = width of each column in the group spans.
;Uses     TBD
;Call By  TBD
;Calls    HTM_GetWidthValue
;Notes    1. The spec says if width is for * defines a portion, the
;            implementation will interpret it as auto width.
;=========================================================================
HTM_SetColGrpWidth
   CALL  HTM_GetWidthValue		;Get column width value
   CP    '*'				;was width a "*" type?
   JR    NZ,HTM_SetColGrpWidth1	;No (was fixed width)
   LD    C,255				;Flag Auto char width
HTM_SetColGrpWidth1
   LD    A,C				;Get width value
   LD    (HTML_colgrpwid),A		;Setup what columns widths are
   RET
;=========================================================================
;Name     HTM_TDTH_Params
;Purpose  To handle the parameters to the TD and TH command for -
;           rowspan, colspan and nowrap
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_TblIndex = array index into array of table formats
;         HTML_TblCurrRow = array index into array of table formats for
;                           rendering.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_TblFmtDat = data updated.
;Uses     TBD
;Call By  TBD
;Calls    HTM_GetCmdParam
;         HTM_strncmp
;         HTM_GetCmdParmVal
;         HTM_CellspanRows
;         HTM_CellspanCols
;         HTM_CellNowrap
;         HTM_CalcCellAddr
;Notes    1. No checks are made to ensure that the rules for rowspan and
;            colspan do not collide.
;         2. There no checks to see if the rows exceed limits of the
;            formats structure or comply with rules.
;=========================================================================
HTM_TDTH_Params
   LD    A,1
   LD    (HTML_colrspan),A		;Assume only spans 1 row
   LD    (HTML_colcspan),A		;assume only spans 1 column
   XOR   A
   LD    (HTML_colcwrap),A		;assume no wrap of text
;
   LD    HL,(HTML_CmdTemp)
HTM_TDTH_Params2
   LD    (HTML_ParamLoc),HL
   CALL  HTM_GetCmdParam		;Locate next parameter
   LD    A,(HL)
   CP    '>'				;no more parameters?
   RET   Z					;Yes
;
   PUSH  HL
   CALL  HTM_FindParmLbl		;Determine the parameter nametype
   POP   HL
   INC   B					;WHY THIS IS NEEDED?????
   LD    A,HTM_P_rowspan
   CP    B					;Is parameter 'rowspan' ?
   JR    Z,HTM_TDTH_Params0a		;Yes
   LD    A,HTM_P_colspan
   CP    B					;Is parameter 'colspan' ?
   JR    Z,HTM_TDTH_Params0a		;Yes
   LD    A,HTM_P_nowrap
   CP    B					;Is parameter 'nowrap' ?
   JR    NZ,HTM_TDTH_Params2		;No (skip the parameter)
;
HTM_TDTH_Params0a
   LD    (HTML_ParamTyp),A		;Save parameter type
   CALL  HTM_GetCmdParmVal		;HL = address of parameter value
;
   LD    A,(HTML_ParamTyp)
   CP    HTM_P_rowspan
   CALL  Z,HTM_CellspanRows		;save number of rows spanned.
;
   LD    A,(HTML_ParamTyp)
   CP    HTM_P_colspan
   CALL  Z,HTM_CellspanCols		;save number of columns spanned
;
   LD    A,(HTML_ParamTyp)
   CP    HTM_P_nowrap
   CALL  Z,HTM_CellNowrap		;save text wrap format flag
;
   JR    HTM_TDTH_Params2		;try for next parameter
;=========================================================================
;Name     HTM_SetupSpans
;Purpose  To setup the table formats spans for rows/columns.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_colrspan = number of rows
;         HTML_colcspan = number of columns
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_SetupSpans
   CALL  HTM_CalcTblFrmtRS		;calc address in row spans table.
   LD    A,(HTML_colrspan)
   LD    (HL),A
;
   CALL  HTM_CalcTblFrmtCS		;calc address in column spans table.
   LD    A,(HTML_colcspan)
   LD    (HL),A
   RET
;=========================================================================
;Name     HTM_SetupTblRowCol
;Purpose  To setup the table formats.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_colcspan = number of columns
;         HTML_colcwrap = text wrap controls for cells.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_SetupTblRowCol
   LD    A,(HTML_colcspan)
   LD    B,A
;
;  Setup the cells format in the table format data.
;   - cell text wrapping
;   - cell merge flag if spanned.
;
   CALL  HTM_CalcTblFrmtCF		;calc address in formats table.
   PUSH  HL
   LD    A,(HTML_colcwrap)
HTM_TDTH_Params3d
   LD    C,(HL)
   RES   7,C				;Flag new column (header)
   SET   6,C
   RES   2,C				;Assume cell has wrap text
   CP    0					;wrap text ?
   JR    Z,HTM_TDTH_Params3a		;yes
   SET   2,C				;Flag cell has no wrap text
HTM_TDTH_Params3a
   SET   0,C				;Flag cell merged
   LD    (HL),C
   INC   HL					;next column
   DJNZ  HTM_TDTH_Params3d		;yes
   POP   HL
   RES   0,(HL)				;Flag first cell is not merged
   RET
;=========================================================================
;Name     HTM_CellspanRows
;Purpose  To save the number of rows spanned.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = address of text to convert to a number.
;         IX = ??
;         IY = ??
;In Vars  None
;Out Regs AF = corrupted.
;         BC = ??
;         DE = ??
;         HL = address of non digit character
;         IX = ??
;         IY = ??
;Out Vars HTML_colrspan = number of rows
;Uses     TBD
;Call By  TBD
;Calls    UTL_readdec
;Notes    1. The maximum span is 255.
;=========================================================================
HTM_CellspanRows
   CALL  UTL_readdec
   LD    (HTML_colrspan),A
   RET
;=========================================================================
;Name     HTM_CellspanCols
;Purpose  To save the number of columns spanned.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = address of text to convert to a number.
;         IX = ??
;         IY = ??
;In Vars  None
;Out Regs AF = corrupted.
;         BC = ??
;         DE = ??
;         HL = address of non digit character
;         IX = ??
;         IY = ??
;Out Vars HTML_colcspan = number of columns
;Uses     TBD
;Call By  TBD
;Calls    UTL_readdec
;Notes    1. The maximum span is 255.
;=========================================================================
HTM_CellspanCols
   CALL  UTL_readdec			;convert text to number
   LD    (HTML_colcspan),A
   RET
;=========================================================================
;Name     HTM_CellNowrap
;Purpose  To save the no wrap flag.
;In Regs  A = ??
;         BC = ??
;         D = ??
;         E = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  None
;Out Regs AF = corrupted.
;         BC = ??
;         D = ??
;         E = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_colcwrap = flag to not wrap text.
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
HTM_CellNowrap
   LD    A,1
   LD    (HTML_colcwrap),A
   RET
;=========================================================================
;Name     HTM_CalcCellWidth
;Purpose  To determine the width of the current table cell by examinig its
;         contents for the width.
;In Regs  A = current width of data.
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_ObjLocn = location of <TD>/<TH> command
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_CurCelWid = width of text in the cell.
;Uses     TBD
;Call By  TBD
;Calls    HTM_FindCmdEnd
;         HTM_NextHTMBank
;         HTM_GetCmd
;Notes    1. assume the end of the data is by a new HTML command.
;         2. The maximum width that can be handled is 255 chars.
;=========================================================================
HTM_CalcCellWidth
   LD    D,0				;D = curr max length of data in cell
   LD    A,1
   LD    (HTML_CurCelWid),A		;current cell width
   LD    HL,(HTML_CmdEnd)
   LD    A,(HTML_CmdEnd+2)
   LD    E,A
   CALL  MEM_SwapInHTML			;Swap in HTML data bank.
   LD    A,(HL)
   CP    '>'
   JR    NZ,HTM_CalcCellWidth2
HTM_CalcCellWidth1
   INC   HL					;Skip past '>' in command
HTM_CalcCellWidth2
   LD    A,E
   CALL  HTM_NextHTMBank		;Swap in HTML data bank.
   LD    E,A
   LD    A,(HL)
   CP    '<'				;is char start of new HTML command?
   JR    Z,HTM_CalcCellWidth3		;Yes
   CP    32					;control code char?
   JR    C,HTM_CalcCellWidth1		;Yes (get another char)
   INC   D					;Increment cell text length
   JR    HTM_CalcCellWidth1
;
HTM_CalcCellWidth3
   LD    A,E
   DEC   HL
   LD    (HTML_CmdEnd),HL		;save for HTML processing.
   INC   HL
   LD    (HTML_CmdEnd+2),A
;
   PUSH  DE
   CALL  HTM_GetCmdObj			;Get HTML Command Object
   POP   DE
;
   LD    HL,(HTML_CmdTemp)
   LD    A,(HL)
   CP    '/'				;command starts with '/'
   JR    NZ,HTM_CalcCellWidth6	;No
   INC   HL					;Skip slash to command name
   LD    A,(HL)
HTM_CalcCellWidth6
   CALL  UTL_tolower
   CP    't'				;found table command type?
   JR    Z,HTM_CalcCellWidth7		;Yes (end of data for table cell)
;
   PUSH  DE
   LD    DE,HTML_C_br
   LD    B,2
   CALL  UTL_strncmpCI			;<BR> command?
   POP   DE
   JR    NZ,HTM_CalcCellWidth5
   CALL  HTM_CalcCellWidth7		;update current cell width
   LD    D,0				;reset cell width for next line
   JR    HTM_CalcCellWidth3a
HTM_CalcCellWidth5
;
;  Check if <IMG...> command, if anything else it assumes it relates
;  to formatting like <I>, <U> or hyperlinking.
;
   PUSH  DE
   LD    DE,HTML_C_img
   LD    B,3
   CALL  UTL_strncmpCI			;Is HTML command <IMG ...> ?
   POP   DE
   JR    NZ,HTM_CalcCellWidth3a	;No
;
;  Need to fix handling of the CPC specific image such that the image
;  widths can be got.
;
   INC   D
HTM_CalcCellWidth3a
   LD    HL,(HTML_ObjLocn)
   LD    A,(HTML_ObjLocn+2)
   JR    HTM_CalcCellWidth1
;
HTM_CalcCellWidth7
   LD    A,(HTML_CurCelWid)		;A = current cell width
   CP    D					;is new width less then current?
   RET   NC					;Yes
   LD    A,D				;A = new length
   LD    (HTML_CurCelWid),A
   RET
;=========================================================================
;Name     HTM_SetupWidth
;Purpose  To setup the width of a column.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_TblIndex = current column index.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_CurCelWid = updated array.
;Uses     TBD
;Call By  html_cmd_td
;Calls    HTM_CalcTblFrmtCW
;Notes    TBD
;=========================================================================
HTM_SetupWidth
   CALL  HTM_CalcTblFrmtCW
   LD    A,(HTML_CurCelWid)
   CP    (HL)				;Is current width > current
   RET   C					;Yes
   INC   A
   INC   A
   LD    (HL),A				;save column width (+ borders if any)
   RET
;=========================================================================
;Name     HTM_FndNextColumn
;Purpose  To find the next available column.
;In Regs  A = ??
;         BC = ??
;         DE = Current Format table index
;         HL = Address in array of row spans
;         IX = ??
;         IY = ??
;In Vars  HTML_TblIndex = current column index
;         HTML_TblRowWid = array of row spans.
;Out Regs AF = ??
;         BC = corrupted
;         DE = update index.
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_TblIndex = updated column index
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    1. All row spans can not be zero.
;=========================================================================
HTM_FndNextColumn
   CALL  HTM_CalcTblFrmtRS
   LD    A,(HL)
   CP    0				;This cell in spanned row?
   JR    Z,HTM_FndNextCol2	;No
   CALL  HTM_CalcTblFrmtCS	;Get number of column spanned
   LD    A,(HL)
   LD    L,A
   LD    H,0
   ADD   HL,DE			;Add to index to next column
   LD    (HTML_TblIndex),HL
   JR    HTM_FndNextColumn
HTM_FndNextCol2
   LD    (HTML_TblIndex),DE
   RET
;=========================================================================
;Name     HTM_GetColWidth
;Purpose  To get the width of the nth column.
;In Regs  A = ??
;         BC = ??
;         DE = Index into the array's
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_TblColWid = array of column spans
;         HTML_TblNRSWid = array of columns widths
;Out Regs A = width of the column
;         B = 0
;         C = number of column spans
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_TblCellWidth = width of the column.
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_GetColWidth
   LD    HL,(HTML_TblColWid)	;Location of table col spans
   ADD   HL,DE
   LD    B,(HL)			;get number of columns cell spans
   LD    C,B
   LD    HL,(HTML_TblNRSWid)	;Location of table col spans
   ADD   HL,DE
   XOR   A
HTM_GetColWid1
   ADD   A,(HL)			;Tally column widths
   INC   HL
   INC   DE
   DJNZ  HTM_GetColWid1
   RET
;=========================================================================
;Name     HTM_SumColumns
;Purpose  To sum up the n column widths to get to cell start column for
;         the table data render start.
;In Regs  A = ??
;         BC = ??
;         E = Number of columns to tally
;         HL = Current column to render
;         IX = ??
;         IY = ??
;In Vars  HTML_TblIndex = index to current column.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_SumColumns
   LD    DE,(HTML_TblIndex)
   LD    A,E
   OR    A				;first column?
   RET   Z				;Yes
   LD    B,E			;num columns to sum
   LD    DE,0			;start index at first column
HTM_SumCols1
   PUSH  BC
   PUSH  HL
   CALL  HTM_GetColWidth	;get width of cell in column
   POP   HL
   PUSH  DE
   LD    E,A
   LD    D,0
   ADD   HL,DE			;add to sum to get new start column
   POP   DE
   LD    A,C
   POP   BC
   CP    1
   JR    Z,HTM_SumCols2
   DEC   B
HTM_SumCols2
   DJNZ  HTM_SumCols1
   RET
;=========================================================================
;Name     HTM_NextTblColumn
;Purpose  To goto the next un-merged column.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_TblIndex = current column index
;         HTML_TblColWid = array of columns
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = end of command.
;         IX = ??
;         IY = ??
;Out Vars HTML_TblIndex = updated column index
;Uses     TBD
;Call By  TBD
;Calls    HTM_CalcCellAddr
;Notes    1. No checks are made to ensure the formats have been setup
;            properly.
;=========================================================================
HTM_NextTblColumn
   CALL  HTM_CalcTblFrmtCS
   LD    D,0
   LD    E,(HL)
   LD    HL,(HTML_TblIndex)
   ADD   HL,DE
   LD    (HTML_TblIndex),HL
   RET
;=========================================================================
;Name     HTM_TableSetBR
;Purpose  To setup the column offset for the <BR> command within tables.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_TblSRX = Base start table column
;         HTML_TblSRY = current table start row
;         HTML_TblIndex = index to current column.
;         HTML_TblCurrRow = table row counter
;         HTML_TblNRSWid = array of rows widths
;         HTML_TblState = table status/flags
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_BR_Table = column for <BR> in cell contents.
;         HTML_Col = updated start column
;         HTML_Row = updated start row
;Uses     TBD
;Call By  TBD
;Calls    HTM_CalcTblFrmtRS
;         HTM_FndNextColumn
;Notes    TBD
;=========================================================================
HTM_TableSetBR
   LD    HL,(HTML_TblCurrRow)
   LD    A,H			;First row?
   OR    L
   JR    Z,HTM_TableSetBR0	;Yes
;
   CALL  HTM_CalcTblFrmtRS	;Get address of row span for this cell
   LD    A,(HL)
   OR    A				;This cell in spanned row?
   CALL  NZ,HTM_FndNextColumn	;Yes, Find First available column
;
HTM_TableSetBR0
   LD    HL,(HTML_TblSRX)	;Start column for table on web-page
   LD    A,E
   OR    A				;At first column?
   CALL  NZ,HTM_SumColumns	;No (HL = next column to render)
;
   LD    (HTML_BR_Table),HL	;setup start col for <BR>.
   LD    (HTML_Col),HL
;
   RET
;=========================================================================
;Name     HTM_ShowTblPlus
;Purpose  To display a page border '+'.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_Col = current web-page col
;         HTML_Row = current web-page row
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    HTM_ShowChar
;Notes    TBD
;=========================================================================
HTM_ShowTblPlus
   LD    B,'+'
   JR    HTM_ShowBrdrChar
;=========================================================================
;Name     HTM_ShwTblColChr
;Purpose  To display a page border '|' on the left margin of cell.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_Col = current web-page col
;         HTML_Row = current web-page row
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_Col = next web-page column for text
;Uses     TBD
;Call By  TBD
;Calls    HTM_ShowChar
;Notes    TBD
;=========================================================================
HTM_ShwTblColChr
   LD    B,'|'
HTM_ShowBrdrChar
   CALL  HTM_ShowChar
   LD    HL,(HTML_Col)
   DEC   HL				;cancel auto inc in routine
   LD    (HTML_Col),HL
   RET
;=========================================================================
;Name     HTM_ShowTblDash
;Purpose  To display a page border '-'.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_Col = current web-page col
;         HTML_Row = current web-page row
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    HTM_ShowChar
;Notes    TBD
;=========================================================================
HTM_ShowTblDash
   LD    B,'-'
   JP    HTM_ShowChar
;=========================================================================
;Name     HTM_FindNxtRow
;Purpose  To find where the bottom row is.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         C = max number of rows where the cells had no spans.
;         B = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TDB
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_FindNxtRow
   LD    DE,&FFFF
   LD    C,1				;Min rows
HTM_FindNxtRow1
   INC   DE
   LD    HL,(HTML_TblFmtDat)
   ADD   HL,DE
   LD    A,(HL)				;Get format of Column
   CP    &C0				;Invalid column
   RET   Z					;Yes
   LD    HL,(HTML_TblRowWid)
   ADD   HL,DE
   LD    A,(HL)
   OR    A					;cell spans more than 1 row
   JR    NZ,HTM_FindNxtRow1		;Yes (skip it)
   LD    HL,(HTML_TblCRows)
   ADD   HL,DE
   LD    A,(HL)
   CP    C					;num rows for this cell < max?
   JR    C,HTM_FindNxtRow1		;Yes
   LD    C,A
   JR    HTM_FindNxtRow1
;=========================================================================
;Name     HTM_ShowRowLine
;Purpose  To display the table cell bottom or top border line.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_Col = current render column
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TDB
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_ShowRowLine
   CALL  HTM_GetColWidth		;get column width for this column
   PUSH  DE
   PUSH  AF
   CALL  HTM_ShowTblPlus
   POP   AF
   CALL  HTM_NxtTblCol			;next HTML_Col
   DEC   A
   LD    B,A
HTM_ShwRowLne1
   PUSH  BC
   CALL  HTM_ShowTblDash
   POP   BC
   DJNZ  HTM_ShwRowLne1
   CALL  HTM_ShowTblPlus
   POP   DE
   RET
;=========================================================================
;Name     HTM_SumTblWidths
;Purpose  To sum the column widths to get the table width.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_TblFmtDat = array of column formats
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_SumTblWidths
   LD    DE,0
   LD    (HTML_Tblwidth),DE
HTM_SumTblWidths1
   LD    HL,(HTML_TblFmtDat)
   ADD   HL,DE
   LD    A,(HL)				;Get format of Column
   CP    &C0				;Invalid column
   RET   Z					;Yes
   CALL  HTM_GetColWidth		;get column width for this column
   LD    HL,(HTML_Tblwidth)
   LD    C,A
   LD    B,0
   ADD   HL,BC
   LD    (HTML_Tblwidth),HL
   JR    HTM_SumTblWidths1
;=========================================================================
;Name     HTM_ShowRowBordrs
;Purpose  To display the table cell bottom/top borders without testing the
;         table border status flag.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_TblSRY = current row after cells displayed.
;         HTML_Row = row which is to display horizontal border
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_Col = updated
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_ShowRowBordrs
   LD    HL,(HTML_TblSRX)		;Start column for table on web-page
   LD    (HTML_Col),HL
   LD    DE,0
HTM_ShwRowBorders1
   LD    HL,(HTML_TblFmtDat)
   ADD   HL,DE
   LD    A,(HL)				;Get format of Column
   CP    &C0				;Invalid column
   RET   Z					;Yes
   LD    HL,(HTML_TblRowWid)
   ADD   HL,DE
   LD    A,(HL)
   CP    0					;spans no rows?
   JR    NZ,HTM_ShwRowBorders2	;No
;
   CALL  HTM_ShowRowLine
   JR    HTM_ShwRowBorders1
;
HTM_ShwRowBorders2
   CALL  HTM_GetColWidth		;get column width for this column
   LD    HL,(HTML_Col)
   LD    C,A
   LD    B,0
   ADD   HL,BC
   LD    (HTML_Col),HL
   JR    HTM_ShwRowBorders1
;=========================================================================
;Name     HTM_ShowColBordrs
;Purpose  To display the table cell column borders for one web-page line.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_TblSRY = current row after cells displayed.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TDB
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_ShowColBordrs
   LD    HL,(HTML_TblSRX)		;Start column for table on web-page
   LD    (HTML_Col),HL
   CALL  HTM_ShwTblColChr		;Show left most column border
   LD    DE,0
HTM_ShwColBrdrs1
   LD    HL,(HTML_TblFmtDat)
   ADD   HL,DE
   LD    A,(HL)				;Get format of Column
   CP    &C0				;Last column?
   RET   Z					;Yes
   CALL  HTM_GetColWidth		;get column width for this column
   PUSH  DE
   LD    HL,(HTML_Col)
   LD    C,A
   LD    B,0
   ADD   HL,BC
   LD    (HTML_Col),HL
   CALL  HTM_ShwTblColChr		;Show right cell border
   POP   DE
   JR    HTM_ShwColBrdrs1
;=========================================================================
;Name     HTM_ShwNColBorders
;Purpose  To display the table cell column borders for a number of
;         web-page lines.
;In Regs  A = ??
;         B = ??
;         C = status of spans in table row
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_TblSRY = current start row before any cells displayed.
;         HTML_TblCRY = current max row after all row cells displayed.
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TDB
;Uses     TBD
;Call By  TBD
;Calls    HTM_ShowColBordrs
;         HTM_SetNextRow
;Notes    TBD
;=========================================================================
HTM_ShwNColBorders
   LD    HL,(HTML_TblSRY)		;Get current start row
   LD    (HTML_Row),HL			;Setup start row for col. borders
   LD    B,C				;Number of web-page lines to update
   INC   B
HTM_ShwNCB1
   PUSH  BC
   LD    HL,(HTML_Col)
   PUSH  HL
   CALL  HTM_ShowColBordrs
   POP   HL
   LD    (HTML_Col),HL
   CALL  HTM_SetNextRow
   POP   BC
   DJNZ  HTM_ShwNCB1
   RET
;=========================================================================
;Name     HTM_ShowAllBorders
;Purpose  To show both column borders and row borders.
;In Regs  A = ??
;         B = ??
;         C = number of display rows of data
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    HTM_ChkTblBorder
;Notes    TBD
;=========================================================================
HTM_ShowAllBorders
   PUSH  BC
   LD    HL,(HTML_TblCurrRow)		;get table row#
   LD    A,H
   OR    L
   JR    Z,HTM_ShwAllBorders1		;first table row display only horiz'
   CALL  HTM_ShwNColBorders		;show vertical borders
;
HTM_ShwAllBorders1
   LD    DE,(HTML_TblCRY)		;Get current start row
   LD    (HTML_Row),DE
   CALL  HTM_ShowRowBordrs		;display horizontal borders (if any)
   POP   BC
   RET
;=========================================================================
;Name     HTM_ShowAllBorders2
;Purpose  To show both column borders and bottom table row borders.
;In Regs  A = ??
;         B = ??
;         C = number of display rows of data
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    HTM_ChkTblBorder
;Notes    TBD
;=========================================================================
HTM_ShowAllBorders2
   CALL  HTM_ShwNColBorders		;show vertical borders
   XOR   A
   LD    HL,(HTML_TblRowWid)
   CALL  HTML_ResetTblData		;max span of rows is 0
   LD    DE,(HTML_TblCRY)		;Get current start row
   LD    (HTML_Row),DE
   JP    HTM_ShowRowBordrs		;display horizontal border
;=========================================================================
;Name     HTM_FindMaxRow
;Purpose  To return the maximum rows of displayed data for those columns
;         which span only 1 row.
;In Regs  A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         B = ??
;         C = max displayed rows of cells which span just 1 row.
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_FindMaxRow
   LD    DE,&FFFF
   LD    C,1
HTM_FindMaxRow1
   INC   DE
   LD    HL,(HTML_TblFmtDat)
   ADD   HL,DE
   LD    A,(HL)				;Get format of Column
   CP    &C0				;Last column?
   RET   Z					;Yes
   LD    HL,(HTML_TblRowWid)
   ADD   HL,DE
   LD    A,(HL)
   OR    A					;spans more then 1 row?
   JR    NZ,HTM_FindMaxRow1		;Yes
   LD    HL,(HTML_TblCRows)
   ADD   HL,DE
   LD    A,(HL)
   CP    C					;Is current max > displayed rows
   JR    C,HTM_FindMaxRow1		;Yes
   LD    C,A
   JR    HTM_FindMaxRow1
;=========================================================================
;Name     HTM_AdjustTblRows
;Purpose  To adjust the displayed data rows for those cells which span
;         many rows.
;In Regs  A = ??
;         B = ??
;         C = max displayed rows of cells which span just 1 row.
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_AdjustTblRows
   LD    DE,&FFFF
HTM_AdjustTblRows1
   INC   DE
   LD    HL,(HTML_TblFmtDat)
   ADD   HL,DE
   LD    A,(HL)				;Get format of Column
   CP    &C0				;Last column?
   RET   Z					;Yes
   LD    HL,(HTML_TblCRows)
   ADD   HL,DE
   LD    A,(HL)
   SUB   A,C				;Adjust num of displayed rows down
   JR    C,HTM_FindMaxRow1		;Adjust failed (leave as is)
   LD    (HL),A
   JR    HTM_FindMaxRow1
;=========================================================================
;Name     HTM_SetupEndRow
;Purpose  To setup what the end row of the displayed cells is.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = new end row of displayed cells.
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_SetupEndRow
   CALL  HTM_FindMaxRow			;Get Max rows
   CALL  HTM_AdjustTblRows		;Adjust counts on multi-rows
   LD    E,C
   LD    D,0
   LD    HL,(HTML_TblSRY)
   ADD   HL,DE
   LD    (HTML_TblCRY),HL		;Calc new bottom column row
   RET
;=========================================================================
;Name     HTM_NxtTblCol
;Purpose  To adjust the column for rendering table data.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_Col = current rendering web-page col
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_Col = update if border are shown.
;Uses     TBD
;Call By  TBD
;Calls    HTM_ChkTblBorder
;Notes    TBD
;=========================================================================
HTM_NxtTblCol
   LD    HL,(HTML_Col)
   INC   HL
   LD    (HTML_Col),HL
   RET
;=========================================================================
;Name     HTM_ShowCell
;Purpose  To display the table cell data contents on the web-page.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_Col = current web-page render col
;         HTML_Row = current web-page render row.
;         HTML_CmdEnd = end of <TD>/<TH> command.
;         HTML_TblCRY = current last row after cells displayed.
;         HTML_TblIndex = current column indexHTML_Col = current web-page col
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_Col = next web-page column for text
;         HTML_Row = next web-page row
;         HTML_CmdEnd = address of next table HTML command
;Uses     TBD
;Call By  TBD
;Calls    HTM_ShowChar
;         HTM_GetCmdObj
;         UTL_strncmp
;         HTM_NewLine
;         HTM_NextHTMBank
;         HTM_SaveCellRows
;         HTM_SetMaxTblRow
;Notes    TBD
;=========================================================================
HTM_ShowCell
   LD    HL,(HTML_TblCRows)
   LD    DE,(HTML_TblIndex)
   ADD   HL,DE
   LD    (HL),1				;start at only 1 displayed data row
;
   LD    DE,(HTML_TblSRY)
   LD    (HTML_Row),DE			;Next row to render in cell
   CALL  HTM_NxtTblCol			;adjust column for data to show
;
   LD    A,(HTML_CmdEnd+2)
   LD    HL,(HTML_CmdEnd)
HTM_ShowCell0
   INC   HL
   CALL  HTM_NextHTMBank
   LD    (HTML_ObjLocn+2),A
   LD    E,A
   LD    A,(HL)
   CP    '<'				;Start of HTML Command?
   JR    Z,HTM_ShowCell1		;Yes
   PUSH  HL
   PUSH  DE
   LD    B,A
   CALL  HTM_ShowChar			;show char on web-page
   POP   DE
   POP   HL
   LD    A,E
   JR    HTM_ShowCell0
HTM_ShowCell1
;
;  Get HTML command
;
   LD    A,E
   LD    (HTML_CmdEnd+2),A
   DEC   HL
   LD    (HTML_CmdEnd),HL		;Save end for HTM_GetTblInfo.
   INC   HL
;
   CALL  HTM_GetCmdObj			;Get HTML Command into KB_Buffer
;
   LD    HL,(HTML_CmdTemp)
   LD    A,(HL)
   CP    '/'
   JR    NZ,HTM_ShowCell1a
   INC   HL
   LD    A,(HL)
HTM_ShowCell1a
   CALL  utl_tolower			;ensure char is in lowercase
   CP    't'				;Start of a table type command?
   JR     Z,HTM_ShowCell4		;Yes (end of text for HTML command)
;
   LD    DE,HTML_C_br
   LD    B,2
   CALL  UTL_strncmpCI
   JR    NZ,HTM_ShowCell3
;
   LD    HL,(HTML_TblCRows)
   LD    DE,(HTML_TblIndex)
   ADD   HL,DE
   INC   (HL)
   CALL  HTM_NewLine
   CALL  HTM_NxtTblCol
;
HTM_ShowCell3
   LD    HL,(HTML_ObjLocn)		;get end of current html command.
   LD    A,(HTML_ObjLocn+2)
   JR    HTM_ShowCell0			;continue to show cell content.
;
HTM_ShowCell4
   RET
;=========================================================================
;Name     HTM_ShowCellData
;Purpose  To display the table cell data contents on the web-page if the
;         current index shows the contents can be displayed.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    HTM_ChkTblEnd
;         HTM_TableSetBR
;         HTM_TDTH_Params
;         HTM_SetupSpans
;         HTM_SetupTblRowCol
;         HTM_ShowCell
;         HTM_SetMaxTblRow
;         HTM_NextTblColumn
;Notes    TBD
;=========================================================================
HTM_ShowCellData
   CALL  HTM_ChkTblEnd			;end of known columns
   RET   Z					;Yes (ignore TD/TH)
   CALL  HTM_TableSetBR			;Setup offset for <BR> and Col
   CALL  HTM_TDTH_Params		;process any parameters to command
   CALL  HTM_SetupSpans			;setup cell row span/column span
   CALL  HTM_SetupTblRowCol		;set tbl formats (Wrap, merges,etc)
   CALL  HTM_ShowCell			;show data on web-page
   JP    HTM_NextTblColumn			;Set next column
