;=========================================================================
;
; Application Routines (text handling routines)
; Copyright (c) 2010 Ray Palmer
;
; Notes
;
;=========================================================================
;Name     HTM_Calc_P_CPC
;Purpose  To calculate the CPC screen coordinates for the <P> command
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_P_Tmp = current <P> window lcoations object
;         PageWdw_SRow = CPC screen row offset on web page
;         PageWdw_SCol = CPC screen column offset on web page
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars Window_FSRow = start row (from zero)
;         Window_FSCol = Start column (from zero)
;         Window_FERow = end row (from zero)
;         Window_FECol = end column (from zero)
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    1. The border frame is limited to the size of the web page
;            stream window.
;=========================================================================
HTM_Calc_P_CPC
;
;  Calc start and end row on CPC screen.
;
   LD    HL,(HTML_P_SRow)		;web-page start row
   LD    BC,(PageWdw_SRow)		;CPC screen offset onto web-page
   CALL  HTML_FrmeChkS			;Check before start row offset
   LD    (Window_FSRow),HL		;setup start row on CPC screen
;
   LD    HL,(HTML_P_ERow)		;web-page end row
   SBC   HL,BC				;Calculate CPC screen end row
   LD    DE,MaxWebPgRow-1		;adjust for base 0
   CALL  HTML_FrmeChkE			;calculate number of rows
   LD    (Window_FERow),HL		;setup end row on CPC screen
;
;  Calc start and end column on CPC screen.
;
   LD    HL,(HTML_P_SCol)		;start column
   LD    BC,(PageWdw_SCol)		;CPC screen offset onto web-page
   CALL  HTML_FrmeChkS			;Check if before start col offset
   LD    (Window_FSCol),HL		;setup column on CPC screen
;
   LD    HL,(HTML_P_ECol)		;end column
   SBC   HL,BC				;Calculate CPC end column
   LD    A,(Scrn_MaxCol)
   DEC   A				;adjust for base 0
   LD    E,A
   LD    D,0
   CALL  HTML_FrmeChkE			;calculate number of rows
   LD    (Window_FECol),HL		;setup end column on CPC screen
   RET
;=========================================================================
;Name     HTM_ChkMaxCol
;Purpose  To check if the column has exceeded known max.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_MaxCol = current max column
;         HTML_Col = current rendering column
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_MaxCol = updated max column
;Uses     TBD
;Call By  TBD
;Calls    TBD
;Notes    TBD
;=========================================================================
HTM_ChkMaxCol
   LD    HL,(HTML_MaxCol)
   LD    DE,(HTML_Col)
   OR    A
   SBC   HL,DE			;Current > max
   RET   NC			;No
   LD    (HTML_MaxCol),DE	;Yes set new max
   RET
