;=========================================================================
;
; Application Routines (UTL)
; Copyright (c) 2010 Ray Palmer
;
;=========================================================================
;Name     UTL_Get_HTMLAddr
;Purpose  Get 3 byte index of a HTML object.
;In Regs  A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = address of 3 byte data
;         IX = ???
;         IY = ??
;In Vars  None.
;Out Regs A = bank number (0-max banks)
;         BC = ??
;         DE = Address of HTML object
;         HL = corrupted.
;         IX = ??
;         IY = ??
;Out Vars None
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    1. The data is got in -little endian format.
;=========================================================================
UTL_Get_HTMLAddr
   CALL  UTL_GetVarWord
   LD    A,(HL)
   INC   HL
   RET
;=========================================================================
;Name     UTL_GetVarWord
;Purpose  Get a 16-bit word.
;In Regs  A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = address of data word to get
;         IX = ???
;         IY = ??
;In Vars  None.
;Out Regs AF = ??
;         B = ??
;         C = ??
;         DE = data word
;         HL = corrupted.
;         IX = ??
;         IY = ??
;Out Vars None
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    1. The data is got in -little endian format.
;=========================================================================
UTL_GetVarWord
   LD    E,(HL)
   INC   HL
   LD    D,(HL)
   INC   HL
   RET
;=========================================================================
;Name     UTL_SetVarWord
;Purpose  Set a 16-bit word.
;In Regs  A = ??
;         B = ??
;         C = ??
;         DE = Value to set
;         HL = address of data word to set
;         IX = ???
;         IY = ??
;In Vars  None.
;Out Regs AF = ??
;         B = ??
;         C = ??
;         DE = data word
;         HL = corrupted.
;         IX = ??
;         IY = ??
;Out Vars None
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    1. The data is set in -little endian format.
;=========================================================================
UTL_SetVarWord
   LD    (HL),E
   INC   HL
   LD    (HL),D
   INC   HL
   RET
;=========================================================================
;Name     HTML_SaveObj
;Purpose  To save current object location to a local variable
;In Regs  A = ??
;         BC = ??
;         DE = Location to save the 3 byte data to.
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_ObjLocn = 3 byte data to copy from.
;Out Regs AF = ??
;         BC = zero
;         DE = Corrupted
;         HL = Corrupted
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
HTML_SaveObj
   LD    HL,HTML_ObjLocn
   LD    BC,3
   LDIR
   RET
;=========================================================================
;Name     UTL_ShowCmdText
;Purpose  To display the command text until another HTML command is
;         detected.
;In Regs  A = RAM bank of HTML data
;         BC = ??
;         DE = ??
;         HL = Address of ">" in the command
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_ObjLocn = corrupted.
;Uses     TBD
;Call By  TBD
;Calls    HTM_NextHTMBank
;         HTM_ShowChar
;Notes    None.
;=========================================================================
UTL_ShowCmdText
   INC   HL					;Next address of HTML data
   CALL  HTM_NextHTMBank		;adjust address if required.
   LD    (HTML_ObjLocn),HL
   LD    (HTML_ObjLocn+2),A
   LD    A,(HL)
   CP    '<'				;Start of HTML command?
   RET   Z					;No (show char on web-page)
   LD    B,A
   LD    C,HTML_F_FntChrs
   CALL  HTM_ShowChar			;Show char on web-page
   LD    HL,(HTML_ObjLocn)
   LD    A,(HTML_ObjLocn+2)
   JR    UTL_ShowCmdText
;=========================================================================
;Name     UTL_CheckInBox
;Purpose  To check if the cursor position is within a box on the web-page.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = address of Forms DB entry
;         IX = ??
;         IY = ??
;In Vars  HTML_FrmsTmp = box dimensions
;         WebPge_Row = current web-page position (row)
;         WebPge_Col = current web-page position (column)
;Out Regs A = 0 in box else non-zero
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    UTL_WithinBounds
;Notes    None
;=========================================================================
UTL_CheckInBox
   LD    BC,(WebPge_Row)
   LD    HL,(WebPge_Col)
;=========================================================================
;Name     UTL_CheckInARea
;Purpose  To check if the cursor position is within a box on the web-page.
;In Regs  A = ??
;         BC = cursor row
;         DE = ??
;         HL = cursor column
;         IX = ??
;         IY = ??
;In Vars  HTML_FrmsTmp = box dimensions
;Out Regs A = 0 in box else non-zero
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    UTL_WithinBounds
;Notes    None
;=========================================================================
UTL_CheckInArea
   PUSH  HL
   LD    HL,(HTML_FrmsSRow)
   LD    DE,(HTML_FrmsERow)
   CALL  UTL_WithinBounds
   POP   BC
   OR    A			;within Row bounds?
   RET   NZ			;No
   LD    HL,(HTML_FrmsSCol)
   LD    DE,(HTML_FrmsECol)
   JP    UTL_WithinBounds
;=========================================================================
;Name     UTL_WithinBounds
;Purpose  To check if the point is with the bounds
;In Regs  A = Flag
;         BC = Point in question
;         DE = End point
;         HL = Start Point
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs A = result (1=before start, 2= after end, 0=inside)
;         BC = ??
;         DE = Corrupted
;         HL = Corrupted
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
UTL_WithinBounds
   LD    A,1		;Assume before start
   OR    A		;Clear Carry flag
   SBC   HL,BC		;Is check point >= start?
   JR    Z,UTL_WinBds1	;Yes
   RET   NC		;point is before start
   INC   A		;Assume now after end
   EX    DE,HL
   OR    A		;Clear Carry flag
   SBC   HL,BC		;Is check point <= End?
   RET   C		;No (point is after end)
UTL_WinBds1
   XOR   A		;flag inside
   RET
;=========================================================================
;Name     UTL_ChkRendState
;Purpose  To check the state of rendering the HTML
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_RenderState = current rendering state
;Out Regs AF = Z Flag Register has state
;         BC = ??
;         DE = ??
;         HL = corrupted
;         IX = ??
;         IY = ??
;Out Vars None
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
UTL_ChkRendState
   LD    HL,HTML_RenderState
   BIT   3,(HL)				;Gathering information for renderer
   RET
;=========================================================================
;Name     MNU_RestoreCfg
;Purpose  To restore memory configuration and window selection.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ???
;         IY = ??
;In Vars  Stream_WebPage = CPC stream number for web-page canvas
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars None
;Uses     TBD
;Call By  TBD
;Calls    Mem_SwapInMain
;         SC_SEL_STREAM (OS)
;Notes    TBD
;=========================================================================
MNU_RestoreCfg
   CALL  Mem_SwapInMain
   LD    A,(Stream_WebPage)
   JP    SC_SEL_STREAM
;=========================================================================
;Name     HTM_DoExecScr
;Purpose  To execute the handler for a script command object
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    1. The script handler expects the object to handle is just
;            text.
;=========================================================================
HTM_DoExecScr
   RET
;=========================================================================
;Name     URL_GetBaseURL
;Purpose  To Setup the Base URL in main memory.
;In Regs  AF = ??
;         BC = ??
;         DE = Location to save base URL
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    MEM_SwapInCfg
;         Mem_SwapInMain
;Notes    TBD
;=========================================================================
URL_GetBaseURL
   CALL  MEM_SwapInCfg
   LD    HL,CFG_BaseURL
   LD    B,URL_Size
   CALL  UTL_strncpy
   XOR   A
   LD    (DE),A
   JP    Mem_SwapInMain
;=========================================================================
;Name     UTL_CopyMem
;Purpose  To copy a number of bytes from banked memory to local memory.
;In Regs  AF = ??
;         B = Number of bytes to copy.
;         C = ??
;         DE = Location to save to
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  IMG_Locn = location of data
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars IMG_Locn = updated
;Uses     TBD
;Call By  TBD
;Calls    MEM_SwapInCfg
;         Mem_SwapInMain
;Notes    TBD
;=========================================================================
UTL_CopyMem
   LDI					;copy a byte of data
   INC   BC				;restore BC
   LD    A,(IMG_Locn+2)
   CALL  HTM_NextHTMBank		;Adjust & swap in bank to save to.
   LD    (IMG_Locn+2),A
   DJNZ  UTL_CopyMem
   LD    (IMG_Locn),HL
   RET
;=========================================================================
;Name     UTL_ScreenRefresh
;Purpose  To flag a screen refresh is requested.
;In Regs  AF = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  TBD
;Out Regs AF = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_RenderState = updated on bit 4
;Uses     TBD
;Call By  TBD
;Calls    CPCB_ResetCursor
;Notes    TBD
;=========================================================================
UTL_ScreenRefresh
   LD    HL,HTML_RenderState
   SET   4,(HL)			;Flag screen refresh
   JP    CPCB_ResetCursor	;force cursor to refresh with new data
;