;=========================================================================
;
; Application Routines (window text display)
; Copyright (c) 2010 Ray Palmer
;
;=========================================================================
;Name     HTM_AdjustRC
;Purpose  To adjust the row and column limits and render position to
;         account for the window's frame border.
;In Regs  A = ??
;         BC = Cursor position in frame
;         DE = end point of frame
;         HL = start point of frame
;         IX = ??
;         IY = ??
;In Vars  Window_FrmeID = frame flags/status
;Out Regs A = corrupted.
;         BC = ??
;         DE = updated if window has a frame
;         HL = updated if window has a frame
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  TBD
;Calls    None
;Notes    None
;=========================================================================
HTM_AdjustRC
   LD    A,(Window_FrmeID)
   BIT   4,A			;Frame has border?
   RET   Z			;No
   DEC   DE			;Adjust for border (end point)
   INC   HL			;Adjust for border (start point)
   INC   BC
   RET
;=========================================================================
;Name     HTM_GetWebPgeCR
;Purpose  To calculate the absolute column/row on the web-page.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_Col = current rendering column on web-page canvas
;         HTML_Row = current rendering row on web-page canvas
;         Window_SCol = Current frame start column on web-page canvas
;         Window_SRow = current frame start row on web-page canvas
;Out Regs A = ??
;         BC = ??
;         DE = Column on screen window (from zero)
;         HL = Row on screen window (from zero)
;         IX = ??
;         IY = ??
;Out Vars TBD
;Uses     TBD
;Call By  HTM_ShowChar
;Calls    None.
;Notes    1. Where there are no frames in use the base of the frame is
;            limited to the CPC screen dimensions.
;=========================================================================
HTM_GetWebPgeCR
   LD    HL,(HTML_Col)		;Current column to print
   LD    BC,(Window_SCol)	;Current Frame web-page base column
   ADD   HL,BC
   EX    DE,HL			;DE = column on screen
   LD    HL,(HTML_Row)		;Current row to print
   LD    BC,(Window_SRow)	;Current Frame web-page base row
   ADD   HL,BC			;Calculate Web-Page Row
   RET
;=========================================================================
;Name     HTM_CalcScrnCR
;Purpose  To calculate the CPC screen column/row.
;In Regs  A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_Col = current rendering column on web-page canvas
;         HTML_Row = current rendering row on web-page canvas
;         Window_SCol = Current frame start column on web-page canvas
;         Window_SRow = current frame start row on web-page canvas
;         PageWdw_SCol = current start column of CPC screen on the web
;                        page
;         PageWdw_SRow = current start row of CPC screen on the web
;                        page
;Out Regs A = ??
;         BC = ??
;         DE = Column on screen window (from zero)
;         HL = Row on screen window (from zero)
;         IX = ??
;         IY = ??
;Out Vars None
;Uses     TBD
;Call By  HTM_ChkWinPos
;Calls    None.
;Notes    1. Where there are no frames in use the base of the frame is
;            limited to the CPC screen dimensions.
;=========================================================================
HTM_CalcScrnCR
   CALL  HTM_GetWebPgeCR	;Calc absolute web-page position.
   LD    A,(Window_FrmeID)
   BIT   4,A			;Frame has border?
   JR    Z,HTM_CalcScrnCR1	;No
   INC   DE			;Adjust cursor column for border
   INC   HL			;Adjust cursor row for border
;
HTM_CalcScrnCR1
   LD    BC,(PageWdw_SRow)	;Current CPC row offset on web-page
   OR    A			;clear carry flag
   SBC   HL,BC			;calc row on for CPC window
   LD    BC,(Window_Roff)
   OR    A
   SBC   HL,BC			;adjust for scrolling of frame
   EX    DE,HL
;
   LD    BC,(PageWdw_SCol)	;Current CPC column offset on web-page
   OR    A			;clear carry flag
   SBC   HL,BC			;calc column on for CPC window
   LD    BC,(Window_Coff)
   OR    A
   SBC   HL,BC			;adjust for scrolling of frame
   EX    DE,HL			;DE = column on screen
;
   RET
;=========================================================================
;Name     HTM_ChkFramePos
;Purpose  To check the current rendering row/column is within the current
;         web-page frame.
;In Regs  A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_Col = current relative column in current frame
;         HTML_Row = current relative row in current frame
;         Window_SCol = Frame base column (absolute)
;         Window_SRow = Frame base row (absolute)
;         Window_ECol = Frame end column (absolute)
;         Window_ERow = frame end row (absolute)
;Out Regs AF = ??
;         B = ??
;         C = ??
;         DE = corrupted.
;         HL = address of HTML_DispStatus
;         IX = ??
;         IY = ??
;Out Vars HTMLDispStatus = updated if not within boundary
;Uses     TBD
;Call By  HTM_CheckOnScrn
;Calls    HTM_AdjustRC
;         UTL_WithinBounds
;Notes    1. The Window frame dimension are from zero and not 1.
;=========================================================================
HTM_ChkFramePos
   LD    DE,(Window_SCol)
   LD    HL,(HTML_Col)
   ADD   HL,DE			;calc frame column
   LD    C,L
   LD    B,H
   EX    DE,HL
   LD    DE,(Window_ECol)
   CALL  HTM_AdjustRC		;adjust for frame
   CALL  UTL_WithinBounds
   OR    A			;within frame's columns?
   CALL  NZ,HTM_ChkWinPos1	;No (flag if before left or after right)
;   
   LD    DE,(Window_SRow)
   LD    HL,(HTML_Row)
   ADD   HL,BC			;calc frame row
   LD    C,L
   LD    B,H
   EX    DE,HL
   LD    DE,(Window_ERow)
   CALL  HTM_AdjustRC		;adjust for frame
   CALL  UTL_WithinBounds
   OR    A			;within frame's rows?
   RET   Z			;Yes
   JR    HTM_ChkWinPos0		;(flag if before top or after bottom)
;=========================================================================
;Name     HTM_ChkWinPos
;Purpose  To check the calculated rendering screen row/Column is within
;         the dimensions of the CPC screen.
;In Regs  A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_SCol = calculated CPC screen column location
;         HTML_SRow = calculated CPC screen row location
;         Window_FSCol = CPC frame base column
;         Window_FSRow = CPC frame base row
;         Window_FECol = CPC frame end column
;         Window_FERow = CPC frame end row
;Out Regs AF = ??
;         BC = ??
;         DE = corrupted.
;         HL = address of HTML_DispStatus
;         IX = ??
;         IY = ??
;Out Vars HTML_SCol = Calculated CPC screen column (starts from 0)
;         HTML_SRow = Calculated CPC screen row (starts from 0)
;         HTML_DispStatus = updated if not within boundary
;Uses     TBD
;Call By  HTM_CheckOnScrn
;Calls    HTM_AdjustRC
;         UTL_WithinBounds
;Notes    TBD
;=========================================================================
HTM_ChkWinPos
   CALL  HTM_CalcScrnCR		;Calculate CPC screen location.
   LD    (HTML_SRow),HL
   LD    (HTML_SCol),DE
   LD    E,H
   LD    HL,HTML_DispStatus
   LD    A,1
   BIT   7,E			;Is row -ve?
   CALL  NZ,HTM_ChkWinPos0	;Yes
;
   BIT   7,D			;Is column -ve?
   JR    NZ,HTM_ChkWinPos1	;Yes
;
   LD    A,(HL)
   AND   &F0			;is position before left/top?
   RET   NZ			;Yes
;
   LD    DE,(Window_FECol)
   LD    HL,(Window_FSCol)
   CALL  HTM_AdjustRC		;adjust edge if framed
   LD    BC,(HTML_SCol)
   CALL  UTL_WithinBounds	;within screen frame columns?
   OR    A
   CALL  NZ,HTM_ChkWinPos1	;No (outside)
;   
   LD    DE,(Window_FERow)
   LD    HL,(Window_FSRow)
   CALL  HTM_AdjustRC		;adjust edge if framed
   LD    BC,(HTML_SRow)
   CALL  UTL_WithinBounds	;within screen frame rows?
   OR    A			;Is position within rows?
   RET   Z			;Yes
;
HTM_ChkWinPos0
   LD    HL,HTML_DispStatus
   SET   7,(HL)			;Assume before top row
   CP    1			;is position before top row?
   RET   Z			;Yes
   RES   7,(HL)
   SET   6,(HL)			;flag after bottom row
   RET
;
HTM_ChkWinPos1
   LD    HL,HTML_DispStatus
   SET   5,(HL)			;Assume before left column
   CP    1			;is position before left column?
   RET   Z			;Yes
   RES   5,(HL)
   SET   4,(HL)			;Flag after right column
   RET
;=========================================================================
;Name     HTM_CheckOnScrn
;Purpose  To check the web-page frame position on the screen.
;In Regs  A = ??
;         B = ??
;         C = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_Col = current rendering column on web-page canvas
;         HTML_Row = current rendering row on web-page canvas
;Out Regs A = status of last check (0=inside, >0 outside)
;         BC = preserved value
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_RenderState updated on bit 6
;         HTML_DispStatus updated on bits 7-4.
;Uses     All registers
;Call By  TBD
;Calls    HTM_ChkFramePos
;         HTM_ChkWinPos
;Notes    TBD
;=========================================================================
HTM_CheckOnScrn
   PUSH  BC
   LD    A,(HTML_DispStatus)
   AND   &0F			;remove status bits (upper nibble)
   LD    (HTML_DispStatus),A
   CALL  HTM_ChkFramePos	;Check if within web-page frame
   OR    A			;position within frame?
   CALL  Z,HTM_ChkWinPos	;Yes, check if visible on CPC screen
   POP   BC
   OR    A			;visible to display character?
   LD    HL,HTML_RenderState
   SET   6,(HL)			;flag position is visible
   RET   Z			;Yes
   RES   6,(HL)			;position is not visible
   RET
;=========================================================================
;Name     HTM_ShowChar
;Purpose  To show the HTML text char or icon.
;In Regs  A = ??
;         B = Text char/Icon to print
;         C = Preserve Flag
;              Bit7 = 0 Character else 1 = Icon
;              if Bit 7 = 1 then bits 0-3 is is icon set.
;              Bit 6 = Image char to display
;              Bit 5 = Character handler (0=Ignore, 1=handle it)
;              Bit 4 = How to handle non-print char
;                      0 = show char as "." else intrepret it.
;              Bit 1 = 0 (system font), 1 (current font)
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;In Vars  HTML_Col = current rendering column on web-page canvas
;         HTML_Row = current rendering row on web-page canvas
;         HTML_RenderState = current rendering status
;         HTML_WrapLocks = wrap control flags
;         HTML_PreserveFlag = file content status
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars Window_FrmeID = updated
;         HTML_SCol = calculated screen column
;         HTML_SRow = calculated screen row
;Uses     All registers
;Call By  TBD
;Calls    HTM_CheckOnScrn
;         HTM_NewLine
;Notes    1. The routine does not handle control codes as a browser is not
;            meant to replicate a screen in a smaller area as if it were a
;            ful screen.
;=========================================================================
HTM_ShowChar
   BIT   7,C			;display icon ?
   JP    NZ,HTM_ShwChrIcn	;Yes
   BIT   6,C			;Image char to display
   JR    Z,HTM_ShwChar0		;No
;=========================================================================
;  Display an image character
;=========================================================================
   CALL  HTM_CheckOnScrn
   LD    A,(HTML_DispStatus)
   AND   &F0			;Is position on screen?
   JR    Z,HTM_ShwChar0a
   LD    HL,Window_FrmeID
   SET   6,(HL)			;flag need scrolling frame
   RET
;
HTM_ShwChar0a
   LD    A,(HTML_SCol)
   LD    HL,(HTML_SRow)
   LD    H,A			;H = Screen Column/L = Screen Row
   JP    ERoutine_ShowImgChr	;Show image character data
;=========================================================================
;  Display normal text
;=========================================================================
HTM_ShwChar0
   LD    A,(HTML_PreserveFlag)
   BIT   7,A			;Show as plain text?
   JR    Z,HTM_ShwChar3		;No
;
   LD    A,B
   CP    LINEFEED_CHAR		;Is char LF?
   JP    Z,HTM_NewLine		;Yes (treat as newline)
;
HTM_ShwChar3
   LD    A,B
   CP    SPACE_CHAR		;Is char below 32?
   RET   C			;Yes (not printable)
;
   CALL  HTM_CheckOnScrn	;check if position is on screen
;
   LD    A,(HTML_DispStatus)
   AND   &F0			;Is position on screen?
   JR    Z,HTM_ShwChar5		;Yes
;
   LD    A,(HTML_WrapLocks)
   BIT   7,A			;Is wrap text enabled?
   JR    Z,HTM_ShwChar4		;No (just goto next position)
;
   CALL  HTM_NewLine		;Yes (setup new line)
   OR    A			;Is new position on screen?
   JR    Z,HTM_ShwChar0		;Yes
;
HTM_ShwChar4
   LD    HL,Window_FrmeID
   SET   6,(HL)			;flag need scrolling frame
   JR    HTM_ShwChar9
;
HTM_ShwChar5
   LD    A,(HTML_RenderState)
   BIT   7,A			;collecting rendering info?
   RET   NZ			;Yes (no displays allowed)
;
   LD    E,0
   LD    A,(HTML_PreserveFlag)
   BIT   7,A			;Show as plain text?
   JR    Z,HTM_ShwChar6		;No (show HTML data only)
   LD    E,255			;flag system font
HTM_ShwChar6
   LD    A,E
   LD    (FontOutput),A
   LD    A,(HTML_SCol)
   LD    HL,(HTML_SRow)
   LD    H,A			;H = Screen Column/L = Screen Row
   LD    A,B
   AND   &7f			;ensure char is 0-127
   CALL  CPCB_DisplayChar	;write char to screen
;
;  Correct screen position back to frame position
;  HL has new position (H=new column, L=row)
;  D = status of request to show character
;  A = font type displayed.
;
   LD    E,H
   LD    HL,HTML_PreserveFlag
   BIT   7,(HL)			;Show as plain text?
   JR    NZ,HTM_ShwChar9
;
   OR    A			;Fixed space font?
   JR    Z,HTM_ShwChar9		;Yes
   LD    A,(HTML_SCol)
   SUB   A,E			;Calcoffset to next column
   LD    C,A			;FIXME
   LD    B,0			;FIXME
   JR    HTM_ShwChar8
;=========================================================================
;  Display an ICon character
;=========================================================================
HTM_ShwChrIcn
   CALL  HTM_CheckOnScrn
;
   LD    A,(HTML_RenderState)
   BIT   7,A			;collecting rendering info?
   JR    NZ,HTM_ShwChar9	;Yes (no displays allowed)
;
   BIT   6,A			;Can we show icon on screen?
   JR    Z,HTM_ShwChar8		;No
;
   LD    A,(HTML_SCol)
   LD    HL,(HTML_SRow)
   LD    H,A			;H = Screen Column/L = Screen Row
   LD    A,B
   RES   7,C
   CALL  CPCB_DisplayIcon
;
HTM_ShwChar9
   LD    BC,1
;
HTM_ShwChar8
   LD    HL,(HTML_Col)
   ADD   HL,BC
   LD    (HTML_Col),HL
   RET
;=========================================================================
;Name     HTM_ShowString
;Purpose  To display a string of text in either 1 line or many lines.
;In Regs  A = HTML Bank
;         B = termination char
;         C = Output Font type
;         DE = ??
;         HL = Address within HTML data to show.
;         IX = ??
;         IY = ??
;In Vars  HTML_Col = current rendering column on web-page canvas
;         HTML_Row = current rendering row on web-page canvas
;         HTML_TextStrlen = length of text in 1 line
;         HTML_TextStrMax = max width of text
;         HTML_TextLinNum = number of lines of text
;Out Regs A = ??
;         BC = ??
;         DE = ??
;         HL = ??
;         IX = ??
;         IY = ??
;Out Vars HTML_TextAddr = address of text
;         Window_FrmeID = bit 6 cleared to prevent frames shown.
;         HTML_Col = updated
;         HTML_Row = updated
;Uses     All registers
;Call By  TBD
;Calls    HTM_ShowChar
;         HTM_NextHTMBank
;Notes    1. This routine is used by the forms, tables, etc.
;=========================================================================
HTM_ShowString
   LD    (HTML_TextAddr),HL
   LD    (HTML_TextAddr+2),A
   LD    A,(HL)
   OR    A				;Default End of text?
   RET   Z				;Yes
   CP    B				;Alternate end of text?
   RET   Z				;Yes
;
   LD    HL,HTML_TextMode
   BIT   6,(HL)				;Hidden output?
   JR    Z,HTM_ShowStrng2		;No
   LD    A,'*'				;Yes
HTM_ShowStrng2
   PUSH  BC
   LD    B,A
   RES   7,B				;Assume system chars
   LD    A,C
   CP    HTML_F_FntChrs
   JR    NZ,HTM_ShowStrng2a
   SET   7,B				;Flag font chars
HTM_ShowStrng2a
   CALL  HTM_ShowChar			;Show char on web-page
   POP   BC
   LD    A,(HTML_TextStrlen)
   OR    A				;end of max output?
   JR    NZ,HTM_ShowStrng3		;No
   LD    A,(HTML_TextLinNum)
   OR    A				;all lines shown?
   RET   Z				;Yes
   DEC   A				;decrement line count
   LD    (HTML_TextLinNum),A
   LD    A,(HTML_TextStrMax)
   JR    HTM_ShowStrng4
HTM_ShowStrng3
   LD    A,(HTML_TextStrlen)
   DEC   A
HTM_ShowStrng4
   LD    (HTML_TextStrlen),A
   LD    HL,(HTML_TextAddr)
   LD    A,(HTML_TextAddr+2)
   INC   HL				;Next address of HTML data
   BIT   7,H				;After bank limit?
   JR    Z,HTM_ShowString		;No
   CP    MaxHTMLBanks			;indexing HTML banks?
   CALL  C,HTM_NextHTMBank		;Yes, adjust address if required.
   JR    HTM_ShowString
;